#!/bin/bash

#optional arguments:
#  -h, --help            show this help message and exit
#  -p PORT, --port PORT  Port number to listen on
#  -b HOST, --bind HOST  The host/address to bind to
#  --websocket_timeout WEBSOCKET_TIMEOUT
#                        Maximum time to allow a websocket to be connected. -1
#                        for infinite.  --websocket_connect_timeout WEBSOCKET_CONNECT_TIMEOUT                        Maximum time to allow a connection to handshake. -1
#                        for infinite
#  -u UNIX_SOCKET, --unix-socket UNIX_SOCKET
#                        Bind to a UNIX socket rather than a TCP host/port
#  --fd FILE_DESCRIPTOR  Bind to a file descriptor rather than a TCP host/port
#                        or named unix socket
#  -e SOCKET_STRINGS, --endpoint SOCKET_STRINGS
#                        Use raw server strings passed directly to twisted
#  -v VERBOSITY, --verbosity VERBOSITY
#                        How verbose to make the output
#  -t HTTP_TIMEOUT, --http-timeout HTTP_TIMEOUT
#                        How long to wait for worker before timing out HTTP
#                        connections
#  --access-log ACCESS_LOG
#                        Where to write the access log (- for stdout, the
#                        default for verbosity=1)
#  --ping-interval PING_INTERVAL
#                        The number of seconds a WebSocket must be idle before
#                        a keepalive ping is sent
#  --ping-timeout PING_TIMEOUT
#                        The number of seconds before a WebSocket is closed if
#                        no response to a keepalive ping
#  --application-close-timeout APPLICATION_CLOSE_TIMEOUT
#                        The number of seconds an ASGI application has to exit
#                        after client disconnect before it is killed
#  --ws-protocol [WS_PROTOCOLS [WS_PROTOCOLS ...]]
#                        The WebSocket protocols you wish to support
#  --root-path ROOT_PATH
#                        The setting for the ASGI root_path variable
#  --proxy-headers       Enable parsing and using of X-Forwarded-For and
#                        X-Forwarded-Port headers and using that as the client
#                        address
#  --proxy-headers-host PROXY_HEADERS_HOST
#                        Specify which header will be used for getting the host
#                        part. Can be omitted, requires --proxy-headers to be
#                        specified when passed. "X-Real-IP" (when passed by
#                        your webserver) is a good candidate for this.
#  --proxy-headers-port PROXY_HEADERS_PORT
#                        Specify which header will be used for getting the port
#                        part. Can be omitted, requires --proxy-headers to be
#                        specified when passed.
#  -s SERVER_NAME, --server-name SERVER_NAME
#                        specify which value should be passed to response
#                        header Server attribute

#--bind <ADDRESS>：指定Daphne绑定的地址和端口。例如--bind 0.0.0.0:8000表示绑定到所有网络接口的8000端口。
#
#--http-timeout <SECONDS>：指定HTTP请求的超时时间，单位为秒。如果在指定的时间内没有收到完整的HTTP请求，Daphne将关闭连接。默认值为5秒。
#
#--root-path <PATH>：指定静态文件的根路径。如果WebSocket请求的路径与静态文件路径匹配，Daphne将尝试提供静态文件内容。
#
#--websocket-timeout <SECONDS>：指定WebSocket连接的超时时间，单位为秒。如果在指定的时间内没有收到任何消息，Daphne将关闭连接。默认值为86400秒（24小时）。
#
#--ping-interval <SECONDS>：指定发送ping消息的时间间隔，单位为秒。Daphne会定期向客户端发送ping消息，以确保连接处于活动状态。默认值为20秒。
#
#--ping-timeout <SECONDS>：指定等待pong消息的超时时间，单位为秒。如果在指定的时间内没有收到pong消息，Daphne将关闭连接。默认值为20秒。
#
#--ws-protocol <PROTOCOL>：指定WebSocket的协议。默认情况下，Daphne会尝试使用websocket和websocket-natvie两种协议，如果无法协商，则使用HTTP协议。您可以通过此参数来指定其他协议。
#
#--no-http-redirect：禁用HTTP重定向功能。默认情况下，如果Daphne接收到一个非WebSocket的HTTP请求，它会将请求重定向到一个WebSocket URL上。



# 设置要检查的进程名称
process_name="nfs_domain_mainserver/nfs_domain_runserver.py"

# 设置等待时间间隔（秒）
interval=5
cpu_count=$(nproc)
# 端口号
ports=""
#启动进程端口
start_process(){
  websocket_count=0
  for (( i = 1;; i++ )); do
    port=808$i
    if [ $port -eq 8082 ]; then
        continue
    fi
    result=$(ps aux|grep -i "$port"|grep -v grep|wc -l)
    if [ "$result" -eq 1 ]; then
        continue
    fi
    # 使用uvicorn启动websocket服务，提升性能
    cd nfs_domain_mainserver/ && python /opt/domain/nfs-domain-mainserver/PYTHONVENV/bin/uvicorn nfs_domain_mainserver.asgi:application --host 0.0.0.0 --port $port --workers 4 &
    #cd nfs_domain_mainserver/ && python /opt/domain/nfs-domain-mainserver/PYTHONVENV/bin/daphne nfs_domain_mainserver.asgi:application -b 0.0.0.0 -p $port &
    ports="$ports $port"
    websocket_count=$((websocket_count+1))
    if [ $websocket_count -eq $((cpu_count/4)) ]; then
        break
    fi
  done
}
start_process

#重启进程
restart_process(){
  for port in $ports ; do
    result=$(ps aux|grep -i "$port"|grep -v grep|wc -l)
    if [ "$result" -eq 1 ]; then
        continue
    fi
    echo "$port 重启"
    cd nfs_domain_mainserver/ && python /opt/domain/nfs-domain-mainserver/PYTHONVENV/bin/uvicorn nfs_domain_mainserver.asgi:application --host 0.0.0.0 --port $port --workers 4 &
  done
}
# 循环检查进程是否存在
while true; do
  # 使用 pgrep 检查进程是否存在
    if pgrep -f "$process_name" > /dev/null; then
        #echo "Process '$process_name' found. Proceeding with the next command."
    # 默认所有websocket端口号都启用
      all_ports_open=true
      for port in $ports; do
          # 检查端口号是否被占用(统计匹配到的行数)
          port_used=$(ps aux|grep -i "$port"|grep -v grep|wc -l)
          if [ $port_used -eq 0 ]; then
              all_ports_open=false
              break
         fi
       done
      if [ "$all_ports_open" = false ]; then
         # echo "$process_name 监听的部分端口不存在，重启进程"
          restart_process
      #else
         # echo "$process_name 监听的所有端口都正常，继续监控"
      fi
    else
      echo "Process '$process_name' not found. Waiting for $interval seconds before checking again."
      start_process
    fi
    sleep $interval
done