#!/bin/bash

PYTHONVENV=/opt/domain/nfs-domain-mainserver/PYTHONVENV
cd /opt/domain/nfs-domain-mainserver

# 检查文件是否存在
if [ -f "glusterfs.sh" ]; then
    # 如果文件不可执行，则添加执行权限
    if [ ! -x "glusterfs.sh" ]; then
        chmod +x "glusterfs.sh"
        echo "已为 glusterfs.sh 添加执行权限。"
    fi
    # start glusterfs
    /usr/sbin/glusterd
    # 执行文件
    ./glusterfs.sh
fi

# Wait for database
echo "Waiting for redis" $REDIS_HOST:$REDIS_PORT
echo "Waiting for mysql" $DB_HOST:$DB_PORT
echo "Waiting for salt" $SALT_URL
curPath=$(readlink -f "$(dirname "$0")")
#mkdir ~/.ssh
#touch ~/.ssh/config
#cat>~/.ssh/config<<EOF
#StrictHostKeyChecking no
#UserKnownHostsFile=/dev/null
#EOF


# 检查 /etc/hosts 文件中是否已经有这条记录
if ! grep -q "${DOMAIN_IP}\s${DOMAIN_SERVER_FQDN}" /etc/hosts; then
    # 如果没有，就添加这条记录
    echo "${DOMAIN_IP} ${DOMAIN_SERVER_FQDN}" >> /etc/hosts
    echo "添加了 ${DOMAIN_IP} ${DOMAIN_SERVER_FQDN} 到 /etc/hosts"
#    echo "${DOMAIN_IP} ${DOMAIN_SERVER_FQDN} 已经存在于 /etc/hosts 中"
fi

# 获取Python 3的版本信息
python_version=$(python3 --version 2>&1)
# 检查命令是否成功执行
if [ $? -ne 0 ]; then
    echo "Python 3 is not installed or the command 'python3' failed."
    exit 1
fi

# 提取版本号
version_number=$(echo $python_version | awk '{print $2}')

# 判断版本
if [[ "$version_number" == 3.6.* ]]; then
        python_version=python3.6
        sleep_t=60
    echo "The current Python version is 3.6.x"
elif [[ "$version_number" == 3.9.* ]]; then
        python_version=python3.9
        sleep_t=20
    echo "The current Python version is 3.9.x"
elif [[ "$version_number" == 3.11.* ]]; then
        python_version=python3.11
        sleep_t=20
    echo "The current Python version is 3.11.x"
else
    echo "The current Python version is neither 3.6 nor 3.9, 3.11, it is: $version_number"
    exit 1
fi

export PYTHONPATH=${PYTHONVENV}/lib64/${python_version}/site-packages:${PYTHONVENV}/lib/${python_version}/site-packages:$PYTHONPATH

if [ "$DEPLOY_ARCH" != "cluster" ]; then
  /opt/domain/nfs-domain-mainserver/wait-for -t $sleep_t $REDIS_HOST:$REDIS_PORT
  /opt/domain/nfs-domain-mainserver/wait-for -t $sleep_t $DB_HOST:$DB_PORT
  /opt/domain/nfs-domain-mainserver/wait-for -t $sleep_t $MASTER_HOST:$MASTER_PORT
fi

#echo  chmod 600 to id_rsa,acceess to reprepro without pawssword
#chmod 600 /root/.ssh/id_rsa

echo check or init db
./stup-mysql-docker.sh start

echo clear cache redis
python domain-clear-redis.py $REDIS_HOST $REDIS_PORT

#echo start dev domain project
#./domain-docker.sh start

echo start domain project
./domain-docker.sh start

echo start celery
./domain-celery.sh

echo start salt event server
./domain-saltevent.sh

echo start django backend scheduler tasks
./domain-sch.sh

echo start websocket server
./domain-websocket.sh &

tail -f logs/web/domain.log &&
echo start logs