from django.db import models

# Create your models here.
import uuid
import hmac
import datetime
import time
from hashlib import sha1
from django.db import models
from django.contrib.auth.hashers import make_password


# from organizations.models import OrgnizationEntity


class AccountEntity(models.Model):
    # user status
    user_status = (
        (0, 'Normal'), (1, 'Disable'), (2, 'Locked')
    )
    # root status  是否开启root权限
    root_status = (
        (0, '否'), (1, '是')
    )
    # system status  系统是否激活
    system_status = (
        (0, '否'), (1, '是')
    )
    # policy status  是否开启默认策略
    policy_status = (
        (0, '失败'), (1, '成功')
    )
    passwd_status = (
        (0, 'NO'), (1, 'YES')
    )
    create_type_choices = (
        (0, '自建'), (1, 'ldap'), (2, 'soap_import')
    )
    cn_name = models.CharField(verbose_name='用户名', max_length=255, null=True, blank=True)
    name = models.CharField(verbose_name='用户账号', max_length=255, unique=True, blank=True)
    phone = models.CharField(verbose_name='用户ID/手机号', max_length=18, null=True, blank=True)
    user_status = models.IntegerField(verbose_name='用户状态', choices=user_status, default=0, blank=True)
    org_info = models.ForeignKey(to='organizations.OrgnizationEntity', on_delete=models.PROTECT, db_constraint=False,
                                 default=1, blank=True, verbose_name='所属组织机构')
    address = models.CharField(verbose_name='所在地址', max_length=255, null=True, blank=True)
    office_address = models.CharField(verbose_name='办公地址', max_length=255, null=True, blank=True)
    remarks = models.CharField(verbose_name='备注', max_length=255, null=True, blank=True)
    is_root = models.IntegerField(verbose_name='用户状态', choices=root_status, default=0, blank=True)
    system_status = models.IntegerField(verbose_name='用户状态', choices=system_status, default=1, blank=True)
    policy_status = models.IntegerField(verbose_name='策略状态', choices=policy_status, default=0, blank=True)

    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_login_ip = models.GenericIPAddressField(null=True, default=None)
    last_accessed = models.DateTimeField(verbose_name='最后访问', null=True, blank=True)
    job_no = models.CharField(verbose_name='工号', max_length=255, null=True, blank=True)
    email = models.CharField(verbose_name='邮箱', max_length=255, null=True, blank=True)
    is_extra_status = models.SmallIntegerField(verbose_name='密码校验是否额外处理', choices=passwd_status, default=0,
                                               blank=True)
    history_password = models.TextField(verbose_name='历史密码，最多保留最近10次', null=True, blank=True)
    create_type = models.IntegerField(verbose_name='创建方式', choices=create_type_choices, blank=True, default=0)
    last_login_time = models.CharField(verbose_name='用户最后登录时间', max_length=50, null=True, blank=True)

    class Meta:
        db_table = 'account_info'
        verbose_name = '终端用户'
        verbose_name_plural = '终端用户'
        ordering = ('created_at',)

    def __str__(self):
        return self.name


class PasswordAccountRule(models.Model):
    """
    用户密码校验规则表
    """
    switch_status = (
        (0, 'NO'), (1, 'YES')
    )
    accounts_check_status = models.SmallIntegerField('是否检查账号', choices=switch_status, blank=True, null=True)
    account_mix_length = models.SmallIntegerField('最小密码长度', blank=True, null=True)
    is_lower = models.SmallIntegerField('是否小写子母', choices=switch_status, blank=True, null=True)
    is_upper = models.SmallIntegerField('是否大写子母', choices=switch_status, blank=True, null=True)
    is_num = models.SmallIntegerField('是否数字', choices=switch_status, blank=True, null=True)
    is_char = models.SmallIntegerField('是否特殊字符', choices=switch_status, blank=True, null=True)
    password_extra_status = models.SmallIntegerField('密码有效期是否例外', choices=switch_status, blank=True, null=True)
    password_extra_account = models.SmallIntegerField('账号是否例外', choices=switch_status, blank=True, null=True)
    is_offline_login = models.SmallIntegerField('是否启用离线登入 0|否，1|是', blank=True, null=True)
    offline_cache_days = models.SmallIntegerField('离线登入缓存时间 空|永久', blank=True, null=True)

    class Meta:
        verbose_name = '用户账号密码校验规则表 '
        verbose_name_plural = verbose_name


class PasswordAccountProhibitModification(models.Model):
    """
    记录禁止修改密码的用户
    """

    # 范围，0 按部门、1按终端
    range_choices = (
        (0, 'organizations'), (1, 'account')
    )
    status_choices = (
        (0, 'N'), (1, 'Y')
    )
    status = models.IntegerField(verbose_name='禁止修改密码,0 关、1 开', choices=status_choices, default=0, blank=True)
    range = models.IntegerField(verbose_name='例外范围,0 按部门、1 按用户', choices=range_choices, default=0, blank=True)
    org_id = models.CharField(verbose_name='所属组织机构id', max_length=256, null=True, blank=True)
    account_id = models.CharField(verbose_name='用户', max_length=256, null=True, blank=True)

    class Meta:
        verbose_name = '用户账号密码禁止修改表 '
        verbose_name_plural = verbose_name

    # # 自定义序列化字段
    # @property
    # def account_name(self):
    #     return self.account.name
    #
    # @property
    # def account_cn_name(self):
    #     return self.account.cn_name
    #
    # @property
    # def account_org_name(self):
    #     return self.account.org_info.name

    def __str__(self):
        return self.pk


class Ldap(models.Model):
    ldap_type_choices = (
        (0, '平台数据源'), (1, 'AD数据源')
    )
    name = models.CharField(verbose_name='ldap_name', max_length=255, blank=True)
    host = models.CharField(verbose_name='host', max_length=255, blank=True)
    port = models.CharField(verbose_name='port', max_length=255, blank=True)
    base_dn = models.CharField(verbose_name='base_dn', max_length=255, blank=True)
    # base_ou = models.CharField(verbose_name='基准ou', max_length=255, null=True, blank=True)
    admin_user_dn = models.CharField(verbose_name='admin', max_length=255, blank=True)
    admin_password = models.CharField(verbose_name='admin_pwd', max_length=255, blank=True)
    ssl = models.IntegerField(verbose_name='ssl连接', default=0, null=True, blank=True)
    ssl_file = models.TextField(verbose_name='ssl证书', null=True, blank=True)
    default_password = models.CharField(verbose_name='默认密码', max_length=255, blank=True)
    point = models.IntegerField(verbose_name='所属节点', null=True, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_update_at = models.DateTimeField(verbose_name='最后修改时间', auto_now=True, null=True, blank=True)
    last_sync_at = models.DateTimeField(verbose_name='最后同步', null=True, blank=True)
    conf = models.TextField(verbose_name='其他配置', null=True, blank=True)
    ldap_type = models.IntegerField(verbose_name='ldap类型', choices=ldap_type_choices, default=1, blank=True)

    class Meta:
        verbose_name = 'ldap'
        verbose_name_plural = verbose_name


class LdapBaseOU(models.Model):
    ldap = models.ForeignKey(verbose_name='ldap_id', to=Ldap, blank=True, on_delete=models.CASCADE)
    base_ou = models.CharField(verbose_name='基准ou', max_length=255, null=True, blank=True)

    class Meta:
        verbose_name = 'ldap_base_ou'
        verbose_name_plural = verbose_name


class LdapSyncStatus(models.Model):
    status = (
        (0, '未迁移'), (1, '已迁移'), (2, '迁移失败'), (3, '迁移中')
    )
    ldap = models.ForeignKey(verbose_name='ldap_id', to=Ldap, blank=True, on_delete=models.CASCADE)
    add_user = models.IntegerField(verbose_name='新增用户数', default=0, blank=True)
    update_user = models.IntegerField(verbose_name='更新用户数', default=0, blank=True)
    fail_user = models.IntegerField(verbose_name='失败用户数', default=0, blank=True)
    add_group = models.IntegerField(verbose_name='新增群组数', default=0, blank=True)
    update_group = models.IntegerField(verbose_name='更新群组数', default=0, blank=True)
    fail_group = models.IntegerField(verbose_name='失败群组数', default=0, blank=True)
    clash_user = models.IntegerField(verbose_name='冲突用户数', default=0, blank=True)
    clash_group = models.IntegerField(verbose_name='冲突群组数', default=0, blank=True)
    sync_at = models.DateTimeField(verbose_name='同步时间', auto_now_add=True, blank=True)
    sync_status = models.IntegerField(verbose_name='同步状态', choices=status, default=0, blank=True)

    class Meta:
        verbose_name = 'ldap迁移详情'
        verbose_name_plural = verbose_name


class LdapAccountEntity(models.Model):
    cn_name = models.CharField(verbose_name='用户名', max_length=255, null=True, blank=True)
    name = models.CharField(verbose_name='用户账号', max_length=255, null=True, blank=True)
    email = models.CharField(verbose_name='邮箱', max_length=255, null=True, blank=True)
    userPrincipalName = models.CharField(verbose_name='userPrincipalName', max_length=255, null=True, blank=True)
    system_status = models.CharField(verbose_name='用户状态', max_length=255, null=True, blank=True)
    guid = models.TextField(verbose_name='ldap中唯一标识ID', null=True, blank=True)
    dn = models.TextField(verbose_name='DN', null=True, blank=True)
    ldap = models.ForeignKey(verbose_name='ldap_id', to=Ldap, blank=True, on_delete=models.CASCADE)
    account_id = models.IntegerField(verbose_name='account_id', null=True)

    class Meta:
        db_table = 'ldaaccount_info'
        verbose_name = '用户ldap中信息'
        verbose_name_plural = '用户ldap中信息'

    def __str__(self):
        return self.name


class LocalUserLogin(models.Model):
    # 范围，0 按部门、1按终端
    range_choices = (
        (0, 'organizations'), (1, 'devices')
    )
    status_choices = (
        (0, 'N'), (1, 'Y')
    )
    status = models.IntegerField(verbose_name='本地用户登陆,0 关、1 开', choices=status_choices, default=0, blank=True)
    range = models.IntegerField(verbose_name='生效范围,0 按部门、1 按终端', choices=range_choices, default=0, blank=True)
    org_id = models.TextField(verbose_name='所属组织机构id', null=True, blank=True)
    device_id = models.TextField(verbose_name='设备id', null=True, blank=True)

    class Meta:
        db_table = 'local_user_login'
        verbose_name = '本地用户登陆规则'
        verbose_name_plural = '本地用户登陆规则'
        ordering = ('pk',)

    def __str__(self):
        return self.pk


class AuthSource(models.Model):
    sync_choices = (
        (0, '关闭'), (1, '开启')
    )
    # cycle_week_choices = (
    #     (0, '每天'), (1, '周一'),(2, '周二'), (3, '周三'),(4, '周四'), (5, '周五'),(6, '周六'),(7, '周日')
    # )
    name = models.CharField(verbose_name='外部源名称', max_length=255, null=True, blank=True)
    addr = models.CharField(verbose_name='外部源地址', max_length=255, null=True, blank=True)
    sync = models.IntegerField(verbose_name='自动同步', choices=sync_choices, default=0, blank=True)
    # sync_week = models.IntegerField(verbose_name='同步周期-周', choices=cycle_week_choices, default=0, blank=True)
    sync_at = models.TimeField(verbose_name='同步周期-时分秒', null=True, blank=True)
    point = models.IntegerField(verbose_name='所属节点', default=1, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)

    class Meta:
        db_table = 'accounts_auth_source'
        verbose_name = '外部认证源'
        verbose_name_plural = '外部认证源'

    def __str__(self):
        return self.name


class AuthRecords(models.Model):
    status = (
        (0, '未迁移'), (1, '已迁移'), (2, '迁移失败'), (3, '迁移中')
    )
    add_user = models.IntegerField(verbose_name='新增用户数', default=0, blank=True)
    update_user = models.IntegerField(verbose_name='更新用户数', default=0, blank=True)
    fail_user = models.IntegerField(verbose_name='失败用户数', default=0, blank=True)
    add_group = models.IntegerField(verbose_name='新增群组数', default=0, blank=True)
    update_group = models.IntegerField(verbose_name='更新群组数', default=0, blank=True)
    fail_group = models.IntegerField(verbose_name='失败群组数', default=0, blank=True)
    clash_user = models.IntegerField(verbose_name='冲突用户数', default=0, blank=True)
    clash_group = models.IntegerField(verbose_name='冲突群组数', default=0, blank=True)
    sync_at = models.DateTimeField(verbose_name='同步时间', auto_now_add=True, blank=True)
    sync_status = models.IntegerField(verbose_name='同步状态', choices=status, default=0, blank=True)
    sync_err_msg = models.TextField(verbose_name='同步异常信息',null=True, blank=True)

    class Meta:
        db_table = 'accounts_auth_records'
        verbose_name = '外部认证源'
        verbose_name_plural = '外部认证源'


class BlackWhiteListsRule(models.Model):
    status_choices = (
        (0, 'N'), (1, 'Y')
    )
    range_choices = (
        (0, '任意'), (1, '全部')
    )
    black_status = models.IntegerField(verbose_name='黑名单,0 关、1 开', choices=status_choices, default=0, blank=True)
    white_status = models.IntegerField(verbose_name='白名单,0 关、1 开', choices=status_choices, default=0, blank=True)
    black_list_range = models.IntegerField(verbose_name='黑名单生效范围', choices=range_choices, default=0, blank=True)
    white_list_range = models.IntegerField(verbose_name='白名单生效范围', choices=range_choices, default=0, blank=True)

    class Meta:
        db_table = 'accounts_black_white_lists_rule'
        verbose_name = '黑白名单规则'
        verbose_name_plural = '黑白名单规则'

class BlackWhiteLists(models.Model):
    list_type = models.CharField(verbose_name='黑白名单', max_length=255, default='black', blank=True)
    op_type = models.CharField(verbose_name='条件', max_length=255, default='equals', blank=True)
    content = models.CharField(verbose_name='名单内容', max_length=255, blank=True)

    class Meta:
        db_table = 'accounts_black_white_lists'
        verbose_name = '黑白名单'
        verbose_name_plural = '黑白名单'


class BackupRestore(models.Model):
    backup_file =models.CharField(verbose_name='备份名称', max_length=255, null=True, blank=True)
    backup_path =models.CharField(verbose_name='备份路径', max_length=255, null=True, blank=True)
    file_size = models.CharField(verbose_name='大小', max_length=255, null=True, blank=True)
    backup_start_time = models.CharField(verbose_name='备份开始时间', max_length=255, null=True, blank=True)
    backup_end_time = models.CharField(verbose_name='备份结束时间', max_length=255, null=True, blank=True)
    is_restore = models.IntegerField(verbose_name='还原', default=0, blank=True)
    restore_start_time = models.CharField(verbose_name='还原开始时间', max_length=255, null=True, blank=True)
    restore_end_time = models.CharField(verbose_name='还原结束时间', max_length=255, null=True, blank=True)

    class Meta:
        db_table = 'accounts_backup_restore'
        verbose_name = '备份还原'
        verbose_name_plural = '备份还原'

class DNSRecords(models.Model):
    record_name = models.CharField(verbose_name='记录名', max_length=255, null=True, blank=True)
    record_type = models.CharField(verbose_name='记录类型', default="A", max_length=255, null=True, blank=True)
    record_data = models.CharField(verbose_name='数据', max_length=255, null=True, blank=True)

    class Meta:
        db_table = 'accounts_dnsrecords'
        verbose_name = 'DNS资源记录'
        verbose_name_plural = 'DNS资源记录'