from django.conf.urls import url
from django.urls import include
from rest_framework.routers import DefaultRouter

from accounts import views

router = DefaultRouter()

router.register('/info', views.AccountViewSet, basename='域账号信息')
router.register('/passwd_rule', views.PasswordAccountRuleViewSet, basename="域用户密码权限校验")
router.register('/local_user_login', views.LocalUserLoginViewSet, basename="本地用户登陆")
router.register('/black_white_lists', views.BlackWhiteListsViewSet, basename="黑白名单")
router.register(r'/ldap/(?P<ldap_id>\d+)/sync/detail', views.LdapSyncStatusViewSet)
router.register('/ldap', views.LdapViewSet)
router.register('/auth_source', views.AuthSourceViewSet)
router.register('/passwd_prohibit_mod', views.PwdProhibitModificationViewSet)
router.register('/soap_records', views.AuthRecordsViewSet, basename='soap数据同步记录')
router.register('/backup_restore', views.BackupRestoreViewSet, basename='ldap备份还原')
router.register('/dns_record', views.DNSRecordstoreViewSet, basename='DNS资源记录')

urlpatterns = [
    url(r'^', include(router.urls)),
    url(r'^/$', views.AccountList.as_view()),
    url(r'^/passwd$', views.AccountResetPwd.as_view()),
    url(r'^/status', views.AccountStatus.as_view()),
    url(r'^/bind', views.AccountBindMachine.as_view()),
    url(r'^/ipa/test', views.FreeIPATest.as_view()),
    url(r'^/(?P<uid>[0-9]+)$', views.AccountDetail.as_view()),
    url(r'^/security$', views.AccountSecurity.as_view()),
    # 批量导入
    url(r'^/batch/(?P<filename>[^/]+)$', views.AccountBatch.as_view()),
    # 批量导入模板下载
    url(r'^/batch/tmpl/(?P<filename>[^/]+)$', views.AccountBatchTmpl.as_view()),
    # ldap导入
    url(r'^/batch/ldap/(?P<filename>[^/]+)$', views.AccountBatchImportLDAP.as_view()),
    # 导出excel
    url(r'^/export$', views.AccountExport.as_view()),

    url(r'^/ldap/ssl_file$', views.LdapSSLFile.as_view()),
]
