import configparser
import copy
import datetime
import io
# Create your views here.
import logging
import json
import re
import threading

import openpyxl.styles
import pandas as pd
import pytz
from django.db import close_old_connections
from django.http import StreamingHttpResponse
from django.conf import settings
from django_filters.rest_framework import DjangoFilterBackend
from django_redis import get_redis_connection
from openpyxl.styles import Font

from rest_framework.views import APIView
from rest_framework.parsers import FormParser  # 解析urlencoded数据包
from rest_framework.parsers import MultiPartParser  # 解析from-data数据包
from rest_framework.views import Response
from rest_framework import status
from rest_framework.viewsets import ModelViewSet
from rest_framework.filters import SearchFilter, OrderingFilter
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.decorators import api_view, permission_classes, authentication_classes

from accounts.serializers import AccountSerializer, PasswordAccountRuleSerializer, LdapSerializer, \
    LdapSyncStatusSerializer, LocalUserLoginSerializer, PwdProhibitModificationSerializer, AuthSourceSerializer, \
    AuthRecordsSerializer, BlackWhiteListsSerializer, BlackWhiteListsRuleSerializer, BackupRestoreSerializer, DNSRecordsSerializer
from accounts.tasks import synchro_data_soap
from accounts.ldaps import get_ldap_ad
from celery_task.celery import app
from devices.serializers import DeviceSerializer
from devices.web_consumers import send_msg_to_web
from message_center.models import MessageCenter
from nfs_domain_mainserver.settings import SALTFILESYSTEMPATH
from users.models import User
from devices.models import Devices
from users.models import User
from users.role_permisson import get_org_permission_list
from utils.authentication import TokenAuthentication, apis_authentication
from utils.check_passwd import check_account
from utils.pagination import MyPagination
from utils.response import FitJSONRenderer
from utils.throttling import UserRateThrottle

from utils.sch_soap_task_member import SoapMember, CONFIG_PATH, get_soapmember, engine
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from django.core.cache import caches
from django_celery_beat.models import PeriodicTask, CrontabSchedule

from rest_framework.pagination import PageNumberPagination
from accounts.models import AccountEntity, PasswordAccountRule, Ldap, LdapBaseOU, LdapSyncStatus, LdapAccountEntity, \
    LocalUserLogin, LocalUserLogin, PasswordAccountProhibitModification, AuthSource, AuthRecords, BlackWhiteLists, \
    BlackWhiteListsRule, BackupRestore, DNSRecords
from organizations.models import OrgnizationEntity
from accounts.freeipa import (
    add_user, reset_user_pwd, user_status_disable,
    user_status_enable, hbacrule_add, hbacrule_del, hbacrule_add_user,
    hbacrule_add_host, hbacrule_add_service, hbacrule_find,
    pwpolicy_show, pwpolicy_mod, change_passwd, user_del, user_unlock,
    user_find, ipa_backup_restore, check_user
)
import accounts.freeipa as freeipa
from accounts.ldaps import LdapAD
from accounts.filters import LdapFilterSet, BlackWhiteListsFilterSet, DNSRecordsFilterSet
from organizations.views import DEPT_ROOT_ID
from rest_framework.decorators import action
from saltapi.tasks import lock_local_user, unlock_local_user, update_cache_sssd_policy, domain_user_unlock_policy
from saltapi.utils import SaltAPI, DefaultPolicy
from utils.utils import get_device, diff_list, sm4_hash, sm4_decrypt, test_ip_port_connection

logger = logging.getLogger(__name__)
scheduler = None
redis_conn = get_redis_connection('redis')


class AccountList(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    @classmethod
    def organ_down_list(cls, num):
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return [int(num)]
        else:
            organ_down_list = organ_down_list + list(items.values_list('id', flat=True))
            while True:
                # logger.info(f'**********{items}*********')
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('id', flat=True))

            organ_down_list.append(int(num))
        return organ_down_list

    # 终端用户分页查询
    def get(self, request, *args, **kwargs):
        device_user_active_dict_str = redis_conn.hgetall("device_user_active")

        # logger.info("device_user_active_dict %s" % list(device_user_active_dict.values()))

        accounts = AccountEntity.objects.all()
        # 根据部门过滤数据，超级管理员不需要
        user = User.objects.filter(username=request.user.username).first()
        org_list = get_org_permission_list(user)

        # 根据部门筛选数据
        if org_list:
            accounts = accounts.filter(org_info_id__in=org_list)

        query_name = request.query_params.get('name')
        if query_name:
            accounts = accounts.filter(name__contains=query_name)

        query_cn_name = request.query_params.get('cn_name')
        if query_cn_name:
            accounts = accounts.filter(cn_name__contains=query_cn_name)

        query_phone = request.query_params.get('phone')
        if query_phone:
            accounts = accounts.filter(phone__contains=query_phone)

        query_user_status = request.query_params.get('user_status')
        if query_user_status:
            accounts = accounts.filter(user_status__exact=query_user_status)

        query_org_id = request.query_params.get('org_id')
        if query_org_id:
            if int(user.org_global_switch_user) == 1:
                org_id_list = [query_org_id]
            else:
                org_id_list = self.organ_down_list(query_org_id)
                # logger.info(f'**********************{org_id_list}')

            accounts = accounts.filter(org_info_id__in=org_id_list)

        query_is_active = request.query_params.get('is_active')
        active_user_list = []
        if device_user_active_dict_str:
            for k, v in device_user_active_dict_str.items():
                # logger.info("v %s" % v)
                if v:
                    active_user_list += json.loads(v.decode('utf-8'))
        # logger.info("active_user_list %s" % active_user_list)

        if query_is_active is not None:
            if int(query_is_active) == 1:
                accounts = accounts.filter(name__in=active_user_list)
            else:
                accounts = accounts.exclude(name__in=active_user_list)

        query_office_address = request.query_params.get('office_address')
        if query_office_address:
            accounts = accounts.filter(office_address__contains=query_office_address)

        accounts = accounts.order_by("-id")
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(accounts, request)
        except:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': 'error',
                'success': False,
                'data': []
            })
        res_list = []
        for account in ret:
            is_active = 0
            if active_user_list:
                if account.name in active_user_list:
                    is_active = 1

            path = [account.org_info.name]
            parent = account.org_info.parent
            while parent.id != 1:
                path.insert(0, parent.name)
                parent = parent.parent
            org_info = ' / '.join(path)

            res_list.append({
                'uid': account.id,
                'cn_name': account.cn_name,
                'name': account.name,
                'phone': account.phone,
                'user_status': account.user_status,
                'org_id': account.org_info.id,
                'org_name': account.org_info.name,
                'org_info': org_info,
                'office_address': account.office_address,
                'is_active': is_active,
                "create_type": account.create_type,
                "last_login_time":account.last_login_time
            })
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '完成终端用户分页查询',
            'total': len(accounts),
            'success': True,
            'data': res_list
        })

    # 新增终端用户
    def post(self, request, *args, **kwargs):
        req_data = request.data
        is_ok, msg = check_account(req_data.get('name'))
        if not is_ok:
            return Response({
                'code': status.HTTP_202_ACCEPTED,
                'msg': msg
            })
        if 'password' in req_data.keys():
            password = req_data.get('password')
        else:
            password = None

        account = AccountEntity()
        account.cn_name = req_data.get('cn_name')
        account.name = req_data.get('name')
        account.phone = req_data.get('phone')
        account.user_status = req_data.get('user_status')
        account.address = req_data.get('address')
        account.remarks = req_data.get('remarks')
        account.org_info_id = req_data.get('org_info_id')
        account.email = req_data.get('email')
        account.job_no = req_data.get('job_no')
        account.office_address = req_data.get('office_address')
        account.policy_status = 1
        # 判断用户名是否存在
        check_set = AccountEntity.objects.filter(name=req_data['name'])
        if check_set:
            return Response({
                'code': status.HTTP_202_ACCEPTED,
                'msg': '终端用户已存在'
            })
        # 判断部门信息
        check_org_set = OrgnizationEntity.objects.filter(pk=req_data['org_info_id'])
        if check_org_set:
            pass
        else:
            return Response({
                'code': status.HTTP_202_ACCEPTED,
                'msg': '部门信息错误'
            })
        account.cn_name = account.name if account.cn_name is None else account.cn_name

        # logger.info(f'{account.name}____{password}')
        # 创建域用户
        if not add_user(account.name, account.cn_name, password):
            return Response({
                'code': status.HTTP_202_ACCEPTED,
                'msg': f'检查用户 {account.name} 是否已存在 ipa 中'
            })

        # 是否赋予root权限
        default_policy = DefaultPolicy()
        if int(default_policy.get_default_policy().get('root', 0)) == 1:
            account.is_root = 1
            freeipa.get_ipa_client().sudorule_add_user('sudo_admin', o_user=req_data.get('name'))
        else:
            account.is_root = 0
        account.save()
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '新增终端用户完成'
        })


class AccountDetail(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    # 终端用户详细信息
    def get(self, request, *args, **kwargs):
        device_user_active_dict_str = redis_conn.hgetall("device_user_active")
        uid = kwargs.get('uid')
        try:
            account = AccountEntity.objects.get(pk=uid)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '终端用户信息不存在'
            })
        # ser_account = AccountEntitySerializer(account, many=True)
        active_user_list = []
        if account:
            if device_user_active_dict_str:
                for k, v in device_user_active_dict_str.items():
                    # logger.info("v %s" % v)
                    if v:
                        active_user_list += json.loads(v.decode('utf-8'))
            is_active = 0
            if active_user_list:
                if account.name in active_user_list:
                    is_active = 1
            res_obj = {
                'uid': account.id,
                'cn_name': account.cn_name,
                'name': account.name,
                'phone': account.phone,
                'user_status': account.user_status,
                'address': account.address,
                'remarks': account.remarks,
                'job_no': account.job_no,
                'org_id': account.org_info.id,
                'org_name': account.org_info.name,
                'email': account.email,
                'office_address': account.office_address,
                'is_active': is_active,
                'create_type': account.create_type,
            }
            return Response({
                'code': status.HTTP_200_OK,
                'msg': '获取终端信息完成',
                'data': res_obj
            })
        else:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '获取终端信息发生错误',
                'data': {}
            })

    # 更新终端用户信息
    def put(self, request, *args, **kwargs):
        uid = kwargs.get('uid')
        req_data = request.data
        try:
            account = AccountEntity.objects.get(pk=uid)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '终端用户信息不存在'
            })
        old_cn_name = account.cn_name
        if req_data.get('cn_name'):
            account.cn_name = req_data.get('cn_name')
        # account.name = req_data['name']
        if req_data.get('phone'):
            account.phone = req_data.get('phone')
        if req_data.get('user_status') is not None:
            account.user_status = req_data.get('user_status')
        if req_data.get('address'):
            account.address = req_data.get('address')
        if req_data.get('remarks'):
            account.remarks = req_data.get('remarks')
        if req_data.get('org_id'):
            account.org_info_id = req_data.get('org_id')
        if req_data.get('job_no'):
            account.job_no = req_data.get('job_no')
        if req_data.get('email'):
            account.email = req_data.get('email')
        if req_data.get('office_address'):
            account.office_address = req_data.get('office_address')
        account.save()

        if old_cn_name != req_data.get('cn_name'):
            freeipa.user_mod(account.name, req_data.get('cn_name'))

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '更新终端用户信息已完成'
        })

    # 删除终端用户
    def delete(self, request, *args, **kwargs):
        uid = kwargs.get('uid')
        try:
            account = AccountEntity.objects.get(pk=uid)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '终端用户信息不存在'
            })
        account.delete()
        # ipa 删除
        user_del(account.name)

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '用户已删除'
        })


# 重置终端用户密码
class AccountResetPwd(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def put(self, request, *args, **kwargs):
        req_data = request.data
        uid = req_data.get('uid')
        new_passwd1 = req_data.get('new_passwd1')
        new_passwd2 = req_data.get('new_passwd2')
        if new_passwd1 != new_passwd2:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '两次密码不一致'
            })
        try:
            account = AccountEntity.objects.get(pk=uid)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '终端用户信息不存在'
            })
        if account:
            reset_user_pwd(account.name, passwd=new_passwd1)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '重置密码成功',
            'data': []
        })


# 更改终端用户状态
class AccountStatus(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def put(self, request, *args, **kwargs):
        req_data = request.data
        uid = req_data.get('uid')
        u_status = req_data.get('status')

        try:
            account = AccountEntity.objects.get(pk=uid)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '终端用户信息不存在'
            })

        if account:
            account.user_status = u_status
            account.save()
            if u_status == 1:
                user_status_disable(account.name)
            if u_status == 0:
                user_status_enable(account.name)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '更新终端用户状态信息成功',
            'data': []
        })


# 绑定 终端设备
class AccountBindMachine(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def post(self, request, *args, **kwargs):
        req_data = request.data
        uid = req_data.get('uid')
        mid = req_data.get('mid')
        # 更新终端设备
        # 创建IPA规则

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '终端用户和设备绑定成功',
            'data': []
        })


class FreeIPATest(APIView):

    def get(self, request, *args, **kwargs):
        rule_name = 'test rule1'
        user_name = 'zzz2'
        host_name = 'mynfs4888.freeipa.cn'
        hbacsvc = ['gdm', 'gdm-password']
        # hbacrule_add( rule_name )
        # hbacrule_add_user(rule_name, user_name)
        # hbacrule_add_host(rule_name, host_name)
        # res = hbacrule_add_service(rule_name, hbacsvc)
        # res = add_user('abcdd','王二')
        # res = hbacrule_del(rule_name)
        logger.info('======================= TEST =======================')
        # res = hbacrule_add(rule_name)
        # res2 = hbacrule_add_host(rule_name, host_name)
        # res3 = hbacrule_add_user(rule_name, user_name)
        # # # res = change_passwd('zod', 'a12345', 'aabb12345')
        # logger.info(json.dumps(res, indent=4))
        # logger.info(json.dumps(res2, indent=4))
        # logger.info(json.dumps(res3, indent=4))

        # freeipa.hbac_add_user_to_host(6, host_name)
        res = freeipa.hbacrule_find('hbac_mynfs4888.freeipa.cn')
        logger.info(json.dumps(res, indent=4))

        return Response({
            'code': status.HTTP_200_OK,
            'msg': 'freeipa test ok',
            # 'data': res,
        })


class AccountSecurity(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, *args, **kwargs):
        # res = pwpolicy_show('ipausers')
        res = pwpolicy_show()
        result = res.get('result')

        def _get_value(res, key):
            return int(res.get(key)[0]) if res.get(key) else None

        # 上游接口参数是秒，前端使用分钟
        # 锁定时间最大显示99999分钟，小于1分钟时返回1
        lock_dur = _get_value(result, 'krbpwdlockoutduration')
        lock_dur = 99999 if lock_dur / 60 >= 99999 else lock_dur / 60
        lock_dur = 1 if lock_dur < 1 else lock_dur

        max_life = _get_value(result, 'krbmaxpwdlife')
        max_life = 180 if max_life >= 180 else max_life
        data = dict(
            maxpwdlife=max_life,
            minpwdlife=_get_value(result, 'krbminpwdlife'),
            pwdhistorylength=_get_value(result, 'krbpwdhistorylength'),
            pwdmindiffchars=_get_value(result, 'krbpwdmindiffchars'),
            pwdminlength=_get_value(result, 'krbpwdminlength'),
            pwdmaxfailure=_get_value(result, 'krbpwdmaxfailure'),
            pwdfailurecountinterval=_get_value(result, 'krbpwdfailurecountinterval'),
            pwdlockoutduration=lock_dur,
        )

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '获取用户安全配置成功',
            'data': data,
        })

    def put(self, request, *args, **kwargs):
        req_data = request.data

        """
        "maxpwdlife": 90,
        "minpwdlife": 1,
        "pwdhistorylength": 0,
        "pwdmindiffchars": 0,
        "pwdminlength": 8,
        "pwdmaxfailure": 6,
        "pwdfailurecountinterval": 60,
        "pwdlockoutduration": 600
        """
        # params = dict(a_cn = 'ipausers')
        params = dict(a_cn='global_policy')

        # 密码过期时间大于180时，配置为最大值20000天
        if req_data.get('maxpwdlife') is not None:
            max_life = req_data.get('maxpwdlife')
            max_life = max_life if max_life < 180 else 20000
            params['o_krbmaxpwdlife'] = max_life

        if req_data.get('minpwdlife') is not None:
            params['o_krbminpwdlife'] = req_data.get('minpwdlife')

        if req_data.get('pwdhistorylength') is not None:
            params['o_krbpwdhistorylength'] = req_data.get('pwdhistorylength')

        # if req_data.get('pwdnosimple') is not None:
        #     pwdnosimple = 4 if req_data.get('pwdnosimple') else 0
        #     params['o_krbpwdmindiffchars'] = pwdnosimple

        if req_data.get('pwdmindiffchars') is not None:
            params['o_krbpwdmindiffchars'] = req_data.get('pwdmindiffchars')

        if req_data.get('pwdminlength') is not None:
            params['o_krbpwdminlength'] = req_data.get('pwdminlength')

        if req_data.get('pwdmaxfailure') is not None:
            params['o_krbpwdmaxfailure'] = req_data.get('pwdmaxfailure')

        if req_data.get('pwdfailurecountinterval') is not None:
            params['o_krbpwdfailurecountinterval'] = req_data.get('pwdfailurecountinterval')

        if req_data.get('pwdlockoutduration') is not None:
            lock_dur = req_data.get('pwdlockoutduration')
            lock_dur = lock_dur * 60 if lock_dur < 99999 else 2147483647
            params['o_krbpwdlockoutduration'] = lock_dur

        if len(params) > 1:
            res = pwpolicy_mod(**params)

            if res.get('msg') is not None:
                return Response({
                    'code': status.HTTP_500_INTERNAL_SERVER_ERROR,
                    'msg': res.get('msg'),
                })
            else:
                return Response({
                    'code': status.HTTP_200_OK,
                    'msg': '修改成功',
                })

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '没有变更',
        })


class AccountBatch(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)
    parsers_classes = [FormParser, MultiPartParser]

    def build_and_get_structure(self, orgs_str):
        if not orgs_str:
            return OrgnizationEntity.objects.get(pk=DEPT_ROOT_ID)
        orgs = [o.strip() for o in orgs_str.split('/')]
        parent_org = None
        # 逐级创建部门
        for i, org in enumerate(orgs):
            parent_org, created = OrgnizationEntity.objects.get_or_create(
                name=org,
                parent_id=DEPT_ROOT_ID if (i == 0) else parent_org.id,
            )
        return parent_org

    @staticmethod
    def switch_global_root(name, global_default_root):
        if global_default_root == 1:
            freeipa.get_ipa_client().sudorule_add_user('sudo_admin', o_user=name)
            return 1
        else:
            return 0

    def xlsx_to_freeipa_db(self, df, filename, total_num):
        default_policy = DefaultPolicy()
        global_default_root = int(default_policy.get_default_policy().get('root', 0))
        # 手机号为选填，不符合规则时存储为空
        df['phone'] = df['phone'].apply(lambda x: x if bool(re.match(r'^\d{10}$', str(x))) else None)
        db_name_list = AccountEntity.objects.values_list('name', flat=True)
        # 筛选出不在账号库表中的数据
        filtered_df = df[~df['name'].isin(db_name_list)]
        # 过滤符合邮箱规则的账号
        pattern = r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$'
        # 使用正则表达式过滤出符合规则的数据
        filtered_df['email'] = filtered_df['email'].apply(lambda x: x if bool(re.match(pattern, str(x))) else None)
        if len(filtered_df) != 0:
            filtered_df['password'] = filtered_df['password'].apply(lambda x: x if x else None)
            # 创建freeipa账号
            filtered_df['add_freeipa_result'] = filtered_df.apply(
                lambda row: add_user(row['name'], row['cn_name'], row['password']), axis=1)
            filtered_df['org_info_id'] = filtered_df['orgs'].apply(lambda x: self.build_and_get_structure(x).id)
            filtered_df['system_status'] = filtered_df['system_status'].apply(lambda x: 1 if x == '是' else 0)
            # 赋予账号root权限
            filtered_df['is_root'] = filtered_df['name'].apply(
                lambda x: self.switch_global_root(x, global_default_root))
            filtered_df = filtered_df[filtered_df['add_freeipa_result'] == True]
            filtered_df['user_status'] = 0
            filtered_df['policy_status'] = 0
            filtered_df['is_extra_status'] = 0
            filtered_df['create_type'] = 0
            filtered_df['created_at'] = datetime.datetime.now()
            selected_columns = filtered_df[['name', 'cn_name', 'phone', 'address', 'office_address', 'remarks', 'email',
                                            'job_no', 'system_status', 'org_info_id', 'is_root', 'user_status',
                                            'policy_status', 'is_extra_status', 'create_type', 'created_at']]
            # logging.info(f"{selected_columns.to_dict('records')}")
            selected_columns.to_sql('account_info', engine, index=False, if_exists='append')
        else:
            logging.info("没有符合要求的数据")

        count_num = len(filtered_df)

        # 将已完成的消息发送给前端websocket
        websocket_msg = {
            'content': f"{filename}已导入完成，成功{count_num}条数据，失败{total_num - count_num}条数据，失败原因用户名，手机号或者邮箱格式不正确"
            if (
                           total_num - count_num) > 0 else f"{filename}已导入完成，成功{count_num}条数据，失败{total_num - count_num}条数据",
            'create_time': datetime.datetime.now(),
            'target_module': '人员导入',
            'message_user': self.request.user.username,
            'message_device': None,
            'download_path': None,
            'is_read': 0,
            'is_broadcast': 0,
        }
        MessageCenter.objects.create(**websocket_msg)
        send_msg_to_web(websocket_msg, 0, self.request.user.username)
        close_old_connections()

    def put(self, request, filename, format=None):
        if not filename.endswith('.xlsx'):
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '请上传xlsx格式文件',
            })
        xlsx_file = request.FILES['file'].read()
        df = pd.read_excel(xlsx_file, engine='openpyxl')
        df = df.iloc[:, :12]
        df = df.dropna(how='all')  # 删除所有完全为空的行
        df = df.dropna(axis=1, how='all') # 删除所有完全为空的列
        logger.info(f"*****\r\n{df.info}")
        df.columns = ['name', 'cn_name', 'orgs', 'password', 'phone', 'email', 'office_address', 'job_no', 'address',
                      'remarks', 'system_status', 'directions']

        # 合规性检查
        if df['name'].isnull().any():
            logger.info(f"*****{df['name']}")
            return Response({'code': status.HTTP_400_BAD_REQUEST, 'msg': '导入失败，用户账号为必输项', 'data': None})
        lengths = df['name'].str.len()
        if any((length < 3 or length > 32) for length in lengths):
            return Response({'code': status.HTTP_400_BAD_REQUEST,
                             'msg': '导入失败，用户账号长度小于3个字符或者超过规定32个字符不可导入', 'data': None})
        if df['cn_name'].isnull().any():
            return Response({'code': status.HTTP_400_BAD_REQUEST, 'msg': '导入失败，姓名为必输项', 'data': None})
        if df['orgs'].isnull().any():
            return Response({'code': status.HTTP_400_BAD_REQUEST, 'msg': '导入失败，所在部门为必输项', 'data': None})
        if df['system_status'].isnull().any():
            return Response({'code': status.HTTP_400_BAD_REQUEST, 'msg': '导入失败，是否立即启用为必输项', 'data': None})
        # 排除示例数据
        if not df.empty:
            # 检查第0行是否在 DataFrame 中
            if 0 in df.index:
                # 排除第0行数据
                df = df.drop(index=0)
        if len(df) == 0:
            return Response({'code': status.HTTP_400_BAD_REQUEST, 'msg': '导入失败，没有需要导入的数据', 'data': None})

        total_num = len(df)

        # 后台处理用户导入
        thread1 = threading.Thread(target=self.xlsx_to_freeipa_db, args=(df, filename, total_num))
        thread1.start()

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '人员导入中，请稍后前往消息中心查看',
            'data': None,
        })


class AccountBatchTmpl(APIView):
    # authentication_classes = []
    # permission_classes = (AllowAny,)
    # throttle_classes = (UserRateThrottle,)
    parsers_classes = [FormParser, MultiPartParser]

    def get(self, request, *args, **kwargs):
        from utils.utils import write_xls
        from django.shortcuts import HttpResponse

        head = ['用户账号\r\n(必填)', '姓名\r\n(必填)', '所在部门\r\n(必填)', '密码\r\n(选填)', '联系电话\r\n(选填)',
                '邮箱\r\n(选填)', '办公地点\r\n(选填)', '工号\r\n(选填)',
                '所在地区\r\n(选填)', '备注\r\n(选填)',
                '是否立即启用\r\n(必填)', '填写说明']
        data = [
            ['nfschina', '系统账号', '部门1/部门1-1', '12345678', '138********', '****@nfs.com', '9楼703', '768282245',
             '湖北省-武汉市-洪山区', '这是示例', '是', '1.用户账号必填，不可重复，用户账号长度3~32位，允许输入中英文，数字或特殊字符;\r\n'
                                                       '2.姓名必填，用户名不得超过32字符，允许输入中英文，数字或特殊字符;\r\n'
                                                       '3.所属部门必填，可直接填写部门名称，或者按照上级部门/下级部门的方式填写；\r\n'
                                                       '4.密码非必填，默认密码12345678，账号密码长度8~32位，允许输入字母，数字或特殊字符;\r\n'
                                                       '5.联系电话非必填，允许输入11位数字；\r\n'
                                                       '6.邮箱（填写说明）：非必填,邮箱填写需校验必须有@符号;不允许整个地址超过 254 个字符；\r\n'
                                                       '''7.办公地点（填写说明）：非必填，限制中英文、数字、特殊字符（~!@#$%^&*( )_ -={}|[]\:"";'<>?,./ ）输入，最多32字符;\r\n'''
                                                       '8.工号非必填，不可重复，工号不得超过30字符，允许输入字母，数字或特殊字符;\r\n'
                                                       '9.所在地区非必填，填写方式为省（自治州）-市（县）-区；\r\n'
                                                       '10.备注非必填，备注不得超过100字，允许输入中英文，数字或特殊字符;\r\n'
                                                       '11.是否立即启用，必填，默认为启用。']
        ]

        wb = write_xls('用户列表', head, data)
        ws = wb.active
        header_font = Font(color='FF0000')
        ws['A1'].font = header_font
        ws['B1'].font = header_font
        ws['C1'].font = header_font
        ws['K1'].font = header_font
        ws.column_dimensions['L'].width = 60
        for r in ws:
            for c in r:
                c.alignment = openpyxl.styles.Alignment(wrapText=True, horizontal='center', vertical='center')
        response = HttpResponse(content_type='application/ms-excel')
        # response['Content-Disposition'] = 'attachment; filename=用户导入模板.xlsx'
        wb.save(response)
        return response


# 导出用户
class AccountExport(APIView):
    # authentication_classes = []
    # permission_classes = (AllowAny,)
    # throttle_classes = (UserRateThrottle,)
    parsers_classes = [FormParser, MultiPartParser]

    def get(self, request, *args, **kwargs):
        from utils.utils import write_xls
        from django.shortcuts import HttpResponse
        accounts = AccountEntity.objects.all().order_by("-id")

        head = ['姓名', '用户账号', '手机号', '所在部门']
        data = []
        for account in accounts:
            account_info = [account.cn_name, account.name, account.phone, account.org_info.name]
            data.append(account_info)
        wb = write_xls('zdyh', head, data)
        logger.info('export all account info to excel1')
        # response = HttpResponse(content_type='application/ms-excel')
        response = HttpResponse(content_type='application/octet-stream')
        response['Content-Disposition'] = 'attachment; filename=zdyh.xlsx'
        wb.save(response)
        return response


# 解析LDAP导出的excel
class AccountBatchImportLDAP(APIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,IsAdminUser)
    # throttle_classes = (UserRateThrottle,)
    parsers_classes = [FormParser, MultiPartParser]

    def build_and_get_structure(self, orgs_str):
        if not orgs_str:
            return OrgnizationEntity.objects.get(pk=DEPT_ROOT_ID)
        orgs = [o.strip() for o in orgs_str.split('@@')]
        parent_org = None
        # 逐级创建部门
        for i, org in enumerate(orgs):
            parent_org, created = OrgnizationEntity.objects.get_or_create(
                name=org,
                parent_id=DEPT_ROOT_ID if (i == 0) else parent_org.id,
            )
        return parent_org

    def put(self, request, filename, format=None):
        from openpyxl import load_workbook
        from io import BytesIO

        if not filename.endswith('.xlsx'):
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '请上传xlsx格式文件',
            })
        file = request.FILES['file'].read()
        wb = load_workbook(filename=BytesIO(file))

        logger.info(wb.worksheets)
        rows = wb.worksheets[0].rows
        total = 0
        count = 0
        for i, row in enumerate(rows):
            # 跳过第一行
            if (i == 0): continue
            total += 1

            objectCategory = row[27].value
            logger.info(objectCategory)
            if objectCategory and objectCategory.find('CN=Person') != -1:
                logger.info('category is person')
            else:
                logger.info('category is other')
                continue

            dn = row[0].value
            logger.info(dn)
            dn = dn[dn.find('OU'):dn.find('DC') - 1]
            dns = dn.split(',')
            orgs_arr = []
            for d in dns:
                if d is not None:
                    ou = d[d.find("=") + 1:]
                    orgs_arr.append(ou)
            orgs_arr.reverse()
            orgs = '@@'
            orgs = orgs.join(orgs_arr)
            logger.info(orgs)
            sAMAccountName = row[15].value
            logger.info(sAMAccountName)
            cn = row[19].value
            logger.info(cn)
            # name = row[24].value
            # logger.info(name)

            # displayName = row[31].value
            # logger.info(displayName)
            sn = row[32].value
            logger.info(sn)
            mail = row[45].value
            logger.info(mail)
            givenName = row[46].value
            logger.info(givenName)
            logger.info("#################################")

            cn_name = cn
            name = sAMAccountName

            # 没有用户名
            if not name: continue

            # 判断用户名是否存在
            check_set = AccountEntity.objects.filter(name=name)
            if check_set: continue

            account = AccountEntity()
            account.cn_name = cn_name
            account.name = name

            # 构建机构的路径
            org = self.build_and_get_structure(orgs)
            account.org_info = org

            account.save()
            count += 1

            # add_user(account.name, name if cn_name is None else cn_name)

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '批量导入成功',
            'data': {
                'total': total,
                'success': count,
            },
        })


class AccountViewSet(ModelViewSet):
    queryset = AccountEntity.objects.all()
    serializer_class = AccountSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    search_fields = ('cn_name', 'name', 'office_address',)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('-id',)

    def get_account_info(self):
        queryset = self.filter_queryset(self.get_queryset())
        serializer = self.get_serializer(queryset, many=True)
        res_data = json.dumps(serializer.data, ensure_ascii=False)
        redis_conn.hset('web_cache', 'account_info', res_data)
        close_old_connections()
        return res_data

    def list(self, request, *args, **kwargs):
        """缓存数据到redis"""
        redis_account_info = redis_conn.hget('web_cache', 'account_info')
        if redis_account_info:
            redis_account_info = redis_account_info.decode('utf-8')
            background_thread = threading.Thread(target=self.get_account_info)
            background_thread.start()
            return Response(json.loads(redis_account_info))
        else:
            queryset = self.queryset.all()[0:2]
            serializer = self.get_serializer(queryset, many=True)
            background_thread = threading.Thread(target=self.get_account_info)
            background_thread.start()
            return Response(serializer.data)

    def batch_filter_queryset(self, queryset):
        """
        :param queryset: 表数据结果集合
        :return: 根据过滤条件筛选返回结果集合
        """
        key = "device_user_active"
        cache = caches['redis']
        device_user_active_dict = cache.get(key)

        query_cn_name = self.request.query_params.get('cn_name', default=None)
        query_name = self.request.query_params.get('name', default=None)

        query_org_id = self.request.query_params.get('org_id', default=None)
        query_phone = self.request.query_params.get('phone', default=None)
        query_office_address = self.request.query_params.get('office_address', default=None)
        query_user_status = self.request.query_params.get('user_status', default=None)
        query_is_active = self.request.query_params.get('is_active', default=None)

        if query_cn_name:
            queryset = queryset.filter(cn_name__icontains=query_cn_name)

        if query_name:
            queryset = queryset.filter(name__icontains=query_name)

        if query_org_id:
            # 判断是否查询直属部门
            if self.request.user.org_global_switch_user == 1:
                queryset = queryset.filter(org_info_id=query_org_id)
            else:
                org_down_list = AccountList().organ_down_list(query_org_id)
                queryset = queryset.filter(org_info_id__in=org_down_list)

        if query_phone:
            queryset = queryset.filter(phone__icontains=query_phone)

        if query_office_address:
            queryset = queryset.filter(office_address__icontains=query_office_address)

        if query_user_status:
            queryset = queryset.filter(user_status=int(query_user_status))

        if query_is_active:
            active_user_list = []
            if device_user_active_dict:
                for k, v in device_user_active_dict.items():
                    if v:
                        active_user_list += v
            if int(query_is_active) == 1:
                queryset = queryset.filter(name__in=active_user_list)
            else:
                queryset = queryset.exclude(name__in=active_user_list)

        return queryset

    def download_account_info(self, queryset):
        queryset = queryset.values_list('id', 'cn_name', 'name', 'org_info__name', 'office_address', 'phone',
                                        'system_status', 'email', 'job_no', 'address', 'remarks',"last_login_time")
        head_data = [u'主键', u'姓名', u'用户账号', u'所属部门', u'办公地点', u'联系电话', u'用户状态', u'邮箱',
                     u'工号',
                     u'所在地址', u'备注',u'用户最后登录时间']

        df = pd.DataFrame(queryset, columns=head_data)
        df['用户状态'] = df['用户状态'].apply(lambda x: '启用' if x == 1 else '禁用')
        df['可登录终端'] = df['主键'].apply(
            lambda x: ';\r\n'.join(
                Devices.objects.filter(bind_account=x).values_list('device_hostname', flat=True)))
        df.drop(columns=['主键', ], inplace=True)

        filename = f'nfs_domain_人员导出_{datetime.datetime.now().strftime("%Y%m%d%H%M%S")}.xlsx'
        df.to_excel(f"{SALTFILESYSTEMPATH}/{filename}", encoding='utf-8', index=False)

        # 将文件已经下载的消息发送给前端websocket
        websocket_msg = {
            'content': f"人员已导出完成, {filename}，总共{len(queryset)}条数据",
            'create_time': datetime.datetime.now(),
            'target_module': '人员导出',
            'message_user': self.request.user.username,
            'message_device': None,
            'download_path': f'/mainapi/client/{filename}/',
            'is_read': 0,
            'is_broadcast': 0,
        }
        MessageCenter.objects.create(**websocket_msg)
        send_msg_to_web(websocket_msg, 0, self.request.user.username)
        close_old_connections()

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def account_batch_operation(self, request, *args, **kwargs):
        """
        域账号批量 导出，删除
        :param request:body {
                        is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        is_delete : 0|不删除 1|删除 空|不处理
                        is_status : 1|禁用 0|启用 2|解锁 空|不处理
                        is_rest_passwd : 0 不修改 1 修改
                        to_organ_id: 目标部门 空|不处理
                        is_down_file: 0|不下载 1|下载 空|不处理
                        filename: 文件名
                        ......: 筛选条件
                        }
        :param args:
        :param kwargs:
        :return:
        """
        is_all = request.data.get('is_all')
        ids_list = request.data.get('ids_list')
        is_delete = request.data.get('is_delete')
        is_status = request.data.get('is_status')
        to_organ_id = request.data.get('to_organ_id')
        is_down_file = request.data.get('is_down_file')
        filename = request.data.get('filename')
        is_rest_passwd = request.data.get('is_rest_passwd')
        new_passwd1 = request.data.get('new_passwd1')
        new_passwd2 = request.data.get('new_passwd2')

        if is_all is None:
            return Response('is_all 为必输选项', status=status.HTTP_400_BAD_REQUEST)

        is_all = int(is_all)

        queryset = self.queryset

        if is_all == 0:
            try:
                ids_list = [int(i) for i in ids_list.split(',')]
            except Exception as exp:
                return Response(f'ids_list参数不符合规范{exp}', status=status.HTTP_400_BAD_REQUEST)
        else:
            queryset = self.batch_filter_queryset(queryset)
            ids_list = queryset.values_list('id', flat=True)

        # 查询过滤结果集
        queryset = queryset.filter(id__in=ids_list)

        # 域用户删除
        if is_delete == 1:
            try:
                for account in queryset:
                    user_del(account.name)
            except Exception as exp:
                return Response(f'批量删除失败，{exp}', status=status.HTTP_400_BAD_REQUEST)
            else:
                LdapAccountEntity.objects.filter(account_id__in=ids_list).delete()
                queryset.delete()
                return Response('批量删除用户成功', status=status.HTTP_200_OK)

        if is_rest_passwd == 1:
            if new_passwd1 != new_passwd2:
                return Response('两次密码不一致', status.HTTP_400_BAD_REQUEST)

            for account in queryset:
                reset_user_pwd(account.name, passwd=new_passwd1)

            return Response('批量重置用户密码成功', status=status.HTTP_200_OK)

        # 域用户状态改变
        if is_status == 0:
            try:
                for account in queryset:
                    user_status_enable(account.name)
            except Exception as exp:
                return Response(f'批量启用域用户失败，{exp}', status=status.HTTP_400_BAD_REQUEST)
            else:
                queryset.update(user_status=0)
                return Response('批量启用域用户成功', status=status.HTTP_200_OK)
        elif is_status == 1:
            try:
                for account in queryset:
                    user_status_disable(account.name)
            except Exception as exp:
                return Response(f'批量禁用域用户失败，{exp}', status=status.HTTP_400_BAD_REQUEST)
            else:
                queryset.update(user_status=1)
                return Response('批量禁用域用户成功', status=status.HTTP_200_OK)
        elif is_status == 2:
            try:
                for account in queryset:
                    user_unlock(account.name)
                    # 解锁终端用户锁定
                    bind_devices = Devices.objects.filter(bind_account=account.id)
                    celery_task = domain_user_unlock_policy.delay(
                        DeviceSerializer(bind_devices, many=True).data,  # celery参数不能是query_set类型
                        account.name
                    )



            except Exception as exp:
                return Response(f'批量解锁域用户失败，{exp}', status=status.HTTP_400_BAD_REQUEST)
            else:
                return Response('批量解锁域用户成功', status=status.HTTP_200_OK)
        else:
            pass

        if to_organ_id:
            if not isinstance(to_organ_id, int):
                return Response('目标部门传参错误，部门id应为整数', status=status.HTTP_200_OK)
            # queryset.update(org_info_id=to_organ_id)
            for account in queryset:
                account.org_info_id = to_organ_id
                account.save()
            return Response('批量修改域用户部门成功', status=status.HTTP_200_OK)

        if is_down_file == 1:
            # 保存文件到服务器
            if len(queryset) == 0:
                return Response('未找到满足条件的数据，导出失败', status=status.HTTP_200_OK)

            # 后台处理域终端下载
            thread1 = threading.Thread(target=self.download_account_info, args=(queryset,))
            thread1.start()

            return Response('人员导出中，请稍后前往消息中心查看', status=status.HTTP_200_OK)

        return Response('缺少必要的功能参数', status=status.HTTP_412_PRECONDITION_FAILED)

    @action(methods=['GET'], detail=False, authentication_classes=[TokenAuthentication])
    def account_list(self, request, *args, **kwargs):
        """
        获取账号列表信息
        :param request: is_all: 0|非全选，1|全选
                        ids_list： [1,2,3,4,...]
        :param args:
        :param kwargs:
        :return:
        """
        is_all = self.request.query_params.get('is_all', None)
        ids_list = self.request.query_params.get('ids_list', None)
        ids_list = ids_list.split(',') if ids_list is not None else []
        queryset = self.get_queryset()
        # logger.info(f"**********{type(is_all)},{type(ids_list)}")
        if is_all is None:
            pass
        elif is_all == str(0):
            if not isinstance(ids_list, list):
                return Response('ids_list传参非法', status=status.HTTP_400_BAD_REQUEST)
            else:
                queryset = self.get_queryset().filter(id__in=ids_list)
        elif is_all == str(1):
            ids_list = self.filter_queryset(queryset).values_list('id', flat=True)
            return Response({'ids_list': ids_list}, status=status.HTTP_200_OK)
        else:
            return Response('is_all传参非法', status=status.HTTP_400_BAD_REQUEST)

        queryset = self.filter_queryset(queryset)

        user_data = request.user
        org_list = get_org_permission_list(user_data)
        if org_list:
            queryset = queryset.filter(org_info__in=org_list)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)


class PasswordAccountRuleViewSet(ModelViewSet):
    """
    域账号密码校验规则,从freeipa获取密码规则信息
    """
    queryset = PasswordAccountRule.objects.all()
    serializer_class = PasswordAccountRuleSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    pagination_class = None

    def list(self, request, *args, **kwargs):
        """
         {
        "maxpwdlife": 90,
        "minpwdlife": 1,
        "pwdhistorylength": 0,
        "pwdmindiffchars": 0,                0-5
        "pwdminlength": 8,
        "pwdmaxfailure": 6,                  最大试错次数  <100 则 accounts_lock_status = 0
        "pwdfailurecountinterval": 60,
        "pwdlockoutduration": 30.0
        }
        """
        res = pwpolicy_show()
        result = res.get('result')

        def _get_value(res, key):
            return int(res.get(key)[0]) if res.get(key) else None

        # 上游接口参数是秒，前端使用分钟
        # 锁定时间最大显示99999分钟，小于1分钟时返回1
        lock_dur = _get_value(result, 'krbpwdlockoutduration')
        if lock_dur:
            lock_dur = 99999 if lock_dur / 60 >= 99999 else lock_dur / 60
            lock_dur = 1 if lock_dur < 1 else lock_dur

        max_life = _get_value(result, 'krbmaxpwdlife')
        if max_life:
            max_life = 180 if max_life >= 180 else max_life
        if _get_value(result, 'krbpwdmaxfailure'):
            accounts_lock_status = 1 if _get_value(result, 'krbpwdmaxfailure') < 100 else 0
        else:
            accounts_lock_status = 0

        data = dict(
            accounts_lock_status=accounts_lock_status,
            pwdmaxfailure=_get_value(result, 'krbpwdmaxfailure'),
            pwdlockoutduration=lock_dur,
            password_strength_status=1 if _get_value(result, 'krbpwdminlength') else 0,
            pwdminlength=_get_value(result, 'krbpwdminlength'),
            pwdmindiffchars=_get_value(result, 'krbpwdmindiffchars'),
            password_validity_status=1 if _get_value(result, 'krbmaxpwdlife') else 0,
            maxpwdlife=max_life,
            minpwdlife=_get_value(result, 'krbminpwdlife'),
            password_extra_status=self.queryset.first().password_extra_status,
            password_extra_account=self.queryset.first().password_extra_account,
            history_passwd_check_status=1 if _get_value(result, 'krbpwdhistorylength') else 0,
            pwdhistorylength=_get_value(result, 'krbpwdhistorylength'),
        )
        # default_policy = DefaultPolicy()
        # data['pwd_prohibit_mod'] = int(default_policy.get_default_policy().get('pwd_prohibit_mod'))
        pwd_prohibit_mod = PasswordAccountProhibitModification.objects.filter(id=1).first()
        data['pwd_prohibit_mod'] = pwd_prohibit_mod.status

        return Response(data, status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def change_passwd_rule(self, request, *args, **kwargs):
        """
        更新域账号密码规则
        """
        req_data = request.data
        params = dict(a_cn='global_policy')

        # 密码过期时间大于180时，配置为最大值20000天
        if req_data.get('maxpwdlife') is not None:
            max_life = req_data.get('maxpwdlife')
            max_life = max_life if max_life < 180 else 20000
            params['o_krbmaxpwdlife'] = max_life

        if req_data.get('minpwdlife') is not None:
            params['o_krbminpwdlife'] = req_data.get('minpwdlife')

        if req_data.get('pwdhistorylength') is not None:
            params['o_krbpwdhistorylength'] = req_data.get('pwdhistorylength')

        if req_data.get('pwdmindiffchars') is not None:
            params['o_krbpwdmindiffchars'] = req_data.get('pwdmindiffchars')

        if req_data.get('pwdminlength') is not None:
            params['o_krbpwdminlength'] = req_data.get('pwdminlength')

        if req_data.get('pwdmaxfailure') is not None:
            params['o_krbpwdmaxfailure'] = req_data.get('pwdmaxfailure')

        if req_data.get('pwdfailurecountinterval') is not None:
            params['o_krbpwdfailurecountinterval'] = req_data.get('pwdfailurecountinterval')

        if req_data.get('pwdlockoutduration') is not None:
            lock_dur = req_data.get('pwdlockoutduration')
            lock_dur = lock_dur * 60 if lock_dur < 99999 else 2147483647
            params['o_krbpwdlockoutduration'] = lock_dur

        # 检验开关
        if req_data.get('accounts_lock_status') == 0:
            # 关闭密码锁定
            params['o_krbpwdmaxfailure'] = 0
            params['o_krbpwdlockoutduration'] = 0
        if req_data.get('password_strength_status') == 0:
            # 关闭密码检查
            params['o_krbpwdminlength'] = 0
            params['o_krbpwdmindiffchars'] = 0
        if req_data.get('password_validity_status') == 0:
            # 关闭密码有效期检查
            params['o_krbmaxpwdlife'] = 0
            params['o_krbminpwdlife'] = 0
        if req_data.get('password_extra_status') == 0:
            # 关闭列外  将用户从ipausers_extra移除  group_remove_member
            client = freeipa.get_ipa_client()
            try:
                res = client.group_show('ipausers_extra')['result']
                logger.info(f"-----{res}------")
                if 'member_user' in res.keys():
                    member_user = res.get('member_user')
                    client.group_remove_member(a_cn='ipausers_extra', o_user=member_user)
            except Exception as exp:
                return Response(f'freeipa 异常，异常信息{exp}', status=status.HTTP_412_PRECONDITION_FAILED)
            else:
                OrgnizationEntity.objects.update(passwd_extra_status=0)
                AccountEntity.objects.filter(is_extra_status=0)
            self.queryset.update(password_extra_status=req_data.get('password_extra_status'))
        else:
            self.queryset.update(password_extra_status=req_data.get('password_extra_status'))
        if req_data.get('history_passwd_check_status') == 0:
            # 关闭历史检查
            params['o_krbpwdhistorylength'] = 0

        if len(params) > 1:
            res = pwpolicy_mod(**params)
            logger.info(f"********{res}********")

            try:
                ipausers_dict = copy.deepcopy(params)
                ipausers_dict['a_cn'] = 'ipausers_extra'
                ipausers_dict['o_krbmaxpwdlife'] = 0
                ipausers_dict['o_krbminpwdlife'] = 0

                pwpolicy_mod(**ipausers_dict)
            except Exception as exp:
                logger.info(f"更新ipausers_extra失败，错误信息{exp}")

            pwd_prohibit_mod = PasswordAccountProhibitModification.objects.filter(id=1).first()
            if req_data.get('pwd_prohibit_mod') == 1:
                # default_policy = DefaultPolicy()
                # default_policy.set_default_policy('pwd_prohibit_mod', req_data.get('pwd_prohibit_mod'))
                pwd_prohibit_mod.status = 1
                pwd_prohibit_mod.save()
            elif req_data.get('pwd_prohibit_mod') == 0:
                pwd_prohibit_mod.status = 0
                pwd_prohibit_mod.save()

            if res.get('msg') is not None:
                if 'no modifications to be performed' == res.get('msg'):
                    return Response('参数没有变更', status=status.HTTP_200_OK)
                else:
                    return Response(f"更新规则失败，错误原因：{res.get('msg')}",
                                    status=status.HTTP_500_INTERNAL_SERVER_ERROR)
            else:
                return Response('修改成功', status=status.HTTP_200_OK)
        else:
            return Response('没有变更', status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def organ_info(self, request, *args, **kwargs):
        """获取列外部门信息"""
        result = OrgnizationEntity.objects.filter(passwd_extra_status=1).values_list('id', flat=True)
        return Response(list(result), status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def account_info(self, request, *args, **kwargs):
        """获取列外账号信息"""
        result = AccountEntity.objects.filter(is_extra_status=1).values_list('id', flat=True)
        return Response(list(result), status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def passwd_extra_org(self, request, *args, **kwargs):
        """
        更新额外部门范围
        :param request:body {
                        password_validity_status 密码有效期开关: 0|关闭； 1|开启
                        org_ids_list: 部门列表
                        }
        :param args:
        :param kwargs:
        :return:
        """
        password_validity_status = request.data.get('password_validity_status')
        org_ids_list = request.data.get('org_ids_list')
        if password_validity_status != 1:
            return Response('密码有效期设置没开，不能调整列外部门范围', status=status.HTTP_400_BAD_REQUEST)

        if not isinstance(org_ids_list, list):
            return Response('org_ids_list 必须是列表', status=status.HTTP_400_BAD_REQUEST)

        old_accounts_list = AccountEntity.objects.filter(is_extra_status=1).values_list('name', flat=True)
        new_org_list = OrgnizationEntity.objects.filter(id__in=org_ids_list).values_list('id', flat=True)
        new_accounts_list = AccountEntity.objects.filter(org_info_id__in=new_org_list).values_list('name', flat=True)

        # a和b的交集，不用管
        old_diff_list = list(set(old_accounts_list).difference(set(new_accounts_list)))
        new_diff_list = list(set(new_accounts_list).difference(set(old_accounts_list)))
        try:
            client = freeipa.get_ipa_client()
            if old_diff_list:
                client.group_remove_member(a_cn='ipausers_extra', o_user=old_diff_list)
            if new_diff_list:
                client.group_add_member(a_cn='ipausers_extra', o_user=new_diff_list)
        except Exception as exp:
            return Response(f'freeipa 异常，异常信息{exp}', status=status.HTTP_412_PRECONDITION_FAILED)
        else:
            self.queryset.update(password_extra_account=0)
            OrgnizationEntity.objects.update(passwd_extra_status=0)
            OrgnizationEntity.objects.filter(id__in=org_ids_list).update(passwd_extra_status=1)
            AccountEntity.objects.filter(id__in=org_ids_list).update(is_extra_status=1)
            return Response('列外部门范围修改成功', status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def passwd_extra_account(self, request, *args, **kwargs):
        """
        更新列外账号范围
        :param request:body {
                        password_validity_status 密码有效期开关: 0|关闭； 1|开启
                        account_ids_list: 账号列表
                        }
        :param args:
        :param kwargs:
        :return:
        """
        password_validity_status = request.data.get('password_validity_status')
        account_ids_list = request.data.get('account_ids_list')
        if password_validity_status != 1:
            return Response('密码有效期设置没开，不能调整列外账号范围', status=status.HTTP_400_BAD_REQUEST)

        if not isinstance(account_ids_list, list):
            return Response('account_ids_list 必须是列表', status=status.HTTP_400_BAD_REQUEST)

        old_accounts_list = AccountEntity.objects.filter(is_extra_status=1).values_list('name', flat=True)
        new_accounts_list = AccountEntity.objects.filter(id__in=account_ids_list).values_list('name', flat=True)

        # a和b的交集，不用管
        old_diff_list = list(set(old_accounts_list).difference(set(new_accounts_list)))
        new_diff_list = list(set(new_accounts_list).difference(set(old_accounts_list)))
        try:
            client = freeipa.get_ipa_client()
            if old_diff_list:
                client.group_remove_member(a_cn='ipausers_extra', o_user=old_diff_list)
            if new_diff_list:
                client.group_add_member(a_cn='ipausers_extra', o_user=new_diff_list)
        except Exception as exp:
            return Response(f'freeipa 异常，异常信息{exp}', status=status.HTTP_412_PRECONDITION_FAILED)
        else:
            self.queryset.update(password_extra_account=1)
            AccountEntity.objects.update(is_extra_status=0)
            AccountEntity.objects.filter(id__in=account_ids_list).update(is_extra_status=1)
            return Response('列外部门范围修改成功', status=status.HTTP_200_OK)

    @action(methods=['get', 'put'], detail=False, authentication_classes=[TokenAuthentication])
    def account_rule_info(self, request, *args, **kwargs):
        """获取账号规则设置"""
        if request.method == 'GET':
            queryset = self.get_queryset().first()
            serializer = self.get_serializer(queryset)
            return Response(serializer.data, status=status.HTTP_200_OK)
        elif request.method == 'PUT':
            partial = kwargs.pop('partial', False)
            instance = self.get_queryset().first()

            request_is_offline_login = request.data.get('is_offline_login')
            request_offline_cache_days = request.data.get('offline_cache_days')
            if request_is_offline_login != instance.is_offline_login or request_offline_cache_days != instance.offline_cache_days:
                devices = Devices.objects.exclude(is_domain=0).filter(is_up=1).all()
                if not devices:
                    return Response({
                        'code': 400,
                        'msg': '设置未提交， 请检查设备是否未启用'
                    })
                # for device in devices.only('id'):
                #     ssh_device, _ = SSHDevicePolicy.objects.get_or_create(device_id=device.id)
                #     ssh_device.status = 2
                #     ssh_device.save()

                is_offline_login = 'False' if int(request_is_offline_login) == 0 else 'True'
                offline_cache_days = int(request_offline_cache_days) if request_offline_cache_days is not None else 999
                celery_task = update_cache_sssd_policy(
                    DeviceSerializer(devices, many=True).data,  # celery参数不能是query_set类型
                    (is_offline_login, offline_cache_days)
                )

            serializer = self.get_serializer(instance, data=request.data, partial=partial)
            serializer.is_valid(raise_exception=True)
            self.perform_update(serializer)
            return Response(serializer.data)
        else:
            return Response(f'方法{request.method}不被允许', status=status.HTTP_200_OK)


class LocalUserLoginViewSet(ModelViewSet):
    """
    锁定本地用户登陆
    """
    queryset = LocalUserLogin.objects.all()
    serializer_class = LocalUserLoginSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    pagination_class = None

    def unlock_local_user_action(self, device_list):
        # 用户解锁
        for device in device_list:
            device_dict = {
                "device_hostname": device.device_hostname,
                "device_ip": device.device_ip,
            }
            unlock_local_user(device_dict, os=device.os, os_release=device.os_release)

    def lock_local_user_action(self, device_list):
        # 用户锁定
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        for device in device_list:
            logger.info("device %s" % (device))
            logger.info("device %s" % (type(device)))
            logger.info("device %s" % (dir(device)))
            logger.info("device.device_hostname %s" % (device.device_hostname))
            logger.info("device.os %s" % (device.os))
            logger.info("device.os_release %s" % (device.os_release))
            # logger.info("device.os_release %s" % (device["os_release"]))
            lock_local_user(salt_api, device.device_hostname, os=device.os, os_release=device.os_release)

    def update(self, request, *args, **kwargs):
        """
        本地用户登陆,0 关、1 开
        """
        # 本地用户默认解锁，现在添加为锁定
        partial = kwargs.pop('partial', False)
        instance = self.get_queryset().first()

        old_device_list = get_device(instance.range, instance.org_id, instance.device_id)
        new_device_list = get_device(request.data.get("range"), request.data.get("org_id"),
                                     request.data.get("device_id"))
        logger.info("old_device_list %s" % (old_device_list))
        logger.info("new_device_list %s" % (new_device_list))

        old_switch_status = instance.status
        new_switch_status = request.data.get("status")



        if new_switch_status != old_switch_status:
            if new_switch_status == 1:
                self.unlock_local_user_action(new_device_list)
            else:
                self.lock_local_user_action(old_device_list)
        else:
            if new_switch_status == 1:
                old_id_list = [x for x in old_device_list if x]
                new_id_list = [x for x in new_device_list if x]
                add_id_list, delete_id_list = diff_list(old_id_list, new_id_list)
                add_device_list = Devices.objects.filter(id__in=add_id_list)
                delete_device_list = Devices.objects.filter(id__in=delete_id_list)
                self.lock_local_user_action(delete_device_list)
                self.unlock_local_user_action(add_device_list)
            else:
                self.lock_local_user_action(old_device_list)

        # # 本地用户默认解锁
        # if request.data.get("status") == 0:
        #     request.data["org_id"] = ""
        #     request.data["device_id"] = ""
        # else:
        #     # 设置的三种情况
        #     # 空 --> AB
        #     # AB --> BC
        #     # AB --> 空
        #
        #     # 第一次为空
        #     if not old_device_list:
        #         # 锁定
        #         logger.info("lock_local_user_action %s " % new_device_list)
        #         self.lock_local_user_action(new_device_list)
        #     else:
        #         # 第一次有值
        #         # 第二次无值，解锁所有用户
        #         if not new_device_list:
        #             self.unlock_local_user_action(old_device_list)
        #             logger.info("unlock_local_user_action %s " % old_device_list)
        #         else:
        #             old_id_list = [x for x in old_device_list if x]
        #             new_id_list = [x for x in new_device_list if x]
        #             add_id_list, delete_id_list = diff_list(old_id_list, new_id_list)
        #
        #             # 锁定
        #             add_device_list = Devices.objects.filter(id__in=add_id_list)
        #             self.lock_local_user_action(add_device_list)
        #             logger.info("lock_local_user_action %s " % add_device_list)
        #
        #             # 解锁
        #             delete_device_list = Devices.objects.filter(id__in=delete_id_list)
        #             self.unlock_local_user_action(delete_device_list)
        #             logger.info("unlock_local_user_action %s " % delete_device_list)

        range_type = request.data.get("range")
        if range_type == 0:
            request.data["device_id"] = ""
        else:
            request.data["org_id"] = ""
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        return Response(serializer.data)


class BlackWhiteListsViewSet(ModelViewSet):
    queryset = BlackWhiteLists.objects.all()
    serializer_class = BlackWhiteListsSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    pagination_class = MyPagination
    filter_class = BlackWhiteListsFilterSet
    ordering = ('-id',)

    def list(self, request, *args, **kwargs):
        data = {}
        query, _ = BlackWhiteListsRule.objects.get_or_create(id=1)
        data['black'] = {'status': query.black_status, 'list_range': query.black_list_range, 'rule': []}
        data['white'] = {'status': query.white_status, 'list_range': query.white_list_range, 'rule': []}
        for i in self.filter_queryset(self.get_queryset()):
            if i.list_type == 'black':
                data.get('black')['rule'].append({'op_type': i.op_type, 'content': i.content})
            else:
                data.get('white')['rule'].append({'op_type': i.op_type, 'content': i.content})
        return Response(data, status=status.HTTP_200_OK)

    def create(self, request, *args, **kwargs):
        '''
         {
            'black':{             # 黑名单
                'status': 0,            # 名单状态
                'list_range': 0,        # 生效范围
                'rule':[                # 名单规则
                    {'op_type': 'startswith', 'content': 'abc'},
                    {'op_type': 'equals', 'content': '123'},
                ]
            },
            'white':{
                'status': 0,
                'list_range': 0,
                'rule':[
                    {'op_type': 'startswith', 'content': 'abc'},
                    {'op_type': 'equals', 'content': '123'},
                ]
            }
        }

        '''
        query, _ = BlackWhiteListsRule.objects.get_or_create(id=1)
        black = request.data.get('black')
        white = request.data.get('white')
        query.black_status = black.get('status')
        query.black_list_range = black.get('list_range')
        query.white_status = white.get('status')
        query.white_list_range = white.get('list_range')
        query.save()
        self.get_queryset().delete()
        black_rule = black.get('rule')
        for i in black_rule:
            i['list_type'] = 'black'

        white_rule = white.get('rule')
        for i in white_rule:
            i['list_type'] = 'white'
        data = black_rule + white_rule
        if data:
            serializer = self.get_serializer(data=data, many=True)
            serializer.is_valid(raise_exception=True)
            self.perform_update(serializer)
        return Response('修改成功', status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False)
    def multiple_update(self, request, *args, **kwargs):
        id = request.query_params.get('id', None)
        list_type = request.data.get('list_type', None)
        op_type = request.data.get('op_type', None)
        content = request.data.get('content', None)
        if not id:
            return Response('缺少参数', status=status.HTTP_400_BAD_REQUEST)
        if list_type not in ('black', 'white') or op_type not in (
                'startswith', 'endswith', 'equals', 'contains') or not content:
            return Response('参数错误', status=status.HTTP_400_BAD_REQUEST)
        query = self.get_queryset().filter(id=id)
        if not query:
            return Response('规则不存在', status=status.HTTP_400_BAD_REQUEST)
        query.update(list_type=list_type, op_type=op_type, content=content)
        return Response('修改成功', status=status.HTTP_400_BAD_REQUEST)

    @action(methods=['delete'], detail=False)
    def multiple_delete(self, request, *args, **kwargs):
        ids_list = request.data.get('ids_list', None)
        if not ids_list:
            return Response('缺少参数', status=status.HTTP_400_BAD_REQUEST)
        ids = ids_list.split(',')
        self.get_queryset().filter(id__in=ids).all().delete()
        return Response('已删除', status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False)
    def change_status(self, request, *args, **kwargs):
        '''
        {'black_status':0, 'black_list_range':1, 'white_status':0, 'white_list_range':1}
        '''
        query, _ = BlackWhiteListsRule.objects.get_or_create(id=1)
        if request.data.get('black_status') in (0, 1):
            query.black_status = request.data.get('black_status')
        if request.data.get('white_status') in (0, 1):
            query.white_status = request.data.get('white_status')
        if request.data.get('black_list_range') in (0, 1):
            query.black_list_range = request.data.get('black_list_range')
        if request.data.get('white_list_range') in (0, 1):
            query.white_list_range = request.data.get('white_list_range')
        query.save()
        return Response('修改成功', status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False)
    def status(self, request, *args, **kwargs):
        '''
        {'black_status':0, 'black_list_range':1, 'white_status':0, 'white_list_range':1}
        '''
        query, _ = BlackWhiteListsRule.objects.get_or_create(id=1)
        serializer = BlackWhiteListsRuleSerializer(query)
        return Response(serializer.data, status=status.HTTP_200_OK)

class BackupRestoreViewSet(ModelViewSet):
    queryset = BackupRestore.objects.all()
    serializer_class = BackupRestoreSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    pagination_class = MyPagination
    ordering = ('-id',)

    def create(self, request, *args, **kwargs):
        command = request.data.get('command')
        if command == 'ipa-restore':
            query = self.get_queryset().filter(id=request.data.get('id')).first()
            backup_path = query.backup_path
        else:
            backup_path = None
        res = ipa_backup_restore(command, backup_path)
        if not res:
            return Response('ldap服务不存在或网络不通', status=status.HTTP_200_OK)
        return Response(res.get('message'), status=status.HTTP_200_OK)

    @action(methods=['POST'], detail=False, authentication_classes=[], permission_classes=[])
    def callback(self, request, *args, **kwargs):
        success = request.data.get('success', None)
        command = request.data.get('command', None)
        start_time = request.data.get('start_time', None)
        end_time = request.data.get('end_time', None)
        backup_file = request.data.get('backup_file', None)
        backup_path = request.data.get('backup_path', None)
        file_size = request.data.get('file_size', None)
        task_id = request.data.get('task_id', None)
        error = request.data.get('error', None)
        if not success:
            logger.info(error)
            return Response('任务执行失败', status=status.HTTP_400_BAD_REQUEST)
        if command == 'ipa-backup':
            BackupRestore.objects.create(
                backup_file=backup_file,backup_path=backup_path,file_size=file_size,
                backup_start_time=start_time,backup_end_time=end_time,
            )
        else:
            querys = self.get_queryset().all()
            for query in querys:
                query.is_restore = 0
                query.save()
            query = self.get_queryset().filter(backup_path=backup_path).first()
            query.is_restore = 1
            query.restore_start_time = start_time
            query.restore_end_time = end_time
            query.save()
        return Response('执行成功', status=status.HTTP_400_BAD_REQUEST)

    @action(methods=['delete'], detail=False)
    def multiple_delete(self, request, *args, **kwargs):
        delete_id = request.data.get('id')
        ids = delete_id.split(',')
        self.get_queryset().filter(id__in=ids).all().delete()
        return Response('已删除', status=status.HTTP_200_OK)


class DNSRecordstoreViewSet(ModelViewSet):
    queryset = DNSRecords.objects.all()
    serializer_class = DNSRecordsSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    pagination_class = MyPagination
    filter_class = DNSRecordsFilterSet
    ordering = ('-id',)

    def create(self, request, *args, **kwargs):
        record_name = request.data.get('record_name')
        record_type = request.data.get('record_type')
        record_data = request.data.get('record_data')
        if not record_name or not record_type or not record_data:
            return Response('缺少参数', status=status.HTTP_400_BAD_REQUEST)
        if record_name == 'server':
            return Response('记录名不能为 server', status=status.HTTP_400_BAD_REQUEST)
        if record_type != "A":
            return Response('记录类型仅支持 A', status=status.HTTP_400_BAD_REQUEST)
        query = self.get_queryset().filter(record_name=record_name).first()
        if query:
            return Response('记录已经存在', status=status.HTTP_400_BAD_REQUEST)
        if freeipa.dnsrecord_show(record_name):
            return Response('记录已经存在', status=status.HTTP_400_BAD_REQUEST)
        if not freeipa.dnsrecord_add(record_name, record_data):
            return Response('记录添加失败', status=status.HTTP_400_BAD_REQUEST)
        DNSRecords.objects.create(record_name=record_name, record_type=record_type, record_data=record_data)
        return Response('记录添加成功', status=status.HTTP_200_OK)

    @action(methods=['delete'], detail=False)
    def multiple_delete(self, request, *args, **kwargs):
        delete_id = request.data.get('id')
        is_all = request.data.get('is_all')
        if is_all == 1:
            querys = self.get_queryset().all()
        else:
            ids = delete_id.split(',')
            querys = self.get_queryset().filter(id__in=ids).all()
        for query in querys:
            record_name = query.record_name
            record_data = query.record_data
            if freeipa.dnsrecord_show(record_name):
                freeipa.dnsrecord_del(record_name)
            query.delete()

        return Response('已删除', status=status.HTTP_200_OK)

class LdapSSLFile(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def post(self, request):
        ssl_file = request.FILES.get('file')
        if ssl_file:
            try:
                full_filename = '/'.join([settings.LDAP_SSL_FILE_DIR, ssl_file.name])
                # save file to local
                with open(full_filename, 'wb') as f:
                    for chunk in ssl_file.chunks():
                        f.write(chunk)
                return Response(full_filename, status=status.HTTP_200_OK)
            except Exception as e:
                logger.info(e)
                return Response('ssl证书上传失败', status=status.HTTP_400_BAD_REQUEST)
        return Response('', status=status.HTTP_200_OK)


class LdapViewSet(ModelViewSet):
    queryset = Ldap.objects.all()
    serializer_class = LdapSerializer
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    filter_class = LdapFilterSet
    ordering = ['-created_at']

    '''
    name:数据源名称
    host:服务器地址
    port:服务器断开
    base_dn:基准DN
    base_ou:基准OU
    admin_user_dn:管理员帐号
    admin_password:管理员密码
    ssl:
    ssl_file:ssl证书
    point:所属节点
    default_password:默认密码
    '''

    @action(methods=['post'], detail=False)
    def test_connection(self, request, *args, **kwargs):
        host = request.data.get('host')
        port = request.data.get('port')
        base_dn = request.data.get('base_dn')
        admin_user_dn = request.data.get('admin_user_dn')
        admin_password = request.data.get('admin_password')
        ldap = LdapAD(host, port, admin_user_dn, admin_password, base_dn)
        if ldap.test_connection():
            return Response('LDAP服务连接成功', status=status.HTTP_200_OK)
        return Response('LDAP服务无法连接', status=status.HTTP_400_BAD_REQUEST)

    @action(methods=['get'], detail=True)
    def sync(self, request, *args, **kwargs):
        ldap_obj = self.get_object()
        base_ou = [ou.base_ou for ou in ldap_obj.ldapbaseou_set.all()]
        ldap = LdapAD(
            ldap_obj.host, ldap_obj.port, ldap_obj.admin_user_dn, ldap_obj.admin_password,
            ldap_obj.base_dn, base_ou
        )
        try:
            if not ldap.sync_group_user():
                return Response('LDAP数据迁移失败', status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            logger.info(e)
            return Response('LDAP数据迁移失败', status=status.HTTP_400_BAD_REQUEST)
        return Response('LDAP数据迁移成功', status=status.HTTP_200_OK)

    def update(self, request, *args, **kwargs):
        kwargs['partial'] = True
        return super(LdapViewSet, self).update(request, *args, **kwargs)

    @action(methods=['delete'], detail=False)
    def multiple_delete(self, request, *args, **kwargs):
        delete_id = request.query_params.get('id')
        ids = delete_id.split(',')
        self.get_queryset().filter(id__in=ids).all().delete()
        return Response('已删除', status=status.HTTP_200_OK)


class LdapSyncStatusViewSet(ModelViewSet):
    queryset = LdapSyncStatus.objects.all()
    serializer_class = LdapSyncStatusSerializer
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ['-sync_at']

    def filter_ldap(self, kwargs):
        ldap_id = kwargs.get('ldap_id')
        queryset = self.queryset.filter(ldap_id=int(ldap_id)).all()
        return queryset

    def list(self, request, *args, **kwargs):
        self.queryset = self.filter_ldap(kwargs)
        return super(LdapSyncStatusViewSet, self).list(request, *args, **kwargs)

    def retrieve(self, request, *args, **kwargs):
        self.queryset = self.filter_ldap(kwargs)
        return super(LdapSyncStatusViewSet, self).retrieve(request, *args, **kwargs)


class PwdProhibitModificationViewSet(ModelViewSet):
    queryset = PasswordAccountProhibitModification.objects.all()
    serializer_class = PwdProhibitModificationSerializer
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    # filter_class = LdapFilterSet
    ordering = ['-pk']

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_queryset().first()
        if not instance:
            return Response({"code": 404, "data": None, "msg": "未找到。"})
        serializer = self.get_serializer(instance)
        return Response({"code": 200, "data": serializer.data, "msg": None})

    def update(self, request, *args, **kwargs):
        """
        param
            {
                "org_id":'1,2',
                "account_id":'1,2',
                "range":0,
                "status":1
            }
        """
        partial = kwargs.pop('partial', False)
        instance = self.get_queryset().first()
        # account_id = req_data.get('account_id')
        # org_id = req_data.get('org_id')
        # range = req_data.get('range')
        # status = req_data.get('status')
        if request.data.get("status") == 0:
            request.data['account_id'] = ''
            request.data['org_id'] = ''

        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        return Response(serializer.data)

    @action(methods=['delete'], detail=False)
    def multiple_delete(self, request, *args, **kwargs):
        delete_id = request.query_params.get('id')
        ids = delete_id.split(',')
        self.get_queryset().filter(id__in=ids).all().delete()
        return Response('已删除', status=status.HTTP_200_OK)


class AuthSourceViewSet(ModelViewSet):
    queryset = AuthSource.objects.all()
    serializer_class = AuthSourceSerializer
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ['-created_at']

    def create(self, request, *args, **kwargs):
        name = request.data.get('name')
        addr = request.data.get('addr')
        sync = request.data.get('sync')
        sync_at = request.data.get('sync_at')
        if ':' not in addr:
            return Response(f'ip和端口地址格式不正确 ', status=status.HTTP_400_BAD_REQUEST)
        if self.queryset.first():
            return Response(f'已经存在认证源,不能再添加', status=status.HTTP_400_BAD_REQUEST)
        if self.queryset.filter(name=name).first():
            return Response(f'{name} 已经存在', status=status.HTTP_400_BAD_REQUEST)
        if self.queryset.filter(addr=addr).first():
            return Response(f'{addr} 已经存在', status=status.HTTP_400_BAD_REQUEST)

        try:
            soap_ip = addr.split(':')[0]
            soap_port = addr.split(':')[1]
            # 将数据写入config.ini中
            config = configparser.ConfigParser()
            config.read(CONFIG_PATH, encoding="utf-8")
            config.set('soap_api', 'soap_ip', soap_ip)
            config.set('soap_api', 'soap_port', soap_port)

            # 保存修改后的内容回写到文件
            with open(CONFIG_PATH, 'w') as configfile:
                config.write(configfile)
        except Exception as e:
            return Response(f'写入配置文件数据失败 ：{e} ', status=status.HTTP_400_BAD_REQUEST)

        try:
            thread1 = threading.Thread(target=synchro_data_soap, args=(1,))  # 启动数据同步
            thread1.start()
            if sync == 1:
                dt_obj = datetime.datetime.strptime(sync_at, "%H:%M:%S")
                # 创建定时任务
                crontab_obj, crontab_bool = CrontabSchedule.objects.update_or_create(
                    minute=dt_obj.minute,
                    hour=dt_obj.hour,
                    day_of_week="*",
                    day_of_month='*',
                    month_of_year='*',
                    timezone=pytz.timezone("Asia/Shanghai"),
                )
                my_task = PeriodicTask.objects.create(
                    name='sync_data_soap',
                    task='accounts.tasks.synchro_data_soap',  # 任务路径
                    crontab=crontab_obj,  # 任务将每1分钟执行一次
                )
                app.conf.beat_schedule[my_task.name] = {
                    'task': my_task.task,
                    'schedule': my_task.crontab,
                }
                logger.info(f'添加定时任务,{app.conf.beat_schedule}')

        except Exception as e:
            return Response(f'创建定时任务失败 ：{e} ', status=status.HTTP_400_BAD_REQUEST)

        return super(AuthSourceViewSet, self).create(request, *args, **kwargs)

    @staticmethod
    def delete_freeipa_user(res_data):
        for i in res_data:
            user_del(i)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        # 删除定时任务
        task_name = 'sync_data_soap'
        she_task = PeriodicTask.objects.filter(name=task_name)
        if not she_task.exists():
            pass
        else:
            she_task.update(enabled=False)
            she_task.delete()

        if task_name in app.conf.beat_schedule:
            app.conf.beat_schedule.pop(task_name)
            logger.info(f"定时任务 '{task_name}' 已删除")
        else:
            logger.info(f"定时任务 '{task_name}' 不存在，跳过删除操作")

        # 清除导入的数据
        organ_items = OrgnizationEntity.objects.filter(create_type=2)
        device_items = Devices.objects.filter(org_info_id__in=organ_items.values_list('id', flat=True))
        accounts_items = AccountEntity.objects.filter(create_type=2)

        device_items.update(org_info_id=DEPT_ROOT_ID)

        thread1 = threading.Thread(target=self.delete_freeipa_user, args=(accounts_items.values_list('name'),))
        thread1.start()

        accounts_items.delete()
        organ_items.delete()

        self.perform_destroy(instance)
        return Response(status=status.HTTP_204_NO_CONTENT)

    @action(methods=['get'], detail=False)
    def re_sync(self, request, *args, **kwargs):
        """全量同步信息"""
        thread1 = threading.Thread(target=synchro_data_soap, args=(1,))  # 启动数据同步
        thread1.start()
        return Response({"code": 200, 'data': '全量已开始', 'msg': None}, status=status.HTTP_200_OK)

    @action(methods=['post'], detail=False)
    def test_connection(self, request, *args, **kwargs):
        addr = request.data.get('addr').strip()
        try:
            addr_list = addr.rsplit(':', 1)
            ip = addr_list[0]
            port = addr_list[1]
        except Exception as e:
            return Response(f'{addr} 错误', status=status.HTTP_400_BAD_REQUEST)

        if test_ip_port_connection(ip, int(port)):
            return Response(f'{addr} 连接成功', status=status.HTTP_200_OK)
        return Response(f'{addr} 无法连接', status=status.HTTP_400_BAD_REQUEST)

    @action(methods=['get'], detail=False, authentication_classes=[], permission_classes=[], throttle_classes=[])
    def prefix(self, request, *args, **kwargs):
        sm = get_soapmember()
        if not sm:
            return Response({"user_prefix": None}, status=status.HTTP_200_OK)
        else:
            return Response({"user_prefix": sm.user_prefix}, status=status.HTTP_200_OK)


class ApisAuth(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = ()

    def post(self, request, *args, **kwargs):
        head = request.META.get('HTTP_USERAGENT')
        req_data = request.data
        ldap_ad = get_ldap_ad()
        sm = SoapMember()
        appid = req_data.get('appid')
        timestamp = req_data.get('timestamp')
        sig = req_data.get('sig')
        userId = req_data.get('userId')
        password = sm4_decrypt(req_data.get('password'), sm.soap_sm4_keys)
        authType = req_data.get('authType')
        if not apis_authentication(appid, timestamp, sig):
            return Response(
                {
                    'status': 'error', 'code': 1, 'message': 'nfs auth error',
                    'data': {'token': None, }
                },
                status=status.HTTP_200_OK
            )
        # check webservice auth
        if ldap_ad:
            res = user_find(userId)
            if not res:
                return Response(
                    {'status': 'error', 'message': '用户名或密码错误！', 'code': 20013, 'data': ''})
            check_res = ldap_ad.auth_user(userId, password)
            if not check_res.get('status'):
                return Response(
                    {'status': 'error', 'message': '用户名或密码错误', 'code': 20013, 'data': ''})
            return Response(
                {
                    'status': 'success',
                    'message': '',
                    'code': 0,
                    'data': sm4_hash(json.dumps({
                        'token': '',
                        'userId': userId
                    }), settings.SM4_KEY)
                }, status=status.HTTP_200_OK)
        elif sm:
            res = user_find(f'{sm.user_prefix}{userId}')
            if not res:
                return Response(
                    {'status': 'error', 'message': '上行认证无此子系统用户信息！', 'code': 20013, 'data': ''})
            userId = res.get('result').get('sn')[0]
            soap_data = {
                "appId": sm.appid,
                "userId": userId.lstrip(sm.user_prefix),
                "password": password,
                "requestId": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
                "authType": authType,
                "requestName": None,
                "requestReason": None,
                'authFlag': "1"
            }
            res = sm.get_auth_by_sm4(soap_data)
            return Response(res, status=status.HTTP_200_OK)
        else:
            res = check_user(userId, password)
            if not res.get('exist'):
                return Response({'status': 'error', 'message': '用户不存在或密码错误或已被锁定！', 'code': 20013, 'data': ''})
            return Response(
                {
                    'status': 'success',
                    'message': '',
                    'code': 0,
                    'data': sm4_hash(json.dumps({
                        'token': '',
                        'userId': userId
                    }), settings.SM4_KEY)
                }, status=status.HTTP_200_OK)


class ApisUpdatePWD(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = ()

    def put(self, request, *args, **kwargs):
        '''
        {'appid':'EMOP','userId':'303013','oldPassword':'SZrz_2022!03','newPassword':'SZrz_2022!04','authType':'M_PASSWORD','otpCode':'311641'}
        '''
        head = request.META.get('HTTP_USERAGENT')
        req_data = request.data
        sm = SoapMember()
        appid = req_data.get('appid')
        timestamp = req_data.get('timestamp')
        sig = req_data.get('sig')
        userId = req_data.get('userId')
        oldPassword = sm4_decrypt(req_data.get('oldPassword'), sm.soap_sm4_keys)
        newPassword = sm4_decrypt(req_data.get('newPassword'), sm.soap_sm4_keys)
        authType = req_data.get('authType')
        otpCode = req_data.get('otpCode')
        if not apis_authentication(appid, timestamp, sig):
            return Response(
                {
                    'status': 'error', 'code': 1, 'message': 'nfs auth error',
                },
                status=status.HTTP_200_OK
            )
        # check webservice auth
        res = user_find(f'{sm.user_prefix}{userId}')
        if not res:
            return Response({'status': 'error', 'message': '上行认证无此子系统用户信息！', 'code': 20013, 'data': ''})
        userId = res.get('result').get('sn')[0]
        soap_data = {
            "appId": sm.appid,
            "userId": userId.lstrip(sm.user_prefix),
            "oldPassword": oldPassword,
            "newPassword": newPassword,
            "authType": authType,
            "otpCode": otpCode,
        }
        res = sm.update_password_emop(soap_data)
        return Response(res, status=status.HTTP_200_OK)


class AuthRecordsViewSet(ModelViewSet):
    queryset = AuthRecords.objects.all()
    serializer_class = AuthRecordsSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('-sync_at',)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def records_batch_operation(self, request, *args, **kwargs):
        """
        soap记录批量操作
        :param request: body {
                        is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        is_delete : 0|不删除 1|删除 空|不处理
                        }
        :param args:
        :param kwargs:
        :return:
        """
        is_all = request.data.get('is_all')
        ids_list = request.data.get('ids_list')
        is_delete = request.data.get('is_delete')
        if is_all is None:
            return Response('is_all 为必输选项', status=status.HTTP_400_BAD_REQUEST)

        is_all = int(is_all)

        queryset = self.queryset

        if is_all == 0:
            try:
                ids_list = [int(i) for i in ids_list.split(',')]
            except Exception as exp:
                return Response(f'ids_list参数不符合规范{exp}', status=status.HTTP_400_BAD_REQUEST)
        else:
            queryset = self.filter_queryset(queryset)
            ids_list = queryset.values_list('id', flat=True)

        # 查询过滤结果集
        queryset = queryset.filter(id__in=ids_list)

        # 域用户删除
        if is_delete == 1:
            queryset.delete()
            return Response('批量删除记录', status=status.HTTP_200_OK)
