from django.db import models

class AdminLog(models.Model):
    is_success = (
        (0, 'N'), (1, 'Y')
    )

    admin_name = models.CharField(verbose_name='管理员', max_length=64)
    admin_nickname = models.CharField(verbose_name='管理员昵称', max_length=64, null=True, blank=True)
    module = models.CharField(verbose_name='功能模块', max_length=32)
    action = models.CharField(verbose_name='操作行为', max_length=128)
    result_status = models.IntegerField(verbose_name='结果状态,0-失败,1-成功', choices=is_success, default=1, blank=True)
    desc = models.CharField(verbose_name='操作描述', max_length=512, null=True, blank=True)
    request = models.TextField(verbose_name='请求参数', max_length=4096, null=True, blank=True)
    response = models.TextField(verbose_name='返回内容', max_length=4096, null=True, blank=True)
    operation_time = models.DateTimeField(verbose_name='操作时间')

    class Meta:
        db_table = 'admin_log'
        verbose_name = '管理员日志'
        verbose_name_plural = '管理员日志'

        # 自定义索引
        indexes = [
            models.Index(fields=['admin_name'], name='idx_admin_log_admin_name'),
            models.Index(fields=['module'], name='idx_admin_log_module'),
            models.Index(fields=['action'], name='idx_admin_log_action'),
        ]

    def __str__(self):
        return self.admin_name
