from django.db import models
 
import datetime
from hashlib import sha1
from django.db import models
from django.contrib.auth.hashers import make_password
 

def _default_get_serial_number():
    date = datetime.datetime.today()
    return date.strftime("%Y%m%d%H%M%S")

# 第N版本，重构数据模型 ************************************************************************************
USER_POLICY_TYPE = "user"
APPLICATION_POLICY_TYPE = "application"
BOOKMARK_POLICY_TYPE="bookmark"
POLICY_DEFAULT_FILE_NAME="default"

def get_policy_int(type_str):
    if type_str == USER_POLICY_TYPE:
        return 0
    elif type_str == APPLICATION_POLICY_TYPE:
        return 1
    elif type_str == BOOKMARK_POLICY_TYPE:
        return 2
    else:
        return -1
    
# 策略状态
browser_policy_status = (
    (0, '禁用'), (1, '启用')
)
# 策略执行状态
browser_policy_complete_status = (
    (0, '下发中'), (1, '成功 '), (2, '失败'), (3, '未应用/默认')
)
# 浏览器策略类型
browser_policy_type = (
    (0, 'user'), (1, 'application '), (2, 'bookmark')
)

class PolicyInfo(models.Model):
    
     
    name = models.CharField(verbose_name='策率名称', max_length=255, null=True, blank=True)
    serial_number = models.CharField(verbose_name='策率编号', max_length=32,default=_default_get_serial_number, unique=True, blank=True)
    policy_status = models.IntegerField(verbose_name='状态', choices=browser_policy_status, default=0, blank=True)
    policy_form_content = models.CharField(verbose_name='策略form内容',max_length=2048,null=True, blank=True) 
    policy_json_content = models.CharField(verbose_name='策略json内容',max_length=2048,null=True, blank=True) 
    remarks = models.CharField(verbose_name='备注',max_length=32 , null=True, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    update_at = models.DateTimeField(verbose_name='更新时间', auto_now=True, blank=True) 
    creator = models.CharField(verbose_name='创建人',max_length=64 , null=True, blank=True)
     
    class Meta:
        abstract = True

    def __str__(self):
        return self.name
     
class PolicyAccount(models.Model):
       
    account = models.ForeignKey(to='accounts.AccountEntity',verbose_name='用户账号',
    on_delete=models.DO_NOTHING, db_constraint=False)
     
    org = models.ForeignKey(to='organizations.OrgnizationEntity',verbose_name='所属部门',
     on_delete=models.DO_NOTHING, db_constraint=False) 

    policy = models.ForeignKey(to='browser.PolicyInfo',verbose_name='用户策略',
    on_delete=models.DO_NOTHING, db_constraint=False,null=True, blank=True)
    status = models.IntegerField(verbose_name='状态', choices=browser_policy_complete_status, default=0, blank=True)
    jobids = models.CharField(verbose_name='salt任务ID',max_length=512 , null=True, blank=True)
    update_seq = models.IntegerField(verbose_name='更新序列号',default=0, blank=True)
    remarks = models.CharField(verbose_name='备注',max_length=32 , null=True, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    update_at = models.DateTimeField(verbose_name='更新时间', auto_now=True, blank=True) 
    creator = models.CharField(verbose_name='创建人',max_length=64 , null=True, blank=True)
     
    class Meta:
        abstract = True

class BrowserUserPolicyInfo(PolicyInfo):
    
    class Meta:
        db_table = 'browser_user_policy_info'
        verbose_name = '浏览器-用户策略信息表'
        verbose_name_plural = '浏览器-用户策略信息表'
        ordering = ('-update_at',)

    def __str__(self):
        return self.name

class BrowserUserPolicyAccount(PolicyAccount):

    policy = models.ForeignKey(to='browser.BrowserUserPolicyInfo',verbose_name='用户策略',
    on_delete=models.DO_NOTHING, db_constraint=False,null=True, blank=True)  
     
    class Meta:
        db_table = 'browser_user_policy_account'
        verbose_name = '浏览器-用户策略和终端用户关系表'
        verbose_name_plural = '浏览器-用户策略和终端用户关系表'
        ordering = ('-update_at',)
 

class BrowserApplicationPolicyInfo(PolicyInfo):
     
     
    class Meta:
        db_table = 'browser_application_policy_info'
        verbose_name = '浏览器-应用策略信息表'
        verbose_name_plural = '浏览器-应用策略信息表'
        ordering = ('-update_at',)

    def __str__(self):
        return self.name

class BrowserApplicationPolicyAccount(PolicyAccount):
     

    policy = models.ForeignKey(to='browser.BrowserApplicationPolicyInfo',verbose_name='用户策略',
    on_delete=models.DO_NOTHING, db_constraint=False,null=True, blank=True)  
    class Meta:
        db_table = 'browser_application_policy_account'
        verbose_name = '浏览器-应用策略和终端用户关系表'
        verbose_name_plural = '浏览器-应用策略和终端用户关系表'
        ordering = ('-update_at',)
 

class BrowserBookmarkPolicyInfo( PolicyInfo):
     
    class Meta:
        db_table = 'browser_bookmark_policy_info'
        verbose_name = '浏览器-收藏策略信息表'
        verbose_name_plural = '浏览器-收藏策略信息表'
        ordering = ('-update_at',)

    def __str__(self):
        return self.name
    
class BrowserBookmarkPolicyAccount(PolicyAccount):
     
    policy = models.ForeignKey(to='browser.BrowserBookmarkPolicyInfo',verbose_name='用户策略',
    on_delete=models.DO_NOTHING, db_constraint=False,null=True, blank=True) 

    class Meta:
        db_table = 'browser_bookmark_policy_account'
        verbose_name = '浏览器-收藏策略和终端用户关系表'
        verbose_name_plural = '浏览器-收藏策略和终端用户关系表'
        ordering = ('-update_at',)
 
    
#使用默认策略 默认策略ID=0
class BrowserPolicyOrgInfo(models.Model):
    
    policy_type = models.IntegerField(verbose_name='浏览器策略类型', choices=browser_policy_type, default=0, blank=True)
    policy_id = models.IntegerField(verbose_name='策略', default=0, null=True, blank=True)
    org_id = models.IntegerField(verbose_name='部门', default=0, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    update_at = models.DateTimeField(verbose_name='更新时间', auto_now=True, blank=True) 
     
    class Meta:
        db_table = 'browser_policy_org_info'
        verbose_name = '浏览器策略-部门信息表'
        verbose_name_plural = '浏览器策略-部门信息表'
        ordering = ('-update_at',)
 

class BrowserPolicyLog(models.Model):
    policy_type = models.IntegerField(verbose_name='浏览器策略类型', choices=browser_policy_type, default=0, blank=True)
    account = models.ForeignKey(to='accounts.AccountEntity',verbose_name='用户账号',
    on_delete=models.DO_NOTHING, db_constraint=False,blank=True,null=True)
    update_seq = models.IntegerField(verbose_name='更新序列号',default=0, blank=True)
    policy = models.IntegerField(verbose_name='策略ID',   default=0, blank=True,null=True)
    jobid = models.CharField(verbose_name='salt任务ID',max_length=512 , null=True, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    update_at = models.DateTimeField(verbose_name='更新时间', auto_now=True, blank=True) 

    device_id = models.IntegerField(verbose_name='设备IP',default=0, blank=True)
    device_ip = models.CharField(verbose_name='设备IP', max_length=255, blank=True)
    device_hostname = models.CharField(verbose_name='设备hostname', max_length=255, null=True, blank=True) 

    fun = models.CharField(verbose_name='策略方法', blank=True, max_length=50, null=True)
    fun_args = models.TextField(verbose_name='args', blank=True, null=True)
    minion = models.CharField(verbose_name='minion key', blank=True, max_length=255, null=True)
    ret = models.TextField(verbose_name='ret', blank=True, null=True)
    success = models.IntegerField(verbose_name='success', blank=True, null=True)
    full_ret = models.TextField(verbose_name='full_ret', blank=True, null=True)
     
     
    class Meta:
        db_table = 'browser_policy_account_log'
        verbose_name = '浏览器-策略和终端用户更新日志表'
        verbose_name_plural = '浏览器-策略和终端用户更新日志表'
        ordering = ('-created_at',)
 