from django_redis import get_redis_connection
from rest_framework import serializers
from browser import models
 

from accounts.models import AccountEntity
from devices.models import Devices
from organizations.models import OrgnizationEntity
class BrowserPolicyConfigSerializer(serializers.ModelSerializer):
    address = serializers.SerializerMethodField()
    org_info_name = serializers.SerializerMethodField()
    cn_name = serializers.SerializerMethodField()

    class Meta:
        model = AccountEntity
        fields = (
            'id', 'cn_name', 'name', 'address', 'org_info', 'org_info_name', 'is_root', 'system_status',
            'policy_status')

    def get_org_info_name(self, obj):
        org_info_items = OrgnizationEntity.objects.all()
        if org_info_items.filter(id=obj.org_info_id).exists():
            org_info_name = org_info_items.get(id=obj.org_info_id).name
        else:
            org_info_name = None

        return org_info_name

    def get_address(self, obj):
        device_itesms = Devices.objects.exclude(is_domain=0).all()
        if device_itesms.filter(accounts_id=obj.id).exists():
            address = device_itesms.filter(accounts_id=obj.id).first().device_ip
        else:
            address = None
        return address

    def get_cn_name(self, obj):
        device_itesms = Devices.objects.exclude(is_domain=0).all()
        if device_itesms.filter(accounts_id=obj.id).exists():
            cn_name = device_itesms.filter(accounts_id=obj.id).first().device_cnname
        else:
            cn_name = None
        return cn_name



from accounts.serializers import AccountEntitySerializer
from organizations.serializers import OrganSerializer
class BrowserUserPolicyInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.BrowserUserPolicyInfo
        fields = "__all__"

class BrowserUserPolicyAccountSerializer(serializers.ModelSerializer):
    account = AccountEntitySerializer(read_only=True)
    org     = OrganSerializer(read_only=True)
    policy = BrowserUserPolicyInfoSerializer(read_only=True)
    account_id = serializers.PrimaryKeyRelatedField(queryset=AccountEntity.objects.all(), source='account', write_only=True)
    org_id     = serializers.PrimaryKeyRelatedField(queryset=OrgnizationEntity.objects.all(), source='org', write_only=True)
    policy_id     = serializers.PrimaryKeyRelatedField(queryset=models.BrowserUserPolicyInfo.objects.all(), source='policy', write_only=True)

    class Meta:
        model = models.BrowserUserPolicyAccount
        fields = "__all__"
    
class BrowserApplicationPolicyInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.BrowserApplicationPolicyInfo
        fields = "__all__"

class BrowserApplicationPolicyAccountSerializer(serializers.ModelSerializer):
    account = AccountEntitySerializer(read_only=True)
    org     = OrganSerializer(read_only=True)
    policy  = BrowserApplicationPolicyInfoSerializer(read_only=True)
    account_id = serializers.PrimaryKeyRelatedField(queryset=AccountEntity.objects.all(), source='account', write_only=True)
    org_id     = serializers.PrimaryKeyRelatedField(queryset=OrgnizationEntity.objects.all(), source='org', write_only=True)
    policy_id  = serializers.PrimaryKeyRelatedField(queryset=models.BrowserApplicationPolicyInfo.objects.all(), source='policy', write_only=True)

    class Meta:
        model = models.BrowserApplicationPolicyAccount
        fields = "__all__"

class BrowserBookmarkPolicyInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.BrowserBookmarkPolicyInfo
        fields = "__all__"

class BrowserBookmarkPolicyAccountSerializer(serializers.ModelSerializer):
    account = AccountEntitySerializer(read_only=True)
    org     = OrganSerializer(read_only=True)
    account_id = serializers.PrimaryKeyRelatedField(queryset=AccountEntity.objects.all(), source='account', write_only=True)
    org_id     = serializers.PrimaryKeyRelatedField(queryset=OrgnizationEntity.objects.all(), source='org', write_only=True)
    policy  = BrowserBookmarkPolicyInfoSerializer(read_only=True)
    policy_id  = serializers.PrimaryKeyRelatedField(queryset=models.BrowserBookmarkPolicyInfo.objects.all(), source='policy', write_only=True)

    class Meta:
        model = models.BrowserBookmarkPolicyAccount
        fields = "__all__"