import os,json,logging
from browser import models, filter, serializers
from saltapi.models import SaltJidDevice
from django.db.models import Q
from accounts.models import AccountEntity
from devices.models import Devices
from browser.models import USER_POLICY_TYPE, APPLICATION_POLICY_TYPE,BOOKMARK_POLICY_TYPE,POLICY_DEFAULT_FILE_NAME,get_policy_int
from browser.util import  get_default_policy_file_path,get_default_policy_form_file_path,deployed_browser_policy_to_device
from browser.util import USER_POLICY_TYPE,APPLICATION_POLICY_TYPE ,BOOKMARK_POLICY_TYPE,POLICY_DEFAULT_FILE_NAME
from browser.util import translate_to_policy,translate_to_application_policy,translate_to_bookmark_policy,saveOrUpdatePolicyOrgInfo
from browser.util import gen_json_file_by_policy_type,deployed_browser_policy_to_device,deployed_browser_policy_by_type,pre_deploy_to_device

logger = logging.getLogger(__name__)

def get_default_form_json(policy_type):
    res_obj = {} 
    policy_file_path = get_default_policy_form_file_path(policy_type)
    if not os.path.isfile(policy_file_path):
        return res_obj
    else:
        # 打开 JSON 文件
        with open(policy_file_path, 'r') as json_file:
            # 加载 JSON 数据
            res_obj = json.load(json_file)
    return res_obj

def update_default_file(policy_type,req_item):
    policy_file_path = get_default_policy_form_file_path(policy_type)
    if not os.path.isfile(policy_file_path):
        return None
    else:
        json_file = open(policy_file_path,'w', encoding='utf-8')
        json_file.write(json.dumps(req_item))
        json_file.close()  
    #update default.json
    
    if policy_type == USER_POLICY_TYPE:
        json_str = translate_to_policy(req_item)
    if policy_type == APPLICATION_POLICY_TYPE:
        json_str = translate_to_application_policy(req_item)
    if policy_type == BOOKMARK_POLICY_TYPE:
        json_str = translate_to_bookmark_policy(req_item)
    json_file = open(get_default_policy_file_path(policy_type),'w', encoding='utf-8')
    json_file.write(json_str)
    json_file.close() 

    if policy_type == USER_POLICY_TYPE:
        defaultPolicySet = models.BrowserUserPolicyAccount.objects.filter(policy_id__isnull=True)
    if policy_type == APPLICATION_POLICY_TYPE:
        defaultPolicySet = models.BrowserApplicationPolicyAccount.objects.filter(policy_id__isnull=True)
    if policy_type == BOOKMARK_POLICY_TYPE:
        defaultPolicySet = models.BrowserBookmarkPolicyAccount.objects.filter(policy_id__isnull=True)

    for userPolicyAccount in defaultPolicySet: 
        gen_json_file_by_policy_type(userPolicyAccount.account_id,userPolicyAccount.policy_id,policy_type)
        pre_deploy_to_device(userPolicyAccount)
        deployed_browser_policy_by_type(policy_type,userPolicyAccount)
    
    return  req_item
def deploy_policy_to_mult_account(policy_type,policy_id,organ_ids,accounts):

    saveOrUpdatePolicyOrgInfo(get_policy_int(policy_type),policy_id,organ_ids)
    if not accounts:
        return
    for account in accounts:
        logger.info(f"deploy policy for account {account.name}")
        
        if policy_type == USER_POLICY_TYPE:
            policyAccount = models.BrowserUserPolicyAccount.objects.filter(account_id= account.id).first()
        if policy_type == APPLICATION_POLICY_TYPE:
            policyAccount = models.BrowserApplicationPolicyAccount.objects.filter(account_id= account.id).first()
        if policy_type == BOOKMARK_POLICY_TYPE:
            policyAccount = models.BrowserBookmarkPolicyAccount.objects.filter(account_id= account.id).first()

        if not  policyAccount:
            continue
        policyAccount.policy_id=policy_id
        policyAccount.save()
        gen_json_file_by_policy_type(policyAccount.account_id,policyAccount.policy_id,policy_type)
        pre_deploy_to_device(policyAccount)
        deployed_browser_policy_by_type(policy_type,policyAccount)
          

def get_jobs(pk_id,policy_type):
    res = []
    if policy_type == USER_POLICY_TYPE:
        policyAccount = models.BrowserUserPolicyAccount.objects.filter(pk= pk_id).first()
    if policy_type == APPLICATION_POLICY_TYPE:
        policyAccount = models.BrowserApplicationPolicyAccount.objects.filter(pk= pk_id).first()
    if policy_type == BOOKMARK_POLICY_TYPE:
        policyAccount = models.BrowserBookmarkPolicyAccount.objects.filter(pk= pk_id).first()

    if policyAccount:
        jobids = policyAccount.jobids
        if jobids:
            saltJidDeviceQuerySet = SaltJidDevice.objects.filter(jid__in=jobids.split(","))
            res = []
            for job in saltJidDeviceQuerySet:
                device = Devices.objects.filter(
                    Q(device_hostname=job.minion) | Q(device_cnname=job.minion)).first()
                job_info = {
                    'policy_no': job.policy_no,
                    'minion': device.device_ip if device else None,
                    'fun': job.fun,
                    'fun_args': job.fun_args,
                    'jid': job.jid,
                    'ret': job.ret,
                    'alter_time': job.alter_time,
                    'success': job.success,
                }
                res.append(job_info)
        else:
            return res
    return res 

def get_logs(pk_id,policy_type):
    res = [] 
   
    if policy_type == USER_POLICY_TYPE:
        policyAccount = models.BrowserUserPolicyAccount.objects.filter(pk= pk_id).first()
    if policy_type == APPLICATION_POLICY_TYPE:
        policyAccount = models.BrowserApplicationPolicyAccount.objects.filter(pk= pk_id).first()
    if policy_type == BOOKMARK_POLICY_TYPE:
        policyAccount = models.BrowserBookmarkPolicyAccount.objects.filter(pk= pk_id).first()

    if policyAccount:
        all_log_item = models.BrowserPolicyLog.objects.filter(policy_type=get_policy_int(policy_type),account_id=policyAccount.account_id,
                                                            update_seq=policyAccount.update_seq).order_by('created_at')
        for item in all_log_item:
            log_info = {
                'created_at': item.created_at,
                'device_ip': item.device_ip  ,
                'device_hostname': item.device_hostname  ,
                'success': item.success  ,
                'fun': item.fun  ,
                'fun_args': item.fun_args  ,
                'minion': item.minion  ,
                'jobid': item.jobid  ,
            }
            res.append(log_info)
    return res
       