import json
from django.db.models import signals
from django.dispatch import receiver
from django.db.models.signals import m2m_changed

from accounts.models import AccountEntity
from devices.models import Devices
from browser import models

# from utils.RedisHelper import RedisHelper
# publicer = RedisHelper()
import os,redis
redis_host = os.environ.get('REDIS_HOST')
redis_port = os.environ.get('REDIS_PORT')
pool = redis.ConnectionPool(host=redis_host, port=redis_port)
redis_conn = redis.Redis(connection_pool=pool, db=0)
import logging
logger = logging.getLogger(__name__)



#监听账户人员的机构变化，根据机构变化情况，查询新的策略
@receiver(signals.pre_save, sender=AccountEntity)
def check_account_org_change(sender,instance,   **kwargs):

    logger.info(f"receive a pre_save signal from account ,it's name is {instance.name}")
    try:
        old = sender.objects.get(pk=instance.pk)
    except sender.DoesNotExist:
        return
    if old.org_info_id != instance.org_info_id:
         logger.info(f"account {instance.name} org changed to {instance.org_info_id}")
         redis_conn.publish(message=json.dumps({"event":"check_account_org_change", "account_id":instance.id,"org_id":instance.org_info_id})
                    , channel='browser_policy_chan')

    return

#监听账户删除，同步删除策略-账户关系表中的数据
@receiver(signals.post_delete, sender=AccountEntity)
def del_account_info(instance, **kwargs):

    logger.info(f"receive a delete signal from account ,it's name is {instance.name}")
    # return
    bupa = models.BrowserUserPolicyAccount.objects.all().filter(account=instance).first()
    if bupa:
        bupa.delete()

    bapa = models.BrowserApplicationPolicyAccount.objects.all().filter(account=instance).first()
    if bapa:
        bapa.delete()

    bmpa = models.BrowserBookmarkPolicyAccount.objects.all().filter(account=instance).first()
    if bmpa:
        bmpa.delete()
    # policy int client,what to do??


bind_account_dict = {}
#监听 账户-设备绑定情况 init 策略-账户信息表
@receiver(m2m_changed, sender=Devices.bind_account.through)
def check_bind_account_change(sender,instance,action, **kwargs):

    logger.info(f"receive a m2m_changed signal , action is {action} bind_account is {instance.bind_account.all()}")
    if action == "pre_add":
        bind_account_dict[instance.device_ip] = list(instance.bind_account.all())
    if action == "post_add":
        add_account_ids = []
        # compare changed account
        if bind_account_dict[instance.device_ip]:
            pre_list = bind_account_dict[instance.device_ip]
            now_list = list( instance.bind_account.all())
            diff_now_pre = set(now_list).difference(set(pre_list))
            logger.info(f"diff now pre is {diff_now_pre}")
            add_account_ids = [account.id for account  in diff_now_pre]
            bind_account_dict[instance.device_ip]=[]
        for account in instance.bind_account.all():
            # account = AccountEntity.objects.get(pk=int(account_id))
            org_info = account.org_info
            #通知 进行策略判断 策略下发终端的处理
            redis_conn.publish(message=json.dumps({"event":"preAddPolicyAccountInfo","account_id":account.id,
                                                   "org_id":org_info.id,"add_account_ids":add_account_ids,"device_id":instance.id})
                    , channel='browser_policy_chan')


