from django.db import models
 
 # CA信息表
class CaInfoModel(models.Model):

    is_default = (
        (0, 'N'), (1, 'Y')
    )

    name = models.CharField(max_length=255, verbose_name='证书名称')
    ca_desc = models.CharField(verbose_name='证书描述', max_length=100, null=False,  blank=True)
    ca_org = models.CharField(verbose_name='证书发布机构', max_length=100, null=False,  blank=True)
    ca_expired = models.CharField(verbose_name='证书有效期，逗号分割开始和结束时间', max_length=100, null=False,  blank=True)
    custome_filename = models.CharField(verbose_name='文件名称', max_length=255, null=False,  blank=True)
    remarks = models.CharField(verbose_name='备注', max_length=100, null=True,  blank=True)
    created_at = models.DateTimeField(verbose_name='更新时间', auto_now_add=True, blank=True)
    last_update = models.DateTimeField(verbose_name='最后执行时间',auto_now = True, null=True, blank=True)
    is_default_install = models.IntegerField(verbose_name='是否系统证书', choices=is_default, default=0,null=False, blank=True)
    is_deleted = models.IntegerField(verbose_name='是否是历史记录', choices=is_default, default=0,null=False, blank=True)

    class Meta:
        db_table = 'ca_info'
        verbose_name = 'CA信息表'
        verbose_name_plural = 'CA信息表'
        ordering = ('created_at',)

    def __str__(self):
        return self.name
# CA下发情况表
class CaDistributeModel(models.Model):

    is_success = (
        (0, 'N'), (1, 'Y')
    )
    is_histroy = (
        (0, 'N'), (1, 'Y')
    )
    #状态
    STATUS_RUNNING = 0
    STATUS_INSTALLED = 1
    STATUS_FAILED = 2
    STATUS_REMOVED = 3
    installed_status_choices = (
        (STATUS_RUNNING, 'running'), (STATUS_INSTALLED, 'installed'), (STATUS_FAILED, 'failed'), (STATUS_REMOVED, 'removed')
    )

    name = models.CharField(max_length=128, verbose_name='证书名称')
    org_id = models.TextField(verbose_name='所属部门', null=False,  blank=True)
    custom_path = models.CharField(verbose_name='下发路径', max_length=255, null=False,  blank=True)
    ca_info = models.ForeignKey(verbose_name='证书信息',to='camanager.CaInfoModel', on_delete=models.SET_NULL, db_constraint=False, null=True, blank=True)
    device_info = models.ForeignKey(verbose_name='终端设备信息',to='devices.Devices', on_delete=models.SET_NULL, db_constraint=False, null=True, blank=True)
    remarks = models.CharField(verbose_name='备注', max_length=255, null=True,  blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_update = models.DateTimeField(verbose_name='最后修改时间',auto_now = True, null=True, blank=True)
    is_success = models.IntegerField(verbose_name='是否成功',choices=is_success, default=0,null=False, blank=True)
    is_histroy = models.IntegerField(verbose_name='历史标识', choices=is_histroy, default=0,null=False, blank=True)
    installed_status = models.IntegerField(verbose_name='状态', choices=installed_status_choices, default=0,null=False, blank=True)

    class Meta:
        db_table = 'ca_distribute'
        verbose_name = 'CA下发情况表'
        verbose_name_plural = 'CA下发情况表'
        ordering = ('created_at',)

    def __str__(self):
        return self.name
