# -*- coding: utf-8 -*-
import os, django, logging
from django.conf import settings

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "nfs_domain_mainserver.settings")
django.setup()

from celery import Celery, platforms

logger = logging.getLogger(__name__)

redis_host = os.environ.get('REDIS_HOST')
redis_port = os.environ.get('REDIS_PORT')
broker_db = os.environ.get('CELERY_BROKER_DB')
backend_db = os.environ.get('CELERY_BACKEND_DB')

# 1.绑定存放任务的仓库
broker = 'redis://{}:{}/{}'.format(redis_host, redis_port, broker_db)
# 2.绑定存放任务执行结果的仓库
backend = 'redis://{}:{}/{}'.format(redis_host, redis_port, backend_db)
# 3.绑定要执行的任务主体路径
include = ['celery_task.tasks']

# 4.实例化Celery,并添加库和任务
app = Celery(broker=broker, backend=backend, include=include)
platforms.C_FORCE_ROOT = True
app.conf.worker_concurrency = 2

# 指定celery的配置来源 用的是项目的配置文件settings.py
app.config_from_object("django.conf:settings")

# 让celery 自动去发现我们的任务（task）
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)  # 你需要在app目录下 新建一个叫tasks.py（一定不要写错）文件
