# -*- coding: utf-8 -*-
from django.db import models
from enum import Enum

class ClientLog(models.Model):
    module_choices = (
        ('device', '终端'), ('account', '终端用户')
    )
    action_choices = (
        ('join_domain', '终端加域'), ('exit_domain', '终端退域'), 
        ('login', '用户登录'), ('logout', '用户登出')
    )
    is_success = (
        (0, 'N'), (1, 'Y')
    )

    module = models.CharField(verbose_name='模块', max_length=32, choices=module_choices, default='device')
    action = models.CharField(verbose_name='操作行为', max_length=64, choices=action_choices)
    device_uuid = models.CharField(verbose_name='终端设备UUID', max_length=255)
    device_cnname = models.CharField(verbose_name='终端别名', max_length=255)
    device_org_id = models.IntegerField(verbose_name='终端所属机构ID', null=True, blank=True)
    account_name = models.CharField(verbose_name='终端用户唯一名', max_length=64, null=True, blank=True)
    account_cnname = models.CharField(verbose_name='终端用户姓名', max_length=64, null=True, blank=True)
    account_org_id = models.IntegerField(verbose_name='终端用户所属机构ID', null=True, blank=True)
    result_status = models.IntegerField(verbose_name='行为结果状态,0-失败,1-成功', choices=is_success, null=True, blank=True)
    desc = models.CharField(verbose_name='行为描述', max_length=1024, null=True, blank=True)
    request = models.TextField(verbose_name='请求参数', max_length=4096, null=True, blank=True)
    response = models.TextField(verbose_name='返回内容', max_length=4096, null=True, blank=True)
    operate_uuid = models.CharField(verbose_name='操作唯一码UUID', max_length=255, null=True, blank=True)
    operation_time = models.DateTimeField(verbose_name='操作时间')
    device_info_json = models.TextField(verbose_name='终端设备信息', max_length=4096, null=True, blank=True)
    account_info_json = models.TextField(verbose_name='终端用户信息', max_length=4096, null=True, blank=True)

    class Meta:
        db_table = 'client_log'
        verbose_name = '客户端(终端)日志'
        verbose_name_plural = '客户端(终端)日志'

        # 自定义索引
        indexes = [
            models.Index(fields=['device_uuid'], name='idx_client_log_device_uuid'),
            models.Index(fields=['account_name'], name='idx_client_log_account_name'),
            models.Index(fields=['operate_uuid'], name='idx_client_log_operate_uuid'),
            models.Index(fields=['module'], name='idx_client_log__module'),
            models.Index(fields=['action'], name='idx_client_log_action'),
        ]

    def __str__(self):
        return self.device_cnname
    
class ClientLogModule(Enum):
    DEVICE = 'device'
    ACCOUNT = 'account'
    
class ClientLogAction(Enum):
    JOIN_DOMAIN = 'join_domain'
    EXIT_DOMAIN = 'exit_domain'
    ACCOUNT_LOGIN = 'login'
    ACCOUNT_LOGOUT = 'logout'
