"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/9/25 下午5:03
@File: consumers.py.py
@Desc: 
"""
import os
import inspect
import ctypes
import json
import logging
import threading
import time
import datetime
from urllib.parse import parse_qs

from asgiref.sync import async_to_sync
from channels.generic.websocket import WebsocketConsumer
from django_redis import get_redis_connection
from django.core.cache import caches
from django.db import close_old_connections
from channels.layers import get_channel_layer
from devices.models import Devices
from django.conf import settings
from utils.RedisHelper import RedisHelper
from utils.authentication import DomainLicenseAuth
from utils.sch_soap_task_member import get_soapmember
from utils.utils import get_uuid, read_licenseB, bytes_to_string, sm4_hash, sm4_decrypt, is_satisfy_black_lists
from devices.utils import bind_device_account
from accounts.freeipa import check_user, user_find
from accounts.ldaps import get_ldap_ad
from accounts.models import AccountEntity
from patch_management.models import PatchFile, PatchDeviceLog, PatchInfo

logger = logging.getLogger(__name__)
conn = get_redis_connection('redis')
channel_layer = get_channel_layer()


class SyncConsumer(WebsocketConsumer):

    # def __init__(self, *args, **kwargs):
    #     super(SyncConsumer).__init__(*args, **kwargs)
    #     self.client_info = None
    #     self.thname = None
    #     self.channel_group_name = None
    #     self.username = None
    #     self.hostname = None
    def device_exist(self, online_devcice_hostname):
        device_items = conn.hget('device_domain_info', online_devcice_hostname)
        if device_items:
            return True
        else:
            device = Devices.objects.filter(device_hostname=online_devcice_hostname).first()
            if device and device.is_domain:
                conn.hset('device_domain_info', online_devcice_hostname, 1)
                close_old_connections()
                return True
            close_old_connections()
        return False

    def pingresult(self, data_json):
        """客户端连接测试"""
        online_devcice_hostname = data_json['data'].get('hostname')
        # logger.info(online_devcice_hostname)
        nextline = json.dumps({"sender": "SERVER", "sn": "9-9-8",
                               "message": {"code": 400, "data": None,
                                           "message": f"处理online错误， {online_devcice_hostname}未找到"}},
                              ensure_ascii=False)
        try:
            if self.device_exist(online_devcice_hostname):
                self.hostname = online_devcice_hostname
                # logger.info(self.hostname)
                if conn.hget('actives', online_devcice_hostname):
                    pass
                else:
                    conn.hset('actives', online_devcice_hostname, 1)
                data_json['sender'] = "SERVER"
                data_json['message'] = "pong"
                nextline = json.dumps(data_json, ensure_ascii=False)  # 读取脚本输出内容
        except Exception as exp:
            logger.info(exp)
        # logger.info(nextline)
        # 发送消息到客户端
        self.send(text_data=nextline)

    def _async_raise(self, tid, exctype):
        """raises the exception, performs cleanup if needed"""
        tid = ctypes.c_long(tid)
        if not inspect.isclass(exctype):
            exctype = type(exctype)
        logger.info(f"'tid': {tid}, 'exctype': {exctype}")
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, ctypes.py_object(exctype))
        if res == 0:
            raise ValueError("invalid thread id")
        elif res != 1:
            # """if it returns a number greater than one, you're in trouble,
            # and you should call it again with exc=NULL to revert the effect"""
            ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, None)
            raise SystemError("PyThreadState_SetAsyncExc failed")

    def stop_thread(self, thread):
        try:
            conn.hdel('device_online', self.client_info[0])
            if self.hostname:
                conn.hdel('actives', self.hostname)
                # 清除redis缓存的通道名称
                # logger.info(f'{self.hostname} redis 缓存的通道信息已经清除')
                conn.hdel('channel_name', self.hostname)
        except Exception as exp:
            logger.info(f'删除redis信息错误, {exp}')
        else:
            logger.info(f'{self.client_info[0]}在线信息已经清除')
        # logger.info(f'正在关闭的线程, {thread.ident}')
        # self._async_raise(thread.ident, SystemExit)
        self.thname[self.channel_group_name] = None

    def connect(self):
        # self.hostname = self.scope['url_route']['kwargs']['hostname']
        self.client_info = self.scope["client"] if "client" in self.scope.keys() else None
        self.username = time.time()  # 临时固定用户名
        # logger.info(f'WebSocket建立连接： {self.username}')
        # 直接从用户指定的通道名称构造通道组名称
        # client_info = "_".join([str(i) for i in self.client_info])
        ip = ""
        if self.client_info:
            ip = self.client_info[0]
        # ip = "172.30.98.36"
        # if ip:
        #     self.channel_group_name = f'msg_{ip}_{self.username}'
        #     self.ip = ip
        # else:
        #     self.channel_group_name = f'msg_{self.username}'
        #     self.ip = ip
        self.ip = ip
        query_string = self.scope['query_string']
        if query_string:
            query_params = {key: value[0] for key, value in parse_qs(query_string.decode('utf-8')).items()}
            self.hostname = query_params.get('hostname').replace(" ", "")
            self.channel_group_name = f'channel_{self.hostname}'
        else:
            self.hostname = None
            self.channel_group_name = f'channel_{ip}'
        self.thname = {f'{self.channel_group_name}': None}

        if conn.hget('device_online', self.client_info[0]):
            pass
        else:
            conn.hset('device_online', self.client_info[0], 1)

        # 加入通道层
        # async_to_sync(…)包装器是必需的，因为ChatConsumer是同步WebsocketConsumer，但它调用的是异步通道层方法。(所有通道层方法都是异步的。)
        async_to_sync(self.channel_layer.group_add)(
            self.channel_group_name,
            self.channel_name
        )

        # 接受WebSocket连接。
        self.accept()

        async_to_sync(self.channel_layer.group_send)(
            self.channel_group_name,
            {
                'type': 'get_message',
            }
        )

    def disconnect(self, close_code):
        # 离开通道
        self.stop_thread(self.thname[self.channel_group_name])

        async_to_sync(self.channel_layer.group_discard)(
            self.channel_group_name,
            self.channel_name
        )

        # logger.info('离开通道')

    # 从WebSocket中接收消息
    def receive(self, text_data=None, bytes_data=None):
        text_data_json = json.loads(text_data)
        sn = text_data_json['sn']
        # logger.info(f'WebSocket接收消息转对象：{text_data_json}, {type(text_data_json)}')
        # 发送消息到通道
        async_to_sync(self.channel_layer.group_send)(
            self.channel_group_name,
            {
                'type': 'get_message',
                'sn': sn,
                'data_json': text_data_json
            }
        )

    def send_message(self, event):
        data_json = event['data_json']
        logger.info("send_message %s" % data_json)
        # Send message to WebSocket
        self.send(text_data=json.dumps(data_json))

    # 从通道中接收消息
    def get_message(self, event):
        """{'type': 'get_message', 'sn': '9-9-9', 'data_json': {'sender': 'CLIENT', 'sn': '9-9-9', 'data': ''}}"""
        # logger.info(f"event: {event}, {type(event)}")
        cache = caches['redis']
        if event.get('sn'):
            sn = event['sn']
            # 判断消息，执行程序
            if sn == '9-9-8':
                hostname = event["data_json"]["data"].get("hostname")
                status = event["data_json"]["data"].get("status")
                self.hostname = hostname

                # 数据结构
                # {
                #     "wangchangyou.os-domain.cpu-os.com":{
                #         "channel_group_name":"msg_172.30.98.87_1686899737.0757215"
                #     }
                # }

                # 第一种方法
                # 将通道名称缓存到redis供后续调用
                redis_channel_name = conn.hget('channel_name', self.hostname)
                if redis_channel_name:
                    redis_channel_name = redis_channel_name.decode('utf-8')
                if self.channel_group_name != redis_channel_name:
                    conn.hset('channel_name', self.hostname, self.channel_group_name)

                # 第二种方法
                # 新增通道
                channel_info = {
                    "channel_group_name": self.channel_group_name
                }
                channel_data = {
                    self.hostname: channel_info
                }
                old_channel_info = cache.get(settings.REMOET_ASSIST_KEY)

                if old_channel_info:

                    old_device = old_channel_info.get(self.hostname)

                    if old_device:
                        # 更新通道
                        old_device["channel_group_name"] = self.channel_group_name
                        cache.set(settings.REMOET_ASSIST_KEY, old_channel_info)
                        cache.persist(settings.REMOET_ASSIST_KEY)
                    else:
                        # 新增通道
                        old_channel_info[self.hostname] = channel_info
                        cache.set(settings.REMOET_ASSIST_KEY, old_channel_info)
                        cache.persist(settings.REMOET_ASSIST_KEY)
                else:
                    cache.set(settings.REMOET_ASSIST_KEY, channel_data)
                    cache.persist(settings.REMOET_ASSIST_KEY)

                # self.thname[f"{self.channel_group_name}"] = threading.Thread(target=self.pingresult,
                #                                                              args=(event.get('data_json'),))
                # self.thname[f"{self.channel_group_name}"].start()

                self.pingresult(event.get('data_json'))
                self.thname[f"{self.channel_group_name}"] = None
                # device = Devices.objects.filter(device_hostname=self.hostname).first()
                # if device:
                #     if device.is_domain == 0 and status == 1:
                #         nextline = json.dumps({
                #             "sender": "SERVER",
                #             "sn": "1-1-3",
                #             "data": {"exit_domain": 1}
                #         }, ensure_ascii=False)
                #         self.send(text_data=nextline)
                # else:
                #     if status == 1:
                #         nextline = json.dumps({
                #             "sender": "SERVER",
                #             "sn": "1-1-3",
                #             "data": {"exit_domain": 1}
                #         }, ensure_ascii=False)
                #         self.send(text_data=nextline)

            # 判断消息
            elif sn == "9-9-9":
                """关闭线程 关闭websocket连接"""
                self.disconnect(self.channel_group_name)
                logger.info(f"关闭websocket连接 {self.thname[self.channel_group_name]}")
            # 远程连接
            # 获取终端版本
            elif sn == "9-8-1":
                data_json = event.get('data_json')
                """
                    sender: 1
                    sn: 9-8-1
                    data: {
                      string hostname: wzx-pc
                      string os: Found/UOS
                      string osversion: 4/5
                      string  clientversion: 1.0.1-m22
                    }

                    {
                        "sender": 1,
                        "sn": "9-8-1",
                        "data": {
                            "hostname": "wzx-pc",
                            "os": "Found",
                            "osversion": "5",
                            "clientversion": "1.0.1-m22"
                        }
                    }

                """
                hostname = data_json["data"].get("hostname")
                clientversion = data_json["data"].get("clientversion")
                key = "version"
                channel_info = cache.get(settings.REMOET_ASSIST_KEY)
                host_info = channel_info.get(hostname)
                logger.info(f'9-8-1: hostname {hostname} channel_info {channel_info}  host_info: {host_info}')

                # 增加主机中版本信息
                host_info[key] = clientversion
                # 数据结构
                # {
                #     "wangchangyou.os-domain.cpu-os.com":{
                #         "channel_group_name":"msg_172.30.98.87_1686899737.0757215",
                #         "version":"1.0.1-m40",
                #     }
                # }
                cache.set(settings.REMOET_ASSIST_KEY, channel_info)
                cache.persist(settings.REMOET_ASSIST_KEY)
                logger.info("存储终端版本号 save key %s clientversion %s" % (key, clientversion))

            # 终端反馈远程协助结果（当需要终端用户点击确认远程时）
            elif sn == "9-8-5":
                data_json = event.get('data_json')
                """
                sender: 1
                sn: 9-8-5
                data: {
                  string hostname: wzx-pc
                  string url
                }

                启动成功,返回格式
                {
                    "sender": 1,
                    "sn": "9-8-5",
                    "data": {
                        "hostname": "wzx-pc",  
                        "op": "start",                     
                        "url": "********",
                        "errmsg":""
                    }
                }
                启动失败,返回格式
                {
                    "sender": 1,
                    "sn": "9-8-5",
                    "data": {
                        "hostname": "wzx-pc",   
                        "op": "start",                     
                        "url": "",
                        "errmsg":"远程协助服务启动失败"
                    }
                }

                关闭失败,返回格式
                {
                    "sender": 1,
                    "sn": "9-8-5",
                    "data": {
                        "hostname": "wzx-pc",   
                        "op": "stop",                     
                        "url": "",
                        "errmsg":"远程协助服务关闭失败"
                    }
                }

                """

                hostname = data_json["data"].get("hostname")
                url_key = "url"
                url = data_json["data"].get("url")
                logger.info(f'9-8-5:  data_json: {data_json}')

                channel_info = cache.get(settings.REMOET_ASSIST_KEY)
                host_info = channel_info.get(hostname)

                # 增加主机中版本信息
                host_info[url_key] = url

                op = data_json["data"].get("op")
                errmsg = data_json["data"].get("errmsg", "")

                host_info["errmsg"] = errmsg
                cache.set(settings.REMOET_ASSIST_KEY, channel_info)
                cache.persist(settings.REMOET_ASSIST_KEY)
                if op == "start":
                    logger.info(f'9-8-5: hostname {hostname} save url_key: {url}')
                # logger.info(f'9-8-5: hostname {hostname} save channel_info: {channel_info}')

            # 用户登陆（获取用户在线信息）
            elif sn == "9-1-1":
                """
                用于获取用户在线信息,用户与终端为多对多关系，如果用户登陆三台终端，离线两台仍然显示在线
                sender: CLIENT
                sn: 9-1-1
                data: {
                  string hostname: wzx-pc
                  string username: wzx12345
                }
                """
                data_json = event.get('data_json')
                hostname = data_json["data"].get("hostname")
                username = data_json["data"].get("username")
                # logger.info("hostname %s username %s" % (hostname, username))
                try:
                    if not self.device_exist(hostname):
                        return False
                except Exception as exp:
                    logger.info(exp)
                    return False

                # 数据结构
                # {
                #   hostname:[username1,username2]
                # }

                key = "device_user_active"
                old_data_str = cache.get(key)
                if isinstance(old_data_str, dict):
                    # 兼容脏数据
                    old_data_str = ""
                if not old_data_str:
                    user_list = [username]
                    data = {
                        hostname: user_list
                    }
                    cache.set(key, json.dumps(data), 65)
                    # logger.info("存储在线用户信息 key %s data %s" % (key, data))
                else:
                    old_data = json.loads(old_data_str)
                    old_user_list = old_data.get(hostname)

                    if hostname in list(old_data.keys()):
                        # 同一用户
                        if username in old_user_list:
                            pass
                        else:
                            # 不同用户,增加用户记录
                            old_user_list.append(username)
                            cache.set(key, json.dumps(old_data), 65)
                            # logger.info("存储在线用户信息 key %s data %s" % (key, old_data))
                    else:
                        # 不同设备，增加主机记录
                        user_list = [username]
                        old_data[hostname] = user_list
                        cache.set(key, json.dumps(old_data), 65)
                        # logger.info("存储在线用户信息 key %s data %s" % (key, old_data))

            # 客户端自主加域 salt
            elif sn == "9-7-1":
                data_json = event.get('data_json')
                """
                sender: 1
                    sn: 9-7-1
                    data: {
                      string hostname: 127.0.0.1@os-domain.cpu-os.com
                      string ip: 127.0.0.1
                      int type: 1  1|申请加域 2|客户端自主加域成功 3|客户端自主加域失败
                      int is_domain: 2  1|加域模式  2|管控模式
                    }
                """
                req_type = data_json.get('data').get('type')
                hostname = data_json.get('data').get('hostname')
                ip = data_json.get('data').get('ip')
                is_domain = data_json.get('data').get('is_domain')
                if req_type == 1:
                    if not self.device_exist(hostname):
                        if not conn.get(f'{ip}_adding_domain'):
                            nextline = json.dumps({
                                "sender": "SERVER",
                                "sn": "9-7-1",
                                "data": {"code": 200, "message": f"{hostname} 可以加域"}
                            }, ensure_ascii=False)
                            self.send(text_data=nextline)
                        else:
                            nextline = json.dumps({
                                "sender": "SERVER",
                                "sn": "9-7-1",
                                "data": {"code": 400, "message": f"{hostname} 正在加域"}
                            }, ensure_ascii=False)
                            self.send(text_data=nextline)
                    else:
                        nextline = json.dumps({
                            "sender": "SERVER",
                            "sn": "9-7-1",
                            "data": {"code": 400, "message": f"{hostname} 已经在域中"}
                        }, ensure_ascii=False)
                        self.send(text_data=nextline)

                elif req_type == 2:
                    signal = {
                        'event_type': 'client_own_domain',
                        'device_ip': ip,
                        'device_hostname': hostname,
                        'success': True,
                        'msg': f'终端{ip}自主加域成功',
                        'is_domain': is_domain
                    }
                    public = RedisHelper()
                    public.public(json.dumps(signal), chan='freeipa')
                    nextline = json.dumps({
                        "sender": "SERVER",
                        "sn": "9-7-1",
                        "data": "pong"
                    }, ensure_ascii=False)
                    self.send(text_data=nextline)
                elif req_type == 3:
                    signal = {
                        'event_type': 'client_own_domain',
                        'device_ip': ip,
                        'device_hostname': hostname,
                        'success': False,
                        'msg': f'终端{ip}自主加域失败',
                        'is_domain': is_domain
                    }
                    public = RedisHelper()
                    public.public(json.dumps(signal), chan='freeipa')
            # 9-5-6  查询终端补丁信息
            elif sn == "9-5-7":
                """
                返回终端补丁安装状态
                sender: CLIENT
                sn: 9-5-7
                data: {
                  device_id: ***，
                  patch_status_list: [{'patchname':"****","status":"0|1|2"}，{'patchname':"****","status":"0|1|2"}]
                }
                :param event:
                :return:
                """
                data_json = event.get('data_json')
                logger.info(f"9-5-7 {data_json}, {type(data_json)}")
                device_id = data_json["data"].get("device_id")
                patch_status_list = data_json["data"].get("patch_status_list")
                if '[' in patch_status_list:
                    patch_status_list = json.loads(patch_status_list)
                else:
                    patch_status_list = []
                if (not patch_status_list) and (isinstance(patch_status_list, list)):
                    PatchDeviceLog.objects.filter(device_id=device_id).update(
                        install_status=0,
                        install_time=datetime.datetime.now(),
                        install_err_msg=None,
                        is_send_status=0,
                        send_msg_time=datetime.datetime.now(),
                        is_receive_status=0
                    )
                else:
                    if patch_status_list is None:
                        patch_status_list = []
                    patch_file_items = PatchFile.objects.all()
                    patch_info_items = PatchInfo.objects.all()
                    for i in patch_status_list:
                        # logger.info(f"******{i},{type(i)}")
                        patch_name = i.get('patchname')
                        install_status = i.get('status')
                        if install_status == 0:
                            install_status = 1
                        elif install_status == 1:
                            install_status = 0
                        elif install_status == 2:
                            install_status = 2
                        else:
                            install_status = 0
                        if patch_file_items.filter(name=patch_name).exists():
                            patch_file_id = patch_file_items.get(name=patch_name).id
                            patch_id = patch_info_items.get(patch_file_id=patch_file_id).id
                            patch_device_log_items = PatchDeviceLog.objects.filter(device_id=device_id,
                                                                                   patch_id=patch_id)
                            if install_status == 0:
                                patch_device_log_items.update(
                                    install_status=install_status,
                                    install_time=datetime.datetime.now(),
                                    install_err_msg=None,
                                    is_send_status=0,
                                    send_msg_time=datetime.datetime.now(),
                                    is_receive_status=0
                                )
                            elif install_status == 1:
                                patch_device_log_items.update(
                                    install_status=install_status,
                                    install_time=datetime.datetime.now(),
                                    install_err_msg='成功',
                                    is_send_status=1,
                                    send_msg_time=datetime.datetime.now(),
                                    is_receive_status=1
                                )
                            else:
                                patch_device_log_items.update(
                                    install_status=install_status,
                                    install_time=datetime.datetime.now(),
                                    is_send_status=1,
                                    send_msg_time=datetime.datetime.now(),
                                    is_receive_status=1
                                )
                        else:
                            pass
            # 9-5-7  查询补丁策略下发信息状态
            elif sn == "9-6-7":
                """
                补丁安装终端返回信息处理
                sender: CLIENT
                sn: 9-6-7
                data: {'reason': 2, 'res': 1, 'taskid': '11-3011'}
                :param event:
                :return:
                """
                data_json = event.get('data_json')
                logger.info(f"9-6-7 {data_json}, {type(data_json)}")
                task_id = data_json["data"].get("taskid")
                device_id = task_id.split('-')[0]
                patch_id = task_id.split('-')[1]
                install_status = data_json["data"].get("res")
                if install_status == 0:
                    install_status = 2
                elif install_status == 1:
                    install_status = 0
                else:
                    install_status = 0
                install_err_msg = data_json["data"].get("reason")
                if install_err_msg == 0:
                    install_err_msg = '成功'
                elif install_err_msg == 1:
                    install_err_msg = '空间不足'
                elif install_err_msg == 2:
                    install_err_msg = '补丁非法(补丁跟系统版本不一致，名称校验）'
                elif install_err_msg == 3:
                    install_err_msg = '下载失败'
                elif install_err_msg == 4:
                    install_err_msg = '安装异常'
                else:
                    pass
                PatchDeviceLog.objects.filter(device_id=device_id, patch_id=patch_id).update(
                    install_status=install_status,
                    install_err_msg=install_err_msg,
                    install_time=datetime.datetime.now(),
                    is_receive_status=1
                )
            # 1-1-0  发送验证码
            elif sn == "1-1-0":
                """
                sender: client
                sn: 1-1-0
                data: {
                  string userNum: xxx
                }

                "sender": "SERVER",
                "sn": "1-1-0",
                "data": {
                    "code": 400 | 200,
                    "userNum": xxx
                    "message": '发送失败！短信异常员工号不存在。'
                } 
                """
                logger.info("======= 1-1-0 ======")
                req_uuid = get_uuid()
                data_json = event.get('data_json')
                data_json.get('data')['uuid'] = req_uuid
                logging.info(data_json)
                user_num = data_json.get('data').get('userNum')
                domain_auth = DomainLicenseAuth()
                if not domain_auth.license_b:
                    nextline = json.dumps({
                        "sender": "SERVER",
                        "sn": "1-1-0",
                        "data": {"code": 400, "message": '服务器未激活,请联系管理员', 'uuid': req_uuid}
                    }, ensure_ascii=False)
                    logger.info(nextline)
                    return self.send(text_data=nextline)
                sm = get_soapmember()
                account = AccountEntity.objects.filter(name=f'{sm.user_prefix}{user_num}').first()
                if account:
                    domain_user_create_type = account.create_type
                else:
                    domain_user_create_type = 0
                if (user_num != settings.DOMAIN_SERVER_ADMIN) and (domain_user_create_type == 2) and (
                        sm.client.connect()):
                    soap_data = {
                        "userId": user_num,
                    }

                    send_res = sm.send_code(soap_data)
                    if send_res.get('status') == 'error':
                        nextline = json.dumps({
                            "sender": "SERVER",
                            "sn": "1-1-0",
                            "data": {"code": 400, "message": send_res.get('message'), "userNum": user_num, 'uuid': req_uuid}
                        }, ensure_ascii=False)
                        logger.info(nextline)
                        return self.send(text_data=nextline)
                nextline = json.dumps({
                    "sender": "SERVER",
                    "sn": "1-1-0",
                    "data": {"code": 200, "userNum": user_num, 'uuid': req_uuid}
                }, ensure_ascii=False)
                logger.info(nextline)
                return self.send(text_data=nextline)
            # 1-1-1  加域
            elif sn == "1-1-1":
                """
                sender: client
                sn: 1-1-1
                data: {
                  string ip: 127.0.0.1
                  string domain_user: test001
                  string domain_user_pwd: 123456
                  string device_cnname: xxx
                  int freejoin: 1  0不是，1 是
                  string authType: M_PASSWORD|密码(默认), M_FINGER|指纹, M_FINGER_CARD|指纹卡, M_OTP|短信
                  string verifyCode:  为短信认证时必输入
                  string fingerPassword:  为指纹卡时必输入
                  string fingerData:  为指纹时必输入
                }

                "sender": "SERVER",
                "sn": "1-1-1",
                "data": {
                    "code": 200, 
                    "message": f"{hostname} 可以加域",
                    "user":"nfs-test001",
                    "domain_server_fqdn": server.os-domain.cpu-os.com
                    "adddomaincmd": 'ipa-client-install --server server.os-domain.cpu-os.com --domain os-domain.cpu-os.com --hostname test-9-13.os-domain.cpu-os.com --password 12345678 --principal admin --ntp-server server.os-domain.cpu-os.com --force-join --mkhomedir --unattended  '

                }  200|可以加域  400|不可以加域

                """
                req_uuid = get_uuid()
                data_json = event.get('data_json')
                data_json.get('data')['uuid'] = req_uuid
                logger.info("======= 1-1-1 ======")
                device_ip = data_json.get('data').get('ip')
                domain_user = data_json.get('data').get('domain_user')
                domain_user_pwd = data_json.get('data').get('domain_user_pwd')
                device_cnname = data_json.get('data').get('device_cnname')
                device_uuid = device_hostname = data_json.get('data').get('device_hostname')
                freejoin = data_json.get('data').get('freejoin')
                auth_type = data_json.get('data').get('authType', 'M_PASSWORD')
                verify_code = data_json.get('data').get('verifyCode')
                finger_data = data_json.get('data').get('fingerData')
                finger_password = data_json.get('data').get('fingerPassword')
                domain_auth = DomainLicenseAuth()
                if not domain_auth.license_b:
                    nextline = json.dumps({
                        "sender": "SERVER",
                        "sn": "1-1-1",
                        "data": {"code": 400, "message": '服务器未激活,请联系管理员', 'uuid': req_uuid}
                    }, ensure_ascii=False)
                    logger.info(nextline)
                    return self.send(text_data=nextline)
                mac = os.environ.get("DOMAIN_MAC")
                license_info = read_licenseB(mac, domain_auth.license_b)
                max_users = bytes_to_string(license_info.get('max_users_res', 0))
                if Devices.objects.exclude(is_domain=0).all().count() >= int(max_users):
                    nextline = json.dumps({
                        "sender": "SERVER",
                        "sn": "1-1-1",
                        "data": {"code": 400, "message": '超出最大终端管控数,请联系管理员', 'uuid': req_uuid}
                    }, ensure_ascii=False)
                    logger.info(nextline)
                    return self.send(text_data=nextline)
                if not device_ip:
                    nextline = json.dumps({
                        "sender": "SERVER",
                        "sn": "1-1-1",
                        "data": {"code": 400, "message": '缺少设备ip', 'uuid': req_uuid}
                    }, ensure_ascii=False)
                    logger.info(nextline)
                    return self.send(text_data=nextline)
                if not device_cnname:
                    nextline = json.dumps({
                        "sender": "SERVER",
                        "sn": "1-1-1",
                        "data": {"code": 400, "message": '缺少设备别名', 'uuid': req_uuid}
                    }, ensure_ascii=False)
                    logger.info(nextline)
                    return self.send(text_data=nextline)
                if Devices.objects.filter(device_cnname=device_cnname).exclude(is_domain=0).first():
                    nextline = json.dumps({
                        "sender": "SERVER",
                        "sn": "1-1-1",
                        "data": {"code": 400, "message": '主机名已被使用，请修改终端别名', 'uuid': req_uuid}
                    }, ensure_ascii=False)
                    logger.info(nextline)
                    return self.send(text_data=nextline)
                # device_cnname = init_device_hostname(domain_user)
                if is_satisfy_black_lists(domain_user):
                    nextline = json.dumps({
                        "sender": "SERVER",
                        "sn": "1-1-1",
                        "data": {"code": 400, "message": '用户符合黑名单规则,禁止加域', 'uuid': req_uuid}
                    }, ensure_ascii=False)
                    logger.info(nextline)
                    return self.send(text_data=nextline)
                ldap_ad = get_ldap_ad()
                sm = get_soapmember()
                if not freejoin:
                    if not domain_user or not domain_user_pwd:
                        nextline = json.dumps({
                            "sender": "SERVER",
                            "sn": "1-1-1",
                            "data": {"code": 400, "message": '缺少域用户或密码', 'uuid': req_uuid}
                        }, ensure_ascii=False)
                        logger.info(nextline)
                        return self.send(text_data=nextline)
                    if domain_user != settings.DOMAIN_SERVER_ADMIN and ldap_ad:
                        account = AccountEntity.objects.filter(name=domain_user).first()
                        if account:
                            if (account.create_type==1):
                                check_res = ldap_ad.auth_user(domain_user,sm4_decrypt(domain_user_pwd, settings.SM4_KEY))
                                if not check_res.get('status'):
                                    nextline = json.dumps({
                                        "sender": "SERVER",
                                        "sn": "1-1-1",
                                        "data": {"code": 400, "message": check_res.get('message'), 'uuid': req_uuid}
                                    }, ensure_ascii=False)
                                    logger.info(nextline)
                                    return self.send(text_data=nextline)
                            else:
                                check_res = check_user(domain_user, sm4_decrypt(domain_user_pwd, settings.SM4_KEY))
                                if not check_res.get('exist'):
                                    nextline = json.dumps({
                                        "sender": "SERVER",
                                        "sn": "1-1-1",
                                        "data": {"code": 400, "message": '用户不存在或密码错误或已被锁定', 'uuid': req_uuid}
                                    }, ensure_ascii=False)
                                    logger.info(nextline)
                                    return self.send(text_data=nextline)
                        else:
                            nextline = json.dumps({
                                "sender": "SERVER",
                                "sn": "1-1-1",
                                "data": {"code": 400, "message": '此账号暂未同步', 'uuid': req_uuid}
                            }, ensure_ascii=False)
                            logger.info(nextline)
                            return self.send(text_data=nextline)

                    elif domain_user != settings.DOMAIN_SERVER_ADMIN and sm:
                        account = AccountEntity.objects.filter(name=f'{sm.user_prefix}{domain_user}').first()
                        if account:
                            if (account.create_type == 2) and (sm.client.connect()):
                                soap_data = {
                                    "appId": sm.appid,
                                    "userId": domain_user,
                                    "password": sm4_decrypt(domain_user_pwd, sm.soap_sm4_keys),
                                    "requestId": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
                                    "authType": auth_type,
                                    "requestName": None,
                                    "requestReason": None,
                                    'authFlag': "1"
                                }
                                if auth_type == 'M_OTP':
                                    soap_data['verifyCode'] = verify_code
                                elif auth_type == 'M_FINGER':
                                    soap_data['fingerData'] = finger_data
                                elif auth_type == 'M_FINGER_CARD':
                                    soap_data['fingerPassword'] = finger_password
                                check_res = sm.get_auth_by_sm4(soap_data)
                                if check_res.get('status') == 'error' and check_res.get('code') not in (10009, 10200):
                                    nextline = json.dumps({
                                        "sender": "SERVER",
                                        "sn": "1-1-1",
                                        "data": {"code": 400, "message": check_res.get('message'), 'uuid': req_uuid}
                                    }, ensure_ascii=False)
                                    logger.info(nextline)
                                    return self.send(text_data=nextline)
                                domain_user = f'{sm.user_prefix}{domain_user}'
                            else:
                                check_res = check_user(domain_user, sm4_decrypt(domain_user_pwd, settings.SM4_KEY))
                                if not check_res.get('exist'):
                                    nextline = json.dumps({
                                        "sender": "SERVER",
                                        "sn": "1-1-1",
                                        "data": {"code": 400, "message": '用户不存在或密码错误或已被锁定', 'uuid': req_uuid}
                                    }, ensure_ascii=False)
                                    logger.info(nextline)
                                    return self.send(text_data=nextline)
                        else:
                            nextline = json.dumps({
                                "sender": "SERVER",
                                "sn": "1-1-1",
                                "data": {"code": 400, "message": '此账号暂未同步', 'uuid': req_uuid}
                            }, ensure_ascii=False)
                            logger.info(nextline)
                            return self.send(text_data=nextline)
                    else:
                        check_res = check_user(domain_user, sm4_decrypt(domain_user_pwd, settings.SM4_KEY))
                        if not check_res.get('exist'):
                            nextline = json.dumps({
                                "sender": "SERVER",
                                "sn": "1-1-1",
                                "data": {"code": 400, "message": '用户不存在或密码错误或已被锁定', 'uuid': req_uuid}
                            }, ensure_ascii=False)
                            logger.info(nextline)
                            return self.send(text_data=nextline)
                else:
                    if sm:
                        domain_user = f'{sm.user_prefix}{domain_user}'
                    if not user_find(domain_user):
                        nextline = json.dumps({
                            "sender": "SERVER",
                            "sn": "1-1-1",
                            "data": {"code": 400, "message": '用户不存在', 'uuid': req_uuid}
                        }, ensure_ascii=False)
                        logger.info(nextline)
                        return self.send(text_data=nextline)
                if conn.get(f'{device_hostname}_adding_domain'):
                    nextline = json.dumps({
                        "sender": "SERVER",
                        "sn": "1-1-1",
                        "data": {"code": 400, "message": f"{device_hostname} 正在加域"}
                    }, ensure_ascii=False)
                    logger.info(nextline)
                    return self.send(text_data=nextline)
                device, is_create = Devices.objects.get_or_create(device_uuid=device_uuid)
                if device.is_domain:
                    nextline = json.dumps({
                        "sender": "SERVER",
                        "sn": "1-1-1",
                        "data": {"code": 400, "message": '终端已在域中', 'uuid': req_uuid}
                    }, ensure_ascii=False)
                    logger.info(nextline)
                    return self.send(text_data=nextline)
                device.device_ip = device_ip
                device.device_hostname = device_hostname
                device.device_cnname = device_cnname
                device.save()
                # domain_uuid = str(uuid.uuid4())
                # conn.hset(domain_uuid, {'salt':False, 'freeipa':False}, 60 * 30)
                cmd = f'ipa-client-install --server {os.environ.get("DOMAIN_SERVER_FQDN")} --domain {os.environ.get("DOMAIN_NAME")} --hostname {device_hostname} --password  {os.environ.get("DOMAIN_SERVER_ADMIN_PASSWD")} --principal admin --ntp-server {os.environ.get("DOMAIN_SERVER_FQDN")} --enable-dns-update --force-join --unattended'
                nextline = json.dumps({
                    "sender": "SERVER",
                    "sn": "1-1-1",
                    "data": {
                        "code": 200,
                        "message": f"{device_hostname} 可以加域",
                        "adddomaincmd": sm4_hash(cmd, settings.SM4_KEY),
                        'uuid': req_uuid,
                        'user': domain_user.lower(),
                        'domain_server_fqdn': os.environ.get("DOMAIN_SERVER_FQDN"),
                        'device_cnname': device_cnname
                    }
                }, ensure_ascii=False)
                logger.info(nextline)
                conn.set(f'{device_hostname}_adding_domain', 1, ex=100, nx=True)
                # logger.info(
                #     '''
                #     salt "{device_hostname}" stats.sls nfs.freeipa-client-init.join_domain piller='{"server":"{fqdn}","domain":"{domain_name}","password":"12345678","hostname":"{device_hostname}","principal":"admin","ntpserver":"{fqdn}"}'
                #     '''.format(device_hostname=device_hostname,fqdn=os.environ.get("DOMAIN_SERVER_FQDN"),domain_name=os.environ.get("DOMAIN_NAME"))
                # )
                return self.send(text_data=nextline)
            elif sn == "1-1-2":
                """
                sender: 1
                sn: 1-1-2
                data: {
                  string device_hostname: xxx.os-domain.cpu-os.com
                  string domain_user: xxx
                  string ip: 120.7.0.1
                  string message: 加域失败原因  当客户端加域失败后，将分析的原因发送给后端，后端在将失败的原因返回给客户端
                  int result: 0  0|加域成功 1|加域失败
                  string log: xxxxxxxx (暂时不用)
                }

                "sender": "SERVER",
                "sn": "1-1-2",
                "data": {"code": 200, "message": "加域成功"}  200|加域成功  400|加域失败

                """
                data_json = event.get('data_json')
                logger.info(data_json)
                result = data_json.get('data').get('result')
                req_uuid = data_json.get('data').get('uuid')
                domain_user = data_json.get('data').get('domain_user')
                device_ip = data_json.get('data').get('ip')
                message = data_json.get('data').get('message')
                log = data_json.get('data').get('log')
                device_hostname = data_json.get('data').get('device_hostname')
                if result == 0:
                    device = Devices.objects.filter(device_uuid=device_hostname).first()
                    account = AccountEntity.objects.filter(name=domain_user).first()
                    device.is_domain = 1
                    if domain_user != os.environ.get('DOMAIN_SERVER_ADMIN'):
                        account = AccountEntity.objects.filter(name=domain_user).first()
                        device.org_info = account.org_info
                        device.bind_account.add(account)
                        device.belong_user = account.cn_name if account.cn_name else domain_user
                    device.save()
                    bind_device_account(device_hostname, account.id)
                    nextline = json.dumps({
                        "sender": "SERVER",
                        "sn": "1-1-2",
                        "data": {"code": 200, "message": '加域成功', 'uuid': req_uuid}
                    }, ensure_ascii=False)
                    signal = {
                        'event_type': 'install',
                        'device_ip': device_ip,
                        'device_hostname': device_hostname,
                        'success': True,
                        'msg': f'终端{device_hostname}加域成功',
                        'os': data_json.get('data').get('os') or 'Nfsdesktop',
                        'os_release': data_json.get('data').get('os_release') or '5.0',
                        'os_arch': data_json.get('data').get('os_arch') or 'x86_64'
                    }
                    public = RedisHelper()
                    public.public(json.dumps(signal), chan='freeipa')
                    logger.info(nextline)
                    self.send(text_data=nextline)
                # alter_time = datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%S.%f')
                # full_ret = {"cmd":"_return","id":device_hostname,"success":False, "return":{"ws.domain.log":log}}
                # SaltJidDevice.objects.add_jid('', 'ws.domain', jid, device_hostname, full_ret.get('return'), 0,
                #                               full_ret,
                #                               alter_time, ['ws.domain'])
                else:
                    nextline = json.dumps({
                        "sender": "SERVER",
                        "sn": "1-1-2",
                        "data": {"code": 400, "message": f'加域失败 {message}', 'uuid': req_uuid}
                    }, ensure_ascii=False)
                    signal = {
                        'event_type': 'install',
                        'device_ip': device_ip,
                        'device_hostname': device_hostname,
                        'success': False,
                        'msg': f'终端{device_hostname}加域失败',
                    }
                    public = RedisHelper()
                    public.public(json.dumps(signal), chan='freeipa')
                    logger.info(nextline)
                    self.send(text_data=nextline)
            elif sn == "1-1-3":
                """
                "sender": "SERVER",
                "sn": "1-1-3",
                "data": {"exit_domain": 1}
                """
                nextline = json.dumps({
                    "sender": "SERVER",
                    "sn": "1-1-3",
                    "data": {"exit_domain": 1}
                }, ensure_ascii=False)
                self.send(text_data=nextline)
            elif sn == "1-1-4":
                """
                sender: 1
                sn: 1-1-4
                data: {
                  string device_hostname: xxx.os-domain.cpu-os.com
                  int result: 0  0|退域成功 1|退域失败
                }
                """
                data_json = event.get('data_json')
                logger.info(data_json)
                # device_hostname = data_json.get('data').get('device_hostname')
                # result = data_json.get('data').get('result')
                # public = RedisHelper()
                # signal = {
                #     'event_type': 'delete',
                #     'device_ip': self.ip,
                #     'device_hostname': device_hostname,
                # }
                # if result == 0:
                #     signal['success'] = True
                #     signal['msg'] = '{message} 退域成功'.format(message=device_hostname)
                #     conn.hdel('wait_exit_domain', device_hostname)
                # else:
                #     signal['success'] = False
                #     signal['msg'] = '{message} 退域失败'.format(message=device_hostname)
                # public.public(json.dumps(signal), chan='freeipa')
            elif sn == "9-9-7":
                """
                设备在线下发终端未处理的指令
                {
                        "sender": client,
                        "sn": "9-9-7",
                        "data": {
                            "hostname": "*****.os"
                        }
                    }
                """
                data_json = event.get('data_json')
                hostname = data_json.get('data').get('hostname')
                logger.info(f'9-9-7:  data_json: {data_json}')
                # 查看管道不在线没发送的消息
                device_unsent_message = conn.hget('client_websocket_unsent', hostname)
                if device_unsent_message is None:
                    pass
                else:
                    device_unsent_message = json.loads(device_unsent_message)
                    if len(device_unsent_message) == 0:
                        conn.hdel('client_websocket_unsent', hostname)
                    else:
                        err_list = []
                        for i in device_unsent_message:
                            try:
                                send_ws_msg(self.channel_group_name, i)
                            except Exception as exp:
                                err_list.append(i)
                        if len(err_list) == 0:
                            conn.hdel('client_websocket_unsent', hostname)
                        else:
                            conn.hset('client_websocket_unsent', hostname, json.dumps(err_list, ensure_ascii=False))
            else:
                pass


#

# 发送websocket消息
def send_ws_msg(channel_group_name, data_json):
    """
    websocket 发送消息
    :param data_josn:
        data_json = {
            "sender": "2",
            "sn": "9-8-1",
            "data": {
                "adminname": "admin"
            }
        }
    :return:
    """
    logger.info("*********** send_ws_msg start ****************")
    logger.info(f'channel_group_name： {channel_group_name}')
    logger.info(f'send messgae data_json： {data_json}')
    sn = data_json["sn"]
    async_to_sync(channel_layer.group_send)(
        channel_group_name,
        {
            "type": "send.message",
            "sender": "2",
            "sn": sn,
            "data_json": data_json
        }
    )
    logger.info("*********** send_ws_msg  end ****************")
    return True
