# -*- coding: utf-8 -*-
from django.db import models

from accounts.models import AccountEntity


# from saltapi.models import SaltPolicy
# Create your models here.


class Devices(models.Model):
    device_up = (
        (0, 'down'), (1, 'up')
    )
    device_delete = (
        (0, 'N'), (1, 'Y')
    )
    device_domain = (
        (0, 'fail'), (1, 'success')
    )
    auto_domain = ((0, 'NO'), (1, 'YES'))
    device_uuid = models.CharField(verbose_name='设备UUID', max_length=255, null=True, blank=True)
    device_ip = models.CharField(verbose_name='设备IP', max_length=255, blank=True)
    device_port = models.IntegerField(verbose_name='设备端口', null=True, blank=True)
    device_username = models.CharField(verbose_name='设备ssh账号', max_length=255, blank=True, null=True)
    device_passwd = models.CharField(verbose_name='设备ssh密码', max_length=255, blank=True, null=True)
    device_auto_domain = models.IntegerField(verbose_name='是否完成加域', choices=auto_domain, default=0, blank=True)
    device_auto_client = models.IntegerField(verbose_name='自动安装客户端', choices=auto_domain, default=1, blank=True)
    device_hostname = models.CharField(verbose_name='设备hostname', max_length=255, null=True, blank=True)
    device_cnname = models.CharField(verbose_name='设备别名', max_length=255, null=True, blank=True)
    device_addr = models.CharField(verbose_name='设备地址', max_length=255, null=True, blank=True)
    device_root = models.CharField(verbose_name='root', max_length=32, default='root', blank=True)
    device_root_pwd = models.CharField(verbose_name='root_pwd', max_length=255, blank=True)
    accounts_id = models.IntegerField(verbose_name='用户ID', null=True, blank=True)
    is_up = models.IntegerField(verbose_name='是否启用', choices=device_up, default=1, blank=True)
    is_delete = models.IntegerField(verbose_name='是否刪除', choices=device_delete, default=0, blank=True)
    note = models.CharField(verbose_name='备注', max_length=255, null=True, blank=True)
    os = models.CharField(verbose_name='系统', max_length=255, null=True, blank=True)
    os_release = models.CharField(verbose_name='系统版本', max_length=255, null=True, blank=True)
    os_arch = models.CharField(verbose_name='系统架构', max_length=255, null=True, blank=True)
    is_domain = models.IntegerField(verbose_name='是否完成加域', choices=device_domain, default=0, blank=True)
    bind_account = models.ManyToManyField(AccountEntity, related_name='devices', blank=True,
                                          verbose_name='设备绑定的终端')
    org_info = models.ForeignKey(to='organizations.OrgnizationEntity', on_delete=models.PROTECT, db_constraint=False,
                                 default=1, blank=True, verbose_name='所属组织机构')
    belong_user = models.CharField(verbose_name='所属用户', max_length=255, null=True, blank=True)
    add_domain_time = models.DateTimeField(verbose_name='加域时间', auto_now_add=True, null=True, blank=True)
    device_uid = models.CharField(verbose_name='设备ID (MAC+SN+硬盘序列号)', max_length=255, null=True, blank=True)
    device_sid = models.CharField(verbose_name='设备SID', max_length=255, null=True, blank=True)

    class Meta:
        verbose_name = '设备'
        verbose_name_plural = '设备'

    def __str__(self):
        return self.device_ip


class DevicesHardware(models.Model):
    device = models.OneToOneField(verbose_name='设备', to=Devices, to_field='id', unique=True, on_delete=models.CASCADE)
    network = models.TextField(verbose_name='网卡', blank=True, null=True)
    cpu_model = models.CharField(verbose_name='处理器', max_length=255, null=True, blank=True)
    cpuarch = models.CharField(verbose_name='cpu架构', max_length=255, null=True, blank=True)
    mainboard = models.CharField(verbose_name='主板', max_length=255, null=True, blank=True)
    kernelrelease = models.CharField(verbose_name='内核版本', max_length=255, null=True, blank=True)
    productname = models.CharField(verbose_name='产品机型', max_length=255, null=True, blank=True)
    mem_total = models.CharField(verbose_name='内存', max_length=255, null=True, blank=True)
    os = models.CharField(verbose_name='操作系统', max_length=255, null=True, blank=True)

    class Meta:
        db_table = 'device_hardware'
        verbose_name = '设备硬件'
        verbose_name_plural = '设备硬件'


class DevicesHistory(models.Model):
    auto_domain = ((0, 'NO'), (1, 'YES'))
    op_type_choices = ((0, 'exit'), (1, 'add'))
    timestamp = models.DateTimeField(verbose_name='时间', auto_now_add=True, blank=True)
    op_type = models.IntegerField(verbose_name='历史', choices=op_type_choices, default=0, blank=True)
    device_uuid = models.CharField(verbose_name='设备UUID', max_length=255, null=True, blank=True)
    device_ip = models.CharField(verbose_name='设备IP', max_length=255, blank=True)
    device_username = models.CharField(verbose_name='设备ssh账号', max_length=255, blank=True, null=True)
    device_passwd = models.CharField(verbose_name='设备ssh密码', max_length=255, blank=True, null=True)
    device_auto_domain = models.IntegerField(verbose_name='是否完成加域', choices=auto_domain, default=0, blank=True)
    device_auto_client = models.IntegerField(verbose_name='自动安装客户端', choices=auto_domain, default=1, blank=True)
    device_hostname = models.CharField(verbose_name='设备hostname', max_length=255, null=True, blank=True)
    device_cnname = models.CharField(verbose_name='设备别名', max_length=255, null=True, blank=True)
    device_addr = models.CharField(verbose_name='设备地址', max_length=255, null=True, blank=True)
    accounts_cnname = models.CharField(verbose_name='用户名', null=True, max_length=255, blank=True)
    accounts_name = models.CharField(verbose_name='用户', null=True, max_length=255, blank=True)
    os = models.CharField(verbose_name='系统', max_length=255, null=True, blank=True)
    os_release = models.CharField(verbose_name='系统版本', max_length=255, null=True, blank=True)
    os_arch = models.CharField(verbose_name='系统架构', max_length=255, null=True, blank=True)
    org_info = models.TextField(verbose_name='组织机构', null=True, blank=True)

    class Meta:
        db_table = 'device_history'
        verbose_name = '设备历史'
        verbose_name_plural = '设备历史'
