from django_redis import get_redis_connection
from rest_framework import serializers

from accounts.models import AccountEntity
from devices.models import Devices, DevicesHistory
from organizations.models import OrgnizationEntity

conn = get_redis_connection('redis')


class DeviceSerializer(serializers.ModelSerializer):
    is_active = serializers.SerializerMethodField()
    org_name = serializers.SerializerMethodField()
    name = serializers.SerializerMethodField()
    cn_name = serializers.SerializerMethodField()

    class Meta:
        model = Devices
        fields = (
            'id', 'device_uuid', 'device_ip', 'device_port', 'device_username',
            "device_passwd", "device_auto_domain", "device_auto_client",
            'device_hostname', "device_cnname", "device_addr", "device_root",
            "device_root_pwd", "accounts_id", 'is_up', "is_delete", "note", "os",
            "os_release", "os_arch", "is_domain", 'belong_user',
            'cn_name', 'name', 'is_active', 'org_info_id', 'org_name', 'bind_account')

    def get_cn_name(self, obj):
        bind_accounts_list = obj.bind_account.all()
        items = AccountEntity.objects.filter(id__in=bind_accounts_list).first()
        if items:
            return items.cn_name
        else:
            return None

    def get_name(self, obj):
        bind_accounts_list = obj.bind_account.all()
        items = AccountEntity.objects.filter(id__in=bind_accounts_list).first()
        if items:
            return items.name
        else:
            return None

    def get_org_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()
        if items:
            return items.name
        else:
            return None

    def get_is_active(self, obj):
        if conn.hget('actives', obj.device_hostname):
            return 'true'
        else:
            return 'false'


class DeviceHistorySerializer(serializers.ModelSerializer):
    # op_type = serializers.SerializerMethodField()
    # def get_op_type(self, obj):
    #     return obj.get_op_type_display()

    class Meta:
        model = DevicesHistory
        # fields = (
        #     'id','op_type','device_uuid','device_ip','device_username',
        #     'device_passwd','device_auto_domain','device_auto_client',
        #     'device_hostname','device_cnname','device_addr','accounts_cnname',
        #     'accounts_name', 'os', 'os_release', 'os_arch', 'org_info','timestamp'
        # )
        fields = (
            'id', 'op_type', 'accounts_cnname', 'device_cnname', 'device_ip',
            'accounts_name', 'org_info', 'timestamp',
        )
        # fields = '__all__'
