import logging
import datetime
from django_redis import get_redis_connection
from concurrent.futures import ProcessPoolExecutor
from celery_task.celery import app
from celery.schedules import crontab
from devices.models import Devices
from devices.utils import Ping

logger = logging.getLogger(__name__)
conn = get_redis_connection('redis')

hour = '1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23'
week = 'mon,tue,wed,thu,fri,sat,sun'
# app.conf.beat_schedule = {
#     'get_device_active':{
#         'task':'devices.tasks.get_device_active',
#         # 'schedule':timedelta(seconds=60),
#         'schedule':crontab(minute='*/1',hour=hour, day_of_week=week),
#     },
#     'clear_device_active':{
#         'task':'devices.tasks.clear_device_active',
#         'schedule':crontab(minute=0, hour=0),  # 每天凌晨12点
#     }
# }

def call_back(res):
    '''
        '20220622':{'ae904e1f99d7b7a8.server.domain':1,...}
        '20220623':{'ae904e1f99d7b7a8.server.domain':1,...}
        ...

        device_active_rate = [
            '2022/06/22','2022/06/23', ...
        ]
        device_active_rate_hour = [
            06/22 10，
        ]
    '''
    today = datetime.datetime.now().strftime('%Y/%m/%d')
    hour = datetime.datetime.now().strftime('%m/%d %H')
    active_rate = conn.exists('device_active_rate')
    if active_rate:
        if conn.llen('device_active_rate') >= 7:
            key = conn.lpop('device_active_rate')
            conn.delete(key.decode('utf-8'))
    if today.encode('utf-8') not in conn.lrange('device_active_rate', 0, -1):
        conn.rpush('device_active_rate', today)

    active_rate_hour = conn.exists('device_active_rate_hour')
    if active_rate_hour:
        if conn.llen('device_active_rate_hour') >= 24:
            key = conn.lpop('device_active_rate_hour')
            conn.delete(key.decode('utf-8'))
    if hour.encode('utf-8') not in conn.lrange('device_active_rate_hour', 0, -1):
        conn.rpush('device_active_rate_hour', hour)
    # if not conn.exists('actives'):
    #     conn.hmset('actives', res)
    # else:
    for k, v in res.items():
        conn.hset('actives', k, v)
        if v == 1:
            conn.hset(today, k, 1)
            conn.hset(hour, k, 1)
        # actives = conn.hgetall('actives')

@app.task(queue='timing')
def get_device_active():
    logger.info('=== get_device_active ===')
    ping = Ping()
    # devices = Devices.objects.filter(is_domain=1).all().values('id', 'device_ip', 'device_hostname')
    devices = Devices.objects.exclude(is_domain=0).all().defer('device_root')
    device_count = devices.count()

    base_num = 100
    if not device_count:
        loop = 0
    elif device_count <= base_num:
        loop = 1
    elif device_count % base_num:
        loop = (device_count // base_num) + 1
    else:
        loop = (device_count // base_num)
    for i in range(loop):
        res = ping.run_async_ping_host(devices[i * base_num:(i + 1) * base_num])
        call_back(res)

@app.task(queue='timing')
def clear_device_active():
    logger.info('=== clear_device_active ===')
    if conn.exists('actives'):
        conn.delete('actives')



