from django.db import models


# 文件管理
class FileManager(models.Model):
    # 文件类型，0 脚本、1 系统补丁、2 文件、3软件、4证书
    file_type_choices = (
        (0, 'script'), (1, 'patch'), (2, 'file'), (3, 'software'), (4, "CA")
    )
    task = models.ForeignKey(to='tasks.Tasks', on_delete=models.PROTECT, db_constraint=False, null=True,
                             blank=True, verbose_name='任务')
    name = models.CharField(max_length=256, verbose_name='文件名称')
    software_id = models.CharField(max_length=256, verbose_name='id', null=True, blank=True)
    main_package = models.CharField(verbose_name='软件主包及版本号', max_length=255, null=False, blank=True)
    desc = models.CharField(max_length=256, verbose_name='文件描述', null=True, blank=True)
    version = models.CharField(verbose_name='版本', max_length=255, null=True, blank=True)
    size = models.CharField(max_length=100, verbose_name='文件大小')
    type = models.IntegerField(verbose_name='文件类型', choices=file_type_choices, default=0, blank=True)
    path = models.CharField(max_length=256, verbose_name='文件路径', null=True, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_update = models.DateTimeField(verbose_name='最后修改时间', auto_now=True, null=True, blank=True)

    class Meta:
        db_table = 'filemanager'
        verbose_name = '文件管理'
        verbose_name_plural = '文件管理'
        ordering = ('created_at',)

    def __str__(self):
        return "%s-%s" % (self.name, self.task.pk)

