from django.db import models


# Create your models here.

class Groups(models.Model):
    # 分组
    # 是否删除
    is_default = (
        (0, 'N'), (1, 'Y')
    )

    name = models.CharField(verbose_name='分组名称', max_length=128)
    order_id = models.IntegerField(verbose_name='排序ID', null=True, blank=True)
    remarks = models.CharField(verbose_name='备注', max_length=255, null=True, blank=True)
    create_user = models.ForeignKey(to='users.User', on_delete=models.SET(1), db_constraint=False, null=True,
                                    blank=True, verbose_name='创建人')
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_update = models.DateTimeField(verbose_name='最后修改时间', auto_now=True, null=True, blank=True)
    is_user = models.IntegerField(verbose_name='是否是用户组 0|终端组， 1|用户组', default=0, null=False,blank=True)
    is_deleted = models.IntegerField(verbose_name='是否删除', choices=is_default, default=0, null=False,
                                     blank=True)

    class Meta:
        db_table = 'groups'
        verbose_name = '分组'
        verbose_name_plural = '分组'
        ordering = ('-order_id',)

    def __str__(self):
        return self.name


class GroupsDetail(models.Model):
    # 分组详情
    # 范围，0 按部门、1按终端、2按分组 ,
    range_choices = (
        (0, 'organizations'), (1, 'devices'), (2, 'groups'), (3, 'account')
    )
    groups_id = models.IntegerField(verbose_name='分组id,groups', null=True, blank=True)
    range = models.IntegerField(verbose_name='范围，0 按部门、1按终端', choices=range_choices, default=0, blank=True)
    org_id = models.TextField(verbose_name='所属组织机构id', null=True, blank=True)
    device_id = models.TextField(verbose_name='设备id', null=True, blank=True)
    add_groups_id = models.IntegerField(verbose_name='添加分组id', null=True, blank=True)
    account_id = models.TextField(verbose_name="账号id", null=True, blank=True)

    class Meta:
        db_table = 'groups_detail'
        verbose_name = '分组详情'
        verbose_name_plural = '分组详情'
        ordering = ('-id',)

    def __str__(self):
        return str(self.id)
