from rest_framework import serializers
from groups.models import Groups, GroupsDetail
from organizations.models import OrgnizationEntity
from devices.models import Devices
from accounts.models import AccountEntity


class GroupsSerializer(serializers.ModelSerializer):
    org_num = serializers.SerializerMethodField(read_only=True)
    device_num = serializers.SerializerMethodField(read_only=True)
    account_num = serializers.SerializerMethodField()

    def get_device_num(self, obj):
        devices_history = GroupsDetail.objects.filter(groups_id=obj.id, range=1)
        device_num = len(devices_history)
        return device_num

    def get_org_num(self, obj):
        org_history = GroupsDetail.objects.filter(groups_id=obj.id, range=0)
        org_num = len(org_history)
        return org_num

    def get_account_num(self, obj):
        account_history = GroupsDetail.objects.filter(groups_id=obj.id, range=3)
        account_num = len(account_history)
        return account_num

    class Meta:
        model = Groups
        fields = ("id", "name", "order_id", "remarks", 'create_user', "is_user", "org_num", "device_num", "account_num")


class GroupsDetailSerializer(serializers.ModelSerializer):
    name = serializers.SerializerMethodField(read_only=True)
    org_name = serializers.SerializerMethodField(read_only=True)

    def get_name(self, obj):
        name = ""
        if obj.range == 0:
            org = OrgnizationEntity.objects.filter(id=obj.org_id).first()
            if org:
                name = org.name
        elif obj.range == 1:
            device = Devices.objects.filter(id=obj.device_id).first()
            if device:
                name = device.device_cnname
        elif obj.range == 2:
            if Groups.objects.filter(id=obj.add_groups_id).exists():
                name = Groups.objects.get(id=obj.add_groups_id).name
        elif obj.range == 3:
            if AccountEntity.objects.filter(id=obj.account_id).exists():
                name = AccountEntity.objects.get(id=obj.account_id).name
        else:
            pass
        return name

    def get_org_name(self, obj):
        org_name = ""
        if obj.range == 0:
            org_parent_id = OrgnizationEntity.objects.filter(id=obj.org_id).first().parent_id
            org = OrgnizationEntity.objects.filter(id=org_parent_id).first()
            if org:
                org_name = org.name
        elif obj.range == 1:
            device = Devices.objects.filter(id=obj.device_id).first()
            if device:
                org_id = device.org_info_id
                org = OrgnizationEntity.objects.filter(id=org_id).first()
                if org:
                    org_name = org.name
        elif obj.range == 3:
            account_items = AccountEntity.objects.filter(id=obj.account_id)
            if account_items.exists():
                org_name = account_items.first().org_info.name
        return org_name

    class Meta:
        model = GroupsDetail
        fields = ('id', 'groups_id', 'range', 'org_id', 'device_id', "name", "org_name", "add_groups_id", "account_id")
