from django.db import models

from nfs_domain_mainserver.settings import FILES_PATH


# Create your models here.
def job_directory_path(instance, filename):
    path = f"jobs_source/{filename}"
    return path

from django.core.files.storage import FileSystemStorage

class CustomStorage(FileSystemStorage):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # 将文件保存到指定路径
        self.location = FILES_PATH


class JobFile(models.Model):
    name = models.CharField('名称', max_length=300, null=True, blank=True)
    size = models.IntegerField('文件大小', default=1, null=True, blank=True)
    file = models.FileField('文件', storage=CustomStorage(), upload_to=job_directory_path)
    mime = models.CharField('文件格式', max_length=300, null=True, blank=True)
    path = models.CharField('地址', max_length=1000, null=True, blank=True)
    create = models.CharField('创建人', max_length=255, blank=True, null=True)
    create_time = models.DateTimeField(verbose_name='创建补丁时间', blank=True, null=True)

    class Meta:
        verbose_name = '任务文件'
        verbose_name_plural = verbose_name
        db_table = 'job_file'

    def __str__(self):
        return self.name


police_range_type_choices = (
    (0, '按部门'), (1, '按终端'), (2, '按 IP 网段'), (3, '按人员'), (4, '按组')
)


class JobInfo(models.Model):
    job_name = models.CharField(verbose_name='任务名称', max_length=255, blank=False, null=False)
    job_type = models.IntegerField(verbose_name='任务类型 1|软件 2|补丁，3|脚本，4|文件', default=3, blank=False)
    job_properties = models.IntegerField(verbose_name='任务属性 0|立即执行 1|定时执行，2|周期执行', default=0,
                                         blank=False)
    upload_files_ids = models.CharField(verbose_name='任务上传的文件id，1，2，3，4，5', max_length=255, blank=False,
                                        null=False)
    upload_files_ids_name = models.TextField(verbose_name='任务上传的文件名称 {}，{}，{}', null=True, blank=True)
    remark = models.TextField(verbose_name='任务备注信息', blank=True, null=True)
    is_notify_user = models.IntegerField(verbose_name='任务是否通知用户,0|不通知；1|通知', default=0, blank=False)
    is_user_cancel = models.IntegerField(verbose_name='任务是否允许用户取消,0|不允许；1|允许', default=0, blank=False)
    range_type = models.IntegerField(verbose_name='任务执行范围的类型', choices=police_range_type_choices, default=0,
                                     blank=False)
    range_value = models.TextField(verbose_name='任务执行范围的内容', blank=True, null=True)
    device_range_value = models.TextField(verbose_name='任务执行范围的内容', blank=True, null=True)
    exception_range_type = models.IntegerField(verbose_name='任务 例外执行范围的类型',
                                               choices=police_range_type_choices,
                                               default=1, blank=False)
    exception_range_value = models.TextField(verbose_name='任务 例外执行范围的内容', max_length=255, blank=True, null=True)
    job_is_delete = models.SmallIntegerField('任务是否删除  0|否；1|是', default=0)
    job_is_delivery = models.SmallIntegerField('任务是否是分发的  0|否；1|是', default=0)
    tar_name_path = models.CharField('压缩包路径', max_length=1000, null=True, blank=True)
    tar_name = models.CharField('压缩包名称', max_length=255, null=True, blank=True)
    jobs_status = models.IntegerField(verbose_name='任务状态 1|未开始 2|执行，3|完成，4|失败',default=0, blank=False)
    creator = models.CharField(verbose_name='任务信息创建人', max_length=64, blank=True, null=True)
    create_time = models.DateTimeField(verbose_name='创建时间', auto_now=True, blank=True, null=True)
    running_time = models.DateTimeField(verbose_name='执行时间', auto_now=True, blank=True, null=True)

    class Meta:
        db_table = 'job_info'
        verbose_name = '任务信息'
        verbose_name_plural = verbose_name
        ordering = ('-running_time',)

class JobResultInfo(models.Model):
    job_info_id = models.IntegerField(verbose_name='任务信息表id', blank=False, null=False)
    job_device_id = models.IntegerField(verbose_name='任务中终端id', blank=False, null=False)
    job_file_ids = models.CharField(verbose_name='任务中处理文件ids', max_length=255, blank=False, null=False)
    job_is_delivery = models.SmallIntegerField('任务是否是分发的  0|否；1|是', default=0)
    tar_name_path = models.CharField('压缩包路径', max_length=1000, null=True, blank=True)
    tar_name = models.CharField('压缩包名称', max_length=255, null=True, blank=True)
    is_send_status = models.SmallIntegerField(verbose_name='是否下发 0|没发送，1|已发送', default=0, blank=True, null=True)
    update_time = models.DateTimeField(verbose_name='任务指令下发时间', auto_now_add=True, blank=True, null=True)
    exe_status = models.IntegerField(verbose_name='任务中文件处理状态 1|未开始，2|执行中, 3|成功，4|失败,', blank=True, null=True)
    job_jid_id = models.CharField(verbose_name='任务中处理文件日志id', max_length=255, blank=True, null=True)

    class Meta:
        db_table = 'job_result_info'
        verbose_name = '任务详细结果信息表'
        verbose_name_plural = verbose_name