"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/9/4 上午11:07
@File: serializers.py
@Desc: 任务管理序列化输出
"""
import logging

from rest_framework import serializers
from devices.models import Devices
from jobs_management import models

logger = logging.getLogger(__name__)


# 类名定为表名称 + Serializer
class JobFileSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.JobFile
        fields = "__all__"


class JobInfoSerializer(serializers.ModelSerializer):
    jobs_status = serializers.SerializerMethodField()
    jobs_percentage = serializers.SerializerMethodField()
    due_date = serializers.SerializerMethodField()

    class Meta:
        model = models.JobInfo
        fields = ('id', 'job_name', 'job_type', 'job_properties', 'upload_files_ids', 'upload_files_ids_name', 'remark',
                  'is_notify_user', 'is_user_cancel', 'range_type', 'range_value', 'device_range_value',
                  'exception_range_type', 'exception_range_value', 'job_is_delete', 'tar_name_path', 'tar_name',
                  'jobs_status', 'jobs_percentage','job_is_delivery', 'due_date', 'creator', 'create_time', 'running_time')

    def get_jobs_status(self, obj):
        device_range_value = obj.device_range_value
        device_list = device_range_value.split(',') if device_range_value else []
        job_info_id = obj.id
        job_res_info_queryset = models.JobResultInfo.objects.filter(job_info_id=job_info_id,
                                                                    job_device_id__in=device_list)
        # 1|未开始，2|执行中, 3|成功，4|失败
        pending_num = job_res_info_queryset.filter(exe_status=1).count()
        running_num = job_res_info_queryset.filter(exe_status=2).count()
        success_num = job_res_info_queryset.filter(exe_status=3).count()
        failure_num = job_res_info_queryset.filter(exe_status=4).count()
        total_num = job_res_info_queryset.count()
        if total_num == success_num:
            return 3
        elif total_num == pending_num:
            return 1
        else:
            if running_num > 0:
                return 2
            elif failure_num > 0:
                return 4
            else:
                return 0

    def get_jobs_percentage(self, obj):
        device_range_value = obj.device_range_value
        device_list = device_range_value.split(',') if device_range_value else []
        job_info_id = obj.id
        job_res_info_queryset = models.JobResultInfo.objects.filter(job_info_id=job_info_id,
                                                                    job_device_id__in=device_list)
        total_num = job_res_info_queryset.count()
        success_num = job_res_info_queryset.filter(exe_status=3).count()
        failure_num = job_res_info_queryset.filter(exe_status=4).count()
        if total_num == 0:
            return f"0/0"
        else:
            return f'{success_num + failure_num}/{total_num}'

    def get_due_date(self, obj):
        device_range_value = obj.device_range_value
        device_list = device_range_value.split(',') if device_range_value else []
        job_info_id = obj.id
        job_res_info_queryset = models.JobResultInfo.objects.filter(job_info_id=job_info_id,
                                                                    job_device_id__in=device_list)
        success_num = job_res_info_queryset.filter(exe_status=3).count()
        failure_num = job_res_info_queryset.filter(exe_status=4).count()
        total_num = job_res_info_queryset.count()

        due_date = None
        if total_num == success_num:
            if total_num != 0:
                due_date = job_res_info_queryset.filter(exe_status=3).order_by('-update_time').first().update_time
                due_date = due_date.strftime('%Y-%m-%d %H:%M:%S') if due_date else None
        elif total_num == (success_num + failure_num):
            if total_num != 0:
                due_date = job_res_info_queryset.filter(exe_status__in=[3, 4]).order_by(
                    '-update_time').first().update_time
                due_date = due_date.strftime('%Y-%m-%d %H:%M:%S') if due_date else None
        else:
            pass

        return due_date


class JobDeviceInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = Devices
        fields = ('id', 'device_cnname', 'device_ip', 'org_info_id', 'is_up',)
