from django.db import models

class MenuInfo(models.Model):
    action = models.CharField(verbose_name='菜单行为(唯一)', max_length=128, unique=True, default='')
    module = models.CharField(verbose_name='菜单所属模块', max_length=32, default='')
    url_match = models.CharField(verbose_name='菜单地址正则表达式', max_length=256, null=True, blank=True)
    desc = models.CharField(verbose_name='菜单中文描述', max_length=128, null=True, blank=True)

    class Meta:
        db_table = 'menu_info'
        verbose_name = '菜单信息表'
        verbose_name_plural = '菜单信息表'

        # 自定义索引
        indexes = [
            models.Index(fields=['url_match'], name='idx_menu_info_url_match'),
        ]

    def __str__(self):
        return self.action


class MenuSubAction(models.Model):
    action = models.CharField(verbose_name='菜单行为', max_length=128, default='')
    sub_action = models.CharField(verbose_name='子行为', max_length=128, default='')
    sub_action_desc = models.CharField(verbose_name='子行为中文描述', max_length=128, null=True, blank=True)

    class Meta:
        db_table = 'menu_sub_action'
        verbose_name = '菜单行为的子行为表'
        verbose_name_plural = '菜单行为的子行为表'

        # 自定义索引
        indexes = [
            models.Index(fields=['action'], name='idx_menu_sub_action_action'),
            models.Index(fields=['sub_action'], name='idx_menu_sub_action_sub_action'),
        ]

    def __str__(self):
        return self.action
