"""
Django settings for nfs_domain_mainserver project.

Generated by 'django-admin startproject' using Django 1.11.28.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.11/ref/settings/
"""

import os
import re
import datetime
import errno
import socket
import yaml
from pathlib import Path
from dotenv import load_dotenv
from distutils.util import strtobool

# from django.db import close_old_connections
# close_old_connections()
# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# HOSTNAME = socket.gethostname()
# LOCALHOST = socket.gethostbyname(HOSTNAME)
# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.11/howto/deployment/checklist/


# SECURITY WARNING: don't run with debug turned on in production!

conf_dir = os.path.join(os.path.dirname(BASE_DIR), "conf")
docker_dir = os.path.join(os.path.dirname(BASE_DIR), "docker")
# env_path = os.path.join(os.path.dirname(BASE_DIR), ".env")
# if not env_path:
#     raise FileNotFoundError("ENV_PATH is not set")
# env_file = Path(env_path)
# if not env_file.exists():
#     raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), env_file)
# load_dotenv(env_path)

try:
    SECRET_KEY = os.environ["SECRET_KEY"]
except Exception:
    raise FileNotFoundError("SECRET_KEY is not set")

DJANGO_DEBUG = os.environ.get("DJANGO_DEBUG")
if DJANGO_DEBUG:
    try:
        # lower('y', 'yes', 't', 'true', 'on', '1')
        DJANGO_DEBUG = strtobool(DJANGO_DEBUG)
    # None, empty, bool...
    except (AttributeError, TypeError, ValueError):
        DJANGO_DEBUG = False
else:
    DJANGO_DEBUG = False
DEBUG = DJANGO_DEBUG

SALT_BASE_DIR = os.environ.get("SALT_BASE_DIR")
if not SALT_BASE_DIR:
    SALT_BASE_DIR = os.path.join(os.path.dirname(BASE_DIR), "docker/salt-docker")

SALT_URL = 'https://127.0.0.1:8000' if not os.environ.get("SALT_URL") else os.environ.get("SALT_URL")
SALT_AUTH = 'salt' if not os.environ.get("SALT_AUTH") else os.environ.get("SALT_AUTH")
SALT_PASSWD = '123456' if not os.environ.get("SALT_PASSWD") else os.environ.get("SALT_PASSWD")
DEFAULTPOLICY = os.path.join(conf_dir, 'default-policy.conf')
LICENSE = os.path.join(conf_dir, 'license.conf')
LICENSE_B_CHECK_LIBRARY = os.path.join(conf_dir, "libreg-check-b.so")
SALT_CONF = SALT_BASE_DIR
# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'corsheaders',
    'rest_framework',
    'channels',
    'django_filters',
    'django_celery_beat',

    # app
    'index',
    'saltapi',
    'devices',
    'users',
    'accounts',
    'organizations',
    'software',
    'system',
    'camanager',
    'jobs_management',
    'filemanager',
    'remote_assist',
    'groups',
    'message_center',
    'browser',
    'tasks',
    'patch_management',
    'logs',
    'reports',
    'softwarecenter',
    'menumanager',
    'policy',
    'adminlog',
    'clientlog',
    'sysios',
    'chunked_upload',
    'fileupload',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    # 'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'utils.middleware.ApiLoggingMiddleware',
]

CORS_ALLOW_CREDENTIALS = True
CORS_ORIGIN_ALLOW_ALL = True
CORS_ORIGIN_WHITELIST = (
    'https://*.*.*.*:*',
)

# ALLOWED_HOSTS = os.environ.get("ALLOWED_HOSTS", "127.0.0.1").split(" ")
ALLOWED_HOSTS = ["*", ]

ROOT_URLCONF = 'nfs_domain_mainserver.urls'

FRONTEND_DIR = os.path.join(os.path.dirname(BASE_DIR), "frontend")
if not os.path.exists(FRONTEND_DIR):
    os.makedirs(FRONTEND_DIR)

FRONT_ROOT = os.path.join(FRONTEND_DIR, 'nfs-domain-mainweb/dist')

SILENCED_SYSTEM_CHECKS = ['urls.W002', 'security.W019']

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            FRONT_ROOT
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'nfs_domain_mainserver.wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases


if os.environ["DB_BACKEND"] == 'sqlite':
    DB_PATH = os.path.join(os.path.dirname(BASE_DIR), 'db')
    if not os.path.exists(DB_PATH):
        os.makedirs(DB_PATH)
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.path.join(DB_PATH, 'mainDB.sqlite3'),
        }
    }
else:
    DATABASES = {
        "default": {
            'ENGINE': 'dj_db_conn_pool.backends.mysql',
            "NAME": os.environ.get("DB_NAME"),
            "USER": os.environ.get("DB_USER"),
            "PASSWORD": os.environ.get("DB_PASS"),
            "HOST": os.environ.get("DB_HOST"),
            "PORT": int(os.environ.get("DB_PORT")),
            'POOL_OPTIONS': {
                'POOL_SIZE': 250,
                'MAX_OVERFLOW': 20,
                'RECYCLE': 3 * 60 * 60,
                'TIMEOUT': 10 * 60,
                'PRE_PING': True,
                'ECHO': False
            }
        }
    }
# os.environ['DJANGO_ALLOW_ASYNC_UNSAFE'] = 'true'

# Password validation
# https://docs.djangoproject.com/en/1.11/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

CACHE_DIR = "/tmp"

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.filebased.FileBasedCache',
        'LOCATION': os.path.join(CACHE_DIR, 'nfs_domain_cache'),
        'OPTIONS': {
            'MAX_ENTRIES': 1000000
        }
    },
    'celery_broker': {
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://{}:{}/1".format(os.environ.get("REDIS_HOST"), os.environ.get('REDIS_PORT')),
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    },
    'celery_backend': {
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://{}:{}/2".format(os.environ.get("REDIS_HOST"), os.environ.get('REDIS_PORT')),
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    },
    'redis': {
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://{}:{}/3".format(os.environ.get("REDIS_HOST"), os.environ.get('REDIS_PORT')),
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    }
}

REST_FRAMEWORK = {
    # 异常配置
    # 'EXCEPTION_HANDLER': 'utils.exception.exception_handler',

    # 全局解析配置, 仅解析json
    'DEFAULT_PARSER_CLASSES': [
        'rest_framework.parsers.JSONParser',
        'rest_framework.parsers.FormParser',
        'rest_framework.parsers.MultiPartParser'
        # 'rest_framework.parsers.FileUploadParser'
    ],
    # 全局过滤筛选条件
    'DEFAULT_FILTER_BACKENDS': [
        'django_filters.rest_framework.DjangoFilterBackend',
        'rest_framework.filters.SearchFilter',
        'rest_framework.filters.OrderingFilter',
    ],

    # 频率配置
    'DEFAULT_THROTTLE_RATES': {
        # 'ping': '3000/minute',
        'anon': '60/minute',
        'user': '3000/minute',
    },
    # 全局序列化时间格式返回样式
    'DATETIME_FORMAT': '%Y-%m-%d %H:%M:%S',
    'DATE_FORMAT': '%Y-%m-%d',
}

# 自定义 drf-jwt 配置
JWT_AUTH = {
    # user => payload
    'JWT_PAYLOAD_HANDLER':
        'rest_framework_jwt.utils.jwt_payload_handler',
    # payload => token
    'JWT_ENCODE_HANDLER':
        'rest_framework_jwt.utils.jwt_encode_handler',
    # token => payload
    'JWT_DECODE_HANDLER':
        'rest_framework_jwt.utils.jwt_decode_handler',
    # token过期时间
    'JWT_EXPIRATION_DELTA': datetime.timedelta(days=1),
    # token刷新的过期时间
    'JWT_REFRESH_EXPIRATION_DELTA': datetime.timedelta(minutes=15),
    'JWT_AUTH_HEADER_PREFIX': 'nfs-',  # 默认token前要加上nfs-
}

# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

LANGUAGE_CODE = 'zh-hans'

TIME_ZONE = 'Asia/Shanghai'

USE_I18N = True

USE_L10N = True

USE_TZ = False

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.11/howto/static-files/

STATIC_URL = '/static/'
STATIC_ROOT = '/static/'
STATICFILES_DIRS = (
    os.path.join(BASE_DIR, 'static'),
)

MEDIA_URL = '/media/'
MEDIA_ROOT = os.path.join(BASE_DIR, 'media')

# APPEND_SLASH = True  # url 严格模式

# logging config
if os.environ.get('LOG_PATH', None):
    LOG_PATH_FILE = os.path.join(os.environ["LOG_PATH"], "domain.log")
    LOG_PATH_FILE_ERROR = os.path.join(os.environ["LOG_PATH"], "domain-error.log")
    CELERY_WORK_LOG_FILE = os.path.join(os.environ["LOG_PATH"], 'celery_work.log')  # 配置Celery执行日志
    CELERY_BEAT_LOG_FILE = os.path.join(os.environ["LOG_PATH"], 'celery_beat.log')  # 配置Celery定时任务日志
else:
    LOG_DIR_PATH = os.path.join(os.path.dirname(BASE_DIR), "logs", 'web')
    if not os.path.exists(LOG_DIR_PATH):
        os.makedirs(LOG_DIR_PATH)
    LOG_PATH_FILE = os.path.join(LOG_DIR_PATH, "domain.log")
    LOG_PATH_FILE_ERROR = os.path.join(LOG_DIR_PATH, "domain-error.log")
    CELERY_WORK_LOG_FILE = os.path.join(LOG_DIR_PATH, 'celery_work.log')  # 配置Celery执行日志
    CELERY_BEAT_LOG_FILE = os.path.join(LOG_DIR_PATH, 'celery_beat.log')  # 配置Celery定时任务日志

# 创建日志目录（如果不存在）
os.makedirs(os.path.dirname(LOG_PATH_FILE), exist_ok=True)
os.makedirs(os.path.dirname(LOG_PATH_FILE_ERROR), exist_ok=True)
os.makedirs(os.path.dirname(CELERY_WORK_LOG_FILE), exist_ok=True)
os.makedirs(os.path.dirname(CELERY_BEAT_LOG_FILE), exist_ok=True)

# login config
LOGIN_ATTEMPT_LIMIT = 4
LOGIN_ATTEMPT_TIMEOUT = 15 * 60  # in seconds (default: 15 minutes)

# user role permission
NFS_ROOT = 'root@admin.com'
NFS_SECADM = 'secadm@admin.com'
NFS_AUDITADM = 'auditadm@admin.com'

DEFAULT_USER_PASSWORD = 123456


class Colors:
    PURPLE = '\033[95m'
    CYAN = '\033[96m'
    DARKCYAN = '\033[36m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    RED = '\033[91m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    RESET = '\033[0m'


from kombu import Exchange, Queue

# celery conf
CELERY_TIMEZONE = TIME_ZONE
CELERY_ENABLE_UTC = False  # 关闭时区
CELERY_ACKS_LATE = True  # 允许重试
CELERYD_FORCE_EXECV = True  # 可以让Celery更加可靠,只有当worker执行完任务后,才会告诉redis,消息被消费，防治死锁
CELERY_DISABLE_RATE_LIMITS = True  # 任务发出后，经过一段时间还未收到acknowledge , 就将任务重新交给其他worker执行
CELERYD_MAX_TASKS_PER_CHILD = 1  # 每个worker最多执行1个任务就会被销毁，可防止内存泄露

# celery beat
DJANGO_CELERY_BEAT_TZ_AWARE = False
CELERY_BEAT_SCHEDULER = 'django_celery_beat.schedulers:DatabaseScheduler'

# 设置队列优先级
CELERYD_PREFETCH_MULTIPLIER = 1
CELERY_QUEUES = (
    Queue("default", Exchange("default"), routing_key="default"),
    Queue("timing", Exchange("timing"), routing_key="timing"),
    Queue("adminlog", Exchange("adminlog"), routing_key="adminlog"),
    Queue("freeipa", Exchange("freeipa"), routing_key="freeipa", queue_arguments={'x-max-priority': 9})
    # 队列优先级数值queue_arguments最高为9
)
# 设置celery默认的队列
CELERY_DEFAULT_QUEUE = 'default'
CELERY_DEFAULT_EXCHANGE = 'default'
CELERY_DEFAULT_ROUTING_KEY = 'default'

SALTSTARTEGYPATH = f"{SALT_BASE_DIR}srv/salt/base"
SALTFILESYSTEMPATH = f"{SALT_BASE_DIR}srv/salt/file_system"
WALLPAPERPATH = f"{SALT_BASE_DIR}srv/salt/file_system/wallpaper"
WALLPAPER_URL = 'mainapi/wallpaper/'
WALLPAPER_ROOT = WALLPAPERPATH
SALTFILES_ROOT = SALTFILESYSTEMPATH
WALLPAPER_TYPE = ['.jpg', '.jpeg', '.png']
if not os.path.exists(WALLPAPERPATH):
    os.makedirs(WALLPAPERPATH)
CERT_FILES_PATH = f"{SALT_BASE_DIR}srv/salt/file_system/certfiles"
if not os.path.exists(CERT_FILES_PATH):
    os.makedirs(CERT_FILES_PATH)
# if os.environ.get('DEPLOY_ARCH') == 'cluster':
#     #  cluster模式下 salt内策略等文件初始配置再由web服务控制
#     pass
# else:
#     with open('{0}/kylin/freeipa_client/conf/dns.conf'.format(SALTFILESYSTEMPATH), 'wt', encoding='utf-8') as f:
#         f.write('{DOMAIN_IP} {DOMAIN_SERVER_FQDN}'.format(DOMAIN_IP=os.environ.get("DOMAIN_IP"),
#                                                           DOMAIN_SERVER_FQDN=os.environ.get("DOMAIN_SERVER_FQDN")))
#     with open('{0}/uos/freeipa_client/conf/dns.conf'.format(SALTFILESYSTEMPATH), 'wt', encoding='utf-8') as f:
#         f.write('{DOMAIN_IP} {DOMAIN_SERVER_FQDN}'.format(DOMAIN_IP=os.environ.get("DOMAIN_IP"),
#                                                           DOMAIN_SERVER_FQDN=os.environ.get("DOMAIN_SERVER_FQDN")))
#     with open('{0}/nfs/freeipa_client/conf/dns.conf'.format(SALTFILESYSTEMPATH), 'wt', encoding='utf-8') as f:
#         f.write('{DOMAIN_IP} {DOMAIN_SERVER_FQDN}'.format(DOMAIN_IP=os.environ.get("DOMAIN_IP"),
#                                                           DOMAIN_SERVER_FQDN=os.environ.get("DOMAIN_SERVER_FQDN")))
#     with open('{0}/nfs-4/freeipa_client/conf/dns.conf'.format(SALTFILESYSTEMPATH), 'wt', encoding='utf-8') as f:
#         f.write('{DOMAIN_IP} {DOMAIN_SERVER_FQDN}'.format(DOMAIN_IP=os.environ.get("DOMAIN_IP"),
#                                                           DOMAIN_SERVER_FQDN=os.environ.get("DOMAIN_SERVER_FQDN")))
#
#     with open('{0}/pre_add_domain.sh'.format(SALTFILESYSTEMPATH), 'r', encoding='utf-8') as f:
#         content = f.read()
#     str_replace = False
#     result = re.search('SERVER_IP=\$1', content)
#     if result:
#         str_replace = content.replace('SERVER_IP=$1', 'SERVER_IP={}'.format(os.environ.get("DOMAIN_IP")), 1)
#     result = re.search('DOMAIN_NAME=\$2', content)
#     if result:
#         str_replace = str_replace.replace('DOMAIN_NAME=$2', 'DOMAIN_NAME={}'.format(os.environ.get("DOMAIN_SERVER_FQDN")),
#                                           1)
#     if str_replace:
#         with open('{0}/pre_add_domain.sh'.format(SALTFILESYSTEMPATH), 'w', encoding='utf-8') as f:
#             f.write(str_replace)

# WEBSOCKET_ACCEPT_ALL = True
ASGI_APPLICATION = 'nfs_domain_mainserver.routing.application'

CHANNEL_LAYERS = {
    "default": {
        "BACKEND": "channels_redis.core.RedisChannelLayer",  # 使用 asgi_redis 作为 backend
        # "BACKEND": "channels_redis.pubsub.RedisPubSubChannelLayer",  # on_disconnect
        # "BACKEND": "channels.layers.InMemoryChannelLayer",

        "CONFIG": {
            "hosts": [(os.environ.get("REDIS_HOST"), 6379)],  # 设置 Redis 的地址和端口
            # 'on_disconnect':'redis.disconnect',
            'capacity': 1500,
            # 'expiry': 10,
        },

    },
}

# FreeIPA Env
DOMAIN_SERVER_FQDN = os.environ.get("DOMAIN_SERVER_FQDN")
DOMAIN_SERVER_ADMIN = os.environ.get("DOMAIN_SERVER_ADMIN")
DOMAIN_SERVER_ADMIN_PASSWD = os.environ.get("DOMAIN_SERVER_ADMIN_PASSWD")
DOMAIN_SERVER_RELM = os.environ.get("DOMAIN_SERVER_RELM")

# Reprepro Env
REPREPRO_CONFIG_FOLDER = os.environ.get("REPREPRO_CONFIG_FOLDER")
REPREPRO_WEBSERVER_PORT = os.environ.get("REPREPRO_WEBSERVER_PORT")
REPREPRO_SSH_PORT = os.environ.get("REPREPRO_SSH_PORT")
KEY_REAL_NAME = os.environ.get("KEY_REAL_NAME")
KEY_COMMENT = os.environ.get("KEY_COMMENT")
KEY_EMAIL = os.environ.get("KEY_EMAIL")
HOSTNAME = os.environ.get("DOMAIN_IP")
DOMAIN_NAME = os.environ.get("DOMAIN_NAME")
PROJECT_NAME = os.environ.get("PROJECT_NAME")
CODE_NAME = os.environ.get("CODE_NAME")

DEVICE_POLICE = [
    'install_freeipa', 'delete_freeipa',
    'firewalld_policy', 'firewalld_level_policy','ssh_policy',
    'pkg_install_policy', 'pkg_install_batch_policy', 'pkg_remove_policy', 'pkg_install_default_policy',
    'browser_default_policy',
]
JOB_POLICE = ['job_file_policy','job_software_policy','job_bash_policy']
USER_POLICY = ['wallpaper_policy', 'file_policy', 'shell_file_policy', 'system_patches_file_policy']

FILE_DIR = os.path.join(os.path.dirname(BASE_DIR), 'file_data')
if not os.path.exists(FILE_DIR):
    os.makedirs(FILE_DIR)

LDAP_SSL_FILE_DIR = os.path.join(FILE_DIR, 'ldap_ssl')
if not os.path.exists(LDAP_SSL_FILE_DIR):
    os.makedirs(LDAP_SSL_FILE_DIR)

# 新建目录，将需要下发的文件上传至此目录，通过docker映射，saltstack进行映射（兼容sls文件中的source）
FILES_PATH = f"{SALT_BASE_DIR}srv/salt/file_system/file_data"
if not os.path.exists(FILES_PATH):
    os.makedirs(FILES_PATH)

DEB_SIGN_KEYID = 'NDNCMzExNTA0NUVDOThCRTdBQzNCNjk3MERBM0E0NzRDNzkzOTI3Ng=='

DEB_FOLDER = '/usr/share/debs/'

REMOET_ASSIST_KEY = "remote_assist"
# 浏览器策率配置文件存放目录
BROWSER_POLICY_FILE_PATH = f"{SALT_BASE_DIR}srv/salt/file_system/browser_data"
SALT_FILE_PATH = f"{SALT_BASE_DIR}srv/salt/file_system"
if not os.path.exists(BROWSER_POLICY_FILE_PATH):
    os.makedirs(BROWSER_POLICY_FILE_PATH)

SM4_KEY = '11HDESaAhiHHugD2'

"""日志文件设置"""
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'standard': {
            'format': '[%(asctime)s] [%(levelname)s] [%(name)s %(funcName)s] [%(filename)s:%(lineno)s] %(message)s '
        },
        'simple': {  # 简单格式
            'format': '%(asctime)s [%(levelname)s] %(name)s %(funcName)s:%(lineno)s %(message)s'
        },
        'verbose': {
            'format': '%(asctime)s %(levelname)s %(message)s'
        },
        'color': {
            'format': Colors.RED + '[%(asctime)s] [%(levelname)s] [%(name)s] [%(filename)s:%(lineno)d] %(message)s' + Colors.RESET
        },
    },
    'handlers': {
        'console': {
            'level': 'INFO',
            'class': 'logging.StreamHandler',
            'formatter': 'standard',
        },
        'default': {
            'level': 'INFO',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': LOG_PATH_FILE,
            'maxBytes': 10485760,  # 单个日志文件的最大大小，这里是10MB
            'backupCount': 10,
            'formatter': 'standard',
        },
        'error': {
            'level': 'INFO',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': LOG_PATH_FILE_ERROR,
            'maxBytes': 10485760,  # 单个日志文件的最大大小，这里是10MB
            'backupCount': 10,
            'formatter': 'standard',
        },
        'celery_work_file': {
            'level': 'INFO',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': CELERY_WORK_LOG_FILE,
            'maxBytes': 10485760,  # 单个日志文件的最大大小，这里是10MB
            'backupCount': 10,
            'formatter': 'standard',
        },
        'celery_beat_file': {
            'level': 'INFO',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': CELERY_BEAT_LOG_FILE,
            'maxBytes': 10485760,  # 单个日志文件的最大大小，这里是10MB
            'backupCount': 10,
            'formatter': 'standard',
        },
    },
    'loggers': {
        '': {
            'handlers': ['default'],
            'level': 'INFO',
            'propagate': False
        },
        'err': {
            'handlers': ['error'],
            'level': 'INFO',
            'propagate': False
        },
        'django.request': {
            'handlers': ['default'],
            'level': 'INFO',
            'propagate': False
        },
        'celery.worker': {
            'handlers': ['celery_work_file'],
            'level': 'INFO',
            'propagate': False
        },
        'celery.beat': {
            'handlers': ['celery_beat_file'],
            'level': 'INFO',
            'propagate': True
        },
    }
}
