from django.db import models
from accounts.models import Ldap


# from accounts.models import AccountEntity

# Create your models here.
class OrgnizationEntity(models.Model):
    switch_status = (
        (0, 'NO'), (1, 'YES')
    )
    create_type_choices = (
        (0, '自建'), (1, 'ldap'), (2, 'soap_import')
    )
    name = models.CharField(max_length=128, verbose_name='名称')
    parent = models.ForeignKey('self', on_delete=models.CASCADE, db_constraint=False, null=False, blank=True, default=1,
                               verbose_name='父部门')
    address = models.CharField(verbose_name='所在地址', max_length=255, null=True, blank=True)
    contact_phone = models.CharField(verbose_name='联系电话', max_length=32, null=True, blank=True)
    executive = models.ForeignKey(to='accounts.AccountEntity', on_delete=models.PROTECT, db_constraint=False, null=True,
                                  blank=True, verbose_name='主管人员')
    remarks = models.CharField(verbose_name='备注', max_length=255, null=True, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_update = models.DateTimeField(verbose_name='最后修改时间', auto_now=True, null=True, blank=True)
    seq_num = models.IntegerField(verbose_name='部门排序标志', blank=True, null=True)
    passwd_extra_status = models.SmallIntegerField('密码有效期校验例外状态', choices=switch_status, blank=True,
                                                   null=True)
    create_type = models.IntegerField(verbose_name='创建方式', choices=create_type_choices, blank=True, default=0)

    class Meta:
        db_table = 'org_info'
        verbose_name = '组织机构'
        verbose_name_plural = '组织机构'
        ordering = ('created_at',)

    def __str__(self):
        return self.name


class LdapOrgnization(models.Model):
    name = models.CharField(verbose_name='组名', max_length=255, null=True, blank=True)
    dn = models.TextField(verbose_name='DN', null=True, blank=True)
    guid = models.TextField(verbose_name='群组唯一标识ID', null=True, blank=True)
    parent_dn = models.TextField(verbose_name='父级DN', null=True, blank=True)
    ldap = models.ForeignKey(verbose_name='ldap_id', to=Ldap, blank=True, on_delete=models.CASCADE)
    org_id = models.IntegerField(verbose_name='组', null=True)

    class Meta:
        db_table = 'ldaporg_info'
        verbose_name = 'ldap组织机构'
        verbose_name_plural = 'ldap组织机构'

    def __str__(self):
        return self.name


class OrgLabel(models.Model):
    org_id = models.IntegerField(verbose_name='部门id', null=True, blank=True)
    org_name = models.CharField(verbose_name='部门名称', max_length=255, blank=True, null=True)
    org_label = models.CharField(verbose_name='部门标签（缩写）', max_length=255, blank=True, null=True)

    class Meta:
        db_table = 'org_label'
