import datetime
import threading

import openpyxl
import pandas as pd
from django.db import close_old_connections
from django.db.models import Max

# Create your views here.
import logging
from django.conf import settings
from django_redis import get_redis_connection
from openpyxl.styles import Font
from rest_framework.decorators import action
from rest_framework.views import APIView
from rest_framework.parsers import JSONParser  # 解析json数据包
from rest_framework.parsers import FormParser  # 解析urlencoded数据包
from rest_framework.parsers import MultiPartParser  # 解析from-data数据包
from rest_framework.renderers import JSONRenderer  # 渲染模块,仅渲染json格式
from rest_framework.viewsets import ModelViewSet, GenericViewSet
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.decorators import api_view, permission_classes, authentication_classes

from devices.models import Devices
from devices.web_consumers import send_msg_to_web
from message_center.models import MessageCenter
from nfs_domain_mainserver.settings import SALTFILESYSTEMPATH
from organizations.serializers import OrganSerializer, OrgLabelSerializer
from users.models import User
from users.role_permisson import get_org_permission_list
from utils.authentication import TokenAuthentication
from utils.pagination import MyPagination
from utils.response import FitJSONRenderer
from utils.throttling import AnonRateThrottle, UserRateThrottle
from rest_framework.permissions import IsAuthenticated, IsAdminUser

from rest_framework.views import Response
from rest_framework import status, mixins
from rest_framework.pagination import PageNumberPagination
from organizations.models import OrgnizationEntity, LdapOrgnization, OrgLabel
from accounts.models import AccountEntity
from utils.utils import str_to_list

# from auditlog.utils import system_log, add_audit_log

logger = logging.getLogger(__name__)

# 部门根节点ID 
DEPT_ROOT_ID = 1


# 获取部门下所有终端用户
def get_sub_account(org_info_id):
    accounts = AccountEntity.objects.all()
    accounts = accounts.filter(org_info__id=org_info_id)
    return accounts


def get_device_num(org_id):
    deivce_num = Devices.objects.filter(org_info=org_id).exclude(is_domain=0).count()
    return deivce_num


def get_org_num(org_id):
    org_num = OrgnizationEntity.objects.filter(parent_id=org_id).count()
    # 顶层部门 parent_id 是1
    if org_id == 1:
        org_num = org_num - 1
    return org_num


def get_sub_account_num(org_id):
    accounts = get_sub_account(org_id)
    sub_account_num = accounts.count()
    return sub_account_num


# 构造树 终端用户
def get_accounts_tree(accounts):
    res_list = []
    for account in accounts:
        res_list.append({
            'pid': account.org_info_id,
            "value": account.id,
            "title": account.cn_name,
            "cn_name": account.cn_name,
            "name": account.name,
            "type": "account"
        })
    return res_list


# 获取PID下一级信息
def get_subs(pid, orgs):
    subs = []
    for org in orgs.order_by('seq_num'):
        if org.parent_id == int(pid):
            subs.append({
                'pid': pid,
                "value": org.id,
                "title": org.name,
                "create_type": org.create_type,
                "seq_num": org.seq_num,
                "children": []
            })

    return subs


# 获取PID下的所有数据 树形结构
def get_subs_tree(pid, orgs):
    p_org = OrgnizationEntity.objects.get(pk=pid)

    subs = get_subs(pid, orgs)
    if len(subs) == 0:
        return subs
    else:
        for ss in subs:
            ss_subs = get_subs_tree(ss['value'], orgs)
            ss['children'] = ss_subs
            if ss_subs is None:
                ss['children'] = []
            accounts = get_accounts_tree(get_sub_account(ss['value']))
            for account in accounts:
                ss['children'].append(account)
    return subs


# 获取PID下的所有数据子部门列表
def get_subs_orgs(pid):
    orgs = OrgnizationEntity.objects.all()
    res_list = []
    subs = get_subs(pid, orgs)

    if len(subs) == 0:
        return res_list
    else:
        for ss in subs:
            res_list.append(ss)
            ss_subs = get_subs_orgs(ss['value'])
            for s in ss_subs:
                res_list.append(s)
    return res_list


# 获取所有上级
def get_all_parent(p_id):
    res_list = []
    org_info = OrgnizationEntity.objects.get(pk=p_id)
    res_list.append({
        "id": org_info.id,
        "name": org_info.name
    })
    if org_info.id == DEPT_ROOT_ID:
        return res_list
    else:
        res_list.extend(get_all_parent(org_info.parent_id))
    return res_list


# 判断r_pid是否是oid的下级
def pidIsInSubs(oid, r_pid):
    inSub = False
    sub_orgs = get_subs_orgs(oid)

    for org in sub_orgs:
        if int(r_pid) == int(org['value']):
            inSub = True
            break

    return inSub


class OrganizationTreeListWithAccount(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    # 获取系统内所有部门 人员 树形数据
    def get(self, request, *args, **kwargs):
        root_id = DEPT_ROOT_ID
        orgs = OrgnizationEntity.objects.all()
        orgs = orgs.filter(id__gt=root_id)
        tops = get_subs_tree(DEPT_ROOT_ID, orgs)
        root_org = OrgnizationEntity.objects.get(pk=root_id)
        res_tree = {
            'pid': DEPT_ROOT_ID,
            "value": root_org.id,
            "title": root_org.name,
            "seq_num": root_org.seq_num,
            "create_type": root_org.create_type,
            "children": tops
        }
        accounts = get_accounts_tree(get_sub_account(root_id))
        for account in accounts:
            res_tree['children'].append(account)
        res_list = []
        res_list.append(res_tree)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '获取部门树形结构列表+终端人员信息完成',
            'success': True,
            'data': res_list
        })


class OrganizationTreeList(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    # 获取系统内所有部门数据
    def get(self, request, *args, **kwargs):
        orgs = OrgnizationEntity.objects.all()

        res_list = []
        for org in orgs:
            # 特殊处理 根节点
            if org.id == org.parent_id:
                org.parent_id = 0
            res_list.append({
                'id': org.id,
                'value': org.id,
                'title': org.name,
                'create_type': org.create_type,
                'pId': org.parent_id,
            })
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '获取部门树形结构列表完成',
            'total': len(orgs),
            'success': True,
            'data': res_list
        })


class OrganizationList(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    @classmethod
    def organ_down_list(cls, num):
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return [int(num)]
        else:
            organ_down_list = organ_down_list + list(items.values_list('id', flat=True))
            while True:
                # logger.info(f'**********{items}*********')
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('id', flat=True))

            organ_down_list.append(int(num))
        return organ_down_list

    # 部门信息 分页查询 条件查询
    def get(self, request, *args, **kwargs):
        orgs = OrgnizationEntity.objects.all()
        user = User.objects.filter(username=request.user.username).first()
        org_list = get_org_permission_list(user)

        # 根据部门筛选数据
        if org_list:
            orgs = orgs.filter(id__in=org_list)

        query_org_id = request.query_params.get('query_org_id')
        if query_org_id:
            query_org_ids = query_org_id.split(",")
            while "" in query_org_ids:
                query_org_ids.remove("")
            orgs = orgs.filter(id__in=query_org_ids)
        orgs = orgs.order_by('-id', 'parent_id')
        page = PageNumberPagination()  # 产生一个分页器对象
        page.page_size = 10  # 默认每页显示的多少条记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'  # 前台控制每页显示的最大条数
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(orgs, request)
        except Exception as e:
            logger.info("query org list except:", e)
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': 'error:',
                'success': False,
                'data': []
            })
        res_list = []
        for org in ret:
            accounts = get_sub_account(org.id)
            sub_account_num = accounts.count()
            if org.executive_id:
                executive_id = org.executive_id
                executive_name = org.executive.cn_name
            else:
                executive_id = ""
                executive_name = ""
            all_parent = get_all_parent(org.parent.id)
            org_id_list = self.organ_down_list(org.id)
            device_num = Devices.objects.filter(org_info_id__in=org_id_list).exclude(is_domain=0).count()
            org_num = get_org_num(org.id)
            res_list.append({
                'oid': org.id,
                'name': org.name,
                'parent_id': org.parent.id,
                'parent_name': org.parent.name,
                'address': org.address,
                'executive_id': executive_id,
                'executive_cn_name': executive_name,
                'contact_phone': org.contact_phone,
                'last_update': org.last_update,
                'remarks': org.remarks,
                'accounts_num': sub_account_num,
                "all_parent": all_parent,
                "device_num": device_num,
                "org_num": org_num,
            })
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '获取分页列表完成',
            'total': len(orgs),
            'success': True,
            'data': res_list
        })

    # 新增部门
    def post(self, request, *args, **kwargs):

        req_data = request.data
        org = OrgnizationEntity()

        org.parent_id = req_data.get('parent_id')
        org.name = req_data.get('name')
        name_org = OrgnizationEntity.objects.filter(name=req_data.get('name')).first()
        if OrgnizationEntity.objects.filter(name=req_data.get('name'), parent_id=req_data.get('parent_id')).exists():
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': req_data.get('name') + "已存在"
            })
        seq_num__max = OrgnizationEntity.objects.filter(parent_id=req_data.get('parent_id')).aggregate(Max('seq_num'))[
            'seq_num__max']
        org.contact_phone = req_data.get('contact_phone')
        org.executive_id = req_data.get('executive_id')
        org.address = req_data.get('address')
        org.remarks = req_data.get('remarks')
        org.seq_num = (1 + seq_num__max) if seq_num__max is not None else 0
        # 判断部门信息
        # check_org_set = OrgnizationEntity.objects.filter(name=req_data['org_info_id'])
        try:
            org.save()
        except Exception as e:
            logger.info(str(e))
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': str(e)
            })
        # 主管人员 部门变更
        executive_id = org.executive_id
        if executive_id:
            executive_account = AccountEntity.objects.get(pk=executive_id)
            executive_account.org_info_id = org.id
            executive_account.save()
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '新增部门成功'
        })


class OrganizationDetail(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    # 获取部门信息详细内容
    def get(self, request, *args, **kwargs):
        oid = kwargs.get('oid')
        org = OrgnizationEntity.objects.get(pk=oid)
        # ser_account = AccountEntitySerializer(account, many=True)
        if org:
            accounts = get_sub_account(org.id)
            sub_account_num = accounts.count()
            executive_id = ""
            executive_cn_name = ""
            if org.executive:
                executive_id = org.executive.id
                executive_cn_name = org.executive.cn_name
            subs = OrgnizationEntity.objects.filter(parent_id=oid, id__gt=DEPT_ROOT_ID)
            subs_list = []
            for subOrg in subs:
                subs_list.append({
                    'id': subOrg.id,
                    'name': subOrg.name
                })
            device_num = get_device_num(org.id)
            org_num = get_org_num(org.id)
            res_obj = {
                'oid': org.id,
                'name': org.name,
                'parent_id': org.parent.id,
                'parent_name': org.parent.name,
                'address': org.address,
                'executive_id': executive_id,
                'executive_cn_name': executive_cn_name,
                'contact_phone': org.contact_phone,
                'last_update': org.last_update,
                'remarks': org.remarks,
                'accounts_num': sub_account_num,
                'subs': subs_list,
                "device_num": device_num,
                "org_num": org_num
            }
            return Response({
                'code': status.HTTP_200_OK,
                'msg': '获取部门信息成功',
                'data': res_obj
            })
        else:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '获取部门信息发生错误',
                'data': {}
            })

    # 修改部门信息
    def put(self, request, *args, **kwargs):
        oid = kwargs.get('oid')

        req_data = request.data

        org = OrgnizationEntity.objects.get(pk=oid)
        if req_data.get('parent_id'):
            r_pid = req_data.get('parent_id')
            if int(oid) == DEPT_ROOT_ID and int(r_pid) > DEPT_ROOT_ID:
                return Response({
                    'code': 501,
                    'msg': "不能修改根部门的上级部门"
                })
            if int(oid) == int(r_pid):
                return Response({
                    'code': 502,
                    'msg': "上级部门不能是自己"
                })
            if pidIsInSubs(oid, r_pid):
                return Response({
                    'code': 502,
                    'msg': "上级部门不能是自己的下级部门"
                })
            org.parent_id = req_data.get('parent_id')

        try:
            parent = OrgnizationEntity.objects.get(pk=org.parent_id)
        except Exception as e:
            logger.info(str(e))
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': "上级节点不存在"
            })
        if req_data.get('name'):
            name_org = OrgnizationEntity.objects.filter(name=req_data.get('name')).exclude(id=oid).first()
            if name_org:
                return Response({
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': req_data.get('name') + "已存在"
                })
            org.name = req_data.get('name')
        if req_data.get('contact_phone'):
            org.contact_phone = req_data.get('contact_phone')
        if req_data.get('executive_id'):
            org.executive_id = req_data.get('executive_id')
        if req_data.get('address'):
            org.address = req_data.get('address')
        if req_data.get('remarks'):
            org.remarks = req_data.get('remarks')
        org.save()

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '修改部门信息完成'
        })

    # 删除部门信息
    def delete(self, request, *args, **kwargs):
        oid = kwargs.get('oid')
        if int(oid) == DEPT_ROOT_ID:
            return Response({
                'code': 403,
                'msg': "不能删除根部门节点"
            })

        accountCount = AccountEntity.objects.filter(org_info__id=oid).count()
        if accountCount > 0:
            return Response({
                'code': 501,
                'msg': "部门下存在子部门或终端,无法删除！"
            })
        subCount = OrgnizationEntity.objects.filter(parent_id=oid, id__gt=DEPT_ROOT_ID).count()
        if subCount > 0:
            return Response({
                'code': 501,
                'msg': "部门下存在子部门或终端,无法删除！"
            })

        device_count = Devices.objects.filter(org_info=oid).count()
        if device_count > 0:
            return Response({
                'code': 501,
                'msg': "部门下存在子部门或终端,无法删除！"
            })
        try:
            org = OrgnizationEntity.objects.get(pk=oid)
            org.delete()
            ldap_org = LdapOrgnization.objects.filter(org_id=oid)
            if ldap_org:
                ldap_org.delete()
            return Response({
                'code': status.HTTP_200_OK,
                'msg': '删除部门信息完成'
            })
        except Exception as e:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': str(e)
            })


class OrganizationExecutive(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    # 变更部门主管  自动加入部门
    def put(self, request, *args, **kwargs):
        oid = kwargs.get('oid')
        req_data = request.data
        if req_data.get('executive_id'):
            executive_id = req_data.get('executive_id')
        else:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': "主管人员参数缺失"
            })
        try:
            org = OrgnizationEntity.objects.get(pk=oid)
            executive_account = AccountEntity.objects.get(pk=executive_id)
        except Exception as e:
            logger.info(str(e))
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': "部门或人员信息错误"
            })

        if org.executive_id and int(org.executive_id) == int(executive_id):
            logger.info(' no change executive')
        else:
            executive_account.org_info_id = oid
            executive_account.save()
            org.executive_id = executive_id
            org.save()

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '变更主管人员成功'
        })


class OrganizationAccount(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    # 移除部门全部成员
    def delete(self, request, *args, **kwargs):
        oid = kwargs.get('oid')
        if int(oid) == DEPT_ROOT_ID:
            return Response({
                'code': 403,
                'msg': "不能移除根部门成员"
            })

        accountCount = AccountEntity.objects.filter(org_info__id=oid).count()
        if accountCount > 0:
            accounts = AccountEntity.objects.filter(org_info__id=oid)
            for account in accounts:
                account.org_info_id = DEPT_ROOT_ID
                account.save()

        try:
            org = OrgnizationEntity.objects.get(pk=oid)
            org.executive_id = ""
            org.save()
            return Response({
                'code': status.HTTP_200_OK,
                'msg': '移除终端用户成功'
            })
        except Exception as e:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': str(e)
            })


@api_view(["PUT"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def org_batch_account_operation(request):
    """
    部门批量加入或移除域用户
    :param request:
        ids_list 用户id
        org_id 部门id
        action 操作  0 加用户，1 移除用户
    :return:
    """
    ids_list_str = request.data.get('ids_list')
    action = request.data.get('action')
    org_id = request.data.get('org_id')

    if not ids_list_str or not org_id:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '参数不能为空'
        })

    ids_list = str_to_list(ids_list_str)
    account_list = AccountEntity.objects.filter(pk__in=ids_list)
    for account in account_list:
        if action == 0:
            account.org_info_id = org_id
            account.save()
        else:
            # 用户删除部门后，会被移动到顶层部门下
            account.org_info_id = 1
            account.save()

    return Response({
        'code': status.HTTP_200_OK,
        'msg': '修改成功'
    })


@api_view(["PUT"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def org_batch_device_operation(request):
    """
    部门批量加入或移除设备
    :param request:
        org_id 部门id
        ids_list 设备id
        action 操作 0 加设备，1 移除设备
    :return:
    """
    ids_list_str = request.data.get('ids_list')
    action = request.data.get('action')
    org_id = request.data.get('org_id')

    if not ids_list_str or not org_id:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '参数不能为空'
        })

    ids_list = str_to_list(ids_list_str)
    device_list = Devices.objects.filter(pk__in=ids_list)
    for device in device_list:
        if action == 0:
            device.org_info_id = org_id
            device.save()
        else:
            # 设备删除部门后，会被移动到顶层部门下
            device.org_info_id = 1
            device.save()

    return Response({
        'code': status.HTTP_200_OK,
        'msg': '修改成功'
    })


class OrganizationtBatch(APIView):
    authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,IsAdminUser)
    # throttle_classes = (UserRateThrottle,)
    parsers_classes = [FormParser, MultiPartParser]

    def build_and_get_structure(self, orgs_str):
        if not orgs_str:
            return OrgnizationEntity.objects.get(pk=DEPT_ROOT_ID)
        orgs = [o.strip() for o in orgs_str.split('/')]
        parent_org = None
        # 逐级创建部门
        for i, org in enumerate(orgs):
            parent_org, created = OrgnizationEntity.objects.get_or_create(
                name=org,
                parent_id=DEPT_ROOT_ID if (i == 0) else parent_org.id,
            )
        return parent_org

    def put(self, request, filename, format=None):
        if not filename.endswith('.xlsx'):
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '请上传xlsx格式文件',
            })
        xlsx_file = request.FILES['file'].read()
        df = pd.read_excel(xlsx_file, engine='openpyxl')
        df.columns = ['name', 'parent_name', 'remarks', 'directions']
        total_num = len(df)
        # 合规性检查
        if df['name'].isnull().any():
            return Response({'code': status.HTTP_400_BAD_REQUEST, 'msg': '导入失败，部门名为必输项', 'data': None})
        lengths = df['name'].str.len()
        if any((length > 32) for length in lengths):
            return Response({'code': status.HTTP_400_BAD_REQUEST,
                             'msg': '导入失败，部门名称超过规定32个字符不可导入', 'data': None})
        if df['parent_name'].isnull().any():
            return Response({'code': status.HTTP_400_BAD_REQUEST, 'msg': '导入失败，上级部门名称为必输项', 'data': None})

        try:
            df['orgs'] = df['parent_name'].astype(str) + '/' + df['name'].astype(str)
            df['org_info_id'] = df['orgs'].apply(lambda x: self.build_and_get_structure(x).id)
            count_num = len(df)
        except Exception as exp:
            logger.info(f"部门导入错误信息{exp}")
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '批量导入失败',
                'data': None,
            })
        else:
            websocket_msg = {
                'content': f"{filename}已导入完成，成功{count_num}条数据，失败{total_num - count_num}条数据",
                'create_time': datetime.datetime.now(),
                'target_module': '部门导入',
                'message_user': self.request.user.username,
                'message_device': None,
                'download_path': None,
                'is_read': 0,
                'is_broadcast': 0,
            }
            MessageCenter.objects.create(**websocket_msg)
            send_msg_to_web(websocket_msg, 0, self.request.user.username)
            return Response({
                'code': status.HTTP_200_OK,
                'msg': '部门导入中，请稍后前往消息中心查看',
                'data': {
                    'total': total_num,
                    'success': count_num,
                },
            })


class OrganizationBatchTmpl(APIView):
    #
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (AllowAny,)
    # throttle_classes = (UserRateThrottle,)
    # parsers_classes = [FormParser, MultiPartParser]

    def get(self, request, *args, **kwargs):
        from utils.utils import write_xls
        from django.shortcuts import HttpResponse

        head = ['部门名称\r\n(必填)', '上级部门\r\n(必填)', '备注\r\n(选填)',
                '填写说明\r\n'
                '''1.部门名称必填，不可重复，限制中英文、数字、特殊字符（~!@#$%^&*( )_ -={}|[]\:"";'<>?,. ）输入，最多32字符;\r\n'''
                '''2.上级部门：该新建部门的直接上级部门，部门层级请用/隔离，如一级部门/二级部门/三级部门/上级部门;\r\n'''
                '''3.备注: 非必填，限制中英文、数字、特殊字符（~!@#$%^&*( )_ -={}|[]\:"";'<>?,./ ）输入, 备注不得超过100字。\r\n'''
                ]
        data = []
        logger.info('get org import templeate')
        wb = write_xls('部门列表', head, data)
        ws = wb.active
        header_font = Font(color='FF0000')
        ws['A1'].font = header_font
        ws['B1'].font = header_font
        ws.column_dimensions['D'].width = 60
        for r in ws:
            for c in r:
                c.alignment = openpyxl.styles.Alignment(wrapText=True, horizontal='center', vertical='center')
        # response = HttpResponse(content_type='application/octet-stream')
        response = HttpResponse(content_type='application/ms-excel')
        response['Content-Disposition'] = 'attachment; filename=部门导入模板.xlsx'
        wb.save(response)
        return response

    # 导出


class OrganizationExport(APIView):
    # authentication_classes = []
    # permission_classes = (AllowAny,)
    # throttle_classes = (UserRateThrottle,)
    parsers_classes = [FormParser, MultiPartParser]

    def get(self, request, *args, **kwargs):
        from utils.utils import write_xls
        from django.shortcuts import HttpResponse
        orgs = OrgnizationEntity.objects.all()
        orgs = orgs.order_by('-id', 'parent_id')
        head = ['部门名称', '上级部门']
        data = []
        for org in orgs:
            org_info = [org.name, org.parent.name]
            data.append(org_info)
        # wb = write_xls('zdyh', head, data)
        df = pd.DataFrame(data, columns=head)
        filename = f'nfs_domain_部门导出_{datetime.datetime.now().strftime("%Y%m%d%H%M%S")}.xlsx'
        df.to_excel(f"{SALTFILESYSTEMPATH}/{filename}", encoding='utf-8', index=False)
        logger.info('export all organazation info to excel')

        # 将文件已经下载的消息发送给前端websocket
        websocket_msg = {
            'content': f"部门已导出完成, {filename}，总共{len(data)}条数据",
            'create_time': datetime.datetime.now(),
            'target_module': '部门导出',
            'message_user': self.request.user.username,
            'message_device': None,
            'download_path': f'/mainapi/client/{filename}/',
            'is_read': 0,
            'is_broadcast': 0,
        }
        MessageCenter.objects.create(**websocket_msg)
        send_msg_to_web(websocket_msg, 0, self.request.user.username)

        # response = HttpResponse(content_type='application/ms-excel')
        # response = HttpResponse(content_type='application/octet-stream')
        # response['Content-Disposition'] = 'attachment; filename=orgs.xlsx'
        # wb.save(response)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': None,
            'data': '请前往通知中心查看',
        })


# 解析LDAP导出的excel
class OrganizationBatchImportLDAP(APIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,IsAdminUser)
    # throttle_classes = (UserRateThrottle,)
    parsers_classes = [FormParser, MultiPartParser]

    def build_and_get_structure(self, orgs_str):
        if not orgs_str:
            return OrgnizationEntity.objects.get(pk=DEPT_ROOT_ID)
        orgs = [o.strip() for o in orgs_str.split('@@')]
        parent_org = None
        # 逐级创建部门
        for i, org in enumerate(orgs):
            parent_org, created = OrgnizationEntity.objects.get_or_create(
                name=org,
                parent_id=DEPT_ROOT_ID if (i == 0) else parent_org.id,
            )
        return parent_org

    def put(self, request, filename, format=None):
        from openpyxl import load_workbook
        from io import BytesIO

        if not filename.endswith('.xlsx'):
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '请上传xlsx格式文件',
            })
        file = request.FILES['file'].read()
        wb = load_workbook(filename=BytesIO(file))

        logger.info(wb.worksheets)
        rows = wb.worksheets[0].rows
        total = 0
        count = 0
        for i, row in enumerate(rows):
            # 跳过第一行
            if (i == 0): continue
            total += 1

            objectCategory = row[12].value
            logger.info(objectCategory)
            if objectCategory and objectCategory.find('CN=Organizational-Unit') != -1:
                logger.info('category is Organizational-Unit')
            else:
                logger.info('category is other')
                continue

            dn = row[0].value
            logger.info(dn)
            dn = dn[dn.find('OU'):dn.find('DC') - 1]
            dns = dn.split(',')
            orgs_arr = []
            for d in dns:
                if d is not None:
                    ou = d[d.find("=") + 1:]
                    orgs_arr.append(ou)
            orgs_arr.reverse()
            orgs = '@@'
            orgs = orgs.join(orgs_arr)
            logger.info(orgs)

            logger.info("#################################")

            # 构建机构的路径
            self.build_and_get_structure(orgs)

            count += 1

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '批量导入成功',
            'data': {
                'total': total,
                'success': count,
            },
        })


class OrgansViewSet(ModelViewSet):
    queryset = OrgnizationEntity.objects.all()
    serializer_class = OrganSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('id',)

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())

        # page = self.paginate_queryset(queryset)
        # if page is not None:
        #     serializer = self.get_serializer(page, many=True)
        #     return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @classmethod
    def organ_down_name(cls, num):
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return None
        else:
            organ_down_list = organ_down_list + list(items.values_list('name', flat=True))
            while True:
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('name', flat=True))

        return ';'.join(organ_down_list)

    @classmethod
    def organ_member_count(cls, num):
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return AccountEntity.objects.filter(org_info_id=num).count()
        else:
            organ_down_list = organ_down_list + list(items.values_list('id', flat=True))
            while True:
                # logger.info(f'**********{items}*********')
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('id', flat=True))

        organ_down_list.append(num)
        return AccountEntity.objects.filter(org_info_id__in=organ_down_list).count()

    @classmethod
    def organ_down_id(cls, num):
        """父节点的id"""
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return [num]
        else:
            organ_down_list = organ_down_list + list(items.values_list('id', flat=True))
            while True:
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('id', flat=True))

        return organ_down_list

    @classmethod
    def organ_up_name(cls, num):
        items = OrgnizationEntity.objects.all()
        name = items.filter(id=num).first().name
        if num == DEPT_ROOT_ID:
            pass
        else:
            while True:
                if items.filter(id=num, parent_id__isnull=False).exists():
                    up_objs = items.get(id=num, parent_id__isnull=False)
                    num = up_objs.parent_id
                    name = items.get(id=num).name + f'/{name}'
                    if num == DEPT_ROOT_ID:
                        break
                else:
                    break
        return name

    def organ_filter_queryset(self, queryset):
        """
        :param queryset: 表数据结果集合
        :return: 根据过滤条件筛选返回结果集合
        """
        query_org_id = self.request.query_params.get('query_org_id', default=None)

        if query_org_id:
            queryset = queryset.filter(id=query_org_id)

        return queryset

    def download_organ_info(self, queryset):
        queryset = queryset.values_list('id', 'name', 'remarks', 'last_update')
        head_data = [u'主键', u'部门名称', '备注', '更新时间']
        df = pd.DataFrame(queryset, columns=head_data)
        df['部门名称'] = df['主键'].apply(lambda x: self.organ_up_name(x))
        df['子部门数量'] = df['主键'].apply(get_org_num)
        df['终端数量'] = df['主键'].apply(get_device_num)
        df['人员数量'] = df['主键'].apply(lambda x: self.organ_member_count(x))
        # df['更新时间'] = df['更新时间'].apply(lambda x: x.strftime('%Y-%m-%d %H:%M:%S') if x is not None else None)
        df.drop(columns=['主键', ], inplace=True)

        order = ['部门名称', '子部门数量', '终端数量', '人员数量', '备注', '更新时间']
        df = df[order]

        # output = io.BytesIO()
        # df.to_excel(output, encoding='utf-8', index=False)
        # output.seek(0)
        #
        # # 将文件传送到web
        # response = StreamingHttpResponse(output)
        # response['Content-Type'] = 'application/octet-stream'
        # response['Content-Disposition'] = 'attachment;filename="{0}"'.format(filename)

        filename = f'nfs_domain_部门导出_{datetime.datetime.now().strftime("%Y%m%d%H%M%S")}.xlsx'
        df.to_excel(f"{SALTFILESYSTEMPATH}/{filename}", encoding='utf-8', index=False)

        # 将文件已经下载的消息发送给前端websocket
        websocket_msg = {
            'content': f"部门已导出完成, {filename}，总共{len(queryset)}条数据",
            'create_time': datetime.datetime.now(),
            'target_module': '部门导出',
            'message_user': self.request.user.username,
            'message_device': None,
            'download_path': f'/mainapi/client/{filename}/',
            'is_read': 0,
            'is_broadcast': 0,
        }
        MessageCenter.objects.create(**websocket_msg)
        send_msg_to_web(websocket_msg, 0, self.request.user.username)
        close_old_connections()

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def organ_batch_operation(self, request, *args, **kwargs):
        """
        部门批量 导出，删除
        :param request:body {
                        is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        is_delete : 0|不删除 1|删除 空|不处理
                        is_down_file: 0|不下载 1|下载 空|不处理
                        filename: 文件名
                        ......: 筛选条件
                        }
        :param args:
        :param kwargs:
        :return:
        """
        is_all = request.data.get('is_all')
        ids_list = request.data.get('ids_list')
        is_delete = request.data.get('is_delete')
        is_down_file = request.data.get('is_down_file')
        filename = request.data.get('filename')

        if is_all is None:
            return Response('is_all 为必输选项', status=status.HTTP_400_BAD_REQUEST)

        is_all = int(is_all)

        queryset = OrgnizationEntity.objects.all()

        if is_all == 0:
            try:
                ids_list = [int(i) for i in ids_list.split(',')]
            except Exception as exp:
                return Response(f'ids_list参数不符合规范{exp}', status=status.HTTP_400_BAD_REQUEST)
        else:
            queryset = self.organ_filter_queryset(queryset)
            ids_list = queryset.values_list('id', flat=True)

        # 查询过滤结果集
        queryset = queryset.filter(id__in=ids_list)

        # 设备删除
        if is_delete == 1:
            # 可以删除部门下含有人员和设备的部门，但需要将设备和人员转移到上级部门
            top_id = ids_list[0]  # 所选部门的最上级部门
            organ_sub_id = queryset.get(id=top_id).parent_id
            AccountEntity.objects.filter(org_info_id__in=ids_list).update(org_info_id=organ_sub_id)
            Devices.objects.filter(org_info_id__in=ids_list).update(org_info_id=organ_sub_id)
            queryset.exclude(id=DEPT_ROOT_ID).delete()

            LdapOrgnization.objects.filter(org_id__in=ids_list).delete()
            return Response('批量删除部门成功', status=status.HTTP_200_OK)

        if is_down_file == 1:
            # 保存文件到服务器
            if queryset.count() == 0:
                return Response('未找到满足条件的数据，导出失败', status=status.HTTP_200_OK)

            # 后台处理域终端下载
            thread1 = threading.Thread(target=self.download_organ_info, args=(queryset,))
            thread1.start()

            return Response('部门导出中，请稍后前往消息中心查看', status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def change_organ_seq(self, request, *args, **kwargs):
        """
        改变同级别部门的顺序
        :param request: body {
                        value: 部门id的值
                        seq_num: 移动到目标位置的序号
                        }
        :param args:
        :param kwargs:
        :return:
        """
        organ_id = request.data.get('value', None)
        seq_num = request.data.get('seq_num', None)
        if (organ_id is None) or (seq_num is None):
            return Response("调整部门请求参数错误", status=status.HTTP_400_BAD_REQUEST)
        old_organ_obj = self.queryset.get(id=organ_id)
        self.queryset.filter(parent_id=old_organ_obj.parent_id, seq_num=seq_num).update(seq_num=old_organ_obj.seq_num)
        self.queryset.filter(id=organ_id).update(seq_num=seq_num)
        return Response("调整部门顺序成功", status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def organ_info_count(self, request, *args, **kwargs):
        """
        获取部门下的终端数据，终端在线和终端离线数量   |   获取部门下的人员数据，人员在线和人员离线数量
        :param request: query_params :  organ_id  部门id
                                        type_info  0|设备 1|人员
        :param args:
        :param kwargs:
        :return:
        """
        user_data = request.user
        org_list = get_org_permission_list(user_data)
        logger.info(f"{org_list}")
        organ_id = self.request.query_params.get('organ_id', None)
        type_info = self.request.query_params.get('type_info', None)
        if (organ_id is None) or (type_info is None):
            return Response("获取部门总数详情参数错误", status=status.HTTP_400_BAD_REQUEST)
        organ_id = int(organ_id)
        type_info = int(type_info)
        redis_client = get_redis_connection('redis')
        organ_name = self.queryset.get(id=organ_id).name
        organ_id_down_list = self.organ_down_id(organ_id)  # 下级部门不包括自己
        organ_id_down_list.append(organ_id)
        if type_info == 0:
            # 获取设备
            device_items = Devices.objects.exclude(is_domain=0).filter(org_info_id__in=organ_id_down_list)
            if org_list:
                device_items = device_items.filter(org_info__in=org_list)
            type_info_total = device_items.count()
            all_fields = redis_client.hgetall('actives')
            online_device = []
            for key, value in all_fields.items():
                if value == b'1':
                    online_device.append(key.decode('utf-8'))
            type_online_count = device_items.filter(device_hostname__in=online_device).count()
            type_offline_count = device_items.exclude(device_hostname__in=online_device).count()
        elif type_info == 1:
            # 获取人员
            if organ_id == 1:
                accounts_items = AccountEntity.objects.all()
                if org_list:
                    accounts_items = accounts_items.filter(org_info__in=org_list)
                type_info_total = accounts_items.count()
            else:
                accounts_items = AccountEntity.objects.filter(org_info_id__in=organ_id_down_list)
                if org_list:
                    accounts_items = accounts_items.filter(org_info__in=org_list)
                type_info_total = accounts_items.filter(org_info_id__in=organ_id_down_list).count()
            # cache = caches['redis']
            # device_user_active_dict_str = cache.get("device_user_active")
            # active_user_list = []
            # if device_user_active_dict_str:
            #     device_user_active_dict = json.loads(device_user_active_dict_str)
            #     for k, v in device_user_active_dict.items():
            #         if v:
            #             active_user_list += v
            # type_online_count = accounts_items.filter(name__in=active_user_list).count()
            # type_offline_count = accounts_items.exclude(name__in=active_user_list).count()
            type_online_count = accounts_items.filter(user_status=0).count()
            type_offline_count = accounts_items.exclude(user_status=0).count()
        else:
            return Response("获取部门总数详情type_info参数错误", status=status.HTTP_400_BAD_REQUEST)

        res_data = {
            'organ_name': organ_name,
            'organ_total': type_info_total,
            'organ_online_count': type_online_count,
            'organ_offline_count': type_offline_count
        }
        return Response(res_data, status=status.HTTP_200_OK)


class OrgLabelViewSet(mixins.ListModelMixin,
                      mixins.CreateModelMixin,
                      mixins.UpdateModelMixin,
                      mixins.DestroyModelMixin,
                      GenericViewSet):
    queryset = OrgLabel.objects.all()
    serializer_class = OrgLabelSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = None
    authentication_classes = (TokenAuthentication,)
    ordering = ('-id',)

    def get_object(self):
        pk = self.kwargs.get('pk')
        return self.queryset.filter(org_id=pk).first()

    def create(self, request, *args, **kwargs):
        org_label = self.request.data.get('org_label', None)
        if org_label is not None:
            if self.queryset.filter(org_label=org_label).exists():
                return Response('部门缩写已存在', status=status.HTTP_400_BAD_REQUEST)
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    def update(self, request, *args, **kwargs):
        org_label = self.request.data.get('org_label', None)
        if org_label is not None:
            if self.queryset.filter(org_label=org_label).exists():
                return Response('部门缩写已存在', status=status.HTTP_400_BAD_REQUEST)
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        return Response(serializer.data)
