from urllib.parse import quote
import os
import requests
from django.conf import settings
import json
import logging

from nfs_domain_mainserver.settings import HOSTNAME

logger = logging.getLogger(__name__)


class AptlyUtil(object):
    # 补丁源服务器
    aptly_server = ""

    aptly_file_dir = "testdir"

    def __init__(self, aptly_server=f"http://{HOSTNAME}:2322"):
        self.aptly_server = aptly_server
        logger.info("aptly_server is %s" % self.aptly_server)

    # 将web上传的文件，上传 存储到repo中
    def add_package(self, patch_file, repo_name, filename):
        full_path = settings.MEDIA_ROOT + "/" + patch_file.file.name
        logger.info("repo_name is %s" % repo_name)
        logger.info("filename is %s" % filename)
        logger.info("full_path is %s" % full_path)

        # 上传到aptly服务的目录
        api_path = "/api/files/" + self.aptly_file_dir
        url = self.aptly_server + api_path
        payload = {}
        files = [('file', (filename, open(full_path, 'rb'), 'application/octet-stream'))]
        headers = {}
        res = requests.request("POST", url, headers=headers, data=payload, files=files)
        logger.info("push file to aptly ,return is %s" % res.text)

        # ADD PACKAGES FROM UPLOADED FILE/DIRECTORY
        api_path = "/api/repos/" + repo_name + "/file/" + self.aptly_file_dir
        url = self.aptly_server + api_path
        res = requests.request("POST", url, headers=headers, data=payload)
        logger.info("ADD PACKAGES FROM UPLOADED FILE/DIRECTORY ,return is %s" % res.text)

        # 更新版本信息
        self.update_source_version()

    def batch_add_package(self, patch_file, repo_name, filename):
        full_path = patch_file
        logger.info("repo_name is %s" % repo_name)
        logger.info("filename is %s" % filename)
        logger.info("full_path is %s" % full_path)

        # 上传到aptly服务的目录
        api_path = "/api/files/" + self.aptly_file_dir
        url = self.aptly_server + api_path
        payload = {}
        files = [('file', (filename, open(full_path, 'rb'), 'application/octet-stream'))]
        headers = {}
        res = requests.request("POST", url, headers=headers, data=payload, files=files)
        logger.info("push file to aptly ,return is %s" % res.text)

        # ADD PACKAGES FROM UPLOADED FILE/DIRECTORY
        api_path = "/api/repos/" + repo_name + "/file/" + self.aptly_file_dir
        url = self.aptly_server + api_path
        res = requests.request("POST", url, headers=headers, data=payload)
        logger.info("ADD PACKAGES FROM UPLOADED FILE/DIRECTORY ,return is %s" % res.text)

        # 更新版本信息
        self.update_publish('server/server')

    def batch_del_package(self, repo_name, keys):
        api_path = "/api/repos/" + repo_name + "/packages"
        url = self.aptly_server + api_path

        payload = json.dumps({
            "PackageRefs": keys
        })
        headers = {
            'Content-Type': 'application/json'
        }

        response = requests.request("DELETE", url, headers=headers, data=payload)

        logger.info(f"删除补丁返回信息{response.text}")

        # 更新版本信息
        self.update_publish('server/server')

    def update_publish(self, publish_url):
        """
        更新服务器资源端版本信息
        :param publish_url: 目前publish_url 仓库发布连接
        :return:
        """
        # UPDATE PUBLISHED LOCAL REPO/SWITCH PUBLISHED SNAPSHOT
        api_path = f"/api/publish/:{publish_url}"
        url = self.aptly_server + api_path
        payload = json.dumps({
            "Signing": {
                "skip": True
            }
        })
        headers = {
            'Content-Type': 'application/json'
        }
        res = requests.request("PUT", url, headers=headers, data=payload)
        logger.info("UPDATE PUBLISHED LOCAL REPO/SWITCH PUBLISHED SNAPSHOT ,return is %s" % res.text)

    # 通过包名 版本 架构 查询repository下的包信息 返回key
    def query_package_keys(self, repo_name, package_name, version, architecture):
        # demo   "http://172.30.98.66:9779/api/repos/testrepo/packages?q=libffi6 (>=3.2.1-8) {amd64}"
        if architecture is not None:
            query_path = "/api/repos/" + repo_name + "/packages?q=" + package_name + " (>=" + version + ") {" + architecture + "}"
        else:
            query_path = "/api/repos/" + repo_name + "/packages?q=" + package_name + " (>=" + version + ")"

        logger.info("query_path is %s" % query_path)
        url = self.aptly_server + query_path
        encoded_url = quote(url, safe=":/?&=")
        payload = {}
        headers = {}
        response = requests.request("GET", encoded_url, headers=headers, data=payload)
        """
        返回示例
        [
            "Pamd64 libffi6 3.2.1-8 aea96db8b073e45a"
        ]
        """
        logger.info(f"查询补丁返回信息{response.text}")
        return json.loads(response.text)

    # 删除repository中的package
    def del_package(self, repo_name, keys):
        api_path = "/api/repos/" + repo_name + "/packages"
        url = self.aptly_server + api_path

        payload = json.dumps({
            "PackageRefs": keys
        })
        headers = {
            'Content-Type': 'application/json'
        }

        response = requests.request("DELETE", url, headers=headers, data=payload)

        logger.info(f"删除补丁返回信息{response.text}")

        # 更新版本信息
        self.update_source_version()

    def update_source_version(self, distribution="5.0"):
        """
        更新服务器资源端版本信息
        :param distribution: 目前distribution 默认5.0 后续需要根据配置调整
        :return:
        """
        # UPDATE PUBLISHED LOCAL REPO/SWITCH PUBLISHED SNAPSHOT
        api_path = "/api/publish/:./" + distribution
        url = self.aptly_server + api_path
        payload = json.dumps({
            "Signing": {
                "skip": True
            }
        })
        headers = {
            'Content-Type': 'application/json'
        }
        res = requests.request("PUT", url, headers=headers, data=payload)
        logger.info("UPDATE PUBLISHED LOCAL REPO/SWITCH PUBLISHED SNAPSHOT ,return is %s" % res.text)

    def get_package_versions(self, repo_name='nfs-5.0', package_name='nfs-domain-client'):
        """获取aptly中包的版本信息"""
        query_path = "/api/repos/" + repo_name + "/packages?q=" + package_name
        url = self.aptly_server + query_path
        encoded_url = quote(url, safe=":/?&=")
        payload = {}
        headers = {}
        response = requests.request("GET", encoded_url, headers=headers, data=payload)
        """
        返回示例
        [
            "Pamd64 libffi6 3.2.1-8 aea96db8b073e45a"
        ]
        """
        logger.info(f"查询包{package_name}返回信息{response.text}")

        try:
            response = requests.request("GET", encoded_url, headers=headers, data=payload)
            response.raise_for_status()
        except requests.RequestException as e:
            return False, str(e)

        response_list = json.loads(response.text)
        res_list = []
        for i in response_list:
            res_list.append(i.split(' ')[2])

        max_value = max(res_list)
        return True, max_value

    def download_package(self, repo_name, package_name, custom_filename, save_dir):
        '''
        支持下载的软件是在web端上传到aptly中的
        repo_name: 仓库名，当软件在aptly中时，repo_name=server-repo
        package_name: 软件名，software_info表中的main_package字段的值
        custom_filename: 软件全名，software_info表中的custom_filename字段的值
        save_dir: 保存路径

        '''
        aptly_download_server = f"http://{HOSTNAME}:9778"
        aptly_package_path = f'/server/pool/main/{package_name[0]}/{package_name}/{custom_filename}'  # e.g: server/pool/main/m/mqttx/MQTTX_1.10.1_amd64.deb
        # 下载包
        deb_response = requests.get(f'{aptly_download_server}{aptly_package_path}', headers={}, stream=True)
        deb_response.raise_for_status()
        os.makedirs(save_dir, exist_ok=True)
        file_path = os.path.join(save_dir, custom_filename)
        with open(file_path, "wb") as f:
            for chunk in deb_response.iter_content(chunk_size=8192):
                f.write(chunk)

        logger.info(f"包 {custom_filename} 下载成功，保存路径: {file_path}")

    def download_patch(self, repo_name, patch_no, patch_file_name, save_dir, patch_source=None):
        '''
        补丁下载
        patch_source: 补丁源 相关信息在patch_source表
            如果补丁源是自带的，即patch_source表中补丁源端口是2322时 patch_source = patch_source_url:9778
            如果补丁源是第三方的，即patch_source表中补丁源端口是6899时 patch_source = patch_source_url:6898
        repo_name: 仓库名，补丁分为系统补丁和安全补丁，系统补丁时 repo_name=sys ; 安全补丁时repo_name=security
        patch_no: 补丁名，patch_info表中的patch_no字段的值
        patch_file_name: 软件全名，patch_info表中的patch_file_name字段的值
        save_dir: 保存路径
        '''
        if not patch_source:
            patch_source = f"http://{HOSTNAME}:9778"
        aptly_package_path = f'/pool/{repo_name}/{patch_no[0]}/{patch_no}/{patch_file_name}'  # e.g: /pool/sys/m/mqttx/MQTTX_1.10.1_amd64.deb
        # 下载包
        deb_response = requests.get(f'{patch_source}{aptly_package_path}', headers={}, stream=True)
        deb_response.raise_for_status()
        os.makedirs(save_dir, exist_ok=True)
        file_path = os.path.join(save_dir, patch_file_name)
        with open(file_path, "wb") as f:
            for chunk in deb_response.iter_content(chunk_size=8192):
                f.write(chunk)

        logger.info(f"补丁 {patch_file_name} 下载成功，保存路径: {file_path}")