from django.db import models
import datetime


# Create your models here.

def user_directory_path(instance, filename):
    path = f"local_source/{filename}"
    return path


class PatchFile(models.Model):
    name = models.CharField('名称', max_length=300, null=True, blank=True)
    size = models.IntegerField('文件大小', default=1, null=True, blank=True)
    file = models.FileField('文件', upload_to=user_directory_path)
    mime = models.CharField('文件格式', max_length=300, null=True, blank=True)
    path = models.CharField('地址', max_length=1000, null=True, blank=True)
    create_by = models.CharField('创建人', max_length=255, blank=True, null=True)
    create_time = models.DateTimeField(verbose_name='创建补丁时间', blank=True, null=True)

    class Meta:
        verbose_name = '补丁文件'
        verbose_name_plural = verbose_name
        db_table = 'patch_file'

    def __str__(self):
        return self.name


class PatchInfo(models.Model):
    patch_name = models.CharField(verbose_name='补丁名称', max_length=255, blank=True, null=True)
    patch_ver = models.CharField(verbose_name='补丁版本', max_length=255, blank=True, null=True)
    patch_no = models.CharField(verbose_name='补丁编号', max_length=255, blank=True, null=True)
    patch_source = models.CharField(verbose_name='补丁源', max_length=255, blank=True, null=True)
    patch_source_name = models.CharField(verbose_name='补丁源名称', max_length=255, blank=True, null=True)
    patch_app_arch = models.CharField(verbose_name='补丁应用构架', max_length=255, blank=True, null=True)
    patch_app_arch_name = models.CharField(verbose_name='补丁应用构架名称', max_length=255, blank=True, null=True)
    patch_app_sys = models.CharField(verbose_name='补丁应用系统', max_length=255, blank=True, null=True)
    patch_app_sys_name = models.CharField(verbose_name='补丁应用系统名称', max_length=255, blank=True, null=True)
    patch_type = models.CharField(verbose_name='补丁类型', max_length=255, blank=True, null=True)
    patch_type_name = models.CharField(verbose_name='补丁类型名称', max_length=255, blank=True, null=True)
    patch_file_id = models.IntegerField(verbose_name='补丁文件id', null=True, blank=True)
    patch_file_name = models.CharField('名称', max_length=300, null=True, blank=True)
    patch_is_delete = models.SmallIntegerField('补丁是否删除  0|否；1|是', default=0)
    update_time = models.DateTimeField(verbose_name='补丁更新时间', auto_now_add=True, blank=True, null=True)
    remark = models.TextField(verbose_name='补丁备注信息', blank=True, null=True)

    class Meta:
        db_table = 'patch_info'


class PatchTypeDict(models.Model):
    patch_type_name = models.CharField(verbose_name='补丁类型名称', max_length=255, blank=True, null=True)
    patch_is_sys = models.SmallIntegerField(verbose_name='补丁类型是否为系统创建  0|否；1|是', default=0)
    patch_is_delete = models.SmallIntegerField(verbose_name='补丁类型名称记录是否删除  0|否；1|是', default=0)
    patch_repo_name = models.CharField(verbose_name='补丁源中repository名称', max_length=255, blank=True, null=True)

    class Meta:
        db_table = 'patch_type_dict'


class PatchSysDict(models.Model):
    patch_sys_ver = models.CharField(verbose_name='补丁应用系统版本', max_length=255, blank=True, null=True)
    patch_is_sys = models.SmallIntegerField(verbose_name='补丁应用系统版本是否为系统创建  0|否；1|是', default=0)
    patch_is_delete = models.SmallIntegerField(verbose_name='补丁类型名称记录是否删除  0|否；1|是', default=0)

    class Meta:
        db_table = 'patch_sys_dict'


class PatchSource(models.Model):
    patch_source_name = models.CharField(verbose_name='补丁源名称', max_length=255, blank=True, null=True)
    patch_source_url = models.CharField(verbose_name='补丁源地址', max_length=255, blank=True, null=True)
    patch_source_port = models.IntegerField(verbose_name='补丁源端口', blank=True, null=True)
    patch_source_enable = models.SmallIntegerField(verbose_name='补丁源是否启用  0|否；1|是', default=0)
    update_time = models.DateTimeField(verbose_name='补丁源更新时间', auto_now_add=True, blank=True, null=True)
    remark = models.TextField(verbose_name='补丁源备注信息', blank=True, null=True)

    class Meta:
        db_table = 'patch_source'


police_range_type_choices = (
    (0, '按部门'), (1, '按终端'), (2, '按 IP 网段'), (3, '按人员'), (4, '按组别')
)
police_status_choices = (
    (1, '启用'), (0, '禁用')
)


def _default_get_serial_number():
    date = datetime.datetime.today()
    # 转年月日时分秒
    return date.strftime("%Y%m%d%H%M%S")
    # return date.strftime("%Y%m%d%H%M%S%f")


class PatchPolicy(models.Model):
    name = models.CharField(verbose_name='补丁策略名称', max_length=255, blank=True, null=True)
    serial_number = models.CharField(verbose_name='补丁策略编号', max_length=32, default=_default_get_serial_number,
                                     blank=True, null=True)
    range_type = models.IntegerField(verbose_name='补丁策略执行范围的类型', choices=police_range_type_choices,
                                     default=0, blank=False)
    range_value = models.TextField(verbose_name='补丁策略执行范围的内容', blank=True, null=True)
    device_range_value = models.TextField(verbose_name='补丁策略执行范围的内容', blank=True, null=True)
    exception_switch_type = models.SmallIntegerField(verbose_name='补丁策略 例外 开关  0|否；1|是', default=0)
    exception_range_type = models.IntegerField(verbose_name='补丁策略 例外执行范围的类型',
                                               choices=police_range_type_choices,
                                               default=1, blank=False)
    exception_range_value = models.TextField(verbose_name='补丁策略 例外执行范围的内容', max_length=255, blank=True,
                                             null=True)
    execution_time = models.DateTimeField(verbose_name='策略开始参与计算时间', default=None, blank=True, null=True)
    status = models.IntegerField(verbose_name='补丁策略状态', choices=police_status_choices, default=0, blank=False)
    patch_ids = models.CharField(verbose_name='补丁信息ID的字符串 逗号分割', max_length=255, blank=True, null=True)
    creator = models.CharField(verbose_name='补丁信息创建人', max_length=64, blank=True, null=True)
    update_time = models.DateTimeField(verbose_name='更新时间', auto_now=True, blank=True, null=True)
    effective_date = models.DateTimeField(verbose_name='策略开始生效时间', default=None, blank=True, null=True)
    expiration_date = models.DateTimeField(verbose_name='策略开始失效时间', default=None, blank=True, null=True)
    retry_count = models.IntegerField(verbose_name='补丁策略重试次数', default=None, blank=True, null=True)
    is_notify_user = models.IntegerField(verbose_name='补丁策略是否通知用户,0|不通知；1|通知', default=0, blank=False)
    is_user_cancel = models.IntegerField(verbose_name='补丁策略是否允许用户取消,0|不允许；1|允许', default=0,
                                         blank=False)
    remark = models.TextField(verbose_name='备注信息', blank=True, null=True)

    class Meta:
        db_table = 'patch_policy'
        verbose_name = '补丁策略信息'
        verbose_name_plural = verbose_name
        ordering = ('-update_time',)


class PatchDeviceLog(models.Model):
    ins_status = (
        (0, '未安装'), (1, '已安装'), (2, '已安装未重启')
    )
    patch_id = models.IntegerField(verbose_name='补丁主键值', blank=True, null=True)
    device_id = models.IntegerField(verbose_name='终端主键值', blank=True, null=True)
    install_status = models.SmallIntegerField('补丁安装状态', choices=ins_status, default=0, blank=True, null=True)
    install_time = models.DateTimeField(verbose_name='补丁安装时间', auto_now_add=True, blank=True, null=True)
    install_err_msg = models.TextField(verbose_name='补丁安装失败原因', blank=True, null=True)
    is_send_status = models.SmallIntegerField('指令是否下发 0没发送，1已发送', default=0, blank=True, null=True)
    send_msg_time = models.DateTimeField(verbose_name='补丁指令下发时间', auto_now_add=True, blank=True, null=True)
    is_receive_status = models.SmallIntegerField('指令应答是否收到 0 没收到，1 收到', default=0, blank=True, null=True)

    class Meta:
        db_table = 'patch_device_log'


class PatchDeliverySettings(models.Model):
    number_queues = models.SmallIntegerField('策略下发队列数量', default=5, blank=True, null=True)
    number_concurrent = models.SmallIntegerField('策略下发并发数量', default=10, blank=True, null=True)
    timeout_period = models.SmallIntegerField('策略下超时响应时间 单位分钟', default=20, blank=True, null=True)

    class Meta:
        db_table = 'patch_delivery_settings'
