"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/9/4 上午11:07
@File: serializers.py
@Desc: 补丁管理序列化输出
"""
import ipaddress
import logging

from django_redis import get_redis_connection
from rest_framework import serializers

from groups.models import GroupsDetail
from patch_management import models
from devices.models import Devices
from accounts.models import AccountEntity
from organizations.models import OrgnizationEntity
from users.role_permisson import get_org_permission_list

# 类名定为表名称 + Serializer
conn = get_redis_connection('redis')


class PatchFileSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.PatchFile
        fields = "__all__"


class PatchInfoSerializer(serializers.ModelSerializer):
    install_total = serializers.SerializerMethodField()

    class Meta:
        model = models.PatchInfo
        fields = ('id', 'patch_name', 'patch_ver', 'patch_no', 'patch_source', 'patch_app_arch', 'patch_app_arch_name',
                  'patch_app_sys', 'patch_app_sys_name', 'patch_type', 'patch_file_id', 'update_time', 'remark',
                  'patch_source_name',
                  'patch_type_name', 'patch_file_name', 'patch_is_delete', 'install_total')

    def get_install_total(self, obj):
        device_id_list = Devices.objects.exclude(is_domain=0).values_list('id', flat=True)

        user_data = self.context['request'].user
        org_list = get_org_permission_list(user_data)
        if org_list:
            device_id_list = Devices.objects.exclude(is_domain=0).filter(org_info__in=org_list).values_list('id', flat=True)

        patch_obj_items = models.PatchDeviceLog.objects.filter(device_id__in=device_id_list).filter(patch_id=obj.id)
        install_success = patch_obj_items.filter(install_status=1).count()
        install_error = patch_obj_items.filter(install_status=0).count()
        install_not_restart = patch_obj_items.filter(install_status=2).count()

        return f"{install_success}/{install_error}/{install_not_restart}"


class PatchTypeDictSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.PatchTypeDict
        fields = '__all__'


class PatchSysDictSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.PatchSysDict
        fields = '__all__'


class PatchSourceSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.PatchSource
        fields = '__all__'


class PatchDevicesSerializer(serializers.ModelSerializer):
    user_name = serializers.SerializerMethodField()
    user_account = serializers.SerializerMethodField()
    org_info_name = serializers.SerializerMethodField()
    online_status = serializers.SerializerMethodField()
    install_total = serializers.SerializerMethodField()

    class Meta:
        model = Devices
        fields = (
            'id', 'device_cnname', 'user_name', 'user_account', 'device_ip', 'org_info_id', 'org_info_name', 'is_up',
            'online_status', 'install_total'
        )

    def get_user_name(self, obj):
        bind_accounts_list = obj.bind_account.all()
        items = AccountEntity.objects.filter(id__in=bind_accounts_list).first()
        if items:
            return items.cn_name
        else:
            return None

    def get_user_account(self, obj):
        bind_accounts_list = obj.bind_account.all()
        items = AccountEntity.objects.filter(id__in=bind_accounts_list).first()
        if items:
            return items.name
        else:
            return None

    def get_org_info_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()
        if items:
            return items.name
        else:
            return None

    def get_online_status(self, obj):
        if conn.hget('actives', obj.device_hostname):
            return 1
        else:
            return 0

    def get_install_total(self, obj):
        patch_id_list = models.PatchInfo.objects.filter(patch_is_delete=0).values_list('id', flat=True)
        patch_obj_items = models.PatchDeviceLog.objects.filter(patch_id__in=patch_id_list).filter(device_id=obj.id)
        install_success = patch_obj_items.filter(install_status=1).count()
        install_error = patch_obj_items.filter(install_status=0).count()
        install_not_restart = patch_obj_items.filter(install_status=2).count()

        return f"{install_success}/{install_error}/{install_not_restart}"


class PatchDeviceLogSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.PatchDeviceLog
        fields = '__all__'


class PatchPolicySerializer(serializers.ModelSerializer):
    install_total = serializers.SerializerMethodField()
    apply_status = serializers.SerializerMethodField()
    patch_num = serializers.SerializerMethodField()

    class Meta:
        model = models.PatchPolicy
        fields = (
            'id', 'name', 'serial_number', 'range_type', 'range_value', 'status', 'execution_time', 'patch_ids',
            'creator', 'update_time','effective_date','expiration_date','retry_count','is_notify_user','is_user_cancel',
            'remark', 'install_total', 'apply_status', 'exception_switch_type', 'exception_range_type',
            'exception_range_value', 'device_range_value', 'patch_num'
        )

    @staticmethod
    def mid_install_status(obj):
        device_items = Devices.objects.exclude(is_domain=0).all()
        group_chat_items = GroupsDetail.objects.all()
        range_type = obj.range_type
        range_value = obj.range_value
        exception_switch_type = obj.exception_switch_type
        exception_range_type = obj.exception_range_type
        exception_range_value = obj.exception_range_value
        res_device = []
        if range_type == 0:
            organ_list = range_value.split(',')
            if device_items.filter(org_info_id__in=organ_list).exists():
                device_list = device_items.filter(org_info_id__in=organ_list).values_list('id', flat=True)
                res_device = list(device_list)
        elif range_type == 1:
            res_device = range_value.split(',')
        elif range_type == 2:
            ip_list = range_value.split(',')
            mid_ip_list = []
            for i in ip_list:
                start_ip, end_ip = i.split('-')
                # 解析起始和结束IP地址为IPv4Address对象
                start_ip = ipaddress.IPv4Address(start_ip)
                end_ip = ipaddress.IPv4Address(end_ip)
                range_ip_list = [str(ipaddress.IPv4Address(ip)) for ip in range(int(start_ip), int(end_ip) + 1)]
                mid_ip_list = range_ip_list + mid_ip_list
            if device_items.filter(device_ip__in=mid_ip_list).exists():
                device_list = device_items.filter(device_ip__in=mid_ip_list).values_list('id', flat=True)
                res_device = list(device_list)
        elif range_type == 3:
            account_list = range_value.split(',')
            if device_items.filter(bind_account__id__in=account_list).exists():
                device_list = device_items.filter(bind_account__id__in=account_list).values_list('id', flat=True)
                res_device = list(device_list)
        elif range_type == 4:
            # 群聊处理
            group_chat_list = range_value.split(',')
            if group_chat_items.filter(groups_id__in=group_chat_list).exists():
                group_chat_filter_items = group_chat_items.filter(groups_id__in=group_chat_list)
                items_0_range = group_chat_filter_items.filter(range=0).values_list('org_id', flat=True)
                items_1_range = group_chat_filter_items.filter(range=1).values_list('device_id', flat=True)
                items_3_range = group_chat_filter_items.filter(range=3).values_list('account_id', flat=True)
                # 处理部门数据
                if device_items.filter(org_info_id__in=items_0_range).exists():
                    device_0_list = device_items.filter(org_info_id__in=items_0_range).values_list('id', flat=True)
                    device_0_list = list(device_0_list)
                else:
                    device_0_list = []
                # 处理设备信息
                device_1_list = list(items_1_range)
                # 处理账号信息
                if device_items.filter(bind_account__id__in=items_3_range).exists():
                    device_3_list = device_items.filter(bind_account__id__in=items_3_range).values_list('id', flat=True)
                    device_3_list = list(device_3_list)
                else:
                    device_3_list = []
                device_list = device_0_list + device_1_list + device_3_list
                res_device = list(set(device_list))
        else:
            res_device = []

        # 排除例外的情况
        if (exception_switch_type == 1) and (res_device is not None):  # 例外开关已开启
            if exception_range_type == 1:  # 例外范围为设备
                exception_range_value = exception_range_value.split(',')
                res_device = [int(i) for i in res_device]  # 强转整形
                exception_range_value = [int(i) for i in exception_range_value]  # 强转整形
                set1 = set(res_device)
                set2 = set(exception_range_value)
                # logging.info(f"set1:{set1},set2:{set2}")
                result_set = set1 - set2
                res_device = list(result_set)

        return res_device

    def get_install_total(self, obj):
        res_device = obj.device_range_value
        if res_device is not None:
            res_device = res_device.split(',')
        else:
            res_device = []
        res_device = [int(i) for i in res_device]
        # logging.info(f"{obj.id}.....{res_device}")
        patch_ids = obj.patch_ids
        patch_ids_list = patch_ids.split(',')
        patch_file_id_list = [int(i) for i in patch_ids_list]
        patch_id_list = models.PatchInfo.objects.filter(patch_is_delete=0).values_list('id', flat=True)
        device_id_list = Devices.objects.exclude(is_domain=0).values_list('id', flat=True)
        patch_obj_items = (
            models.PatchDeviceLog.objects.filter(device_id__in=device_id_list).
            filter(device_id__in=res_device).
            filter(patch_id__in=patch_id_list).
            filter(patch_id__in=patch_file_id_list)
        )

        apply_totals = patch_obj_items.values('device_id').distinct().count()
        apply_false_count = patch_obj_items.filter(install_status=0).values('device_id').distinct().count()
        apply_success_count = apply_totals - apply_false_count

        return f"{apply_success_count}/{apply_false_count}/{apply_totals}"

    def get_apply_status(self, obj):
        res_device = obj.device_range_value
        if res_device is not None:
            res_device = res_device.split(',')
        else:
            res_device = []
        res_device = [int(i) for i in res_device]
        patch_ids = obj.patch_ids
        patch_ids_list = patch_ids.split(',')
        patch_file_id_list = [int(i) for i in patch_ids_list]
        patch_obj_items = models.PatchDeviceLog.objects.filter(device_id__in=res_device).filter(
            patch_id__in=patch_file_id_list)

        apply_totals = patch_obj_items.values('device_id').distinct().count()
        apply_false_count = patch_obj_items.filter(install_status=0).values('device_id').distinct().count()
        apply_success_count = apply_totals - apply_false_count

        try:
            apply_status = "{:.2f}".format((apply_success_count / apply_totals) * 100)
        except ZeroDivisionError as exp:
            apply_status = "{:.2f}".format(0 * 100)

        return apply_status

    def get_patch_num(self, obj):
        patch_ids_list = obj.patch_ids.split(',')
        return len(patch_ids_list)


class PatchDeliverySettingsSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.PatchDeliverySettings
        fields = '__all__'
