from collections import UserList

from django.db import models

class Ou(models.Model):
    is_inheritable = models.BooleanField(default=False)
    creator = models.CharField(max_length=64, blank=True, null=True)
    create_time = models.DateTimeField(blank=True, null=True, auto_now_add=True)
    updater = models.CharField(max_length=64, blank=True, null=True)
    update_time = models.DateTimeField(blank=True, null=True, auto_now=True)

    class Meta:
        db_table = 'policy_ou'

class OuFilter(models.Model):
    class ExpressionChoices(models.IntegerChoices):
        AND = 0, 'AND'
        OR = 1, 'OR'
        NOT = 2, 'NOT'

    class ObjectTypeChoices(models.IntegerChoices):
        USER = 0, 'USER'
        USER_GROUP = 1, 'USER_GROUP'
        DEVICE = 10, 'DEVICE'
        DEVICE_GROUP = 11, 'DEVICE_GROUP'
        DEPARTMENT = 20, 'DEPARTMENT'

    ou = models.ForeignKey(Ou, null=True, on_delete=models.SET_NULL, related_name='filters', db_constraint=False)
    object_id = models.BigIntegerField(null=True, blank=True)
    object_type = models.PositiveSmallIntegerField(choices=ObjectTypeChoices.choices, null=True)
    expression = models.PositiveSmallIntegerField(choices=ExpressionChoices.choices, null=True)
    creator = models.CharField(max_length=64, blank=True, null=True)
    create_time = models.DateTimeField(blank=True, null=True, auto_now_add=True)
    updater = models.CharField(max_length=64, blank=True, null=True)
    update_time = models.DateTimeField(blank=True, null=True, auto_now=True)

    class Meta:
        db_table = 'policy_filter'

class Gpo(models.Model):
    class GpoType(models.TextChoices):
        USER = 'user', 'USER'
        MACHINE = 'machine', 'MACHINE'

    name = models.CharField(max_length=32, unique=True)
    type = models.CharField(choices=GpoType.choices, null=True, max_length=14)
    priority = models.IntegerField(blank=True, default=0)
    ou = models.ForeignKey(Ou, null=True, on_delete=models.SET_NULL, related_name='gpos', db_constraint=False)
    is_mandatory = models.BooleanField(default=False)
    status = models.PositiveSmallIntegerField(default=0)
    remark = models.CharField(blank=True, null=True, max_length=256)
    creator = models.CharField(max_length=64, blank=True, null=True)
    create_time = models.DateTimeField(blank=True, null=True, auto_now_add=True)
    update_time = models.DateTimeField(blank=True, null=True, auto_now=True)
    updater = models.CharField(max_length=64, blank=True, null=True)

    class Meta:
        db_table = 'policy_gpo'


class Policy(models.Model):
    name = models.CharField(max_length=32)
    type = models.CharField(max_length=32)
    path = models.CharField(max_length=255, default='/')
    value = models.CharField(max_length=255)
    opt_value = models.CharField(max_length=255, blank=True, null=True)
    is_disabled = models.IntegerField(blank=True, null=True)
    parent_id = models.BigIntegerField(blank=True, null=True)
    creator = models.CharField(max_length=64, blank=True, null=True)
    create_time = models.DateTimeField(blank=True, null=True, auto_now_add=True)
    updater = models.CharField(max_length=64, blank=True, null=True)
    update_time = models.DateTimeField(blank=True, null=True, auto_now=True)

    class Meta:
        db_table = 'policy_policy'


class GpoPolicy(models.Model):
    gpo = models.ForeignKey(Gpo, null=True, on_delete=models.SET_NULL, related_name='policies', db_constraint=False)
    # policy = models.ForeignKey(Policy, null=True, on_delete=models.SET_NULL, db_constraint=False)
    path = models.CharField(max_length=255, default='/')
    value = models.TextField(blank=True, null=True)
    creator = models.CharField(max_length=64, blank=True, null=True)
    create_time = models.DateTimeField(blank=True, null=True, auto_now_add=True)
    updater = models.CharField(max_length=64, blank=True, null=True)
    update_time = models.DateTimeField(blank=True, null=True, auto_now=True)
    
    class Meta:
        db_table = 'policy_gpo_policy'