import logging
import ipaddress

from rest_framework import serializers

import software
from devices.models import Devices
from groups.models import GroupsDetail
from policy.models import Gpo, Ou, OuFilter, GpoPolicy, Policy

logger = logging.getLogger(__name__)

class PolicyOuFilterSerializer(serializers.ModelSerializer):
    class Meta:
        model = OuFilter
        fields = '__all__'

class PolicyOuSerializer(serializers.ModelSerializer):
    filters = PolicyOuFilterSerializer(many=True)

    def update(self, instance, validated_data):
        filters = validated_data.pop('filters')
        instance.filters.all().delete()
        for f in filters:
            f['ou_id'] = instance.id
            PolicyOuFilterSerializer().create({**f})
        return super().update(instance, validated_data)

    def create(self, validated_data):
        filters = validated_data.pop('filters')
        instance = super().create(validated_data)
        for f in filters:
            f['ou_id'] = instance.id
            PolicyOuFilterSerializer().create({**f})
        return instance

    class Meta:
        model = Ou
        fields = '__all__'

class PolicySerializer(serializers.ModelSerializer):
    class Meta:
        model = Policy
        fields = '__all__'

class PolicyGpoPolicySerializer(serializers.ModelSerializer):
    # policy = PolicySerializer(read_only=True)

    class Meta:
        model = GpoPolicy
        fields = '__all__'

class PolicyGpoSerializer(serializers.ModelSerializer):
    ou = PolicyOuSerializer()
    policies = PolicyGpoPolicySerializer(many=True)

    def update(self, instance, validated_data):
        ou = validated_data.pop('ou')
        PolicyOuSerializer().update(instance.ou, ou)
        policies = validated_data.pop('policies')
        instance.policies.all().delete()
        for policy in policies:
            policy['gpo_id'] = instance.id
            PolicyGpoPolicySerializer().create({**policy})
        return super().update(instance, validated_data)

    def create(self, validated_data):
        ou = validated_data.pop('ou')
        policies = validated_data.pop('policies')
        ou = PolicyOuSerializer().create(ou)
        validated_data['ou_id'] = ou.id
        instance = super().create(validated_data)
        for policy in policies:
            policy['gpo_id'] = instance.id
            PolicyGpoPolicySerializer().create({**policy})
        return instance

    class Meta:
        model = Gpo
        fields = '__all__'

