import ast
import json

### 系统配置项--相关策略解析
SYSTEM_CONFIG_USER_POLICY_PATH_PREFIX = '/policies/user/systemconfig/'
SYSTEM_CONFIG_SYSTEM_POLICY_PATH_PREFIX = '/policies/system/systemconfig/'

# 系统更新默认配置
sys_updates_default_dict = {
    "system_update_switch": 0,
    "update_style": 0,
    "check_time_days": 0,
    "check_time_time": 00,
    "download_update_method": 0
}

def translate_systemconfig_policy(policies, policy_type = 'system'):
    """
    系统配置项,策略值解析和转换：将策略的'键值对'转换为saltapi需要的格式
    """
    res = None
    # 判断类型
    if policy_type == 'user':
        path = SYSTEM_CONFIG_USER_POLICY_PATH_PREFIX
    elif policy_type == 'system':
        path = SYSTEM_CONFIG_SYSTEM_POLICY_PATH_PREFIX
    else:
        raise Exception('Wrong policy_type')
    # 初始化
    res = {
        'wallpaper': None,
        'screensavers': None,
        'sys_updates': None
    }
    # 遍历gpo_policy策略,根据key拼装出最终的结果
    for policy in policies:
        if not policy.path.startswith(path):
            continue
        # 分析各个策略
        key = policy.path[len(path):]
        value = policy.value
        # 背景(壁纸)策略
        if key.startswith('wallpaper'):
            # 默认值
            if res['wallpaper'] is None:
                res['wallpaper'] = {
                    'active': 0,
                    'wallpaper_id': 0
                }
            # lstrip对于'wallpaper/wallpaper_xxxxx',只能分开操作
            key = key.lstrip('wallpaper').lstrip('/')
            if key == 'enable':
                if value == 'true':
                    res['wallpaper']['active'] = 1
            elif key == 'wallpaper_id':
                res['wallpaper']["wallpaper_id"] = value
        # 锁屏(屏幕保护)策略
        elif key.startswith('screensavers'):
            # 默认值
            if res['screensavers'] is None:
                res['screensavers'] = {
                    'lock_screen_time': None,
                    'screen_saver_time': None,
                    'lock_activate_screensaver': 0,
                    'lock_sleep_screensaver': 0,
                    'use_for_greeter': 0,
                    'lock_wallpaper_switch': 0,
                    'lock_wallpaper_id': None,
                    'screensaver_cycle_switch':0,
                    'screensaver_folder_id': None,
                    'screensaver_cycle_time': None,
                }
            key = key[len('screensavers/'):]
            # 锁屏时间(单位:秒)
            if key == 'lock_screen_time':
                res['screensavers']["lock_screen_time"] = value
            # 屏幕保护时间(单位:分钟)
            elif key == 'screen_saver_time':
                res['screensavers']["screen_saver_time"] = value
            # 激活屏保时锁屏(开关)
            elif key == 'lock_activate_screensaver':
                if value == 'true':
                    res['screensavers']["lock_activate_screensaver"] = 1
            # 睡眠时锁屏(开关)
            elif key == 'lock_sleep_screensaver':
                if value == 'true':
                    res['screensavers']["lock_sleep_screensaver"] = 1
            # 锁屏壁纸(开关)
            elif key == 'lock_wallpaper_switch':
                if value == 'true':
                    res['screensavers']["lock_wallpaper_switch"] = 1
            # 锁屏壁纸
            elif key == 'lock_wallpaper_id':
                res['screensavers']["lock_wallpaper_id"] = value
            # 锁屏壁纸添加到登录界面(开关)
            elif key == 'use_for_greeter':
                if value == 'true':
                    res['screensavers']["use_for_greeter"] = 1
            # 屏幕保护轮播(开关)
            elif key == 'screensaver_cycle_switch':
                if value == 'true':
                    res['screensavers']["screensaver_cycle_switch"] = 1
            # 屏幕保护轮播图片文件夹
            elif key == 'screensaver_folder_id':
                res['screensavers']["screensaver_folder_id"] = value
            # 屏幕保护轮播时间(单位:秒)
            elif key == 'screensaver_cycle_time':
                res['screensavers']["screensaver_cycle_time"] = value
        # 系统更新策略
        elif key.startswith('sys_updates'):
            # 默认值
            if res['sys_updates'] is None:
                res['sys_updates'] = sys_updates_default_dict
            key = key[len('sys_updates/'):]
            if key == 'system_update_switch':
                if value == 'true':
                    res['sys_updates']["system_update_switch"] = 1
            elif key == 'update_style':
                if value == 'true':
                    res['sys_updates']["update_style"] = 1
            elif key == 'check_time_days':
                res['sys_updates']["check_time_days"] = value
            elif key == 'check_time_time':
                res['sys_updates']["check_time_time"] = value.strip("\"")
            elif key == 'download_update_method':
                res['sys_updates']["download_update_method"] = value
    # 返回最终结果
    return res

### 系统工具(系统激活，水印)--相关配置解析
SYSTEM_TOOLS_USER_POLICY_PATH_PREFIX = '/policies/user/systemtools/'
SYSTEM_TOOLS_SYSTEM_POLICY_PATH_PREFIX = '/policies/system/systemtools/'

# 水印默认配置， 当取消水印时使用此配置，透明度为0
watermark_default_dict = {
    "watermark_switch_status": 0,
    "user_defined_text": "0000",  # 固定0000
    "watermark_position": "全屏",
    "watermark_rotate": 30,
    "watermark_font": "黑体",
    "watermark_font_size": 50,
    "watermark_font_color": '7f7f7f',
    "watermark_font_transparency": 00,  # 固定00
    "watermark_defined_text_type": [0], #0-6,共7个,分别对应用户名、设备别名、设备主机名、设备IP、设备MAC、设备当前时间、用户自定义
}

def translate_systemtools_policy(policies, policy_type = 'system'):
    """
    系统工具策略值解析和转换：将策略的'键值对'转换为saltapi需要的格式
    """
    res = None
    # 判断类型
    if policy_type == 'user':
        path = SYSTEM_TOOLS_USER_POLICY_PATH_PREFIX
    elif policy_type == 'system':
        path = SYSTEM_TOOLS_SYSTEM_POLICY_PATH_PREFIX
    else:
        raise Exception('Wrong policy_type')
    # 初始化
    res = {
        'activation': None,
        'watermark': None
    }
    # 遍历gpo_policy策略,根据key拼装出最终的结果
    for policy in policies:
        if not policy.path.startswith(path):
            continue
        key = policy.path[len(path):]
        value = policy.value
        # 如果是系统激活的设置
        if key.startswith('activation'):
            # 默认值
            if res['activation'] is None:
                res['activation'] = {
                    'active': 0,
                    'kms_server': None
                }
            key = key[len('activation/'):]
            if key == 'enable':
                if value == 'true':
                    res['activation']['active'] = 1
            elif key == 'kms_server':
                res['activation']["kms_server"] = value
        # 如果是水印设置
        elif key.startswith('watermark'):
            # 默认值
            if res['watermark'] is None:
                res['watermark'] = watermark_default_dict
            key = key[len('watermark/'):]
            if key == 'watermark_switch_status':
                # 当水印设置打开时更改配置值
                if value == 'true':
                    res['watermark']["watermark_switch_status"] = 1
            # 其他属性
            elif key == 'user_defined_text1':
                # 此时value是一个数组字符串
                watermark_defined_text_type = []
                value_array = json.loads(value)
                for item in value_array:
                    if item == 'require_account_name':
                        watermark_defined_text_type.append(0)
                    elif item == 'require_device_cnname':
                        watermark_defined_text_type.append(1)
                    elif item == 'require_device_hostname':
                        watermark_defined_text_type.append(2)
                    elif item == 'require_device_ip':
                        watermark_defined_text_type.append(3)
                    elif item == 'require_device_mac':
                        watermark_defined_text_type.append(4)
                    elif item == 'require_device_current_time':
                        watermark_defined_text_type.append(5)
                    elif item == 'custom':
                        watermark_defined_text_type.append(6)
                res['watermark']["watermark_defined_text_type"] = watermark_defined_text_type
            elif key == 'user_defined_text':
                res['watermark']["user_defined_text"] = value
            elif key == 'watermark_position':
                res['watermark']["watermark_position"] = value
            elif key == 'watermark_rotate':
                res['watermark']["watermark_rotate"] = value
            elif key == 'watermark_font':
                res['watermark']["watermark_font"] = value
            elif key == 'watermark_font_size':
                res['watermark']["watermark_font_size"] = value
            elif key == 'watermark_font_color':
                res['watermark']["watermark_font_color"] = value
            elif key == 'watermark_font_transparency':
                res['watermark']["watermark_font_transparency"] = float(value) if isinstance(value, str) else 0
    # 返回最终结果
    return res

### 外设管控(USB管控)--相关配置解析
PERIPHERAL_USER_POLICY_PATH_PREFIX = '/policies/user/peripheral/'
PERIPHERAL_SYSTEM_POLICY_PATH_PREFIX = '/policies/system/peripheral/'

def translate_peripheral_policy(policies, policy_type = 'system'):
    """
    系统工具策略值解析和转换：将策略的'键值对'转换为saltapi需要的格式
    """
    res = None
    # 判断类型
    if policy_type == 'user':
        path = PERIPHERAL_USER_POLICY_PATH_PREFIX
    elif policy_type == 'system':
        path = PERIPHERAL_SYSTEM_POLICY_PATH_PREFIX
    else:
        raise Exception('Wrong policy_type')
    # 初始化
    res = {
        'usb': None
    }
    for policy in policies:
        #logger.info(f"machine.peripheral_policy.policy={policy}")
        if not policy.path.startswith(path):
            continue
        key = policy.path[len(path):]
        value = policy.value
        # 如果是系统激活的设置
        if key.startswith('usb'):
            # 默认值
            if res['usb'] is None:
                res['usb'] = {
                    'enable': 0
                }
            key = key[len('usb/'):]
            if key == 'enable':
                if value == 'true':
                    res['usb']['enable'] = 1
    return res

### root管控--相关配置解析
ROOT_USER_POLICY_PATH_PREFIX = '/policies/user/root/'
ROOT_SYSTEM_POLICY_PATH_PREFIX = '/policies/system/root/'

def translate_root_policy(policies, policy_type = 'system'):
    """
    root策略值解析和转换：将策略的'键值对'转换为saltapi需要的格式
    """
    # 判断类型
    if policy_type == 'user':
        path = ROOT_USER_POLICY_PATH_PREFIX
    elif policy_type == 'system':
        path = ROOT_SYSTEM_POLICY_PATH_PREFIX
    else:
        raise Exception('Wrong policy_type')
    # 初始化
    res = {}
    for policy in policies:
        if not policy.path.startswith(path):
            continue
        res[policy.path.rsplit('/')[-1]] = policy.value
    if 'enable' not in res:
        return None
    return res