from django.db import models
from devices.models import Devices


class StatusEnum(models.IntegerChoices):
    NULL = 0, 'Null'
    SUCCESS = 1, 'Success'
    GOING = 2, 'Going'
    FAIL = 3, 'Fail'


class EnabledEnum(models.IntegerChoices):
    NO = 0, 'No'
    YES = 1, 'Yes'


class RemoteAssist(models.Model):
    argee = models.IntegerField(verbose_name='远程协助用户确认需求, 0 关 1开, 默认值1', choices=EnabledEnum.choices,
                                default=EnabledEnum.NO, blank=True)
    concat = models.IntegerField(verbose_name='远程协助自动重新连接, 0 关 1开, 默认值0', choices=EnabledEnum.choices,
                                 default=EnabledEnum.YES, blank=True)
    concat_time = models.IntegerField(verbose_name='重连间隔时间, 默认值5', default=5, blank=True)
    sessions = models.IntegerField(verbose_name='远程协助空闲会话确认, 0 关 1开 , 默认值1', choices=EnabledEnum.choices,
                                   default=EnabledEnum.NO, blank=True)
    sessions_time = models.IntegerField(verbose_name='远程协助空闲会话时间, 范围1-120, 默认值5', default=5, blank=True)
    need_passwd = models.IntegerField(verbose_name='要求输入密码', choices=EnabledEnum.choices, default=EnabledEnum.NO,
                                      blank=True, null=True)
    passwd = models.CharField(blank=True, max_length=64, null=True)
    status = models.IntegerField(choices=StatusEnum.choices, default=StatusEnum.NULL, blank=True, null=True)
    enabled = models.IntegerField(choices=EnabledEnum.choices, default=EnabledEnum.YES, blank=True, null=True)
    global_enabled = models.IntegerField(choices=EnabledEnum.choices, default=EnabledEnum.YES, blank=True, null=True)

    class Meta:
        db_table = 'remote_assist'
        verbose_name = '远程规则'
        verbose_name_plural = '远程规则'
        ordering = ('-id',)

    def __str__(self):
        return self.pk


class UserRemoteAssistDevice(models.Model):
    # 用户远程连接设备表，保证一个用户同一时间只能连接一台终端
    delete_choice = (
        (0, 'N'), (1, 'Y')
    )
    user = models.ForeignKey(to='users.User', on_delete=models.SET(1), db_constraint=False, null=True,
                             blank=True, verbose_name='管理员')
    device = models.ForeignKey(to='devices.Devices', on_delete=models.PROTECT, db_constraint=False, null=True,
                               blank=True, verbose_name='终端')
    is_delete = models.IntegerField(verbose_name='是否刪除', choices=delete_choice, default=0, blank=True)

    class Meta:
        db_table = 'user_remote_assist_device'
        verbose_name = '用户远程连接设备'
        verbose_name_plural = '用户远程连接设备'
        ordering = ('-id',)

    def __str__(self):
        return self.pk


class RemoteAssistDevice(models.Model):
    argee = models.IntegerField(choices=EnabledEnum.choices, default=EnabledEnum.NO, blank=True, null=True)
    concat = models.IntegerField(choices=EnabledEnum.choices, default=EnabledEnum.YES, blank=True, null=True)
    concat_time = models.IntegerField(blank=True, null=True)
    sessions = models.IntegerField(choices=EnabledEnum.choices, default=EnabledEnum.NO, blank=True, null=True)
    sessions_time = models.IntegerField(default=5, blank=True, null=True)
    need_passwd = models.IntegerField(choices=EnabledEnum.choices, default=EnabledEnum.NO, blank=True, null=True)
    passwd = models.CharField(blank=True, max_length=64, null=True)
    status = models.IntegerField(choices=StatusEnum.choices, default=StatusEnum.NULL, blank=True, null=True)
    enabled = models.IntegerField(choices=EnabledEnum.choices, default=EnabledEnum.YES, blank=True, null=True)
    global_enabled = models.IntegerField(choices=EnabledEnum.choices, default=EnabledEnum.YES, blank=True, null=True)
    device = models.OneToOneField(Devices, related_name='remote_assist', unique=True, null=True,
                                  on_delete=models.SET_NULL, db_constraint=False)
    policy_no = models.CharField(verbose_name='策略编号', max_length=255, null=True, blank=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'remote_assist_device'
        verbose_name = '远程协助配置表'
