import logging

from rest_framework import serializers

from devices.serializers import DeviceSerializer
from remote_assist.models import RemoteAssist, RemoteAssistDevice, StatusEnum

logger = logging.getLogger(__name__)


class RemoteAssistSerializer(serializers.ModelSerializer):
    class Meta:
        model = RemoteAssist
        fields = '__all__'


class RemoteAssistDeviceSerializer(serializers.ModelSerializer):
    class Meta:
        model = RemoteAssistDevice
        fields = '__all__'


class RADeviceSerializer(DeviceSerializer):
    remote_assist = RemoteAssistDeviceSerializer()

    class Meta(DeviceSerializer.Meta):
        fields = DeviceSerializer.Meta.fields + ('remote_assist',)

    # def to_representation(self, instance):
    #     representation = super().to_representation(instance)
    #     if isinstance(representation.get('request_object'), OrderedDict):
    #         representation['remote_assist'] = dict(representation['remote_assist'])
    #     return representation

    def update(self, instance, validated_data):
        data = validated_data.get('remote_assist', {})
        if data:
            remote_assist, _ = RemoteAssistDevice.objects.get_or_create(device=instance)
            remote_assist.argee = data.get('argee', remote_assist.argee)
            remote_assist.concat = data.get('concat', remote_assist.concat)
            remote_assist.concat_time = data.get('concat_time', remote_assist.concat_time)
            remote_assist.sessions = data.get('sessions', remote_assist.sessions)
            remote_assist.sessions_time = data.get('sessions_time', remote_assist.sessions_time)
            remote_assist.need_passwd = data.get('need_passwd', remote_assist.need_passwd)
            remote_assist.passwd = data.get('passwd', remote_assist.passwd)
            remote_assist.status = StatusEnum.GOING
            remote_assist.save()
            instance.remote_assist = remote_assist

        return instance

    def create(self, validated_data):
        data = validated_data.get('remote_assist', {})
        instance = data['device']
        if data:
            remote_assist = RemoteAssistDevice.objects.create(device=instance)
            remote_assist.argee = data.get('argee', remote_assist.argee)
            remote_assist.concat = data.get('concat', remote_assist.concat)
            remote_assist.concat_time = data.get('concat_time', remote_assist.concat_time)
            remote_assist.sessions = data.get('sessions', remote_assist.sessions)
            remote_assist.sessions_time = data.get('sessions_time', remote_assist.sessions_time)
            remote_assist.need_passwd = data.get('need_passwd', remote_assist.need_passwd)
            remote_assist.passwd = data.get('passwd', remote_assist.passwd)
            remote_assist.status = StatusEnum.GOING
            remote_assist.save()
            instance.remote_assist = remote_assist

        return instance
