import base64
import logging
from datetime import datetime

from devices.client_consumers import msg_to_client
from remote_assist.models import RemoteAssistDevice, RemoteAssist
from remote_assist.utils import get_redis_device_info

logger = logging.getLogger(__name__)

def remote_assist_policy_to_device(device, data):
    data_json = {
        "sender": "SERVER",
        "sn": "10-1-1",
        "data": {
            'enabled': data.get('enabled', None),
            'argee': data.get('argee', None),
            'needpasswd': data.get('need_passwd', None),
            'timestamp': datetime.now().timestamp(),
        }
    }
    passwd = data.get('passwd', None)
    if passwd:
        data_json['data']['passwd'] = base64.b64encode(passwd.encode("utf-8")).decode("utf-8")
    channel_group_name = get_redis_device_info(device)
    if not channel_group_name:
        # 无 websocket
        logger.error('no channel_group_name')
    else:
        # 发送消息
        logger.info(data_json)
        msg_to_client(1, channel_group_name, data_json)

def get_device_remote_assist_policy(device):
    remote_assist_device = RemoteAssistDevice.objects.filter(device=device).first()
    remote_assist_default = RemoteAssist.objects.filter(pk=1).first()
    return remote_assist_device if remote_assist_device else remote_assist_default
