import subprocess
import hashlib
import base64

from xmlrpc.server import SimpleXMLRPCServer
from socketserver import ThreadingMixIn
from functools import wraps
from concurrent.futures import ThreadPoolExecutor


THREAD_POOL = ThreadPoolExecutor()


class ThreadXMLRPCServer(ThreadingMixIn, SimpleXMLRPCServer): pass



def rpc_response(code, data):
    res = {'code':code,'data':data}
    return res

def auth_rpc_permission(func):
    @wraps(func)
    def is_auth(*args, **kwargs):
        if not args:
            return rpc_response(400, 'not permission')
        if args[-0] != 'nfs':
            return rpc_response(400, 'not permission')
        return func(*args)
    return is_auth



def calculate_md5(filename):
    with open(filename, 'rb') as f:
        md5 = hashlib.md5()
        while True:
            data = f.read(4096)
            if not data:
                break
            md5.update(data)
        return md5.hexdigest()

@auth_rpc_permission
def send_file(auth, filename, data):
    with open(f'/tmp/{filename}', 'wb') as f:
        f.write(data.data)
    return calculate_md5(f'/tmp/{filename}')



if __name__ == '__main__':
    print('*** start reprepro rpc *** ')
    # from .rpc import RpcView, ThreadXMLRPCServer, get_agent
    # rpcview = RpcView()
    server = ThreadXMLRPCServer(("0.0.0.0", 25252), allow_none=True)
    server.register_function(send_file)
    server.serve_forever()