import subprocess
import sys
import datetime

from xmlrpc.server import SimpleXMLRPCServer
from socketserver import ThreadingMixIn
from functools import wraps
from concurrent.futures import ThreadPoolExecutor


THREAD_POOL = ThreadPoolExecutor()


class ThreadXMLRPCServer(ThreadingMixIn, SimpleXMLRPCServer): pass



def rpc_response(code, data):
    res = {'code':code,'data':data}
    return res

def auth_rpc_permission(func):
    @wraps(func)
    def is_auth(*args, **kwargs):
        if not args:
            return rpc_response(400, 'not permission')
        if args[-0] != 'nfs':
            return rpc_response(400, 'not permission')
        return func(*args)
    return is_auth


def ntp_rpc(ntp):
    cmd_args = ['bash', 'ntp.sh', ntp]
    result = subprocess.run(cmd_args, capture_output=True, text=True)
    # 打印脚本的输出
    print(result.stdout)
    # 打印脚本的错误信息
    print(result.stderr)
    # 打印脚本的返回码
    print("返回码:", result.returncode)

@auth_rpc_permission
def ntp_rpc_interface(auth, ntp):
    THREAD_POOL.submit(ntp_rpc, ntp)
    return rpc_response(200, {'success':True})




if __name__ == '__main__':
    print('*** start rpc ')

    if len(sys.argv) == 1:
        port = '5252'
    else:
        port = sys.argv[-1]
    if not port.isdigit():
        raise Exception('端口号错误')
    print('*** rpc server port : {} '.format(port))
    # from .rpc import RpcView, ThreadXMLRPCServer, get_agent
    # rpcview = RpcView()
    server = ThreadXMLRPCServer(("0.0.0.0", int(port)), allow_none=True)

    server.register_function(ntp_rpc_interface)
    server.serve_forever()