import datetime
import os
import shutil
import subprocess

from xmlrpc.server import SimpleXMLRPCServer
from socketserver import ThreadingMixIn
from functools import wraps
from concurrent.futures import ThreadPoolExecutor

THREAD_POOL = ThreadPoolExecutor()


class ThreadXMLRPCServer(ThreadingMixIn, SimpleXMLRPCServer): pass


def rpc_response(code, data):
    res = {'code': code, 'data': data}
    return res


def auth_rpc_permission(func):
    @wraps(func)
    def is_auth(*args, **kwargs):
        if not args:
            return rpc_response(400, 'not permission')
        if args[-0] != 'nfs':
            return rpc_response(400, 'not permission')
        return func(*args)

    return is_auth


# def remove_ssh_host(host):
#     cmd_args = f'ssh-keygen -f "/root/.ssh/known_hosts" -R "{host}"'
#     result = subprocess.run(cmd_args, capture_output=True, text=True)
#     # 打印脚本的输出
#     print(result.stdout)
#     # 打印脚本的错误信息
#     print(result.stderr)
#     # 打印脚本的返回码
#     print("返回码:", result.returncode)
#     return {'stdout':result.stdout, 'stderr':result.stderr, 'returncode':result.returncode}
@auth_rpc_permission
def move_file(auth, source_path, destination_path):
    """
    :param source_path: 为实际的文件路径
    :param destination_path: 为目标目录路径
    :return:
    """
    try:
        # 使用 shutil.move 函数移动文件
        shutil.move(source_path, destination_path)
        print(f"File moved successfully from {source_path} to {destination_path}")
        return 200, None
    except Exception as e:
        print(f"Error: {e}")
        return 400, e


@auth_rpc_permission
def delete_old_log_files(auth, directory):
    """
    清除以nfs_domain开头15天以上的文件
    :param directory:
    :return:
    """
    current_date = datetime.datetime.now()
    threshold_date = current_date - datetime.timedelta(days=15)

    for filename in os.listdir(directory):
        if filename.startswith("nfs_domain_") and filename.endswith(".tar.gz"):
            file_path = os.path.join(directory, filename)
            creation_time = datetime.datetime.fromtimestamp(os.path.getctime(file_path))

            if creation_time < threshold_date:
                os.remove(file_path)
                print(f"Deleted old file: {file_path}")

    return 200, 'success'


@auth_rpc_permission
def remove_ssh_host_interface(auth, host):
    # THREAD_POOL.submit(remove_ssh_host, host)
    # cmd_args = f'ssh-keygen -f "/root/.ssh/known_hosts" -R "{host}"'
    cmd_args = ['ssh-keygen', '-f', '/root/.ssh/known_hosts', '-R', host]
    result = subprocess.run(cmd_args, capture_output=True, text=True)
    # 打印脚本的输出
    # print(result.stdout)
    # 打印脚本的错误信息
    # print(result.stderr)
    # 打印脚本的返回码
    # print("返回码:", result.returncode)
    return rpc_response(200, {'stdout': result.stdout, 'stderr': result.stderr, 'returncode': result.returncode})


if __name__ == '__main__':
    print('*** start salt rpc *** ')
    # from .rpc import RpcView, ThreadXMLRPCServer, get_agent
    # rpcview = RpcView()
    server = ThreadXMLRPCServer(("0.0.0.0", 15252), allow_none=True)
    server.register_function(remove_ssh_host_interface)
    server.register_function(move_file, 'move_file')
    server.register_function(delete_old_log_files, 'delete_old_log_files')
    server.serve_forever()
