# -*- coding: utf-8 -*-
import logging
import json
import uuid
from PIL.JpegImagePlugin import JpegImageFile
from django.db import models, connection
from django.conf import settings
from django.db.models.fields.files import ImageFieldFile
from devices.models import Devices

logger = logging.getLogger(__name__)


# # Create your models here.
#
#
# class PhotosProxy(models.Model):
#     @classmethod
#     def get_sharding_model(cls, user_name=None):
#
#         class Meta:
#             db_table = 'Photos_%s' % user_name
#         attrs = {
#             '__module__': cls.__module__,
#             '__doc__': 'using Photos_%s table' % user_name,
#             'Meta': Meta
#         }
#         return type(str('Photos%s' % user_name), (cls, ), attrs)
#
#     id = models.AutoField(primary_key=True)
#     owner = models.CharField(max_length=20)
#     name = models.CharField(max_length=256)
#
#
# new_N = PhotosProxy.get_sharding_model(user_name)
# with connection.schema_editor() as schema_editor:
#     try:
#         schema_editor.create_model(new_N)
#     except:
#         print('exist')
#
# print(new_N)
# d0 = new_N()
# d0.owner = user_name
# new_N.save(d0)
#
# Photos = PhotosProxy.get_sharding_model(owner)
# userResult = Photos.objects.all()


# class SaltPolicy(models.Model):
#     is_up = (
#         (0, 'down'), (1, 'up')
#     )
#     strategy_name = models.CharField(verbose_name='策略描述', max_length=255, null=True, blank=True)
#     strategy_addr = models.FileField(verbose_name='策略位置', upload_to='salt/policy', null=True, max_length=225)
#     strategy_func = models.CharField(verbose_name='策略调用方法', max_length=225, null=True)
#     strategy_up = models.IntegerField(verbose_name='是否启用', choices=is_up, default=1, blank=True)
#     created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
#
#     class Meta:
#         db_table = 'salt_policy'
#         verbose_name = '策略'
#         verbose_name_plural = '策略'
#
#     def __str__(self):
#         return self.strategy_name


class SSHDevicePolicy(models.Model):
    on_off = (
        (0, 'off'), (1, 'on')
    )
    state = (
        (0, 'null'), (1, 'success'), (2, 'going'), (3, 'fail')
    )
    status = models.IntegerField(verbose_name='状态', choices=state, default=0, blank=True)
    ssh = models.IntegerField(verbose_name='是否开启', choices=on_off, default=0, blank=True)
    policy_no = models.CharField('策略编号', max_length=255, null=True, blank=True)
    device = models.OneToOneField(verbose_name='设备', to=Devices, to_field='id', unique=True, related_name='sshdp',
                                  on_delete=models.CASCADE)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'device_ssh'
        verbose_name = '设备ssh策略'
        verbose_name_plural = '设备ssh策略'


class CaDevicePolicy(models.Model):
    state = (
        (0, 'null'), (1, 'success'), (2, 'going'), (3, 'fail')
    )
    status = models.IntegerField(verbose_name='状态', choices=state, default=0, blank=True)
    ca_name = models.CharField(verbose_name='ca证书名称', max_length=255, blank=True, null=True)
    policy_no = models.CharField('策略编号', max_length=255, null=True, blank=True)
    device_id = models.IntegerField(verbose_name='设备ID', blank=True, null=False)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'device_ca'
        verbose_name = '设备CA证书策略'
        verbose_name_plural = '设备Ca证书策略'


class USBDevicePolicy(models.Model):
    on_off = (
        (0, 'off'), (1, 'on')
    )
    state = (
        (0, 'null'), (1, 'success'), (2, 'going'), (3, 'fail')
    )
    status = models.IntegerField(verbose_name='状态', choices=state, default=0, blank=True)
    usb = models.IntegerField(verbose_name='是否开启', choices=on_off, default=0, blank=True)
    usb_rw_status = models.CharField(verbose_name='usb读写权限控制', blank=True, max_length=64, null=True)
    org_info_id = models.IntegerField(verbose_name='部门id', blank=True, null=True)
    org_usb_rw = models.CharField(verbose_name='部门usb读写权限控制', blank=True, max_length=64, null=True)
    policy_no = models.CharField('策略编号', max_length=255, null=True, blank=True)
    device = models.OneToOneField(verbose_name='设备', to=Devices, to_field='id', unique=True, related_name='usbdp',
                                  on_delete=models.CASCADE)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'device_usb'
        verbose_name = '设备usb策略'
        verbose_name_plural = '设备usb策略'

class LocalUserPWDPolicy(models.Model):
    state = (
        (0, 'null'), (1, 'success'), (2, 'going'), (3, 'fail')
    )
    status = models.IntegerField(verbose_name='状态', choices=state, default=0, blank=True)
    user = models.CharField(verbose_name='本地用户', blank=True, max_length=64, null=True)
    policy_no = models.CharField('策略编号', max_length=255, null=True, blank=True)
    device = models.OneToOneField(verbose_name='设备', to=Devices, to_field='id', unique=True,on_delete=models.CASCADE)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'local_user_pwd'
        verbose_name = '本地用户密码策略'
        verbose_name_plural = '本地用户密码策略'


class ActivationDevicePolicy(models.Model):
    is_active = (
        (0, 'y'), (1, 'n')
    )
    state = (
        (0, 'fail'), (1, 'success'), (2, 'going')
    )
    status = models.IntegerField(verbose_name='状态', choices=state, default=0, blank=True)
    active = models.IntegerField(verbose_name='是否激活', choices=is_active, default=0, blank=True)
    kms_server = models.CharField(verbose_name='激活服务器', max_length=255, blank=True, null=True)
    device = models.OneToOneField(verbose_name='设备', to=Devices, to_field='id', unique=True, on_delete=models.CASCADE)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'device_active'
        verbose_name = '设备激活策略'
        verbose_name_plural = '设备激活策略'


class FirewalldDevicePolicy(models.Model):
    on_off = (
        (0, 'off'), (1, 'on')
    )
    level = (
        (0, '初级'), (1, '中级'), (2, '高级'), (3, '自定义')
    )
    state = (
        (0, 'null'), (1, 'success'), (2, 'going'), (3, 'fail')
    )
    status = models.IntegerField(verbose_name='状态', choices=state, default=0, blank=True)
    firewalld = models.IntegerField(verbose_name='是否开启', choices=on_off, default=0, blank=True)
    firewalld_level = models.IntegerField(verbose_name='防火墙级别', choices=level, default=1, blank=True)
    policy_no = models.CharField('策略编号', max_length=255, null=True, blank=True)
    device = models.OneToOneField(verbose_name='设备', to=Devices, to_field='id', unique=True, related_name='fwdp',
                                  on_delete=models.CASCADE)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'device_firewalld'
        verbose_name = '设备firewalld策略'
        verbose_name_plural = '设备firewalld策略'


class FirewalldRulesPolicy(models.Model):
    level_choices = (
        (1, '预配置'), (2, '中级'), (3, '高级')
    )
    protocol_choices = (
        (1, 'HTTP'),
        (2, 'HTTPS'),
        (3, 'TFP'),
        (4, 'IMAP'),
        (5, 'NFS'),
        (6, 'POP3'),
        (7, 'Samba'),
        (8, 'SSH'),
        (9, 'CUPS'),

        (10, 'TCP'),
        (11, 'UDP'),
        (12, 'BOTH'),
    )

    inp_outp_choices = (
        (1, '传入'), (2, '传出')
    )

    ip_type_choices = (
        (1, 'ipv4'), (2, 'ipv6')
    )

    option_choices = (
        (1, '允许'),
        (2, '禁止'),
        (3, '拒绝'),
        (4, '限制'),
    )

    level = models.CharField(verbose_name='规则级别', max_length=255, blank=True, null=True)
    protocol = models.CharField(verbose_name='通信协议', max_length=255, blank=True, null=True)
    state = models.CharField(verbose_name='禁用状态', max_length=255, blank=True, null=True)
    in_out = models.CharField(verbose_name='传入传出', max_length=255, blank=True, null=True)
    ip_type = models.CharField(verbose_name='传入传出', max_length=255, blank=True, null=True)
    external_addr = models.CharField(verbose_name='外部地址', blank=True, max_length=255, null=True)
    local_port = models.CharField(verbose_name='本地端口', blank=True, max_length=255, null=True)
    external_port = models.CharField(verbose_name='外部端口', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'firewalld_rules'
        verbose_name = '防火墙规则'
        verbose_name_plural = '防火墙规则'

    def access_control(self):
        if self.in_out == '传入':
            if self.state == '允许':
                return '允许入站'
            if self.state == '禁止':
                return '禁止入站'
            if self.state == '拒绝':
                return '拒绝入站'
            if self.state == '限制':
                return '限制入站'
        else:
            if self.state == '允许':
                return '允许连出'
            if self.state == '禁止':
                return '禁止连出'
            if self.state == '拒绝':
                return '拒绝连出'
            if self.state == '限制':
                return '限制连出'
        return ''


class FirewalldWhiteBlackRosterPolicy(models.Model):
    roster_choices = (
        (0, 'White'),
        (1, 'Black'),
        (2, 'Off'),
    )
    roster = models.IntegerField(verbose_name='状态', choices=roster_choices, default=2, blank=True)
    site = models.TextField(verbose_name='站点', null=True)

    # is_del = models.IntegerField(verbose_name='是否删除', default=0, blank=True, null=True)

    class Meta:
        db_table = 'firewalld_white_black_roster'
        verbose_name = '防火墙黑白名单'
        verbose_name_plural = '防火墙黑白名单'


class FirewalldWhiteBlackRosterDevicePolicy(models.Model):
    state = (
        (0, 'null'), (1, 'success'), (2, 'going'), (3, 'fail')
    )
    status = models.IntegerField(verbose_name='状态', choices=state, default=0, blank=True)
    device = models.OneToOneField(verbose_name='设备', to=Devices, to_field='id', unique=True,
                                  on_delete=models.CASCADE)
    policy_no = models.CharField('策略编号', max_length=255, null=True, blank=True)
    roster = models.TextField(verbose_name='站点', null=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'firewalld_white_black_roster_device'
        verbose_name = '设备防火墙黑白名单'
        verbose_name_plural = '设备防火墙黑白名单'


class WaitDevicePolicy(models.Model):
    device_id = models.IntegerField(verbose_name='设备ID', blank=True, null=False)
    # device_ip = models.CharField(verbose_name='设备IP', blank=True, max_length=64, null=False)
    policy = models.CharField(verbose_name='策略', blank=True, max_length=255, null=False)
    arg = models.TextField(verbose_name='策略参数', blank=True, null=True)

    class Meta:
        db_table = 'device_wait_policy'
        verbose_name = '设备未执行的策略'
        verbose_name_plural = '设备未执行的策略'


class Wallpaper(models.Model):
    name = models.CharField(verbose_name='壁纸名', blank=True, max_length=255, null=True)
    is_default = models.IntegerField(verbose_name='是否默认为壁纸', blank=True, default=0)
    image = models.CharField(verbose_name='壁纸路径', max_length=255, blank=True, null=False)

    @property
    def image_name(self):
        return self.name

    @property
    def image_routing(self):
        return '/{}{}'.format(settings.WALLPAPER_URL, self.name)

    class Meta:
        db_table = 'wallpaper'
        verbose_name = '壁纸'
        verbose_name_plural = verbose_name


class WallpaperDevicePolicy(models.Model):
    state = (
        (0, 'null'), (1, 'success'), (2, 'going'), (3, 'fail')
    )
    device = models.OneToOneField(verbose_name='设备', to=Devices, to_field='id', unique=True, on_delete=models.CASCADE)
    wallpaper_id = models.IntegerField(verbose_name='壁纸ID', null=True, blank=True)
    status = models.IntegerField(verbose_name='状态', choices=state, default=0, blank=True)
    policy_no = models.CharField('策略编号', max_length=255, null=True, blank=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'device_wallpaper'
        verbose_name = '设备壁纸策略'
        verbose_name_plural = '设备壁纸策略'
        # unique_together = (('device', 'wallpaper_id'))


class SaltJidManage(models.Manager):
    def add_jid(self, policy_no, fun, jid, minion, ret, success, full_ret, alter_time, fun_atgs):
        jid = self.create(policy_no=policy_no, fun=fun, jid=jid, minion=minion, ret=ret, success=success,
                          full_ret=full_ret,
                          alter_time=alter_time, fun_args=fun_atgs)
        return jid

    def get_jid(self, jid):
        return self.filter(jid=jid).first()

    def get_full_ret(self, jid):
        job = self.filter(jid=jid).first()
        # return json.loads(job.full_ret)
        return job.full_ret


class SaltJidDevice(models.Model):
    policy_no = models.CharField('策略编号', max_length=255, null=True, blank=True)
    fun = models.CharField(verbose_name='策略方法', blank=True, max_length=50, null=False)
    fun_args = models.TextField(verbose_name='args', blank=True, null=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, unique=True)
    minion = models.CharField(verbose_name='minion key', blank=True, max_length=255, null=False)
    ret = models.TextField(verbose_name='ret', blank=True, null=False)
    success = models.IntegerField(verbose_name='success', blank=True, null=False)
    full_ret = models.TextField(verbose_name='full_ret', blank=True, null=False)
    alter_time = models.DateTimeField(verbose_name='时间', auto_now_add=True)

    objects = SaltJidManage()

    def __str__(self):
        return self.fun + self.fun_args + ' ' + self.jid

    class Meta:
        db_table = 'device_jids'
        verbose_name = 'device_jids'
        verbose_name_plural = verbose_name


class Screensavers(models.Model):
    """
    屏幕保护程序基础配置表
    默认配置一条记录，屏幕保护程序关闭， 间隔时间 10*60s
    """
    switch_status = models.SmallIntegerField('屏幕保护开关 0|关闭； 1|开启', blank=True, null=False, default=0)
    interval = models.CharField('时间间隔，单位为分钟', max_length=8, blank=True, null=True)

    class Meta:
        db_table = 'screensavers'


class ScreensaversDevicePolicy(models.Model):
    state = (
        (0, 'null'), (1, 'success'), (2, 'going'), (3, 'fail')
    )
    device = models.OneToOneField(verbose_name='设备', to=Devices, to_field='id', unique=True, on_delete=models.CASCADE)
    switch_status = models.SmallIntegerField('屏幕锁定开关 0|关闭； 1|开启', blank=True, null=False, default=0)
    interval = models.CharField('时间间隔，单位为秒', max_length=8, blank=True, null=True)
    policy_no = models.CharField('策略编号', max_length=255, null=True, blank=True)
    status = models.IntegerField(verbose_name='状态', choices=state, default=0, blank=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'device_screensavers'
        verbose_name = '设备屏幕策略'
        verbose_name_plural = '设备屏幕策略'


class FilePolicy(models.Model):
    state = (
        (0, 'null'), (1, 'success'), (2, 'going'), (3, 'fail')
    )
    device = models.OneToOneField(verbose_name='设备', to=Devices, to_field='id', unique=True, on_delete=models.CASCADE)
    file = models.ForeignKey(to='filemanager.FileManager', on_delete=models.PROTECT, db_constraint=False, null=True,
                             blank=True, verbose_name='文件')
    status = models.IntegerField(verbose_name='状态', choices=state, default=0, blank=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'file_policy'
        verbose_name = '文件下发策略'
        verbose_name_plural = '文件下发策略'


class PolicyInfo(models.Model):
    """
    策略信息表
    """
    policy_status = (
        (0, 'failure'), (1, 'success'), (2, 'running'), (3, 'stop'), (4, 'atq')  # atq 尚未执行的
    )
    policy_range = (
        (0, '部门筛选'), (1, '终端筛选'), (2, '终端分组筛选')
    )
    policy_name = models.CharField('策略名称', max_length=255, null=True, blank=True)
    policy_no = models.CharField('策略编号', max_length=255, unique=True, null=True, blank=True)
    policy_device_range = models.IntegerField('策略执行范围', choices=policy_range, null=True, blank=True)
    policy_device_web = models.CharField('策略执行回显范围', max_length=255, null=True, blank=True)
    policy_device_list = models.CharField('策略适用范围', max_length=255, null=True, blank=True)
    policy_device_stop = models.CharField('策略暂停的终端范围', max_length=255, null=True, blank=True)
    policy_remark = models.CharField('策略备注', max_length=255, null=True, blank=True)
    policy_create_user = models.CharField('策略创建人', max_length=255, null=True, blank=True)
    policy_create_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    policy_update_time = models.DateTimeField('策略更新时间', null=True, blank=True)
    policy_run_status = models.IntegerField('策略运行状态', choices=policy_status, default=4, blank=True)
    policy_priority_level = models.IntegerField('策略优先级', default=0, blank=True)
    policy_type = models.CharField('配置策略种类', max_length=255, null=True, blank=True)
    policy_context = models.TextField(verbose_name='策略配置信息', null=True, blank=True)

    class Meta:
        db_table = 'policy_info'
        verbose_name = '策略信息表'
        verbose_name_plural = verbose_name


class PolicyDictInfo(models.Model):
    """
    策略唯表信息
    """
    policy_type = models.CharField('策略类型名称', max_length=255, null=True, blank=True)
    policy_type_name = models.CharField('策略类型名称', max_length=255, null=True, blank=True)
    policy_model_context = models.TextField(verbose_name='策略模板信息', null=True, blank=True)

    class Meta:
        db_table = 'policy_dict_info'
        verbose_name = '策略唯表信息'
        verbose_name_plural = verbose_name


class DevicePolicyInfo(models.Model):
    index = models.BigIntegerField(blank=True, null=True)
    device_id = models.TextField(blank=True, null=True)
    software_id = models.TextField(blank=True, null=True)
    usb_switch = models.TextField(blank=True, null=True)
    usb_rw_status = models.TextField(blank=True, null=True)
    firewalld_switch = models.TextField(blank=True, null=True)
    firewalld_level = models.TextField(blank=True, null=True)
    firewalld_white_switch = models.TextField(blank=True, null=True)
    firewalld_white_list = models.TextField(blank=True, null=True)
    firewalld_black_switch = models.TextField(blank=True, null=True)
    firewalld_black_list = models.TextField(blank=True, null=True)
    remote_ssh_switch = models.TextField(blank=True, null=True)
    screen_lock_switch = models.TextField(blank=True, null=True)
    screen_lock_time = models.TextField(blank=True, null=True)
    wallpaper_id = models.TextField(blank=True, null=True)
    root_switch = models.TextField(blank=True, null=True)
    activation_switch = models.TextField(blank=True, null=True)
    activation_address = models.TextField(blank=True, null=True)
    default_ca_switch = models.TextField(blank=True, null=True)
    default_ca_id = models.TextField(blank=True, null=True)

    class Meta:
        db_table = 'device_policy_info'
        verbose_name = '终端策略信息表'


class WaterMark(models.Model):
    """
    水印设置表
    默认配置一条记录，水印关闭
    """

    watermark_switch_status = models.SmallIntegerField('水印开关 0|关闭； 1|开启', blank=True, null=False, default=0)
    require_account_name = models.SmallIntegerField('水印上是否显示登入用户 0|不； 1|显示', blank=True, null=False,
                                                    default=1)
    require_device_cnname = models.SmallIntegerField('水印上是否显示终端别名 0|不； 1|显示', blank=True, null=False,
                                                     default=0)
    require_device_hostname = models.SmallIntegerField('水印上是否显示主机名 0|不； 1|显示', blank=True, null=False,
                                                       default=0)
    require_device_ip = models.SmallIntegerField('水印上是否显示ip地址 0|不； 1|显示', blank=True, null=False, default=1)
    require_device_mac = models.SmallIntegerField('水印上是否显示mac地址 0|不； 1|显示', blank=True, null=False,
                                                  default=0)
    require_device_current_time = models.SmallIntegerField('水印上是否显示设备当前时间 0|不； 1|显示', blank=True,
                                                           null=False, default=1)
    user_defined_text = models.CharField(verbose_name='自定义文本', max_length=255, null=True, blank=True)
    watermark_position = models.CharField(verbose_name='水印显示的位置', max_length=255, null=True, blank=True,
                                          default='全屏')
    watermark_rotate = models.CharField(verbose_name='水印旋转角度', max_length=255, null=True, blank=True, default=0)
    watermark_font = models.CharField(verbose_name='水印字体', max_length=255, null=True, blank=True, default='黑体')
    watermark_font_size = models.SmallIntegerField('水印字体大小', blank=True, null=False, default=16)
    watermark_font_color = models.CharField(verbose_name='水印字体颜色', max_length=255, null=True, blank=True,
                                            default='ff0000')
    watermark_font_transparency = models.DecimalField('水印字体透明度', max_digits=10, decimal_places=2, default=0.4)

    class Meta:
        db_table = 'watermark'
        verbose_name = '水印设置表'


class WaterMarkDevicePolicy(models.Model):
    status = models.IntegerField(verbose_name='策略运行状态 0|无 1|成功 2|正在运行 3|失败', default=0, blank=True)
    watermark_switch_status = models.SmallIntegerField('水印开关 0|关闭； 1|开启', blank=True, null=False, default=0)
    require_account_name = models.SmallIntegerField('水印上是否显示登入用户 0|不； 1|显示', blank=True, null=False,
                                                    default=1)
    require_device_cnname = models.SmallIntegerField('水印上是否显示终端别名 0|不； 1|显示', blank=True, null=False,
                                                     default=0)
    require_device_hostname = models.SmallIntegerField('水印上是否显示主机名 0|不； 1|显示', blank=True, null=False,
                                                       default=0)
    require_device_ip = models.SmallIntegerField('水印上是否显示ip地址 0|不； 1|显示', blank=True, null=False, default=1)
    require_device_mac = models.SmallIntegerField('水印上是否显示mac地址 0|不； 1|显示', blank=True, null=False,
                                                  default=0)
    require_device_current_time = models.SmallIntegerField('水印上是否显示设备当前时间 0|不； 1|显示', blank=True,
                                                           null=False, default=1)
    user_defined_text = models.CharField(verbose_name='自定义文本', max_length=255, null=True, blank=True)
    watermark_position = models.CharField(verbose_name='水印显示的位置', max_length=255, null=True, blank=True,
                                          default='全屏')
    watermark_rotate = models.CharField(verbose_name='水印旋转角度', max_length=255, null=True, blank=True, default=0)
    watermark_font = models.CharField(verbose_name='水印字体', max_length=255, null=True, blank=True, default='黑体')
    watermark_font_size = models.SmallIntegerField(verbose_name='水印字体大小', blank=True, null=True, default=16)
    watermark_font_color = models.CharField(verbose_name='水印字体颜色', max_length=255, null=True, blank=True,
                                            default='ff0000')
    watermark_font_transparency = models.DecimalField(verbose_name='水印字体透明度', max_digits=10, decimal_places=2,
                                                      default=0.4)
    device_id = models.IntegerField(verbose_name='设备id', blank=True, null=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'device_watermark'
        verbose_name = '设备水印策略记录表'
        verbose_name_plural = '设备水印策略'


class AccountLoginPolicy(models.Model):
    status = models.IntegerField(verbose_name='策略运行状态 0|无 1|成功 2|正在运行 3|失败', default=0, blank=True)
    login_switch_status = models.SmallIntegerField('登入登出策略管控开关 0|不管控； 1|管空', blank=True, null=False,
                                                   default=0)
    apply_policy_time = models.SmallIntegerField('应用策略时机 0|注销时； 1|登入时; 2|登入登出， 3|立即', blank=True, null=False,
                                                 default=0)
    file_id = models.IntegerField(verbose_name='文件id', blank=True, null=True)
    account_id = models.IntegerField(verbose_name='用户id', blank=True, null=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'policy_account_login'
        verbose_name = '账户登入登出策略记录表'
        verbose_name_plural = '账户登入登出策略记录'


class AccountDefaultPolicy(models.Model):
    login_switch_status = models.SmallIntegerField('登入登出策略管控开关 0|不管控； 1|管空', blank=True, null=False,
                                                   default=0)
    apply_policy_time = models.SmallIntegerField('应用策略时机 0|注销时； 1|登入时; 2|立即', blank=True, null=False,
                                                 default=0)
    file_names = models.CharField(verbose_name='文件名称', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'policy_account_default'
        verbose_name = '账户登入登出默认策略表'
        verbose_name_plural = '账户登入登出默认策略'


class DesktopThemePolicy(models.Model):
    status = models.IntegerField(verbose_name='策略运行状态 0|无 1|成功 2|正在运行 3|失败', default=0, blank=True)
    theme_switch_status = models.SmallIntegerField('桌面主题策略管控开关 0|不管控； 1|管空',
                                                   blank=True, null=False, default=0)
    style_theme = models.SmallIntegerField('样式主题 0|经典主题； 1|暗色主题;', blank=True, null=False, default=0)
    gtk_theme = models.SmallIntegerField('GTK主题 0|桔梗； 1|琥珀色; 2|朱砂; 3|蔻梢; 4|经典', blank=True, null=False, default=0)
    icon_theme = models.SmallIntegerField('图标主题 0|时尚； 1|经典暗色;', blank=True, null=False, default=0)
    cursor_theme = models.SmallIntegerField('光标主题 0|灵动白； 1|炫酷黑;', blank=True, null=False, default=0)
    is_sunset = models.SmallIntegerField('是否跟随日出日落 0|否； 1|是;', blank=True, null=False, default=0)
    win_decoration_switch = models.SmallIntegerField('窗口装饰开关 0|否； 1|是;', blank=True, null=False, default=0)
    win_transparency = models.CharField(verbose_name='窗口透明度', blank=True, max_length=255, null=True)
    account_id = models.IntegerField(verbose_name='用户id', blank=True, null=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'policy_desktop_theme'
        verbose_name = '账户桌面主题信息表'


class SystemUpdatesPolicy(models.Model):
    status = models.IntegerField(verbose_name='策略运行状态 0|无 1|成功 2|正在运行 3|失败', default=0, blank=True)
    system_update_switch = models.SmallIntegerField('系统更新策略管控开关 0|不管控； 1|管控',
                                                    blank=True, null=False, default=0)
    update_style = models.SmallIntegerField('系统更新方式 0|从不更新； 1|定时更新;', blank=True, null=False, default=0)
    check_time_days = models.SmallIntegerField(
        '定时检查更新 0|每天； 1|每周一;2|每周二;3|每周三;4|每周四;5|每周五;6|每周六;7|每周;', blank=True, null=False,
        default=0)
    check_time_time = models.CharField(verbose_name='检查时间点', blank=True, max_length=255, null=True)
    download_update_method = models.SmallIntegerField('下载更新方式 0|手动； 1|自动;', blank=True, null=False, default=0)
    device_id = models.IntegerField(verbose_name='终端id', blank=True, null=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'policy_system_update'
        verbose_name = '系统更新设置表'


class SoftwareRunsPolicy(models.Model):
    status = models.IntegerField(verbose_name='策略运行状态 null|无 1|成功 2|正在运行 3|失败', default=0, blank=True)
    software_runs_switch = models.SmallIntegerField('软件运行策略管控开关 0|不管控； 1|管控',
                                                    blank=True, null=False, default=0)
    software_runs_style = models.SmallIntegerField('软件运行方式 0|黑名单； 1|白名单，2|黑白名单', blank=True, null=False,
                                                   default=0)
    runs_white_switch = models.SmallIntegerField('白名单开关 0|关闭； 1|开启;', blank=True, null=False, default=0)
    white_software_list = models.CharField(verbose_name='白名单软件列表', blank=True, max_length=255, null=True)
    runs_black_switch = models.SmallIntegerField('黑名单开关 0|关闭； 1|开启;', blank=True, null=False, default=0)
    black_software_list = models.TextField(verbose_name='黑名单软件列表', blank=True, null=True)
    device_id = models.IntegerField(verbose_name='终端id', blank=True, null=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'policy_software_runs'
        verbose_name = '软件运行设置表'


class ScreenSaverPolicy(models.Model):
    status = models.IntegerField(verbose_name='策略运行状态 null|无 1|成功 2|正在运行 3|失败', default=0, blank=True)
    screen_saver_switch = models.SmallIntegerField('屏幕保护策略管控开关 0|不管控； 1|管控',
                                                   blank=True, null=False, default=0)
    lock_screen_time = models.CharField(verbose_name='闲置的锁屏时间', blank=True, max_length=255, null=True)
    screen_saver_time = models.CharField(verbose_name='闲置的屏幕保护时间', blank=True, max_length=255, null=True)
    lock_activate_screensaver = models.SmallIntegerField('激活屏保锁屏 0|否； 1|是;', blank=True, null=False, default=0)
    lock_sleep_screensaver = models.SmallIntegerField('睡前和休眠时锁屏 0|否； 1|是;', blank=True, null=False, default=0)
    screensaver_cycle_time = models.CharField(verbose_name='屏幕保护循环时间', blank=True, max_length=255, null=True)
    device_id = models.IntegerField(verbose_name='终端id', blank=True, null=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)
    use_for_greeter = models.SmallIntegerField('锁屏壁纸添加到登录界面 0|否； 1|是;', blank=True, null=False, default=0)
    lock_wallpaper_id = models.IntegerField(verbose_name='锁屏壁纸ID', null=True, blank=True)
    screensaver_folder_id = models.IntegerField(verbose_name='屏幕保护轮播文件夹ID', null=True, blank=True)

    class Meta:
        db_table = 'policy_screen_saver'
        verbose_name = '屏幕保护策略表'


class FileRedirectionPolicy(models.Model):
    status = models.IntegerField(verbose_name='策略运行状态 null|无 1|成功 2|正在运行 3|失败', default=0, blank=True)
    file_redirection_switch = models.SmallIntegerField('文件重定向策略管控开关 0|不管控； 1|管控', blank=True,
                                                       null=False, default=0)
    file_redirection_list = models.TextField(verbose_name='文件重定向文件列表', blank=True, null=True)
    device_id = models.IntegerField(verbose_name='终端id', blank=True, null=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'policy_file_redirection'
        verbose_name = '文件重定向策略表'


class BrowserFavoritesPolicy(models.Model):
    status = models.IntegerField(verbose_name='策略运行状态 null|无 1|成功 2|正在运行 3|失败', default=0, blank=True)
    browser_favorites_switch = models.SmallIntegerField('游览器收藏夹策略管控开关 0|不管控； 1|管控', blank=True,
                                                        null=False, default=0)
    browser_favorites_redirection = models.CharField(verbose_name='游览器收藏夹地址', max_length=255, blank=True, null=True)
    account_id = models.IntegerField(verbose_name='用户id', blank=True, null=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'policy_browser_favorites'
        verbose_name = '游览器收藏夹策略表'


class TrustedSitesPolicy(models.Model):
    status = models.IntegerField(verbose_name='策略运行状态 null|无 1|成功 2|正在运行 3|失败', default=0, blank=True)
    trusted_sites_switch = models.SmallIntegerField('游览器可信任站点策略管控开关 0|不管控； 1|管控', blank=True,
                                                    null=False, default=0)
    trusted_sites_list = models.TextField(verbose_name='游览器可信任站点路径列表', blank=True, null=True)
    account_id = models.IntegerField(verbose_name='用户id', blank=True, null=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)

    class Meta:
        db_table = 'policy_trusted_sites'
        verbose_name = '游览器可信任站点策略表'

class Shell(models.Model):
    type = (
        (0, 'related'), (1, 'custom')
    )
    name = models.CharField(verbose_name='脚本名称', max_length=32, null=True, unique=True)
    file_path = models.CharField(verbose_name='文件', blank=True, null=True, max_length=256)
    type = models.IntegerField(verbose_name='脚本类型', choices=type, default=0, blank=True)
    remark = models.CharField(verbose_name='备注', max_length=256, null=True)
    deleted = models.BooleanField(verbose_name='删除', default=False)

    creator = models.CharField(max_length=64, blank=True, null=True)
    create_time = models.DateTimeField(blank=True, null=True, auto_now_add=True)
    update_time = models.DateTimeField(blank=True, null=True, auto_now=True)
    updater = models.CharField(max_length=64, blank=True, null=True)

    class Meta:
        db_table = 'shell_info'
        verbose_name = '脚本库'
        verbose_name_plural = '脚本库'

class ShellDevicePolicy(models.Model):
    on_off = (
        (0, 'off'), (1, 'on')
    )
    state = (
        (0, 'null'), (1, 'success'), (2, 'going'), (3, 'fail')
    )
    type = (
        (0, 'related'), (1, 'custom')
    )
    status = models.IntegerField(verbose_name='状态', choices=state, default=0, blank=True)
    shell_switch = models.IntegerField(verbose_name='是否开启', choices=on_off, default=0, blank=True)
    shell_type = models.IntegerField(verbose_name='脚本类型', choices=type, default=0, blank=True)
    shell = models.ForeignKey(to=Shell, related_name='shell', verbose_name='脚本ID', blank=True, null=True, on_delete=models.SET_NULL)
    file_path = models.CharField(verbose_name='文件', blank=True, null=True, max_length=256)
    org_info_id = models.IntegerField(verbose_name='部门id', blank=True, null=True)
    policy_no = models.CharField('策略编号', max_length=255, null=True, blank=True)
    device = models.OneToOneField(verbose_name='设备', to=Devices, to_field='id', unique=True, related_name='shell_policy',
                                  on_delete=models.CASCADE)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)
    remark = models.CharField(verbose_name='备注', max_length=256, null=True)

    creator = models.CharField(max_length=64, blank=True, null=True)
    create_time = models.DateTimeField(blank=True, null=True, auto_now_add=True)
    update_time = models.DateTimeField(blank=True, null=True, auto_now=True)
    updater = models.CharField(max_length=64, blank=True, null=True)

    class Meta:
        db_table = 'device_shell'
        verbose_name = '设备shell策略'
        verbose_name_plural = '设备shell策略'