import logging
import os

from rest_framework import serializers

from accounts.models import AccountEntity
from devices.models import Devices
from devices.serializers import DeviceSerializer
from nfs_domain_mainserver import settings
from organizations.models import OrgnizationEntity
from saltapi.models import Wallpaper, Screensavers, PolicyInfo, PolicyDictInfo, FirewalldRulesPolicy, \
    FirewalldWhiteBlackRosterPolicy, WaterMark, WaterMarkDevicePolicy, ActivationDevicePolicy, AccountLoginPolicy, \
    AccountDefaultPolicy, DesktopThemePolicy, SystemUpdatesPolicy, SoftwareRunsPolicy, ScreenSaverPolicy, \
    FileRedirectionPolicy, BrowserFavoritesPolicy, TrustedSitesPolicy, Shell, ShellDevicePolicy


class WallpaperSerializer(serializers.ModelSerializer):
    class Meta:
        model = Wallpaper
        fields = ('id', 'image_name', 'is_default', 'image_routing')


class ScreenSaversSerializer(serializers.ModelSerializer):
    class Meta:
        model = Screensavers
        fields = '__all__'


class PolicyInfoSerializer(serializers.ModelSerializer):
    policy_exe_status = serializers.SerializerMethodField()

    class Meta:
        model = PolicyInfo
        fields = ('id', 'policy_name', 'policy_no', 'policy_device_range', 'policy_device_web', 'policy_device_list',
                  'policy_device_stop',
                  'policy_remark', 'policy_create_user', 'policy_update_time', 'policy_run_status',
                  'policy_priority_level',
                  'policy_type', 'policy_context', 'policy_exe_status')

    def get_policy_exe_status(self, obj):
        policy_device_list = obj.policy_device_list
        policy_device_stop = obj.policy_device_stop
        if policy_device_list:
            policy_device_list = policy_device_list.split(',')
        else:
            policy_device_list = []
        if policy_device_stop:
            policy_device_stop = policy_device_stop.split(',')
        else:
            policy_device_stop = []
        policy_device_list = [i for i in policy_device_list if i is not '']
        policy_device_stop = [i for i in policy_device_stop if i is not '']

        res_dict = {
            "device_total": 0,
            "device_run": 0,
            "percent": '0',
        }
        if obj.policy_run_status != 3:
            res_dict['device_total'] = len(policy_device_list) + len(policy_device_stop)
            res_dict['device_run'] = len(policy_device_list)
            if res_dict['device_total'] == 0:
                res_dict['percent'] = '100.00'
            else:
                res_dict['percent'] = '{:.2f}'.format(100 * (res_dict['device_run'] / res_dict['device_total']))
        else:
            res_dict['device_total'] = len(policy_device_list) + len(policy_device_stop)
            res_dict['device_run'] = 0
            res_dict['percent'] = '0'
        return res_dict


class PolicyDictInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = PolicyDictInfo
        fields = '__all__'


class FirewalldRuleSerializers(serializers.ModelSerializer):
    class Meta:
        model = FirewalldRulesPolicy
        fields = (
            'id', 'level', 'access_control', 'protocol', 'state', 'in_out', 'ip_type', 'external_addr', 'local_port',
            'external_port')


class FirewalldRosterSerializers(serializers.ModelSerializer):
    class Meta:
        model = FirewalldWhiteBlackRosterPolicy
        fields = "__all__"


class WaterMarkSerializer(serializers.ModelSerializer):
    class Meta:
        model = WaterMark
        fields = '__all__'


class WaterMarkDevicePolicySerializer(serializers.ModelSerializer):
    org_name = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()
    watermark_switch_status = serializers.SerializerMethodField()
    user_defined_text = serializers.SerializerMethodField()
    watermark_position = serializers.SerializerMethodField()
    watermark_rotate = serializers.SerializerMethodField()
    watermark_font = serializers.SerializerMethodField()
    watermark_font_size = serializers.SerializerMethodField()
    watermark_font_color = serializers.SerializerMethodField()
    watermark_font_transparency = serializers.SerializerMethodField()

    class Meta:
        model = Devices
        fields = ('id', 'device_cnname', 'device_ip', 'org_info_id', 'org_name', 'status', 'watermark_switch_status',
                  'user_defined_text', 'watermark_position', 'watermark_rotate', 'watermark_font',
                  'watermark_font_size', 'watermark_font_color', 'watermark_font_transparency')

    def get_org_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()

        if items:
            return items.name
        else:
            return None

    def get_status(self, obj):
        watermarkdevicepolicy_items = WaterMarkDevicePolicy.objects.all()
        watermarkdevicepolicy_item = watermarkdevicepolicy_items.filter(device_id=obj.id).first()
        if watermarkdevicepolicy_item:
            return watermarkdevicepolicy_item.status
        else:
            return None

    def get_watermark_switch_status(self, obj):
        watermarkdevicepolicy_items = WaterMarkDevicePolicy.objects.all()
        watermarkdevicepolicy_item = watermarkdevicepolicy_items.filter(device_id=obj.id).first()
        if watermarkdevicepolicy_item:
            return watermarkdevicepolicy_item.watermark_switch_status
        else:
            return None

    def get_user_defined_text(self, obj):
        watermarkdevicepolicy_items = WaterMarkDevicePolicy.objects.all()
        watermarkdevicepolicy_item = watermarkdevicepolicy_items.filter(device_id=obj.id).first()
        if watermarkdevicepolicy_item:
            return watermarkdevicepolicy_item.user_defined_text
        else:
            return None

    def get_watermark_position(self, obj):
        watermarkdevicepolicy_items = WaterMarkDevicePolicy.objects.all()
        watermarkdevicepolicy_item = watermarkdevicepolicy_items.filter(device_id=obj.id).first()
        if watermarkdevicepolicy_item:
            return watermarkdevicepolicy_item.watermark_position
        else:
            return '全屏'

    def get_watermark_rotate(self, obj):
        watermarkdevicepolicy_items = WaterMarkDevicePolicy.objects.all()
        watermarkdevicepolicy_item = watermarkdevicepolicy_items.filter(device_id=obj.id).first()
        if watermarkdevicepolicy_item:
            return watermarkdevicepolicy_item.watermark_rotate
        else:
            return 0

    def get_watermark_font(self, obj):
        watermarkdevicepolicy_items = WaterMarkDevicePolicy.objects.all()
        watermarkdevicepolicy_item = watermarkdevicepolicy_items.filter(device_id=obj.id).first()
        if watermarkdevicepolicy_item:
            return watermarkdevicepolicy_item.watermark_font
        else:
            return '黑体'

    def get_watermark_font_size(self, obj):
        watermarkdevicepolicy_items = WaterMarkDevicePolicy.objects.all()
        watermarkdevicepolicy_item = watermarkdevicepolicy_items.filter(device_id=obj.id).first()
        if watermarkdevicepolicy_item:
            return watermarkdevicepolicy_item.watermark_font_size
        else:
            return 16

    def get_watermark_font_color(self, obj):
        watermarkdevicepolicy_items = WaterMarkDevicePolicy.objects.all()
        watermarkdevicepolicy_item = watermarkdevicepolicy_items.filter(device_id=obj.id).first()
        if watermarkdevicepolicy_item:
            return watermarkdevicepolicy_item.watermark_font_color
        else:
            return 'ff0000'

    def get_watermark_font_transparency(self, obj):
        watermarkdevicepolicy_items = WaterMarkDevicePolicy.objects.all()
        watermarkdevicepolicy_item = watermarkdevicepolicy_items.filter(device_id=obj.id).first()
        if watermarkdevicepolicy_item:
            return watermarkdevicepolicy_item.watermark_font_transparency
        else:
            return 0.4


class ActivePolicySerializers(serializers.ModelSerializer):
    org_name = serializers.SerializerMethodField()
    active = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()

    class Meta:
        model = Devices
        fields = ('id', 'device_cnname', 'device_ip', 'org_info_id', 'org_name', 'active', 'status')

    def get_org_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()
        if items:
            return items.name
        else:
            return None

    def get_active(self, obj):
        Activation_items = ActivationDevicePolicy.objects.all()
        items = Activation_items.filter(device_id=obj.id).first()
        if items:
            return items.active
        else:
            return 0

    def get_status(self, obj):
        Activation_items = ActivationDevicePolicy.objects.all()
        items = Activation_items.filter(device_id=obj.id).first()
        if items:
            return items.status
        else:
            return None


class AccountLoginPolicySerializers(serializers.ModelSerializer):
    org_name = serializers.SerializerMethodField()
    login_switch_status = serializers.SerializerMethodField()
    apply_policy_time = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()

    class Meta:
        model = AccountEntity
        fields = ('id', 'name', 'org_info_id', 'org_name', 'login_switch_status', 'status', 'apply_policy_time')

    def get_org_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()
        if items:
            return items.name
        else:
            return None

    def get_login_switch_status(self, obj):
        AccountLogin_items = AccountLoginPolicy.objects.all()
        items = AccountLogin_items.filter(account_id=obj.id).first()
        if items:
            return items.login_switch_status
        else:
            return 0

    def get_apply_policy_time(self, obj):
        AccountLogin_items = AccountLoginPolicy.objects.all()
        items = AccountLogin_items.filter(account_id=obj.id).first()
        if items:
            return items.apply_policy_time
        else:
            return 0

    def get_status(self, obj):
        AccountLogin_items = AccountLoginPolicy.objects.all()
        items = AccountLogin_items.filter(account_id=obj.id).first()
        if items:
            return items.status
        else:
            return None


class AccountDefaultPolicySerializer(serializers.ModelSerializer):
    class Meta:
        model = AccountDefaultPolicy
        fields = '__all__'


class AccountDesktopThemePolicySerializers(serializers.ModelSerializer):
    org_name = serializers.SerializerMethodField()
    theme_switch_status = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()

    class Meta:
        model = AccountEntity
        fields = ('id', 'name', 'org_info_id', 'org_name', 'theme_switch_status', 'status')

    def get_org_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()
        if items:
            return items.name
        else:
            return None

    def get_theme_switch_status(self, obj):
        account_policy_items = DesktopThemePolicy.objects.all()
        items = account_policy_items.filter(account_id=obj.id).first()
        if items:
            return items.theme_switch_status
        else:
            return 0

    def get_status(self, obj):
        account_policy_items = DesktopThemePolicy.objects.all()
        items = account_policy_items.filter(account_id=obj.id).first()
        if items:
            return items.status
        else:
            return None


class DeviceSystemUpdatesPolicySerializers(serializers.ModelSerializer):
    org_name = serializers.SerializerMethodField()
    system_update_switch = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()

    class Meta:
        model = Devices
        fields = ('id', 'device_cnname', 'device_ip', 'org_info_id', 'org_name', 'system_update_switch', 'status')

    def get_org_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()
        if items:
            return items.name
        else:
            return None

    def get_system_update_switch(self, obj):
        device_items = SystemUpdatesPolicy.objects.all()
        items = device_items.filter(device_id=obj.id).first()
        if items:
            return items.system_update_switch
        else:
            return 0

    def get_status(self, obj):
        device_items = SystemUpdatesPolicy.objects.all()
        items = device_items.filter(device_id=obj.id).first()
        if items:
            return items.status
        else:
            return None


class DeviceSoftwareRunsPolicySerializers(serializers.ModelSerializer):
    org_name = serializers.SerializerMethodField()
    software_runs_switch = serializers.SerializerMethodField()
    software_runs_style = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()

    class Meta:
        model = Devices
        fields = (
            'id', 'device_cnname', 'device_ip', 'org_info_id', 'org_name', 'software_runs_switch',
            'software_runs_style',
            'status')

    def get_org_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()
        if items:
            return items.name
        else:
            return None

    def get_software_runs_switch(self, obj):
        device_items = SoftwareRunsPolicy.objects.all()
        items = device_items.filter(device_id=obj.id).first()
        if items:
            return items.software_runs_switch
        else:
            return 0

    def get_software_runs_style(self, obj):
        device_items = SoftwareRunsPolicy.objects.all()
        items = device_items.filter(device_id=obj.id).first()
        if items:
            return items.software_runs_style
        else:
            return None

    def get_status(self, obj):
        device_items = SoftwareRunsPolicy.objects.all()
        items = device_items.filter(device_id=obj.id).first()
        if items:
            return items.status
        else:
            return None


class DeviceScreenSaverPolicySerializers(serializers.ModelSerializer):
    org_name = serializers.SerializerMethodField()
    screen_saver_switch = serializers.SerializerMethodField()
    jid = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()

    class Meta:
        model = Devices
        fields = ('id', 'device_cnname', 'device_ip', 'org_info_id', 'org_name', 'screen_saver_switch', 'jid', 'status')

    def get_org_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()
        if items:
            return items.name
        else:
            return None

    def get_screen_saver_switch(self, obj):
        device_items = ScreenSaverPolicy.objects.all()
        items = device_items.filter(device_id=obj.id).first()
        if items:
            return items.screen_saver_switch
        else:
            return 0

    def get_jid(self, obj):
        device_items = ScreenSaverPolicy.objects.all()
        items = device_items.filter(device_id=obj.id).first()
        if items:
            return items.jid
        else:
            return None

    def get_status(self, obj):
        device_items = ScreenSaverPolicy.objects.all()
        items = device_items.filter(device_id=obj.id).first()
        if items:
            return items.status
        else:
            return None


class DeviceFileRedirectionPolicySerializers(serializers.ModelSerializer):
    org_name = serializers.SerializerMethodField()
    file_redirection_switch = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()

    class Meta:
        model = Devices
        fields = ('id', 'device_cnname', 'device_ip', 'org_info_id', 'org_name', 'file_redirection_switch', 'status')

    def get_org_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()
        if items:
            return items.name
        else:
            return None

    def get_file_redirection_switch(self, obj):
        device_items = FileRedirectionPolicy.objects.all()
        items = device_items.filter(device_id=obj.id).first()
        if items:
            return items.file_redirection_switch
        else:
            return 0

    def get_status(self, obj):
        device_items = FileRedirectionPolicy.objects.all()
        items = device_items.filter(device_id=obj.id).first()
        if items:
            return items.status
        else:
            return None


class AccountTrustedSitesPolicySerializers(serializers.ModelSerializer):
    org_name = serializers.SerializerMethodField()
    trusted_sites_switch = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()

    class Meta:
        model = AccountEntity
        fields = ('id', 'name', 'org_info_id', 'org_name', 'trusted_sites_switch', 'status')

    def get_org_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()
        if items:
            return items.name
        else:
            return None

    def get_trusted_sites_switch(self, obj):
        account_policy_items = TrustedSitesPolicy.objects.all()
        items = account_policy_items.filter(account_id=obj.id).first()
        if items:
            return items.trusted_sites_switch
        else:
            return 0

    def get_status(self, obj):
        account_policy_items = TrustedSitesPolicy.objects.all()
        items = account_policy_items.filter(account_id=obj.id).first()
        if items:
            return items.status
        else:
            return None


class AccountBrowserFavoritesPolicySerializers(serializers.ModelSerializer):
    org_name = serializers.SerializerMethodField()
    browser_favorites_switch = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()

    class Meta:
        model = AccountEntity
        fields = ('id', 'name', 'org_info_id', 'org_name', 'browser_favorites_switch', 'status')

    def get_org_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()
        if items:
            return items.name
        else:
            return None

    def get_browser_favorites_switch(self, obj):
        account_policy_items = BrowserFavoritesPolicy.objects.all()
        items = account_policy_items.filter(account_id=obj.id).first()
        if items:
            return items.browser_favorites_switch
        else:
            return 0

    def get_status(self, obj):
        account_policy_items = BrowserFavoritesPolicy.objects.all()
        items = account_policy_items.filter(account_id=obj.id).first()
        if items:
            return items.status
        else:
            return None


class SystemUpdatesPolicySerializer(serializers.ModelSerializer):
    class Meta:
        model = SystemUpdatesPolicy
        fields = '__all__'


class ShellInfoSerializer(serializers.ModelSerializer):
    file_info = serializers.SerializerMethodField(read_only=True)

    def get_file_info(self, obj):
        if not obj.file_path:
            return  {}
        shell_upload_path = os.path.join(settings.MEDIA_ROOT, obj.file_path)
        if not os.path.exists(shell_upload_path):
            return {}
        try:
            with open(shell_upload_path, 'rb') as f:
                return {
                    'content': f.read().decode('utf-8'),
                    'size': os.path.getsize(shell_upload_path),
                    'name': os.path.basename(shell_upload_path)
                }
        except Exception as e:
            return {}

    class Meta:
        model = Shell
        fields = '__all__'

class ShellPolicySerializer(serializers.ModelSerializer):
    file_info = serializers.SerializerMethodField(read_only=True)

    def get_file_info(self, obj):
        if not obj.file_path:
            return  {}
        shell_upload_path = os.path.join(settings.MEDIA_ROOT, obj.file_path)
        if not os.path.exists(shell_upload_path):
            return {}
        try:
            with open(shell_upload_path, 'rb') as f:
                return {
                    'content': f.read().decode('utf-8'),
                    'size': os.path.getsize(shell_upload_path),
                    'name': os.path.basename(shell_upload_path)
                }
        except Exception as e:
            return {}

    class Meta:
        model = ShellDevicePolicy
        fields = '__all__'

class ShellDevicePolicySerializer(DeviceSerializer):
    shell_policy = ShellPolicySerializer()

    def update(self, instance, validated_data):
        data = validated_data.get('shell_policy', {})
        data['device_id'] = instance.id
        ShellDevicePolicy.objects.update_or_create(device=instance, defaults=data)

    # def create(self, validated_data):
    #     data = validated_data.get('shell_policy', {})
    #     data['device_id'] = instance.id
    #     instance.shell_policy = ShellDevicePolicy.objects.update_or_create(device=instance, defaults=data)

    class Meta(DeviceSerializer.Meta):
        fields = DeviceSerializer.Meta.fields + ('shell_policy',)
