import logging
import json
import os
import shutil
import time
import uuid
from datetime import datetime
from datetime import timedelta
import yaml
from urllib.parse import urlparse
from django_redis import get_redis_connection
from celery_task.celery import app
from django.conf import settings

from filemanager.models import FileManager
from saltapi.utils import SaltSHH, SaltAPI
from saltapi.models import SSHDevicePolicy, USBDevicePolicy, FirewalldDevicePolicy, WallpaperDevicePolicy, Wallpaper, \
    WaitDevicePolicy, ActivationDevicePolicy, ScreensaversDevicePolicy, Screensavers, \
    FirewalldWhiteBlackRosterDevicePolicy, LocalUserPWDPolicy, ShellDevicePolicy, ScreenSaverPolicy
from saltapi.singnals import policy_singnal_sender
from saltapi.output import check_output
from utils.RedisHelper import RedisHelper
from utils.utils import timeout_func
from accounts.models import AccountEntity
from tasks.models import Tasks, TaskDeviceExec
from devices.serializers import DeviceSerializer
from software.deb_util import getDebInfo

# 浏览器策略执行
from browser.models import BrowserPolicyLog
from browser.models import USER_POLICY_TYPE, APPLICATION_POLICY_TYPE, BOOKMARK_POLICY_TYPE, POLICY_DEFAULT_FILE_NAME
from devices.models import Devices


logger = logging.getLogger(__name__)


@app.task(queue='freeipa')
def install_minion(roster, tgt, fun='state.sls', arg=['salt-minion-init.install']):
    # c_path = os.path.join(settings.SALT_CONF, 'etc/salt/master')
    # master_opts = salt.config.client_config(c_path)
    # master_opts['roster_file'] = os.path.join(settings.SALT_CONF, 'etc/salt/roster')
    # salt_ssh = SSHClient(c_path=c_path, mopts=master_opts)
    roster_path = os.path.join(settings.SALT_CONF, 'etc/salt/roster')
    with open(roster_path, 'w', encoding='utf-8') as f_roster:
        yaml.dump(roster, f_roster)

        minion_id_path = os.path.join(settings.SALT_CONF, 'srv/salt/file_system/minions/conf/minion_id')
        with open(minion_id_path, 'w', encoding='utf-8') as f_minion:
            f_minion.write(tgt)
    salt_ssh = SaltSHH()
    if arg:
        res = salt_ssh.salt_ssh.cmd(tgt=tgt, fun=fun, arg=arg)
    else:
        res = salt_ssh.salt_ssh.cmd(tgt=tgt, fun=fun)
    return res


def get_os(os, os_release=''):
    os = os.lower()
    if os == 'uos':
        return 'uos'
    elif os == 'kylin':
        if os_release.lower() == 'kylin v10':
            return 'kylin-v10'
        else:
            return 'kylin'
    if '5.0' in os_release:
        return 'nfs'
    else:
        return 'nfs-4'


def pre_add_domain(salt_api, tgt, **kwargs):
    pillar = {
        'server': os.environ.get("DOMAIN_SERVER_FQDN"),
        'ip': os.environ.get("DOMAIN_IP"),
    }
    arg = [
        "{os}.freeipa-client-init.pre_add_domain".format(
            os=get_os(kwargs.get('os'), os_release=kwargs.get('os_release'))),
        "pillar={pillar}".format(pillar=pillar), 'queue=true']
    logger.info('####### celery pre_add_domain ######')
    logger.info(arg)
    result = salt_api.deploy(tgt, arg)
    logger.info(result)
    logger.info('#######---######')
    res = result['return'][0]
    if not res.get(tgt):
        return False
    _, _, num_failed = check_output.output(res.get(tgt))
    if num_failed:
        return False
    return True


def join_domain(salt_api, tgt, **kwargs):
    pillar = {
        'server': os.environ.get("DOMAIN_SERVER_FQDN"),
        'domain': os.environ.get("DOMAIN_NAME"),
        'password': os.environ.get("DOMAIN_SERVER_ADMIN_PASSWD"),
        'hostname': tgt,
        'principal': os.environ.get("DOMAIN_SERVER_ADMIN"),
        'ntpserver': os.environ.get("DOMAIN_SERVER_FQDN")
    }
    arg = [
        "{os}.freeipa-client-init.join_domain".format(os=get_os(kwargs.get('os'), os_release=kwargs.get('os_release'))),
        "pillar={pillar}".format(pillar=pillar), 'queue=true']
    logger.info('####### celery join_domain ######')
    logger.info(arg)
    result = salt_api.deploy(tgt, arg)
    logger.info(result)
    logger.info('#######---######')
    res = result['return'][0]
    if not res.get(tgt):
        return False
    _, _, num_failed = check_output.output(res.get(tgt))
    if num_failed:
        return False
    return True


def lock_local_user(salt_api, tgt, **kwargs):
    arg = ['{os}.user-manage.lock_local_user'.format(os=get_os(kwargs.get('os'), os_release=kwargs.get('os_release'))),
           'queue=true']
    logger.info('####### celery lock_local_user ######')
    result = salt_api.deploy(tgt, arg)
    logger.info(result)
    logger.info('#######---######')
    res = result['return'][0]
    if not res.get(tgt):
        return False
    _, _, num_failed = check_output.output(res.get(tgt))
    if num_failed:
        return False
    return True


def ntp_sls(salt_api, tgt, **kwargs):
    pillar = {'server': os.environ.get("DOMAIN_IP")}
    arg = ['{os}.freeipa-client-init.ntp'.format(os=get_os(kwargs.get('os'))), "pillar={pillar}".format(pillar=pillar),
           'queue=true']
    logger.info('####### celery ntp_sls ######')
    result = salt_api.deploy(tgt, arg)
    logger.info(result)
    logger.info('#######---######')
    res = result['return'][0]
    if not res.get(tgt):
        return False
    _, _, num_failed = check_output.output(res.get(tgt))
    if num_failed:
        return False
    return True


def unlock_local_user(device, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {
        'event_type': 'unlock_local_user',
        'device_ip': device.get('device_ip'),
        'device_hostname': device.get('device_hostname'),
        'success': True,
        'msg': '解锁用户已下发'
    }
    signal.update(kwargs)
    arg = [
        '{os}.user-manage.unlock_local_user'.format(os=get_os(kwargs.get('os'), os_release=kwargs.get('os_release'))),
        'queue=true']
    result = salt_api.async_deploy(device.get('device_hostname'), arg)
    logger.info('####### celery unlock_local_user ######')
    logger.info(result)
    logger.info('#######---######')
    if isinstance(result, dict):
        signal['success'] = False
        signal['status'] = 3
        signal['msg'] = '解锁用户失败 {message}'.format(message=result.get('message'))  # delete freeipa fail
        public.public(json.dumps(signal), chan='policy')
        return
    public.public(json.dumps(signal), chan='policy')


def allow_access(salt_api, tgt, **kwargs):
    # if get_os(kwargs.get('os')) not in ('uos', 'kylin'):
    #     return False
    pillar = {'server': os.environ.get("DOMAIN_IP")}
    arg = ['{os}.firewall-manage.allow_access'.format(os=get_os(kwargs.get('os'), kwargs.get('os_release'))),
           "pillar={pillar}".format(pillar=pillar), 'queue=true']
    logger.info('####### celery allow_access ######')
    result = salt_api.deploy(tgt, arg)
    logger.info(result)
    logger.info('#######---######')
    res = result['return'][0]
    if not res.get(tgt):
        return False
    _, _, num_failed = check_output.output(res.get(tgt))
    if num_failed:
        return False
    return True


def ssh_port_recovery(device, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {
        'event_type': 'ssh_port_recovery',
        'device_ip': device.get('device_ip'),
        'device_hostname': device.get('device_hostname'),
        'success': True,
        'msg': '端口策略已下发'
    }
    signal.update(kwargs)
    arg = [
        '{os}.ssh-manage.ssh_port_recovery'.format(os=get_os(kwargs.get('os'), os_release=kwargs.get('os_release'))),
        'queue=true']
    result = salt_api.async_deploy(device.get('device_hostname'), arg)
    logger.info('####### celery ssh_port_recovery_policy ######')
    logger.info(result)
    logger.info('#######---######')
    if isinstance(result, dict):
        signal['success'] = False
        signal['status'] = 3
        signal['msg'] = '端口策略失败 {message}'.format(message=result.get('message'))
        public.public(json.dumps(signal), chan='policy')
        return
    public.public(json.dumps(signal), chan='policy')


def ssh_port_setup(salt_api, tgt, **kwargs):
    pillar = {
        'port': 22,
    }
    ssh_port = ['{os}.ssh-manage.ssh_port_setup'.format(os=get_os(kwargs.get('os'), kwargs.get('os_release'))),
                "pillar={pillar}".format(pillar=pillar), 'queue=true']
    result = salt_api.deploy(tgt, ssh_port)
    logger.info('####### celery ssh_port_setup_policy ######')
    logger.info(result)
    logger.info('#######---######')
    res = result['return'][0]
    if not res.get(tgt):
        return False
    _, _, num_failed = check_output.output(res.get(tgt))
    if num_failed:
        return False
    return True


def stop_minion(device, **kwargs):
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    ssh_port = ['{os}.salt-minion.stop'.format(os=get_os(kwargs.get('os'), kwargs.get('os_release'))), 'queue=true']
    result = salt_api.deploy(device.get('device_hostname'), ssh_port)
    logger.info('####### celery stop_minion ######')
    logger.info(result)
    logger.info('#######---######')
    res = result['return'][0]
    if not res.get(device.get('device_hostname')):
        return False
    _, _, num_failed = check_output.output(res.get(device.get('device_hostname')))
    if num_failed:
        return False
    salt_api.delete_key(device.get('device_hostname'))
    return True


def nfs(salt_api, device, signal, **kwargs):
    # pre_add_domain(salt_api, device.get('device_hostname'), **kwargs)
    arg = '{os}.freeipa-client-init.install'.format(os=get_os(kwargs.get('os'), os_release=kwargs.get('os_release')))
    result = salt_api.deploy(device.get('device_hostname'), arg)
    logger.info('####### celery install_freeipa ######')
    logger.info(result)
    logger.info('#######---######')
    res = result['return'][0]

    if not res.get(device.get('device_hostname')):
        signal['success'] = False
        signal['msg'] = 'install freeipa fail'
        # public.public(json.dumps(signal), chan='freeipa')
        return 1003, signal
    _, _, num_failed = check_output.output(res.get(device.get('device_hostname')))
    if num_failed:
        signal['success'] = False
        signal['msg'] = 'install freeipa fail'
        # public.public(json.dumps(signal), chan='freeipa')
        return 1004, signal
    if not join_domain(salt_api, device.get('device_hostname'), **kwargs):
        signal['success'] = False
        signal['msg'] = 'join domain fail'
        # public.public(json.dumps(signal), chan='freeipa')
        return 1005, signal
    # if not lock_local_user(salt_api, device.get('device_hostname'), **kwargs):
    #     signal['success'] = False
    #     signal['msg'] = 'lock local user fail'
    # public.public(json.dumps(signal), chan='freeipa')
    # return 1006, signal
    return 1000, signal


def uos(salt_api, device, signal):
    arg = 'uos.freeipa-client-init.install'
    result = salt_api.deploy(device.get('device_hostname'), arg)
    logger.info('####### celery install_freeipa ######')
    logger.info(result)
    logger.info('#######---######')
    res = result['return'][0]

    if not res.get(device.get('device_hostname')):
        signal['success'] = False
        signal['msg'] = 'install freeipa fail'
        # public.public(json.dumps(signal), chan='freeipa')
        return 1003, signal
    _, _, num_failed = check_output.output(res.get(device.get('device_hostname')))
    if num_failed:
        signal['success'] = False
        signal['msg'] = 'install freeipa fail'
        # public.public(json.dumps(signal), chan='freeipa')
        return 1004, signal
    if not join_domain(salt_api, device.get('device_hostname'), os='uos'):
        signal['success'] = False
        signal['msg'] = 'join domain fail'
        # public.public(json.dumps(signal), chan='freeipa')
        return 1005, signal
    # if not lock_local_user(salt_api, device.get('device_hostname'), os='uos'):
    #     signal['success'] = False
    #     signal['msg'] = 'lock local user fail'
    # public.public(json.dumps(signal), chan='freeipa')
    # return 1006, signal
    return 1000, signal


def kylin(salt_api, device, signal):
    arg = 'kylin.freeipa-client-init.install'
    result = salt_api.deploy(device.get('device_hostname'), arg)
    logger.info('####### celery install_freeipa ######')
    logger.info(result)
    logger.info('#######---######')
    res = result['return'][0]

    if not res.get(device.get('device_hostname')):
        signal['success'] = False
        signal['msg'] = 'install freeipa fail'
        # public.public(json.dumps(signal), chan='freeipa')
        return 1003, signal
    _, _, num_failed = check_output.output(res.get(device.get('device_hostname')))
    if num_failed:
        signal['success'] = False
        signal['msg'] = 'install freeipa fail'
        # public.public(json.dumps(signal), chan='freeipa')
        return 1004, signal
    # if not ntp_sls(salt_api, device.get('device_hostname'), os='kylin'):
    #     signal['success'] = False
    #     signal['msg'] = 'ntp fail'
    #     # public.public(json.dumps(signal), chan='freeipa')
    #     return 1007, signal
    if not join_domain(salt_api, device.get('device_hostname'), os='kylin'):
        signal['success'] = False
        signal['msg'] = 'join domain fail'
        # public.public(json.dumps(signal), chan='freeipa')
        return 1005, signal
    # if not lock_local_user(salt_api, device.get('device_hostname'), os='kylin'):
    #     signal['success'] = False
    #     signal['msg'] = 'lock local user fail'
    #     public.public(json.dumps(signal), chan='freeipa')
    # return 1006, signal
    return 1000, signal


def kylin_v10(salt_api, device, signal):
    arg = 'kylin-v10.freeipa-client-init.install'
    result = salt_api.deploy(device.get('device_hostname'), arg)
    logger.info('####### celery install_freeipa ######')
    logger.info(result)
    logger.info('#######---######')
    res = result['return'][0]

    if not res.get(device.get('device_hostname')):
        signal['success'] = False
        signal['msg'] = 'install freeipa fail'
        # public.public(json.dumps(signal), chan='freeipa')
        return 1003, signal
    _, _, num_failed = check_output.output(res.get(device.get('device_hostname')))
    if num_failed:
        signal['success'] = False
        signal['msg'] = 'install freeipa fail'
        # public.public(json.dumps(signal), chan='freeipa')
        return 1004, signal
    # if not ntp_sls(salt_api, device.get('device_hostname'), os='kylin'):
    #     signal['success'] = False
    #     signal['msg'] = 'ntp fail'
    #     # public.public(json.dumps(signal), chan='freeipa')
    #     return 1007, signal
    if not join_domain(salt_api, device.get('device_hostname'), os='kylin', os_release='kylin v10'):
        signal['success'] = False
        signal['msg'] = 'join domain fail'
        # public.public(json.dumps(signal), chan='freeipa')
        return 1005, signal
    # if not lock_local_user(salt_api, device.get('device_hostname'), os='kylin', os_release='kylin v10'):
    #     signal['success'] = False
    #     signal['msg'] = 'lock local user fail'
    # public.public(json.dumps(signal), chan='freeipa')
    # return 1006, signal
    return 1000, signal


@timeout_func(28)
def salt_ping(salt_api, device):
    num = 0
    while True:
        if num > 10:
            return False
        result = salt_api.test_ping(device.get('device_hostname'))
        logger.info(result)
        if not result or not result[device.get('device_hostname')]:
            num += 1
            time.sleep(3)
        else:
            return result


@app.task(queue='freeipa')
def install_freeipa(device, accounts_id, **kwargs):
    logger.info('####### celery install_freeipa ######')
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    public = RedisHelper()
    signal = {
        'event_type': 'install',
        'device_ip': device.get('device_ip'),
        'device_hostname': device.get('device_hostname'),
        'success': True,
        'msg': 'install freeipa success'
    }

    try:
        result = salt_ping(salt_api, device)
    except TimeoutError:
        signal['success'] = False
        signal['msg'] = 'install freeipa fail not found salt minion'
        public.public(json.dumps(signal), chan='freeipa')
        return 1002

    # os_info = salt_api.cmd_run(device.get('device_hostname'),
    #                            "cat /etc/os-release | grep ID= | grep  -v VERSION_ID | awk -F= '{print $2}'")
    # device_os = os_info.get(device.get('device_hostname'))
    os_info = salt_api.grain(device.get('device_hostname'),
                             ['lsb_distrib_release', 'lsb_distrib_id', 'lsb_distrib_description', 'osarch'])
    device_os = os_info[device.get('device_hostname')].get('lsb_distrib_id')
    os_release = os_info[device.get('device_hostname')].get('lsb_distrib_release')
    os_arch = os_info[device.get('device_hostname')].get('osarch').lower()
    signal['os'] = device_os
    signal['os_release'] = os_release
    signal['os_arch'] = os_arch.lower()
    if get_os(device_os, os_release=os_release) == 'uos':
        code, s = uos(salt_api, device, signal)
    elif get_os(device_os, os_release=os_release) == 'kylin':
        code, s = kylin(salt_api, device, signal)
    elif get_os(device_os, os_release=os_release) == 'kylin-v10':
        code, s = kylin_v10(salt_api, device, signal)
    else:
        code, s = nfs(salt_api, device, signal, os=device_os, os_release=os_release)
    if code == 1000:
        ssh_port_setup(salt_api, device.get('device_hostname'), os=device_os, os_release=os_release)

        if accounts_id != os.environ.get('DOMAIN_SERVER_ADMIN'):
            from devices.utils import bind_device_account
            bind_device_account(device.get('device_hostname'), accounts_id)
        else:
            from accounts.freeipa import hbacrule_add, hbacrule_add_host
            rule_name = 'hbac_' + device.get('device_hostname')
            hbacrule_add(rule_name)
            hbacrule_add_host(rule_name, device.get('device_hostname'))
    public.public(json.dumps(s), chan='freeipa')
    return code
    # return signal


@app.task(queue='freeipa')
def delete_freeipa(device, **kwargs):
    public = RedisHelper()
    signal = {
        'event_type': 'delete',
        'device_ip': device.get('device_ip'),
        'device_hostname': device.get('device_hostname'),
        'success': 'deleting',
        'msg': '退域策略已下发'
    }
    signal.update(kwargs)
    pillar = {
        'type': device.get('is_domain')
    }
    arg = [
        '{os}.freeipa-client-init.exit_domain'.format(os=get_os(device.get('os'), os_release=device.get('os_release'))),
        "pillar={pillar}".format(pillar=pillar),
        'queue=true']
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    salt_api.kill_all_jobs(device.get('device_hostname'))
    result = salt_api.async_deploy(device.get('device_hostname'), arg)
    logger.info('####### celery delete_freeipa ######')
    logger.info(result)
    logger.info('#######---######')
    if isinstance(result, dict):
        signal['success'] = False
        signal['msg'] = '退域失败 {message}'.format(message=result.get('message'))  # delete freeipa fail
        public.public(json.dumps(signal), chan='freeipa')
        return
    # unlock_local_user(device, os=device.get('os'), os_release=device.get('os_release'), chan='freeipa', signal=False)
    # ssh_port_recovery(device, os=device.get('os'), os_release=device.get('os_release'), chan='freeipa', signal=False)
    # stop_minion(device, os=device.get('os'), os_release=device.get('os_release'), chan='freeipa', signal=False)
    public.public(json.dumps(signal), chan='freeipa')
    return


@app.task(queue='default')
def wallpaper_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'wallpaper_policy', 'arg': arg}
    signal.update(kwargs)
    wallpaper = Wallpaper.objects.filter(id=arg).first()
    pillar = {
        'wallpaper': wallpaper.name,
        'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
    }
    pillar.update(kwargs)
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        account = AccountEntity.objects.filter(id=device.get('accounts_id')).first()
        if not device.get('accounts_id') or not account:
            signal['success'] = False
            signal['msg'] = '壁纸 策略设置失败， 设备未绑定用户！'
            signal['status'] = 0
            if 'policy_no' in kwargs.keys():
                pass
            else:
                public.public(json.dumps(signal), chan='policy')
            continue
        pillar['domain_user'] = account.name.lower()
        result = salt_api.async_deploy(device.get('device_hostname'), ['{os}.background-manage.set-background'.format(
            os=get_os(device.get('os'), os_release=device.get('os_release'))), "pillar={pillar}".format(pillar=pillar),
            'queue=true'])
        logger.info('####### celery wallpaper ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '壁纸 策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 3
            if 'policy_no' in kwargs.keys():
                pass
            else:
                public.public(json.dumps(signal), chan='policy')
            # public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '壁纸 策略已经下发'
        signal['status'] = 2
        wallpaper_device, _ = WallpaperDevicePolicy.objects.get_or_create(device_id=device.get('id'))
        wallpaper_device.wallpaper_id = arg
        wallpaper_device.status = signal['status']
        wallpaper_device.policy_no = kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        wallpaper_device.save()

        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': 'wallpaper_policy',
            'arg': arg,
            'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        })
        if 'policy_no' in kwargs.keys():
            pass
        else:
            public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def system_activation_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'system_activation_policy', 'arg': arg}
    signal.update(kwargs)
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            'inline_url': arg,
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.kms-manage.kms-manage".format(os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery system_activation_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '系统激活 策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '系统激活 策略已经下发'
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def firewalld_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'firewalld_policy', 'arg': int(arg)}
    signal.update(kwargs)
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        if int(arg) == 1:
            pillar = {
                'enable': 'true',
                'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
            }
        else:
            pillar = {
                'enable': 'false',
                'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
            }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), ["{os}.firewall-manage.enable_firewall".format(
            os=get_os(device.get('os'), os_release=device.get('os_release'))), "pillar={pillar}".format(pillar=pillar),
            'queue=true'])
        logger.info('####### celery firewalld_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '防火墙 策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 3
            if 'policy_no' in kwargs.keys():
                pass
            else:
                public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '防火墙 策略已经下发'
        signal['status'] = 2
        firewalld_device, _ = FirewalldDevicePolicy.objects.get_or_create(device_id=device.get('id'))
        firewalld_device.firewalld = int(arg)
        firewalld_device.policy_no = kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        firewalld_device.status = signal['status']
        firewalld_device.save()

        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': 'firewalld_policy',
            'arg': arg,
            'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        })
        if 'policy_no' in kwargs.keys():
            pass
        else:
            public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def firewalld_level_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'firewalld_level_policy', 'arg': int(arg)}
    signal.update(kwargs)
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        if int(arg) == 1:
            pillar = {
                'level': str(arg),
                'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
            }
        else:
            pillar = {
                'level': str(arg),
                'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
            }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), ["{os}.firewall-manage.set_firewall_level".format(
            os=get_os(device.get('os'), os_release=device.get('os_release'))), "pillar={pillar}".format(pillar=pillar),
            'queue=true'])
        logger.info('####### celery firewalld_level_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '防火墙等级 策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 3
            if 'policy_no' in kwargs.keys():
                pass
            else:
                public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '防火墙等级 策略已经下发'
        signal['status'] = 2
        firewalld_device, _ = FirewalldDevicePolicy.objects.get_or_create(device_id=device.get('id'))
        firewalld_device.firewalld_level = int(arg)
        firewalld_device.status = signal['status']
        firewalld_device.policy_no = kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        firewalld_device.save()

        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': 'firewalld_level_policy',
            'arg': arg,
            'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        })
        if 'policy_no' in kwargs.keys():
            pass
        else:
            public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def firewalld_black_white_roster_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'set_firewall_roster'}
    signal.update(kwargs)
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            'roster': json.dumps(arg),
            'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'),
                                       ["{os}.firewall-manage.set_firewall_roster".format(
                                           os=get_os(device.get('os'), os_release=device.get('os_release'))),
                                           "pillar={pillar}".format(pillar=pillar),
                                           'queue=true'])
        logger.info('####### celery set_firewall_roster ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '防火墙黑白名单 策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 3
            if 'policy_no' in kwargs.keys():
                pass
            else:
                public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '防火墙黑白名单 策略已经下发'
        signal['status'] = 2
        firewalld_device, _ = FirewalldWhiteBlackRosterDevicePolicy.objects.get_or_create(device_id=device.get('id'))
        firewalld_device.roster = arg
        firewalld_device.policy_no = kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        firewalld_device.status = signal['status']
        firewalld_device.save()

        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': 'firewalld_black_white_roster_policy',
            'arg': arg,
            'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        })
        if 'policy_no' in kwargs.keys():
            pass
        else:
            public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def usb_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'usb_policy', 'arg': arg}
    signal.update(kwargs)

    usb_switch = arg['usb']
    usb_rw_status = arg['usb_rw_status']

    pillar = {
        'usb': usb_switch,
        'usb_rw_status': usb_rw_status,
        'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
    }

    for device in devices:
        if usb_switch == 1:
            usb = ['{os}.device-control.usb_enable'.format(
                os=get_os(device.get('os'), os_release=device.get('os_release'))),
                "pillar={pillar}".format(pillar=pillar), 'queue=true']
        else:
            usb = ['{os}.device-control.usb_disable'.format(
                os=get_os(device.get('os'), os_release=device.get('os_release'))),
                "pillar={pillar}".format(pillar=pillar), 'queue=true']
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        result = salt_api.async_deploy(device.get('device_hostname'), usb)
        logger.info('####### celery usb ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = 'usb 策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 3
            if 'policy_no' in kwargs.keys():
                pass
            else:
                public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = 'usb 策略已经下发'
        signal['status'] = 2
        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': 'usb_policy',
            'arg': usb,
            'usb_rw_status': usb_rw_status,
            'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        })
        usb_device, _ = USBDevicePolicy.objects.get_or_create(device_id=device.get('id'))
        usb_device.usb = usb_switch
        usb_device.status = signal['status']
        usb_device.policy_no = kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        usb_device.usb_rw_status = usb_rw_status
        usb_device.save()
        if 'policy_no' in kwargs.keys():
            pass
        else:
            public.public(json.dumps(signal), chan='policy')

@app.task(queue='default')
def local_user_pwd_change_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'local_user_pwd_change_policy', 'arg': arg}
    signal.update(kwargs)

    pillar = {
        'local_user': arg,
    }

    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        result = salt_api.async_deploy(device.get('device_hostname'), ['{os}.user-manage.local_user_pwd_change'.format(
            os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar), 'queue=true'])
        logger.info('####### celery local_user_pwd_change_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '本地用户密码重置 策略设置失败'
            signal['status'] = 3
            public.public(json.dumps(signal), chan='policy')
            continue
        signal['success'] = True
        signal['msg'] = '本地用户密码重置 策略已经下发'
        signal['status'] = 2
        local_user_pwd, _ = LocalUserPWDPolicy.objects.get_or_create(device_id=device.get('id'))
        local_user_pwd.user = arg
        local_user_pwd.status = signal['status']
        local_user_pwd.save()
        public.public(json.dumps(signal), chan='policy')

@app.task(queue='default')
def reboot_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'reboot_policy', 'arg': arg}
    signal.update(kwargs)
    pillar = {
        'status': arg,
    }
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        result = salt_api.async_deploy(device.get('device_hostname'), ['{os}.reboot-manage.reboot'.format(
            os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar), 'parallel=true'])
        logger.info('####### celery reboot_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '{arg} 策略设置失败， {message}'.format(arg=arg, message=result.get('message'))
            signal['status'] = 3
            public.public(json.dumps(signal), chan='policy')
            continue
        signal['success'] = True
        signal['msg'] = '{arg} 策略已经下发'.format(arg=arg)
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def update_cache_sssd_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'update_cache_sssd_policy', 'arg': arg}
    signal.update(kwargs)
    pillar = {
        'is_offline_login': arg[0],
        'offline_cache_days': arg[1]
    }
    pillar.update(kwargs)
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        result = salt_api.async_deploy(device.get('device_hostname'), ['{os}.common_manage.update_restart_sssd'.format(
            os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar), 'parallel=true'])
        logger.info('####### celery update_cache_sssd_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '离线登入策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 3

            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '离线登入策略已经下发'
        signal['status'] = 2

        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': 'update_cache_sssd_policy',
            'arg': arg,
        })
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def domain_user_unlock_policy(devices, arg, **kwargs):
    """
    解锁终端域用户
    """
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'domain_user_unlock_policy', 'arg': arg}
    signal.update(kwargs)
    pillar = {'domain_user_name': arg}
    pillar.update(kwargs)
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        result = salt_api.async_deploy(device.get('device_hostname'), ['{os}.user-manage.unlock_domain_user'.format(
            os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar), 'parallel=true'])
        logger.info('####### celery domain_user_unlock_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = f"解锁用户{arg}策略失败， {result.get('message')}"
            signal['status'] = 3

            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = f'解锁用户{arg}策略已经下发'
        signal['status'] = 2

        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': 'domain_user_unlock_policy',
            'arg': arg,
        })
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def ssh_policy(devices, arg, **kwargs):
    arg = int(arg)
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'ssh_policy', 'arg': arg}
    signal.update(kwargs)
    pillar = kwargs
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        if arg == 1:
            ssh = [
                '{os}.ssh-manage.ssh_enable'.format(os=get_os(device.get('os'), os_release=device.get('os_release'))),
                "pillar={pillar}".format(pillar=pillar),
                'queue=true']
        else:
            ssh = [
                '{os}.ssh-manage.ssh_disable'.format(os=get_os(device.get('os'), os_release=device.get('os_release'))),
                "pillar={pillar}".format(pillar=pillar),
                'queue=true']
        result = salt_api.async_deploy(device.get('device_hostname'), ssh)
        logger.info('####### celery ssh ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = 'ssh 策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 3
            if 'policy_no' in kwargs.keys():
                pass
            else:
                public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = 'ssh 策略已经下发'
        signal['status'] = 2
        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': 'ssh_policy',
            'arg': arg,
            'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        })
        ssh_device, _ = SSHDevicePolicy.objects.get_or_create(device_id=device.get('id'))
        ssh_device.ssh = arg
        ssh_device.status = signal['status']
        ssh_device.policy_no = kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        ssh_device.save()
        if 'policy_no' in kwargs.keys():
            pass
        else:
            public.public(json.dumps(signal), chan='policy')


def pkg_install_single(device, software_id, softwareInstalledInfo, event_type, software_policy_id=0, task_id=0,
                       **kwargs):
    from software.models import SoftwareInstalledModel, SoftwareInfoModel
    conn = get_redis_connection('redis')
    reprepro_time = conn.get('reprepro_time')
    if not reprepro_time:
        reprepro_time = int(time.time())
        conn.set("reprepro_time", reprepro_time)
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    pkg = softwareInstalledInfo.main_package
    version = softwareInstalledInfo.main_version
    signal = {'event_type': event_type}
    signal.update(kwargs)

    device_id = device.get('id')
    device_ip = device.get('device_ip')
    device_hostname = device.get('device_hostname')
    signal['device_id'] = device_id
    signal['device_ip'] = device_ip
    signal['device_hostname'] = device_hostname
    signal['task_id'] = task_id

    reprepro_addr = settings.HOSTNAME
    software = SoftwareInfoModel.objects.filter(pk=software_id, is_delete=0).first()
    if not software:
        logger.info("software %s is None" % software_id)
        return False

    dependency_str = ""
    if software.dependency_info:
        if isinstance(software.dependency_info, str):
            dependency_info = eval(software.dependency_info)
        else:
            dependency_info = software.dependency_info

        for deb_name in dependency_info:
            deb_path = settings.DEB_FOLDER + deb_name

            tmp_deb_info = getDebInfo(deb_path)
            tmp_pkg = tmp_deb_info['package']
            tmp_version = tmp_deb_info['version']
            dependency_str += ' %s=%s' % (tmp_pkg, tmp_version)
    install_all_str = " %s=%s " % (pkg, version) + dependency_str
    pillar = {
        'pkg': pkg,
        'version': version,
        'install_all_str': install_all_str,
        'reprepro': reprepro_addr,
        'software_id': software_id,
        'software_name': software.name,
        'os_arch': device.get('os_arch').lower() if device.get('os_arch') else 'x86_64',
        'task_id': task_id,
        'reprepro_time': reprepro_time.decode() if isinstance(reprepro_time, bytes) else reprepro_time
    }
    if software_policy_id:
        pillar["software_policy_id"] = software_policy_id
    pillar.update(kwargs)

    os = get_os(device.get('os'), os_release=device.get('os_release'))
    if software.source.id == 1:
        arg = ["{os}.software-manage.install".format(os=os),
               "pillar={pillar}".format(pillar=pillar), 'queue=true']
    else:
        address = urlparse(software.source.address)
        domain_ip = address.hostname
        domain_port = software.source.domain_port if software.source.domain_port else 80
        pillar["domain_ip"] = domain_ip
        pillar["domain_port"] = domain_port
        arg = ["{os}.software-manage.source_install".format(os=os),
               "pillar={pillar}".format(pillar=pillar), 'queue=true']
    logger.info('####### ARG #######')
    logger.info(event_type)
    logger.info(arg)
    logger.info(device_hostname)
    result = salt_api.async_deploy(device_hostname, arg)
    logger.info('####### RES #######')
    logger.info(result)
    file_obj_id = FileManager.objects.get(software_id=software_id, task_id=task_id).id
    if task_id != 0:
        TaskDeviceExec.objects.filter(task__pk=task_id, file_obj=file_obj_id, device__pk=device.get('id')).update(
            exec_status=2)

    if isinstance(result, dict):
        signal['success'] = False
        signal['msg'] = '软件{software_name}安装 失败， {message}'.format(software_name=softwareInstalledInfo.name,
                                                                         message=result.get('message'))
        softwareInstalledInfo.installed_status = SoftwareInstalledModel.STATUS_FAILED
        softwareInstalledInfo.save()
        if task_id != 0:
            TaskDeviceExec.objects.filter(task__pk=task_id, file_obj=file_obj_id, device__pk=device.get('id')).update(
                exec_status=5)

        RedisHelper().public(json.dumps(signal), chan='policy')
        return

    # signal['success'] = True
    # signal['msg'] = '软件{software_name}安装 策略已经下发'.format(software_name=softwareInstalledInfo.name)
    # signal['status'] = SoftwareInstalledModel.STATUS_RUNNING
    sender_params = {
        'device_id': device_id,
        'device_ip': device_ip,
        'policy': event_type,
        'arg': software_id,
        'task_id': task_id
    }
    if software_policy_id:
        sender_params["software_policy_id"] = software_policy_id
    policy_singnal_sender(sender_params)

    softwareInstalledInfo.installed_status = SoftwareInstalledModel.STATUS_RUNNING
    softwareInstalledInfo.save()
    # RedisHelper().public(json.dumps(signal), chan='policy')

    # res = salt_api.pkg_install(device_hostname, arg)
    # logger.info(res)

    # data = res.get(device_hostname)
    # status = SoftwareInstalledModel.STATUS_FAILED
    # if data is None:
    #     policy_singnal_sender({
    #         'device_id':device_id,
    #         'device_ip':device_ip,
    #         'policy': event_type,
    #         'arg':arg
    #     })

    #     signal['success'] = False
    #     signal['msg'] = '软件安装策略失败，组件未启动'
    # else:
    #     # 失败时data对象是字符串
    #     if isinstance(data, str):
    #         signal['success'] = False
    #         signal['msg'] = '软件安装失败'
    #         logger.error('安装失败 %s', data)
    #     # 已安装返回空，认为成功
    #     elif data.get(arg) is not None or not bool(data):
    #         signal['success'] = True
    #         signal['msg'] = '软件安装成功'
    #         logger.info('包信息 %s', data)
    #         # 安装成功后 修改状态
    #         status = SoftwareInstalledModel.STATUS_INSTALLED
    #     else:
    #         signal['success'] = False
    #         signal['msg'] = '软件安装失败'
    #         logger.error('安装失败 %s', data)

    # softwareInstalledInfo.installed_status = status
    # # softwareInstalledInfo.is_histroy_install = 0
    # softwareInstalledInfo.save()

    # RedisHelper().public(json.dumps(signal), chan='policy')


# 软件安装策略
# @app.task(queue='default', name='pkg_install_policy')
@app.task(queue='default', time_limite=600)
def pkg_install_policy(device, software_id, software_policy_id=0, **kwargs):
    logger.info("pkg_install_policy")
    from software.models import SoftwareInstalledModel
    from software.models import SoftwareInfoModel

    if isinstance(device, list):
        device = device[0]

    software = SoftwareInfoModel.objects.filter(pk=software_id, is_delete=0).first()
    if not software:
        logger.info("software %s is None" % software_id)
        return False
    softwareInstalledInfo = SoftwareInstalledModel.objects.filter(main_package=software.main_package,
                                                                  main_version=software.main_version).filter(
        device_info=device.get('id')).first()
    if not softwareInstalledInfo:
        softwareInstalledInfo = SoftwareInstalledModel()
        softwareInstalledInfo.software_info = software
        softwareInstalledInfo.name = software.name
        softwareInstalledInfo.main_package = software.main_package
        softwareInstalledInfo.main_version = software.main_version
        softwareInstalledInfo.device_info_id = device.get('id')
        softwareInstalledInfo.save()
        logger.info('创建新的软件安装记录')

    event_type = 'pkg_install_policy'
    pkg_install_single(device, software_id, softwareInstalledInfo, event_type, software_policy_id)


# 软件安装策略    批量安装
@app.task(queue='default', name='pkg_install_batch_policy', time_limite=600)
def pkg_install_batch_policy(device, software_installed_ids):
    from software.models import SoftwareInstalledModel

    softwareInstalledInfos = SoftwareInstalledModel.objects.filter(pk__in=software_installed_ids)

    event_type = 'pkg_install_policy'
    for softwareInstalledInfo in softwareInstalledInfos:
        pkg_install_single(device, softwareInstalledInfo.software_info_id, softwareInstalledInfo, event_type)


# 软件卸载策略
@app.task(queue='default', name='pkg_remove_policy')
def pkg_remove_policy(device, software_id, software_policy_id, **kwargs):
    from software.models import SoftwareInstalledModel
    from software.models import SoftwareInfoModel

    if isinstance(device, list):
        device = device[0]

    software = SoftwareInfoModel.objects.filter(pk=software_id, is_delete=0).first()
    if not software:
        logger.info("software %s is None" % software_id)
        return False
    softwareInstalledInfo = SoftwareInstalledModel.objects.filter(device_info_id=device.get('id')).filter(
        main_package=software.main_package, main_version=software.main_version).first()

    event_type = 'pkg_remove_policy'
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': event_type}

    device_id = device.get('id')
    device_ip = device.get('device_ip')
    device_hostname = device.get('device_hostname')
    signal['device_id'] = device_id
    signal['device_ip'] = device_ip
    signal['device_hostname'] = device_hostname

    pillar = {
        'pkg': software.main_package,
        'version': software.main_version,
        'software_id': software_id,
        'software_name': software.name
    }
    if software_policy_id:
        pillar["software_policy_id"] = software_policy_id
    pillar.update(kwargs)
    os = get_os(device.get('os'), os_release=device.get('os_release'))
    arg = ["{os}.software-manage.remove".format(os=os),
           "pillar={pillar}".format(pillar=pillar), 'queue=true']
    logger.info('####### ARG %s #######', event_type)
    logger.info(arg)
    logger.info(device_hostname)
    result = salt_api.async_deploy(device_hostname, arg)
    logger.info('####### RES %s #######', event_type)
    logger.info(result)

    if isinstance(result, dict):
        signal['success'] = False
        signal['msg'] = '软件{software_name}卸载 失败， {message}'.format(software_name=softwareInstalledInfo.name,
                                                                         message=result.get('message'))
        signal['status'] = SoftwareInstalledModel.STATUS_FAILED
        softwareInstalledInfo.installed_status = SoftwareInstalledModel.STATUS_FAILED
        softwareInstalledInfo.save()
        RedisHelper().public(json.dumps(signal), chan='policy')
        return

    signal['success'] = True
    signal['msg'] = '软件{software_name}卸载 策略已经下发'.format(software_name=softwareInstalledInfo.name)
    signal['status'] = SoftwareInstalledModel.STATUS_RUNNING
    if software_policy_id:
        signal["software_policy_id"] = software_policy_id

    sender_params = {
        'device_id': device_id,
        'device_ip': device_ip,
        'policy': event_type,
        'arg': software_id
    }
    if software_policy_id:
        sender_params["software_policy_id"] = software_policy_id
    policy_singnal_sender(sender_params)
    RedisHelper().public(json.dumps(signal), chan='policy')
    return

    # res = salt_api.pkg_remove(device_hostname, arg)
    # logger.info(res)

    # data = res.get(device_hostname)
    # if data is None:
    #     policy_singnal_sender({
    #         'device_id':device_id,
    #         'device_ip':device_ip,
    #         'policy': event_type,
    #         'arg':arg
    #     })

    #     signal['success'] = False
    #     signal['msg'] = '软件卸载策略失败，组件未启动'
    # else:
    #     # 失败时data对象是字符串
    #     if isinstance(data, str):
    #         signal['success'] = False
    #         signal['msg'] = '软件卸载失败'
    #         logger.error('卸载失败 %s', data)
    #     # 已安装返回空，认为成功
    #     elif data.get(arg) is not None or not bool(data):
    #         signal['success'] = True
    #         signal['msg'] = '软件卸载成功'
    #         logger.info('包信息 %s', data)

    #         #  卸载成功后 修改状态
    #         softwareInstalledInfo = SoftwareInstalledModel.objects.get(pk=software_installed_id)
    #         softwareInstalledInfo.installed_status = SoftwareInstalledModel.STATUS_REMOVED
    #         # softwareInstalledInfo.is_histroy_install = 1
    #         softwareInstalledInfo.save()
    #     else:
    #         signal['success'] = False
    #         signal['msg'] = '软件卸载失败'
    #         logger.error('卸载失败 %s', data)

    # RedisHelper().public(json.dumps(signal), chan='policy')


# 软件默认安装策略
@app.task(queue='default', name='pkg_install_default_policy')
def pkg_install_default_policy():
    from software.models import SoftwareInfoModel, SoftwareInstalledModel, SoftwarePolicy
    from utils.utils import str_to_list, get_device
    logger.info("pkg_install_default_policy")

    try:
        # 不过滤已成功的策略是为了兼容策略执行成功后，部门新添加了用户
        policy_list = SoftwarePolicy.objects.filter(policy_status=1, is_delete=0)

        for policy in policy_list:
            device_list = get_device(policy.range, policy.org_id, policy.device_id)

            software_list = str_to_list(policy.software_id)
            software_policy_id = policy.id

            device_data = DeviceSerializer(device_list, many=True).data
            for device in device_data:
                for software_id in software_list:
                    software = SoftwareInfoModel.objects.filter(pk=software_id, is_delete=0).first()
                    if not software:
                        logger.info("software %s is None" % software_id)
                        continue
                    softwareInstalledInfo = SoftwareInstalledModel.objects.filter(main_package=software.main_package,
                                                                                  main_version=software.main_version).filter(
                        device_info=device.get("id")).first()
                    if not softwareInstalledInfo:
                        softwareInstalledInfo = SoftwareInstalledModel()
                        softwareInstalledInfo.software_info = software
                        softwareInstalledInfo.name = software.name
                        softwareInstalledInfo.main_package = software.main_package
                        softwareInstalledInfo.main_version = software.main_version
                        softwareInstalledInfo.device_info_id = device.get('id')
                        softwareInstalledInfo.save()
                        logger.info('创建新的软件安装记录')

                    event_type = 'pkg_install_policy'

                    if softwareInstalledInfo.installed_status == SoftwareInstalledModel.STATUS_INSTALLED:
                        # installed status
                        pass
                    else:
                        pkg_install_single(device, software_id, softwareInstalledInfo, event_type, software_policy_id)
    except Exception as exp:
        logger.info(f"**software********{exp}************")


# 浏览器策略 按照设备下发 开机默认执行
@app.task(queue='default', name='browser_default_policy')
def browser_default_policy(device, **kwargs):
    event_type = 'browser_default_policy'
    publicer = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    logger.info("browser_default_policy")

    device_id = device.get("id")

    bind_accounts_list = Devices.objects.filter(id=device_id).values_list('bind_account', flat=True)
    if not bind_accounts_list:
        logger.info('####### not found bind account for device ######')
        return
    for account_id in bind_accounts_list:
        # 对每一个绑定的账号进行下发
        browser_deploy_policy(device, USER_POLICY_TYPE, account_id)
        browser_deploy_policy(device, APPLICATION_POLICY_TYPE, account_id)
        browser_deploy_policy(device, BOOKMARK_POLICY_TYPE, account_id)


# 按照设备  策略类型  登录人 下发
@app.task(queue='default', name='browser_deploy_policy')
def browser_deploy_policy(device, policy_type, account_id, **kwargs):
    event_type = 'deploy_policy_browser'
    publicer = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    update_seq = kwargs.get("update_seq", 0)
    policy_id = kwargs.get("policy_id", 0)
    # try:
    device_id = device.get("id")
    file_name = str(account_id) + ".json"
    pillar = {
        'file_name': file_name,
        'browser_policy_type': policy_type,
        'browser_policy_id': policy_id,
    }

    account = AccountEntity.objects.filter(id=account_id).first()
    if account:
        pillar['domain_user'] = account.name.lower()
    else:
        pillar['domain_user'] = "root"
    device_hostname = device.get('device_hostname')
    if not device_hostname:
        logger.info(f"device hostname not found...")
        return
    if device.get('os'):
        system_os = get_os(device.get('os'), os_release=device.get('os_release'))
    else:
        system_os = get_os("", os_release=device.get('os_release') if device.get('os_release') else '')
    device_ip = device.get('device_ip')
    result = salt_api.async_deploy(device_hostname, ['{os}.browser-control.policy_file_{policy_type}'.format(
        os=system_os, policy_type=policy_type), "pillar={pillar}".format(pillar=pillar),
        'queue=true'])

    logger.info(
        f"deploy policy type {policy_type},account id {account_id} to device {device_hostname},result is {result}")
    signal = {'event_type': event_type, 'arg': device}
    signal.update(kwargs)

    signal['device_id'] = device_id
    signal['device_ip'] = device_ip
    signal['device_hostname'] = device_hostname

    browserPolicyLog = BrowserPolicyLog()
    browserPolicyLog.account_id = account_id
    browserPolicyLog.update_seq = update_seq
    browserPolicyLog.policy = policy_id
    browserPolicyLog.device_id = device_id
    browserPolicyLog.device_ip = device_ip
    browserPolicyLog.device_hostname = device_hostname
    policy_type_str = ""
    if policy_type == USER_POLICY_TYPE:
        browserPolicyLog.policy_type = 0
        policy_type_str = "用户策略"
    if policy_type == APPLICATION_POLICY_TYPE:
        browserPolicyLog.policy_type = 1
        policy_type_str = "新标签页应用"
    if policy_type == BOOKMARK_POLICY_TYPE:
        browserPolicyLog.policy_type = 2
        policy_type_str = "书签"
    jid = result
    if isinstance(result, dict):
        signal['success'] = False
        signal['msg'] = '浏览器 策略设置下发失败， {message}'.format(message=result.get('message'))
        jid = result['jid']
        logger.info(f"策略设置下发失败，result is {result}")
    else:
        signal['success'] = True
        signal['msg'] = '浏览器 ' + policy_type_str + ' 策略设置已经下发'

    browserPolicyLog.jobid = jid
    browserPolicyLog.save()

    publicer.public(json.dumps(signal), chan='policy')
    publicer.public(json.dumps({"event": "browser_deploy_policy", "account_id": account_id, "policy_type": policy_type,
                                "policy_id": policy_id, "update_seq": update_seq, "jid": jid})
                    , chan='browser_policy_chan')


@app.task(queue='default')
def delete_policy_forder(device, **kwargs):
    public = RedisHelper()
    signal = {
        'event_type': 'delete browser_policy',
        'device_ip': device.get('device_ip'),
        'device_hostname': device.get('device_hostname'),
        'success': 'deleting',
        'msg': '浏览器 退域策略设置已下发'
    }
    signal.update(kwargs)
    pillar = {
        'type': device.get('is_domain')
    }

    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    system_os = get_os(device.get('os'), os_release=device.get('os_release'))
    device_hostname = device.get('device_hostname')
    result = salt_api.async_deploy(device_hostname, ['{os}.browser-control.policy_file_delete'.format(
        os=system_os), "pillar={pillar}".format(pillar=pillar),
        'queue=true'])

    if isinstance(result, dict):
        signal['success'] = False
        signal['msg'] = '退域失败 {message}'.format(message=result.get('message'))  # delete freeipa fail
        public.public(json.dumps({"event": "browser_policy_delete", "signal": signal})
                      , chan='browser_policy_chan')
        return

    public.public(json.dumps({"event": "browser_policy_delete", "signal": signal})
                  , chan='browser_policy_chan')
    return


# 证书策略 执行
@app.task(queue='default', name='ca_apply_policy', time_limite=600)
def ca_apply_policy(device, cainfo, **kwargs):
    try:
        event_type = 'ca_apply_policy'
        public = RedisHelper()
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        signal = {'event_type': event_type, 'arg': cainfo}
        signal.update(kwargs)
        pillar = {
            'certname': cainfo['name'],
            'certfile': cainfo['custome_filename'],
            'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        }

        account = AccountEntity.objects.filter(id=device.get('accounts_id')).first()

        pillar['domain_user'] = account.name.lower()
        result = salt_api.async_deploy(device.get('device_hostname'), [
            '{os}.cert-manager.set-cert'.format(os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar), 'queue=true'])

        logger.info(result)
        logger.info('#######--result_ca-######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = 'CA 策略设置失败， {message}'.format(message=result.get('message'))
            if 'policy_no' in kwargs.keys():
                pass
            else:
                public.public(json.dumps(signal), chan='policy')
            return

        signal['success'] = True
        signal['msg'] = 'CA 策略设置已经下发'
        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': event_type,
            'arg': cainfo,
            'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        })
        if 'policy_no' in kwargs.keys():
            pass
        else:
            public.public(json.dumps(signal), chan='policy')
    except Exception as exp:
        logger.info(f"**ca********{exp}************")


@app.task(queue='default')
def check_sudo_policy(devices, arg, **kwargs):
    """
    重启sssd服务激活root权限
    :param arg: 开关状态
    :param devices: 设备列表
    :param kwargs:
    :return:
    """
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'check_sudo_policy'}
    signal.update(kwargs)
    pillar = {
        'on_off': arg.get('on_off'),
        'domain_name': arg.get("domain_name"),
        'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
    }
    account_id = arg.get('account_id')
    pillar.update(kwargs)
    if not devices:
        signal['device_id'] = None
        signal['device_ip'] = arg.get("domain_user")
        signal['status'] = 1
        signal['success'] = True
        signal['msg'] = 'root 策略设置成功'
        if 'policy_no' in kwargs.keys():
            pass
        else:
            public.public(json.dumps(signal), chan='policy')
    else:
        for device in devices:
            # logger.info(f'####### {device} ######')
            signal['device_id'] = device.get('id')
            signal['device_ip'] = device.get('device_ip')
            signal['device_hostname'] = device.get('device_hostname')
            account = AccountEntity.objects.filter(id=account_id).first()
            # logger.info(f'##1##### {account} ######')
            if not device.get('bind_account') or not account:
                signal['success'] = False
                signal['msg'] = 'check_sudo 策略设置失败， 设备未绑定用户！'
                signal['status'] = 0
                if 'policy_no' in kwargs.keys():
                    pass
                else:
                    public.public(json.dumps(signal), chan='policy')
                continue
            # 使用屏幕保护的用户
            pillar['domain_user'] = account.name.lower()
            # res = salt_api.test_ping(device.get('device_ip'))
            result = salt_api.async_deploy(
                device.get('device_hostname'),
                ['{os}.sudo-manage.check_sudo_permission'.format(
                    os=get_os(device.get('os'), os_release=device.get('os_release'))),
                    "pillar={pillar}".format(pillar=pillar),
                    'queue=true']
            )
            logger.info('####### celery restart_root_sssd ######')
            logger.info(result)
            logger.info('#######---######')
            if isinstance(result, dict):
                signal['success'] = False
                signal['msg'] = 'root 策略设置失败， 组件未启动'
                signal['status'] = 3
                if 'policy_no' in kwargs.keys():
                    pass
                else:
                    public.public(json.dumps(signal), chan='policy')
                continue

            signal['success'] = True
            signal['msg'] = 'root 策略已下发'
            signal['status'] = 2

            policy_singnal_sender(
                {
                    'device_id': device.get('id'),
                    'device_ip': device.get('device_ip'),
                    'policy': 'check_sudo_policy',
                    'arg': arg,
                    'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
                }
            )

            if 'policy_no' in kwargs.keys():
                pass
            else:
                public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def watermark_policy(devices, arg, **kwargs):
    """
    水印策略设置
    """
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'watermark_policy'}
    signal.update(arg)

    if arg.get('watermark_position') == '全屏':
        watermark_position = 0
    elif arg.get('watermark_position') == '左上角':
        watermark_position = 1
    elif arg.get('watermark_position') == '右上角':
        watermark_position = 2
    elif arg.get('watermark_position') == '左下角':
        watermark_position = 3
    elif arg.get('watermark_position') == '右下角':
        watermark_position = 4
    elif arg.get('watermark_position') == '屏幕中心':
        watermark_position = 5
    else:
        watermark_position = 0

    # 显示内容的类型转换为字符串
    watermark_defined_text_type = arg.get('watermark_defined_text_type')
    watermark_defined_text_type_str = ','.join(str(x) for x in watermark_defined_text_type)

    hex_16_watermark_font_transparency = hex(int(float(arg.get('watermark_font_transparency')) * 255))

    watermark_setting_text = (f"{arg.get('user_defined_text')}\n"
                              f"{watermark_position}\n"
                              f"{arg.get('watermark_rotate')}\n"
                              f"{arg.get('watermark_font')}\n"
                              f"{arg.get('watermark_font_color')}\n"
                              f"{arg.get('watermark_font_size')}\n")
                              #f"{hex_16_watermark_font_transparency[2:]}")
    if arg.get('watermark_switch_status') == 1:
        watermark_setting_text = watermark_setting_text + f"{hex_16_watermark_font_transparency[2:]}"
    else:
        watermark_setting_text = watermark_setting_text + "00"

    pillar = {
        "watermark_switch_status": arg.get('watermark_switch_status'),
        "watermark_setting_text": watermark_setting_text,
        "watermark_defined_text_type": watermark_defined_text_type_str,
    }
    pillar.update(kwargs)

    if not devices:
        signal['device_id'] = None
        signal['device_ip'] = None
        signal['status'] = 1
        signal['success'] = True
        signal['msg'] = '没有设备去设置水印策略'
        public.public(json.dumps(signal), chan='policy')
    else:
        for device in devices:
            signal['device_id'] = device.get('id')
            signal['device_ip'] = device.get('device_ip')
            signal['device_hostname'] = device.get('device_hostname')
            if not device.get('bind_account'):
                signal['success'] = False
                signal['msg'] = '水印 策略设置失败， 设备未绑定用户！'
                signal['status'] = 0
                public.public(json.dumps(signal), chan='policy')
                continue
            pillar['domain_name'] = device.get('device_hostname')
            result = salt_api.async_deploy(
                device.get('device_hostname'),
                ['{os}.watermark-manage.set-watermark'.format(
                    os=get_os(device.get('os'), os_release=device.get('os_release'))),
                    "pillar={pillar}".format(pillar=pillar),
                    'queue=true']
            )
            logger.info('####### celery watermark_policy ######')
            logger.info(result)
            logger.info('#######---######')
            if isinstance(result, dict):
                signal['success'] = False
                signal['msg'] = '水印 策略设置失败， 组件未启动'
                signal['status'] = 3
                public.public(json.dumps(signal), chan='policy')
                continue

            signal['success'] = True
            signal['msg'] = '水印 策略已下发'
            signal['status'] = 2

            policy_singnal_sender(
                {
                    'device_id': device.get('id'),
                    'device_ip': device.get('device_ip'),
                    'policy': 'watermark_policy',
                    'arg': arg,
                }
            )

            public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def compress_upload_file_policy(devices, arg, **kwargs):
    """
    压缩下载文件
    :param arg: 开关状态
    :param devices: 设备列表
    :param kwargs:
    :return:
    """
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'compress_upload_file_policy'}
    signal.update(kwargs)
    pillar = {
        'file_path': arg.get('file_path'),
        'tmp_filename': os.path.basename(arg.get('file_path')),
        'domain_name': arg.get("domain_name"),
        'upload_file_name': f"nfs_domain_file_{datetime.now().strftime('%Y-%m-%d-%H:%M:%S')}.tar.gz",
        'username': arg.get("username"),
        'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
    }
    pillar.update(kwargs)
    if not devices:
        signal['device_id'] = None
        signal['device_ip'] = arg.get("domain_user")
        signal['status'] = 1
        signal['success'] = True
        signal['msg'] = '下载日志文件失败'
        if 'policy_no' in kwargs.keys():
            pass
        else:
            public.public(json.dumps(signal), chan='policy')
    else:
        for device in devices:
            logger.info(f'####### {device} ######')
            signal['device_id'] = device.get('id')
            signal['device_ip'] = device.get('device_ip')
            signal['device_hostname'] = device.get('device_hostname')
            # logger.info(f'##1##### {account} ######')
            if not device.get('bind_account'):
                signal['success'] = False
                signal['msg'] = 'compress_upload_file_policy 策略设置失败， 设备未绑定用户！'
                signal['status'] = 0
                if 'policy_no' in kwargs.keys():
                    pass
                else:
                    public.public(json.dumps(signal), chan='policy')
                continue
            # res = salt_api.test_ping(device.get('device_ip'))
            pillar['domain_name'] = device.get('device_hostname')
            result = salt_api.async_deploy(
                device.get('device_hostname'),
                ['{os}.file_operations.compress_and_upload'.format(
                    os=get_os(device.get('os'), os_release=device.get('os_release'))),
                    "pillar={pillar}".format(pillar=pillar),
                    'queue=true']
            )
            logger.info('####### celery compress_upload_file_policy ######')
            logger.info(result)
            logger.info('#######---######')
            if isinstance(result, dict):
                signal['success'] = False
                signal['msg'] = 'compress_upload_file_policy 策略设置失败， 组件未启动'
                signal['status'] = 3
                if 'policy_no' in kwargs.keys():
                    pass
                else:
                    public.public(json.dumps(signal), chan='policy')
                continue

            signal['success'] = True
            signal['msg'] = 'compress_upload_file_policy 策略已下发'
            signal['status'] = 2

            policy_singnal_sender(
                {
                    'device_id': device.get('id'),
                    'device_ip': device.get('device_ip'),
                    'policy': 'compress_upload_file_policy',
                    'arg': arg,
                    'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
                }
            )

            if 'policy_no' in kwargs.keys():
                pass
            else:
                public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def screensavers_policy(devices, arg, **kwargs):
    """
    屏幕保护策略
    :param devices: 设备列表
    :param arg:  屏幕保护程序时间  字典
    :param kwargs:
    :return:
    """
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'screensavers_policy', 'arg': arg}
    signal.update(kwargs)
    pillar = {
        'time': arg.get('interval'),
        'switch_status': arg.get('switch_status'),
        'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
    }
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        # res = salt_api.test_ping(device.get('device_ip'))
        result = salt_api.async_deploy(
            device.get('device_hostname'),
            ['{os}.screen-protect-manage.set-lock-screen-time'.format(
                os=get_os(device.get('os'), os_release=device.get('os_release'))),
                "pillar={pillar}".format(pillar=pillar),
                'queue=true']
        )
        logger.info('####### celery screensavers ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '锁屏 策略设置失败， 组件未启动'
            signal['status'] = 3
            if 'policy_no' in kwargs.keys():
                pass
            else:
                public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '锁屏策略已下发'
        signal['status'] = 2

        screensavers_device, _ = ScreensaversDevicePolicy.objects.get_or_create(device_id=device.get('id'))
        screensavers_device.interval = arg.get('interval')
        screensavers_device.policy_no = kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
        screensavers_device.switch_status = arg.get('switch_status')
        screensavers_device.status = signal['status']
        screensavers_device.save()

        policy_singnal_sender(
            {
                'device_id': device.get('id'),
                'device_ip': device.get('device_ip'),
                'policy': 'screensavers_policy',
                'arg': arg,
                'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
            }
        )

        if 'policy_no' in kwargs.keys():
            pass
        else:
            public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def task_file_policy(**param):
    """
    :param param:
        {
            "devices":devices,
            "task_id":1,
            "local_path":"/0/14/2023-02-27/nfsdomain-install.sh"
            "file_name":"/0/14/2023-02-27/nfsdomain-install.sh"
        }
    :return:
    """
    logger.info(f"param :{param}")
    devices = param.get("devices")
    task_id = param.get("task_id")
    local_path = param.get("local_path")
    file_name = param.get("file_name")
    quality = param.get("quality")
    exec_time = param.get("exec_time")

    device_id_str = param.get("device_id_str", "")
    # check_task_device(task_id)
    if not devices:
        from devices.models import Devices
        device_id_list = device_id_str.split(",")
        device_list = Devices.objects.filter(id__in=device_id_list)
        devices = DeviceSerializer(device_list, many=True).data

    event_type = 'task_file_policy'
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    task = Tasks.objects.get(pk=task_id)
    pillar = {
        'task_id': task_id,
        'file_name': file_name,
        'local_path': local_path,
        'exec_time': exec_time
    }
    signal = {'event_type': event_type, 'arg': pillar}
    for device in devices:
        logger.info("task_id %s ,action device %s" % (task_id, device["id"]))
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        account = AccountEntity.objects.filter(id=device.get('accounts_id')).first()
        if account:
            pillar['domain_user'] = account.name.lower()
            # 设置默认为域用户的家目录
            home_path = "/home/%s" % account.name.lower()
        else:
            # 无域用户时,为home目录下
            home_path = "/home/"
            pillar['domain_user'] = "root"
        task_file_path = task.file_path if task.file_path else home_path
        tar_name = "%s.tar" % (str(task_id))
        download_path = os.path.join(task_file_path, tar_name)
        pillar['download_path'] = download_path
        pillar['download_dir'] = task_file_path

        # result = salt_api.async_deploy(device.get('device_hostname'), ['{os}.task-manage.task_file'.format(
        #     os=get_os(device.get('os'), os_release=device.get('os_release'))), "pillar={pillar}".format(pillar=pillar),
        #     'queue=true'])
        system_os = get_os(device.get('os'), os_release=device.get('os_release'))
        device_hostname = device.get('device_hostname')
        if quality == 0:
            result = salt_api.async_deploy(device.get('device_hostname'), ['{os}.task-manage.task_file'.format(
                os=system_os), "pillar={pillar}".format(pillar=pillar),
                'queue=true'])
            TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=2)
        elif quality == 1:
            # 添加定时任务
            result = salt_api.async_deploy(device_hostname, ['{os}.task-manage.task_file_schedule'.format(
                os=system_os), "pillar={pillar}".format(pillar=pillar), 'queue=true'])
            # 下发到客户端
            salt_api.refresh_pillar(device_hostname)
        else:
            # 添加计划任务
            result = salt_api.async_deploy(device_hostname, ['{os}.task-manage.task_file_crontab'.format(
                os=system_os), "pillar={pillar}".format(pillar=pillar), 'queue=true'])
            # 下发到客户端
            salt_api.refresh_pillar(device_hostname)
            TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=2)
        logger.info('####### celery shell ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = 'task_file 策略设置失败， {message}'.format(message=result.get('message'))
            public.public(json.dumps(signal), chan='policy')
            TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=5)
            return

        signal['success'] = True
        signal['msg'] = 'task_file 策略设置已经下发'

        offline_arg = {
            'devices': [device],
            'device_id_str': str(device.get('id')),
            'task_id': task_id,
            'file_name': file_name,
            'tmp_file_name': param.get("tmp_file_name"),
            "quality": param.get("quality")
        }

        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': event_type,
            'arg': offline_arg
        })
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def task_shell_policy(**param):
    """
    :param param:
        {
            "devices":devices,
            "task_id":1,
            "tmp_file_name":"nfsdomain-install.sh",
            "file_name":"/0/14/2023-02-27/nfsdomain-install.sh",
        }
    :return:
    """
    devices = param.get("devices")
    task_id = param.get("task_id")
    file_name = param.get("file_name")
    tmp_file_name = param.get("tmp_file_name")
    task_class = param.get("task_class")
    quality = param.get("quality")
    exec_time = param.get("exec_time")

    device_id_str = param.get("device_id_str", "")

    job_name = "task_%s" % (task_id)
    # check_task_device(task_id)
    if not devices:
        from devices.models import Devices
        device_id_list = device_id_str.split(",")
        device_list = Devices.objects.filter(id__in=device_id_list)
        devices = DeviceSerializer(device_list, many=True).data

    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)

    pillar = {
        'tmp_file_name': tmp_file_name,
        'file_name': file_name,
        'task_id': task_id,
        'exec_time': exec_time
    }
    event_type = 'task_shell_policy'
    signal = {'event_type': event_type, 'arg': pillar}

    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        system_os = get_os(device.get('os'), os_release=device.get('os_release'))
        device_hostname = device.get('device_hostname')
        if quality == 0:
            result = salt_api.async_deploy(device.get('device_hostname'), ['{os}.task-manage.task_shell'.format(
                os=system_os), "pillar={pillar}".format(pillar=pillar),
                'queue=true'])
            TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=2)
        elif quality == 1:
            # 添加定时任务
            result = salt_api.async_deploy(device_hostname, ['{os}.task-manage.task_shell_schedule'.format(
                os=system_os), "pillar={pillar}".format(pillar=pillar), 'queue=true'])
            # 启用任务
            salt_api.schedule_enable_job(device_hostname, job_name)
            # 下发到客户端
            salt_api.refresh_pillar(device_hostname)
        else:
            # 添加计划任务
            result = salt_api.async_deploy(device_hostname, ['{os}.task-manage.task_shell_crontab'.format(
                os=system_os), "pillar={pillar}".format(pillar=pillar), 'queue=true'])
            # 启用任务
            salt_api.schedule_enable_job(device_hostname, job_name)
            # 下发到客户端
            salt_api.refresh_pillar(device_hostname)
            TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=2)

        logger.info('####### celery shell ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = 'task_shell 策略设置失败， {message}'.format(message=result.get('message'))
            public.public(json.dumps(signal), chan='policy')
            TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=5)
            return

        signal['success'] = True
        signal['msg'] = 'task_shell 策略设置已经下发'

        offline_arg = {
            'devices': [device],
            'device_id_str': str(device.get('id')),
            'task_id': task_id,
            'file_name': file_name,
            'tmp_file_name': param.get("tmp_file_name"),
            "quality": param.get("quality")
        }

        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': event_type,
            'arg': offline_arg
        })
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def task_system_patches_policy(**param):
    devices = param.get("devices")
    task_id = param.get("task_id")
    file_name = param.get("file_name")
    tmp_file_name = param.get("tmp_file_name")
    quality = param.get("quality")
    exec_time = param.get("exec_time")

    device_id_str = param.get("device_id_str", "")
    # check_task_device(task_id)
    if not devices:
        from devices.models import Devices
        device_id_list = device_id_str.split(",")
        device_list = Devices.objects.filter(id__in=device_id_list)
        devices = DeviceSerializer(device_list, many=True).data

    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    event_type = 'task_system_patches_policy'
    signal = {'event_type': event_type, 'arg': param}
    pillar = {
        'task_id': task_id,
        'file_name': file_name,
        'exec_time': exec_time,
        'tmp_file_name': tmp_file_name
    }
    for device in devices:
        try:
            signal['device_id'] = device.get('id')
            signal['device_ip'] = device.get('device_ip')
            signal['device_hostname'] = device.get('device_hostname')
            system_os = get_os(device.get('os'), os_release=device.get('os_release'))
            device_hostname = device.get('device_hostname')
            if quality == 0:
                result = salt_api.async_deploy(device.get('device_hostname'),
                                               ['{os}.task-manage.task_system_patches'.format(
                                                   os=system_os), "pillar={pillar}".format(pillar=pillar),
                                                   'queue=true'])
                TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=2)
            elif quality == 1:
                # 添加定时任务
                result = salt_api.async_deploy(device_hostname, ['{os}.task-manage.task_system_patches_schedule'.format(
                    os=system_os), "pillar={pillar}".format(pillar=pillar), 'queue=true'])
                # 下发到客户端
                salt_api.refresh_pillar(device_hostname)
            else:
                # 添加计划任务
                result = salt_api.async_deploy(device_hostname, ['{os}.task-manage.task_system_patches_crontab'.format(
                    os=system_os), "pillar={pillar}".format(pillar=pillar), 'queue=true'])
                # 下发到客户端
                salt_api.refresh_pillar(device_hostname)
                TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=2)
            logger.info('####### celery system_patches ######')
            logger.info(result)
            logger.info('#######---######')
            if isinstance(result, dict):
                signal['success'] = False
                signal['msg'] = 'task_system_patches 策略设置失败， {message}'.format(message=result.get('message'))
                public.public(json.dumps(signal), chan='policy')
                TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=5)
                return

            signal['success'] = True
            signal['msg'] = 'task_system_patches 策略设置已经下发'

            offline_arg = {
                'devices': [device],
                'device_id_str': str(device.get('id')),
                'task_id': task_id,
                'file_name': file_name,
                'tmp_file_name': param.get("tmp_file_name"),
                "quality": param.get("quality")
            }

            policy_singnal_sender({
                'device_id': device.get('id'),
                'device_ip': device.get('device_ip'),
                'policy': event_type,
                'arg': offline_arg
            })
            public.public(json.dumps(signal), chan='policy')
        except Exception as exp:
            logger.info(f"下发系统补丁失败，失败原因 {exp}")


def check_task_status(task_id):
    """
    执行完成后，更新状态
    """
    total_device = TaskDeviceExec.objects.filter(task__pk=task_id)
    total_device_num = total_device.count()
    fail_device_num = total_device.filter(exec_status=5).count()
    success_device_num = total_device.filter(exec_status=4).count()
    logger.info(f"\r\n"
                f"任务执行情况统计 task {task_id}, \r\n"
                f"fail_device_num {fail_device_num},\r\n"
                f"success_device_num {success_device_num},\r\n"
                f"total_device_num {total_device_num}")
    rate_str = "0/0,0"
    if total_device == 0:
        return rate_str
    if success_device_num == 0:
        rate_str = "0/%s,0" % total_device_num
    else:
        rate = round((success_device_num / total_device_num) * 100, 2)
        rate_str = "%s/%s,%s" % (success_device_num, total_device_num, rate)
    task = Tasks.objects.filter(pk=task_id).first()
    if task.quality == 0:
        if total_device_num == (success_device_num + fail_device_num):
            success_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            Tasks.objects.filter(pk=task_id).update(status=4, rate=rate_str, success_time=success_time)
        else:
            if fail_device_num > 0:
                # 有失败的任务即为失败
                Tasks.objects.filter(pk=task_id).update(status=5, rate=rate_str)
            else:
                # 还有未执行算作执行中
                Tasks.objects.filter(pk=task_id).update(status=2, rate=rate_str)

    elif task.quality == 1:
        exec_time = datetime.strptime(task.exec_time, "%Y-%m-%d %H:%M:%S")
        # 定时任务时间比默认时间早10秒
        now = datetime.now() + timedelta(seconds=12)
        if now >= exec_time:
            if total_device_num == success_device_num:
                success_time = now.strftime('%Y-%m-%d %H:%M:%S')
                Tasks.objects.filter(pk=task_id).update(status=4, rate=rate_str, success_time=success_time)
            else:
                if fail_device_num > 0:
                    # 有失败的任务即为失败
                    Tasks.objects.filter(pk=task_id).update(status=5, rate=rate_str)
                else:
                    # 还有未执行算作执行中
                    Tasks.objects.filter(pk=task_id).update(status=2, rate=rate_str)
    else:
        pass


@app.task(queue='default', name='task_ca_policy', time_limite=600)
def task_ca_policy(**param):
    devices = param.get('devices')
    task_id = param.get('task_id')
    file_name = param.get('file_name')
    tmp_file_name = param.get('tmp_file_name')
    quality = param.get("quality")
    exec_time = param.get("exec_time")

    device_id_str = param.get("device_id_str", "")

    # check_task_device(task_id)
    if not devices:
        from devices.models import Devices
        device_id_list = device_id_str.split(",")
        device_list = Devices.objects.filter(id__in=device_id_list)
        devices = DeviceSerializer(device_list, many=True).data

    logger.info('####### celery task_ca_apply_policy ######')
    event_type = 'task_ca_policy'
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': event_type, 'arg': task_id}
    signal.update(param)

    pillar = {
        'task_id': task_id,
        'file_name': file_name,
        'exec_time': exec_time,
        'tmp_file_name': tmp_file_name
    }
    job_name = "task_%s" % (task_id)
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        system_os = get_os(device.get('os'), os_release=device.get('os_release'))
        device_hostname = device.get('device_hostname')
        if quality == 0:
            result = salt_api.async_deploy(device.get('device_hostname'), ['{os}.task-manage.task_ca'.format(
                os=system_os), "pillar={pillar}".format(pillar=pillar),
                'queue=true'])
            TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=2)
        elif quality == 1:
            # 添加定时任务
            result = salt_api.async_deploy(device_hostname, ['{os}.task-manage.task_ca_schedule'.format(
                os=system_os), "pillar={pillar}".format(pillar=pillar), 'queue=true'])
            # 启用任务
            salt_api.schedule_enable_job(device_hostname, job_name)
            # 下发到客户端
            salt_api.refresh_pillar(device_hostname)
        else:
            # 添加计划任务
            result = salt_api.async_deploy(device_hostname, ['{os}.task-manage.task_ca_crontab'.format(
                os=system_os), "pillar={pillar}".format(pillar=pillar), 'queue=true'])
            # 启用任务
            salt_api.schedule_enable_job(device_hostname, job_name)
            # 下发到客户端
            salt_api.refresh_pillar(device_hostname)
            TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=2)
        logger.info('####### celery CA  ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = 'task_ca 策略设置失败， {message}'.format(message=result.get('message'))
            public.public(json.dumps(signal), chan='policy')
            TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=5)
            return

        signal['success'] = True
        signal['msg'] = 'task_ca 策略设置已经下发'

        offline_arg = {
            'devices': [device],
            'device_id_str': str(device.get('id')),
            'task_id': task_id,
            'file_name': file_name,
            'tmp_file_name': param.get("tmp_file_name"),
            "quality": param.get("quality")
        }

        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': event_type,
            'arg': offline_arg
        })
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default', name='task_software_policy', time_limite=600)
def task_software_policy(**param):
    from software.models import SoftwareInfoModel, SoftwareInstalledModel
    devices = param.get('devices')
    task_id = param.get('task_id')
    local_software_list = param.get('local_software_list')
    software_id_list = param.get('software_id_list')
    device_id_str = param.get("device_id_str", "")
    quality = param.get("quality")
    exec_time = param.get("exec_time")

    # check_task_device(task_id)

    if not devices:
        from devices.models import Devices
        from utils.utils import str_to_list
        device_id_list = str_to_list(device_id_str)
        device_list = Devices.objects.filter(id__in=device_id_list)
        devices = DeviceSerializer(device_list, many=True).data

    event_type = 'task_software_policy'
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': event_type, 'arg': task_id}
    signal.update(param)
    job_name = "task_%s" % (task_id)
    # 本地源软件
    if local_software_list:
        pillar = {
            'task_id': task_id,
            'software_list': local_software_list,
            'exec_time': exec_time
        }

        logger.info("pillar %s" % pillar)
        for device in devices:
            try:
                signal['device_id'] = device.get('id')
                signal['device_ip'] = device.get('device_ip')
                signal['device_hostname'] = device.get('device_hostname')
                system_os = get_os(device.get('os'), os_release=device.get('os_release'))
                device_hostname = device.get('device_hostname')
                if quality == 0:
                    result = salt_api.async_deploy(device.get('device_hostname'),
                                                   ['{os}.task-manage.task_software'.format(
                                                       os=system_os), "pillar={pillar}".format(pillar=pillar),
                                                       'queue=true'])
                    TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=2)
                elif quality == 1:
                    # 添加定时任务
                    result = salt_api.async_deploy(device_hostname, ['{os}.task-manage.task_software_schedule'.format(
                        os=system_os), "pillar={pillar}".format(pillar=pillar), 'queue=true'])
                    # 启用任务
                    salt_api.schedule_enable_job(device_hostname, job_name)
                    # 下发到客户端
                    salt_api.refresh_pillar(device_hostname)
                else:
                    # 添加计划任务
                    result = salt_api.async_deploy(device_hostname, ['{os}.task-manage.task_software_crontab'.format(
                        os=system_os), "pillar={pillar}".format(pillar=pillar), 'queue=true'])
                    TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=2)
                    # 启用任务
                    salt_api.schedule_enable_job(device_hostname, job_name)
                    # 下发到客户端
                    salt_api.refresh_pillar(device_hostname)
                logger.info('####### celery task_software_policy  ######')
                logger.info(result)
                logger.info('#######---######')
                if isinstance(result, dict):
                    signal['success'] = False
                    signal['msg'] = 'task_software_policy 策略设置失败， {message}'.format(message=result.get('message'))
                    public.public(json.dumps(signal), chan='policy')
                    TaskDeviceExec.objects.filter(task__pk=task_id, device__pk=device.get('id')).update(exec_status=5)
                    return

                signal['success'] = True
                # signal['msg'] = 'task_software_policy 策略设置已经下发'

                offline_arg = {
                    'devices': [device],
                    'device_id_str': str(device.get('id')),
                    'task_id': task_id,
                    "quality": param.get("quality"),
                    "local_software_list": param.get('local_software_list'),
                    "software_id_list": param.get('software_id_list')
                }

                policy_singnal_sender({
                    'device_id': device.get('id'),
                    'device_ip': device.get('device_ip'),
                    'policy': event_type,
                    'arg': offline_arg
                })
                # public.public(json.dumps(signal), chan='policy')
            except Exception as exp:
                logger.info(f"软件下载失败，错误原因{exp}")

    # 软件源软件
    software_list = SoftwareInfoModel.objects.filter(pk__in=software_id_list, is_delete=0)
    for software in software_list:
        for device in devices:
            softwareInstalledInfo = SoftwareInstalledModel.objects.filter(main_package=software.main_package,
                                                                          main_version=software.main_version).filter(
                device_info=device.get('id')).first()
            if not softwareInstalledInfo:
                softwareInstalledInfo = SoftwareInstalledModel()
                softwareInstalledInfo.software_info = software
                softwareInstalledInfo.name = software.name
                softwareInstalledInfo.main_package = software.main_package
                softwareInstalledInfo.main_version = software.main_version
                softwareInstalledInfo.device_info_id = device.get('id')
                softwareInstalledInfo.save()
                logger.info('创建新的软件安装记录')

            # event_type = 'pkg_install_policy'
            pkg_install_single(device, software.id, softwareInstalledInfo, event_type, 0, task_id)

    # check_task_status(task_id)


@app.task(queue='default', name='task_stop', time_limite=600)
def schedule_task_stop_policy(**param):
    device_hostname_list = param.get('device_hostname_list')
    task_id = param.get('task_id')
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    job_name = "task_%s" % (task_id)
    # 停止任务
    for device_hostname in device_hostname_list:
        logger.info("schedule_disable_job  device_hostname %s job_name %s" % (device_hostname, job_name))
        salt_api.schedule_disable_job(device_hostname, job_name)
    return True


@app.task(queue='default', name='task_stop', time_limite=600)
def schedule_task_start_policy(**param):
    device_hostname_list = param.get('device_hostname_list')
    task_id = param.get('task_id')
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    job_name = "task_%s" % (task_id)
    # 停止任务
    for device_hostname in device_hostname_list:
        logger.info("schedule_enable_job  device_hostname %s job_name %s" % (device_hostname, job_name))
        salt_api.schedule_enable_job(device_hostname, job_name)
    return True


@app.task(queue='default', name='task_stop', time_limite=600)
def schedule_task_delete_policy(**param):
    device_hostname_list = param.get('device_hostname_list')
    task_id = param.get('task_id')
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    job_name = "task_%s" % (task_id)
    # 停止任务
    for device_hostname in device_hostname_list:
        logger.info("schedule_delete  device_hostname %s job_name %s" % (device_hostname, job_name))
        salt_api.schedule_delete(device_hostname, job_name)
    return True


def check_task_device(task_id=None):
    """
    兼容创建任务后，任务下发时，部门人员发生改变
    :param task:
    :return:
    """
    from devices.models import Devices
    if not task_id:
        return False
    task = Tasks.objects.filter(pk=task_id).first()
    if task.org_id:
        all_task_device_exec = TaskDeviceExec.objects.filter(task__pk=task_id)
        all_device_id = []
        if all_task_device_exec:
            all_device_id = [x.device.id for x in all_task_device_exec if x]

        org_id_list = eval(task.org_id)
        new_device_id_list = []

        device_list = Devices.objects.filter(org_info__in=org_id_list)
        for device in device_list:
            new_device_id_list.append(device.id)
            if device.id in all_device_id:
                # 已有的执行记录
                continue
            else:
                # 新增的执行记录
                task_deivce_exec = TaskDeviceExec()
                task_deivce_exec.task = task
                task_deivce_exec.device = device
                task_deivce_exec.exec_status = 0
                task_deivce_exec.exec_time = task.exec_time
                task_deivce_exec.save()
        delete_device_list = list(set(all_device_id) - set(new_device_id_list))
        if len(delete_device_list) > 0:
            # 删除的执行记录
            TaskDeviceExec.objects.filter(task__pk=task_id, device__id__in=delete_device_list).delete()
    return True


@app.task(queue='default')
def account_login_logout_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'account_login_logout_policy', 'arg': arg}
    file_id = arg.get("file_id", 0)
    file_obj = FileManager.objects.filter(id=file_id).first()
    if file_obj:
        file_path = file_obj.path.split("file_data")[1]
    else:
        file_path = ''
    signal.update(kwargs)
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            "login_switch_status": arg.get("login_switch_status", 0),
            "apply_policy_time": int(arg.get("apply_policy_time", 0)),
            "file_path": file_path,
            "account_id": arg.get("account_id")
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.login_out_script.cp_script".format(os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery account_login_logout_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '账号脚本策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '账号脚本策略已经下发'
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def account_desktop_theme_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'account_desktop_theme_policy', 'arg': arg}
    signal.update(kwargs)

    theme_switch_status = arg.get('theme_switch_status')
    style_theme = arg.get('style_theme')
    gtk_theme = arg.get('gtk_theme')
    icon_theme = arg.get('icon_theme')
    cursor_theme = arg.get('cursor_theme')
    is_sunset = arg.get('is_sunset')
    win_decoration_switch = arg.get('win_decoration_switch')
    win_transparency = arg.get('win_transparency')

    gtk_dict = {
        0: 'Platycodon Grandiflorum',  # 桔梗
        1: 'Amber',  # 琥珀色
        2: 'Cinnabar',  # 朱砂
        3: 'Chloe',  # 蔻梢
        4: 'Found'  # 经典
    }

    style_theme = 'Found' if int(style_theme) == 0 else 'FoundDark'
    gtk_theme = gtk_dict.get(int(gtk_theme)) if gtk_theme is not None else 'Found'
    icon_theme = 'Found' if int(icon_theme) == 0 else 'FoundDark'
    cursor_theme = 'FoundSnow' if int(cursor_theme) == 0 else 'Found'
    is_sunset = 'false' if int(is_sunset) == 0 else 'true'
    win_decoration_switch = 'false' if int(win_decoration_switch) == 0 else 'true'
    win_transparency = "{:.2f}".format(float(win_transparency / 100))

    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            "desktop_theme_switch_status": theme_switch_status,
            "theme_val_list": f"{style_theme} '{gtk_theme}' {icon_theme} {cursor_theme} {is_sunset} {win_decoration_switch} {win_transparency}",
            "account_id": arg.get("account_id")
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.deskprofile.theme.theme".format(os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery account_desktop_theme_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '账号桌面主题策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '账号桌面主题策略已经下发'
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def device_system_updates_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'device_system_updates_policy', 'arg': arg}
    signal.update(kwargs)
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')

        update_style = arg.get("update_style", 0)
        check_time_days = arg.get("check_time_days", 0)
        check_time_time = arg.get("check_time_time", 0)
        download_update_method = arg.get("download_update_method", 0)

        pillar = {
            "system_update_switch": arg.get("software_runs_switch", 0),
            "checkupdate": 'false' if int(update_style) == 0 else 'true',
            "check-day": int(check_time_days),
            "check-hour": int(check_time_time),
            "distupgrade-opts": download_update_method,
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.system-update.auto_update_system".format(
                os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery device_system_updates_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '终端更新配置策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '终端更新配置策略已经下发'
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def device_screen_saver_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'device_screen_saver_policy', 'arg': arg}
    signal.update(kwargs)

    lock_activate_screensaver = arg.get('lock_activate_screensaver')
    lock_sleep_screensaver = arg.get('lock_sleep_screensaver')
    use_for_greeter = arg.get('use_for_greeter')
    lock_wallpaper_switch = arg.get('lock_wallpaper_switch')
    screensaver_cycle_switch = arg.get('screensaver_cycle_switch')
    #screensaver_folder_id = arg.get('screensaver_folder_id')

    #总开关(锁屏时间或屏保时间有管控就开启)
    screen_saver_switch = 0
    if arg.get('lock_screen_time') is not None or arg.get('screen_saver_time') is not None:
        screen_saver_switch = 1
    # 默认值
    lock_screen_time = int(arg.get('lock_screen_time')) * 60 if arg.get('lock_screen_time') is not None else 0
    screen_saver_time = int(arg.get('screen_saver_time')) * 60 if arg.get('screen_saver_time') is not None else 0
    screensaver_cycle_time = arg.get('screensaver_cycle_time') if arg.get('screensaver_cycle_time') is not None else 15
    lock_activate_screensaver = 'true' if lock_activate_screensaver == 1 else 'false'
    lock_sleep_screensaver = 'true' if lock_sleep_screensaver == 1 else 'false'
    use_for_greeter = 'true' if use_for_greeter == 1 else 'false'
    # 锁屏壁纸
    wallpaper_name = ""
    if lock_wallpaper_switch == 1:
        wallpaper = Wallpaper.objects.filter(id=arg.get('lock_wallpaper_id')).first()
        wallpaper_name = wallpaper.name
    # 屏幕保护轮播文件夹(暂无处理)
    screensaver_folder_name = ""

    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            "screen_saver_switch": screen_saver_switch,
            "lockscreen_val_list": f"{lock_screen_time} {screen_saver_time} {lock_activate_screensaver} {lock_sleep_screensaver} {screensaver_cycle_time} {use_for_greeter}",
            "lockscreen_wallpaper": wallpaper_name,
            "screensaver_cycle_folder": screensaver_folder_name,
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.deskprofile.lock-screen.lock-screen".format(
                os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery device_screen_saver_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '终端屏幕保护策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '终端屏幕保护策略已经下发'
        signal['status'] = 2

        # 保存数据
        action_device_items, _ = ScreenSaverPolicy.objects.get_or_create(device_id=device.get('id'))
        action_device_items.status = 2
        action_device_items.screen_saver_switch = screen_saver_switch
        action_device_items.lock_screen_time = arg.get('lock_screen_time')
        action_device_items.screen_saver_time = arg.get('screen_saver_time')
        action_device_items.lock_activate_screensaver = arg.get('lock_activate_screensaver')
        action_device_items.lock_sleep_screensaver = arg.get('lock_sleep_screensaver')
        action_device_items.screensaver_cycle_time = screensaver_cycle_time
        action_device_items.use_for_greeter = arg.get('use_for_greeter')
        action_device_items.lock_wallpaper_id = arg.get('lock_wallpaper_id')
        action_device_items.screensaver_folder_id = arg.get('screensaver_folder_id')
        action_device_items.save()

        if 'policy_no' in kwargs.keys():
            pass
        else:
            public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def device_software_runs_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'device_software_runs_policy', 'arg': arg}
    signal.update(kwargs)

    software_runs_switch = arg.get('software_runs_switch')
    software_runs_style = arg.get('software_runs_style')
    runs_white_switch = arg.get('runs_white_switch')
    white_software_list = arg.get('white_software_list')
    runs_black_switch = arg.get('runs_black_switch')
    black_software_list = arg.get('black_software_list')

    if runs_black_switch == 0:
        add_delete = 'delete'
        app_path_list = black_software_list
    else:
        add_delete = 'add'
        app_path_list = black_software_list

    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            "software_runs_switch": software_runs_switch,
            "add_delete": add_delete,
            "app_path_list": app_path_list
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.black_software.add_blacklist".format(
                os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery device_software_runs_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '终端软件运行策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '终端软件运行策略已经下发'
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def device_file_redirection_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'device_file_redirection_policy', 'arg': arg}
    signal.update(kwargs)

    file_redirection_switch = arg.get('file_redirection_switch')
    file_redirection_list = arg.get('file_redirection_list')

    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            "file_redirection_switch": file_redirection_switch,
            "file_redirection_list": file_redirection_list
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.deskprofile.folder-redirection.ln_folder_folder".format(
                os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery device_file_redirection_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '终端文件重定向策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '终端文件重定向策略已经下发'
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def account_trusted_sites_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'account_trusted_sites_policy', 'arg': arg}
    signal.update(kwargs)

    trusted_sites_switch = arg.get('trusted_sites_switch')
    trusted_sites_list = arg.get('trusted_sites_list')

    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            "trusted_sites_switch": trusted_sites_switch,
            "trusted_sites_list": json.dumps(trusted_sites_list),
            "account_id": arg.get("account_id")
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.deskprofile.browser.popup_policy".format(
                os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery account_trusted_sites_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '账号可信任站点策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '账号可信任站点策略已经下发'
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')


@app.task(queue='default')
def account_browser_favorites_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'account_browser_favorites_policy', 'arg': arg}
    signal.update(kwargs)

    browser_favorites_switch = arg.get('browser_favorites_switch')
    browser_favorites_redirection = arg.get('browser_favorites_redirection')

    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            "browser_favorites_switch": browser_favorites_switch,
            "browser_favorites_redirection": browser_favorites_redirection,
            "account_id": arg.get("account_id")
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.deskprofile.browser.mv_bookmark".format(
                os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery account_browser_favorites_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '账号游览器收藏夹策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '账号游览器收藏夹策略已经下发'
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')

@app.task(bind=True)
def browser_policy(self, devices, arg, **kwargs):
    logger.info("=======celery browser_policy task start=========")
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'browser_policy', 'arg': arg}
    signal.update(kwargs)

    policies = arg.get('policies')
    users = arg.get('users')
    type = arg.get('type')
    if type == 'user':
        usernames = AccountEntity.objects.filter(id__in=users).all()
        user_dict = {u.id: u.name for u in usernames}
        users_new = []
        for u in users:
            users_new.append(user_dict[u] if u in user_dict else u)

    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            "policies": json.dumps(policies),
            "type": type,
            "task_id": self.request.id
        }
        if type == 'user':
            pillar['users'] = users_new
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.policy.browser.push_browser_policies".format(
                os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])

        logger.info('####### celery browser_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            # for item in result['return']:
            #     for k,v in item.items():
            #         if not v:
            signal['success'] = False
            signal['msg'] = '浏览器策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            # logger.info("=======celery browser_policy task retry trigger=========")
            # raise Exception(signal)

        signal['success'] = True
        signal['msg'] = '浏览器策略已经下发'
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')
        return


@app.task(bind=True)
def shell_policy(self, devices, arg, **kwargs):
    file_path = arg.get("file_path")

    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)

    pillar = {
        'file_name': file_path,
        'tmp_file_name': f'{uuid.uuid4()}.sh',
        "task_id": self.request.id
    }
    event_type = 'shell_policy'
    signal = {'event_type': event_type, 'arg': pillar}

    from_path = os.path.join(settings.MEDIA_ROOT, file_path)
    to_path = os.path.join(settings.SALT_FILE_PATH, file_path)
    if not os.path.exists(from_path):
        signal['success'] = False
        signal['msg'] = 'shell 策略设置失败， 文件 {message} 不存在'.format(message=from_path)
        public.public(json.dumps(signal), chan='policy')
        logger.info(f"devices: {devices}")
        ShellDevicePolicy.objects.filter(device__in=map(lambda x: x.get('id'), devices)).update(status=3)
        return
    if not os.path.exists(os.path.dirname(to_path)):
        os.mkdir(os.path.dirname(to_path))
    shutil.copy(from_path, to_path)

    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        system_os = get_os(device.get('os'), os_release=device.get('os_release'))
        # device_hostname = device.get('device_hostname')

        result = salt_api.async_deploy(device.get('device_hostname'), ['{os}.shell-manage.shell'.format(
            os=system_os), "pillar={pillar}".format(pillar=pillar),
            'queue=true'])
        ShellDevicePolicy.objects.filter(device__pk=device.get('id')).update(status=2)

        logger.info('####### celery shell ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = 'shell 策略设置失败， {message}'.format(message=result.get('message'))
            public.public(json.dumps(signal), chan='policy')
            ShellDevicePolicy.objects.filter(device__pk=device.get('id')).update(status=5)
            return

        signal['success'] = True
        signal['msg'] = 'shell 策略设置已经下发'

        public.public(json.dumps(signal), chan='policy')

@app.task(queue='default')
def job_file_policy(**kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    devices_items = Devices.objects.filter(is_up=1,is_delete=0).all()
    signal = {'event_type': 'job_file_policy'}
    signal.update(kwargs)
    if devices_items.filter(id=kwargs.get('device_id',None)).exists():
        devices = devices_items.filter(id=kwargs.get('device_id',None)).values()
    else:
        devices = []
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            "download_path": kwargs.get("download_path", None),
            "file_name": kwargs.get("file_name", None),
            "download_dir": kwargs.get("download_dir", None),
            'is_notify_user': kwargs.get('is_notify_user', 0),
            'is_user_cancel': kwargs.get('is_user_cancel', 0),
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.task-manage.task_file".format(os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery job_file_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '文件任务设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '文件任务已经下发'
        signal['status'] = 2

        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': 'job_file_policy',
            'arg': kwargs,
        })

        public.public(json.dumps(signal), chan='policy')

@app.task(queue='default')
def job_bash_policy(**kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    devices_items = Devices.objects.filter(is_up=1,is_delete=0).all()
    signal = {'event_type': 'job_bash_policy'}
    signal.update(kwargs)
    if devices_items.filter(id=kwargs.get('device_id',None)).exists():
        devices = devices_items.filter(id=kwargs.get('device_id',None)).values()
    else:
        devices = []
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            'is_notify_user': kwargs.get('is_notify_user', 0),
            'is_user_cancel': kwargs.get('is_user_cancel', 0),
            "tmp_file_name": kwargs.get("tmp_file_name", None),
            "file_name": kwargs.get("file_name", None),
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.task-manage.task_shell".format(os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery job_bash_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '脚本任务设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '脚本任务已经下发'
        signal['status'] = 2

        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': 'job_bash_policy',
            'arg': kwargs,
        })

        public.public(json.dumps(signal), chan='policy')

@app.task(queue='default')
def job_software_policy(**kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    devices_items = Devices.objects.filter(is_up=1,is_delete=0).all()
    signal = {'event_type': 'job_software_policy'}
    signal.update(kwargs)
    if devices_items.filter(id=kwargs.get('device_id',None)).exists():
        devices = devices_items.filter(id=kwargs.get('device_id',None)).values()
    else:
        devices = []
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            'job_info_id': kwargs.get('id'),
            'is_notify_user': kwargs.get('is_notify_user', 0),
            'is_user_cancel': kwargs.get('is_user_cancel', 0),
            "download_path": kwargs.get("download_path", None),
            "file_name": kwargs.get("file_name", None),
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.task-manage.task_software".format(os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery job_software_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '软件任务设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '软件任务已经下发'
        signal['status'] = 2

        policy_singnal_sender({
            'device_id': device.get('id'),
            'device_ip': device.get('device_ip'),
            'policy': 'job_software_policy',
            'arg': kwargs,
        })

        public.public(json.dumps(signal), chan='policy')

@app.task(queue='default')
def desktop_theme_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'desktop_theme_policy', 'arg': arg}
    signal.update(kwargs)

    theme_switch_status = arg.get('theme_switch_status', 1)
    style_theme = arg.get('style_theme')
    gtk_theme = arg.get('gtk_theme')
    icon_theme = arg.get('icon_theme')
    cursor_theme = arg.get('cursor_theme')
    is_sunset = arg.get('is_sunset')
    win_decoration_switch = arg.get('win_decoration_switch')
    win_transparency = arg.get('win_transparency')

    style_dict = {
        0: 'Found',
        1: 'FoundDark',
    }

    gtk_dict = {
        0: 'Platycodon Grandiflorum',  # 桔梗
        1: 'Amber',  # 琥珀色
        2: 'Cinnabar',  # 朱砂
        3: 'Chloe',  # 蔻梢
        4: 'Found',  # 经典
    }

    cursor_dict = {
        0: 'FoundSnow',
        1: 'Found',
    }

    sunset_dict = {
        0: 'false',
        1: 'true',
    }

    style_theme = style_dict.get(int(style_theme)) if style_theme is not None else None
    gtk_theme = gtk_dict.get(int(gtk_theme)) if gtk_theme is not None else None
    icon_theme = style_dict.get(int(icon_theme)) if icon_theme is not None else None
    cursor_theme = cursor_dict.get(int(cursor_theme)) if cursor_theme is not None else None
    is_sunset = sunset_dict.get(int(is_sunset)) if is_sunset is not None else None
    win_decoration_switch = sunset_dict.get(int(win_decoration_switch)) if win_decoration_switch is not None else None
    win_transparency = "{:.2f}".format(float(win_transparency / 100)) if win_transparency is not None else None

    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        pillar = {
            "desktop_theme_switch_status": theme_switch_status,
            "theme_val_list": f"{style_theme} '{gtk_theme}' {icon_theme} {cursor_theme} {is_sunset} {win_decoration_switch} {win_transparency}",
            # "account_id": arg.get("account_id")
        }
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), [
            "{os}.deskprofile.theme.theme".format(os=get_os(device.get('os'), os_release=device.get('os_release'))),
            "pillar={pillar}".format(pillar=pillar),
            'queue=true'
        ])
        logger.info('####### celery desktop_theme_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '桌面主题策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '桌面主题策略已经下发'
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')

@app.task(queue='default')
def password_rule_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'password_rule_policy', 'arg': arg}
    signal.update(kwargs)

    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        system_os = get_os(device.get('os'), os_release=device.get('os_release'))
        # device_hostname = device.get('device_hostname')

        pillar = arg
        pillar.update(kwargs)
        result = salt_api.async_deploy(device.get('device_hostname'), ['{os}.policy.password.password'.format(
            os=system_os), "pillar={pillar}".format(pillar=pillar),
            'queue=true'])
        logger.info('####### celery password_rule_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = '密码策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = '密码策略已经下发'
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')
    pass

@app.task(queue='default')
def root_policy(devices, arg, **kwargs):
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'root_policy', 'arg': arg}
    signal.update(kwargs)
    # root管控值 true/false
    root_switch = arg['enable']
    # 如果不是'true'或'false'就不处理
    if root_switch != 'true' and root_switch != 'false':
        return
    pillar = {
        'root_enable': root_switch,
        'policy_no': kwargs['policy_no'] if 'policy_no' in kwargs.keys() else None
    }
    pillar.update(kwargs)
    for device in devices:
        signal['device_id'] = device.get('id')
        signal['device_ip'] = device.get('device_ip')
        signal['device_hostname'] = device.get('device_hostname')
        if root_switch == "true":
            result = salt_api.async_deploy(device.get('device_hostname'),['{os}.policy.root.root_enable'.format(
                os=get_os(device.get('os'),os_release=device.get('os_release'))), "pillar={pillar}".format(pillar=pillar),
                'queue=true'])
        else:
            result = salt_api.async_deploy(device.get('device_hostname'), ['{os}.policy.root.root_disable'.format(
                os=get_os(device.get('os'), os_release=device.get('os_release'))),
                "pillar={pillar}".format(pillar=pillar),
                'queue=true'])
        logger.info('####### celery root_policy ######')
        logger.info(result)
        logger.info('#######---######')
        if isinstance(result, dict):
            signal['success'] = False
            signal['msg'] = 'root策略设置失败， {message}'.format(message=result.get('message'))
            signal['status'] = 0
            public.public(json.dumps(signal), chan='policy')
            continue

        signal['success'] = True
        signal['msg'] = 'root策略已经下发'
        signal['status'] = 2
        public.public(json.dumps(signal), chan='policy')

