from os.path import basename

from django.conf.urls import url
from django.urls import include
from rest_framework.routers import DefaultRouter

from saltapi import views

router = DefaultRouter()

router.register('screensavers', views.ScreenSaversViewSet, basename='屏幕保护功能')
router.register('default_watermark', views.WaterMarkViewSet, basename='默认水印设置')
router.register('watermark_info', views.WaterMarkInfoViewSet, basename='水印设置列表')
router.register('policy_list', views.PolicyInfoViewSet, basename='策略列表信息')
router.register('policy_dict_type', views.PolicyDictInfoViewSet, basename='策略种类列表信息')
router.register('active_policy_info', views.ActivePolicyViewSet, basename='系统激活策略列表')
router.register('firewalld_rules', views.FirewalldRuleViewSet, basename='防火墙配置信息')
router.register('firewalld-black-white-roster', views.FirewalldRosterPolicy, basename='防火墙黑白名单')
router.register('device_policy', views.DevicePolicyViewSet, basename='终端策略')
router.register('account_policy', views.AccountPolicyViewSet, basename='用户策略')
router.register('shell_list', views.ShellInfoViewSet, basename='脚本库列表')
router.register('shell_policy', views.ShellPolicyViewset, basename='脚本策略')
router.register('shell_policy_device', views.ShellDevicePolicyViewSet, basename='设备脚本策略列表')

urlpatterns = [
    url(r'^', include(router.urls)),
    url(r'^all-key$', views.SaltAllKey.as_view(), name='SaltAllKey'),

    url(r'^active-policy/$', views.ActivePolicy.as_view(), name='ActivePolicy'),
    url(r'^kms-server/$', views.KMSServer.as_view(), name='KMSServer'),
    url(r'^root-policy/$', views.RootPolicy.as_view(), name='RootPolicy'),

    url(r'^ssh/(?P<device_uuid>\S+)/(?P<option>\S+)$', views.SSHRemote.as_view(), name='SSHRemote'),
    url(r'^ssh-policy$', views.SSHPolicy.as_view(), name='SSHPolicy'),
    url(r'^ssh-device$', views.SSHDevice.as_view(), name='SSHDevice'),
    url(r'^ssh_device_batch_update/$', views.SSHDeviceBatchUpdate.as_view(), name='ssh_device_batch_update'),

    url(r'^usb/(?P<device_id>[0-9]+)$', views.USBReord.as_view(), name='USBReord'),
    url(r'^usb-policy$', views.USBPolicy.as_view(), name='USBPolicy'),
    url(r'^usb-device$', views.USBDevice.as_view(), name='USBDevice'),
    url(r'^usb-multiple/$', views.USBDevice.as_view(), name='USBDevice-multiple'),

    url(r'^wallpaper$', views.WallpaperAPIView.as_view(), name='WallpaperAPIView-GET-POST'),
    url(r'^wallpaper/(?P<wallpaper_id>[0-9]+)$', views.WallpaperAPIView.as_view(), name='WallpaperAPIView-PUT-DELETE'),
    url(r'^wallpaper-policy$', views.WallpaperPolicy.as_view(), name='WallpaperPolicy-GET'),
    url(r'^wallpaper-policy/(?P<wallpaper_id>[0-9]+)$', views.WallpaperPolicy.as_view(), name='WallpaperPolicy-POST'),
    url(r'^wallpaper-device$', views.WallpaperDevice.as_view(), name='WallpaperDevice'),

    url(r'^firewalld/$', views.FirewalldPolicy.as_view(), name='FirewalldPolicy'),
    url(r'^firewalld-device$', views.FirewalldDevice.as_view(), name='FirewalldDevice'),
    # url(r'^firewalld-black-white-roster$', views.FirewalldRosterDevice.as_view(), name='FirewalldRosterDevice'),
    # url(r'^firewalld_device_batch_update/$', views.FirewalldDeviceBatchUpdate.as_view(), name='firewalld_device_batch_update'),

    url(r'^jid-all$', views.SaltJids.as_view(), name='SaltJids'),  # 策略日志
    url(r'^jid-detail/(?P<jid>\S+)$', views.SaltJidRandered.as_view(), name='SaltJidRandered'),  # 策略日志
    # url(r'^domainevent/', views.DomainEvent, name='DomainEvent'),  # dwebsocket

    url(r'^shell_policy_default$', views.ShellDefaultPolicy.as_view(), name='ShellDefaultPolicy'),
]
