# -*- coding: utf-8 -*-
import copy
import datetime
import os
import json
import re
import logging
import threading
import time
import uuid
from crypt import methods
from http import HTTPStatus

import pandas
from django.conf import settings
# caches['redis'].get('res_key') 使用redis缓存， caches['default'].get('res_key') 使用default缓存
from django.core.cache import caches
from django_redis import get_redis_connection
from django.db.models import Q
from rest_framework.decorators import action
from rest_framework.views import APIView
from rest_framework.permissions import IsAuthenticated
from rest_framework.views import Response
from rest_framework import status, mixins
from rest_framework.viewsets import GenericViewSet, ModelViewSet, ReadOnlyModelViewSet

from accounts.serializers import AccountEntitySerializer
from groups.models import Groups, GroupsDetail
from saltapi.filters import PolicyInfoFilter, ShellFilter, ShellDevicePolicyFilter
from saltapi.forms import UploadShellFileForm
from users.models import User
from users.role_permisson import get_org_permission_list
from utils.pandas_policy import policy_main
from utils.pagination import MyPagination
from utils.response import FitJSONRenderer
from utils.authentication import TokenAuthentication, DomainLicenseAuth
from utils.permission import IsAdminUser
from utils.throttling import AnonRateThrottle, UserRateThrottle
from utils.utils import pager_currency

from saltapi.utils import SaltAPI, DefaultPolicy
from saltapi.tasks import usb_policy, ssh_policy, wallpaper_policy, firewalld_policy, firewalld_level_policy, \
    screensavers_policy, firewalld_black_white_roster_policy, reboot_policy, watermark_policy, system_activation_policy, \
    account_login_logout_policy, account_desktop_theme_policy, device_system_updates_policy, device_screen_saver_policy, \
    device_software_runs_policy, device_file_redirection_policy, account_trusted_sites_policy, \
    account_browser_favorites_policy, shell_policy
from saltapi.models import USBDevicePolicy, SSHDevicePolicy, SaltJidDevice, Wallpaper, WallpaperDevicePolicy, \
    FirewalldDevicePolicy, Screensavers, ScreensaversDevicePolicy, PolicyInfo, PolicyDictInfo, FirewalldRulesPolicy, \
    FirewalldWhiteBlackRosterPolicy, WaterMark, WaterMarkDevicePolicy, ActivationDevicePolicy, AccountLoginPolicy, \
    AccountDefaultPolicy, DesktopThemePolicy, SystemUpdatesPolicy, SoftwareRunsPolicy, ScreenSaverPolicy, \
    FileRedirectionPolicy, BrowserFavoritesPolicy, TrustedSitesPolicy, Shell, ShellDevicePolicy
from saltapi.output import highstate_output, nested_output
from saltapi.serializers import WallpaperSerializer, ScreenSaversSerializer, PolicyInfoSerializer, \
    PolicyDictInfoSerializer, FirewalldRuleSerializers, FirewalldRosterSerializers, WaterMarkSerializer, \
    WaterMarkDevicePolicySerializer, ActivePolicySerializers, AccountLoginPolicySerializers, \
    AccountDefaultPolicySerializer, AccountDesktopThemePolicySerializers, DeviceSystemUpdatesPolicySerializers, \
    DeviceSoftwareRunsPolicySerializers, DeviceScreenSaverPolicySerializers, DeviceFileRedirectionPolicySerializers, \
    AccountBrowserFavoritesPolicySerializers, AccountTrustedSitesPolicySerializers, SystemUpdatesPolicySerializer, \
    ShellInfoSerializer, ShellDevicePolicySerializer, ShellPolicySerializer
from devices.models import Devices
from devices.serializers import DeviceSerializer
from accounts.models import AccountEntity
from organizations.models import OrgnizationEntity
from organizations.views import get_sub_account
from ansi2html import Ansi2HTMLConverter
from asgiref.sync import async_to_sync
from channels.generic.websocket import WebsocketConsumer
from utils.utils import get_serializer_error

logger = logging.getLogger(__name__)


class SaltAllKey(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        all_key = salt_api.list_all_key()
        # {'minions': ['192.168.3.54'], 'minions_pre': ['192.168.3.45'], 'minions_rejected': [], 'minions_denied': [], 'local': ['master.pem', 'master.pub']}
        return Response({
            'code': 200,
            'data': all_key
        })


class ActivePolicy(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        domain_auth = DomainLicenseAuth()
        return Response({
            'code': 200,
            'data': {'active': int(domain_auth.kms_active)}
        })

    def post(self, request):
        active = request.data.get('active')
        domain_auth = DomainLicenseAuth()
        domain_auth.add_kms_active(active)
        return Response({
            'code': 200,
            'msg': '设置成功'
        })


class KMSServer(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        domain_auth = DomainLicenseAuth()
        return Response({
            'code': 200,
            'data': {'kms_server': domain_auth.kms_server}
        })

    def post(self, request):
        kms_server = request.data.get('kms_server')
        if not kms_server:
            return Response({
                'code': 400,
                'msg': '设置失败，地址错误'
            })
        domain_auth = DomainLicenseAuth()
        domain_auth.add_kms_server(kms_server)
        return Response({
            'code': 200,
            'msg': '设置成功'
        })


class RootPolicy(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        default_policy = DefaultPolicy()
        return Response({
            'code': 200,
            'data': {'root': int(default_policy.get_default_policy().get('root'))}
        })

    def post(self, request):
        option = 'root'
        value = request.data.get('value')
        default_policy = DefaultPolicy()
        default_policy.set_default_policy(option, value)
        return Response({
            'code': 200,
            'msg': '设置成功'
        })


class SSHPolicy(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        default_policy = DefaultPolicy()
        return Response({
            'code': 200,
            'data': {'ssh': int(default_policy.get_default_policy().get('ssh'))}
        })

    def post(self, request):
        option = 'ssh'
        value = request.data.get('value')
        default_policy = DefaultPolicy()
        default_policy.set_default_policy(option, value)
        return Response({
            'code': 200,
            'msg': '设置成功'
        })


class SSHDevice(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    @classmethod
    def organ_down_list(cls, num):
        """获取父级及以下的部门"""
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return [int(num)]
        else:
            organ_down_list = organ_down_list + list(items.values_list('id', flat=True))
            while True:
                # logger.info(f'**********{items}*********')
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('id', flat=True))

            organ_down_list.append(int(num))
        return organ_down_list

    def get(self, request):
        device_all = Devices.objects.exclude(is_domain=0).all().defer('device_root').order_by('-id')

        # 根据部门过滤数据，超级管理员不需要
        user = User.objects.filter(username=request.user.username).first()
        org_list = get_org_permission_list(user)

        # 根据部门筛选数据
        if org_list:
            device_all = device_all.filter(org_info__in=org_list)

        device_cnname = request.GET.get('device_cnname')
        device_ip = request.GET.get('device_ip')
        accounts_id = request.GET.get('accounts_id')
        ssh = request.GET.get('device_ssh')
        org_id = request.GET.get('org_id')
        cn_name = request.GET.get('cn_name')
        name = request.GET.get('name')
        status = request.GET.get('status')
        if device_cnname:
            device_all = device_all.filter(device_cnname__icontains=device_cnname)

        if device_ip:
            device_all = device_all.filter(device_ip__icontains=device_ip)
        if accounts_id:
            account = AccountEntity.objects.filter(id=accounts_id).first()
            device_all = device_all.filter(accounts_id=account.id)
        if name:
            accounts = AccountEntity.objects.filter(name__icontains=name).all().values_list('id', flat=True)
            device_all = device_all.filter(accounts_id__in=accounts)
        if org_id:
            org = self.organ_down_list(org_id)
            if not org:
                return Response({
                    'code': 200,
                    "pageCount": 1,
                    "current": 1,
                    "pageSize": 10,
                    "total": 0,
                    "data": []
                })
            device_all = device_all.filter(org_info__in=org)
        total_number = device_all.count()
        page_info = pager_currency(request, total_number)
        devices = device_all[page_info.get('start'):page_info.get('limit')]
        res_list = []
        for device in devices:
            ssh_device = SSHDevicePolicy.objects.filter(device_id=device.id).first()
            if ssh == '1':
                if not ssh_device or ssh_device.ssh != 1: continue
            if ssh == '0':
                if ssh_device and ssh_device.ssh != 0: continue

            device_info = {}
            device_ip = device.device_ip
            device_info['id'] = device.id
            device_info['device_ip'] = device_ip
            device_info['device_cnname'] = device.device_cnname
            device_info['accounts_id'] = device.accounts_id

            if not ssh_device:
                device_info['device_ssh'] = None
                device_info['status'] = 0
                device_info['jid'] = None
            else:
                device_info['device_ssh'] = ssh_device.ssh
                device_info['status'] = ssh_device.status
                device_info['jid'] = ssh_device.jid
            if status and int(status) != device_info['status']: continue
            accounts_id = device.accounts_id
            account = AccountEntity.objects.filter(id=accounts_id).first()
            if account:
                device_info['cn_name'] = account.cn_name
                device_info['name'] = account.name
                org = account.org_info
                device_info['org_name'] = org.name
            else:
                device_info['cn_name'] = None
                device_info['name'] = None
                device_info['org_name'] = None
            res_list.append(device_info)
        return Response({
            'code': 200,
            'pageCount': page_info.get('page_count'),
            'current': page_info.get('current_page'),
            'pageSize': page_info.get('per_page'),
            'total': total_number,
            'data': res_list
        })

    def post(self, request):
        device_id = request.data.get('device_id')
        # org_id = request.data.get('org_id')
        # if org_id:
        #     accounts_ids = []
        #     for org in org_id:
        #         accounts = get_sub_account(org)
        #         if accounts:
        #             accounts_ids = accounts_ids + list(accounts.values_list('id', flat=True))
        #     if not accounts_ids:
        #         return Response({
        #             'code': 400,
        #             'msg': '部门下无用户'
        #         })
        #   devices = Devices.objects.exclude(is_domain=0).filter(is_up=1, accounts_id__in=accounts_ids).all()
        # else:
        devices = Devices.objects.exclude(is_domain=0).filter(is_up=1, id__in=device_id).all()
        if not devices:
            return Response({
                'code': 400,
                'msg': '设置未提交， 请检查设备是否未启用'
            })
        for device in devices.only('id'):
            ssh_device, _ = SSHDevicePolicy.objects.get_or_create(device_id=device.id)
            ssh_device.status = 2
            ssh_device.save()
        celery_task = ssh_policy.delay(
            DeviceSerializer(devices, many=True).data,  # celery参数不能是query_set类型
            request.data.get('device_ssh', 0)
        )

        return Response({
            'code': 200,
            'msg': '设置已提交',
            'task_id': celery_task.id
        })


class SSHDeviceBatchUpdate(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def post(self, request):
        """
        ssh终端管理批量更新启用禁用
        :param request:
        :return:
        """
        try:
            ids_list_str = request.data.get('ids_list')
            is_all = request.data.get('is_all', 0)

            ids_list = []
            if ids_list_str:
                if "," in ids_list_str:
                    ids_list = ids_list_str.split(",")
                else:
                    ids_list = [ids_list_str]

            if is_all == 1:
                device_all = Devices.objects.exclude(is_domain=0).all().defer('device_root').order_by('-id')
            else:
                device_all = Devices.objects.exclude(is_domain=0).filter(is_up=1, id__in=ids_list).all()

            # 根据部门过滤数据，超级管理员不需要
            user = User.objects.filter(username=request.user.username).first()
            org_list = get_org_permission_list(user)

            # 根据部门筛选数据
            if org_list:
                device_all = device_all.filter(org_info__in=org_list)

            device_cnname = request.data.get('device_cnname')
            device_ip = request.data.get('device_ip')
            accounts_id = request.data.get('accounts_id')
            ssh = request.data.get('device_ssh')
            org_name = request.data.get('org_name')
            cn_name = request.data.get('cn_name')
            name = request.data.get('name')
            policy_status = request.data.get('status')
            if device_cnname:
                device_all = device_all.filter(device_cnname__icontains=device_cnname)
            if device_ip:
                device_all = device_all.filter(device_ip=device_ip)
            if accounts_id:
                account = AccountEntity.objects.filter(id=accounts_id).first()
                device_all = device_all.filter(accounts_id=account.id)
            if name:
                accounts = AccountEntity.objects.filter(name__icontains=name).all().values_list('id', flat=True)
                device_all = device_all.filter(accounts_id__in=accounts)
            if org_name:
                org = OrgnizationEntity.objects.filter(name__icontains=org_name).first()
                if not org:
                    return Response({
                        'code': 400,
                        'msg': '此部门下无用户'
                    })
                accounts = org.accountentity_set.all()
                if cn_name:
                    accounts = accounts.filter(cn_name__icontains=cn_name)
                device_all = device_all.filter(accounts_id__in=[account.id for account in accounts])
            devices = device_all

            if not devices:
                return Response({
                    'code': 400,
                    'msg': '设置未提交， 请检查设备是否未启用 或 部门下是否有设备'
                })

            for device in devices.only('id'):
                ssh_device = SSHDevicePolicy.objects.filter(device_id=device.id).first()
                if ssh == '1':
                    if not ssh_device or ssh_device.ssh != 1:
                        continue
                if ssh == '0':
                    if ssh_device and ssh_device.ssh != 0:
                        continue

                ssh_device, _ = SSHDevicePolicy.objects.get_or_create(device_id=device.id)
                ssh_device.status = 2
                ssh_device.save()
            celery_task = ssh_policy.delay(
                DeviceSerializer(devices, many=True).data,  # celery参数不能是query_set类型
                request.data.get('device_ssh', 0)
            )
            return Response({
                'code': 200,
                'msg': '设置已提交',
                'task_id': celery_task.id
            })
        except Exception as e:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '请求参数错误 %s' % e
            })


class SSHRemote(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, device_uuid, option):
        device = Devices.objects.filter(device_uuid=device_uuid).exclude(is_domain=0).first()
        if not device:
            return Response({
                'code': 4001,
                'msg': '设备不存在或未加域'
            })
        if option not in ('start', 'stop'):
            return Response({
                'code': 4001,
                'msg': 'ssh option 参数不正确'
            })
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        if option == 'start':
            res = salt_api.service_start(device.get('device_hostname'), ['ssh'])
        else:
            res = salt_api.service_stop(device.get('device_hostname'), ['ssh'])
        data = res.get(device.device_hostname)
        if not data:
            return Response({
                'code': 4001,
                'msg': '设备不在线'
            })
        res = {
            'code': 2000,
            'msg': 'ssh %s 成功' % option,
            'device_root': device.device_root,
            'device_root_pwd': device.device_root_pwd,
            '': res['return'][0][device.device_hostname]
        }
        return Response(res)


class USBPolicy(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        default_policy = DefaultPolicy()
        usb_rw_status = default_policy.get_default_policy().get('usb_usb_rw_status')
        if usb_rw_status == 'rw':
            usb_promission = 0
        else:
            usb_promission = 1
        return Response({
            'code': 200,
            'data': {'usb': int(default_policy.get_default_policy().get('usb')),
                     'usb_promission': usb_promission}
        })

    def post(self, request):
        option = 'usb'
        value = request.data.get('usb')
        # usb_promission = request.data.get('usb_promission')
        usb_promission = 0
        if usb_promission == 1:
            usb_promission = "r"
        elif usb_promission == 0:
            usb_promission = "rw"
        else:
            return Response({'code': 401, 'msg': f'usb_promission参数错误 {usb_promission}'})
        default_policy = DefaultPolicy()
        default_policy.set_default_policy(option, value)
        default_policy.set_default_policy('usb_rw_status', usb_promission)
        return Response({
            'code': 200,
            'msg': '设置成功'
        })


class USBDevice(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    @classmethod
    def organ_down_list(cls, num):
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return [int(num)]
        else:
            organ_down_list = organ_down_list + list(items.values_list('id', flat=True))
            while True:
                # logger.info(f'**********{items}*********')
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('id', flat=True))

            organ_down_list.append(int(num))
        return organ_down_list

    def get(self, request):
        device_all = Devices.objects.exclude(is_domain=0).all().defer('device_root').order_by('-id')

        # 根据部门过滤数据，超级管理员不需要
        user = User.objects.filter(username=request.user.username).first()
        org_list = get_org_permission_list(user)

        # 根据部门筛选数据
        if org_list:
            device_all = device_all.filter(org_info__in=org_list)

        device_cnname = request.GET.get('device_cnname')
        device_ip = request.GET.get('device_ip')
        accounts_id = request.GET.get('accounts_id')
        usb = request.GET.get('device_usb')
        org_id = request.GET.get('org_id')
        cn_name = request.GET.get('cn_name')
        name = request.GET.get('name')
        status = request.GET.get('status')
        if device_ip:
            device_all = device_all.filter(device_ip__icontains=device_ip)
        if device_cnname:
            device_all = device_all.filter(device_cnname__icontains=device_cnname)
        if accounts_id:
            account = AccountEntity.objects.filter(id=accounts_id).first()
            device_all = device_all.filter(accounts_id=account.id)
        if name:
            accounts = AccountEntity.objects.filter(name__icontains=name).all().values_list('id', flat=True)
            device_all = device_all.filter(accounts_id__in=accounts)
        if org_id:
            org = self.organ_down_list(org_id)
            if not org:
                return Response({
                    'code': 200,
                    "pageCount": 1,
                    "current": 1,
                    "pageSize": 10,
                    "total": 0,
                    "data": []
                })
            # qtm 特殊情况 设备无用户
            faw = AccountEntity.objects.filter(name='faw').first()
            if faw:
                device_all = device_all.filter(org_info_id__in=org)
            else:
                accounts = AccountEntity.objects.filter(org_info_id__in=org)
                if cn_name:
                    accounts = accounts.filter(cn_name=cn_name)
                device_all = device_all.filter(accounts_id__in=accounts.values_list('id', flat=True))
        total_number = device_all.count()
        page_info = pager_currency(request, total_number)
        devices = device_all[page_info.get('start'):page_info.get('limit')]
        res_list = []
        for device in devices:
            usb_device = USBDevicePolicy.objects.filter(device_id=device.id).first()
            if usb == '1':
                if not usb_device or usb_device.usb != 1: continue
            if usb == '0':
                if usb_device and usb_device.usb != 0: continue
            device_info = {}
            device_ip = device.device_ip
            device_info['id'] = device.id
            device_info['device_ip'] = device_ip
            device_info['accounts_id'] = device.accounts_id
            device_info['device_cnname'] = device.device_cnname

            if not usb_device:
                device_info['device_usb'] = None
                device_info['status'] = 0
                device_info['jid'] = None
            else:
                device_info['device_usb'] = device.usbdp.usb
                device_info['status'] = device.usbdp.status
                device_info['jid'] = device.usbdp.jid
            if status and int(status) != device_info['status']: continue
            accounts_id = device.accounts_id
            account = AccountEntity.objects.filter(id=accounts_id).first()
            if OrgnizationEntity.objects.filter(id=device.org_info_id).exists():
                organ_name = OrgnizationEntity.objects.get(id=device.org_info_id).name
            else:
                organ_name = None
            if account:
                device_info['cn_name'] = account.cn_name
                device_info['name'] = account.name
                OrgnizationEntity.objects.filter(id=device.org_info_id).exists()
                device_info['org_name'] = organ_name
            else:
                device_info['cn_name'] = None
                device_info['name'] = None
                device_info['org_name'] = organ_name
            res_list.append(device_info)
        return Response({
            'code': 200,
            'pageCount': page_info.get('page_count'),
            'current': page_info.get('current_page'),
            'pageSize': page_info.get('per_page'),
            'total': total_number,
            'data': res_list
        })

    def post(self, request):
        device_id = request.data.get('device_id')
        org_id = request.data.get('org_id')

        usb = request.data.get('device_usb')
        usb_promission = request.data.get('usb_promission')
        if usb_promission == 1:
            usb_promission = "r"
        elif usb_promission == 0:
            usb_promission = "rw"
        else:
            usb_promission = None
        if not device_id and not org_id:
            return Response({
                'code': 400,
                'msg': '缺少参数'
            })

        if org_id:
            accounts_ids = []
            for org in org_id:
                accounts = get_sub_account(org)
                if accounts:
                    accounts_ids = accounts_ids + list(accounts.values_list('id', flat=True))
            if not accounts_ids:
                return Response({
                    'code': 400,
                    'msg': '部门下无用户'
                })

            devices = Devices.objects.exclude(is_domain=0).filter(is_up=1, accounts_id__in=accounts_ids).all()
        else:
            devices = Devices.objects.exclude(is_domain=0).filter(is_up=1, id__in=device_id).all()

        if not devices:
            return Response({
                'code': 400,
                'msg': '设置未提交， 请检查设备是否未启用 或 部门下是否有设备'
            })

        for device in devices.only('id'):
            if AccountEntity.objects.filter(id=device.accounts_id).exists():
                org_info_id = AccountEntity.objects.get(id=device.accounts_id).org_info_id
            else:
                org_info_id = None
            usb_devices, _ = USBDevicePolicy.objects.get_or_create(device_id=device.id)
            usb_devices.usb_rw_status = usb_promission
            usb_devices.org_info_id = org_info_id
            usb_devices.org_usb_rw = usb_promission
            usb_devices.status = 2
            usb_devices.save()

        if org_id:
            celery_task = usb_policy.delay(
                DeviceSerializer(devices, many=True).data,  # celery参数不能是query_set类型
                {"usb": usb, "usb_rw_status": usb_promission},
            )
        else:
            # 单独启停usb，涉及读写权限处理
            dev_account_id = Devices.objects.get(id=device_id[0])
            if AccountEntity.objects.filter(id=dev_account_id.accounts_id).exists():
                org_info_id = AccountEntity.objects.get(id=dev_account_id.accounts_id).org_info_id
                if USBDevicePolicy.objects.filter(org_info_id=org_info_id, org_usb_rw__isnull=False).exists():
                    usb_promission = USBDevicePolicy.objects.filter(org_info_id=org_info_id,
                                                                    org_usb_rw__isnull=False).first().org_usb_rw
                else:
                    usb_promission = DefaultPolicy().get_default_policy().get('usb_rw_status')
            else:
                usb_promission = DefaultPolicy().get_default_policy().get('usb_rw_status')

            celery_task = usb_policy.delay(
                DeviceSerializer(devices, many=True).data,  # celery参数不能是query_set类型
                {"usb": usb, "usb_rw_status": usb_promission},
            )

        return Response({
            'code': 200,
            'msg': '设置已提交',
            'task_id': celery_task.id
        })

    def put(self, request):
        """
        批量 usb，启用，停用
        :param request: body {is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        device_usb： 0,1,}
                        query_params{......: 筛选条件}
        :return:
        """
        is_all = request.data.get('is_all')
        ids_list = request.data.get('ids_list')
        device_usb = request.data.get('device_usb')

        if is_all is None:
            return Response('is_all 为必输选项', status=status.HTTP_400_BAD_REQUEST)
        if ids_list is None:
            return Response('ids_list 为必输选项', status=status.HTTP_400_BAD_REQUEST)
        if device_usb is None:
            return Response('device_usb 为必输选项', status=status.HTTP_400_BAD_REQUEST)

        is_all = int(is_all)
        device_usb = int(device_usb)  # 将字符转换为数字

        if device_usb not in [0, 1]:
            return Response('device_usb 传参错误', status=status.HTTP_400_BAD_REQUEST)

        queryset = Devices.objects.exclude(is_domain=0).all()

        if is_all == 0:
            ids_list = [int(i) for i in ids_list.split(',')]
        else:
            device_cnname = self.request.query_params.get('device_cnname', default=None)
            device_ip = self.request.query_params.get('device_ip', default=None)
            org_name = self.request.query_params.get('org_name', default=None)
            name = self.request.query_params.get('name', default=None)
            query_device_usb = self.request.query_params.get('device_usb', default=None)
            policy_status = self.request.query_params.get('status', default=None)
            if device_cnname:
                queryset = queryset.filter(device_cnname=device_cnname)
            if device_ip:
                queryset = queryset.filter(device_ip=device_ip)
            if org_name:
                org_items = OrgnizationEntity.objects.all()
                if org_items.filter(name=org_name).exists():
                    org_items_id = org_items.get(name=org_name).id
                    accounts_list = AccountEntity.objects.filter(org_info_id=org_items_id).all().values_list('id',
                                                                                                             flat=True)
                else:
                    accounts_list = []
                queryset = queryset.filter(accounts_id__in=accounts_list)
            if name:
                accounts_list = AccountEntity.objects.filter(name__startswith=name).all().values_list('id', flat=True)
                queryset = queryset.filter(accounts_id__in=accounts_list)
            if query_device_usb:
                device_id_list = USBDevicePolicy.objects.filter(usb=int(query_device_usb)).values_list('device_id',
                                                                                                       flat=True)
                queryset = queryset.filter(id__in=device_id_list)

            if policy_status:
                device_id_list = USBDevicePolicy.objects.filter(status=policy_status).values_list('device_id',
                                                                                                  flat=True)
                queryset = queryset.filter(id__in=device_id_list)

            ids_list = queryset.values_list('id', flat=True)

        # 下发usb策略程序到终端
        device_all = Devices.objects.filter(id__in=ids_list).all().defer('device_root').order_by('-id')
        for device in device_all.only('id'):
            if AccountEntity.objects.filter(id=device.accounts_id).exists():
                org_info_id = AccountEntity.objects.get(id=device.accounts_id).org_info_id
                if USBDevicePolicy.objects.filter(org_info_id=org_info_id, org_usb_rw__isnull=False).exists():
                    usb_promission = USBDevicePolicy.objects.filter(org_info_id=org_info_id,
                                                                    org_usb_rw__isnull=False).first().org_usb_rw
                else:
                    usb_promission = DefaultPolicy().get_default_policy().get('usb_rw_status')
            else:
                usb_promission = DefaultPolicy().get_default_policy().get('usb_rw_status')
            usb_devices, _ = USBDevicePolicy.objects.get_or_create(device_id=device.id)
            usb_devices.usb_rw_status = usb_promission
            usb_devices.status = 2
            usb_devices.save()

            logger.info(f"********************{device_usb}")
            celery_task = usb_policy.delay(
                [DeviceSerializer(device).data],  # celery参数不能是query_set类型
                {"usb": device_usb, "usb_rw_status": usb_promission},
            )

        return Response({'code': 200, 'msg': '批量设置USB策略成功'}, status=status.HTTP_200_OK)


class USBReordOLD(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, device_id):
        id_serial = request.GET.get('id_serial')
        action = request.GET.get('action')
        manufactor = request.GET.get('manufactor')

        device = Devices.objects.exclude(is_domain=0, id=device_id).first()
        if not device:
            return Response({
                'success': False,
                'errorMessage': '此设备未加域'
            })
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)

        if id_serial and manufactor:
            sql = 'SELECT `devpath`,`product`,`id_serial`,`manufactor` FROM `firm_info` where id_serial like "{id_serial}%" and  manufactor like "{manufactor}%";'.format(
                id_serial=id_serial, manufactor=manufactor
            )
        elif id_serial:
            sql = 'SELECT `devpath`,`product`,`id_serial`,`manufactor` FROM `firm_info` where id_serial like "{id_serial}%";'.format(
                id_serial=id_serial
            )
        elif manufactor:
            sql = 'SELECT `devpath`,`product`,`id_serial`,`manufactor` FROM `firm_info` where manufactor like "{manufactor}%";'.format(
                manufactor=manufactor
            )
        else:
            sql = 'SELECT `devpath`,`product`,`id_serial`,`manufactor` FROM `firm_info`;'

        firm_info_res = salt_api.sqlite_fetch(device.device_hostname,
                                              ['/usr/share/nfs-domain-client/db/data.db', '{0}'.format(sql)])

        firm_query_list = firm_info_res[device.device_hostname]
        if isinstance(firm_query_list, bool):
            logger.warning(firm_info_res)
            return Response({
                'success': False,
                'errorMessage': '终端 组件 不在线'
            })
        if isinstance(firm_query_list, str):
            logger.warning(firm_info_res)
            return Response({
                'success': False,
                'errorMessage': firm_query_list
            })
        if len(firm_query_list) == 0:
            return Response({
                'success': True,
                'pageCount': 0,
                'current': 0,
                'pageSize': 0,
                'total': 0,
                'data': []
            })
        data = []
        # for info in firm_info_res:
        #     if action:
        #         sql = 'SELECT * FROM `usb_event` where action="{action}" and devpath="{devpath}" and product="{product}" order by `id` desc;'.format(
        #             action=action,devpath=info[0],product=info[1]
        #         )
        #     else:
        #         sql = 'SELECT * FROM `usb_event` where devpath="{devpath}" and product="{product}" order by `id` desc;'.format(
        #             devpath=info[0], product=info[1]
        #         )
        #     res = salt_api.sqlite_fetch(device.device_hostname,
        #                                 ['/usr/share/nfs-domain-client/db/data.db', '{0}'.format(sql)])
        #
        #     query_list = res[device.device_hostname]
        #     if isinstance(query_list, bool):
        #         continue
        #     if isinstance(query_list, str):
        #         continue
        #     for query in query_list:
        #         data.append({
        #             'id': query[0],
        #             'id_serial':info[2],
        #             'manufactor':info[3],
        #             'action': query[2],
        #             'time': query[4],
        #         })

        if action:
            sql = 'SELECT `id`,`devpath`,`product`,`action`,`time`  FROM `usb_event` where action="{action}" order by `id` desc;'.format(
                action=action
            )
        else:
            sql = 'SELECT `id`,`devpath`,`product`,`action`,`time`  FROM `usb_event`order by `id` desc;'

        usb_event_res = salt_api.sqlite_fetch(device.device_hostname,
                                              ['/usr/share/nfs-domain-client/db/data.db', '{0}'.format(sql)])
        event_query_list = usb_event_res[device.device_hostname]
        if isinstance(event_query_list, bool):
            logger.warning(usb_event_res)
            return Response({
                'success': False,
                'errorMessage': '终端 组件 不在线'
            })
        if isinstance(event_query_list, str):
            logger.warning(usb_event_res)
            return Response({
                'success': False,
                'errorMessage': event_query_list
            })
        if len(event_query_list) == 0:
            return Response({
                'success': True,
                'pageCount': 0,
                'current': 0,
                'pageSize': 0,
                'total': 0,
                'data': []
            })

        for firm_query in firm_query_list:
            for event_query in event_query_list:
                logger.info(firm_query[0])
                logger.info(firm_query[1])
                logger.info(event_query[1])
                logger.info(event_query[2])

                if firm_query[0] == event_query[1] and firm_query[1] == event_query[2]:
                    data.append({
                        'id': event_query[0],
                        'id_serial': firm_query[2],
                        'manufactor': firm_query[3],
                        'action': event_query[3],
                        'time': event_query[4],
                    })
                    continue

        total_number = len(data)
        page_info = pager_currency(request, total_number)
        logger.info(data)
        return Response({
            'success': True,
            'pageCount': page_info.get('page_count'),
            'current': page_info.get('current_page'),
            'pageSize': page_info.get('per_page'),
            'total': total_number,
            'data': data[page_info.get('start'):page_info.get('limit')]
        })


class USBReord(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, device_id):
        id_serial = request.GET.get('id_serial')
        action = request.GET.get('action')
        logger.info(f'****{action}****')
        manufactor = request.GET.get('manufactor')

        device = Devices.objects.exclude(is_domain=0).filter(id=device_id).first()
        if not device:
            return Response({
                'success': False,
                'errorMessage': '此设备未加域'
            })
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)

        if id_serial and manufactor:
            sql = 'SELECT `id`,`id_serial`,`action`,`manufactor`,`time` FROM `usb_event` where id_serial like "{id_serial}%" and  manufactor like "{manufactor}%";'.format(
                id_serial=id_serial, manufactor=manufactor
            )
        elif id_serial:
            sql = 'SELECT `id`,`id_serial`,`action`,`manufactor`,`time` FROM `usb_event` where id_serial like "{id_serial}%";'.format(
                id_serial=id_serial
            )
        elif manufactor:
            sql = 'SELECT `id`,`id_serial`,`action`,`manufactor`,`time` FROM `usb_event` where manufactor like "{manufactor}%";'.format(
                manufactor=manufactor
            )
        elif action:
            sql = 'SELECT `id`,`id_serial`,`action`,`manufactor`,`time` FROM `usb_event` where action like "{action}%";'.format(
                action=action
            )
        else:
            sql = 'SELECT `id`,`id_serial`,`action`,`manufactor`,`time` FROM `usb_event`;'

        usb_info_res = salt_api.sqlite_fetch(device.device_hostname,
                                             ['/usr/share/nfs-domain-client/db/data.db', '{0}'.format(sql)])

        usb_query_list = usb_info_res[device.device_hostname]
        if isinstance(usb_query_list, bool):
            logger.warning(usb_info_res)
            return Response({
                'success': False,
                'errorMessage': '终端 组件 不在线'
            })
        if isinstance(usb_query_list, str) or len(usb_query_list) == 0:
            logger.warning(usb_info_res)
            return Response({
                'success': True,
                'pageCount': 0,
                'current': 0,
                'pageSize': 0,
                'total': 0,
                'data': []
            })
        data = []

        for usb_query in usb_query_list:
            data.append({
                'id': usb_query[0],
                'id_serial': usb_query[1],
                'action': usb_query[2],
                'manufactor': usb_query[3],
                'time': usb_query[4],
            })

        total_number = len(data)
        page_info = pager_currency(request, total_number)
        # logger.info(data)
        return Response({
            'success': True,
            'pageCount': page_info.get('page_count'),
            'current': page_info.get('current_page'),
            'pageSize': page_info.get('per_page'),
            'total': total_number,
            'data': data[page_info.get('start'):page_info.get('limit')]
        })


class WallpaperAPIView(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def is_contains_chinese(self, s):
        for _char in s:
            if '\u4e00' <= _char <= '\u9fa5':
                return True
        return False

    def get(self, request):
        wallpapers = Wallpaper.objects.all().defer('image').order_by('-id')
        total_number = wallpapers.count()
        data = {
            'code': 200,
            'total': total_number,
        }
        if request.GET.get('pageSize'):
            page_info = pager_currency(request, total_number)
            wallpapers = wallpapers[page_info.get('start'):page_info.get('limit')]
            data.setdefault('current', page_info.get('current_page'))
            data.setdefault('pageCount', page_info.get('page_count'))
            data.setdefault('pageSize', page_info.get('per_page'))
        data.setdefault('data', WallpaperSerializer(instance=wallpapers, many=True).data)
        return Response(data)

    def post(self, request):
        image = request.FILES.get('file')
        image_name = image.name
        if len(image_name) > 30:
            return Response({
                'code': 400,
                'msg': '图片名字过长'
            })
        # if self.is_contains_chinese(image_name):
        #     return Response({
        #         'code': 400,
        #         'msg': '图片名中不能包含中文'
        #     })
        logger.info(f"*******{image_name.split('.')[0]}*****")
        if re.findall('[?~!@#$%^&.*_-]+', image_name.split('.')[0]):
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '名称中不能含有特殊字符'
            })
        ext = os.path.splitext(image_name)[1]
        if ext.lower() not in settings.WALLPAPER_TYPE:
            return Response({
                'code': 400,
                'msg': '图片格式有误，仅支持：{}'.format(settings.WALLPAPER_TYPE)
            })
        if Wallpaper.objects.filter(name=image_name).first():
            return Response({
                'code': 400,
                'msg': '图片名重复'
            })

        image_path = '/'.join([settings.WALLPAPERPATH, image_name])
        with open(image_path, 'wb') as f:
            for chunk in image.chunks():
                f.write(chunk)
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        if os.environ.get('DEPLOY_ARCH') == 'cluster':
            if not salt_api.api_wallpaper_add(image_path, image.name):
                return Response({
                    'code': 400,
                    'msg': '图片上传失败'
                })
        Wallpaper.objects.create(name=image.name, image=image_path)
        return Response({
            'code': 200,
            'msg': '图片上传成功'
        })

    def put(self, request, wallpaper_id):
        name = request.data.get('image_name')

        re_name = os.path.splitext(name)
        if re.findall('[?~!@#$%^&*._-]+', re_name[0]):
            return {
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '名称中不能含有特殊字符'
            }
        ext = re_name[1]
        if ext.lower() not in settings.WALLPAPER_TYPE:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '图片格式有误，仅支持：{}'.format(settings.WALLPAPER_TYPE)
            })

        wallpaper = Wallpaper.objects.filter(id=wallpaper_id).first()
        if Wallpaper.objects.filter(name=name).first():
            return Response({
                'code': 400,
                'msg': '图片名重复'
            })
        image = wallpaper.image
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        if os.environ.get('DEPLOY_ARCH') == 'cluster':
            if not salt_api.api_wallpaper_modify(os.path.basename(image), name):
                return Response({
                    'code': 400,
                    'msg': '修改失败'
                })
        new_image = os.path.join(settings.WALLPAPERPATH, name)
        os.rename(image, new_image)
        wallpaper.name = name
        wallpaper.image = new_image
        wallpaper.save()
        return Response({
            'code': 200,
            'msg': '修改成功'
        })

    def delete(self, request, wallpaper_id):
        wallpaper = Wallpaper.objects.filter(id=wallpaper_id).first()
        if not wallpaper:
            return Response({
                'code': 400,
                'msg': '壁纸不存在'
            })
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        if os.environ.get('DEPLOY_ARCH') == 'cluster':
            if not salt_api.api_wallpaper_delete(os.path.basename(wallpaper.image)):
                return Response({
                    'code': 400,
                    'msg': '删除失败'
                })
        try:
            os.remove(wallpaper.image)
        except:
            pass
        finally:
            wallpaper.delete()

        return Response({
            'code': 200,
            'msg': '删除成功'
        })


class WallpaperPolicy(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        default_wallpaper = Wallpaper.objects.filter(is_default=1).first()
        if not default_wallpaper:
            return Response({
                'code': 200,
                'data': None
            })
        return Response({
            'code': 200,
            'data': WallpaperSerializer(default_wallpaper).data
        })

    def post(self, request, wallpaper_id):
        default_wallpaper = Wallpaper.objects.filter(is_default=1).first()
        if default_wallpaper:
            default_wallpaper.is_default = 0
            default_wallpaper.save()

        if wallpaper_id in (0, '0'):
            return Response({
                'code': 200,
                'msg': '默认壁纸已经取消'
            })
        wallpaper = Wallpaper.objects.filter(id=wallpaper_id).first()
        wallpaper.is_default = 1
        wallpaper.save()
        return Response({
            'code': 200,
            'msg': '默认壁纸设置成功'
        })


class WallpaperDevice(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    @classmethod
    def organ_down_list(cls, num):
        """获取父级及以下的部门"""
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return [int(num)]
        else:
            organ_down_list = organ_down_list + list(items.values_list('id', flat=True))
            while True:
                # logger.info(f'**********{items}*********')
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('id', flat=True))

            organ_down_list.append(int(num))
        return organ_down_list

    def get(self, request):
        device_all = Devices.objects.exclude(is_domain=0).all().defer('device_root').order_by('-id')

        # 根据部门过滤数据，超级管理员不需要
        user = User.objects.filter(username=request.user.username).first()
        org_list = get_org_permission_list(user)

        # 根据部门筛选数据
        if org_list:
            device_all = device_all.filter(org_info__in=org_list)

        device_cnname = request.GET.get('device_cnname')
        device_ip = request.GET.get('device_ip')
        accounts_id = request.GET.get('accounts_id')
        org_id = request.GET.get('org_id')
        cn_name = request.GET.get('cn_name')
        name = request.GET.get('name')
        switch_status = request.GET.get('switch_status')
        status = request.GET.get('status')
        is_wallpaper = request.GET.get('is_wallpaper')
        wallpaper_name = request.GET.get('image_name')

        if device_cnname:
            device_all = device_all.filter(device_cnname__icontains=device_cnname)
        if device_ip:
            device_all = device_all.filter(device_ip__icontains=device_ip)
        if accounts_id:
            account = AccountEntity.objects.filter(id=accounts_id).first()
            device_all = device_all.filter(accounts_id=account.id)
        if name:
            accounts = AccountEntity.objects.filter(name__icontains=name).all().values_list('id', flat=True)
            device_all = device_all.filter(accounts_id__in=accounts)
        if switch_status:
            device_id_list = ScreensaversDevicePolicy.objects.filter(switch_status=switch_status).values_list(
                'device_id')
            device_all = device_all.filter(id__in=device_id_list)
        if org_id:
            org = self.organ_down_list(org_id)
            device_all = device_all.filter(org_info_id__in=org)
        if wallpaper_name:
            wallpaper = Wallpaper.objects.filter(name__icontains=wallpaper_name).first()
            if not wallpaper:
                return Response({
                    'code': 200,
                    "pageCount": 1,
                    "current": 1,
                    "pageSize": 10,
                    "total": 0,
                    "data": []
                })
            device_id_list = WallpaperDevicePolicy.objects.filter(wallpaper_id=wallpaper.id).values_list('device_id',
                                                                                                         flat=True)
            device_all = device_all.filter(id__in=device_id_list)

        total_number = device_all.count()
        page_info = pager_currency(request, total_number)
        devices = device_all[page_info.get('start'):page_info.get('limit')]
        res_list = []
        for device in devices:
            if is_wallpaper == '1' and not device.fwdp: continue
            if is_wallpaper == '0' and device.fwdp: continue

            device_info = {}
            device_ip = device.device_ip
            device_info['id'] = device.id
            device_info['device_ip'] = device_ip
            device_info['accounts_id'] = device.accounts_id
            device_info['device_cnname'] = device.device_cnname
            # wallpaper_device = WallpaperDevicePolicy.objects.filter(device_id=device.id).first()
            # if wallpaper_device and wallpaper_device.wallpaper_id:
            #     wallpaper = Wallpaper.objects.filter(id=wallpaper_device.wallpaper_id).first()
            #     device_info['image_name'] = wallpaper.name
            #     device_info['image_routing'] = wallpaper.image_routing
            #     device_info['status'] = wallpaper_device.status
            #     device_info['jid'] = wallpaper_device.jid
            # else:
            #     device_info['image_name'] = None
            #     device_info['image_routing'] = None
            #     device_info['status'] = None
            #     device_info['jid'] = None
            screen_device = ScreensaversDevicePolicy.objects.filter(device_id=device.id).first()
            if screen_device and screen_device.device_id:
                device_info['switch_status'] = screen_device.switch_status
                device_info['interval'] = (
                        int(screen_device.interval) / 60) if screen_device.interval is not None else None
                device_info['status'] = screen_device.status
                device_info['jid'] = screen_device.jid
            else:
                device_info['switch_status'] = None
                device_info['interval'] = None
                device_info['status'] = None
                device_info['jid'] = None

            if status and int(status) != device_info['status']: continue
            accounts_id = device.accounts_id
            account = AccountEntity.objects.filter(id=accounts_id).first()
            if account:
                device_info['cn_name'] = account.cn_name
                device_info['name'] = account.name
            else:
                device_info['cn_name'] = None
                device_info['name'] = None

            device_info['org_name'] = device.org_info.name
            res_list.append(device_info)
        return Response({
            'code': 200,
            'pageCount': page_info.get('page_count'),
            'current': page_info.get('current_page'),
            'pageSize': page_info.get('per_page'),
            'total': total_number,
            'data': res_list
        })

    def post(self, request):
        device_id = request.data.get('device_id')
        org_id = request.data.get('org_id')
        wallpaper_id = request.data.get('wallpaper_id', None)
        interval = request.data.get('interval', None)
        switch_status = request.data.get('switch_status')
        if switch_status == 1:
            interval = interval * 60
        else:
            interval = 0
        if not device_id and not org_id:
            return Response({
                'code': 400,
                'msg': '缺少参数'
            })
        if org_id:
            accounts_ids = []
            for org in org_id:
                accounts = get_sub_account(org)
                if accounts:
                    accounts_ids = accounts_ids + list(accounts.values_list('id', flat=True))

            if not accounts_ids:
                devices = Devices.objects.exclude(is_domain=0).filter(is_up=1, org_info_id__in=org_id).all()
            else:
                devices = Devices.objects.exclude(is_domain=0).filter(is_up=1).filter(Q(accounts_id__in=accounts_ids)|Q(org_info_id__in=org_id))
        else:
            devices = None
        if not devices:
            return Response({
                'code': 400,
                'msg': '设置未提交， 请检查设备是否未启用 或 部门下是否有设备'
            })
        if wallpaper_id is not None:
            for device in devices.only('id'):
                wallpaper_device, _ = WallpaperDevicePolicy.objects.get_or_create(device_id=device.id)
                wallpaper_device.wallpaper_id = wallpaper_id
                wallpaper_device.status = 2
                wallpaper_device.save()

            celery_task = wallpaper_policy.delay(
                DeviceSerializer(devices, many=True).data,  # celery参数不能是query_set类型
                wallpaper_id
            )
        if switch_status is not None:
            # 下发屏幕保护程序到终端
            for device in devices.only('id'):
                screensavers_device, _ = ScreensaversDevicePolicy.objects.get_or_create(device_id=device.id)
                screensavers_device.interval = int(interval)
                screensavers_device.status = 2
                screensavers_device.save()
            celery_task = screensavers_policy.delay(
                DeviceSerializer(devices, many=True).data,  # celery参数不能是query_set类型
                {'interval': int(interval), 'switch_status': switch_status}
            )

        return Response({
            'code': 200,
            'msg': '设置已提交',
            'task_id': celery_task.id
        })


class FirewalldPolicy(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        default_policy = DefaultPolicy()
        return Response({
            'code': 200,
            'data': {
                'firewalld': int(default_policy.get_default_policy().get('firewalld')),
                'firewalld_level': int(default_policy.get_default_policy().get('firewalld_level')),
            }
        })

    def post(self, request):
        firewalld = request.data.get('firewalld')
        firewalld_level = request.data.get('firewalld_level')
        default_policy = DefaultPolicy()
        if firewalld in (0, 1):
            default_policy.set_default_policy('firewalld', firewalld)
        if firewalld_level in (0, 1, 2, 3):
            default_policy.set_default_policy('firewalld_level', firewalld_level)
        return Response({
            'code': 200,
            'msg': '设置成功'
        })


class FirewalldRosterPolicy(ModelViewSet):
    queryset = FirewalldWhiteBlackRosterPolicy.objects.all().order_by('-id')
    serializer_class = FirewalldRosterSerializers
    # pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    throttle_classes = (UserRateThrottle,)

    # ordering = ('-pk')

    def list(self, request, *args, **kwargs):
        data = {
            'white': [],
            'black': []
        }
        default_policy = DefaultPolicy()
        data['rule'] = int(default_policy.get_default_policy().get('firewalld_roster'))
        white = self.get_queryset().filter(roster=0).all()
        black = self.get_queryset().filter(roster=1).all()
        serializer = self.get_serializer(white, many=True)
        data['white'] = serializer.data
        serializer = self.get_serializer(black, many=True)
        data['black'] = serializer.data
        return Response(data)

    @action(methods=['put'], detail=False)
    def multiple_set(self, request, *args, **kwargs):
        '''
        data = {
            'rule':0,  0 white, 1 black, 2 off
            'white':[],
            'black':[],
        }
        '''
        req_data = request.data
        default_policy = DefaultPolicy()
        default_policy.set_default_policy('firewalld_roster', req_data.get('rule'))
        self.get_queryset().delete()
        for item in req_data.get('white'):
            serializer = self.get_serializer(data={'roster': 0, 'site': item})
            serializer.is_valid(raise_exception=True)
            self.perform_create(serializer)

        for item in req_data.get('black'):
            serializer = self.get_serializer(data={'roster': 1, 'site': item})
            serializer.is_valid(raise_exception=True)
            self.perform_create(serializer)
        arg = ['Other', [], []]
        if req_data.get('rule') == 0:
            arg[0] = 'White'
        elif req_data.get('rule') == 1:
            arg[0] = 'Black'
        else:
            arg[0] = 'Other'
        arg[1] = list(self.get_queryset().filter(roster=0).values_list('site', flat=True))
        arg[2] = list(self.get_queryset().filter(roster=1).values_list('site', flat=True))
        devices = Devices.objects.exclude(is_domain=0).filter(is_up=1).all()
        for device in devices:
            firewalld_black_white_roster_policy.delay(
                [DeviceSerializer(device).data],  # celery参数不能是query_set类型
                arg,
                signal=False
            )
        return Response({'msg': '设置成功'})


class FirewalldDevice(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    @classmethod
    def organ_down_list(cls, num):
        """获取父级及以下的部门"""
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return [int(num)]
        else:
            organ_down_list = organ_down_list + list(items.values_list('id', flat=True))
            while True:
                # logger.info(f'**********{items}*********')
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('id', flat=True))

            organ_down_list.append(int(num))
        return organ_down_list

    def get(self, request):
        device_all = Devices.objects.exclude(is_domain=0).all().defer('device_root').order_by('-id')

        # 根据部门过滤数据，超级管理员不需要
        user = User.objects.filter(username=request.user.username).first()
        org_list = get_org_permission_list(user)

        # 根据部门筛选数据
        if org_list:
            device_all = device_all.filter(org_info__in=org_list)

        device_cnname = request.GET.get('device_cnname')
        device_ip = request.GET.get('device_ip')
        accounts_id = request.GET.get('accounts_id')
        org_id = request.GET.get('org_id')
        cn_name = request.GET.get('cn_name')
        name = request.GET.get('name')
        status = request.GET.get('status')
        firewalld = request.GET.get('device_firewalld')
        firewalld_level = request.GET.get('firewalld_level')

        if device_cnname:
            device_all = device_all.filter(device_cnname__icontains=device_cnname)
        if device_ip:
            device_all = device_all.filter(device_ip__icontains=device_ip)
        if accounts_id:
            account = AccountEntity.objects.filter(id=accounts_id).first()
            device_all = device_all.filter(accounts_id=account.id)
        if name:
            accounts = AccountEntity.objects.filter(name__icontains=name).all().values_list('id', flat=True)
            device_all = device_all.filter(accounts_id__in=accounts)
        if org_id:
            org = self.organ_down_list(org_id)
            if not org:
                return Response({
                    'code': 200,
                    "pageCount": 1,
                    "current": 1,
                    "pageSize": 10,
                    "total": 0,
                    "data": []
                })
            accounts = AccountEntity.objects.filter(org_info_id__in=org)
            if cn_name:
                accounts = accounts.filter(cn_name=cn_name)
            device_all = device_all.filter(accounts_id__in=accounts.values_list('id', flat=True))
        total_number = device_all.count()
        page_info = pager_currency(request, total_number)
        devices = device_all[page_info.get('start'):page_info.get('limit')]
        res_list = []
        for device in devices:
            firewalld_device = FirewalldDevicePolicy.objects.filter(device_id=device.id).first()
            if firewalld:
                if not firewalld_device or firewalld_device.firewalld != int(firewalld): continue
            if firewalld_level:
                if not firewalld_device or firewalld_device.firewalld_level != int(firewalld_level): continue
            if status in ('0',):
                if firewalld_device: continue
            elif status in ('1', '2', '3'):
                if not firewalld_device or firewalld_device.status != int(status): continue
            device_info = {}
            device_ip = device.device_ip
            device_info['id'] = device.id
            device_info['device_ip'] = device_ip
            device_info['accounts_id'] = device.accounts_id
            device_info['device_cnname'] = device.device_cnname

            if not firewalld_device:
                device_info['device_firewalld'] = None
                device_info['device_firewalld_level'] = None
                device_info['status'] = 0
                device_info['jid'] = None
            else:
                device_info['device_firewalld'] = firewalld_device.firewalld
                device_info['device_firewalld_level'] = firewalld_device.firewalld_level
                device_info['status'] = firewalld_device.status
                device_info['jid'] = firewalld_device.jid
            accounts_id = device.accounts_id
            account = AccountEntity.objects.filter(id=accounts_id).first()
            if account:
                device_info['cn_name'] = account.cn_name
                device_info['name'] = account.name
                org = account.org_info
                device_info['org_name'] = org.name
            else:
                device_info['cn_name'] = None
                device_info['name'] = None
                device_info['org_name'] = None
            res_list.append(device_info)
        return Response({
            'code': 200,
            'pageCount': page_info.get('page_count'),
            'current': page_info.get('current_page'),
            'pageSize': page_info.get('per_page'),
            'total': total_number,
            'data': res_list
        })

    def post(self, request):
        device_id = request.data.get('device_id')
        org_id = request.data.get('org_id')
        firewalld = request.data.get('firewalld')
        firewalld_level = request.data.get('firewalld_level')
        if not device_id and not org_id:
            return Response({
                'code': 400,
                'msg': '缺少参数'
            })

        if org_id:
            accounts_ids = []
            for org in org_id:
                accounts = get_sub_account(org)
                if accounts:
                    accounts_ids = accounts_ids + list(accounts.values_list('id', flat=True))
            if not accounts_ids:
                return Response({
                    'code': 400,
                    'msg': '部门下无用户'
                })

            devices = Devices.objects.exclude(is_domain=0).filter(is_up=1, accounts_id__in=accounts_ids).all()
        else:
            devices = Devices.objects.exclude(is_domain=0).filter(is_up=1, id__in=device_id).all()

        if not devices:
            return Response({
                'code': 400,
                'msg': '设置未提交， 请检查设备是否未启用 或 部门下是否有设备'
            })
        for device in devices.only('id'):
            wallpaper_device, _ = FirewalldDevicePolicy.objects.get_or_create(device_id=device.id)
            if firewalld in (0, 1) and wallpaper_device.firewalld != firewalld:
                wallpaper_device.firewalld = firewalld
                wallpaper_device.save()
                firewalld_policy.delay(
                    [DeviceSerializer(device).data],  # celery参数不能是query_set类型
                    firewalld
                )
            if device.os.lower() == 'kylin' or device.os.lower() == 'uos': continue
            if firewalld_level in (0, 1, 2) and wallpaper_device.firewalld_level != firewalld_level:
                wallpaper_device.firewalld_level = firewalld_level
                wallpaper_device.save()
                firewalld_level_policy.delay(
                    [DeviceSerializer(device).data],  # celery参数不能是query_set类型
                    firewalld_level
                )
        return Response({
            'code': 200,
            'msg': '设置已提交',
        })


class FirewalldDeviceBatchUpdate(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def post(self, request):
        try:
            ids_list_str = request.data.get('ids_list')
            is_all = request.data.get('is_all', 0)

            ids_list = []
            if ids_list_str:
                if "," in ids_list_str:
                    ids_list = ids_list_str.split(",")
                else:
                    ids_list = [ids_list_str]

            if is_all == 1:
                device_all = Devices.objects.exclude(is_domain=0).all().defer('device_root').order_by('-id')
            else:
                device_all = Devices.objects.exclude(is_domain=0).filter(is_up=1, id__in=ids_list).all()

            # 根据部门过滤数据，超级管理员不需要
            user = User.objects.filter(username=request.user.username).first()
            org_list = get_org_permission_list(user)

            # 根据部门筛选数据
            if org_list:
                device_all = device_all.filter(org_info__in=org_list)

            device_cnname = request.data.get('device_cnname')
            device_ip = request.data.get('device_ip')
            accounts_id = request.data.get('accounts_id')
            org_name = request.data.get('org_name')
            cn_name = request.data.get('cn_name')
            name = request.data.get('name')
            policy_status = request.data.get('status')
            firewalld = request.data.get('firewalld')
            firewalld_level = request.data.get('firewalld_level')

            if device_cnname:
                device_all = device_all.filter(device_cnname__icontains=device_cnname)
            if device_ip:
                device_all = device_all.filter(device_ip=device_ip)
            if accounts_id:
                account = AccountEntity.objects.filter(id=accounts_id).first()
                device_all = device_all.filter(accounts_id=account.id)
            if name:
                accounts = AccountEntity.objects.filter(name__icontains=name).all().values_list('id', flat=True)
                device_all = device_all.filter(accounts_id__in=accounts)
            if org_name:
                org = OrgnizationEntity.objects.filter(name__icontains=org_name).first()
                if not org:
                    return Response({
                        'code': 400,
                        'msg': '设置未提交， 请检查设备是否未启用 或 部门下是否有设备'
                    })
                accounts = org.accountentity_set.all()
                if cn_name:
                    accounts = accounts.filter(cn_name__icontains=cn_name)
                device_all = device_all.filter(accounts_id__in=[account.id for account in accounts])

            devices = device_all
            if not devices:
                return Response({
                    'code': 400,
                    'msg': '设置未提交， 请检查设备是否未启用 或 部门下是否有设备'
                })

            for device in devices.only('id'):
                wallpaper_device, _ = FirewalldDevicePolicy.objects.get_or_create(device_id=device.id)
                if firewalld in (0, 1) and wallpaper_device.firewalld != firewalld:
                    wallpaper_device.firewalld = firewalld
                    wallpaper_device.save()
                    firewalld_policy.delay(
                        [DeviceSerializer(device).data],  # celery参数不能是query_set类型
                        firewalld
                    )
                if device.os.lower() == 'kylin' or device.os.lower() == 'uos': continue
                if firewalld_level in (0, 1, 2) and wallpaper_device.firewalld_level != firewalld_level:
                    wallpaper_device.firewalld_level = firewalld_level
                    wallpaper_device.save()
                    firewalld_level_policy.delay(
                        [DeviceSerializer(device).data],  # celery参数不能是query_set类型
                        firewalld_level
                    )
            return Response({
                'code': 200,
                'msg': '设置已提交',
            })

        except Exception as e:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '请求参数错误 %s' % e
            })


# @accept_websocket
# def DomainEvent(request, **kwargs):
#     if request.is_websocket():
#         while True:
#             recv = request.websocket.wait()
#             logger.info(recv)
#             if str(recv, encoding='utf-8') == 'off':
#                 request.websocket.send('xxx'.encode('utf-8'))
#                 break
#             logger.info('====')
#             logger.info(request.websocket)
#             request.websocket.send('hhh'.encode('utf-8'))
#             try:
#                 for recv in redis_sub.listen():
#                     logger.info(recv)
#                     # if recv.get('type') == 'unsubscribe':break
#                     data = json.loads(recv.get('data').decode('utf-8'))
#                     logger.info(data)
#                     logger.info(request.websocket)
#                     request.websocket.send(json.dumps(data, ensure_ascii=False).encode('utf-8'))
#             except Exception as e:
#                 logger.info(e)
#                 break


class DomainEvent(WebsocketConsumer):
    # websocket连接时会执行  为了实现线程高可用，上生产应使用celery+supervisor
    def connect(self):
        # 这里会取到url中传入的report_id
        # self.report_name = self.scope['url_route']['kwargs']['report_id']
        # self.report_group_name = 'chat_%s' % self.report_name
        self.group_name = 'channel'

        # group_add 加入一个小组
        # channel_layer下面的方法属于通道层方法是异步的，因此使用async_to_sync进行包装
        async_to_sync(self.channel_layer.group_add)(
            self.group_name,
            self.channel_name
        )
        # accept表示建立连接，否则可以使用close
        self.accept()

    # 断开websocket连接时会执行
    def disconnect(self, close_code):
        # 断开与小组的连接 group_discard
        async_to_sync(self.channel_layer.group_discard)(
            self.group_name,
            self.channel_name
        )
        logger.info('前端 websocket close success')

    # 接收到消息会执行
    def receive(self, text_data):
        # text_data 接收到的消息
        # recv = json.loads(text_data)
        logger.info(text_data)

        # group_send 向小组发送消息
        # self.report_group_name 指小组的名称
        # 'type': 'chat_message'  会调用chat_message发送消息
        async_to_sync(self.channel_layer.group_send)(
            self.group_name,
            {
                'type': 'push_message',
                'message': text_data
            }
        )

    # group_send被调用时，会执行指定选择的方法
    def push_message(self, event):
        message = event['message']
        # message['msg'] = message['msg'].decode('unicode_escape')
        self.send(text_data=json.dumps({
            'message': message
        }))


class ClientPing(WebsocketConsumer):

    def __init__(self):
        self.redis_cache = caches['redis']
        super(ClientPing, self).__init__()

    def refresh_device(self, device_uuid):
        self.redis_cache.set(device_uuid, 1, 3)

    def remove_device(self, device_uuid):
        self.redis_cache.delete(device_uuid)

    def connect(self):
        logging.info('#####')
        logging.info(self.scope['url_route'])
        self.device_uuid = self.scope['url_route']['kwargs']['device_uuid']
        logging.info(self.device_uuid)
        self.refresh_device(self.device_uuid)
        # accept表示建立连接，否则可以使用close
        self.accept()

    # 断开websocket连接时会执行
    def disconnect(self, close_code):
        self.remove_device(self.device_uuid)
        logger.info('websocket close success')

    # 接收到消息会执行
    def receive(self, text_data):
        # text_data 接收到的消息
        logger.info(text_data)
        recv = json.loads(text_data)
        if recv == 'ping':
            self.refresh_device(self.device_uuid)


# 所有策略信息
class SaltJids(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        device_ip = request.GET.get('minion')
        policy_no = request.GET.get('policy_no')
        jid = request.GET.get('jid')
        jobs = SaltJidDevice.objects.all().order_by('-id')
        if device_ip:
            device_ls = Devices.objects.filter(device_ip__icontains=device_ip).values_list('device_uuid', flat=True)
            jobs = jobs.filter(minion__in=device_ls)
        if policy_no:
            jobs = jobs.filter(policy_no__icontains=policy_no)
        if jid:
            jobs = jobs.filter(jid__icontains=jid)

        total_number = jobs.count()
        page_info = pager_currency(request, total_number)
        jobs = jobs[page_info.get('start'):page_info.get('limit')]

        res = []
        for job in jobs:
            device = Devices.objects.filter(
                Q(device_hostname=job.minion) | Q(device_cnname=job.minion)).first()
            job_info = {
                'policy_no': job.policy_no,
                'minion': device.device_ip if device else None,
                'fun': job.fun,
                'fun_args': job.fun_args,
                'jid': job.jid,
                'ret': job.ret,
                'alter_time': job.alter_time
            }
            res.append(job_info)
        return Response({
            'code': 200,
            'data': res,
            'pageCount': page_info.get('page_count'),
            'current': page_info.get('current_page'),
            'pageSize': page_info.get('per_page'),
            'total': total_number,
        })


class SaltJidRandered(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, jid):
        job = SaltJidDevice.objects.get_jid(jid)
        if job.fun in ['state.sls', 'state.apply', 'state.highstate']:
            formatted, _, _ = highstate_output.output({job.minion: json.loads(job.ret)})
        else:
            formatted = nested_output.output({job.minion: json.loads(job.ret)})
        conv = Ansi2HTMLConverter(inline=False, scheme='xterm')
        html_detail = conv.convert(formatted, ensure_trailing_newline=True)
        return Response(html_detail)


class ScreenSaversViewSet(mixins.RetrieveModelMixin,
                          mixins.ListModelMixin,
                          mixins.UpdateModelMixin,
                          GenericViewSet):
    queryset = Screensavers.objects.all()
    serializer_class = ScreenSaversSerializer
    pagination_class = None
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('id',)

    def update(self, request, *args, **kwargs):
        """
        更新屏幕保护配置
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        # 检查屏幕保护开关状态是否改变
        # old_switch_status = instance.switch_status
        # new_switch_status = request.data.get('switch_status')
        # if old_switch_status != new_switch_status:
        #     if new_switch_status == 1:
        #         interval = request.data.get('interval')
        #     else:
        #         interval = 0
        #
        #     # 下发屏幕保护程序到终端
        #     device_all = Devices.objects.exclude(is_domain=0).all().defer('device_root').order_by('-id')
        #     for device in device_all.only('id'):
        #         screensavers_device, _ = ScreensaversDevicePolicy.objects.get_or_create(device_id=device.id)
        #         screensavers_device.interval = int(interval) * 60
        #         screensavers_device.switch_status = new_switch_status
        #         screensavers_device.status = 2
        #         screensavers_device.save()
        #     celery_task = screensavers_policy.delay(
        #         DeviceSerializer(device_all, many=True).data,  # celery参数不能是query_set类型
        #         {'interval': int(interval) * 60, 'switch_status': new_switch_status}
        #     )

        self.perform_update(serializer)

        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        return Response(serializer.data)

    def device_filter_queryset(self, queryset):
        """
        :param queryset: 表数据结果集合
        :return: 根据过滤条件筛选返回结果集合
        """
        device_cnname = self.request.query_params.get('device_cnname', default=None)
        name = self.request.query_params.get('name', default=None)
        device_ip = self.request.query_params.get('device_ip', default=None)
        org_name = self.request.query_params.get('org_name', default=None)
        image_name = self.request.query_params.get('image_name', default=None)
        policy_status = self.request.query_params.get('status', default=None)
        if device_cnname:
            queryset = queryset.filter(device_cnname=device_cnname)
        if device_ip:
            queryset = queryset.filter(device_ip=device_ip)
        if name:
            accounts_list = AccountEntity.objects.filter(name__startswith=name).all().values_list('id', flat=True)
            queryset = queryset.filter(accounts_id__in=accounts_list)
        if org_name:
            org_items = OrgnizationEntity.objects.all()
            if org_items.filter(name=org_name).exists():
                org_items_id = org_items.get(name=org_name).id
                accounts_list = AccountEntity.objects.filter(org_info_id=org_items_id).all().values_list('id',
                                                                                                         flat=True)
            else:
                accounts_list = []
            queryset = queryset.filter(accounts_id__in=accounts_list)
        if image_name:
            wallpaper_items = Wallpaper.objects.all()
            if wallpaper_items.filter(name__startswith=image_name).exists():
                wallpaper_items_id = wallpaper_items.get(name__startswith=image_name).id
                device_id_list = WallpaperDevicePolicy.objects.filter(wallpaper_id=wallpaper_items_id).values_list(
                    'device_id', flat=True)
            else:
                device_id_list = []
            queryset = queryset.filter(id__in=device_id_list)

        if policy_status:
            device_id_list = WallpaperDevicePolicy.objects.filter(status=policy_status).values_list('device_id',
                                                                                                    flat=True)
            queryset = queryset.filter(id__in=device_id_list)

        return queryset

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def multiple_wallpaper(self, request, *args, **kwargs):
        """
        批量设置壁纸 程序
        :param request: body {is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        wallpaper_id： ,}
                        query_params{......: 筛选条件}
        """

        is_all = request.data.get('is_all')
        ids_list = request.data.get('ids_list')
        wallpaper_id = request.data.get('wallpaper_id')

        if is_all is None:
            return Response('is_all 为必输选项', status=status.HTTP_400_BAD_REQUEST)
        if ids_list is None:
            return Response('ids_list 为必输选项', status=status.HTTP_400_BAD_REQUEST)
        if wallpaper_id is None:
            return Response('wallpaper_id 为必输选项', status=status.HTTP_400_BAD_REQUEST)

        is_all = int(is_all)
        wallpaper_id = int(wallpaper_id)  # 将字符转换为数字

        queryset = Devices.objects.exclude(is_domain=0).all()

        if is_all == 0:
            ids_list = [int(i) for i in ids_list.split(',')]
        else:
            queryset = self.device_filter_queryset(queryset)
            ids_list = queryset.values_list('id', flat=True)

        # 下发壁纸程序到终端
        device_all = Devices.objects.filter(id__in=ids_list).all().defer('device_root').order_by('-id')
        for device in device_all.only('id'):
            wallpaper_device, _ = WallpaperDevicePolicy.objects.get_or_create(device_id=device.id)
            wallpaper_device.wallpaper_id = wallpaper_id
            wallpaper_device.status = 2
            wallpaper_device.save()

        celery_task = wallpaper_policy.delay(
            DeviceSerializer(device_all, many=True).data,  # celery参数不能是query_set类型
            wallpaper_id
        )

        return Response('批量设置屏保成功', status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def multiple_screensavers(self, request, *args, **kwargs):
        """
        批量设置锁屏保护 程序
        :param request: body {is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        switch_status： 0：未启用, 1: 启用
                        interval: 0  从不}
                        query_params{......: 筛选条件}
        """

        is_all = request.data.get('is_all')
        ids_list = request.data.get('ids_list')
        switch_status = request.data.get('switch_status')
        interval = request.data.get('interval')

        if is_all is None:
            return Response('is_all 为必输选项', status=status.HTTP_400_BAD_REQUEST)
        if ids_list is None:
            return Response('ids_list 为必输选项', status=status.HTTP_400_BAD_REQUEST)
        if switch_status is None:
            return Response('switch_status 为必输选项', status=status.HTTP_400_BAD_REQUEST)
        if interval is None:
            return Response('interval 为必输选项', status=status.HTTP_400_BAD_REQUEST)

        is_all = int(is_all)
        switch_status = int(switch_status)  # 将字符转换为数字

        if switch_status not in [0, 1]:
            return Response('switch_status 传参错误', status=status.HTTP_400_BAD_REQUEST)

        queryset = Devices.objects.exclude(is_domain=0).all()

        if is_all == 0:
            ids_list = [int(i) for i in ids_list.split(',')]
        else:
            queryset = self.device_filter_queryset(queryset)
            ids_list = queryset.values_list('id', flat=True)

        if switch_status == 1:
            interval = interval
        else:
            interval = 0

        # 下发屏幕保护程序到终端
        device_all = Devices.objects.filter(id__in=ids_list).all().defer('device_root').order_by('-id')
        if device_all.filter(is_up=0).exists():
            return Response('终端中有未启用的设备', status=status.HTTP_400_BAD_REQUEST)
        for device in device_all.only('id'):
            screensavers_device, _ = ScreensaversDevicePolicy.objects.get_or_create(device_id=device.id)
            screensavers_device.interval = int(interval) * 60
            screensavers_device.switch_status = switch_status
            screensavers_device.status = 2
            screensavers_device.save()
        celery_task = screensavers_policy.delay(
            DeviceSerializer(device_all, many=True).data,  # celery参数不能是query_set类型
            {'interval': int(interval) * 60, 'switch_status': switch_status}
        )

        return Response('批量设置屏保成功', status=status.HTTP_200_OK)


class PolicyDictInfoViewSet(ReadOnlyModelViewSet):
    queryset = PolicyDictInfo.objects.all()
    serializer_class = PolicyDictInfoSerializer
    pagination_class = None
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('-id',)


class PolicyInfoViewSet(ModelViewSet):
    queryset = PolicyInfo.objects.all()
    serializer_class = PolicyInfoSerializer
    pagination_class = MyPagination
    filterset_class = PolicyInfoFilter
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('-policy_create_time', '-policy_update_time', '-id')

    @staticmethod
    def device_status(device_id, no, stop_list):
        logger.info(f"{no}, {device_id}, {stop_list}")
        if no == 0:
            return 3 if str(device_id) in stop_list else 2
        elif no == 1:
            return 3 if str(device_id) in stop_list else 2
        elif no == 2:
            return 3 if str(device_id) in stop_list else 2
        elif no == 3:
            return 4
        elif no == 4:
            return 4
        else:
            return 5

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def policy_device(self, request, *args, **kwargs):
        """
        获取设备策略状态
        :param request: id: 策略id
        :param args:
        :param kwargs:
        :return: 策略id的策略详情信息
        """
        items_id = self.request.query_params.get('id')
        if self.queryset.filter(id=items_id).exists():
            items = self.queryset.get(id=items_id)
            device_string = f"{items.policy_device_list if items.policy_device_list else ''},{items.policy_device_stop if items.policy_device_stop else ''}"
            stop_items_list = items.policy_device_stop.split(',') if items.policy_device_stop else []
            items_list = device_string.split(',')
            items_list = [i for i in items_list if i is not '']
            res_data = Devices.objects.filter(id__in=items_list).values('id', 'device_hostname', 'device_cnname',
                                                                        'device_ip', 'org_info_id', 'is_up')
            if len(res_data):
                conn = get_redis_connection('redis')
                res_pd = pandas.DataFrame(res_data)
                res_pd['is_active'] = res_pd['device_hostname'].apply(
                    lambda x: 1 if conn.hget('actives', x) == b'1' else 0)
                res_pd['org_name'] = res_pd['org_info_id'].apply(
                    lambda x: OrgnizationEntity.objects.get(id=x).name if OrgnizationEntity.objects.filter(
                        id=x).exists() else None)
                res_pd['policy_status'] = res_pd['id'].apply(
                    lambda x: self.device_status(x, items.policy_run_status, stop_items_list))
                res_pd['is_stop'] = res_pd['id'].apply(lambda x: 0 if x in stop_items_list else 1)
                return Response(res_pd.to_dict('records'), status=status.HTTP_200_OK)
            else:
                return Response({[]}, status=status.HTTP_200_OK)
        else:
            return Response(f'主键{items_id}查询记录不存在', status=status.HTTP_404_NOT_FOUND)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def policy_change_device(self, request, *args, **kwargs):
        """
        获取操作策略终端开启或者关闭
        :param request: body  id: 策略id
                              device_id : 终端id
                              is_stop： 0 暂定 1重新执行
        :param args:
        :param kwargs:
        :return: 策略id的策略详情信息
        """

        items_id = request.data.get('id')
        device_id = request.data.get('device_id')
        is_stop = request.data.get('is_stop')
        if self.queryset.filter(id=items_id).exists():
            items = self.queryset.get(id=items_id)
            if is_stop == 1:
                policy_device_stop = items.policy_device_stop.split(',') if items.policy_device_stop else []
                try:
                    policy_device_stop.remove(str(device_id))
                except Exception as exp:
                    return Response(f'主键{device_id}查询记录不存在{exp}', status=status.HTTP_404_NOT_FOUND)
                items.policy_device_stop = ','.join(policy_device_stop) if policy_device_stop else None
                if items.policy_device_list:
                    items.policy_device_list = items.policy_device_list + ',' + str(device_id)
                else:
                    items.policy_device_list = str(device_id)
            else:
                policy_device_list = items.policy_device_list.split(',') if items.policy_device_list else []
                try:
                    policy_device_list.remove(str(device_id))
                except Exception as exp:
                    return Response(f'主键{device_id}查询记录不存在{exp}', status=status.HTTP_404_NOT_FOUND)

                items.policy_device_list = ','.join(policy_device_list) if policy_device_list else None
                if items.policy_device_stop:
                    items.policy_device_stop = items.policy_device_stop + ',' + str(device_id)
                else:
                    items.policy_device_stop = str(device_id)

            items.save()
            thread1 = threading.Thread(target=policy_main)
            thread1.start()
            return Response('ok', status=status.HTTP_200_OK)
        else:
            return Response(f'主键{items_id}查询记录不存在', status=status.HTTP_404_NOT_FOUND)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def policy_copy(self, request, *args, **kwargs):
        """
        :param request: id: 策略id
        :param args:
        :param kwargs:
        :return: 策略id的策略详情信息
        """
        items_id = self.request.query_params.get('id')
        if self.queryset.filter(id=items_id).exists():
            items_copy = self.queryset.filter(id=items_id).values()[0]
            del items_copy['id']
            del items_copy['policy_create_time']
        else:
            return Response(f'主键{items_id}查询记录不存在', status=status.HTTP_404_NOT_FOUND)

        # 复制条目数据初始化
        items_copy['policy_no'] = datetime.datetime.now().strftime('%Y%m%d%H%M%S')  # 生成策略序号
        items_copy['policy_run_status'] = 4
        items_copy['policy_create_user'] = request.user.username
        items_copy['policy_update_time'] = datetime.datetime.now()
        self.queryset.create(**items_copy)
        thread1 = threading.Thread(target=policy_main)
        thread1.start()

        return Response('复制记录成功', status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def sync_policy(self, request, *args, **kwargs):
        # thread1 = threading.Thread(target=calculate_policy)
        # thread1.start()
        return Response('ok', status=status.HTTP_200_OK)

    def create(self, request, *args, **kwargs):
        request_data = request.data.copy()
        policy_device_range = request_data['policy_device_web'].split(',')
        if request_data['policy_device_range'] == 0:
            # 按部门选择终端
            devices_list = Devices.objects.exclude(is_domain=0).filter(org_info_id__in=policy_device_range).values_list(
                'id', flat=True)
            request_data['policy_device_list'] = ','.join([str(i) for i in devices_list])
        elif request_data['policy_device_range'] == 1:
            # 按终端筛选终端
            request_data['policy_device_list'] = request_data['policy_device_web']
        elif request_data['policy_device_range'] == 2:
            org_id_list = GroupsDetail.objects.filter(groups_id__in=policy_device_range).values_list('org_id',
                                                                                                     flat=True)
            device_id = GroupsDetail.objects.filter(groups_id__in=policy_device_range).values_list('device_id',
                                                                                                   flat=True)
            devices_org_list = Devices.objects.exclude(is_domain=0).filter(org_info_id__in=org_id_list).values_list(
                'id',
                flat=True)
            devices_list = [str(i) for i in devices_org_list] + list(device_id)
            # 删除空值，去重
            devices_list = list(set(devices_list))
            devices_list = [str(i) for i in devices_list if i is not None]
            request_data['policy_device_list'] = ','.join(devices_list)
        else:
            return Response('policy_device_range传参数错误', status=status.HTTP_400_BAD_REQUEST)

        if request_data['policy_device_list'] == ','.join([]):
            return Response('策略未选中终端', status=status.HTTP_400_BAD_REQUEST)

        request_data['policy_create_user'] = request.user.username
        request_data['policy_update_time'] = datetime.datetime.now()
        serializer = self.get_serializer(data=request_data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        thread1 = threading.Thread(target=policy_main)
        thread1.start()
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    def update(self, request, *args, **kwargs):
        request_data = request.data.copy()
        request_data['policy_create_user'] = request.user.username
        request_data['policy_update_time'] = datetime.datetime.now()
        partial = kwargs.pop('partial', False)
        instance = self.get_object()

        policy_device_web = request_data['policy_device_web'].split(',')
        if request_data['policy_device_range'] == 0:
            # 按部门选择终端
            devices_list = Devices.objects.exclude(is_domain=0).filter(
                org_info_id__in=policy_device_web).values_list(
                'id', flat=True)
            request_data['policy_device_list'] = ','.join([str(i) for i in devices_list])
        elif request_data['policy_device_range'] == 1:
            # 按终端筛选终端
            request_data['policy_device_list'] = request_data['policy_device_web']
        elif request_data['policy_device_range'] == 2:
            org_id_list = GroupsDetail.objects.filter(groups_id__in=policy_device_web).values_list('org_id',
                                                                                                   flat=True)
            device_id = GroupsDetail.objects.filter(groups_id__in=policy_device_web).values_list('device_id',
                                                                                                 flat=True)
            devices_org_list = Devices.objects.exclude(is_domain=0).filter(org_info_id__in=org_id_list).values_list(
                'id',
                flat=True)
            devices_list = [str(i) for i in devices_org_list] + list(device_id)
            # 删除空值，去重
            devices_list = list(set(devices_list))
            devices_list = [str(i) for i in devices_list if i is not None]
            request_data['policy_device_list'] = ','.join(devices_list)
        else:
            return Response('policy_device_range传参数错误', status=status.HTTP_400_BAD_REQUEST)

        if request_data['policy_device_list'] == ','.join([]):
            return Response('策略未选中终端', status=status.HTTP_400_BAD_REQUEST)

        serializer = self.get_serializer(instance, data=request_data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}
        thread1 = threading.Thread(target=policy_main)
        thread1.start()
        return Response(serializer.data)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def policy_switch(self, request, *args, **kwargs):
        """
        批量暂停，重新执行，删除
        :param request:body {
                        is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        is_stop: 1|启用；0|暂停 空|不处理
                        is_delete : 0|不删除 1|删除 空|不处理
                        ......: 筛选条件
                        }
        :param args:
        :param kwargs:
        :return:
        """
        is_all = request.data.get('is_all')
        ids_list = request.data.get('ids_list')
        is_stop = request.data.get('is_stop')
        is_delete = request.data.get('is_delete')

        if is_all is None:
            return Response('is_all 为必输选项', status=status.HTTP_400_BAD_REQUEST)
        if ids_list is None:
            return Response('ids_list 为必输选项', status=status.HTTP_400_BAD_REQUEST)

        is_all = int(is_all)

        queryset = self.queryset

        if is_all == 0:
            ids_list = [int(i) for i in ids_list.split(',')]
        else:
            queryset = self.filter_queryset(self.get_queryset())
            ids_list = queryset.values_list('id', flat=True)

        # 查询过滤结果集
        queryset = queryset.filter(id__in=ids_list)
        # 策略 暂停，启用
        if is_stop == 1:
            queryset.update(policy_run_status=2)
            thread1 = threading.Thread(target=policy_main)
            thread1.start()
            return Response(f'批量重新执行策略成功', status=status.HTTP_200_OK)
        elif is_stop == 0:
            queryset.update(policy_run_status=3)
            thread1 = threading.Thread(target=policy_main)
            thread1.start()
            return Response(f'批量暂停策略成功', status=status.HTTP_200_OK)
        else:
            pass

        # 策略 删除
        if is_delete == 1:
            queryset.delete()
            thread1 = threading.Thread(target=policy_main)
            thread1.start()
            return Response(f'批量删除策略成功', status=status.HTTP_200_OK)


class FirewalldRuleViewSet(ModelViewSet):
    queryset = FirewalldRulesPolicy.objects.all()
    serializer_class = FirewalldRuleSerializers
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('-pk')

    def create(self, request, *args, **kwargs):
        '''
        level: 规则级别
        protocol: 通信协议
        status: 禁用状态
        in_out: 传入
        port:
        external_addr:
        external_port
        '''
        req_data = request.data
        level = req_data.get('level')
        protocol = req_data.get('protocol')
        state = req_data.get('state')
        in_out = req_data.get('in_out')
        local_port = req_data.get('local_port')
        external_addr = req_data.get('external_addr')
        external_port = req_data.get('external_port')
        data_list = []
        if level == '预配置':
            external_addr = '任何地方'
            if protocol == 'HTTP':
                if in_out == '传入':
                    local_port = '80'
                    external_port = '任何端口'
                else:
                    local_port = '任何端口'
                    external_port = '80'
            elif protocol == 'HTTPS':
                if in_out == '传入':
                    local_port = '443'
                    external_port = '任何端口'
                else:
                    local_port = '任何端口'
                    external_port = '443'
            elif protocol == 'TFP':
                if in_out == '传入':
                    local_port = '21'
                    external_port = '任何端口'
                else:
                    local_port = '任何端口'
                    external_port = '21'
            elif protocol == 'IMAP':
                if in_out == '传入':
                    local_port = '143'
                    external_port = '任何端口'
                else:
                    local_port = '任何端口'
                    external_port = '143'
            elif protocol == 'NFS':
                if in_out == '传入':
                    local_port = '2049'
                    external_port = '任何端口'
                else:
                    local_port = '任何端口'
                    external_port = '2049'
            elif protocol == 'POP3':
                if in_out == '传入':
                    local_port = '110'
                    external_port = '任何端口'
                else:
                    local_port = '任何端口'
                    external_port = '110'
            elif protocol == 'Samba':
                if in_out == '传入':
                    local_port = 'Samba'
                    external_port = '任何端口'
                else:
                    local_port = '任何端口'
                    external_port = 'Samba'
            elif protocol == 'SMTP':
                if in_out == '传入':
                    local_port = '25'
                    external_port = '任何端口'
                else:
                    local_port = '任何端口'
                    external_port = '25'
            elif protocol == 'SSH':
                if in_out == '传入':
                    local_port = '22'
                    external_port = '任何端口'
                else:
                    local_port = '任何端口'
                    external_port = '22'
            elif protocol == 'CUPS':
                if in_out == '传入':
                    local_port = 'CUPS'
                    external_port = '任何端口'
                else:
                    local_port = '任何端口'
                    external_port = 'CUPS'
            else:
                return Response({'msg': '配置有误'}, status=status.HTTP_400_BAD_REQUEST)
            self.get_queryset().filter(local_port=local_port).all().delete()
            data_list.append(
                {'level': level, 'protocol': protocol, 'state': state, 'in_out': in_out,
                 'local_port': local_port, 'ip_type': 'ipv4', 'external_port': external_port,
                 'external_addr': external_addr},
            )
            data_list.append(
                {'level': level, 'protocol': protocol, 'state': state, 'in_out': in_out,
                 'local_port': local_port, 'ip_type': 'ipv6', 'external_port': external_port,
                 'external_addr': external_addr},
            )
            serializer = self.get_serializer(data=data_list, many=True)
            serializer.is_valid(raise_exception=True)
            self.perform_create(serializer)
            headers = self.get_success_headers(serializer.data)
            return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)
        external_addr = '任何地方'
        if protocol not in ('TCP', 'UDP', 'BOTH'):
            return Response({'msg': '配置有误'}, status=status.HTTP_400_BAD_REQUEST)
        if not local_port or not local_port.isdigit():
            return Response({'msg': '配置有误'}, status=status.HTTP_400_BAD_REQUEST)
        if level == '高级':
            external_addr = req_data.get('external_addr')
            if not external_port or not external_port.isdigit() or not external_addr:
                return Response({'msg': '配置有误'}, status=status.HTTP_400_BAD_REQUEST)

        if protocol == 'TCP':
            if local_port in ('80', '443', '21', '143', '2049', '110', '20', '22'):
                if in_out == '传入':
                    self.get_queryset().filter(local_port=local_port).all().delete()
                else:
                    self.get_queryset().filter(external_port=local_port).all().delete()
        if level == '简单':
            if in_out == '传入':
                external_port = '任何端口'
                local_port = local_port
                self.get_queryset().filter(protocol=protocol, local_port=local_port).all().delete()
            else:
                external_port = local_port
                self.get_queryset().filter(protocol=protocol, external_port=local_port).all().delete()
                local_port = '任何端口'
            data_list.append(
                {'level': level, 'protocol': protocol, 'state': state, 'in_out': in_out,
                 'local_port': local_port, 'ip_type': 'ipv6', 'external_port': external_port,
                 'external_addr': external_addr},
            )

        data_list.append(
            {'level': level, 'protocol': protocol, 'state': state, 'in_out': in_out,
             'local_port': local_port, 'ip_type': 'ipv4', 'external_port': external_port,
             'external_addr': external_addr},
        )

        serializer = self.get_serializer(data=data_list, many=True)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    @action(methods=['delete'], detail=False)
    def multiple_delete(self, request, *args, **kwargs):
        delete_id = request.query_params.get('id')
        ids = delete_id.split(',')
        if ids:
            self.get_queryset().filter(id__in=ids).all().delete()
        else:
            self.get_queryset().all().delete()
        return Response('已删除', status=status.HTTP_200_OK)


class WaterMarkViewSet(mixins.RetrieveModelMixin,
                       mixins.UpdateModelMixin,
                       GenericViewSet):
    queryset = WaterMark.objects.all()
    serializer_class = WaterMarkSerializer
    pagination_class = None
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('id',)


class WaterMarkInfoViewSet(ModelViewSet):
    queryset = Devices.objects.exclude(is_domain=0).all()
    serializer_class = WaterMarkDevicePolicySerializer
    pagination_class = MyPagination
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('id',)

    def filter_queryset(self, queryset):
        queryset = super(WaterMarkInfoViewSet, self).filter_queryset(self.queryset)

        if self.request.query_params.get('device_cnname', None):
            params_dict = self.request.query_params.get('device_cnname', None)
            queryset = queryset.filter(device_cnname__icontains=params_dict)

        if self.request.query_params.get('device_ip', None):
            params_dict = self.request.query_params.get('device_ip', None)
            queryset = queryset.filter(device_ip__icontains=params_dict)

        if self.request.query_params.get('org_info_id', None):
            params_dict = self.request.query_params.get('org_info_id', None)
            queryset = queryset.filter(org_info_id=params_dict)

        if self.request.query_params.get('watermark_switch_status', None):
            params_dict = self.request.query_params.get('watermark_switch_status', None)
            if params_dict == str(1):
                watermark_device_policy = WaterMarkDevicePolicy.objects.filter(
                    watermark_switch_status=params_dict).values_list('device_id', flat=True)
                queryset = queryset.filter(id__in=watermark_device_policy)
            else:
                watermark_device_policy = WaterMarkDevicePolicy.objects.filter(
                    watermark_switch_status=1).values_list('device_id', flat=True)
                queryset = queryset.exclude(id__in=watermark_device_policy)

        if self.request.query_params.get('status', None):
            params_dict = self.request.query_params.get('status', None)
            watermark_device_policy = WaterMarkDevicePolicy.objects.filter(
                status=params_dict).values_list('device_id', flat=True)
            queryset = queryset.filter(id__in=watermark_device_policy)

        return queryset

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_operation(self, request, *args, **kwargs):
        """批量操作水印设置，部门操作"""
        is_all = self.request.data.get('is_all')
        org_id_list = self.request.data.get('org_id_list')
        watermark_ids_str = self.request.data.get('ids_list')
        device_queryset = self.filter_queryset(self.get_queryset())

        if is_all == 0:
            if org_id_list:
                device_queryset = device_queryset.filter(org_info_id__in=org_id_list)
            else:
                policy_ids_list = watermark_ids_str.split(',') if watermark_ids_str != '' else []
                device_queryset = device_queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            pass
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        # 水印默认配置， 当取消水印时使用此配置，透明度为0
        watermark_params_dict = {
            "watermark_switch_status": 0,
            "user_defined_text": "0000",  # 固定0000
            "watermark_position": "全屏",
            "watermark_rotate": 30,
            "watermark_font": "黑体",
            "watermark_font_size": 50,
            "watermark_font_color": '7f7f7f',
            "watermark_font_transparency": 00  # 固定00
        }

        if self.request.data.get('watermark_switch_status') == 1:  # 开启水印
            watermark_params_dict = {
                "watermark_switch_status": self.request.data.get('watermark_switch_status'),
                # "require_account_name": self.request.data.get('require_account_name'),
                # "require_device_cnname": self.request.data.get('require_device_cnname'),
                # "require_device_hostname": self.request.data.get('require_device_hostname'),
                # "require_device_ip": self.request.data.get('require_device_ip'),
                # "require_device_mac": self.request.data.get('require_device_mac'),
                # "require_device_current_time": self.request.data.get('require_device_current_time'),
                "user_defined_text": self.request.data.get('user_defined_text'),
                "watermark_position": self.request.data.get('watermark_position'),
                "watermark_rotate": self.request.data.get('watermark_rotate'),
                "watermark_font": self.request.data.get('watermark_font'),
                "watermark_font_size": self.request.data.get('watermark_font_size'),
                "watermark_font_color": self.request.data.get('watermark_font_color'),
                "watermark_font_transparency": self.request.data.get('watermark_font_transparency')
            }

        if not device_queryset:
            return Response("没有符合要求的数据", status=status.HTTP_400_BAD_REQUEST)

        for device in device_queryset.only('id'):
            watermark_device, _ = WaterMarkDevicePolicy.objects.get_or_create(device_id=device.id)
            watermark_device.status = 2
            watermark_device.watermark_switch_status = self.request.data.get('watermark_switch_status')
            # watermark_device.require_account_name = self.request.data.get('require_account_name'),
            # watermark_device.require_device_cnname = self.request.data.get('require_device_cnname'),
            # watermark_device.require_device_hostname = self.request.data.get('require_device_hostname'),
            # watermark_device.require_device_ip = self.request.data.get('require_device_ip'),
            # watermark_device.require_device_mac = self.request.data.get('require_device_mac'),
            # watermark_device.require_device_current_time = self.request.data.get('require_device_current_time'),
            watermark_device.user_defined_text = self.request.data.get('user_defined_text')
            watermark_device.watermark_position = self.request.data.get('watermark_position')
            watermark_device.watermark_rotate = self.request.data.get('watermark_rotate')
            watermark_device.watermark_font = self.request.data.get('watermark_font')
            watermark_device.watermark_font_size = self.request.data.get('watermark_font_size')
            watermark_device.watermark_font_color = self.request.data.get('watermark_font_color')
            watermark_device.watermark_font_transparency = self.request.data.get('watermark_font_transparency')
            watermark_device.save()
        watermark_policy.delay(
            DeviceSerializer(device_queryset, many=True).data,  # celery参数不能是query_set类型
            watermark_params_dict
        )

        return Response({'msg': '水印策略设置已提交'}, status=status.HTTP_200_OK)


class ActivePolicyViewSet(mixins.RetrieveModelMixin,
                          mixins.UpdateModelMixin,
                          mixins.ListModelMixin,
                          GenericViewSet):
    queryset = Devices.objects.exclude(is_domain=0)
    serializer_class = ActivePolicySerializers
    pagination_class = MyPagination
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('id',)

    def filter_queryset(self, queryset):
        queryset = super(ActivePolicyViewSet, self).filter_queryset(self.queryset)

        if self.request.query_params.get('device_cnname', None):
            params_dict = self.request.query_params.get('device_cnname', None)
            queryset = queryset.filter(device_cnname__icontains=params_dict)

        if self.request.query_params.get('device_ip', None):
            params_dict = self.request.query_params.get('device_ip', None)
            queryset = queryset.filter(device_ip__icontains=params_dict)

        if self.request.query_params.get('org_info_id', None):
            params_dict = self.request.query_params.get('org_info_id', None)
            queryset = queryset.filter(org_info_id=params_dict)

        if self.request.query_params.get('status', None):
            params_dict = self.request.query_params.get('status', None)
            ids_list = ActivationDevicePolicy.objects.filter(status=params_dict).values_list('device_id', flat=True)
            queryset = queryset.filter(id__in=ids_list)

        if self.request.query_params.get('active', None):
            params_dict = self.request.query_params.get('active', None)
            ids_list = ActivationDevicePolicy.objects.filter(active=params_dict).values_list('device_id', flat=True)
            queryset = queryset.filter(id__in=ids_list)

        return queryset

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_operation(self, request, *args, **kwargs):
        """批量操作系统设置，部门操作"""
        is_all = self.request.data.get('is_all')
        org_id_list = self.request.data.get('org_id_list')
        activepolicy_ids_str = self.request.data.get('ids_list')
        kms_active = self.request.data.get('active')
        kms_server = self.request.data.get('kms_server')

        # device_id_list = ActivationDevicePolicy.objects.filter(active=1).values_list('device_id', flat=True)

        if is_all == 0:
            device_queryset = self.filter_queryset(self.get_queryset())
            if org_id_list:
                device_queryset = device_queryset.filter(org_info_id__in=org_id_list)
            else:
                policy_ids_list = activepolicy_ids_str.split(',')
                device_queryset = device_queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            device_queryset = Devices.objects.exclude(is_domain=0).all()
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if not device_queryset:
            return Response("没有符合要求的数据", status=status.HTTP_400_BAD_REQUEST)

        # device_queryset = device_queryset.filter(id__in=device_id_list)
        # if not device_queryset:
        #     return Response('没有选中加域的设备', status=status.HTTP_400_BAD_REQUEST)

        for device in device_queryset.only('id'):
            action_device_items, _ = ActivationDevicePolicy.objects.get_or_create(device_id=device.id)
            action_device_items.status = 2
            action_device_items.active = 0
            action_device_items.kms_server = kms_server
            action_device_items.save()

        system_activation_policy.delay(
            DeviceSerializer(device_queryset, many=True).data,  # celery参数不能是query_set类型
            kms_server,
            singal=False
        )

        return Response({'msg': '激活策略设置已提交'}, status=status.HTTP_200_OK)

    @action(methods=['get', 'put'], detail=False, authentication_classes=[TokenAuthentication])
    def default_active(self, request, *args, **kwargs):
        """获取系统的默认策略"""
        domain_auth = DomainLicenseAuth()
        if request.method == 'GET':
            result_dict = {'active': int(domain_auth.kms_active), 'kms_server': domain_auth.kms_server}
            return Response(result_dict, status=status.HTTP_200_OK)
        elif request.method == 'PUT':
            active = self.request.data.get('active')
            kms_server = self.request.data.get('kms_server')
            if not isinstance(active, int):
                return Response('active参数必须为数字', status=status.HTTP_400_BAD_REQUEST)
            if active == 1:
                if not kms_server:
                    return Response('kms_server参数必须项', status=status.HTTP_400_BAD_REQUEST)
                domain_auth.add_kms_active(active)
                domain_auth.add_kms_server(kms_server)
                result_dict = {'active': int(domain_auth.kms_active), 'kms_server': domain_auth.kms_server}
                return Response(result_dict, status=status.HTTP_200_OK)
            if active == 0:
                domain_auth.add_kms_active(active)
                result_dict = {'active': int(domain_auth.kms_active)}
                return Response(result_dict, status=status.HTTP_200_OK)
        else:
            return Response('请求方式错误，get.put', status=status.HTTP_400_BAD_REQUEST)


class DevicePolicyViewSet(GenericViewSet):
    queryset = Devices.objects.exclude(is_domain=0)
    serializer_class = DeviceSerializer
    pagination_class = MyPagination
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('id',)

    def filter_queryset(self, queryset):
        queryset = super(DevicePolicyViewSet, self).filter_queryset(self.queryset)

        if self.request.query_params.get('device_cnname', None):
            params_dict = self.request.query_params.get('device_cnname', None)
            queryset = queryset.filter(device_cnname__icontains=params_dict)

        if self.request.query_params.get('device_ip', None):
            params_dict = self.request.query_params.get('device_ip', None)
            queryset = queryset.filter(device_ip__icontains=params_dict)

        if self.request.query_params.get('org_info_id', None):
            params_dict = self.request.query_params.get('org_info_id', None)
            queryset = queryset.filter(org_info_id=params_dict)

        if self.request.query_params.get('system_update_switch', None):
            params_dict = self.request.query_params.get('system_update_switch', None)
            ids_list = SystemUpdatesPolicy.objects.filter(active=params_dict).values_list('device_id', flat=True)
            queryset = queryset.filter(id__in=ids_list)

        if self.request.query_params.get('software_runs_switch', None):
            params_dict = self.request.query_params.get('software_runs_switch', None)
            ids_list = SoftwareRunsPolicy.objects.filter(software_runs_switch=params_dict).values_list('device_id',
                                                                                                       flat=True)
            queryset = queryset.filter(id__in=ids_list)

        if self.request.query_params.get('screen_saver_switch', None):
            params_dict = self.request.query_params.get('screen_saver_switch', None)
            ids_list = ScreenSaverPolicy.objects.filter(screen_saver_switch=params_dict).values_list('device_id',
                                                                                                     flat=True)
            queryset = queryset.filter(id__in=ids_list)

        if self.request.query_params.get('file_redirection_switch', None):
            params_dict = self.request.query_params.get('file_redirection_switch', None)
            ids_list = FileRedirectionPolicy.objects.filter(file_redirection_switch=params_dict).values_list(
                'device_id', flat=True)
            queryset = queryset.filter(id__in=ids_list)

        return queryset

    @action(methods=['get'], detail=False, url_name='device_system_update',
            serializer_class=DeviceSystemUpdatesPolicySerializers, authentication_classes=[TokenAuthentication])
    def device_system_update(self, request, *args, **kwargs):
        """
        设备更新配置策略
        """
        queryset = self.filter_queryset(self.get_queryset())

        if self.request.query_params.get('status', None):
            params_dict = self.request.query_params.get('status', None)
            if int(params_dict) != 0:
                ids_list = SystemUpdatesPolicy.objects.filter(status=params_dict).values_list('device_id', flat=True)
                queryset = queryset.filter(id__in=ids_list)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @action(methods=['get', 'put'], detail=False, url_name='default_device_system_update',
            serializer_class=SystemUpdatesPolicySerializer, authentication_classes=[TokenAuthentication])
    def default_device_system_update(self, request, *args, **kwargs):
        """
        默认设备更新配置策略
        默认 的数据表中id=1
        """
        default_id = 1
        if request.method == 'GET':
            instance = SystemUpdatesPolicy.objects.get(id=default_id)
            serializer = self.get_serializer(instance)
            return Response(serializer.data)
        elif request.method == 'PUT':
            partial = kwargs.pop('partial', False)
            instance = SystemUpdatesPolicy.objects.get(id=default_id)
            serializer = self.get_serializer(instance, data=request.data, partial=partial)
            serializer.is_valid(raise_exception=True)
            serializer.save()

            if getattr(instance, '_prefetched_objects_cache', None):
                # If 'prefetch_related' has been applied to a queryset, we need to
                # forcibly invalidate the prefetch cache on the instance.
                instance._prefetched_objects_cache = {}
            return Response(serializer.data)
        else:
            return Response(f'方法{request.method}不被允许', status=status.HTTP_400_BAD_REQUEST)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_system_updates(self, request, *args, **kwargs):
        """批量操作系统更新设置，部门操作"""
        is_all = self.request.data.get('is_all')
        org_id_list = self.request.data.get('org_id_list')
        policy_ids_str = self.request.data.get('ids_list')
        system_update_switch = self.request.data.get('system_update_switch')
        update_style = self.request.data.get('update_style')
        check_time_days = self.request.data.get('check_time_days')
        check_time_time = self.request.data.get('check_time_time')
        download_update_method = self.request.data.get('download_update_method')

        if is_all == 0:
            device_queryset = self.filter_queryset(self.get_queryset())
            if self.request.query_params.get('status', None):
                params_dict = self.request.query_params.get('status', None)
                if int(params_dict) != 0:
                    ids_list = SystemUpdatesPolicy.objects.filter(status=params_dict).values_list('device_id',
                                                                                                  flat=True)
                    device_queryset = device_queryset.filter(id__in=ids_list)
            if org_id_list:
                device_queryset = device_queryset.filter(org_info_id__in=org_id_list)
            else:
                policy_ids_list = policy_ids_str.split(',')
                device_queryset = device_queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            device_queryset = Devices.objects.exclude(is_domain=0).all()
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if not device_queryset:
            return Response('没有选中加域的设备', status=status.HTTP_400_BAD_REQUEST)

        for device in device_queryset.only('id'):
            action_device_items, _ = SystemUpdatesPolicy.objects.get_or_create(device_id=device.id)
            action_device_items.status = 2
            action_device_items.system_update_switch = system_update_switch
            action_device_items.update_style = update_style
            action_device_items.check_time_days = check_time_days
            action_device_items.check_time_time = check_time_time
            action_device_items.download_update_method = download_update_method
            action_device_items.save()

        device_system_updates_policy.delay(
            DeviceSerializer(device_queryset, many=True).data,  # celery参数不能是query_set类型
            {
                "system_update_switch": system_update_switch,
                "update_style": update_style,
                "check_time_days": check_time_days,
                "check_time_time": check_time_time,
                "download_update_method": download_update_method
            },
            singal=False
        )

        return Response({'msg': '系统更新设置策略设置已提交'}, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, url_name='device_software_runs',
            serializer_class=DeviceSoftwareRunsPolicySerializers, authentication_classes=[TokenAuthentication])
    def device_software_runs(self, request, *args, **kwargs):
        """
        软件运行策略
        """
        queryset = self.filter_queryset(self.get_queryset())

        if self.request.query_params.get('status', None):
            params_dict = self.request.query_params.get('status', None)
            if int(params_dict) != 0:
                ids_list = SoftwareRunsPolicy.objects.filter(status=params_dict).values_list('device_id', flat=True)
                queryset = queryset.filter(id__in=ids_list)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_software_runs(self, request, *args, **kwargs):
        """批量操作软件运行策略，部门操作"""
        is_all = self.request.data.get('is_all')
        org_id_list = self.request.data.get('org_id_list')
        policy_ids_str = self.request.data.get('ids_list')
        software_runs_switch = self.request.data.get('software_runs_switch')
        software_runs_style = self.request.data.get('software_runs_style')
        runs_white_switch = self.request.data.get('runs_white_switch')
        white_software_list = self.request.data.get('white_software_list')
        runs_black_switch = self.request.data.get('runs_black_switch')
        black_software_list = self.request.data.get('black_software_list')

        if is_all == 0:
            device_queryset = self.filter_queryset(self.get_queryset())
            if self.request.query_params.get('status', None):
                params_dict = self.request.query_params.get('status', None)
                if int(params_dict) != 0:
                    ids_list = SoftwareRunsPolicy.objects.filter(status=params_dict).values_list('device_id',
                                                                                                 flat=True)
                    device_queryset = device_queryset.filter(id__in=ids_list)
            if org_id_list:
                device_queryset = device_queryset.filter(org_info_id__in=org_id_list)
            else:
                policy_ids_list = policy_ids_str.split(',')
                device_queryset = device_queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            device_queryset = Devices.objects.exclude(is_domain=0).all()
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if not device_queryset:
            return Response('没有选中加域的设备', status=status.HTTP_400_BAD_REQUEST)

        for device in device_queryset.only('id'):
            action_device_items, _ = SoftwareRunsPolicy.objects.get_or_create(device_id=device.id)
            action_device_items.status = 2
            action_device_items.software_runs_switch = software_runs_switch
            action_device_items.software_runs_style = software_runs_style
            action_device_items.runs_white_switch = runs_white_switch
            action_device_items.white_software_list = white_software_list
            action_device_items.runs_black_switch = runs_black_switch
            action_device_items.black_software_list = black_software_list
            action_device_items.save()

        device_software_runs_policy.delay(
            DeviceSerializer(device_queryset, many=True).data,  # celery参数不能是query_set类型
            {
                "software_runs_switch": software_runs_switch,
                "software_runs_style": software_runs_style,
                "runs_white_switch": runs_white_switch,
                "white_software_list": white_software_list,
                "runs_black_switch": runs_black_switch,
                "black_software_list": black_software_list
            },
            singal=False
        )

        return Response({'msg': '软件运行策略设置已提交'}, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, url_name='device_screen_saver',
            serializer_class=DeviceScreenSaverPolicySerializers, authentication_classes=[TokenAuthentication])
    def device_screen_saver(self, request, *args, **kwargs):
        """
        屏幕保护策略
        """
        queryset = self.filter_queryset(self.get_queryset())

        if self.request.query_params.get('status', None):
            params_dict = self.request.query_params.get('status', None)
            if int(params_dict) != 0:
                ids_list = ScreenSaverPolicy.objects.filter(status=params_dict).values_list('device_id', flat=True)
                queryset = queryset.filter(id__in=ids_list)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_screen_saver(self, request, *args, **kwargs):
        """批量操作屏幕保护策略，部门操作"""
        is_all = self.request.data.get('is_all')
        org_id_list = self.request.data.get('org_id_list')
        policy_ids_str = self.request.data.get('ids_list')
        screen_saver_switch = self.request.data.get('screen_saver_switch')
        lock_screen_time = self.request.data.get('lock_screen_time')
        screen_saver_time = self.request.data.get('screen_saver_time')
        lock_activate_screensaver = self.request.data.get('lock_activate_screensaver')
        lock_sleep_screensaver = self.request.data.get('lock_sleep_screensaver')
        screensaver_cycle_time = self.request.data.get('screensaver_cycle_time')

        if is_all == 0:
            device_queryset = self.filter_queryset(self.get_queryset())
            if self.request.query_params.get('status', None):
                params_dict = self.request.query_params.get('status', None)
                if int(params_dict) != 0:
                    ids_list = ScreenSaverPolicy.objects.filter(status=params_dict).values_list('device_id',
                                                                                                flat=True)
                    device_queryset = device_queryset.filter(id__in=ids_list)
            if org_id_list:
                device_queryset = device_queryset.filter(org_info_id__in=org_id_list)
            else:
                policy_ids_list = policy_ids_str.split(',')
                device_queryset = device_queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            device_queryset = Devices.objects.exclude(is_domain=0).all()
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if not device_queryset:
            return Response('没有选中加域的设备', status=status.HTTP_400_BAD_REQUEST)

        # for device in device_queryset.only('id'):
        #     action_device_items, _ = ScreenSaverPolicy.objects.get_or_create(device_id=device.id)
        #     action_device_items.status = 2
        #     action_device_items.screen_saver_switch = screen_saver_switch
        #     action_device_items.lock_screen_time = lock_screen_time
        #     action_device_items.screen_saver_time = screen_saver_time
        #     action_device_items.lock_activate_screensaver = lock_activate_screensaver
        #     action_device_items.lock_sleep_screensaver = lock_sleep_screensaver
        #     action_device_items.screensaver_cycle_time = screensaver_cycle_time
        #     action_device_items.save()

        device_screen_saver_policy.delay(
            DeviceSerializer(device_queryset, many=True).data,  # celery参数不能是query_set类型
            {
                "screen_saver_switch": screen_saver_switch,
                "lock_screen_time": lock_screen_time,
                "screen_saver_time": screen_saver_time,
                "lock_activate_screensaver": lock_activate_screensaver,
                "lock_sleep_screensaver": lock_sleep_screensaver,
                "screensaver_cycle_time": screensaver_cycle_time
            },
            singal=False
        )

        return Response({'msg': '屏幕保护策略设置已提交'}, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, url_name='device_file_redirection',
            serializer_class=DeviceFileRedirectionPolicySerializers, authentication_classes=[TokenAuthentication])
    def device_file_redirection(self, request, *args, **kwargs):
        """
        文件重定向策略
        """
        queryset = self.filter_queryset(self.get_queryset())

        if self.request.query_params.get('status', None):
            params_dict = self.request.query_params.get('status', None)
            if int(params_dict) != 0:
                ids_list = FileRedirectionPolicy.objects.filter(status=params_dict).values_list('device_id', flat=True)
                queryset = queryset.filter(id__in=ids_list)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_file_redirection(self, request, *args, **kwargs):
        """批量操作文件重定向策略，部门操作"""
        is_all = self.request.data.get('is_all')
        org_id_list = self.request.data.get('org_id_list')
        policy_ids_str = self.request.data.get('ids_list')
        file_redirection_switch = self.request.data.get('file_redirection_switch')
        file_redirection_list = self.request.data.get('file_redirection_list')

        if is_all == 0:
            device_queryset = self.filter_queryset(self.get_queryset())
            if self.request.query_params.get('status', None):
                params_dict = self.request.query_params.get('status', None)
                if int(params_dict) != 0:
                    ids_list = FileRedirectionPolicy.objects.filter(status=params_dict).values_list('device_id',
                                                                                                    flat=True)
                    device_queryset = device_queryset.filter(id__in=ids_list)
            if org_id_list:
                device_queryset = device_queryset.filter(org_info_id__in=org_id_list)
            else:
                policy_ids_list = policy_ids_str.split(',')
                device_queryset = device_queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            device_queryset = Devices.objects.exclude(is_domain=0).all()
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if not device_queryset:
            return Response('没有选中加域的设备', status=status.HTTP_400_BAD_REQUEST)

        for device in device_queryset.only('id'):
            action_device_items, _ = FileRedirectionPolicy.objects.get_or_create(device_id=device.id)
            action_device_items.status = 2
            action_device_items.file_redirection_switch = file_redirection_switch
            action_device_items.file_redirection_list = file_redirection_list
            action_device_items.save()

        device_file_redirection_policy.delay(
            DeviceSerializer(device_queryset, many=True).data,  # celery参数不能是query_set类型
            {
                "file_redirection_switch": file_redirection_switch,
                "file_redirection_list": file_redirection_list,
            },
            singal=False
        )

        return Response({'msg': '文件重定向策略设置已提交'}, status=status.HTTP_200_OK)


class AccountPolicyViewSet(GenericViewSet):
    queryset = AccountEntity.objects.all()
    serializer_class = AccountEntitySerializer
    pagination_class = MyPagination
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('id',)

    def filter_queryset(self, queryset):
        queryset = super(AccountPolicyViewSet, self).filter_queryset(self.queryset)

        logger.info(f"{self.request.query_params}")

        if self.request.query_params.get('name', None):
            params_dict = self.request.query_params.get('name', None)
            queryset = queryset.filter(name__icontains=params_dict)

        if self.request.query_params.get('org_info_id', None):
            params_dict = self.request.query_params.get('org_info_id', None)
            queryset = queryset.filter(org_info_id=params_dict)

        if self.request.query_params.get('login_switch_status', None):
            params_dict = self.request.query_params.get('login_switch_status', None)
            ids_list = AccountLoginPolicy.objects.filter(login_switch_status=params_dict).values_list('account_id',
                                                                                                      flat=True)
            queryset = queryset.filter(id__in=ids_list)

        if self.request.query_params.get('apply_policy_time', None):
            params_dict = self.request.query_params.get('apply_policy_time', None)
            ids_list = AccountLoginPolicy.objects.filter(apply_policy_time=params_dict).values_list('account_id',
                                                                                                    flat=True)
            queryset = queryset.filter(id__in=ids_list)

        if self.request.query_params.get('theme_switch_status', None):
            params_dict = self.request.query_params.get('theme_switch_status', None)
            ids_list = DesktopThemePolicy.objects.filter(theme_switch_status=params_dict).values_list('account_id',
                                                                                                      flat=True)
            queryset = queryset.filter(id__in=ids_list)

        if self.request.query_params.get('browser_favorites_switch', None):
            params_dict = self.request.query_params.get('browser_favorites_switch', None)
            ids_list = BrowserFavoritesPolicy.objects.filter(browser_favorites_switch=params_dict).values_list(
                'account_id', flat=True)
            queryset = queryset.filter(id__in=ids_list)

        if self.request.query_params.get('trusted_sites_switch', None):
            params_dict = self.request.query_params.get('trusted_sites_switch', None)
            ids_list = TrustedSitesPolicy.objects.filter(trusted_sites_switch=params_dict).values_list('account_id',
                                                                                                       flat=True)
            queryset = queryset.filter(id__in=ids_list)

        return queryset

    @action(methods=['get'], detail=False, url_name='account_login_logout',
            serializer_class=AccountLoginPolicySerializers, authentication_classes=[TokenAuthentication])
    def account_login_logout(self, request, *args, **kwargs):
        """
        获取账号登入登出策略
        """
        queryset = self.filter_queryset(self.get_queryset())

        if self.request.query_params.get('status', None):
            params_dict = self.request.query_params.get('status', None)
            if int(params_dict) != 0:
                ids_list = AccountLoginPolicy.objects.filter(status=params_dict).values_list('account_id', flat=True)
                queryset = queryset.filter(id__in=ids_list)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_login_logout(self, request, *args, **kwargs):
        """批量操作登入登出策略，部门操作"""
        is_all = self.request.data.get('is_all')
        org_id_list = self.request.data.get('org_id_list')
        policy_ids_str = self.request.data.get('ids_list')
        login_switch_status = self.request.data.get('login_switch_status')
        apply_policy_time = self.request.data.get('apply_policy_time')
        file_id = self.request.data.get('file_names')

        if is_all == 0:
            queryset = self.filter_queryset(self.get_queryset())

            if self.request.query_params.get('status', None):
                params_dict = self.request.query_params.get('status', None)
                if int(params_dict) != 0:
                    ids_list = AccountLoginPolicy.objects.filter(status=params_dict).values_list('account_id',
                                                                                                 flat=True)
                    queryset = queryset.filter(id__in=ids_list)

            if org_id_list:
                queryset = queryset.filter(org_info_id__in=org_id_list)
            else:
                policy_ids_list = policy_ids_str.split(',')
                queryset = queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            queryset = AccountEntity.objects.all()
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if not queryset:
            return Response('没有匹配到满足条件的账号', status=status.HTTP_400_BAD_REQUEST)

        no_devices = False
        for account_obj in queryset.only('id'):
            devices_queryset = account_obj.devices.all()
            if not devices_queryset:
                continue
            else:
                account_login_items, _ = AccountLoginPolicy.objects.get_or_create(account_id=account_obj.id)
                account_login_items.status = 2
                account_login_items.file_id = file_id
                account_login_items.login_switch_status = login_switch_status
                account_login_items.apply_policy_time = apply_policy_time
                account_login_items.save()

                account_login_logout_policy.delay(
                    DeviceSerializer(devices_queryset, many=True).data,  # celery参数不能是query_set类型
                    {"login_switch_status": login_switch_status,
                     "file_id": file_id,
                     "apply_policy_time": apply_policy_time,
                     "account_id": account_obj.id},
                    singal=False
                )
                no_devices = True
        if not no_devices:
            return Response('没有匹配到满足条件的设备', status=status.HTTP_400_BAD_REQUEST)
        else:
            return Response({'msg': '脚本策略设置已提交'}, status=status.HTTP_200_OK)

        # @action(methods=['get', 'put'], detail=False,
        #         serializer_class=AccountDefaultPolicySerializer, authentication_classes=[TokenAuthentication])
        # def account_default_policy(self, request, *args, **kwargs):
        #     """获取账号登入登出默认策略"""
        #     queryset = AccountDefaultPolicy.objects.all()
        #     if request.method == 'GET':
        #         instance = queryset.first()
        #         serializer = self.get_serializer(instance)
        #         return Response(serializer.data)
        #     elif request.method == 'PUT':
        #         partial = kwargs.pop('partial', False)
        #         instance = queryset.first()
        #         serializer = self.get_serializer(instance, data=request.data, partial=partial)
        #         serializer.is_valid(raise_exception=True)
        #         serializer.save()
        #
        #         if getattr(instance, '_prefetched_objects_cache', None):
        #             # If 'prefetch_related' has been applied to a queryset, we need to
        #             # forcibly invalidate the prefetch cache on the instance.
        #             instance._prefetched_objects_cache = {}
        #
        #         return Response(serializer.data)
        #     else:
        #         return Response('请求方式错误，get.put', status=status.HTTP_400_BAD_REQUEST)

    @action(methods=['get'], detail=False, url_name='account_desktop_theme',
            serializer_class=AccountDesktopThemePolicySerializers, authentication_classes=[TokenAuthentication])
    def account_desktop_theme(self, request, *args, **kwargs):
        """
        获取账号桌面主题策略
        """
        queryset = self.filter_queryset(self.get_queryset())

        if self.request.query_params.get('status', None):
            params_dict = self.request.query_params.get('status', None)
            if int(params_dict) != 0:
                ids_list = DesktopThemePolicy.objects.filter(status=params_dict).values_list('account_id',
                                                                                             flat=True)
                queryset = queryset.filter(id__in=ids_list)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_desktop_theme(self, request, *args, **kwargs):
        """批量操作桌面主题策略，部门操作"""
        is_all = self.request.data.get('is_all')
        org_id_list = self.request.data.get('org_id_list')
        policy_ids_str = self.request.data.get('ids_list')

        theme_switch_status = self.request.data.get('theme_switch_status')
        style_theme = self.request.data.get('style_theme')
        gtk_theme = self.request.data.get('gtk_theme')
        icon_theme = self.request.data.get('icon_theme')
        cursor_theme = self.request.data.get('cursor_theme')
        is_sunset = self.request.data.get('is_sunset')
        win_decoration_switch = self.request.data.get('win_decoration_switch')
        win_transparency = self.request.data.get('win_transparency')

        if is_all == 0:
            queryset = self.filter_queryset(self.get_queryset())

            if self.request.query_params.get('status', None):
                params_dict = self.request.query_params.get('status', None)
                if int(params_dict) != 0:
                    ids_list = DesktopThemePolicy.objects.filter(status=params_dict).values_list('account_id',
                                                                                                 flat=True)
                    queryset = queryset.filter(id__in=ids_list)

            if org_id_list:
                queryset = queryset.filter(org_info_id__in=org_id_list)
            else:
                policy_ids_list = policy_ids_str.split(',')
                queryset = queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            queryset = AccountEntity.objects.all()
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if not queryset:
            return Response('没有匹配到满足条件的账号', status=status.HTTP_204_NO_CONTENT)

        no_devices = False
        for account_obj in queryset.only('id'):
            devices_queryset = account_obj.devices.all()
            if not devices_queryset:
                continue
            else:
                account_login_items, _ = DesktopThemePolicy.objects.get_or_create(account_id=account_obj.id)
                account_login_items.status = 2
                account_login_items.theme_switch_status = theme_switch_status
                account_login_items.style_theme = style_theme
                account_login_items.gtk_theme = gtk_theme
                account_login_items.icon_theme = icon_theme
                account_login_items.cursor_theme = cursor_theme
                account_login_items.is_sunset = is_sunset
                account_login_items.win_decoration_switch = win_decoration_switch
                account_login_items.win_transparency = win_transparency
                account_login_items.save()

                account_desktop_theme_policy.delay(
                    DeviceSerializer(devices_queryset, many=True).data,  # celery参数不能是query_set类型
                    {"theme_switch_status": self.request.data.get('theme_switch_status'),
                     "style_theme": self.request.data.get('style_theme'),
                     "gtk_theme": self.request.data.get('gtk_theme'),
                     "icon_theme": self.request.data.get('icon_theme'),
                     "cursor_theme": self.request.data.get('cursor_theme'),
                     "is_sunset": self.request.data.get('is_sunset'),
                     "win_decoration_switch": self.request.data.get('win_decoration_switch'),
                     "win_transparency": self.request.data.get('win_transparency'),
                     "account_id": account_obj.id},
                    singal=False
                )
                no_devices = True

        if not no_devices:
            return Response('没有匹配到满足条件的设备', status=status.HTTP_400_BAD_REQUEST)
        else:
            return Response({'msg': '桌面主题策略设置已提交'}, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, url_name='account_trusted_sites',
            serializer_class=AccountTrustedSitesPolicySerializers, authentication_classes=[TokenAuthentication])
    def account_trusted_sites(self, request, *args, **kwargs):
        """
        获取账号桌面主题策略
        """
        queryset = self.filter_queryset(self.get_queryset())

        if self.request.query_params.get('status', None):
            params_dict = self.request.query_params.get('status', None)
            if int(params_dict) != 0:
                ids_list = TrustedSitesPolicy.objects.filter(status=params_dict).values_list('account_id',
                                                                                             flat=True)
                queryset = queryset.filter(id__in=ids_list)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_trusted_sites(self, request, *args, **kwargs):
        """批量操作可信任站点策略，部门操作"""
        is_all = self.request.data.get('is_all')
        org_id_list = self.request.data.get('org_id_list')
        policy_ids_str = self.request.data.get('ids_list')

        trusted_sites_switch = self.request.data.get('trusted_sites_switch')
        trusted_sites_list = self.request.data.get('trusted_sites_list')

        if is_all == 0:
            queryset = self.filter_queryset(self.get_queryset())

            if self.request.query_params.get('status', None):
                params_dict = self.request.query_params.get('status', None)
                if int(params_dict) != 0:
                    ids_list = TrustedSitesPolicy.objects.filter(status=params_dict).values_list('account_id',
                                                                                                 flat=True)
                    queryset = queryset.filter(id__in=ids_list)

            if org_id_list:
                queryset = queryset.filter(org_info_id__in=org_id_list)
            else:
                policy_ids_list = policy_ids_str.split(',')
                queryset = queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            queryset = AccountEntity.objects.all()
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if not queryset:
            return Response('没有匹配到满足条件的账号', status=status.HTTP_204_NO_CONTENT)

        no_devices = False
        for account_obj in queryset.only('id'):
            devices_queryset = account_obj.devices.all()
            if not devices_queryset:
                continue
            else:
                account_login_items, _ = TrustedSitesPolicy.objects.get_or_create(account_id=account_obj.id)
                account_login_items.status = 2
                account_login_items.trusted_sites_switch = trusted_sites_switch
                account_login_items.trusted_sites_list = trusted_sites_list
                account_login_items.save()

                account_trusted_sites_policy.delay(
                    DeviceSerializer(devices_queryset, many=True).data,  # celery参数不能是query_set类型
                    {"trusted_sites_switch": trusted_sites_switch,
                     "trusted_sites_list": trusted_sites_list,
                     "account_id": account_obj.id},
                    singal=False
                )
                no_devices = True

        if not no_devices:
            return Response('没有匹配到满足条件的设备', status=status.HTTP_400_BAD_REQUEST)
        else:
            return Response({'msg': '可信任站点策略设置已提交'}, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, url_name='account_browser_favorites',
            serializer_class=AccountBrowserFavoritesPolicySerializers, authentication_classes=[TokenAuthentication])
    def account_browser_favorites(self, request, *args, **kwargs):
        """
        获取账号游览器收藏夹策略
        """
        queryset = self.filter_queryset(self.get_queryset())

        if self.request.query_params.get('status', None):
            params_dict = self.request.query_params.get('status', None)
            if int(params_dict) != 0:
                ids_list = BrowserFavoritesPolicy.objects.filter(status=params_dict).values_list('account_id',
                                                                                                 flat=True)
                queryset = queryset.filter(id__in=ids_list)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_browser_favorites(self, request, *args, **kwargs):
        """批量操作游览器收藏夹策略，部门操作"""
        is_all = self.request.data.get('is_all')
        org_id_list = self.request.data.get('org_id_list')
        policy_ids_str = self.request.data.get('ids_list')

        browser_favorites_switch = self.request.data.get('browser_favorites_switch')
        browser_favorites_redirection = self.request.data.get('browser_favorites_redirection')

        if is_all == 0:
            queryset = self.filter_queryset(self.get_queryset())

            if self.request.query_params.get('status', None):
                params_dict = self.request.query_params.get('status', None)
                if int(params_dict) != 0:
                    ids_list = BrowserFavoritesPolicy.objects.filter(status=params_dict).values_list('account_id',
                                                                                                     flat=True)
                    queryset = queryset.filter(id__in=ids_list)

            if org_id_list:
                queryset = queryset.filter(org_info_id__in=org_id_list)
            else:
                policy_ids_list = policy_ids_str.split(',')
                queryset = queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            queryset = AccountEntity.objects.all()
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if not queryset:
            return Response('没有匹配到满足条件的账号', status=status.HTTP_204_NO_CONTENT)

        no_devices = False
        for account_obj in queryset.only('id'):
            devices_queryset = account_obj.devices.all()
            if not devices_queryset:
                continue
            else:
                account_login_items, _ = BrowserFavoritesPolicy.objects.get_or_create(account_id=account_obj.id)
                account_login_items.status = 2
                account_login_items.browser_favorites_switch = browser_favorites_switch
                account_login_items.browser_favorites_redirection = browser_favorites_redirection
                account_login_items.save()

                account_browser_favorites_policy.delay(
                    DeviceSerializer(devices_queryset, many=True).data,  # celery参数不能是query_set类型
                    {"browser_favorites_switch": browser_favorites_switch,
                     "browser_favorites_redirection": browser_favorites_redirection,
                     "account_id": account_obj.id},
                    singal=False
                )
                no_devices = True

        if not no_devices:
            return Response('没有匹配到满足条件的设备', status=status.HTTP_400_BAD_REQUEST)
        else:
            return Response({'msg': '游览器收藏夹策略设置已提交'}, status=status.HTTP_200_OK)

from rest_framework.filters import OrderingFilter, SearchFilter
from django_filters.rest_framework import DjangoFilterBackend

class ShellInfoViewSet(ModelViewSet):
    queryset = Shell.objects.all()
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = ShellFilter
    serializer_class = ShellInfoSerializer
    # filterset_fields = '__all__'
    # search_fields = ('name', 'remark',)
    ordering_fields = '__all__'

    def create(self, request, *args, **kwargs):
        if Shell.objects.filter(name=request.data['name']):
            return Response('脚本名称已存在', status=status.HTTP_400_BAD_REQUEST)
        request.data['creator'] = request.user.username
        request.data['updater'] = request.user.username
        return super(ShellInfoViewSet, self).create(request, args, kwargs)

    def update(self, request, *args, **kwargs):
        if Shell.objects.filter(name=request.data['name']).exclude(id=self.get_object().id):
            return Response('脚本名称已存在', status=status.HTTP_400_BAD_REQUEST)
        if 'file_content' in request.data:
            if 'file_path' not in request.data:
                request.data['file_path'] = self.get_object().file_path
            if 'file_path' not in request.data:
                request.data['file_path'] = os.path.join('shell_policy', uuid.uuid4().hex, 'custom_shell.sh')
            shell_upload_path = os.path.join(settings.MEDIA_ROOT, request.data['file_path'])
            if not os.path.exists(os.path.dirname(shell_upload_path)):
                os.makedirs(os.path.dirname(shell_upload_path))
            with open(shell_upload_path, 'w+') as dest:
                dest.write(request.data['file_content'])
        request.data['updater'] = request.user.username
        res = super(ShellInfoViewSet, self).update(request, args, kwargs)
        ShellDevicePolicy.objects.filter(shell_type=0, shell_id=self.get_object().id).update(file_path=request.data['file_path'])
        default_shell_policy = DefaultPolicy().get_default_policy().get('shell_policy')
        try:
            if default_shell_policy:
                default_shell_policy = json.loads(default_shell_policy)
        except:
            pass
        if default_shell_policy and default_shell_policy.get('shell_type') == 0 and default_shell_policy.get('shell_id') == self.get_object().id:
            default_shell_policy['shell_path'] = request.data['file_path']
            DefaultPolicy().set_default_policy('shell_policy', default_shell_policy)
        return res


    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_operation(self, request, *args, **kwargs):
        """批量操作，启用，禁用， 删除"""
        is_all = self.request.data.get('is_all')
        is_delete = self.request.data.get('is_delete')
        ids_str = self.request.data.get('ids_list', '')

        if is_all == 0:
            ids_list = ids_str.split(',')
            queryset = self.queryset.filter(id__in=ids_list)
        elif is_all == 1:
            queryset = self.filter_queryset(self.get_queryset())
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)
        queryset.update(updater=request.user.username)

        if is_delete == 1:
            ShellDevicePolicy.objects.filter(shell_type=0, shell__in=queryset.all()).update(shell_type=1, shell_id=None)
            queryset.delete()

        return Response("修改完成" if is_delete != 1 else "删除完成", status=status.HTTP_200_OK)

class ShellPolicyViewset(mixins.RetrieveModelMixin,
                   mixins.DestroyModelMixin,
                   mixins.ListModelMixin,
                   GenericViewSet):
    queryset = ShellDevicePolicy.objects.all()
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    # filterset_class = ShellDevicePolicyFilter
    serializer_class = ShellPolicySerializer
    filterset_fields = '__all__'
    ordering_fields = '__all__'

    @action(methods=['post'], detail=False, authentication_classes=[TokenAuthentication])
    def upload_shell_policy_file(self, request):
        form = UploadShellFileForm(request.POST, request.FILES)
        if not form.is_valid():
            return Response(json.dumps(form.errors), status = status.HTTP_400_BAD_REQUEST)

        f = request.FILES["file"]
        path = os.path.join('shell_policy', uuid.uuid4().hex, f.name)
        shell_upload_path = os.path.join(settings.MEDIA_ROOT, path)
        if not os.path.exists(os.path.dirname(shell_upload_path)):
            os.makedirs(os.path.dirname(shell_upload_path))
        with open(shell_upload_path, 'wb+') as dest:
            for chunk in f.chunks():
                dest.write(chunk)

        return Response({'name': f.name, 'path': path})

class ShellDevicePolicyViewSet(mixins.RetrieveModelMixin, mixins.ListModelMixin, GenericViewSet):
    queryset = Devices.objects.exclude(is_domain=0).prefetch_related('shell_policy').all()
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = ShellDevicePolicyFilter
    serializer_class = ShellDevicePolicySerializer
    # filterset_fields = '__all__'
    search_fields = ('shell_info__shell_switch', 'device_hostname', 'device_cnname')
    ordering_fields = '__all__'

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_operation(self, request, *args, **kwargs):
        data = self.request.data.copy()
        is_all = data.pop('is_all', 0)
        ids_str = data.pop('ids_list', '')
        org_id = data.pop('org_id', None)
        data = data.get('shell_policy')

        if is_all == 0:
            if ids_str:
                ids_list = ids_str.split(',')
                queryset = self.queryset.filter(pk__in=ids_list)
            elif org_id:
                # ids_list = org_id.split(',')
                queryset = self.queryset.filter(org_info__in=org_id)
        elif is_all == 1:
            if ids_str or org_id:
                return Response("is_all=1时，不能传入'ids_list'或'org_id'", status=status.HTTP_400_BAD_REQUEST)
            queryset = self.filter_queryset(self.get_queryset())
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if not data:
            return Response("shell_policy 参数错误", status=status.HTTP_400_BAD_REQUEST)
        data['updater'] = request.user.username

        devices = queryset.all()
        if not devices:
            return Response('设置未提交， 请检查设备是否未启用 或 部门下是否有设备', status= 400)

        for d in devices:
            data['device_id'] = d.id
            if 'file_content' in data:
                if data['shell_type'] == 0:
                    shell = Shell.objects.get(pk=data['shell_id'])
                    path = shell.file_path
                else:
                    # path = d.shell_policy.file_path
                    # if not path or path == '':
                    path = os.path.join('shell_policy', uuid.uuid4().hex, 'custom_shell.sh')
                data['file_path'] = path
                shell_upload_path = os.path.join(settings.MEDIA_ROOT, path)
                if not os.path.exists(os.path.dirname(shell_upload_path)):
                    os.makedirs(os.path.dirname(shell_upload_path))
                with open(shell_upload_path, 'w+') as dest:
                    dest.write(data['file_content'])
                data.pop('file_content')
            p, created = ShellDevicePolicy.objects.update_or_create(device=d, defaults=data)
            if p.shell_switch == 1:
                shell_policy.delay([DeviceSerializer(d).data], {'file_path': p.file_path})

        return Response("修改完成", status=status.HTTP_200_OK)

class ShellDefaultPolicy(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        default_policy = DefaultPolicy()
        p = default_policy.get_default_policy().get('shell_policy')
        try:
            if p:
                p = json.loads(p)
        except Exception as e:
            logger.warning(e)
            pass
        if not p:
            p = {
                "shell_switch": 0,
                "shell_type": 0
            }
        if 'file_path' not in p:
            p['file_info'] = {}
        else:
            shell_upload_path = os.path.join(settings.MEDIA_ROOT, p['file_path'])
            if not os.path.exists(shell_upload_path):
                p['file_info'] = {}
            try:
                with open(shell_upload_path, 'rb') as f:
                    p['file_info'] = {
                        'content': f.read().decode('utf-8'),
                        'size': os.path.getsize(shell_upload_path),
                        'name': os.path.basename(shell_upload_path)
                    }
            except Exception as e:
                p['file_info'] = {}
        return Response({
            'code': 200,
            'data': p
        })

    def post(self, request):
        data = request.data
        if 'file_content' in data:
            if data['shell_type'] == 0:
                shell = Shell.objects.get(pk=data['shell_id'])
                path = shell.file_path
            else:
                path = os.path.join('shell_policy', uuid.uuid4().hex, 'custom_shell.sh')
            data['file_path'] = path
            shell_upload_path = os.path.join(settings.MEDIA_ROOT, path)
            if not os.path.exists(os.path.dirname(shell_upload_path)):
                os.makedirs(os.path.dirname(shell_upload_path))
            with open(shell_upload_path, 'w+') as dest:
                dest.write(data['file_content'])
            data.pop('file_content')

        if data.get('shell_type', 1) == 0 and data.get('shell_id', None):
            data['file_path'] = Shell.objects.get(id=data.get('shell_id')).file_path

        default_policy = DefaultPolicy()
        default_policy.set_default_policy('shell_policy', json.dumps(data))
        return Response({
            'code': 200,
            'msg': '设置成功'
        })
