import logging
import datetime
import os
import re
import time

import django
import pandas as pd
import pymysql
import salt.config
import salt.utils.event
import json
import websocket

from django.conf import settings

from devices.client_consumers import msg_to_client

logger = logging.getLogger(__name__)
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "nfs_domain_mainserver.settings")
django.setup()

from saltapi.utils import SaltAPI
from devices.models import Devices
from django_redis import get_redis_connection

from django.db import close_old_connections
from multiprocessing import Process
from utils.RedisHelper import RedisHelper
from utils.rpc_client import SaltRpcClient
from devices.utils import get_device, save_device_hardware, system_activation, clear_device_hardware, del_device, \
    del_device_group, save_device_history
from saltapi.singnals import run_wait_policy, run_default_policy, clear_user_policy, clear_wait_policy, \
    clear_device_policy, parse_policy_result, clear_software_installed_infos, clear_device_domain_info, \
    save_policy_jid, save_policy_result, get_pillar, clear_task_infos, clear_device_jid, clear_device_user_active, \
    save_device_domain_info_to_reids, run_local_user_pwd_change_policy
from saltapi.models import SaltJidDevice
from channels.layers import get_channel_layer
from asgiref.sync import async_to_sync

from software.reprepro import include_deb, include_deb_list

from accounts.freeipa import hbacrule_del, sudorule_all_remove_user, sudorule_all_add_user

from browser.redis_worker import BrowserPolicyChannelWorker

from sysios.utils import ISOFactory

DB_NAME = settings.DATABASES['default']['NAME']
DB_USER = settings.DATABASES['default']['USER']
DB_PASSWORD = settings.DATABASES['default']['PASSWORD']
DB_HOST = settings.DATABASES['default']['HOST']
DB_PORT = settings.DATABASES['default']['PORT']

redis_conn = get_redis_connection('redis')


class SaltEvent(object):
    def run(self):
        logger.info("SaltEvent Process started")
        salt = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        for eachevent in salt.get_events():
            # logger.info(f"获取minion返回信息 \r\n{eachevent}")
            close_old_connections()
            if eachevent.get('data') and eachevent.get('data').get('tag') == 'minion_start':
                minion_id = eachevent.get('data').get('id')
                logger.info(f"minion 启动: {minion_id}")
                device = get_device(minion_id)
                if not isinstance(device, bool):
                    if device.is_domain:
                        try:
                            save_device_hardware(device)
                        except Exception as e:
                            logger.info('--- save_device_hardware error ---')
                            logger.info(e)
                        try:
                            system_activation(device)
                        except Exception as e:
                            logger.info('--- system_activation error ---')
                            logger.info(e)
                        try:
                            run_default_policy(device)
                        except Exception as e:
                            logger.info('--- run_default_policy error ---')
                            logger.info(e)
                        try:
                            run_wait_policy(device)
                        except Exception as e:
                            logger.info('--- run_wait_policy error ---')
                            logger.info(e)
                        try:
                            run_local_user_pwd_change_policy(device)
                        except Exception as e:
                            logger.info('--- run_local_user_pwd_change_policy error ---')
                            logger.info(e)
            new_policy_match = re.search('^salt/job/(?P<job_id>[0-9]+)/new', eachevent.get('tag'))
            if new_policy_match:
                # TODO:记录正在执行的 SALT 策略日志
                logger.info(f"记录正在执行的SALT 策略日志 {eachevent}")
            ret_policy_match = re.search('^salt/job/(?P<jid>[0-9]+)/ret/(?P<minion_id>\S+)', eachevent.get('tag'))
            if ret_policy_match:
                # salt返回策略执行结果
                # TODO:记录SALT 策略日志
                logger.info(f"记录SALT 策略结果日志 {eachevent}")
                jid = ret_policy_match.group('jid')
                # minion_id = ret_policy_match.group('minion_id')
                full_ret = eachevent.get('data')
                if not full_ret:
                    salt.jobs_info(jid)
                    # jid_status = salt.job_jid_status(jid)
                    continue
                fun = full_ret.get('fun')
                if fun not in ['state.sls', 'pkg.install', 'pkg.remove', 'cp.get_file', 'service.start',
                               'service.stop', 'cmd.run']: continue
                fun_args = full_ret.get('fun_args')
                if fun_args in [
                    ["ss -nt | grep :4505 | awk '{print $4}' | awk -F: '{print $1}'"],
                    ["dmidecode -t processor | awk '/Version: /{print $2}'"],
                ]: continue
                try:
                    pillar = get_pillar(fun_args[1])
                    policy_no = pillar.get('policy_no') if 'policy_no' in pillar.keys() else None
                except Exception as exp:
                    policy_no = None
                    logger.info(f"fun_args ****** {fun_args},{exp}")

                minion_id = full_ret.get('id')
                jid = full_ret.get('jid')
                ret = full_ret.get('return')
                success = full_ret.get('success', True)
                alter_time = datetime.datetime.strptime(full_ret.get('_stamp'), '%Y-%m-%dT%H:%M:%S.%f')
                device = get_device(minion_id) if minion_id != 'isofactory' else ISOFactory()
                try:
                    SaltJidDevice.objects.add_jid(policy_no, fun, jid, minion_id, json.dumps(ret), success,
                                                  json.dumps(full_ret),
                                                  alter_time, fun_args)
                except Exception as e:
                    logger.info(e)
                if fun == 'state.sls':
                    try:
                        # save_policy_jid(fun_args, jid, device)
                        parse_policy_result(fun_args, jid, device, ret)
                    except Exception as e:
                        logger.exception(e)
                close_old_connections()


class SaltEventWS(object):
    def __init__(self):
        self.uri = f'ws://{os.environ["MASTER_HOST"]}:{os.environ["MASTER_WS_PORT"]}'

    def on_message(self, ws, eachevent):
        # print(ws.recv())
        logger.info(f"Received: {eachevent}")
        eachevent = json.loads(eachevent)
        close_old_connections()
        if eachevent.get('tag') == 'minion_start':
            # minion 启动
            minion = eachevent.get('data')
            minion_id = minion.get('id')
            device = get_device(minion_id)
            if not isinstance(device, bool):
                if device.is_domain:
                    try:
                        save_device_hardware(device)
                    except Exception as e:
                        logger.info('--- save_device_hardware error ---')
                        logger.info(e)
                    try:
                        system_activation(device)
                    except Exception as e:
                        logger.info('--- system_activation error ---')
                        logger.info(e)
                    try:
                        run_default_policy(device)
                    except Exception as e:
                        logger.info('--- run_default_policy error ---')
                        logger.info(e)
                    try:
                        run_wait_policy(device)
                    except Exception as e:
                        logger.info('--- run_wait_policy error ---')
                        logger.info(e)
                    try:
                        run_local_user_pwd_change_policy(device)
                    except Exception as e:
                        logger.info('--- run_local_user_pwd_change_policy error ---')
                        logger.info(e)
        if re.search('^salt/job/(?P<job_id>[0-9]+)/new', eachevent.get('tag')):
            # salt策略正在下发
            pass
        if re.search('^salt/job/(?P<job_id>[0-9]+)/ret/(?P<hostname>\S+)', eachevent.get('tag')):
            # salt返回策略执行结果
            # TODO:记录SALT 策略日志
            logger.info(f"记录SALT 策略日志 {eachevent}")
            full_ret = eachevent.get('data')
            fun = full_ret.get('fun')
            if fun not in ['state.sls', 'pkg.install', 'pkg.remove', 'cp.get_file', 'service.start',
                           'service.stop', 'cmd.run']: return
            fun_args = full_ret.get('fun_args')
            if fun_args in [
                ["ss -nt | grep :4505 | awk '{print $4}' | awk -F: '{print $1}'"],
            ]: return
            try:
                pillar = get_pillar(fun_args[1])
                policy_no = pillar.get('policy_no') if 'policy_no' in pillar.keys() else None
            except Exception as exp:
                policy_no = None
                logger.info(f"fun_args ****** {fun_args},{exp}")

            minion_id = full_ret.get('id')
            jid = full_ret.get('jid')
            ret = full_ret.get('return')
            success = full_ret.get('success', True)
            alter_time = datetime.datetime.strptime(full_ret.get('_stamp'), '%Y-%m-%dT%H:%M:%S.%f')
            device = get_device(minion_id)
            try:
                SaltJidDevice.objects.add_jid(policy_no, fun, jid, minion_id, json.dumps(ret), success,
                                              json.dumps(full_ret),
                                              alter_time, fun_args)
            except Exception as e:
                logger.info(e)
            if fun == 'state.sls':
                try:
                    # save_policy_jid(fun_args, jid, device)
                    parse_policy_result(fun_args, jid, device, ret)
                except Exception as e:
                    logger.info(e)
        close_old_connections()

    def on_error(self, ws, error):
        logger.info(f"Error: {error} {self.uri}")
        time.sleep(5)

    def on_close(self, *args):
        logger.info("### closed ###")

    def on_open(self, ws):
        # 通过salt api接口将 手动加域的 文件下载到web容器内
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        files = ['install-domain-client_1.0.sh', 'install-domain-client_2.0.sh', 'pre_add_domain.sh']
        for file in files:
            salt_api.api_files_download(f'/srv/salt/file_system/{file}', f'{settings.SALTFILESYSTEMPATH}/{file}')
        with open('{0}/pre_add_domain.sh'.format(settings.SALTFILESYSTEMPATH), 'r', encoding='utf-8') as f:
            content = f.read()
        str_replace = False
        result = re.search('SERVER_IP=\$1', content)
        if result:
            str_replace = content.replace('SERVER_IP=$1', 'SERVER_IP={}'.format(os.environ.get("DOMAIN_IP")), 1)
        result = re.search('DOMAIN_NAME=\$2', content)
        if result:
            str_replace = str_replace.replace('DOMAIN_NAME=$2',
                                              'DOMAIN_NAME={}'.format(os.environ.get("DOMAIN_SERVER_FQDN")),
                                              1)
        if str_replace:
            with open('{0}/pre_add_domain.sh'.format(settings.SALTFILESYSTEMPATH), 'w', encoding='utf-8') as f:
                f.write(str_replace)
        logger.info("### salt web opened ###")
        ws.send("websocket client ready")

    def run(self):
        while True:
            try:
                # websocket.enableTrace(True)  # 开启调试输出
                ws = websocket.WebSocketApp(self.uri,
                                            on_message=self.on_message,
                                            on_error=self.on_error,
                                            on_close=self.on_close)
                ws.on_open = self.on_open
                ws.run_forever()
            except Exception as e:
                logger.info(e)
                time.sleep(3)


def SaltEventProcess():
    if os.environ.get('DEPLOY_ARCH') == 'cluster':
        salt_event = SaltEventWS()
    else:
        salt_event = SaltEvent()
    p = Process(target=salt_event.run)
    p.start()
    while True:
        if not p.is_alive():  # 如果子进程挂掉
            logger.info("SaltEvent process  Restarting...")
            p = Process(target=salt_event.run)
            p.start()
        time.sleep(1)


def push(channel, msg):
    channel_layer = get_channel_layer()
    async_to_sync(channel_layer.group_send)(
        channel,
        {"type": "push_message", "message": msg}
    )


def ws_push(channel, data):
    # message_notice = save_message_notice(msg)
    # msg['created_at'] = message_notice.created_at.strftime("%Y-%m-%d %H:%M:%S")
    channel_layer = get_channel_layer()
    async_to_sync(channel_layer.group_send)(
        channel,
        {"type": "send_message", 'data_json': data}
    )


def DefaultChannelWorker(subscribes):
    '''
    recv = {
            'type': 'message',
            'pattern': None,
            'channel': b'default_channel',
            'data': b'{
                'task':'task type'
                'device_ip':ip,
                'success': True,
                'msg':'install freeipa success'
            }'
        }

    '''
    logger.info("DefaultChannelWorker Process started")
    redis_sub = subscribes.subscribe(chan='default_channel')
    while True:
        logger.info('######## chan=default_channel #######')
        recv = redis_sub.parse_response()
        res = subscribes.handle_message(recv)
        data = json.loads(res.get('data').decode('utf-8'))
        logger.info(data)
        pass


def DefaultChannelWorkerProcess(subscribes):
    p = Process(target=DefaultChannelWorker, args=(subscribes,))
    p.start()
    while True:
        if not p.is_alive():  # 如果子进程挂掉
            logger.info("DefaultChannelWorker process  Restarting...")
            p = Process(target=DefaultChannelWorker, args=(subscribes,))
            p.start()
        time.sleep(1)


def FreeipaWorker(subscribes):
    '''
    recv = {
            'type': 'message',
            'pattern': None,
            'channel': b'default_channel',
            'data': b'{
                'task':'task type'
                'device_ip':ip,
                'success': True,
                'msg':'install freeipa success'
            }'
        }

    '''
    logger.info("FreeipaWorker Process started")

    redis_sub = subscribes.subscribe(chan='freeipa')
    while True:
        logger.info('######## chan=freeipa #######')
        recv = redis_sub.parse_response()
        res = subscribes.handle_message(recv)
        data = json.loads(res.get('data').decode('utf-8'))
        logger.info(f'收到的信息策略 {data}')
        close_old_connections()
        device = get_device(data.get('device_hostname'))

        # 客户端自主加域
        if data.get('event_type') == 'client_own_domain':
            if data.get('success') in (True, 'True'):
                device_hostname = data.get('device_hostname')
                device_uuid = data.get('device_hostname')
                device_ip = data.get('device_ip')
                device_cnname = data.get('device_ip')
                device_auto_domain = 1
                device_auto_client = 1
                org_info_id = 1
                is_domain = data.get('is_domain')
                if device:
                    device.device_hostname = device_hostname
                    device.device_uuid = device_uuid
                    device.device_ip = device_ip
                    device.device_cnname = device_cnname
                    device.is_domain = is_domain
                    device.save()
                else:
                    device = Devices.objects.create(
                        device_hostname=device_hostname, device_uuid=device_uuid, device_ip=device_ip,
                        device_cnname=device_cnname, device_auto_domain=device_auto_domain,
                        device_auto_client=device_auto_client,
                        org_info_id=org_info_id, is_domain=is_domain
                    )
                try:
                    save_device_domain_info_to_reids(device)
                    save_device_hardware(device)
                    save_device_history(device, 1)
                except Exception as e:
                    logger.info('--- client_own_domain error ---')
                    logger.info(e)
            push('channel', data)
        # 主动加域
        if data.get('event_type') == 'auto_install_client':
            if data.get('success') in (True, 'True'):
                try:
                    device.is_domain = data.get('is_domain')
                    # device.os = data.get('os')
                    # device.os_release = data.get('os_release')
                    # device.os_arch = data.get('os_arch')
                    device.save()
                    logger.info(f"***********1*********")
                    save_device_hardware(device)
                    logger.info(f"***********2*********")
                    save_device_domain_info_to_reids(device)
                    logger.info(f"***********3*********")
                    save_device_history(device, 1)
                except Exception as e:
                    logger.info('--- auto_install_client error ---')
                    logger.info(e)
            push('channel', data)
        # 加域模式
        if data.get('event_type') == 'install':
            if data.get('success') in (True, 'True'):
                device.is_domain = 1
                device.os = data.get('os')
                device.os_release = data.get('os_release')
                device.os_arch = data.get('os_arch')
                device.save()
                # run_default_policy(device)
                save_device_domain_info_to_reids(device)
                try:
                    save_device_history(device, 1)
                except Exception as e:
                    logger.info('--- save_device_history error ---')
                    logger.info(e)
            else:
                device.device_hostname = None
                device.device_cnname = None
                device.is_domain = 0
                device.bind_account.clear()
                device.delete()
            push('channel', data)
        # 退域
        if data.get('event_type') == 'delete':
            if data.get('success') in (True, 'True'):
                try:
                    save_device_history(device, 0)
                except Exception as e:
                    logger.info('--- save_device_history error ---')
                    logger.info(e)
                try:
                    clear_user_policy(device)
                except Exception as e:
                    logger.info('--- clear_user_policy error ---')
                    logger.info(e)
                try:
                    clear_device_policy(device)
                except Exception as e:
                    logger.info('--- clear_device_policy error ---')
                    logger.info(e)
                try:
                    clear_device_hardware(device)
                except Exception as e:
                    logger.info('--- clear_device_hardware error ---')
                    logger.info(e)
                try:
                    clear_wait_policy(device)
                except Exception as e:
                    logger.info('--- clear_wait_policy error ---')
                    logger.info(e)
                try:
                    clear_software_installed_infos(device)
                except Exception as e:
                    logger.info('--- clear software install log error ---')
                    logger.info(e)
                # try:
                #     clear_task_infos(device)
                # except Exception as e:
                #     logger.info('--- clear task install log error ---')
                #     logger.info(e)
                try:
                    clear_device_user_active(device)
                except Exception as e:
                    logger.info('--- clear device user active log error ---')
                    logger.info(e)
                try:
                    SaltRpcClient().remove_ssh_host(device.device_ip)
                except Exception as e:
                    logger.info('--- clear salt ssh hostname error ---')
                    logger.info(e)
                # try:
                #     clear_device_jid(device)
                # except Exception as e:
                #     logger.info('--- clear salt jid log error ---')
                #     logger.info(e)
                try:
                    # freeipa delete hbac rule
                    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
                    rule_name = 'hbac_' + str(device.device_hostname)
                    hbacrule_del(rule_name)
                    device.bind_account.clear()
                    res = salt_api.delete_key(device.device_hostname)
                    logger.info(f"退域成功删除秘钥， {res}")
                    del_device_group(device.id)
                    clear_device_domain_info(device)
                    # del_device(data.get('device_hostname'))
                except Exception as exp:
                    logger.info('--- clear hbacrule & db data ---')
                    logger.info(exp)
            push('channel', data)
        # 退域
        if data.get('event_type') == 'ws_delete':
            send_msg = {"sender": "SERVER", "sn": "1-1-3", "data": {"exit_domain": 1}}
            redis_channel_name = redis_conn.hget('client_channel_name', device.device_hostname)
            if redis_channel_name:
                redis_channel_name = redis_channel_name.decode('utf-8')
                msg_to_client(1, redis_channel_name, send_msg)
            else:
                # 处理未被发送出去的指令
                device_unsent_value = redis_conn.hget('client_websocket_unsent', device.device_hostname)
                if device_unsent_value:
                    device_unsent_value = device_unsent_value.decode('utf-8')
                    device_unsent_value = json.loads(device_unsent_value)
                    device_unsent_value.append(send_msg)
                    redis_conn.hset('client_websocket_unsent', device.device_hostname,
                                    json.dumps(device_unsent_value, ensure_ascii=False))
                else:
                    redis_conn.hset('client_websocket_unsent', device.device_hostname,
                                    json.dumps([send_msg], ensure_ascii=False))

        conn = get_redis_connection('redis')
        try:
            conn.delete(f'{device.device_ip}_adding_domain')
        except Exception as e:
            logger.info(e)
        close_old_connections()


def FreeipaWorkerProcess(subscribes):
    p = Process(target=FreeipaWorker, args=(subscribes,))
    p.start()
    while True:
        if not p.is_alive():  # 如果子进程挂掉
            logger.info("FreeipaWorker process  Restarting...")
            p = Process(target=FreeipaWorker, args=(subscribes,))
            p.start()
        time.sleep(1)


def WebSocketEvent(subscribes):
    logger.info("WebSocketEvent Process started")
    redis_sub = subscribes.subscribe(chan='policy')
    for recv in redis_sub.listen():
        data = json.loads(recv.get('data').decode('utf-8'))
        logger.info("WebSocketEvent data %s" % data)
        # close_old_connections()
        # try:
        #     save_policy_result(data)
        # except Exception as e:
        #     logger.info(e)
        msg = data.get('msg')
        if not msg:
            continue
        # if not data.get('signal'):
        #     continue
        if data.get('signal') != False:
            push('channel', data)


def WebSocketEventProcess(subscribes):
    p = Process(target=WebSocketEvent, args=(subscribes,))
    p.start()
    while True:
        if not p.is_alive():  # 如果子进程挂掉
            logger.info("WebSocketEvent process  Restarting...")
            p = Process(target=WebSocketEvent, args=(subscribes,))
            p.start()
        time.sleep(1)


def RepreproChannelWorker(subscribes):
    '''
    reprepro channel worker

    '''
    logger.info("RepreproChannelWorker Process started")
    redis_sub = subscribes.subscribe(chan='reprepro_channel')
    conn = get_redis_connection('redis')
    while True:
        if conn.get("reprepro_key"):
            time.sleep(20)
            continue

        # logger.info('########redis chan=reprepro_channel is wating msg #######')
        recv = redis_sub.parse_response()
        res = subscribes.handle_message(recv)
        data = json.loads(res.get('data').decode('utf-8'))
        logger.info(data)
        # full_filename = data.get('full_filename')
        # filename = data.get('filename')
        # codename = data.get('codename')
        # main_package = data.get('main_package')
        # r = include_deb(full_filename, filename, main_package, codename)
        include_deb_list(data)
        conn.set("reprepro_key", 1, 60)
        conn.set("reprepro_time", int(time.time()))
        close_old_connections()


def RepreproChannelWorkerProcess(subscribes):
    p = Process(target=RepreproChannelWorker, args=(subscribes,))
    p.start()
    while True:
        if not p.is_alive():  # 如果子进程挂掉
            logger.info("RepreproChannelWorker process  Restarting...")
            p = Process(target=RepreproChannelWorker, args=(subscribes,))
            p.start()
        time.sleep(1)


def ClientWebSocketEvent(subscribes):
    def push(channel, msg):
        channel_layer = get_channel_layer()
        async_to_sync(channel_layer.group_send)(
            channel,
            {"type": "push_message", "message": msg}
        )

    redis_sub = subscribes.subscribe(chan='policy')
    for recv in redis_sub.listen():
        logger.info(recv)
        data = json.loads(recv.get('data').decode('utf-8'))
        push(data.get('device_uuid'), data)


def close_db_old_connections():
    while True:
        close_old_connections()
        time.sleep(3)


if __name__ == '__main__':
    '''
    RedisHelper:
        channel:
            - default_channel  默认消息队列
            - install_freeipa  安装freeipa消息队列
    '''
    logger.info('=========salt event start============')
    redis_helper = RedisHelper()

    default_channel_worker_p = Process(target=DefaultChannelWorkerProcess, args=(redis_helper,))
    default_channel_worker_p.start()

    freeipa_channel_worker_p = Process(target=FreeipaWorkerProcess, args=(redis_helper,))
    freeipa_channel_worker_p.start()

    websocket_channel_worker_p = Process(target=WebSocketEventProcess, args=(redis_helper,))
    websocket_channel_worker_p.start()

    # reprepro worker channel
    reprepro_channel_worker_p = Process(target=RepreproChannelWorkerProcess, args=(redis_helper,))
    reprepro_channel_worker_p.start()

    # reprepro worker channel
    browserPolicyChannelWorker = Process(target=BrowserPolicyChannelWorker, args=(redis_helper,))
    browserPolicyChannelWorker.daemon = True
    browserPolicyChannelWorker.start()

    close_db_p = Process(target=close_db_old_connections)
    close_db_p.daemon = True
    close_db_p.start()

    event_p = Process(target=SaltEventProcess)
    event_p.start()

    default_channel_worker_p.join()
    freeipa_channel_worker_p.join()
    websocket_channel_worker_p.join()

    event_p.join()
    logger.info('=========salt event end============')
