try:
    from apt.debfile import DebPackage
    apt_import = 'python-apt'
except ImportError:
    from pydpkg import Dpkg as DebPackage
    apt_import = 'pydpkg'
import logging
import os

# DebPackage = ""

logger = logging.getLogger(__name__)


def get_file_size_in_mb(file_path):
    try:
        # 获取文件大小（以字节为单位）
        size_bytes = os.path.getsize(file_path)
        # 转换为兆字节并四舍五入保留一位小数
        size_mb = round(size_bytes / (1000 * 1000), 1)
        return f"{size_mb:.1f} MB"
    except Exception as e:
        logger.info(f"Error: {e}")
        return None

# 读取deb包到版本 包名称 信息
def getDebInfo(deb):
    try:
        deb_info = DebPackage(deb)
    except Exception as e:
        logger.info(str(e))
        return None

    logger.info('#################deb Package info#########################')
    if apt_import == 'python-apt':
        logger.info(deb_info._sections.get('Package'))
        logger.info(deb_info._sections.get('Version'))
        logger.info(deb_info._sections.get('Section'))
        res_package = deb_info._sections.get('Package')
        res_version = deb_info._sections.get('Version')
        res_section = deb_info._sections.get('Section')
        installed_size = deb_info._sections.get('Installed-Size')
        Maintainer = deb_info._sections.get('Maintainer')
        Priority = deb_info._sections.get('Priority')
        Homepage = deb_info._sections.get('Homepage')
        Description = deb_info._sections.get('Description')
        res_architecture = deb_info._sections.get('Architecture')
    else:
        logger.info(deb_info.get('Package'))
        logger.info(deb_info.get('Version'))
        logger.info(deb_info.get('Section'))
        res_package = deb_info.get('Package')
        res_version = deb_info.get('Version')
        res_section = deb_info.get('Section')
        installed_size = deb_info.get('Installed-Size')
        Maintainer = deb_info.get('Maintainer')
        Priority = deb_info.get('Priority')
        Homepage = deb_info.get('Homepage')
        Description = deb_info.get('Description')
        res_architecture = deb_info.get('Architecture')
    logger.info(f"Package:{res_package};Version:{res_version};Section:{res_section}")
    if res_architecture in ('x86_64', 'amd64'):
        res_architecture = 'X86'
    elif res_architecture in ('aarch64', 'arm64'):
        res_architecture = 'ARM'
    else:
        pass
    return {
        "package": res_package,
        "version": res_version,
        'section': res_section,
        'architecture': res_architecture,
        'installed_size': installed_size,
        'Maintainer': Maintainer,
        'Priority': Priority,
        'Homepage': Homepage,
        'Description': Description,
        'deb_size': get_file_size_in_mb(deb)
    }

# 给aptly软件源头做架构字符串处理
# 读取deb包中版本 包名称 信息
def getDebInfoForAptly(deb):
    try:
        deb_info = DebPackage(deb)
    except Exception as e:
        logger.info(str(e))
        return None

    logger.info('#################deb Package info#########################')
    logger.info(deb_info._sections.get('Package'))
    logger.info(deb_info._sections.get('Version'))
    logger.info(deb_info._sections.get('Section'))
    Architecture = deb_info._sections.get('Architecture')
    if Architecture in ('x86_64', 'amd64'):
        Architecture = 'X86'
    elif Architecture in ('aarch64', 'arm64'):
        Architecture = 'ARM'
    else:
        pass
    return {
        "package": deb_info._sections.get('Package'),
        "version": deb_info._sections.get('Version'),
        'section': deb_info._sections.get('Section'),
        'architecture': Architecture,
        'installed_size': deb_info._sections.get('Installed-Size'),
        'Maintainer': deb_info._sections.get('Maintainer'),
        'Priority': deb_info._sections.get('Priority'),
        'Homepage': deb_info._sections.get('Homepage'),
        'Description': deb_info._sections.get('Description'),
    }

def getDebSection(deb):
    Section = ""
    try:
        deb_info = DebPackage(deb)
        Section = deb_info._sections['Section']
    except Exception as e:
        logger.info(str(e))
        return ""

    return Section

