from django.db import models
import datetime


class SoftwareType(models.Model):
    delete = (
        (0, 'N'), (1, 'Y')
    )
    # 0 为平台类型，1 为自定义类型
    type_choice = (
        (0, 'platform'), (1, 'custom')
    )
    name = models.CharField(max_length=128, verbose_name='类型名称')
    type = models.IntegerField(verbose_name='软件类型名称类别', choices=type_choice, default=1, blank=True)
    is_delete = models.IntegerField(verbose_name='是否刪除', choices=delete, default=0, blank=True)

    class Meta:
        db_table = 'software_type'
        verbose_name = '软件类型'
        verbose_name_plural = verbose_name

    def __str__(self):
        return self.name


class SoftwareOS(models.Model):
    delete = (
        (0, 'N'), (1, 'Y')
    )
    # 0 为平台类型，1 为自定义类型
    type_choice = (
        (0, 'platform'), (1, 'custom')
    )
    name = models.CharField(max_length=128, verbose_name='系统名称')
    type = models.IntegerField(verbose_name='类型', choices=type_choice, default=1, blank=True)
    is_delete = models.IntegerField(verbose_name='是否刪除', choices=delete, default=0, blank=True)

    class Meta:
        db_table = 'software_os'
        verbose_name = '软件适配系统'
        verbose_name_plural = verbose_name

    def __str__(self):
        return self.name


class SoftwareArch(models.Model):
    delete = (
        (0, 'N'), (1, 'Y')
    )
    # 0 为平台类型，1 为自定义类型
    type_choice = (
        (0, 'platform'), (1, 'custom')
    )
    name = models.CharField(max_length=128, verbose_name='架构名称')
    type = models.IntegerField(verbose_name='类型', choices=type_choice, default=1, blank=True)
    is_delete = models.IntegerField(verbose_name='是否刪除', choices=delete, default=0, blank=True)

    class Meta:
        db_table = 'software_arch'
        verbose_name = '软件架构'
        verbose_name_plural = verbose_name

    def __str__(self):
        return self.name


class SoftwareSource(models.Model):
    delete = (
        (0, 'N'), (1, 'Y')
    )
    # 0 为平台源，1 为自定义源
    type_choice = (
        (0, 'platform'), (1, 'custom')
    )
    # 0 为启用，1 为禁用
    status_choice = (
        (0, 'N'), (1, 'Y')
    )
    # 0 为链接成功，1 为链接失败
    link_status_choice = (
        (0, 'N'), (1, 'Y')
    )
    name = models.CharField(max_length=128, verbose_name='系统名称')
    # 软件源同步地址为获取数据同步的地址 http://172.30.98.241:8080, 软件源地址 172.30.98.241 软件源端口 80
    address = models.CharField(max_length=128, verbose_name='软件源同步地址', null=True, blank=True)
    domain_port = models.CharField(max_length=128, verbose_name='软件源端口', null=True, blank=True)
    source_port = models.CharField(max_length=128, verbose_name='源端口', null=True, blank=True)
    type = models.IntegerField(verbose_name='软件源类型', choices=type_choice, default=0, blank=True)
    status = models.IntegerField(verbose_name='状态', choices=status_choice, default=0, blank=True)
    remarks = models.CharField(verbose_name='备注', max_length=255, null=True, blank=True)
    link_status = models.IntegerField(verbose_name='链接状态', choices=link_status_choice, default=0, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_update = models.DateTimeField(verbose_name='最后修改时间', auto_now=True, null=True, blank=True)
    is_delete = models.IntegerField(verbose_name='是否刪除', choices=delete, default=0, blank=True)

    class Meta:
        db_table = 'software_source'
        verbose_name = '软件源'
        verbose_name_plural = verbose_name

    def __str__(self):
        return self.name


# 软件基础信息表
class SoftwareInfoModel(models.Model):
    is_default = (
        (0, 'N'), (1, 'Y')
    )
    delete_choice = (
        (0, 'N'), (1, 'Y')
    )
    name = models.CharField(max_length=128, verbose_name='软件名称')
    main_package = models.CharField(verbose_name='软件主包及版本号', max_length=255, null=False, blank=True)
    main_version = models.CharField(verbose_name='软件主包及版本号', max_length=255, null=False, blank=True)
    custome_filename = models.CharField(verbose_name='用户上传deb文件名称', max_length=255, null=False, blank=True)
    description = models.CharField(verbose_name='软件描述信息', max_length=4096, null=True, blank=True)
    remarks = models.CharField(verbose_name='备注', max_length=255, null=True, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', blank=True)
    last_update = models.DateTimeField(verbose_name='最后修改时间', auto_now=True, null=True, blank=True)
    is_default_install = models.IntegerField(verbose_name='是否默认安装', choices=is_default, default=0, null=False,
                                             blank=True)
    software_type = models.ForeignKey(verbose_name='软件类型', to=SoftwareType, to_field='id', null=True,
                                      on_delete=models.SET_NULL)
    os_id = models.CharField(verbose_name='适配系统', max_length=255, null=False, blank=True)
    arch_id = models.CharField(verbose_name='软件架构', max_length=255, null=False, blank=True)
    os_name = models.CharField(verbose_name='适配系统名称', max_length=255, null=False, blank=True)
    arch_name = models.CharField(verbose_name='软件架构名称', max_length=255, null=False, blank=True)
    source = models.ForeignKey(verbose_name='软件源', to=SoftwareSource, to_field='id', null=True,
                               on_delete=models.SET_NULL)
    is_delete = models.IntegerField(verbose_name='是否刪除', choices=delete_choice, default=0, blank=True)
    deb_size = models.CharField(verbose_name='软件包大小', max_length=255, null=False, blank=True)

    class Meta:
        db_table = 'software_info'
        verbose_name = '软件信息列表'
        verbose_name_plural = '软件信息列表'
        ordering = ('created_at',)

    def __str__(self):
        return self.name


# 软件安装情况表
class SoftwareInstalledModel(models.Model):
    is_success = (
        (0, 'N'), (1, 'Y')
    )
    is_histroy = (
        (0, 'N'), (1, 'Y')
    )
    # 安装软件状态
    STATUS_RUNNING = 0
    STATUS_INSTALLED = 1
    STATUS_FAILED = 2
    STATUS_REMOVED = 3
    installed_status_choices = (
        (STATUS_RUNNING, 'running'), (STATUS_INSTALLED, 'installed'), (STATUS_FAILED, 'failed'),
        (STATUS_REMOVED, 'removed')
    )

    name = models.CharField(max_length=128, verbose_name='软件名称')
    main_package = models.CharField(verbose_name='软件主包及版本号', max_length=255, null=False, blank=True)
    main_version = models.CharField(verbose_name='软件主包及版本号', max_length=255, null=False, blank=True)
    software_info = models.ForeignKey(verbose_name='软件信息', to='software.SoftwareInfoModel',
                                      on_delete=models.SET_NULL, db_constraint=False, null=True, blank=True)
    device_info = models.ForeignKey(verbose_name='终端设备信息', to='devices.Devices', on_delete=models.SET_NULL,
                                    db_constraint=False, null=True, blank=True)
    remarks = models.CharField(verbose_name='备注', max_length=255, null=True, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_update = models.DateTimeField(verbose_name='最后修改时间', auto_now=True, null=True, blank=True)
    is_success_install = models.IntegerField(verbose_name='是否安装成功', choices=is_success, default=0, null=False,
                                             blank=True)
    is_histroy_install = models.IntegerField(verbose_name='历史记录标识', choices=is_histroy, default=0, null=False,
                                             blank=True)
    installed_status = models.IntegerField(verbose_name='软件安装状态', choices=installed_status_choices, default=0,
                                           null=False, blank=True)
    task_id = models.IntegerField(verbose_name='任务ID', default=0, null=False, blank=True)

    class Meta:
        db_table = 'software_installed'
        verbose_name = '软件安装情况表'
        verbose_name_plural = '软件安装情况表'
        ordering = ('created_at',)

    def __str__(self):
        return self.name


police_range_type_choices = (
    (0, '按部门'), (1, '按终端'), (2, '按 IP 网段'), (3, '按人员'), (4, '按终端组'), (5, '按用户组')
)
police_status_choices = (
    (1, '启用'), (0, '禁用')
)


def _default_get_serial_number():
    date = datetime.datetime.today()
    # 转年月日时分秒
    return date.strftime("%Y%m%d%H%M%S")
    # return date.strftime("%Y%m%d%H%M%S%f")


class SoftwarePolicy(models.Model):
    name = models.CharField(verbose_name='软件策略名称', max_length=255, blank=True, null=True)
    serial_number = models.CharField(verbose_name='软件策略编号', max_length=32, default=_default_get_serial_number,
                                     blank=True, null=True)
    policy_type = models.IntegerField(verbose_name='软件策略的类型,0|终端策略；1|用户策略', default=0, blank=False)
    is_enforced = models.IntegerField(verbose_name='软件策略是否强制,0|非强制；1|强制', default=0, blank=False)
    effective_date = models.DateTimeField(verbose_name='策略开始生效时间', default=None, blank=True, null=True)
    expiration_date = models.DateTimeField(verbose_name='策略开始失效时间', default=None, blank=True, null=True)
    retry_count = models.IntegerField(verbose_name='软件策略重试次数', default=None, blank=True, null=True)
    is_notify_user = models.IntegerField(verbose_name='软件策略是否通知用户,0|不通知；1|通知', default=0, blank=False)
    is_user_cancel = models.IntegerField(verbose_name='软件策略是否允许用户取消,0|不允许；1|允许', default=0,
                                         blank=False)
    range_type = models.IntegerField(verbose_name='软件策略执行范围的类型', choices=police_range_type_choices,
                                     default=0, blank=False)
    range_value = models.TextField(verbose_name='软件策略执行范围的内容', blank=True, null=True)
    exception_switch_type = models.SmallIntegerField(verbose_name='软件策略 例外 开关  0|否；1|是', default=0)
    exception_range_type = models.IntegerField(verbose_name='软件策略 例外执行范围的类型',
                                               choices=police_range_type_choices,
                                               default=1, blank=False)
    exception_range_value = models.TextField(verbose_name='软件策略 例外执行范围的内容', max_length=255, blank=True,
                                             null=True)
    apply_range_value = models.TextField(verbose_name='这条软件策略实际应用执行范围的内容', blank=True, null=True)
    execution_time = models.DateTimeField(verbose_name='策略开始参与计算时间', default=None, blank=True, null=True)
    status = models.IntegerField(verbose_name='软件策略状态', choices=police_status_choices, default=0, blank=False)
    software_ids = models.CharField(verbose_name='软件信息ID的字符串 逗号分割', max_length=255, blank=True, null=True)
    software_black_ids = models.CharField(verbose_name='软件黑名单信息ID的字符串 逗号分割', max_length=255, blank=True,
                                          null=True)
    creator = models.CharField(verbose_name='软件信息创建人', max_length=64, blank=True, null=True)
    update_time = models.DateTimeField(verbose_name='更新时间', auto_now=True, blank=True, null=True)
    remark = models.TextField(verbose_name='备注信息', blank=True, null=True)

    class Meta:
        db_table = 'software_policy'
        verbose_name = '软件策略信息'
        verbose_name_plural = verbose_name
        ordering = ('-update_time',)


class SoftwareUserPolicy(models.Model):
    serial_number = models.CharField(verbose_name='软件信息生成时的编号', max_length=32, blank=True, null=True)
    account_id = models.IntegerField(verbose_name='用户ID', null=True, blank=True)
    account_name = models.CharField(verbose_name='用户账号', max_length=255, unique=True, blank=True)
    retry_count = models.IntegerField(verbose_name='软件策略重试次数', default=None, blank=True, null=True)
    is_notify_user = models.IntegerField(verbose_name='软件策略是否通知用户,0|不通知；1|通知', default=0, blank=False)
    is_user_cancel = models.IntegerField(verbose_name='软件策略是否允许用户取消,0|不允许；1|允许', default=0,
                                         blank=False)
    software_ids = models.CharField(verbose_name='软件信息ID的字符串 逗号分割', max_length=255, blank=True, null=True)
    software_ids_names = models.TextField(verbose_name='软件包名称和版本', null=True, blank=True)
    software_black_ids = models.CharField(verbose_name='软件黑名单信息ID的字符串 逗号分割', max_length=255, blank=True,
                                          null=True)
    software_black_ids_names = models.TextField(verbose_name='软件包黑名单名称和版本', null=True, blank=True)
    update_time = models.DateTimeField(verbose_name='更新时间', auto_now=True, blank=True, null=True)

    class Meta:
        db_table = 'software_user_policy'
        verbose_name = '用户软件信息'
        ordering = ('-update_time',)


class SoftwareDevicePolicy(models.Model):
    serial_number = models.CharField(verbose_name='软件信息生成时的编号', max_length=32, blank=True, null=True)
    device_id = models.IntegerField(verbose_name='用户ID', null=True, blank=True)
    device_name = models.CharField(verbose_name='用户账号', max_length=255, unique=True, blank=True)
    retry_count = models.IntegerField(verbose_name='软件策略重试次数', default=None, blank=True, null=True)
    is_notify_user = models.IntegerField(verbose_name='软件策略是否通知用户,0|不通知；1|通知', default=0, blank=False)
    is_user_cancel = models.IntegerField(verbose_name='软件策略是否允许用户取消,0|不允许；1|允许', default=0,
                                         blank=False)
    software_ids = models.CharField(verbose_name='软件信息ID的字符串 逗号分割', max_length=255, blank=True, null=True)
    software_ids_names = models.TextField(verbose_name='软件包名称和版本', null=True, blank=True)
    software_black_ids = models.CharField(verbose_name='软件黑名单信息ID的字符串 逗号分割', max_length=255, blank=True,
                                          null=True)
    software_black_ids_names = models.TextField(verbose_name='软件包黑名单名称和版本', null=True, blank=True)
    update_time = models.DateTimeField(verbose_name='更新时间', auto_now=True, blank=True, null=True)

    class Meta:
        db_table = 'software_device_policy'
        verbose_name = '设备软件信息'
        ordering = ('-update_time',)

"""
dict_string = {
    'sender': 'SERVER',
    'sn': '9-2-1',
    'data': {
        'taskid': '888',
        'task_name': '软件策略',
        'user_software': {
            'user_account': '用户账号',
            'user_serial_number': '用户软件策略序列号',
            'is_notify_user': '软件安装是否通知用户,0|不通知；1|通知',
            'is_user_cancel': '软件安装是否允许用户取消,0|不允许；1|允许',
            'software_debpackage': [{'name': '*****', 'version': '***'}, {'name': '*****', 'version': '***'}],    # 软件安装
            'software_black_debpackage': [{'name': '*****', 'version': '***'}, {'name': '*****', 'version': '***'}], # 软件黑名单
        },
        'device_software': {
            'device_hostname': '终端主机名',
            'device_serial_number': '终端软件策略序列号',
            'is_notify_user': '软件安装是否通知用户,0|不通知；1|通知',
            'is_user_cancel': '软件安装是否允许用户取消,0|不允许；1|允许',
            'software_debpackage': [{'name': '*****', 'version': '***'}, {'name': '*****', 'version': '***'}],
            'software_black_debpackage': [{'name': '*****', 'version': '***'}, {'name': '*****', 'version': '***'}],
        }
    }}

"""