import os
import base64
import logging
import time

from django.conf import settings
from software.deb_util import getDebSection
from saltapi.utils import DefaultPolicy

from utils.rpc_client import RepreproRpcClient
logger = logging.getLogger(__name__)


# 添加包到仓库
def include_deb(full_filename, filename, main_package, codename="stretch"):
    default_policy = DefaultPolicy()
    # scp -P 2322 docker*.deb debian@172.30.86.226:
    cp_cmd = 'scp -P ' + settings.REPREPRO_SSH_PORT + ' ' + full_filename + ' root@' + settings.HOSTNAME + ':' + '/tmp/'
    logger.info(cp_cmd)
    r = os.popen(cp_cmd)
    # md = RepreproRpcClient().send_file(full_filename, filename)
    # logger.info(md)
    Section = getDebSection(full_filename)
    sign_key_switch = default_policy.get_default_policy().get('sign_key_deb', None)
    if sign_key_switch == '1':
        sign_keyid = base64.b64decode(settings.DEB_SIGN_KEYID.encode('utf-8')).decode('utf-8')
        sign_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "debsigs --sign=origin -k {sign_keyid} '.format(
            sign_keyid=sign_keyid) + '/tmp/' + filename + '"'
        os.popen(sign_cmd)
    if codename == "stretch":
        if Section:
            includedeb_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/debian includedeb stretch ' + '/tmp/' + filename + '"'
            # ssh -p 2322 debian@172.30.86.226 "reprepro -b /var/www/repos/apt/debian includedeb stretch *.deb"
        else:
            includedeb_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/debian -S unknown includedeb stretch ' + '/tmp/' + filename + '"'
        logger.info(includedeb_cmd)
        r = os.popen(includedeb_cmd)

    else:
        if Section:
            includedeb_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/{codename} includedeb {codename} '.format(
                codename=codename) + '/tmp/' + filename + '"'
            # ssh -p 2322 debian@172.30.86.226 "reprepro -b /var/www/repos/apt/debian includedeb stretch *.deb"
        else:
            includedeb_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/{codename} -S unknown includedeb {codename} '.format(
                codename=codename) + '/tmp/' + filename + '"'
        logger.info(includedeb_cmd)
        r = os.popen(includedeb_cmd)

    if main_package == 'nfs-domain-client':
        includedeb_nfs4_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/nfs4 includedeb nfs4 ' + '/tmp/' + filename + '"'
        os.popen(includedeb_nfs4_cmd)
        includedeb_tiger_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/tiger includedeb tiger ' + '/tmp/' + filename + '"'
        os.popen(includedeb_tiger_cmd)
    # Exporting indices...
    # Skipping inclusion of 'apt' '2.2.4' in 'stretch|main|amd64', as it has already '2.2.4'.


def include_deb_list(data):
    """

    :param data:  data_list {'event_type': 'includedeb', 'msg': 'include deb to reprepro', 'data': [{'codename': 'nfs4', 'custome_filename': 'hotpot7_2.21.3-p-scb2.4-release_amd64.deb', 'full_filename': '/usr/share/debs/hotpot7_2.21.3-p-scb2.4-release_amd64.deb', 'Section': 'otherosfs', 'main_package': 'hotpot7'}, {'codename': 'x86_64', 'custome_filename': 'hotpot7_2.21.3-p-scb2.4-release_amd64.deb', 'full_filename': '/usr/share/debs/hotpot7_2.21.3-p-scb2.4-release_amd64.deb', 'Section': 'otherosfs', 'main_package': 'hotpot7'}]}


    :return:
    """
    default_policy = DefaultPolicy()
    data_list = data.get("data")
    logger.info("include_deb_list data_list %s" % data_list)
    codename_list = [x['codename'] for x in data_list]

    code_file_name_dict = {}
    section_code_file_name_dict = {}
    for codename in codename_list:
        code_file_name_dict[codename] = []
    for codename in codename_list:
        section_code_file_name_dict[codename] = []

    for data in data_list:
        full_filename = data.get('full_filename')
        filename = data.get('custome_filename')
        codename = data.get('codename')
        main_package = data.get('main_package')
        Section = data.get('Section')

        cp_cmd = 'scp -P ' + settings.REPREPRO_SSH_PORT + ' ' + full_filename + ' root@' + settings.HOSTNAME + ':' + '/tmp/'
        logger.info(cp_cmd)
        r = os.popen(cp_cmd)
        # md = RepreproRpcClient().send_file(full_filename, filename)
        # logger.info(md)
        # Section = getDebSection(full_filename)

        sign_key_switch = default_policy.get_default_policy().get('sign_key_deb', None)
        if sign_key_switch == '1':
            sign_keyid = base64.b64decode(settings.DEB_SIGN_KEYID.encode('utf-8')).decode('utf-8')
            sign_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "debsigs --sign=origin -k {sign_keyid} '.format(
                sign_keyid=sign_keyid) + '/tmp/' + filename + '"'
            os.popen(sign_cmd)

        if main_package == 'nfs-domain-client':
            includedeb_nfs4_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/nfs4 includedeb nfs4 ' + '/tmp/' + filename + '"'
            os.popen(includedeb_nfs4_cmd)
            includedeb_tiger_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/tiger includedeb tiger ' + '/tmp/' + filename + '"'
            os.popen(includedeb_tiger_cmd)

        if Section:
            section_code_file_name_dict[codename].append(filename)
        else:
            code_file_name_dict[codename].append(filename)
    time.sleep(3)

    for codename, file_name_list in code_file_name_dict.items():
        if file_name_list:
            file_name_list = list(set(file_name_list))
            file_name_str = " ".join(file_name_list)
            if codename == "stretch":
                includedeb_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/debian -S unknown includedeb stretch ' + '/tmp/' + file_name_str + '"'
            else:
                includedeb_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/{codename} -S unknown includedeb {codename} '.format(
                    codename=codename) + '/tmp/' + file_name_str + '"'
            logger.info(includedeb_cmd)
            r = os.popen(includedeb_cmd)
            # 暂停五秒，兼容reprepro 吃包时数据锁
            time.sleep(5)

    for codename, file_name_list in section_code_file_name_dict.items():
        if file_name_list:
            file_name_list = list(set(file_name_list))
            file_name_str = " ".join(file_name_list)
            if codename == "stretch":
                includedeb_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/debian includedeb stretch ' + '/tmp/' + file_name_str + '"'
            else:
                includedeb_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/{codename} includedeb {codename} '.format(
                    codename=codename) + '/tmp/' + file_name_str + '"'
            logger.info(includedeb_cmd)
            r = os.popen(includedeb_cmd)
            time.sleep(5)


# 移除仓库包
def remove_deb(software_name, codename):
    # ssh -p 2322 debian@172.30.86.226 "reprepro -b /var/www/repos/apt/stretch remove stretch nfs4"
    remove_deb_cmd = 'ssh -p ' + settings.REPREPRO_SSH_PORT + ' root@' + settings.HOSTNAME + ' "reprepro -b /var/www/repos/apt/%s remove %s %s "' % (codename, codename, software_name)
    logger.info(remove_deb_cmd)
    r = os.popen(remove_deb_cmd)
    time.sleep(2)