import logging

from rest_framework import serializers

import software
from devices.models import Devices
from groups.models import GroupsDetail
from software.models import SoftwareInstalledModel, SoftwareInfoModel, SoftwareType, SoftwareOS, SoftwareArch, \
    SoftwareSource, SoftwarePolicy
import ipaddress


class SoftwareInfoModelSerializer(serializers.ModelSerializer):
    class Meta:
        model = SoftwareInfoModel
        fields = '__all__'


class SoftwareInstalledSerializer(serializers.ModelSerializer):
    class Meta:
        model = SoftwareInstalledModel
        fields = '__all__'


class SoftwareTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = SoftwareType
        fields = ('id', 'name', 'type', 'is_delete')


class SoftwareOSSerializer(serializers.ModelSerializer):
    class Meta:
        model = SoftwareOS
        fields = ('id', 'name', 'type', 'is_delete')


class SoftwareArchSerializer(serializers.ModelSerializer):
    class Meta:
        model = SoftwareArch
        fields = ('id', 'name', 'is_delete')


class SoftwareSourceSerializer(serializers.ModelSerializer):
    class Meta:
        model = SoftwareSource
        fields = ('id', 'name', 'address', 'type', 'status', 'remarks', 'link_status', 'last_update', 'domain_port','source_port')


class SoftwarePolicySerializer(serializers.ModelSerializer):
    software_num = serializers.SerializerMethodField()

    class Meta:
        model = SoftwarePolicy
        fields = (
            'id', 'name', 'serial_number', 'range_type', 'range_value', 'status', 'execution_time', 'software_ids',
            'creator', 'update_time',
            'remark', 'exception_switch_type', 'exception_range_type',
            'exception_range_value', 'apply_range_value', 'software_num', 'policy_type', 'is_enforced',
            'effective_date', 'expiration_date', 'retry_count', 'is_notify_user', 'is_user_cancel', 'software_black_ids'
        )

    @staticmethod
    def mid_install_status(obj):
        device_items = Devices.objects.exclude(is_domain=0).all()
        group_chat_items = GroupsDetail.objects.all()
        range_type = obj.range_type
        range_value = obj.range_value
        exception_switch_type = obj.exception_switch_type
        exception_range_type = obj.exception_range_type
        exception_range_value = obj.exception_range_value
        res_device = []
        if range_type == 0:
            organ_list = range_value.split(',')
            if device_items.filter(org_info_id__in=organ_list).exists():
                device_list = device_items.filter(org_info_id__in=organ_list).values_list('id', flat=True)
                res_device = list(device_list)
        elif range_type == 1:
            res_device = range_value.split(',')
        elif range_type == 2:
            ip_list = range_value.split(',')
            mid_ip_list = []
            for i in ip_list:
                start_ip, end_ip = i.split('-')
                # 解析起始和结束IP地址为IPv4Address对象
                start_ip = ipaddress.IPv4Address(start_ip)
                end_ip = ipaddress.IPv4Address(end_ip)
                range_ip_list = [str(ipaddress.IPv4Address(ip)) for ip in range(int(start_ip), int(end_ip) + 1)]
                mid_ip_list = range_ip_list + mid_ip_list
            if device_items.filter(device_ip__in=mid_ip_list).exists():
                device_list = device_items.filter(device_ip__in=mid_ip_list).values_list('id', flat=True)
                res_device = list(device_list)
        elif range_type == 3:
            account_list = range_value.split(',')
            if device_items.filter(bind_account__id__in=account_list).exists():
                device_list = device_items.filter(bind_account__id__in=account_list).values_list('id', flat=True)
                res_device = list(device_list)
        elif range_type == 4:
            # 群聊处理
            group_chat_list = range_value.split(',')
            if group_chat_items.filter(groups_id__in=group_chat_list).exists():
                group_chat_filter_items = group_chat_items.filter(groups_id__in=group_chat_list)
                items_0_range = group_chat_filter_items.filter(range=0).values_list('org_id', flat=True)
                items_1_range = group_chat_filter_items.filter(range=1).values_list('device_id', flat=True)
                items_3_range = group_chat_filter_items.filter(range=3).values_list('account_id', flat=True)
                # 处理部门数据
                if device_items.filter(org_info_id__in=items_0_range).exists():
                    device_0_list = device_items.filter(org_info_id__in=items_0_range).values_list('id', flat=True)
                    device_0_list = list(device_0_list)
                else:
                    device_0_list = []
                # 处理设备信息
                device_1_list = list(items_1_range)
                # 处理账号信息
                if device_items.filter(bind_account__id__in=items_3_range).exists():
                    device_3_list = device_items.filter(bind_account__id__in=items_3_range).values_list('id', flat=True)
                    device_3_list = list(device_3_list)
                else:
                    device_3_list = []
                device_list = device_0_list + device_1_list + device_3_list
                res_device = list(set(device_list))
        else:
            res_device = []

        # 排除例外的情况
        if (exception_switch_type == 1) and (res_device is not None):  # 例外开关已开启
            if exception_range_type == 1:  # 例外范围为设备
                exception_range_value = exception_range_value.split(',')
                res_device = [int(i) for i in res_device]  # 强转整形
                exception_range_value = [int(i) for i in exception_range_value]  # 强转整形
                set1 = set(res_device)
                set2 = set(exception_range_value)
                # logging.info(f"set1:{set1},set2:{set2}")
                result_set = set1 - set2
                res_device = list(result_set)

        return res_device

    def get_software_num(self, obj):
        software_ids_list = obj.software_ids.split(',') if obj.software_ids else []
        return len(software_ids_list)


class SoftwareModelSerializer(serializers.ModelSerializer):
    software_type_name = serializers.SerializerMethodField()
    software_os_name = serializers.SerializerMethodField()
    software_arch_name = serializers.SerializerMethodField()
    software_source_name = serializers.SerializerMethodField()

    class Meta:
        model = SoftwareInfoModel
        fields = ('id',
                  'name', 'main_package', 'main_version', 'remarks', 'os_id', 'software_os_name', 'arch_id',
                  'software_arch_name',
                  'software_type', 'software_type_name', 'source', 'software_source_name')

    def get_software_type_name(self, obj):
        software_type_name = obj.software_type.name
        return software_type_name

    def get_software_source_name(self, obj):
        software_source_name = obj.source.name
        return software_source_name

    def get_software_arch_name(self, obj):
        return obj.arch_name

    def get_software_os_name(self, obj):
        return obj.os_name
