# from django.conf import settings
import time
import hashlib
import requests
import json
import logging
import datetime
from django.db.models import Max

logger = logging.getLogger(__name__)


def test_software_source(domain):
    """
    测试软件源连接
    :return:
    """
    url = "/SoftwareCenterSyncApi/apis/getSofts"
    req_url = domain + url
    ts = int(time.time())
    secertkey = "domain_api_2023"
    appid = 1000
    limit = 1

    # sig 格式 appid#时间戳#secertkey
    # sig = "1000#16781543728#domain_api_2023"
    sig = "%s#%s#%s" % (appid, ts, secertkey)
    md5_sig = hashlib.md5(sig.encode("utf-8")).hexdigest()

    params = {
        "appid": appid,
        "timestamp": ts,
        "limit": limit,
        "sig": md5_sig,
        "gtTime": -1,
    }
    logger.info("test_software_source request url %s" % req_url)
    logger.info("test_software_source request params %s" % params)
    resp = requests.get(url=req_url, params=params)
    logger.info("test_software_source request resp code %s" % resp.status_code)
    if resp.status_code != 200:
        return False
    return True


def sync_software_source(id):
    """
    软件中心同步
    :return:
    """
    from software.models import SoftwareSource, SoftwareInfoModel, SoftwareType, SoftwareOS, SoftwareArch
    source = SoftwareSource.objects.filter(id=id).first()
    url = "/SoftwareCenterSyncApi/apis/getSofts"
    req_url = f"http://{source.address}:{source.domain_port}" + url
    ts = int(time.time())
    secertkey = "domain_api_2023"
    appid = 1000
    limit = 10000

    # sig 格式 appid#时间戳#secertkey
    # sig = "1000#16781543728#domain_api_2023"
    sig = "%s#%s#%s" % (appid, ts, secertkey)
    md5_sig = hashlib.md5(sig.encode("utf-8")).hexdigest()

    # 获取此软件源中软件最大的创建时间
    max_software_dict = SoftwareInfoModel.objects.filter(source_id=id, is_delete=0).aggregate(Max("created_at"))
    max_created_at = max_software_dict.get('created_at__max')
    if max_created_at:
        gtTime = int(max_created_at.timestamp())
    else:
        gtTime = -1

    params = {
        "appid": appid,
        "timestamp": ts,
        "limit": limit,
        "sig": md5_sig,
        "gtTime": gtTime,
    }

    logger.info("test_software_source request url %s" % req_url)
    logger.info("test_software_source request params %s" % params)
    resp = requests.get(url=req_url, params=params)
    logger.info("test_software_source request resp code %s" % resp.status_code)
    if resp.status_code != 200:
        return False

    resp_data = json.loads(resp.text)
    data_list = resp_data.get("data")
    logger.info("test_software_source request resp data_list %s" % data_list)
    # [{
    #     "id":"1484460892207710209",
    #     "name":"Xonotic",
    #     "deb":"xonotic",
    #     "icon":"http://localhost:8080/images_64/icon/1/1481621122590965761/1484460892207710209?random=dea732b1-e771-41d8-be7d-15656693fa21",
    #     "version":"0.8.2-1",
    #     "info":"Xonotic是一款基于 Nexuiz 2.5.2 的第一人称射击游戏，受 Unreal Tournament 以及 Quake 启发的竞技场射击游戏。\n",
    #     "category":"游戏",
    #     "arch":"x86_64",
    #     "os":"4.0(SP0),3.1,4.0(SP1)",
    #     "isdel":1,
    #     "updateTime":1642758031309
    # }]
    if not isinstance(data_list, list):
        return False

    for data in data_list:
        # print(data)
        # # 过滤已下架的软件
        # if data["isdel"] == 0:
        #     continue
        if SoftwareInfoModel.objects.filter(name=data["name"], main_version=data["version"], arch_name=data['arch'],
                                            is_delete=0).exists():
            continue

        # 获取软件类型
        software_type = SoftwareType.objects.filter(name=data["category"], is_delete=0).first()
        if not software_type:
            software_type = SoftwareType.objects.create(**{
                "name": data["category"],
                "type": 0,
                "is_delete": 0
            })
        # 13 位时间戳
        ts = data["updateTime"] / 1000
        ts_obj = datetime.datetime.fromtimestamp(ts)
        logger.info("software source data %s" % data)
        software = SoftwareInfoModel.objects.create(
            name=data["name"],
            main_version=data["version"],
            main_package=data["deb"],
            custome_filename=data["deb"],
            description=data["info"],
            remarks="",
            software_type_id=software_type.id,
            created_at=ts_obj,
            source=source,
            os_name=data['os'],
            arch_name=data['arch']
        )

        # # 多软件系统
        # os_str = data.get('os')
        # if "," in os_str:
        #     os_name_list = os_str.split(",")
        # else:
        #     os_name_list = [os_str]
        #
        # for software_os_name in os_name_list:
        #     software_os = SoftwareOS.objects.filter(name=software_os_name, is_delete=0).first()
        #     software.os.add(software_os)
        #
        # # 多软件架构
        # arch_name_str = data.get('arch')
        # if "," in arch_name_str:
        #     arch_name_list = arch_name_str.split(",")
        # else:
        #     arch_name_list = [arch_name_str]
        #
        # for arch_name in arch_name_list:
        #     software_arch = SoftwareArch.objects.filter(name=arch_name, is_delete=0).first()
        #     software.arch.add(software_arch)
    return True


if __name__ == "__main__":
    domain = 'http://172.30.98.241:8080'
    # test_software_center(domain)
    sync_software_source(domain)

    a = {'id': '1484460892207710209', 'name': 'Xonotic', 'deb': 'xonotic',
         'icon': 'http://172.30.98.241:8080/images_64/icon/1/1481621122590965761/1484460892207710209?random=7a5fdd26-a65a-4224-a6eb-6b404c82001e',
         'version': '0.8.2-1',
         'info': 'Xonotic是一款基于 Nexuiz 2.5.2 的第一人称射击游戏，受 Unreal Tournament 以及 Quake 启发的竞技场射击游',
         'arch': 'x86_64',
         'os': '4.0(SP0),3.1,4.0(SP1)',
         'isdel': 1,
         'updateTime': 1642758031309}

    # print(json.dumps(a, indent=4))
