import logging
from utils.utils import str_to_list
from accounts.models import AccountEntity
from devices.models import Devices
from software.models import SoftwareInfoModel, SoftwareInstalledModel


def update_policy_status(obj):
    logging.info("update_policy_status %s" % obj)
    if obj.range == 0:
        # 部门选择
        org_id_list = str_to_list(obj.org_id)
        accounts = AccountEntity.objects.filter(org_info__id__in=org_id_list)
        account_id_list = [x.id for x in accounts if x]
        device_list = Devices.objects.filter(accounts_id__in=account_id_list)
    else:
        # 终端选择
        device_id_list = str_to_list(obj.device_id)
        device_list = Devices.objects.filter(id__in=device_id_list)

    device_num = len(device_list)

    if device_num == 0:
        return
    if not obj.software_id:
        return

    # 获取软件总数
    software_id_list = str_to_list(obj.software_id)
    software_list = SoftwareInfoModel.objects.filter(id__in=software_id_list, is_delete=0)
    software_total_num = len(software_list)

    # 成功的设备数
    device_success_num = 0
    device_fail_num = 0
    for device in device_list:
        software_installed = SoftwareInstalledModel.objects.filter(device_info_id=device.id,
                                                                   software_info_id__in=software_id_list)
        software_success = software_installed.filter(installed_status=SoftwareInstalledModel.STATUS_INSTALLED)
        software_fail = software_installed.filter(installed_status=SoftwareInstalledModel.STATUS_FAILED)

        software_success_num = software_success.count()
        software_fail_num = 0
        if software_fail:
            software_fail_num = software_fail.count()

        if software_success_num == software_total_num:
            # 所有软件安装成功才算设备成功
            device_success_num += 1
        if software_fail_num > 0:
            device_fail_num += 1

    logging.info("update_policy_status  policy_id %s" % obj.id)
    logging.info("update_policy_status software_id_list %s" % software_id_list)
    logging.info("update_policy_status  device_success_num %s" % device_success_num)
    logging.info("update_policy_status  device_fail_num %s" % device_fail_num)
    logging.info("update_policy_status  device_num %s" % device_num)
    if device_success_num == device_num:
        obj.exec_status = 1
    if device_fail_num > 0:
        # 一条失败都为失败
        obj.exec_status = 3
    obj.save()
    return True
