# -*- coding: utf-8 -*-

from django.db import models
from devices.models import Devices


class SoftwareCenter(models.Model):
    is_status = (
        (0, 'N'), (1, 'Y')
    )

    package_name = models.CharField(verbose_name='软件包名', max_length=128, unique=True)
    softwareid = models.CharField(verbose_name='软件ID', max_length=1024)
    title = models.CharField(verbose_name='软件名称', max_length=128, null=True, blank=True)
    cv = models.CharField(verbose_name='cv', max_length=1024, null=True, blank=True)
    iv = models.CharField(verbose_name='iv', max_length=255, null=True, blank=True)
    nv = models.CharField(verbose_name='nv', max_length=255, null=True, blank=True)
    cmd = models.CharField(verbose_name='软件执行Exec', max_length=255, null=True, blank=True)
    short_desc = models.CharField(verbose_name='软件备注', max_length=255, null=True, blank=True)
    en_title = models.CharField(verbose_name='软件英文名', max_length=255, null=True, blank=True)
    en_desc = models.CharField(verbose_name='软件英文描述', max_length=255, null=True, blank=True)
    iconPath = models.CharField(verbose_name='软件Icon', max_length=255, null=True, blank=True)
    size = models.IntegerField(verbose_name='软件大小', null=True, blank=True)
    status = models.IntegerField(verbose_name='软件状态', choices=is_status, default=0, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_update = models.DateTimeField(verbose_name='最后修改时间', auto_now=True, null=True, blank=True)

    class Meta:
        db_table = 'software_center'
        verbose_name = '软件中心表'
        verbose_name_plural = '软件中心表'
        ordering = ('created_at',)

    def __str__(self):
        return self.package_name


class SoftwareCenterDevice(models.Model):
    is_status = (
        (0, 'N'), (1, 'Y')
    )
    software_center = models.ForeignKey(SoftwareCenter, on_delete=models.CASCADE, verbose_name='软件ID')
    device = models.ForeignKey(Devices, on_delete=models.CASCADE, verbose_name='设备ID')
    status = models.IntegerField(verbose_name='是否安装', choices=is_status, default=1, blank=True)

    class Meta:
        db_table = 'software_center_device'
        verbose_name = '终端软件中心关系表'
        verbose_name_plural = '终端软件中心关系表'

    def __str__(self):
        return self.package_name