"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/11/22 下午5:45
@File: start_scheduler_task.py
@Desc: 加域服务器后台定时任务
"""
import logging
import os
import time

import django
from django_redis import get_redis_connection



os.environ.setdefault("DJANGO_SETTINGS_MODULE", "nfs_domain_mainserver.settings")
django.setup()

from apscheduler.events import EVENT_JOB_EXECUTED, EVENT_JOB_ERROR
from utils.sch_backend_task import scheduler, my_listener, collection_line_task
from utils.sch_patch import patch_task
from utils.redis_domain_device import RedisCacheDevice
from browser.tasks import query_browser_policy_status
from utils.software_policy_analyse import software_calculation_policy

logger = logging.getLogger(__name__)
redis_conn = get_redis_connection('redis')


if __name__ == "__main__":
    RedisCacheDevice().handle()
    logger.info('添加数据库的缓存db--->redis')
    scheduler.add_job(patch_task, 'interval', minutes=10, coalesce=True, max_instances=1)
    scheduler.add_job(collection_line_task, 'cron', hour='0-23', coalesce=True, max_instances=1)  # 设备在离线任
    # scheduler.add_job(calculate_policy, 'interval', minutes=90, coalesce=True, max_instances=1)  # 计算策略优先级任务
    redis_conn.hset('patch_db_cache', 'sch_patch_task', '0')
    # 浏览器策略下发状态完成情况定时查询
    scheduler.add_job(query_browser_policy_status, 'interval', minutes=10, coalesce=True, max_instances=1)
    # 软件策略生效期任务
    scheduler.add_job(software_calculation_policy, 'cron', hour='0-23', coalesce=True, max_instances=1)
    # 添加任务监听失败，并重新提起任务
    scheduler.add_listener(my_listener, EVENT_JOB_EXECUTED | EVENT_JOB_ERROR)
    try:
        scheduler.start()
    except (KeyboardInterrupt, SystemExit):
        pass

    while True:
        time.sleep(1)
