#!/usr/bin/python

import sys
import os
import time
import datetime

try:
    import pymysql
except Exception as e:
    assert e

if not os.environ.get("DB_NAME"):
    DATABASES = {
        "ENGINE": "django.db.backends.mysql",
        "NAME": 'nfs-domain',
        "USER": 'root',
        "PASSWORD": '123456',
        "HOST": '127.0.0.1',
        "PORT": 3306,
    }
else:
    DATABASES = {
        "ENGINE": "django.db.backends.mysql",
        "NAME": os.environ.get("DB_NAME"),
        "USER": os.environ.get("DB_USER"),
        "PASSWORD": os.environ.get("DB_PASS"),
        "HOST": os.environ.get("DB_HOST"),
        "PORT": int(os.environ.get("DB_PORT")),
    }


def init_db():
    conn = pymysql.connect(
        host=DATABASES["HOST"],
        port=DATABASES["PORT"],
        user=DATABASES["USER"],
        password=DATABASES["PASSWORD"],
    )
    cursor = conn.cursor()
    create_db_sql = f"CREATE DATABASE IF NOT EXISTS `{DATABASES['NAME']}` DEFAULT CHARACTER SET utf8; "
    cursor.execute(create_db_sql)
    conn.close()
    cursor.close()


def conn_mysql():
    conn = pymysql.connect(
        host=DATABASES["HOST"],
        port=DATABASES["PORT"],
        user=DATABASES["USER"],
        password=DATABASES["PASSWORD"],
        database=DATABASES["NAME"],
        charset='utf8',
        autocommit=True
    )
    cursor = conn.cursor(pymysql.cursors.DictCursor)
    return conn, cursor


def init_data(conn, cursor):
    now = int(time.time())
    timeArray = time.localtime(now)
    local_time = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
    aptly_local_ip = os.environ.get('HOSTNAME')
    aptly_local_port = os.environ.get('REPREPRO_SSH_PORT')

    check_screensavers_sql = '''select id from `screensavers`;'''

    create_screensavers_sql = '''INSERT INTO `screensavers` (`id`, `switch_status`, `interval`) VALUES (%s, %s, %s);'''

    try:
        res = cursor.execute(check_screensavers_sql)
        if not res:
            cursor.executemany(create_screensavers_sql, [(1, 0, '900')])
            time.sleep(1)
    except Exception as exp:
        print('screen config setting err :', exp)
    else:
        print('create screensavers config')

    check_passwordaccountrule_sql = '''select id from `accounts_passwordaccountrule`;'''

    create_passwordaccountrule_sql = """INSERT INTO `accounts_passwordaccountrule` (`id`, `accounts_check_status`, `account_mix_length`, `is_lower`, `is_upper`, `is_num`, `is_char`, `password_extra_status`) VALUES (%s, %s, %s, %s,%s, %s, %s, %s);"""

    try:
        res = cursor.execute(check_passwordaccountrule_sql)
        if not res:
            cursor.executemany(create_passwordaccountrule_sql, [(1, 0, 6, 1, 1, 1, 1, 0)])
            time.sleep(1)
    except Exception as exp:
        print('passwordaccountrule config setting err :', exp)
    else:
        print('create passwordaccountrule config')

    create_permission_sql = """INSERT INTO `users_permission` (`id`, `name`, `type`, `method`, `parent_id`) VALUES (%s, %s, %s, %s, %s);"""
    try:
        cursor.executemany(create_permission_sql, [
            (1, '首页', '目录', 'homepage', None),
            (2, '部门管理', '目录', 'list_of_departments', None),
            (3, '用户管理', '目录', 'users_management', None),
            (4, '终端管理', '目录', 'list_of_device', None),
            (5, '策略管理', '目录', 'policy_management', None),
            (6, '任务管理', '目录', 'task_management', None),
            (7, '软件管理', '目录', 'software_management', None),
            (8, '日志管理', '目录', 'review_the_logs', None),
            (9, '平台管理', '目录', 'platform_management', None),
            (25, '补丁管理', '目录', 'patch_management', None),
            (29, '浏览器管理', '目录', 'browser_management', None),
            (56, '系统部署', '目录', 'sysdeploy_management', None),
            (30, '消息管理', '目录', 'message_management', None),
            (10, '用户列表', '菜单', 'list_of_users', 3),
            (31, '认证对接', '菜单', 'soap_migration', 3),
            (11, 'AD对接', '菜单', 'data_migration', 3),
            (13, '软件列表', '菜单', 'list_of_software', 7),
            (15, '策略执行日志', '菜单', 'policy_execution_log', 8),
            (16, '管理员权限', '菜单', 'admin_privileges', 9),
            (17, '规则设置', '菜单', 'rules_setting', 9),
            (18, '时钟同步', '菜单', 'clock_synchronization', 9),
            (20, 'usb策略', '菜单', 'usb_of_policy', 5),
            # (21, '防火墙策略', '菜单', 'firewalld_of_policy', 5),
            # (22, '远程策略', '菜单', 'ssh_of_policy', 5),
            (23, '桌面策略', '菜单', 'desktop_of_policy', 5),
            (24, '系统策略', '菜单', 'system_of_policy', 5),
            (26, '补丁列表', '菜单', 'list_of_patch', 25),
            (27, '补丁策略', '菜单', 'policy_of_patch', 25),
            (28, '补丁源管理', '菜单', 'source_of_patch', 25),
            (32, '用户策略', '菜单', 'policy_of_accounts', 29),
            (33, '书签策略', '菜单', 'policy_of_bookmarks', 29),
            (34, '新标签页应用策略', '菜单', 'policy_of_tags', 29),
            (35, '水印策略', '菜单', 'watermark_of_policy', 5),
            (36, '激活策略', '菜单', 'active_of_policy', 5),
            (37, '文件重定向策略', '菜单', 'refile_of_policy', 5),
            (38, '桌面主题策略', '菜单', 'desktop_theme_of_policy', 5),
            (39, '更新策略', '菜单', 'systemupdate_of_policy', 5),
            (40, '屏幕保护策略', '菜单', 'screensaver_of_policy', 5),
            (41, '软件运行策略', '菜单', 'app_running_of_policy', 5),
            (42, '登入登出策略', '菜单', 'login_logout_of_policy', 5),
            (43, '可信任站点', '菜单', 'policy_of_sites', 29),
            (44, '收藏夹策略', '菜单', 'policy_of_favorites', 29),
            (45, 'DNS', '菜单', 'DNS', 9),
            (46, '备份还原', '菜单', 'backup_restore', 9),
            (47, '报表管理', '目录', 'report_management', None),
            (48, '人员日志', '菜单', 'account_log', 8),
            (49, '终端硬件信息', '菜单', 'device_hardware', 47),
            (50, '终端信息', '菜单', 'device_info', 47),
            (14, '软件源管理', '菜单', 'software_resource_management', 7),
            (51, '远程策略', '菜单', 'remote_of_policy', 5),
            (19, '软件策略', '菜单', 'software_of_policy', 7),
            (52, '客户端设置', '菜单', 'client_of_opt', 9),
            (53, '企业定制', '菜单', 'client_of_div', 9),
            (54, '终端日志', '菜单', 'client_of_log', 8),
            (55, '管理员日志', '菜单', 'admin_of_log', 8),
            (57, '镜像列表', '菜单', 'sysiso_info', 56),
            (58, '策略列表', '菜单', 'list_of_policy_v2', 5),
        ])
        time.sleep(1)
        print('create permission config')
    except Exception as exp:
        print('permission config setting err :', exp)

    check_policy_dict_info_sql = '''select id from `policy_dict_info`;'''

    create_policy_dict_info_sql = """INSERT INTO `policy_dict_info` (`id`, `policy_type_name`, `policy_model_context`, `policy_type`) VALUES (%s, %s, %s, %s);"""

    try:
        res = cursor.execute(check_policy_dict_info_sql)
        if not res:
            cursor.executemany(create_policy_dict_info_sql, [
                (1, '软件策略', '{\"software_id\":\"\"}', 'software_py'),
                (2, 'usb策略', '{\"usb_switch\":1,\"usb_rw_status\":\"rw\"}', 'usb_py'),
                (3, '防火墙策略',
                 '{\"firewalld_switch\":0,\"firewalld_level\":1,\"firewalld_white_switch\":0,\"firewalld_white_list\":[],\"firewalld_black_switch\":0,\"firewalld_black_list\":[]}',
                 'firewalld_py'),
                (4, '远程策略', '{\"remote_ssh_switch\":1}', 'remote_py'),
                (5, '桌面策略', '{\"screen_lock_switch\":1,\"screen_lock_time\":300,\"wallpaper_id\":6}', 'desktop_py'),
                (6, '系统策略',
                 '{\"root_switch\":0,\"activation_switch\":0,\"activation_address\":\"https://cloudaccount.os.nfschina.com/\",\"default_ca_switch\":0,\"default_ca_id\":\"\"}',
                 'system_py')
            ])
            time.sleep(1)
    except Exception as exp:
        print('policy_dict_info config setting err :', exp)
    else:
        print('create policy_dict_info config')

    # 初始化三元用户
    create_user_sql = '''
        INSERT INTO users_user (id, username, password, is_superuser, is_staff, is_active, is_netdom, created_at,remark,org_global_switch_device,org_global_switch_user)
          VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s);
        '''

    try:
        cursor.executemany(create_user_sql, [
            # (1, 'admin', 'd4e8e6deaa7b1f8381e09e3e6b83e36f0b681c5c', 1, 1, 0, 1, local_time, '超级管理员', 0, 0),
            (1, 'admin', 'de44948a690e6a3ffbd8cef3fdb631b8bf91dc68', 1, 1, 0, 1, local_time, '超级管理员', 0, 0),
        ])
        print("create superuser admin")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print("admin exists")

    # 初始化根部门信息
    # INSERT INTO `nfs-domain`.org_info (name,address,contact_phone,remarks,created_at,last_update,executive_id,parent_id) VALUES
    # ('顶层部门',NULL,NULL,NULL,'2022-04-29 10:40:30.000','2022-05-10 10:28:36.615',NULL,1);
    create_org_info_sql = '''
        INSERT INTO org_info (name,address,contact_phone,remarks,created_at,last_update,executive_id,parent_id,create_type)
          VALUES ("顶层部门",NULL,NULL,NULL,%s,%s,NULL,1,0)
        '''
    check_org_info_sql = '''
        SELECT id FROM org_info WHERE name='顶层部门';
    '''
    try:
        res = cursor.execute(check_org_info_sql)
        if not res:
            cursor.execute(create_org_info_sql, [
                datetime.datetime.now(), datetime.datetime.now()
            ])
            print("create org_info root")
            time.sleep(1)

    except pymysql.err.IntegrityError:
        print('create org_info error')

    create_software_type_sql = '''INSERT INTO software_type (id,name,type,is_delete) VALUES (%s, %s, %s, %s);'''
    try:
        cursor.executemany(create_software_type_sql, [
            (1, '安全', 0, 0),
            (2, '办公', 0, 0),
            (3, '开发', 0, 0),
            (4, '社交', 0, 0),
            (5, '图形', 0, 0),
            (6, '网络', 0, 0),
            (7, '系统', 0, 0),
            (8, '影音', 0, 0),
            (9, '游戏', 0, 0),
            (10, '教育', 0, 0),
            (11, '其他', 0, 0),
            (12, '驱动', 0, 0),
        ])
        print("create software_type config")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('software_type inited')

    # 创建补丁策略下发配置信息
    create_patch_delivery_settings = '''INSERT INTO patch_delivery_settings (id,number_queues,number_concurrent,timeout_period) VALUES (%s, %s, %s, %s)'''
    try:
        cursor.executemany(create_patch_delivery_settings, [(1, 1, 10, 5)])
        print("create patch_delivery_settings config")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('patch_delivery_settings inited')

    # 创建加域规则默认策略
    create_sys_domain_rule = '''INSERT INTO sys_domain_rule (id,sms_switch,hostname_switch,hostname_first_type,hostname_first_name,hostname_second_type,hostname_second_name,edit_hostname_switch,other_org_label,is_up_search) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)'''
    try:
        cursor.executemany(create_sys_domain_rule, [(1, 0, 0, 0, None, 3, None, 1, None, 1)])
        print("create sys_domain_rule config")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('sys_domain_rule inited')

    # 创建设备更新默认策略
    create_sys_update_policy = '''INSERT INTO policy_system_update (`id`, `status`, `system_update_switch`, `update_style`, `check_time_days`, `check_time_time`, `download_update_method`) VALUES (%s, %s, %s, %s, %s, %s, %s)'''
    try:
        cursor.executemany(create_sys_update_policy, [(1, 0, 0, 0, 0, 2, 0)])
        print("create sys_update_policy config")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('sys_update_policy inited')

    create_software_os_sql = '''INSERT INTO software_os (id,name,type,is_delete) VALUES (%s, %s, %s, %s);'''
    try:
        cursor.executemany(create_software_os_sql, [
            (1, '方德桌面4.0', 0, 0),
            (2, '方德桌面5.0', 0, 0),
        ])
        print("create software_os config")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('software_os inited')

    """"""

    create_watermark_dict_sql = '''INSERT INTO `watermark` (id, watermark_switch_status, require_account_name, require_device_cnname, require_device_hostname, require_device_ip, require_device_mac, require_device_current_time, user_defined_text, watermark_position, watermark_rotate, watermark_font, watermark_font_size, watermark_font_color, watermark_font_transparency) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s);'''
    try:
        cursor.executemany(create_watermark_dict_sql, [
            (1, 0, 1, 0, 0, 1, 0, 1, None, '全屏', '0', '黑体', 16, 'ff0000', 0.40),
        ])
        print("create watermark config")
        time.sleep(1)
    except Exception as exp:
        print(f'watermark error {exp}')

    create_patch_sys_dict_sql = '''INSERT INTO patch_sys_dict (id,patch_sys_ver,patch_is_delete,patch_is_sys) VALUES (%s, %s, %s, %s);'''
    try:
        cursor.executemany(create_patch_sys_dict_sql, [
            (1, 'V4.0', 0, 1),
            (2, 'V5.0', 0, 1),
        ])
        print("create patch_sys_dict config")
        time.sleep(1)
    except Exception as exp:
        print(f'patch_sys_dict error {exp}')

    create_patch_type_dict_sql = '''INSERT INTO patch_type_dict (id,patch_type_name,patch_is_delete,patch_is_sys,patch_repo_name) VALUES (%s, %s, %s, %s, %s);'''
    try:
        cursor.executemany(create_patch_type_dict_sql, [
            (1, '系统补丁', 0, 1, 'sysupdate-repo'),
            (2, '安全补丁', 0, 1, 'securityupdate-repo'),
        ])
        print("create patch_type_dict config")
        time.sleep(1)
    except Exception as exp:
        print(f'patch_type_dict error {exp}')

    create_software_arch_sql = '''INSERT INTO software_arch (id,name,type,is_delete) VALUES (%s, %s, %s, %s);'''
    try:
        cursor.executemany(create_software_arch_sql, [
            (1, 'X86', 0, 0),
            (2, 'ARM', 0, 0),
        ])
        print("create software_arch config")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('software_arch inited')

    create_software_source_sql = '''INSERT INTO software_source (id,name,type,status,link_status,created_at,last_update,is_delete) VALUES (%s, %s, %s, %s,%s,%s,%s, %s);'''
    try:
        cursor.executemany(create_software_source_sql, [
            (1, '平台软件源', 0, 0, 0, datetime.datetime.now(), datetime.datetime.now(), 0),
        ])
        print("create software_source config")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('software_source inited')

    create_patch_source_sql = '''INSERT INTO patch_source (`id`, `patch_source_name`, `patch_source_url`, `patch_source_port`, `patch_source_enable`, `update_time`, `remark`) VALUES (%s, %s, %s, %s,%s,%s,%s);'''
    try:
        cursor.executemany(create_patch_source_sql, [
            (1, '平台补丁源', aptly_local_ip, aptly_local_port, 1, datetime.datetime.now(), None),
        ])
        print("create patch_source config")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('patch_source inited')

    # 远程策略
    create_remote_assist_sql = '''INSERT INTO remote_assist (id, argee, concat, concat_time, sessions, sessions_time, status, enabled, global_enabled, need_passwd) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s);'''
    try:
        cursor.executemany(create_remote_assist_sql, [
            (1, 1, 0, 5, 1, 5, 0, 1, 1, 0),
        ])
        print("create remote_assist config")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('remote_assist inited')

    # 本地账号登陆规则
    create_local_user_login_sql = """ INSERT INTO `local_user_login` (`id`, `status`, `range`, `org_id`, `device_id`) 
        VALUES (%s, %s, %s, %s, %s); 
    """

    try:
        cursor.executemany(create_local_user_login_sql, [
            (1, 1, 0, "", ""),
        ])
        print("create local_user_login config")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('local_user_login inited')

    # 本地账号登陆规则
    create_passwd_prohibit_mod_sql = """ INSERT INTO `accounts_passwordaccountprohibitmodification` (`id`, `status`, `range`, `org_id`, `account_id`) 
        VALUES (%s, %s, %s, %s, %s); 
    """

    try:
        cursor.executemany(create_passwd_prohibit_mod_sql, [
            (1, 0, 0, "", ""),
        ])
        print("create passwd_prohibit_mod config")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('passwd_prohibit_mod inited')

    # default AD
    create_default_ad_sql = '''INSERT INTO accounts_ldap (id,name,host,port,base_dn,admin_user_dn,admin_password,default_password,created_at,ldap_type) VALUES (%s, %s, %s, %s,%s,%s,%s, %s,%s, %s);'''
    try:
        cursor.executemany(create_default_ad_sql, [
            (1, '平台源', ' ', ' ', ' ', ' ', ' ', ' ', datetime.datetime.now(), 0),
        ])
        print("create default AD")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('default AD inited')

    # menu_info和menu_sub_action
    try:
        __init_menu_info_and_sub_action(cursor=cursor)
    except Exception as exp:
        print('create table menu_info and menu_sub_action err :', exp)
    else:
        print('create table menu_info and menu_sub_action success.')

    jids = "CREATE TABLE IF not EXISTS `jids` (`jid` varchar(255) NOT NULL,`load` mediumtext NOT NULL,UNIQUE KEY `jid` (`jid`) ) ENGINE=InnoDB DEFAULT CHARSET=utf8;"

    salt_returns = "CREATE TABLE IF not EXISTS `salt_returns` (\
        `fun` VARCHAR ( 50 ) NOT NULL,\
        `jid` VARCHAR ( 255 ) NOT NULL,\
        `return` MEDIUMTEXT NOT NULL,\
        `id` VARCHAR ( 255 ) NOT NULL,\
        `success` VARCHAR ( 10 ) NOT NULL,\
        `full_ret` MEDIUMTEXT NOT NULL,\
        `alter_time` TIMESTAMP DEFAULT CURRENT_TIMESTAMP,\
        KEY `id` ( `id` ),\
        KEY `jid` ( `jid` ),\
        KEY `fun` ( `fun` ) \
        ) ENGINE = INNODB DEFAULT CHARSET = utf8;"

    salt_enevts = "CREATE TABLE IF not EXISTS `salt_events` (\
        `id` BIGINT NOT NULL AUTO_INCREMENT,\
        `tag` VARCHAR ( 255 ) NOT NULL,\
        `data` MEDIUMTEXT NOT NULL,\
        `alter_time` TIMESTAMP DEFAULT CURRENT_TIMESTAMP,\
        `master_id` VARCHAR ( 255 ) NOT NULL,\
        PRIMARY KEY ( `id` ),\
        KEY `tag` ( `tag` ) \
        ) ENGINE = INNODB DEFAULT CHARSET = utf8;"

    try:
        cursor.execute(jids)
        cursor.execute(salt_returns)
        cursor.execute(salt_enevts)
        print('init salt table')
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('init salt table error')

    # 纳管终端统计默认策略
    create_system_device_stats = '''INSERT INTO system_device_stats (id,is_switch,stats_max) VALUES (%s, %s, %s)'''
    try:
        cursor.executemany(create_system_device_stats, [(1, 0, 0)])
        print("create system_device_stats config")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('system_device_stats inited')

    # 系统厂商初始值
    create_default_sysios_factory_sql = '''INSERT INTO sysios_factory (id,name,remarks,created_at) VALUES (%s, %s, %s, %s);'''
    try:
        cursor.executemany(create_default_sysios_factory_sql, [
            (1, '方德', '中科方德', datetime.datetime.now()),
        ])
        print("create default sysios_factory")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('default sysios_factory inited')

    # 系统架构初始值
    create_default_sysios_arch_sql = '''INSERT INTO sysios_arch (id,name,created_at) VALUES (%s, %s, %s);'''
    try:
        cursor.executemany(create_default_sysios_arch_sql, [
            (1, 'X86', datetime.datetime.now()),
        ])
        print("create default sysios_arch")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('default sysios_arch inited')

    # 系统语言初始值
    create_default_sysios_language_sql = '''INSERT INTO sysios_language (id,name,value,created_at) VALUES (%s, %s, %s, %s);'''
    try:
        cursor.executemany(create_default_sysios_language_sql, [
            (1, '中文', 'zh_CN', datetime.datetime.now()),
        ])
        print("create default sysios_language")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('default sysios_language inited')

    # 系统键盘布局初始值
    create_default_sysios_keyboard_sql = '''INSERT INTO sysios_keyboard (id,name,value, created_at) VALUES (%s, %s, %s, %s);'''
    try:
        cursor.executemany(create_default_sysios_keyboard_sql, [
            (1, '汉语', 'default', datetime.datetime.now()),
        ])
        print("create default sysios_keyboard")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('default sysios_keyboard inited')

    # 系统键盘布局初始值
    create_default_sysios_timezone_sql = '''INSERT INTO sysios_timezone (id,name,value,created_at) VALUES (%s, %s, %s, %s);'''
    try:
        cursor.executemany(create_default_sysios_timezone_sql, [
            (1, '上海', 'Asia/Shanghai', datetime.datetime.now()),
        ])
        print("create default sysios_timezone")
        time.sleep(1)
    except pymysql.err.IntegrityError:
        print('default sysios_timezone inited')


    # 关闭游标和连接
    cursor.close()
    conn.close()


def __init_menu_info_and_sub_action(cursor):
    """
    初始化菜单表和菜单子行为表
    """
    try:
        # 先判断menu_sub_action是否存在,如果不存在就删除原有的menu_info和创建menu_sub_action
        check_menu_sub_action_sql = '''SELECT COUNT(*) FROM information_schema.tables WHERE table_schema = DATABASE() AND table_name = 'menu_sub_action';'''
        recreate_menu_info_sql = """
            DROP TABLE `menu_info`;
            CREATE TABLE IF NOT EXISTS `menu_info` (`id` int(11) NOT NULL AUTO_INCREMENT, `url_match` varchar(256) DEFAULT NULL, `desc` varchar(128) DEFAULT NULL, `action` varchar(128) NOT NULL, `module` varchar(32) NOT NULL, PRIMARY KEY (`id`), UNIQUE KEY `action` (`action`), KEY `idx_menu_info_url_match` (`url_match`)) ENGINE=InnoDB DEFAULT CHARSET=utf8mb3;
            CREATE TABLE IF NOT EXISTS  `menu_sub_action` (`id` int(11) NOT NULL AUTO_INCREMENT, `action` varchar(128) NOT NULL, `sub_action` varchar(128) NOT NULL, `sub_action_desc` varchar(128) DEFAULT NULL, PRIMARY KEY (`id`), KEY `idx_menu_sub_action_action` (`action`), KEY `idx_menu_sub_action_sub_action` (`sub_action`)) ENGINE=InnoDB DEFAULT CHARSET=utf8mb3;
            """
        res = cursor.execute(check_menu_sub_action_sql)
        if not res:
            cursor.execute(recreate_menu_info_sql)
        #
        time.sleep(1)

        # menu_info
        truncate_menu_info_sql = '''TRUNCATE TABLE `menu_info`;'''
        # 先清空数据,再重新插入
        cursor.execute(truncate_menu_info_sql)
        time.sleep(1)
        create_menu_info_sql = '''INSERT INTO `menu_info` (`action`,`module`,`url_match`,`desc`) VALUES (%s, %s, %s, %s);'''
        cursor.executemany(create_menu_info_sql, [
            ('account_passwd_prohibit_mod_x', 'account', '^/mainapi/account/passwd_prohibit_mod/[0-9]+/$',
             '账号锁定规则'),
            ('account_passwd_prohibit_mod', 'account', '^/mainapi/account/passwd_prohibit_mod/$', '账号锁定规则'),
            ('adminlog_search_actions', 'logs', '^/mainapi/adminlog/search/actions/$', '管理员日志:行为列表'),
            ('adminlog_search_login', 'logs', '^/mainapi/adminlog/search/login/$', '管理员日志:管理员登录登出日志'),
            ('adminlog_search_org', 'logs', '^/mainapi/adminlog/search/org/$', '管理员日志:组织架构日志'),
            ('adminlog_search_account', 'logs', '^/mainapi/adminlog/search/account/$', '管理员日志:终端用户日志'),
            ('adminlog_search_device', 'logs', '^/mainapi/adminlog/search/device/$', '管理员日志:终端日志'),
            ('adminlog_search_software', 'logs', '^/mainapi/adminlog/search/software/$', '管理员日志:软件日志'),
            ('adminlog_search_patch', 'logs', '^/mainapi/adminlog/search/patch/$', '管理员日志:补丁日志'),
            ('adminlog_search_logs', 'logs', '^/mainapi/adminlog/search/logs/$', '管理员日志:日志'),
            ('adminlog_search_user', 'logs', '^/mainapi/adminlog/search/user/$', '管理员日志:平台日志'),
            ('adminlog_search_salt', 'logs', '^/mainapi/adminlog/search/salt/$', '管理员日志:策略日志'),
            ('adminlog_search_tasks', 'logs', '^/mainapi/adminlog/search/tasks/$', '管理员日志:日志'),
            ('adminlog_search_admins', 'logs', '^/mainapi/adminlog/search/admins/$', '管理员日志:管理员列表'),
            ('adminlog_search_export_login', 'logs', '^/mainapi/adminlog/search/export_login/$',
             '管理员日志导出:登录登出日志'),
            ('adminlog_search_export_org', 'logs', '^/mainapi/adminlog/search/export_org/$',
             '管理员日志导出:组织架构日志'),
            ('adminlog_search_export_account', 'logs', '^/mainapi/adminlog/search/export_account/$',
             '管理员日志导出:终端用户日志'),
            ('adminlog_search_export_device', 'logs', '^/mainapi/adminlog/search/export_device/$',
             '管理员日志导出:终端日志'),
            ('adminlog_search_export_software', 'logs', '^/mainapi/adminlog/search/export_software/$',
             '管理员日志导出:软件日志'),
            ('adminlog_search_export_patch', 'logs', '^/mainapi/adminlog/search/export_patch/$',
             '管理员日志导出:补丁日志'),
            ('adminlog_search_export_logs', 'logs', '^/mainapi/adminlog/search/export_logs/$', '管理员日志导出:日志'),
            ('adminlog_search_export_user', 'logs', '^/mainapi/adminlog/search/export_user/$',
             '管理员日志导出:平台日志'),
            ('adminlog_search_export_salt', 'logs', '^/mainapi/adminlog/search/export_salt/$',
             '管理员日志导出:策略日志'),
            ('adminlog_search_export_tasks', 'logs', '^/mainapi/adminlog/search/export_tasks/$', '管理员日志导出:日志'),
            ('account_info', 'account', '^/mainapi/account/info/$', '域账号'),
            ('account_info_account_list', 'account', '^/mainapi/account/info/account_list/$', '域账号列表信息'),
            ('account_auth_source_prefix', 'account', '^/mainapi/account/auth_source/prefix/$', '认证源前缀'),
            ('account_passwd_rule', 'account', '^/mainapi/account/passwd_rule/$', '域用户密码权限校验'),
            ('account_black_white_lists', 'account', '^/mainapi/account/black_white_lists/$', '黑白名单'),
            ('adminlog_search', 'adminlog', '^/mainapi/adminlog/search$', '管理员日志'),
            ('adminlog_alltypes', 'adminlog', '^/mainapi/adminlog/alltypes$', '管理员日志'),
            ('account_backup_restore_callback', 'account', '^/mainapi/account/backup_restore/callback/$',
             'ldap备份还原'),
            ('account_dns_record_multiple_delete', 'account', '^/mainapi/account/dns_record/multiple_delete/$',
             'DNS管理'),
            ('account_x', 'account', '^/mainapi/account/[0-9]+$', '域用户'),
            ('account_ldap_x', 'account', '^/mainapi/account/ldap/[0-9]+/$', 'AD数据源'),
            ('account_ldap_x_sync', 'account', '^/mainapi/account/ldap/[0-9]+/sync/$', '迁移AD数据源'),
            ('account_ldap_x_sync_detail', 'account', '^/mainapi/account/ldap/[0-9]+/sync/detail/$', '迁移'),
            ('account_ldap_x_sync_detail_x', 'account', '^/mainapi/account/ldap/[0-9]+/sync/detail/[0-9]+/$',
             '迁移详情'),
            ('account_batch_tmpl_xlsx', 'account', '^/mainapi/account/batch/tmpl/[^/]+$', '下载模块'),
            ('apis_auth', 'apis', '^/apis/auth$', '认证'),
            ('apis_password', 'apis', '^/apis/password$', '密码'),
            ('account', 'account', '^/mainapi/account/$', '终端用户信息'),
            ('account_auth_source_x', 'account', '^/mainapi/account/auth_source/[0-9]+/$', '外部源'),
            ('account_ldap', 'account', '^/mainapi/account/ldap/$', 'AD数据源'),
            ('account_ldap_pk', 'account', '^/mainapi/account/ldap/pk/$', 'AD数据源'),
            (
            'account_local_user_login_x', 'account', '^/mainapi/account/local_user_login/[0-9]+/$', '本地用户登录规则'),
            ('account_soap_records_x', 'account', '^/mainapi/account/soap_records/[0-9]+/_[0-9]+/$',
             'SOAP同步数据记录详情'),
            ('account_info_account_batch_operation', 'account', '^/mainapi/account/info/account_batch_operation/$',
             '域账号'),
            ('account_passwd_rule_change_passwd_rule', 'account', '^/mainapi/account/passwd_rule/change_passwd_rule/$',
             '域用户密码权限校验'),
            ('account_local_user_login_lock_local_user_action', 'account',
             '^/mainapi/account/local_user_login/lock_local_user_action/$', '本地用户锁定'),
            ('account_local_user_login_unlock_local_user_action', 'account',
             '^/mainapi/account/local_user_login/unlock_local_user_action/$', '本地用户解锁'),
            ('account_local_user_login_update', 'account', '^/mainapi/account/local_user_login/update/$',
             '本地用户默认锁定'),
            ('account_id', 'account', '^/mainapi/account/[0-9]+$', '终端用户信息'),
            ('account_auth_source_create', 'account', '^/mainapi/account/auth_source/create/$', '认证源'),
            ('account_auth_source_destroy', 'account', '^/mainapi/account/auth_source/destroy/$', '认证源'),
            ('account_auth_source_test_connection', 'account', '^/mainapi/account/auth_source/test_connection/$',
             '测试认证源'),
            ('account_batch_file', 'account', '^/mainapi/account/batch/[^/]+$', '导入域账号'),
            ('account_batch_ldap_file', 'account', '^/mainapi/account/batch/ldap/[^/]+$', '导入AD数据源'),
            ('account_bind', 'account', '^/mainapi/account/bind$', '绑定终端设备'),
            (
            'account_black_white_lists_change_status', 'account', '^/mainapi/account/black_white_lists/change_status/$',
            '修改黑白名单状态'),
            ('account_black_white_lists_create', 'account', '^/mainapi/account/black_white_lists/create/$',
             '新增黑白名单'),
            ('account_black_white_lists_multiple_delete', 'account',
             '^/mainapi/account/black_white_lists/multiple_delete/$', '删除黑白名单'),
            ('account_black_white_lists_multiple_update', 'account',
             '^/mainapi/account/black_white_lists/multiple_update/$', '修改黑白名单'),
            ('account_ldap_multiple_delete', 'account', '^/mainapi/account/ldap/multiple_delete/$', '删除AD数据源连接'),
            ('account_ldap_ssl_file', 'account', '^/mainapi/account/ldap/ssl_file/$', '上传SSL证书'),
            ('account_ldap_test_connection', 'account', '^/mainapi/account/ldap/test_connection/$', '测试AD数据源'),
            ('account_passwd', 'account', '^/mainapi/account/passwd$', '重置终端用户密码'),
            ('account_passwd_prohibit_mod_multiple_delete', 'account',
             '^/mainapi/account/passwd_prohibit_mod/multiple_delete/$', '删除密码禁止模式'),
            ('account_passwd_prohibit_mod_update', 'account', '^/mainapi/account/passwd_prohibit_mod/update/$',
             '修改密码禁止模式'),
            ('account_passwd_rule_account_rule_info', 'account', '^/mainapi/account/passwd_rule/account_rule_info/$',
             '账号规则'),
            ('account_passwd_rule_passwd_extra_account', 'account',
             '^/mainapi/account/passwd_rule/passwd_extra_account/$', '列外账号范围'),
            ('account_passwd_rule_passwd_extra_org', 'account', '^/mainapi/account/passwd_rule/passwd_extra_org/$',
             '额外部门范围'),
            ('account_security', 'account', '^/mainapi/account/security$', '用户安全配置'),
            ('account_soap_records_records_batch_operation', 'account',
             '^/mainapi/account/soap_records/records_batch_operation/$', '批量操作SOAP记录'),
            ('account_status', 'account', '^/mainapi/account/status$', '终端用户状态'),
            ('account_backup_restore', 'account', '^/mainapi/account/backup_restore/$', '备份还原'),
            ('account_dns_record', 'account', '^/mainapi/account/dns_record/$', 'DNS资源记录'),
            ('browser_policy_application_info_all', 'browser', '^/mainapi/browser/policy/application/info/all/$',
             '浏览器新标签页应用策略'),
            ('browser_policy_user_info_all', 'browser', '^/mainapi/browser/policy/user/info/all/$', '浏览器用户策略'),
            ('browser_policy_bookmark_info_all', 'browser', '^/mainapi/browser/policy/bookmark/info/all/$',
             '浏览器书签策略'),
            ('browser_policy_user_info', 'browser', '^/mainapi/browser/policy/user/info/$', '浏览器用户策略配置'),
            ('browser_policy_user_info_x', 'browser', '^/mainapi/browser/policy/user/info/[0-9]+/$',
             '浏览器用户策略配置'),
            ('browser_policy_user_default', 'browser', '^/mainapi/browser/policy/user/default/$', '浏览器用户默认策略'),
            ('browser_policy_user_default_x', 'browser', '^/mainapi/browser/policy/user/default/[0-9]+/$',
             '浏览器默认用户策略'),
            ('browser_policy_bookmark_info', 'browser', '^/mainapi/browser/policy/bookmark/info/$',
             '浏览器书签默认策略'),
            ('browser_policy_bookmark_info_x', 'browser', '^/mainapi/browser/policy/bookmark/info/[0-9]+/$',
             '浏览器书签默认策略'),
            ('browser_policy_bookmark_default', 'browser', '^/mainapi/browser/policy/bookmark/default/$',
             '浏览器书签默认策略'),
            ('browser_policy_bookmark_default_x', 'browser', '^/mainapi/browser/policy/bookmark/default/[0-9]+/$',
             '浏览器书签默认策略'),
            ('browser_policy_application_account', 'browser', '^/mainapi/browser/policy/application/account/$',
             '浏览器新标签页应用策略'),
            ('browser_policy_application_account_x', 'browser', '^/mainapi/browser/policy/application/account/[0-9]+/$',
             '浏览器新标签页应用策略'),
            ('browser_policy_application_info', 'browser', '^/mainapi/browser/policy/application/info/$',
             '浏览器新标签页应用策略配置'),
            ('browser_policy_application_info_x', 'browser', '^/mainapi/browser/policy/application/info/[0-9]+/$',
             '浏览器新标签页应用策略配置'),
            ('browser_policy_application_default', 'browser', '^/mainapi/browser/policy/application/default/$',
             '浏览器新标签页应用默认策略'),
            ('browser_policy_application_default_x', 'browser', '^/mainapi/browser/policy/application/default/[0-9]+/$',
             '浏览器新标签页应用默认策略'),
            ('browser_policy_user_account', 'browser', '^/mainapi/browser/policy/user/account/$', '浏览器用户策略'),
            ('browser_policy_user_account_x', 'browser', '^/mainapi/browser/policy/user/account/[0-9]+/$',
             '浏览器用户策略'),
            ('browser_policy_user_account_multiple_org_change_policy', 'browser',
             '^/mainapi/browser/policy/user/account/multiple-org-change-policy/$', '浏览器用户策略'),
            ('browser_policy_bookmark_account', 'browser', '^/mainapi/browser/policy/bookmark/account/$',
             '浏览器书签策略'),
            ('browser_policy_bookmark_account_x', 'browser', '^/mainapi/browser/policy/bookmark/account/[0-9]+/$',
             '浏览器书签策略'),
            ('browser_policy_bookmark_account_multiple_org_change_policy', 'browser',
             '^/mainapi/browser/policy/bookmark/account/multiple-org-change-policy/$', '浏览器书签策略'),
            ('ca_default', 'ca', '^/mainapi/ca/default/$', 'CA证书'),
            ('ca_default_base_info', 'ca', '^/mainapi/ca/default/base_info/$', 'CA证书状态'),
            ('ca_default_ca_info', 'ca', '^/mainapi/ca/default/ca_info/$', '默认CA证书信息'),
            ('ca_distribute', 'ca', '^/mainapi/ca/distribute$', 'CA证书'),
            ('ca_file_file', 'ca', '^/mainapi/ca/file/[^/]+$', '上传CA文件'),
            ('ca_info', 'ca', '^/mainapi/ca/info$', 'CA信息'),
            ('ca_info_id', 'ca', '^/mainapi/ca/info/[0-9]+$', 'CA信息'),
            ('ca_policy', 'ca', '^/mainapi/ca/policy$', '系统默认策略'),
            ('ca_policy_id', 'ca', '^/mainapi/ca/policy/[^/]+$', 'CA证书'),
            ('ca_switch_switch', 'ca', '^/mainapi/ca/switch/[^/]+$', '默认策略开关'),
            ('ca_default_x', 'ca', '^/mainapi/ca/default/[0-9]+/$', 'CA默认证书'),
            ('client_nfs_domain_x', 'client', '^/mainapi/client/nfs_domain_[^/]+/$', '下载文件'),
            ('clientlog_search_device_join_domain', 'logs', '^/mainapi/clientlog/search/device_join_domain/$',
             '终端加域日志'),
            ('clientlog_search_device_exit_domain', 'logs', '^/mainapi/clientlog/search/device_exit_domain/$',
             '终端退域日志'),
            ('clientlog_search_account_login', 'logs', '^/mainapi/clientlog/search/account_login/$',
             '终端用户登录登出日志'),
            ('clientlog_search_export_device_join_domain', 'logs',
             '^/mainapi/clientlog/search/export_device_join_domain/$', '终端加域日志'),
            ('clientlog_search_export_device_exit_domain', 'logs',
             '^/mainapi/clientlog/search/export_device_exit_domain/$', '终端退域日志'),
            ('clientlog_search_export_account_login', 'logs', '^/mainapi/clientlog/search/export_account_login/$',
             '终端用户登录登出日志'),
            ('device_sync_policy_x', 'device', '^/mainapi/device/sync/policy/[^/]+/$', '终端设备策略'),
            ('device_info_x_remote_addr', 'device', '^/mainapi/device/info/[0-9]+/remote_addr/$', 'ssh远程'),
            ('device_details_hardware_x', 'device', '^/mainapi/device/details-hardware/[0-9]+/$', '终端硬件详情'),
            ('device_details_basic_x', 'device', '^/mainapi/device/details-basic/[0-9]+$', '终端详情'),
            ('device_info_device_list', 'device', '^/mainapi/device/info/device_list/$', '终端列表信息'),
            ('device_ping', 'device', '^/mainapi/device/ping/$', '终端连接'),
            ('device_all', 'device', '^/mainapi/device/all/$', '部门全局开关'),
            ('device_get_client', 'device', '^/mainapi/device/get-client/$', '终端'),
            ('device_bind_accounts_x', 'device', '^/mainapi/device/bind_accounts/[0-9]+/$', '可登录域用户'),
            ('device_bind_accounts', 'device', '^/mainapi/device/bind_accounts/$', '终端绑定域用户'),
            ('device_details_basic', 'device', '^/mainapi/device/details-basic/$', '终端信息'),
            ('device_domain', 'device', '^/mainapi/device/domain/$', '终端加域'),
            ('device_domain_id', 'device', '^/mainapi/device/domain/S+/$', '终端退域'),
            ('device_info', 'device', '^/mainapi/device/info/$', '主动加域'),
            ('device_info_bind_device', 'device', '^/mainapi/device/info/bind_device/$', '用户绑定设备'),
            ('device_info_device_file_info', 'device', '^/mainapi/device/info/device_file_info/$', '下载终端文件'),
            ('device_info_device_switch', 'device', '^/mainapi/device/info/device_switch/$', '启停设备'),
            ('device_info_devices_import', 'device', '^/mainapi/device/info/devices_import/$',
             '终端主动加域导入模板数据'),
            (
            'device_info_devices_user_update', 'device', '^/mainapi/device/info/devices_user_update/$', '终端所属用户'),
            ('device_login', 'device', '^/mainapi/device/login/$', '终端上线'),
            ('device_sync_policy_id', 'device', '^/mainapi/device/sync/policy/S+/$', '终端策略'),
            ('filemanager_file', 'filemanager', '^/mainapi/filemanager/file/$', '上传文件'),
            ('filemanager_file_id', 'filemanager', '^/mainapi/filemanager/file/[0-9]+/$', '文件'),
            ('fileupload_sysios_chunk_upload', 'fileupload', '^/mainapi/fileupload/sysios/chunk_upload/$', '镜像文件分片上传'),
            ('fileupload_sysios_chunk_upload_complete', 'fileupload', '^/mainapi/fileupload/sysios/chunk_upload_complete/$', '镜像文件分片上传完成'),
            ('groups_group_x', 'groups', '^/mainapi/groups/group/[0-9]+/$', '组'),
            ('groups_group', 'groups', '^/mainapi/groups/group/$', '组操作'),
            ('groups_group_group_account_import', 'groups', '^/mainapi/groups/group/group_account_import/$',
             '终端分组用户模板导入'),
            ('groups_group_detail', 'groups', '^/mainapi/groups/group_detail/$', '组'),
            ('groups_group_detail_batch_delete', 'groups', '^/mainapi/groups/group_detail/batch_delete/$', '删除组'),
            ('groups_group_detail_groups_all', 'groups', '^/mainapi/groups/group_detail/groups_all/$', '嵌套分组'),
            ('index_stats_device_user', 'index', '^/mainapi/index/stats/device_user/$', '数据大屏:设备与用户'),
            ('index_stats_summary', 'index', '^/mainapi/index/stats/summary/$', '数据大屏:汇总'),
            ('index_stats_gch_schedule', 'index', '^/mainapi/index/stats/gch_schedule/$', '数据大屏:国产化推广进度'),
            ('index_stats_device_health', 'index', '^/mainapi/index/stats/device_health/$', '数据大屏:终端健康情况'),
            ('index_info_software_info', 'index', '^/mainapi/index/info/software_info/$', '首页软件信息'),
            ('index_info_usb_info', 'index', '^/mainapi/index/info/usb_info/$', '首页USB信息'),
            ('index_info_devices_online', 'index', '^/mainapi/index/info/devices_online/$', '首页在线终端'),
            ('index_info_sub_domain_forms', 'index', '^/mainapi/index/info/sub_domain_forms/$', '首页子域'),
            ('index_info_policy_info', 'index', '^/mainapi/index/info/policy_info/$', '首页策略信息'),
            ('index_info_index_count', 'index', '^/mainapi/index/info/index_count/$', '首页总数'),
            ('index_info_sub_domain', 'index', '^/mainapi/index/info/sub_domain/$', '首页子域'),
            ('index_info_task_info', 'index', '^/mainapi/index/info/task_info/$', '首页任务'),
            ('logs_data_export', 'logs', '^/mainapi/logs/data/export/$', '日志导出'),
            ('message_center_info', 'message_center', '^/mainapi/message_center/info/$', '消息详情'),
            ('message_center_info_target_module_content', 'message_center',
             '^/mainapi/message_center/info/target_module_content/$', '策略日志'),
            ('message_center_info_multiple_switch', 'message_center', '^/mainapi/message_center/info/multiple_switch/$',
             '消息操作'),
            ('org_info', 'org', '^/mainapi/org/info/$', '部门详情'),
            ('org_x', 'org', '^/mainapi/org/[0-9]+$', '部门'),
            ('org_tree', 'org', '^/mainapi/org/tree$', '部门树'),
            ('org_info_organ_info_count', 'org', '^/mainapi/org/info/organ_info_count/$', '部门信息'),
            ('org', 'org', '^/mainapi/org/$', '部门'),
            ('org_account_id', 'org', '^/mainapi/org/account/[0-9]+$', '移除部门全部成员'),
            ('org_batch_file', 'org', '^/mainapi/org/batch/[^/]+$', '导入部门'),
            ('org_batch_ldap_file', 'org', '^/mainapi/org/batch/ldap/[^/]+$', '导入部门AD数据源'),
            ('org_executive_id', 'org', '^/mainapi/org/executive/[0-9]+$', '部门主管'),
            ('org_info_change_organ_seq', 'org', '^/mainapi/org/info/change_organ_seq/$', '同级别部门的顺序'),
            ('org_info_organ_batch_operation', 'org', '^/mainapi/org/info/organ_batch_operation/$', '部门导出'),
            ('org_label', 'org', '^/mainapi/org/label/$', '部门标信息'),
            ('org_org_batch_account_operation', 'org', '^/mainapi/org/org_batch_account_operation$',
             '部门加入移除域用户'),
            ('org_org_batch_device_operation', 'org', '^/mainapi/org/org_batch_device_operation$', '部门加入移除设备'),
            ('patch_patch_status_policy_exe_totals', 'patch', '^/mainapi/patch/patch_status/policy_exe_totals/$',
             '策略执行总数'),
            ('patch_device', 'patch', '^/mainapi/patch/device/$', '补丁终端'),
            ('patch_sys_dict', 'patch', '^/mainapi/patch/sys_dict/$', '补丁应用系统维护接口'),
            ('patch_type_dict', 'patch', '^/mainapi/patch/type_dict/$', '补丁类型维护接口'),
            ('patch_source_test_ping', 'patch', '^/mainapi/patch/source/test_ping/$', '测试补丁源'),
            ('patch_source_x', 'patch', '^/mainapi/patch/source/[0-9]+/$', '补丁源'),
            ('patch_info_x', 'patch', '^/mainapi/patch/info/[0-9]+/$', '补丁详情'),
            ('patch_policy_x', 'patch', '^/mainapi/patch/policy/[0-9]+/$', '补丁策略详情'),
            ('patch_patch_status_policy_exe_details', 'patch', '^/mainapi/patch/patch_status/policy_exe_details/$',
             '补丁源执行情况'),
            ('patch_policy_duplicate', 'patch', '^/mainapi/patch/policy/duplicate/$', '复制补丁策略'),
            ('patch_policy_settings_x', 'patch', '^/mainapi/patch/policy_settings/[0-9]+/$', '补丁配置信息'),
            ('patch_device_device_2_xlsx', 'patch', '^/mainapi/patch/device/device_2_xlsx/$', '终端列表导出'),
            ('patch_info', 'patch', '^/mainapi/patch/info/$', '补丁源数据'),
            ('patch_info_patch_2_xlsx', 'patch', '^/mainapi/patch/info/patch_2_xlsx/$', '补丁'),
            ('patch_patch_file', 'patch', '^/mainapi/patch/patch_file/$', '补丁文件上传'),
            ('patch_policy', 'patch', '^/mainapi/patch/policy/$', '补丁策略'),
            ('patch_policy_batch_operation', 'patch', '^/mainapi/patch/policy/batch_operation/$', '启用禁用补丁策略'),
            ('patch_source', 'patch', '^/mainapi/patch/source/$', '补丁源'),
            ('patch_sys_dict_update_patch_sys', 'patch', '^/mainapi/patch/sys_dict/update_patch_sys/$', '补丁应用系统'),
            ('patch_type_dict_update_patch_type', 'patch', '^/mainapi/patch/type_dict/update_patch_type/$', '补丁类别'),
            ('policy_gpo', 'policy', '^/mainapi/policy/gpo/$', '组策略'),
            ('policy_gpo_x', 'policy', '^/mainapi/policy/gpo/[0-9]+/$', '组策略'),
            ('remote_assists_stop_remote_assist', 'remote_assists', '^/mainapi/remote_assists/stop_remote_assist/$', '远程协助'),
            ('remote_assists_check_remote_assist', 'remote_assists', '^/mainapi/remote_assists/check_remote_assist/$', '远程协助'),
            ('reports_data_hardwarelist', 'reports', '^/mainapi/reports/data/hardwarelist/$', '终端硬件报表'),
            ('remote_assists_remote_assist_x', 'remote_assists', '^/mainapi/remote_assists/remote_assist/[0-9]+/$',
             '远程策略'),
            ('remote_assists_remote_assist', 'remote_assists', '^/mainapi/remote_assists/remote_assist/$',
             'pk所指定的模型对象'),
            ('remote_assists_device_batch_operation', 'salt', '^/mainapi/remote_assists/device/batch_operation/$',
             '远程协助策略'),
            ('salt_jid_detail_x', 'salt', '^/mainapi/salt/jid-detail/[0-9]+$', '补丁终端'),
            ('software_sync_software_source', 'software', '^/mainapi/software/sync_software_source/$', '软件源同步'),
            ('system_terminal_check_hostname', 'system', '^/mainapi/system/terminal/check_hostname/$', '终端Hostname'),
            ('salt_device_policy_device_system_update', 'salt', '^/mainapi/salt/device_policy/device_system_update/$',
             '设备更新策略'),
            ('system_terminal', 'system', '^/mainapi/system/terminal/$', '终端系统配置'),
            ('salt_account_policy_batch_trusted_sites', 'salt', '^/mainapi/salt/account_policy/batch_trusted_sites/$',
             '浏览器用户可信任站点设置'),
            ('salt_account_policy_batch_browser_favorites', 'salt',
             '^/mainapi/salt/account_policy/batch_browser_favorites/$', '浏览器部门收藏夹设置'),
            ('software_test_software_source', 'software', '^/mainapi/software/test_software_source/$', '测试软件源'),
            ('salt_wallpaper_policy_x', 'salt', '^/mainapi/salt/wallpaper-policy/[0-9]+$', '壁纸策略'),
            ('salt_watermark_info_batch_operation', 'salt', '^/mainapi/salt/watermark_info/batch_operation/$',
             '水印设置'),
            ('salt_active_policy_info_batch_operation', 'salt', '^/mainapi/salt/active_policy_info/batch_operation/$',
             '系统激活策略'),
            ('salt_device_policy_batch_file_redirection', 'salt',
             '^/mainapi/salt/device_policy/batch_file_redirection/$', '部门批量文件重定向设置'),
            ('salt_account_policy_batch_desktop_theme', 'salt', '^/mainapi/salt/account_policy/batch_desktop_theme/$',
             '用户主题设置'),
            ('salt_device_policy_batch_system_updates', 'salt', '^/mainapi/salt/device_policy/batch_system_updates/$',
             '部门批量更新设置'),
            ('salt_device_policy_batch_screen_saver', 'salt', '^/mainapi/salt/device_policy/batch_screen_saver/$',
             '部门批量屏幕保护设置'),
            ('salt_device_policy_batch_software_runs', 'salt', '^/mainapi/salt/device_policy/batch_software_runs/$',
             '部门批量软件运行设置'),
            ('salt_account_policy_batch_login_logout', 'salt', '^/mainapi/salt/account_policy/batch_login_logout/$',
             '部门批量用户登录登出设置'),
            ('software_info_x', 'software', '^/mainapi/software/info/[0-9]+$', '软件'),
            ('salt_default_watermark_x', 'salt', '^/mainapi/salt/default_watermark/[0-9]+/$', '水印策略'),
            ('salt_active_policy_info_default_active', 'salt', '^/mainapi/salt/active_policy_info/default_active/$',
             '终端激活策略'),
            ('salt_account_policy_account_trusted_sites', 'salt',
             '^/mainapi/salt/account_policy/account_trusted_sites/$', '可信任站点策略'),
            ('salt_account_policy_account_browser_favorites', 'salt',
             '^/mainapi/salt/account_policy/account_browser_favorites/$', '游览器收藏夹策略'),
            ('salt_jid_all', 'salt', '^/mainapi/salt/jid-all$', '策略日志'),
            ('salt_watermark_info', 'salt', '^/mainapi/salt/watermark_info/$', '水印设置'),
            ('salt_active_policy_info', 'salt', '^/mainapi/salt/active_policy_info/$', '系统激活策略'),
            ('salt_device_policy_device_file_redirection', 'salt',
             '^/mainapi/salt/device_policy/device_file_redirection/$', '终端重定向策略'),
            ('salt_account_policy_account_desktop_theme', 'salt',
             '^/mainapi/salt/account_policy/account_desktop_theme/$', '桌面主题'),
            ('salt_device_policy_device_screen_saver', 'salt', '^/mainapi/salt/device_policy/device_screen_saver/$',
             '屏幕保护策略'),
            ('salt_device_policy_device_software_runs', 'salt', '^/mainapi/salt/device_policy/device_software_runs/$',
             '终端软件运行策略'),
            ('salt_account_policy_account_login_logout', 'salt', '^/mainapi/salt/account_policy/account_login_logout/$',
             '登入登出策略'),
            ('software_source_x', 'software', '^/mainapi/software/source/[0-9]+/$', '软件源'),
            ('salt_firewalld_device_batch_update', 'salt', '^/mainapi/salt/firewalld_device_batch_update/$',
             '启用禁用防火墙'),
            ('salt_policy_list_x', 'salt', '^/mainapi/salt/policy_list/[0-9]+/$', '策略信息'),
            ('salt_reboot_device', 'salt', '^/mainapi/salt/reboot-device/$', '设备关机重启策略'),
            ('salt_screensavers_x', 'salt', '^/mainapi/salt/screensavers/[0-9]+/$', '屏幕保护策略'),
            ('software_software_policy_x', 'software', '^/mainapi/software/software_policy/[0-9]+/$', '软件策略'),
            ('system_domain_rule_x', 'system', '^/mainapi/system/domain_rule/[0-9]+/$', '加域规则'),
            ('salt_active_policy', 'salt', '^/mainapi/salt/active-policy/$', '策略'),
            ('salt_firewalld', 'salt', '^/mainapi/salt/firewalld/$', '防火墙'),
            ('salt_firewalld_rules', 'salt', '^/mainapi/salt/firewalld_rules/$', '防火墙配置'),
            ('salt_firewalld_rules_multiple_delete', 'salt', '^/mainapi/salt/firewalld_rules/multiple_delete/$',
             '删除防火墙配置'),
            ('salt_firewalld_black_white_roster_multiple_set', 'salt',
             '^/mainapi/salt/firewalld-black-white-roster/multiple_set/$', '防火墙黑白名单'),
            ('salt_firewalld_device', 'salt', '^/mainapi/salt/firewalld-device$', '防火墙设备'),
            ('salt_kms_server', 'salt', '^/mainapi/salt/kms-server/$', 'KMS服务'),
            ('salt_policy_list', 'salt', '^/mainapi/salt/policy_list/$', '策略终端'),
            ('salt_policy_list_policy_change_device', 'salt', '^/mainapi/salt/policy_list/policy_change_device/$',
             '操作策略终端开启者关闭'),
            ('salt_policy_list_policy_switch', 'salt', '^/mainapi/salt/policy_list/policy_switch/$', '暂停'),
            ('salt_root_policy', 'salt', '^/mainapi/salt/root-policy/$', 'ROOT策略'),
            ('salt_screensavers', 'salt', '^/mainapi/salt/screensavers/$', '屏幕保护配置'),
            ('salt_screensavers_multiple_screensavers', 'salt', '^/mainapi/salt/screensavers/multiple_screensavers/$',
             '锁屏保护'),
            (
            'salt_screensavers_multiple_wallpaper', 'salt', '^/mainapi/salt/screensavers/multiple_wallpaper/$', '壁纸'),
            ('salt_ssh_device_batch_update', 'salt', '^/mainapi/salt/ssh_device_batch_update/$', 'ssh终端管理启用禁用'),
            ('salt_ssh_device', 'salt', '^/mainapi/salt/ssh-device$', 'ssh终端'),
            ('salt_ssh_policy', 'salt', '^/mainapi/salt/ssh-policy$', 'ssh策略'),
            ('salt_usb_device', 'salt', '^/mainapi/salt/usb-device$', '启停USB设备'),
            ('salt_usb_multiple', 'salt', '^/mainapi/salt/usb-multiple/$', '启停USB设备'),
            ('salt_usb_policy', 'salt', '^/mainapi/salt/usb-policy$', 'USB策略'),
            ('salt_wallpaper', 'salt', '^/mainapi/salt/wallpaper$', '壁纸'),
            ('salt_wallpaper_id', 'salt', '^/mainapi/salt/wallpaper/[0-9]+$', '壁纸'),
            ('salt_wallpaper_device', 'salt', '^/mainapi/salt/wallpaper-device$', '默认壁纸设备'),
            ('salt_wallpaper_policy', 'salt', '^/mainapi/salt/wallpaper-policy$', '默认壁纸策略'),
            ('software_add_software_task', 'software', '^/mainapi/software/add_software_task/$', '软件任务'),
            ('software_arch', 'software', '^/mainapi/software/arch/$', '软件适配架构'),
            ('software_batch', 'software', '^/mainapi/software/batch$', '软件安装'),
            ('software_deb_file', 'software', '^/mainapi/software/deb/[^/]+$', '上传deb包'),
            ('software_default_id', 'software', '^/mainapi/software/default/[0-9]+$', '默认安装软件'),
            ('software_info', 'software', '^/mainapi/software/info/$', '软件'),
            ('software_info_id', 'software', '^/mainapi/software/info/[0-9]+$', '软件'),
            ('software_os_x', 'software', '^/mainapi/software/os/$', '软件适配系统'),
            ('software_single', 'software', '^/mainapi/software/single$', '安装卸载软件'),
            ('software_software_batch_delete', 'software', '^/mainapi/software/software_batch_delete/$', '软件'),
            ('software_software_policy', 'software', '^/mainapi/software/software_policy/$', '软件策略'),
            ('software_software_policy_batch_delete', 'software', '^/mainapi/software/software_policy/batch_delete/$',
             '软件策略'),
            ('software_software_policy_copy_policy', 'software', '^/mainapi/software/software_policy/copy_policy/$',
             '复制软件策略'),
            ('software_software_policy_update_status', 'software', '^/mainapi/software/software_policy/update_status/$',
             '启用禁用软件策略'),
            ('software_source', 'software', '^/mainapi/software/source/$', '软件源'),
            ('software_source_source_update_status', 'software', '^/mainapi/software/source/source_update_status/$',
             '软件源启用禁用'),
            ('software_type', 'software', '^/mainapi/software/type/$', '软件类型'),
            ('software_policy', 'software', '^/mainapi/software/policy/$', '软件策略'),
            ('software_policy_x', 'software', '^/mainapi/software/policy/[0-9]+/$', '软件策略'),
            ('software_policy_batch_operation', 'software', '^/mainapi/software/policy/batch_operation/$',
             '批量修改软件策略'),
            ('system_config', 'system', '^/mainapi/system/config/$', '系统'),
            (
            'system_terminal_change_password', 'system', '^/mainapi/system/terminal/change_password/$', '终端账号密码'),
            (
            'system_terminal_multiple_org_root', 'system', '^/mainapi/system/terminal/multiple_org_root/$', '系统激活'),
            (
            'system_terminal_multiple_root', 'system', '^/mainapi/system/terminal/multiple_root/$', 'root权限开启关闭'),
            ('system_device_stats_x', 'system', '^/mainapi/system/device_stats/[0-9]+/$', '终端纳管进度统计'),
            ('system_device_opt_x', 'system', '^/mainapi/system/device_opt/[0-9]+/$', '客户端设置'),
            ('sysios_iso', 'sysios', '^/mainapi/sysios/iso/$', '镜像管理'),
            ('sysios_iso_all', 'sysios', '^/mainapi/sysios/iso/all/$', '镜像查询'),
            ('sysios_iso_x', 'sysios', '^/mainapi/sysios/iso/[0-9]+/$', '镜像管理'),
            ('sysios_file_upload', 'sysios', '^/mainapi/sysios/file/upload/$', '镜像文件上传'),
            ('sysios_file_download_x', 'sysios', '^/mainapi/sysios/file/download/(?P<file_id>[0-9]+)/$', '镜像文件下载'),
            ('sysios_iso_factory', 'sysios', '^/mainapi/sysios/iso/factory/$', '系统厂商管理'),
            ('sysios_iso_factory_save', 'sysios', '^/mainapi/sysios/iso/factory/save/$', '系统厂商保存'),
            ('sysios_iso_arch', 'sysios', '^/mainapi/sysios/iso/arch/$', '系统架构管理'),
            ('sysios_iso_arch_save', 'sysios', '^/mainapi/sysios/iso/arch/save/$', '系统架构保存'),
            ('sysios_iso_language', 'sysios', '^/mainapi/sysios/iso/language/$', '系统语言管理'),
            ('sysios_iso_language_save', 'sysios', '^/mainapi/sysios/iso/language/save/$', '系统语言保存'),
            ('sysios_iso_keyboard', 'sysios', '^/mainapi/sysios/iso/keyboard/$', '系统键盘布局管理'),
            ('sysios_iso_keyboard_save', 'sysios', '^/mainapi/sysios/iso/keyboard/save/$', '系统键盘布局保存'),
            ('sysios_iso_timezone', 'sysios', '^/mainapi/sysios/iso/timezone/$', '系统时区管理'),
            ('sysios_iso_timezone_save', 'sysios', '^/mainapi/sysios/iso/timezone/save/$', '系统时区保存'),
            ('tasks_task_x', 'tasks', '^/mainapi/tasks/task/[0-9]+/$', '任务'),
            ('tasks_task_detail_device', 'tasks', '^/mainapi/tasks/task_detail_device/$', '任务详情'),
            ('tasks_task_list', 'tasks', '^/mainapi/tasks/task_list/$', '任务'),
            ('tasks_schedule_task', 'tasks', '^/mainapi/tasks/schedule_task/$', '周期任务和启用关闭'),
            ('tasks_task', 'tasks', '^/mainapi/tasks/task/$', '任务'),
            ('tasks_task_id', 'tasks', '^/mainapi/tasks/task/[0-9]+/$', '任务'),
            ('tasks_task_device_exec', 'tasks', '^/mainapi/tasks/task_device_exec/$', '任务状态'),
            ('user_permission', 'user', '^/mainapi/user/permission/$', '访问权限'),
            ('user_info_x', 'user', '^/mainapi/user/info/[0-9]+/$', '管理员'),
            ('user_activationdomain', 'user', '^/mainapi/user/activationDomain/$', '激活服务器'),
            ('user_detail', 'user', '^/mainapi/user/detail/$', '用户信息'),
            ('user_info', 'user', '^/mainapi/user/info/$', '用户'),
            ('user_info_change_password', 'user', '^/mainapi/user/info/change_password/$', '用户密码'),
            ('user_login', 'user', '^/mainapi/user/login/$', '用户登录'),
            ('user_logout', 'user', '^/mainapi/user/logout/$', '用户登出'),
            ('user_multiple_change_password', 'user', '^/mainapi/user/multiple/change_password/$', '修改密码'),
            ('user_multiple_user_activate', 'user', '^/mainapi/user/multiple/user_activate/$', '启用禁用管理员'),
            ('user_multiple_user_delete', 'user', '^/mainapi/user/multiple/user_delete/$', '管理员'),
            ('user_multiple_user_passwd', 'user', '^/mainapi/user/multiple/user_passwd/$', '管理员密码'),
            ('user_multiple_user_update', 'user', '^/mainapi/user/multiple/user_update/$', '用户基本信息'),
            ('user_qrcodractivationdomain', 'user', '^/mainapi/user/QRCodrActivationDomain/$', '激活码激活服务器'),
            ('user_resetpassword', 'user', '^/mainapi/user/resetpassword/$', '重置密码'),
            ('user_useradminsystem', 'user', '^/mainapi/user/userAdminSystem/$', '系统管理员'),
            ('user_useradminsystem_id', 'user', '^/mainapi/user/userAdminSystem/S+$', '锁定解锁系统管理员'),
            ('user_info_my_info', 'user', '^/mainapi/user/info/my_info/$', '终端用户信息'),
            ('user_checkDomainLicense', 'user', '^/mainapi/user/checkDomainLicense/$', '服务器激活状态'),
            ('user_org_global_switch', 'user', '^/mainapi/user/org_global_switch/$', '部门全局开关'),
            ('user_set_org_global_switch', 'user', '^/mainapi/user/set_org_global_switch/$', '开关:仅展示部门直属终端'),
            ('wallpaper_x', 'wallpaper', '^/mainapi/wallpaper/[^/]+$', '上传壁纸')
        ])
        print("create menu info config")
        #
        time.sleep(5)

        # menu_sub_action
        truncate_menu_sub_action_sql = '''TRUNCATE TABLE `menu_sub_action`;'''
        # 先清空数据,再重新插入
        cursor.execute(truncate_menu_sub_action_sql)
        time.sleep(5)
        #
        create_menu_sub_action_sql = '''INSERT INTO `menu_sub_action` (`action`, `sub_action`, `sub_action_desc`) VALUES (%s, %s, %s);'''
        cursor.executemany(create_menu_sub_action_sql, [
            ('account_passwd_prohibit_mod_x', 'account_passwd_prohibit_mod_x_get', '查询规则:账号锁定'),
            ('account_passwd_prohibit_mod_x', 'account_passwd_prohibit_mod_x_post', '设置规则:账号锁定'),
            ('account_passwd_prohibit_mod_x', 'account_passwd_prohibit_mod_x_put', '设置规则:账号锁定'),
            ('account_passwd_prohibit_mod_x', 'account_passwd_prohibit_mod_x_delete', '删除规则:账号锁定'),
            ('account_passwd_prohibit_mod', 'account_passwd_prohibit_mod_get', '查询规则:账号锁定'),
            ('adminlog_search_actions', 'adminlog_search_actions_get', '查询日志:管理员行为列表'),
            ('adminlog_search_login', 'adminlog_search_login_get', '查询日志:管理员登录登出'),
            ('adminlog_search_org', 'adminlog_search_org_get', '查询:组织架构管理日志'),
            ('adminlog_search_account', 'adminlog_search_account_get', '查询:终端人员日志'),
            ('adminlog_search_device', 'adminlog_search_device_get', '查询:终端管理日志'),
            ('adminlog_search_software', 'adminlog_search_software_get', '查询:软件管理日志'),
            ('adminlog_search_patch', 'adminlog_search_patch_get', '查询:补丁管理日志'),
            ('adminlog_search_logs', 'adminlog_search_logs_get', '查询:日志管理日志'),
            ('adminlog_search_user', 'adminlog_search_user_get', '查询:平台管理日志'),
            ('adminlog_search_salt', 'adminlog_search_salt_get', '查询:策略管理日志'),
            ('adminlog_search_tasks', 'adminlog_search_tasks_get', '查询:任务管理日志'),
            ('adminlog_search_admins', 'adminlog_search_admins_get', '查询:管理员列表'),
            ('adminlog_search_export_login', 'adminlog_search_export_login_get', '日志导出:管理员登录登出'),
            ('adminlog_search_export_org', 'adminlog_search_export_org_get', '日志导出:组织架构管理日志'),
            ('adminlog_search_export_account', 'adminlog_search_export_account_get', '日志导出:终端人员日志'),
            ('adminlog_search_export_device', 'adminlog_search_export_device_get', '查日志导出询:终端管理日志'),
            ('adminlog_search_export_software', 'adminlog_search_export_software_get', '日志导出:软件管理日志'),
            ('adminlog_search_export_patch', 'adminlog_search_export_patch_get', '日志导出:补丁管理日志'),
            ('adminlog_search_export_logs', 'adminlog_search_export_logs_get', '日志导出:日志管理日志'),
            ('adminlog_search_export_user', 'adminlog_search_export_user_get', '日志导出:平台管理日志'),
            ('adminlog_search_export_salt', 'adminlog_search_export_salt_get', '日志导出:策略管理日志'),
            ('adminlog_search_export_tasks', 'adminlog_search_export_tasks_get', '日志导出:任务管理日志'),
            ('account_info', 'account_info_get', '查询:域账号'),
            ('account_info', 'account_info_post', '新增:域账号'),
            ('account_info', 'account_info_put', '修改:域账号'),
            ('account_info', 'account_info_delete', '删除:域账号'),
            ('account_info_account_list', 'account_info_account_list_get', '查询:域账号列表'),
            ('account_auth_source_prefix', 'account_auth_source_prefix_get', '查询:认证源前缀'),
            ('account_passwd_rule', 'account_passwd_rule_get', '查询:域用户密码权限校验'),
            ('account_passwd_rule', 'account_passwd_rule_post', '新增:域用户密码权限校验'),
            ('account_passwd_rule', 'account_passwd_rule_put', '修改:域用户密码权限校验'),
            ('account_passwd_rule', 'account_passwd_rule_delete', '删除:域用户密码权限校验'),
            ('account_black_white_lists', 'account_black_white_lists_get', '查询:域用户黑白名单'),
            ('account_black_white_lists', 'account_black_white_lists_post', '修改:域用户黑白名单'),
            ('adminlog_search', 'adminlog_search_get', '查询:管理员日志'),
            ('account_x', 'account_x_get', '查询:域用户'),
            ('account_x', 'account_x_post', '新增:域用户'),
            ('account_x', 'account_x_put', '修改:域用户'),
            ('account_x', 'account_x_delete', '删除:域用户'),
            ('account_ldap_x', 'account_ldap_x_get', '查询:AD数据源'),
            ('account_ldap_x', 'account_ldap_x_post', '新增:AD数据源'),
            ('account_ldap_x', 'account_ldap_x_put', '修改:AD数据源'),
            ('account_ldap_x', 'account_ldap_x_delete', '删除:AD数据源'),
            ('account_ldap_x_sync', 'account_ldap_x_sync_put', 'AD数据源迁移'),
            ('account_ldap_x_sync_detail', 'account_ldap_x_sync_detail_get', '查询:AD数据源迁移'),
            ('account_ldap_x_sync_detail_x', 'account_ldap_x_sync_detail_x_get', '查询:AD数据源迁移详情'),
            ('account_local_user_login_x', 'account_local_user_login_x_get', '查询:本地用户登录规则'),
            ('account_local_user_login_x', 'account_local_user_login_x_put', '设置:本地用户登录规则'),
            ('account_batch_tmpl_xlsx', 'account_batch_tmpl_xlsx_get', '下载模块'),
            ('account', 'account_get', '查询:域用户'),
            ('account', 'account_post', '新增:域用户'),
            ('account_auth_source_x', 'account_auth_source_x_get', '查询:外部源'),
            ('account_ldap', 'account_ldap_get', '查询:AD数据源'),
            ('account_ldap', 'account_ldap_post', '新增:AD数据源'),
            ('account_ldap', 'account_ldap_put', '修改:AD数据源'),
            ('account_ldap', 'account_ldap_delete', '删除:AD数据源'),
            ('account_soap_records_x', 'account_soap_records_x_get', '查询:SOAP数据同步记录'),
            ('account_info_account_batch_operation', 'account_info_account_batch_operation_put', '批量修改:域账号'),
            ('account_info_account_batch_operation', 'account_info_account_batch_operation_delete', '批量删除:域账号'),
            ('account_info_account_batch_operation', 'account_info_account_batch_operation_rest_passwd',
             '重置域账号密码'),
            ('account_info_account_batch_operation', 'account_info_account_batch_operation_enable', '启用域账号'),
            ('account_info_account_batch_operation', 'account_info_account_batch_operation_disable', '禁用域账号'),
            ('account_info_account_batch_operation', 'account_info_account_batch_operation_unlock', '解锁域账号'),
            ('account_info_account_batch_operation', 'account_info_account_batch_operation_org_put', '调整域账号部门'),
            ('account_info_account_batch_operation', 'account_info_account_batch_operation_down_file', '导出域账号'),
            ('account_passwd_rule_change_passwd_rule', 'account_passwd_rule_change_passwd_rule_put',
             '修改:域账号密码规则'),
            ('account_local_user_login_lock_local_user_action', 'account_local_user_login_lock_local_user_action_put',
             '锁定:本地用户'),
            ('account_local_user_login_unlock_local_user_action',
             'account_local_user_login_unlock_local_user_action_put', '解锁:本地用户'),
            ('account_local_user_login_update', 'account_local_user_login_update_put', '锁定或解锁:本地用户'),
            ('account_id', 'account_id_get', '查询:域用户详情'),
            ('account_id', 'account_id_put', '修改:域用户详情'),
            ('account_id', 'account_id_delete', '删除:域用户详情'),
            ('account_auth_source_create', 'account_auth_source_create_post', '新增:认证源'),
            ('account_auth_source_destroy', 'account_auth_source_destroy_delete', '删除:认证源'),
            ('account_auth_source_test_connection', 'account_auth_source_test_connection_put', '测试:认证源'),
            ('account_batch_file', 'account_batch_file_put', '批量导入:域账号'),
            ('account_batch_ldap_file', 'account_batch_ldap_file_put', '批量导入:AD数据源'),
            ('account_bind', 'account_bind_post', '终端绑定设备'),
            ('account_black_white_lists_change_status', 'account_black_white_lists_change_status_put',
             '修改:黑白名单状态'),
            ('account_black_white_lists_create', 'account_black_white_lists_create_post', '新增:黑白名单'),
            ('account_black_white_lists_multiple_delete', 'account_black_white_lists_multiple_delete_delete',
             '删除:黑白名单'),
            ('account_black_white_lists_multiple_update', 'account_black_white_lists_multiple_update_put',
             '修改:黑白名单'),
            ('account_ldap_multiple_delete', 'account_ldap_multiple_delete_delete', '批量删除:AD数据源'),
            ('account_ldap_ssl_file', 'account_ldap_ssl_file_post', '上传:SSL证书'),
            ('account_ldap_test_connection', 'account_ldap_test_connection_post', '测试:AD数据源'),
            ('account_passwd', 'account_passwd_put', '重置:域用户密码'),
            ('account_passwd_prohibit_mod_multiple_delete', 'account_passwd_prohibit_mod_multiple_delete_delete',
             '删除:域用户密码禁止模式'),
            ('account_passwd_prohibit_mod_update', 'account_passwd_prohibit_mod_update_put', '修改:域用户密码禁止模式'),
            ('account_passwd_rule_account_rule_info', 'account_passwd_rule_account_rule_info_get', '查询:账号规则设置'),
            ('account_passwd_rule_account_rule_info', 'account_passwd_rule_account_rule_info_put', '修改:账号规则设置'),
            ('account_passwd_rule_passwd_extra_account', 'account_passwd_rule_passwd_extra_account_put',
             '修改:列外域用户范围'),
            ('account_passwd_rule_passwd_extra_org', 'account_passwd_rule_passwd_extra_org_put', '修改:额外部门范围'),
            ('account_security', 'account_security_get', '查询:域用户安全配置'),
            ('account_security', 'account_security_put', '修改:域用户安全配置'),
            ('account_soap_records_records_batch_operation', 'account_soap_records_records_batch_operation_put',
             '批量修改:SOAP操作记录'),
            ('account_status', 'account_status_put', '修改:域用户状态'),
            ('account_backup_restore', 'account_backup_restore_get', '查询:AD数据源备份'),
            ('account_backup_restore', 'account_backup_restore_post', '新增:AD数据源备份'),
            ('account_dns_record', 'account_dns_record_get', '查询:DNS资源记录'),
            ('account_dns_record', 'account_dns_record_post', '新增:DNS资源记录'),
            ('account_backup_restore_callback', 'account_backup_restore_callback_post', '还原:AD数据源备份'),
            ('account_dns_record_multiple_delete', 'account_dns_record_multiple_delete_delete', '删除:DNS资源记录'),
            ('browser_policy_user_info', 'browser_policy_user_info_get', '查询:浏览器用户策略配置'),
            ('browser_policy_user_info', 'browser_policy_user_info_post', '新增:浏览器用户策略配置'),
            ('browser_policy_user_info_x', 'browser_policy_user_info_x_get', '查询:浏览器用户策略配置'),
            ('browser_policy_user_info_x', 'browser_policy_user_info_x_put', '修改:浏览器用户策略配置'),
            ('browser_policy_user_info_x', 'browser_policy_user_info_x_delete', '删除:浏览器用户策略配置'),
            ('browser_policy_user_default', 'browser_policy_user_default_get', '查询:浏览器默认用户策略'),
            ('browser_policy_user_default_x', 'browser_policy_user_default_x_put', '修改:浏览器默认用户策略'),
            ('browser_policy_bookmark_info', 'browser_policy_bookmark_info_get', '查询:浏览器书签策略配置列表'),
            ('browser_policy_bookmark_info', 'browser_policy_bookmark_info_post', '新增:浏览器书签策略配置'),
            ('browser_policy_bookmark_info_x', 'browser_policy_bookmark_info_x_get', '查询:浏览器书签策略配置详情'),
            ('browser_policy_bookmark_info_x', 'browser_policy_bookmark_info_x_put', '修改:浏览器书签策略配置'),
            ('browser_policy_bookmark_info_x', 'browser_policy_bookmark_info_x_delete', '删除:浏览器书签策略配置'),
            ('browser_policy_bookmark_default', 'browser_policy_bookmark_default_get', '查询:浏览器默认书签策略列表'),
            ('browser_policy_bookmark_default_x', 'browser_policy_bookmark_default_x_put', '修改:浏览器默认书签策略'),
            ('browser_policy_application_account', 'browser_policy_application_account_get', '查询:浏览器应用策略列表'),
            ('browser_policy_application_account', 'browser_policy_application_account_post', '新增:浏览器应用策略'),
            ('browser_policy_application_account_x', 'browser_policy_application_account_x_get',
             '查询:浏览器应用策略详情'),
            ('browser_policy_application_account_x', 'browser_policy_application_account_x_put', '修改:浏览器应用策略'),
            ('browser_policy_application_account_x', 'browser_policy_application_account_x_delete',
             '删除:浏览器应用策略'),
            ('browser_policy_application_info', 'browser_policy_application_info_get', '查询:浏览器应用策略配置列表'),
            ('browser_policy_application_info', 'browser_policy_application_info_post', '新增:浏览器应用策略配置'),
            ('browser_policy_application_info_x', 'browser_policy_application_info_x_get',
             '查询:浏览器应用策略配置详情'),
            ('browser_policy_application_info_x', 'browser_policy_application_info_x_put', '修改:浏览器应用策略配置'),
            (
            'browser_policy_application_info_x', 'browser_policy_application_info_x_delete', '删除:浏览器应用策略配置'),
            ('browser_policy_application_default', 'browser_policy_application_default_get', '查询:浏览器应用默认策略'),
            ('browser_policy_application_default_x', 'browser_policy_application_default_x_put',
             '修改:浏览器应用默认策略'),
            ('browser_policy_user_account', 'browser_policy_user_account_get', '查询:浏览器用户策略列表'),
            ('browser_policy_user_account', 'browser_policy_user_account_post', '新增:浏览器用户策略'),
            ('browser_policy_user_account_x', 'browser_policy_user_account_x_get', '查询:浏览器用户策略详情'),
            ('browser_policy_user_account_x', 'browser_policy_user_account_x_put', '修改:浏览器用户策略'),
            ('browser_policy_user_account_x', 'browser_policy_user_account_x_delete', '删除:浏览器用户策略'),
            ('browser_policy_bookmark_account', 'browser_policy_bookmark_account_get', '查询:浏览器书签策略'),
            ('browser_policy_bookmark_account', 'browser_policy_bookmark_account_post', '新增:浏览器书签策略'),
            ('browser_policy_bookmark_account_x', 'browser_policy_bookmark_account_x_get', '查询:浏览器书签策略详情'),
            ('browser_policy_bookmark_account_x', 'browser_policy_bookmark_account_x_put', '新增:浏览器书签策略'),
            ('browser_policy_bookmark_account_x', 'browser_policy_bookmark_account_x_delete', '新增:浏览器书签策略'),
            ('browser_policy_bookmark_account_multiple_org_change_policy',
             'browser_policy_bookmark_account_multiple_org_change_policy_put', '批量部门设置:浏览器书签策略'),
            ('browser_policy_user_account_multiple_org_change_policy',
             'browser_policy_user_account_multiple_org_change_policy_put', '部门批量设置:浏览器用户策略'),
            ('client_nfs_domain_x', 'client_nfs_domain_x_get', '下载文件'),
            ('ca_default', 'ca_default_get', '查询:CA证书'),
            ('ca_default', 'ca_default_post', '新增:CA证书'),
            ('ca_default_base_info', 'ca_default_base_info_get', '查询:CA证书信息状态'),
            ('ca_default_base_info', 'ca_default_base_info_put', '修改:CA证书状态'),
            ('ca_default_ca_info', 'ca_default_ca_info_put', '上传:默认CA证书'),
            ('ca_distribute', 'ca_distribute_get', '查询:下发CA证书'),
            ('ca_distribute', 'ca_distribute_post', '新增:下发CA证书'),
            ('ca_distribute', 'ca_distribute_put', '修改:下发CA证书'),
            ('ca_distribute', 'ca_distribute_delete', '删除:下发CA证书'),
            ('ca_file_file', 'ca_file_file_put', '上传:CA证书文件'),
            ('ca_info_id', 'ca_info_id_get', '查询:CA证书详情'),
            ('ca_info_id', 'ca_info_id_post', '新增:CA证书详情'),
            ('ca_info_id', 'ca_info_id_put', '修改:CA证书详情'),
            ('ca_info_id', 'ca_info_id_delete', '删除:CA证书详情'),
            ('ca_policy', 'ca_policy_get', '查询:CA证书策略'),
            ('ca_policy', 'ca_policy_put', '修改:CA证书策略'),
            ('ca_policy_id', 'ca_policy_id_get', '查询:CA证书策略详情'),
            ('ca_policy_id', 'ca_policy_id_put', '下发:CA证书'),
            ('ca_switch_switch', 'ca_switch_switch_get', '查询:CA策略查询策略'),
            ('ca_switch_switch', 'ca_switch_switch_put', '设置:CA策略查询策略'),
            ('ca_default_x', 'ca_default_x_put', '修改:CA默认证书'),
            ('ca_default_x', 'ca_default_x_delete', '删除:CA默认证书'),
            ('clientlog_search_device_join_domain', 'clientlog_search_device_join_domain_get', '查询:终端加域日志'),
            ('clientlog_search_device_exit_domain', 'clientlog_search_device_exit_domain_get', '查询:终端退域日志'),
            ('clientlog_search_account_login', 'clientlog_search_account_login_get', '查询:域用户登录登出日志'),
            ('clientlog_search_export_device_join_domain', 'clientlog_search_export_device_join_domain_get',
             '导出:终端加域日志'),
            ('clientlog_search_export_device_exit_domain', 'clientlog_search_export_device_exit_domain_get',
             '导出:终端退域日志'),
            ('clientlog_search_export_account_login', 'clientlog_search_export_account_login_get',
             '导出:域用户登录登出日志'),
            ('device_sync_policy_x', 'device_sync_policy_x_get', '查询:终端策略'),
            ('device_sync_policy_x', 'device_sync_policy_x_post', '新增:终端策略'),
            ('device_info_x_remote_addr', 'device_info_x_remote_addr_get', '查询:SSH远程'),
            ('device_details_hardware_x', 'device_details_hardware_x_get', '查询:终端硬件详情'),
            ('device_details_hardware_x', 'device_details_hardware_x_put', '修改:终端硬件详情'),
            ('device_details_basic_x', 'device_details_basic_x_get', '查询:终端详情'),
            ('device_details_basic_x', 'device_details_basic_x_post', '修改:终端详情'),
            ('device_info_device_list', 'device_info_device_list_get', '查询:终端列表信息'),
            ('device_ping', 'device_ping_get', '连接:终端'),
            ('device_all', 'device_all_get', '查询:终端列表'),
            ('device_get_client', 'device_get_client_get', '查询:终端配置'),
            ('device_bind_accounts', 'device_bind_accounts_get', '查询:可登录的域用户'),
            ('device_bind_accounts', 'device_bind_accounts_post', '新增:可登录的域用户'),
            ('device_bind_accounts_x', 'device_bind_accounts_xdelete', '删除:可登录的域用户'),
            ('device_details_basic', 'device_details_basic_get', '查询:终端详情'),
            ('device_details_basic', 'device_details_basic_post', '新增:终端详情'),
            ('device_domain', 'device_domain_get', '查询:已加域终端'),
            ('device_domain_id', 'device_domain_id_get', '查询:已加域终端详情'),
            ('device_domain_id', 'device_domain_id_post', '终端加域'),
            ('device_domain_id', 'device_domain_id_delete', '终端退域'),
            ('device_info', 'device_info_get', '查询:终端列表'),
            ('device_info', 'device_info_post', '终端主动加域'),
            ('device_info_bind_device', 'device_info_bind_device_get', '查询:可登录用户端列表'),
            ('device_info_bind_device', 'device_info_bind_device_post', '新增:可登录用户终端'),
            ('device_info_bind_device', 'device_info_bind_device_delete', '删除:可登录用户终端'),
            ('device_info_device_file_info', 'device_info_device_file_info_post', '下载:终端文件'),
            ('device_info_device_switch', 'device_info_device_switch_put', '批量修改:终端数据'),
            ('device_info_device_switch', 'device_info_device_switch_delete', '批量删除:终端数据'),
            ('device_info_device_switch', 'device_info_device_switch_enable', '启用终端'),
            ('device_info_device_switch', 'device_info_device_switch_disable', '禁用终端'),
            ('device_info_device_switch', 'device_info_device_switch_power_shutdown', '终端关机'),
            ('device_info_device_switch', 'device_info_device_switch_power_reboot', '终端重启'),
            ('device_info_device_switch', 'device_info_device_switch_org_put', '调整终端部门'),
            ('device_info_device_switch', 'device_info_device_switch_down_file', '导出终端'),
            ('device_info_devices_import', 'device_info_devices_import_put', '导入:终端主动加域模板数据'),
            ('device_info_devices_user_update', 'device_info_devices_user_update_put', '修改:终端所属用户'),
            ('device_login', 'device_login_post', '新增:终端上线'),
            ('device_sync_policy_id', 'device_sync_policy_id_get', '查询:终端策略'),
            ('device_sync_policy_id', 'device_sync_policy_id_post', '新增:终端策略'),
            ('filemanager_file', 'filemanager_file_get', '查询:上传的文件列表'),
            ('filemanager_file_id', 'filemanager_file_id_get', '查询:文件详情'),
            ('filemanager_file_id', 'filemanager_file_id_post', '上传:文件'),
            ('filemanager_file_id', 'filemanager_file_id_put', '修改:上传的文件'),
            ('filemanager_file_id', 'filemanager_file_id_delete', '删除:上传的文件'),
            ('fileupload_sysios_chunk_upload', 'fileupload_sysios_chunk_upload_post', '分片上传:镜像文件'),
            ('fileupload_sysios_chunk_upload_complete', 'fileupload_sysios_chunk_upload_complete_post', '分片上传:镜像文件上传完成'),
            ('groups_group', 'groups_group_get', '查询:组列表'),
            ('groups_group', 'groups_group_post', '新增:组'),
            ('groups_group_group_account_import', 'groups_group_group_account_import_put', '导入:终端分组用户模板'),
            ('groups_group_detail', 'groups_group_detail_get', '查询:组详情'),
            ('groups_group_detail', 'groups_group_detail_put', '修改:组'),
            ('groups_group_detail', 'groups_group_detail_delete', '删除:组'),
            ('groups_group_detail_batch_delete', 'groups_group_detail_batch_delete_delete', '删除:批量删除组'),
            ('groups_group_detail_groups_all', 'groups_group_detail_groups_all_post', '新增:嵌套分组'),
            ('index_stats_device_user', 'index_stats_device_user_get', '数据大屏:设备与用户'),
            ('index_stats_summary', 'index_stats_summary_get', '数据大屏:汇总'),
            ('index_stats_gch_schedule', 'index_stats_gch_schedule_get', '数据大屏:国产化推广进度'),
            ('index_stats_device_health', 'index_stats_device_health_get', '数据大屏:终端健康情况'),
            ('index_info_software_info', 'index_info_software_info_get', '查询:首页软件信息'),
            ('index_info_usb_info', 'index_info_usb_info_get', '查询:首页USB信息'),
            ('index_info_devices_online', 'index_info_devices_online_get', '查询:首页在线终端'),
            ('index_info_sub_domain_forms', 'index_info_sub_domain_forms_get', '查询:首页子域'),
            ('index_info_policy_info', 'index_info_policy_info_get', '查询:首页策略信息'),
            ('index_info_index_count', 'index_info_index_count_get', '查询:首页总数'),
            ('index_info_sub_domain', 'index_info_sub_domain_get', '查询:首页子域'),
            ('index_info_task_info', 'index_info_task_info_get', '查询:首页任务'),
            ('logs_data_export', 'logs_data_export_post', '导出:日志'),
            ('message_center_info', 'message_center_info_get', '查询:消息详情'),
            ('message_center_info_target_module_content', 'message_center_info_target_module_content_get',
             '查询:策略日志'),
            ('message_center_info_multiple_switch', 'message_center_info_multiple_switch_put', '批量修改:消息状态'),
            ('org_info', 'org_info_get', '查询:部门'),
            ('org_x', 'org_x_get', '查询:部门详情'),
            ('org_x', 'org_x_put', '修改:部门'),
            ('org_x', 'org_x_delete', '删除:部门'),
            ('org_tree', 'org_tree_get', '查询:部门树'),
            ('org_info_organ_info_count', 'org_info_organ_info_count_get', '查询:部门总数'),
            ('org', 'org_get', '查询:部门列表'),
            ('org', 'org_post', '新增:部门'),
            ('org_account_id', 'org_account_id_delete', '移除:部门全部成员'),
            ('org_batch_file', 'org_batch_file_put', '批量导入:部门'),
            ('org_batch_ldap_file', 'org_batch_ldap_file_put', '批量导入:部门AD数据源'),
            ('org_executive_id', 'org_executive_id_put', '修改:部门主管'),
            ('org_info_change_organ_seq', 'org_info_change_organ_seq_put', '修改:部门顺序'),
            ('org_info_organ_batch_operation', 'org_info_organ_batch_operation_put', '批量修改:部门数据'),
            ('org_info_organ_batch_operation', 'org_info_organ_batch_operation_delete', '批量删除:部门'),
            ('org_info_organ_batch_operation', 'org_info_organ_batch_operation_down_file', '导出:部门数据'),
            ('org_label', 'org_label_get', '查询:部门标签信息'),
            ('org_label', 'org_label_post', '新增:部门标签信息'),
            ('org_label', 'org_label_put', '修改:部门标签信息'),
            ('org_label', 'org_label_delete', '删除:部门标签信息'),
            ('org_org_batch_account_operation', 'org_org_batch_account_operation_put', '批量修改:添加或移除域用户'),
            ('org_org_batch_device_operation', 'org_org_batch_device_operation_put', '批量修改:添加或移除域用户终端'),
            ('patch_patch_status_policy_exe_totals', 'patch_patch_status_policy_exe_totals_get',
             '查询:补丁安装统计数据'),
            ('patch_device', 'patch_device_get', '查询:终端安装的补丁信息'),
            ('patch_sys_dict', 'patch_sys_dict_put', '更新:补丁应用系统数据'),
            ('patch_type_dict', 'patch_type_dict_put', '更新:补丁类别'),
            ('patch_source_test_ping', 'patch_source_test_ping_get', '测试:测试补丁源'),
            ('patch_info_x', 'patch_info_x_get', '查询:补丁详情'),
            ('patch_info_x', 'patch_info_x_post', '新增:补丁详情'),
            ('patch_info_x', 'patch_info_x_put', '修改:补丁详情'),
            # ('patch_info_x', 'patch_info_x_delete', '删除:补丁详情'),
            ('patch_patch_status_policy_exe_details', 'patch_patch_status_policy_exe_details_get',
             '查询:补丁策略执行状态'),
            ('patch_policy_duplicate', 'patch_policy_duplicate_get', '复制:补丁策略'),
            ('patch_policy_settings_x', 'patch_policy_settings_x_get', '查询:补丁策略下发设置'),
            ('patch_policy_settings_x', 'patch_policy_settings_x_post', '新增:补丁策略下发设置'),
            ('patch_policy_settings_x', 'patch_policy_settings_x_put', '修改:补丁策略下发设置'),
            ('patch_policy_settings_x', 'patch_policy_settings_x_delete', '删除:补丁策略下发设置'),
            ('patch_device_device_2_xlsx', 'patch_device_device_2_xlsx_put', '批量导出:终端补丁信息'),
            ('patch_info', 'patch_info_get', '查询:补丁列表'),
            ('patch_info', 'patch_info_post', '新增:补丁'),
            ('patch_info', 'patch_info_put', '修改:补丁'),
            ('patch_info_patch_2_xlsx', 'patch_info_patch_2_xlsx_delete', '删除:补丁'),
            ('patch_info_patch_2_xlsx', 'patch_info_patch_2_xlsx_down_file', '导出:补丁'),
            ('patch_patch_file', 'patch_patch_file_post', '上传:补丁文件'),
            ('patch_policy', 'patch_policy_get', '查询:补丁策略列表'),
            ('patch_policy', 'patch_policy_post', '新增:补丁策略'),
            ('patch_policy_x', 'patch_policy_x_get', '查询:补丁策略详情'),
            ('patch_policy_x', 'patch_policy_x_put', '修改:补丁策略'),
            ('patch_policy_x', 'patch_policy_x_delete', '删除:补丁策略'),
            ('patch_policy_batch_operation', 'patch_policy_batch_operation_put', '批量修改:补丁策略'),
            ('patch_policy_batch_operation', 'patch_policy_batch_operation_delete', '批量删除:补丁策略'),
            ('patch_policy_batch_operation', 'patch_policy_batch_operation_enable', '批量启用:补丁策略'),
            ('patch_policy_batch_operation', 'patch_policy_batch_operation_disable', '批量禁用:补丁策略'),
            ('patch_source', 'patch_source_get', '查询:补丁源列表'),
            ('patch_source', 'patch_source_post', '新增:补丁源'),
            ('patch_source_x', 'patch_source_x_get', '查询:补丁源详情'),
            ('patch_source_x', 'patch_source_x_put', '修改:补丁源'),
            ('patch_source_x', 'patch_source_x_delete', '删除:补丁源'),
            ('patch_sys_dict_update_patch_sys', 'patch_sys_dict_update_patch_sys_put', '修改:补丁适配系统'),
            ('patch_type_dict_update_patch_type', 'patch_type_dict_update_patch_type_put', '修改:补丁类别'),
            ('policy_gpo', 'policy_gpo_get', '查询:组策略列表'),
            ('policy_gpo', 'policy_gpo_post', '新增:组策略'),
            ('policy_gpo_x', 'policy_gpo_x_get', '查询:组策略详情'),
            ('policy_gpo_x', 'policy_gpo_x_put', '修改:组策略'),
            ('policy_gpo_x', 'policy_gpo_x_delete', '删除:组策略'),
            ('remote_assists_stop_remote_assist', 'remote_assists_stop_remote_assist_get', '关闭:远程协助'),
            ('remote_assists_check_remote_assist', 'remote_assists_check_remote_assist_get', '验证:远程协助'),
            ('reports_data_hardwarelist', 'reports_data_hardwarelist_get', '查询:终端设备硬件详情报表'),
            ('remote_assists_remote_assist', 'remote_assists_remote_assist_put', '获取:远程协助'),
            ('remote_assists_device_batch_operation', 'remote_assists_device_batch_operation_put', '修改:远程协助策略'),
            ('remote_assists_remote_assist_x', 'remote_assists_remote_assist_x_put', '修改:默认远程协助策略'),
            ('salt_jid_detail_x', 'salt_jid_detail_x_get', '查询:策略日志'),
            ('salt_screensavers', 'salt_screensavers_get', '查询:屏幕保护配置'),
            ('salt_screensavers', 'salt_screensavers_post', '新增:屏幕保护配置'),
            ('salt_screensavers', 'salt_screensavers_put', '修改:屏幕保护配置'),
            ('salt_screensavers', 'salt_screensavers_delete', '删除:屏幕保护配置'),
            ('salt_screensavers_multiple_screensavers', 'salt_screensavers_multiple_screensavers_put',
             '批量修改:锁屏保护设置'),
            ('salt_screensavers_multiple_wallpaper', 'salt_screensavers_multiple_wallpaper_put', '批量修改:壁纸设置'),
            ('salt_default_watermark_x', 'salt_default_watermark_x_get', '查询:默认水印设置'),
            ('salt_default_watermark_x', 'salt_default_watermark_x_post', '新增:默认水印设置'),
            ('salt_default_watermark_x', 'salt_default_watermark_x_put', '修改:默认水印设置'),
            ('salt_default_watermark_x', 'salt_default_watermark_x_delete', '删除:默认水印设置'),
            ('salt_active_policy_info_default_active', 'salt_active_policy_info_default_active_get',
             '查询:终端默认激活策略'),
            ('salt_active_policy_info_default_active', 'salt_active_policy_info_default_active_put',
             '修改:终端默认激活策略'),
            ('salt_account_policy_account_trusted_sites', 'salt_account_policy_account_trusted_sites_get',
             '查询:用户策略的可信任站点'),
            ('salt_account_policy_account_browser_favorites', 'salt_account_policy_account_browser_favorites_get',
             '查询:用户游览器收藏夹策略'),
            ('salt_jid_all', 'salt_jid_all_get', '查询:策略日志'),
            ('salt_watermark_info', 'salt_watermark_info_get', '查询:水印设置'),
            ('salt_watermark_info', 'salt_watermark_info_post', '新增:水印设置'),
            ('salt_watermark_info', 'salt_watermark_info_put', '修改:水印设置'),
            ('salt_watermark_info', 'salt_watermark_info_delete', '删除:水印设置'),
            ('salt_active_policy_info', 'salt_active_policy_info_get', '查询:系统激活策略'),
            ('salt_active_policy_info', 'salt_active_policy_info_post', '新增:系统激活策略'),
            ('salt_active_policy_info', 'salt_active_policy_info_put', '修改:系统激活策略'),
            ('salt_active_policy_info', 'salt_active_policy_info_delete', '删除:系统激活策略'),
            ('salt_device_policy_device_file_redirection', 'salt_device_policy_device_file_redirection_get',
             '查询:终端文件重定向策略'),
            ('salt_account_policy_account_desktop_theme', 'salt_account_policy_account_desktop_theme_get',
             '查询:用户桌面主题策略'),
            ('salt_device_policy_device_screen_saver', 'salt_device_policy_device_screen_saver_get',
             '查询:屏幕保护策略'),
            ('salt_device_policy_device_software_runs', 'salt_device_policy_device_software_runs_get',
             '查询:终端软件运行策略'),
            ('salt_account_policy_account_login_logout', 'salt_account_policy_account_login_logout_get',
             '查询:用户登入登出策略'),
            ('salt_device_policy_device_system_update', 'salt_device_policy_device_system_update_get',
             '查询:终端更新配置策略'),
            ('salt_account_policy_batch_trusted_sites', 'salt_account_policy_batch_trusted_sites_put',
             '批量修改:用户可信任站点策略'),
            ('salt_account_policy_batch_browser_favorites', 'salt_account_policy_batch_browser_favorites_put',
             '批量修改:游览器收藏夹策略'),
            ('salt_firewalld_device_batch_update', 'salt_firewalld_device_batch_update_post', '批量更新:防火墙策略'),
            ('salt_reboot_device', 'salt_reboot_device_get', '查询:终端重启策略'),
            ('salt_reboot_device', 'salt_reboot_device_post', '设置:终端重启策略'),
            ('salt_active_policy', 'salt_active_policy_get', '查询:激活策略'),
            ('salt_active_policy', 'salt_active_policy_post', '设置:激活策略'),
            ('salt_firewalld', 'salt_firewalld_get', '查询:防火墙策略'),
            ('salt_firewalld', 'salt_firewalld_post', '设置:防火墙策略'),
            ('salt_firewalld_rules', 'salt_firewalld_rules_get', '查询:防火墙规则'),
            ('salt_firewalld_rules', 'salt_firewalld_rules_post', '设置:防火墙规则'),
            ('salt_firewalld_rules', 'salt_firewalld_rules_put', '修改:防火墙规则'),
            ('salt_firewalld_rules', 'salt_firewalld_rules_delete', '删除:防火墙规则'),
            ('salt_firewalld_rules_multiple_delete', 'salt_firewalld_rules_multiple_delete_delete', '删除:防火墙规则'),
            ('salt_firewalld_black_white_roster_multiple_set', 'salt_firewalld_black_white_roster_multiple_set_get',
             '查询:防火墙黑白名单'),
            ('salt_firewalld_black_white_roster_multiple_set', 'salt_firewalld_black_white_roster_multiple_set_put',
             '设置:防火墙黑白名单'),
            ('salt_firewalld_device', 'salt_firewalld_device_get', '查询:设备的防火墙策略'),
            ('salt_firewalld_device', 'salt_firewalld_device_post', '设置:设备的防火墙策略'),
            ('salt_kms_server', 'salt_kms_server_get', '查询:KMS策略'),
            ('salt_kms_server', 'salt_kms_server_post', '设置:KMS策略'),
            ('salt_policy_list', 'salt_policy_list_get', '查询:策略列表'),
            ('salt_policy_list', 'salt_policy_list_post', '设置:策略'),
            ('salt_policy_list', 'salt_policy_list_put', '修改:策略启用状态'),
            ('salt_policy_list', 'salt_policy_list_delete', '删除:策略'),
            ('salt_policy_list_policy_change_device', 'salt_policy_list_policy_change_device_put',
             '批量修改:策略终端启用状态'),
            ('salt_policy_list_policy_switch', 'salt_policy_list_policy_switch_put', '批量修改:策略启用状态'),
            ('salt_root_policy', 'salt_root_policy_get', '查询:ROOT权限策略'),
            ('salt_root_policy', 'salt_root_policy_post', '设置:ROOT权限策略'),
            ('salt_root_policy', 'salt_root_policy_put', '修改:ROOT权限策略'),
            ('salt_root_policy', 'salt_root_policy_delete', '删除:ROOT权限策略'),
            ('salt_ssh_device_batch_update', 'salt_ssh_device_batch_update_post', '批量启用或禁用:终端SSH设置'),
            ('salt_ssh_device', 'salt_ssh_device_get', '查询:终端SSH设置'),
            ('salt_ssh_device', 'salt_ssh_device_post', '设置:终端SSH设置'),
            ('salt_usb_device', 'salt_usb_device_get', '查询:终端USB启停状态'),
            ('salt_usb_device', 'salt_usb_device_post', '设置:终端USB启停状态'),
            ('salt_usb_device', 'salt_usb_device_put', '批量设置:终端USB启停状态'),
            ('salt_usb_device', 'salt_usb_device_delete', '删除:终端USB启停状态'),
            ('salt_usb_multiple', 'salt_usb_multiple_post', '批量新增:终端USB设置'),
            ('salt_usb_multiple', 'salt_usb_multiple_put', '批量修改:终端USB设置'),
            ('salt_usb_multiple', 'salt_usb_multiple_delete', '批量删除:终端USB设置'),
            ('salt_usb_policy', 'salt_usb_policy_get', '查询:终端USB策略启用状态'),
            ('salt_usb_policy', 'salt_usb_policy_post', '设置:终端USB策略启用状态'),
            ('salt_usb_policy', 'salt_usb_policy_put', '批量设置:终端USB策略启用状态'),
            ('salt_usb_policy', 'salt_usb_policy_delete', '删除:终端USB策略启用状态'),
            ('salt_wallpaper', 'salt_wallpaper_get', '查询:壁纸'),
            ('salt_wallpaper', 'salt_wallpaper_post', '新增:壁纸'),
            ('salt_wallpaper_id', 'salt_wallpaper_id_put', '修改:壁纸'),
            ('salt_wallpaper_id', 'salt_wallpaper_id_delete', '删除:壁纸'),
            ('salt_wallpaper_device', 'salt_wallpaper_device_get', '查询:终端锁屏策略'),
            ('salt_wallpaper_device', 'salt_wallpaper_device_post', '设置:终端锁屏策略'),
            ('salt_wallpaper_device', 'salt_wallpaper_device_put', '批量设置:终端锁屏策略'),
            ('salt_wallpaper_device', 'salt_wallpaper_device_delete', '删除:终端锁屏策略'),
            ('salt_wallpaper_policy', 'salt_wallpaper_policy_get', '查询:终端壁纸策略'),
            ('salt_wallpaper_policy', 'salt_wallpaper_policy_post', '新增:终端壁纸策略'),
            ('salt_wallpaper_policy_id', 'salt_wallpaper_policy_id_put', '修改:终端壁纸策略'),
            ('salt_wallpaper_policy_id', 'salt_wallpaper_policy_id_delete', '删除:终端壁纸策略'),
            ('salt_watermark_info_batch_operation', 'salt_watermark_info_batch_operation_put', '批量修改:水印设置'),
            ('salt_active_policy_info_batch_operation', 'salt_active_policy_info_batch_operation_put',
             '批量修改:系统激活策略'),
            ('salt_device_policy_batch_file_redirection', 'salt_device_policy_batch_file_redirection_put',
             '批量修改:文件重定向策略'),
            ('salt_account_policy_batch_desktop_theme', 'salt_account_policy_batch_desktop_theme_put',
             '批量修改:桌面主题策略'),
            ('salt_device_policy_batch_system_updates', 'salt_device_policy_batch_system_updates_put',
             '批量修改:系统更新策略'),
            ('salt_device_policy_batch_screen_saver', 'salt_device_policy_batch_screen_saver_put',
             '批量修改:屏幕保护策略'),
            ('salt_device_policy_batch_software_runs', 'salt_device_policy_batch_software_runs_put',
             '批量修改:软件运行策略'),
            ('salt_account_policy_batch_login_logout', 'salt_account_policy_batch_login_logout_put',
             '批量修改:用户登录登出策略'),
            ('software_source', 'software_source_get', '查询:软件源'),
            ('software_source', 'software_source_post', '新增:软件源'),
            ('software_source', 'software_source_put', '修改:软件源'),
            ('software_source', 'software_source_delete', '删除:软件源'),
            ('software_source_source_update_status', 'software_source_source_update_status_put',
             '批量修改:软件源启用状态'),
            ('software_software_policy', 'software_software_policy_get', '查询:软件策略'),
            ('software_software_policy', 'software_software_policy_post', '新增:软件策略'),
            ('software_software_policy', 'software_software_policy_put', '修改:软件策略'),
            ('software_software_policy', 'software_software_policy_delete', '删除:软件策略'),
            ('software_software_policy_copy_policy', 'software_software_policy_copy_policy_get', '查询:复制软件策略'),
            ('software_software_policy_copy_policy', 'software_software_policy_copy_policy_post', '新增:复制软件策略'),
            ('software_software_policy_copy_policy', 'software_software_policy_copy_policy_put', '修改:复制软件策略'),
            (
            'software_software_policy_copy_policy', 'software_software_policy_copy_policy_delete', '删除:复制软件策略'),
            ('software_software_policy_update_status', 'software_software_policy_update_status_put',
             '批量修改:软件启用状态'),
            ('software_sync_software_source', 'software_sync_software_source_get', '同步:软件源'),
            ('software_test_software_source', 'software_test_software_source_get', '测试:软件源'),
            ('software_add_software_task', 'software_add_software_task_x_post', '新增:下发软件任务'),
            ('software_arch', 'software_arch_get', '查询:软件适配架构'),
            ('software_arch', 'software_arch_post', '新增:软件适配架构'),
            ('software_arch', 'software_arch_put', '修改:软件适配架构'),
            ('software_arch', 'software_arch_delete', '删除:软件适配架构'),
            ('software_batch', 'software_batch_post', '批量安装:软件'),
            ('software_deb_file', 'software_deb_file_put', '上传:DEB包'),
            ('software_default_id', 'software_default_id_get', '查询:软件默认安装策略'),
            ('software_info', 'software_info_get', '查询:软件列表'),
            ('software_info', 'software_info_post', '新增:软件'),
            ('software_info_id', 'software_info_id_get', '查询:软件'),
            ('software_info_id', 'software_info_id_put', '修改:软件'),
            ('software_info_id', 'software_info_id_delete', '删除:软件'),
            ('software_os_x', 'software_os_x_get', '查询:软件适配系统'),
            ('software_os_x', 'software_os_x_post', '新增:软件适配系统'),
            ('software_os_x', 'software_os_x_put', '修改:软件适配系统'),
            ('software_os_x', 'software_os_x_delete', '删除:软件适配系统'),
            ('software_single', 'software_single_post', '安装:软件'),
            ('software_single', 'software_single_put', '卸载:软件'),
            ('software_software_batch_delete', 'software_software_batch_delete_delete', '批量删除:软件'),
            ('software_type', 'software_type_get', '查询:软件类型'),
            ('software_type', 'software_type_post', '新增:软件类型'),
            ('software_type', 'software_type_put', '修改:软件类型'),
            ('software_type', 'software_type_delete', '删除:软件类型'),
            ('software_policy', 'software_policy_get', '查询:软件策略列表'),
            ('software_policy', 'software_policy_post', '新增:软件策略'),
            ('software_policy_x', 'software_policy_x_get', '查询:软件策略详情'),
            ('software_policy_x', 'software_policy_x_put', '修改:软件策略'),
            ('software_policy_x', 'software_policy_x_delete', '删除:软件策略'),
            ('software_policy_batch_operation', 'software_policy_batch_operation_put', '批量修改:软件策略'),
            ('software_policy_batch_operation', 'software_policy_batch_operation_delete', '批量删除:软件策略'),
            ('software_policy_batch_operation', 'software_policy_batch_operation_enable', '启用:软件策略'),
            ('software_policy_batch_operation', 'software_policy_batch_operation_disable', '禁用:软件策略'),
            ('system_domain_rule_x', 'system_domain_rule_x_get', '查询:加域规则'),
            ('system_domain_rule_x', 'system_domain_rule_x_post', '新增:加域规则'),
            ('system_domain_rule_x', 'system_domain_rule_x_put', '修改:加域规则'),
            ('system_domain_rule_x', 'system_domain_rule_x_delete', '删除:加域规则'),
            ('system_terminal', 'system_terminal_get', '查询:终端系统配置'),
            ('system_terminal', 'system_terminal_post', '新增:终端系统配置'),
            ('system_terminal', 'system_terminal_put', '修改:终端系统配置'),
            ('system_terminal', 'system_terminal_delete', '删除:终端系统配置'),
            ('system_terminal_check_hostname', 'system_terminal_check_hostname_get', '查询:终端Hostname'),
            ('system_terminal_check_hostname', 'system_terminal_check_hostname_post', '新增:终端Hostname'),
            ('system_terminal_check_hostname', 'system_terminal_check_hostname_put', '修改:终端Hostname'),
            ('system_terminal_check_hostname', 'system_terminal_check_hostname_delete', '删除:终端Hostname'),
            ('system_domain_rule_x', 'system_domain_rule_x_get', '查询:加域规则设置'),
            ('system_domain_rule_x', 'system_domain_rule_x_post', '新增:加域规则设置'),
            ('system_domain_rule_x', 'system_domain_rule_x_put', '修改:加域规则设置'),
            ('system_domain_rule_x', 'system_domain_rule_x_delete', '删除:加域规则设置'),
            ('system_config', 'system_config_get', '查询:系统配置'),
            ('system_config', 'system_config_post', '新增:系统配置'),
            ('system_config', 'system_config_put', '修改:系统配置'),
            ('system_config', 'system_config_delete', '删除:系统配置'),
            ('system_terminal_change_password', 'system_terminal_change_password_put', '修改:终端账号密码'),
            ('system_terminal_multiple_org_root', 'system_terminal_multiple_org_root_put',
             '批量修改:部门ROOT账号启用状态'),
            ('system_terminal_multiple_root', 'system_terminal_multiple_root_put', '批量修改:ROOT权限启用状态'),
            ('system_device_stats_x', 'system_device_stats_x_get', '查询:终端纳管进度统计'),
            ('system_device_stats_x', 'system_device_stats_x_put', '修改:终端纳管进度统计'),
            ('system_device_opt_x', 'system_device_opt_x_get', '查询:客户端设置'),
            ('system_device_opt_x', 'system_device_opt_x_put', '修改:客户端设置'),
            ('sysios_iso', 'sysios_iso_get', '查询:镜像列表'),
            ('sysios_iso_all', 'sysios_iso_all_get', '查询:已成功镜像列表'),
            ('sysios_iso', 'sysios_iso_post', '新增:镜像'),
            ('sysios_iso_x', 'sysios_iso_x_get', '查询:镜像详情'),
            ('sysios_iso_x', 'sysios_iso_x_put', '修改:镜像'),
            ('sysios_iso_x', 'sysios_iso_x_delete', '删除:镜像'),
            ('sysios_file_upload', 'sysios_file_upload_post', '上传:镜像文件'),
            ('sysios_file_download_x', 'sysios_file_download_x_get', '下载:镜像文件'),
            ('sysios_iso_factory', 'sysios_iso_factory_get', '查询:系统厂商列表'),
            ('sysios_iso_factory_save', 'sysios_iso_factory_save_post', '保存:系统厂商数据'),
            ('sysios_iso_arch', 'sysios_iso_arch_get', '查询:系统架构列表'),
            ('sysios_iso_arch_save', 'sysios_iso_arch_save_post', '保存:系统架构数据'),
            ('sysios_iso_language', 'sysios_iso_language_get', '查询:系统语言列表'),
            ('sysios_iso_language_save', 'sysios_iso_language_save_post', '保存:系统语言数据'),
            ('sysios_iso_keyboard', 'sysios_iso_keyboard_get', '查询:系统键盘布局列表'),
            ('sysios_iso_keyboard_save', 'sysios_iso_keyboard_save_post', '保存:系统键盘布局数据'),
            ('sysios_iso_timezone', 'sysios_iso_timezone_get', '查询:系统时区列表'),
            ('sysios_iso_timezone_save', 'sysios_iso_timezone_save_post', '保存:系统时区数据'),
            ('tasks_task', 'tasks_task_get', '查询:任务'),
            ('tasks_task', 'tasks_task_post', '新增:任务'),
            ('tasks_task_id', 'tasks_task_id_get', '查询:任务详情'),
            ('tasks_task_id', 'tasks_task_id_put', '修改:任务'),
            ('tasks_task_id', 'tasks_task_id_delete', '删除:任务'),
            ('tasks_task_detail_device', 'tasks_task_detail_device_get', '查询:任务终端列表'),
            ('tasks_task_list', 'tasks_task_list_get', '查询:任务列表'),
            ('tasks_schedule_task', 'tasks_schedule_task_post', '定时任务:启用或关闭周期任务'),
            ('tasks_task_device_exec', 'tasks_task_device_exec_put', '修改:任务状态'),
            ('user_activationdomain', 'user_activationdomain_post', '激活:服务器'),
            ('user_detail', 'user_detail_get', '查询:管理员详情'),
            ('user_detail', 'user_detail_put', '修改:管理员详情'),
            ('user_info', 'user_info_get', '查询:管理员列表'),
            ('user_info', 'user_info_post', '新增:管理员'),
            ('user_info_x', 'user_info_x_get', '查询:管理员详情'),
            ('user_info_x', 'user_info_x_put', '修改:管理员'),
            ('user_info_x', 'user_info_x_delete', '删除:管理员'),
            ('user_info_change_password', 'user_info_change_password_put', '修改:密码'),
            ('user_login', 'user_login_post', '用户登录'),
            ('user_logout', 'user_logout_post', '用户登出'),
            ('user_multiple_change_password', 'user_multiple_change_password_put', '修改:管理员密码'),
            ('user_multiple_user_activate', 'user_multiple_user_activate_put', '批量修改:管理员启用状态'),
            ('user_multiple_user_delete', 'user_multiple_user_delete_delete', '批量删除:管理员'),
            ('user_multiple_user_passwd', 'user_multiple_user_passwd_put', '批量修改:管理员密码'),
            ('user_multiple_user_update', 'user_multiple_user_update_put', '批量修改:管理员基本信息'),
            ('user_qrcodractivationdomain', 'user_qrcodractivationdomain_get', '查询:激活码激活的服务器'),
            ('user_qrcodractivationdomain', 'user_qrcodractivationdomain_post', '激活码激活:服务器'),
            ('user_resetpassword', 'user_resetpassword_post', '重置:密码'),
            ('user_checkDomainLicense', 'user_checkDomainLicense_get', '查询:激活码激活码状态'),
            ('user_org_global_switch', 'user_org_global_switch_get', '查询:部门直属人员或终端'),
            ('user_permission', 'user_permission_get', '查询:管理员权限'),
            ('user_permission', 'user_permission_post', '新增:管理员权限'),
            ('user_permission', 'user_permission_put', '修改:管理员权限'),
            ('user_permission', 'user_permission_delete', '删除:管理员权限'),
            ('user_set_org_global_switch', 'user_set_org_global_switch_get', '设置:打开或关闭部门直属人员或终端'),
            ('wallpaper_x', 'wallpaper_x_get', '查询:上传壁纸'),
            ('wallpaper_x', 'wallpaper_x_post', '新增:上传壁纸'),
            ('wallpaper_x', 'wallpaper_x_put', '修改:上传壁纸'),
            ('wallpaper_x', 'wallpaper_x_delete', '删除:上传壁纸')
        ])
        print("create menu sub action")
    except Exception as exp:
        print('init menu_info and menue_sub_action data err :', exp)


if __name__ == '__main__':
    if len(sys.argv) > 1 and sys.argv[1] == 'initdb':
        init_db()
    else:
        conn, cursor = conn_mysql()
        init_data(conn, cursor)
