# -*- coding: utf-8 -*-
from django.db import models

class SysiosIso(models.Model):
    is_config_choices = (
        (0, '关'), (1, '开')
    )
    is_domain_choices = (
        (0, '关'), (1, '开')
    )
    status_choices = (
        (0, '未执行'), (1, '成功'), (2, '执行中'), (3, '失败')
    )
    iso_type_choices = (
        (0, '本地上传的基础镜像'), (1, '基于基础镜像构建的镜像')
    )

    name = models.CharField(verbose_name='镜像名', max_length=255)
    sysios_factory = models.ForeignKey(to='SysiosFactory', on_delete=models.PROTECT, db_constraint=False,
                                 null=True, blank=True, verbose_name='厂商')
    sysios_arch = models.ForeignKey(to='SysiosArch', on_delete=models.PROTECT, db_constraint=False,
                                 null=True, blank=True, verbose_name='架构')
    is_config = models.IntegerField(verbose_name='系统是否自定义配置,0:关,1:开', choices=is_config_choices, default=0)
    sysios_config = models.ForeignKey(to='SysiosConfig', on_delete=models.PROTECT, db_constraint=False,
                                 null=True, blank=True, verbose_name='系统自定义配置')
    is_domain = models.IntegerField(verbose_name='系统是否自动加域,0:关,1:开', choices=is_domain_choices, default=0)
    sysios_domain = models.ForeignKey(to='SysiosDomain', on_delete=models.PROTECT, db_constraint=False,
                                      null=True, blank=True, verbose_name='系统的加域设置')
    sysios_config_filepath = models.TextField(verbose_name='系统自定义配置的json文件路径', null=True, blank=True)
    base_version = models.CharField(verbose_name='镜像基准版本', max_length=255, null=True, blank=True)
    remarks = models.TextField(verbose_name='备注', null=True, blank=True)
    source_iso_name = models.CharField(verbose_name='原镜像文件名', max_length=255, null=True, blank=True)
    source_iso_size = models.CharField(verbose_name='原镜像大小', max_length=255, null=True, blank=True)
    source_iso_path = models.TextField(verbose_name='原镜像文件存储路径,fastDFS中的路径',null=True, blank=True)
    iso_path = models.TextField(verbose_name='镜像文件存储路径,fastDFS中的路径',null=True, blank=True)
    iso_size = models.CharField(verbose_name='镜像文件存储路径,fastDFS中的路径',max_length=255, null=True, blank=True)
    status = models.IntegerField(verbose_name='策略状态:3:失败,1:成功,2:执行中', choices=status_choices, default=0)
    jid = models.CharField(verbose_name='salt jid',max_length=255, null=True, blank=True)
    creator = models.CharField(verbose_name='创建人',max_length=255, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    updator = models.CharField(verbose_name='修改人',max_length=255, null=True, blank=True)
    update_at = models.DateTimeField(verbose_name='修改时间', auto_now=True, null=True, blank=True)
    iso_type = models.IntegerField(verbose_name='镜像类型,0-本地上传,1-已构镜像', choices=iso_type_choices, default=0)

    class Meta:
        db_table = 'sysios_iso'
        verbose_name = '系统镜像表'
        verbose_name_plural = '系统镜像表'

        # 自定义索引
        indexes = [
            models.Index(fields=['name'], name='idx_sysios_iso_name'),
            models.Index(fields=['source_iso_name'], name='idx_sysios_iso_source_iso_name'),
        ]

    def __str__(self):
        return self.name

class SysiosFactory(models.Model):
    name = models.CharField(verbose_name='厂商名', max_length=255)
    remarks = models.TextField(verbose_name='厂商备注', null=True, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    update_at = models.DateTimeField(verbose_name='修改时间', auto_now=True, null=True, blank=True)

    class Meta:
        db_table = 'sysios_factory'
        verbose_name = '系统厂商表'
        verbose_name_plural = '系统厂商表'

        # 自定义索引
        indexes = [
            models.Index(fields=['name'], name='idx_sysios_factory_name'),
        ]

class SysiosArch(models.Model):
    name = models.CharField(verbose_name='架构名', max_length=255)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    update_at = models.DateTimeField(verbose_name='修改时间', auto_now=True, null=True, blank=True)

    class Meta:
        db_table = 'sysios_arch'
        verbose_name = '系统架构表'
        verbose_name_plural = '系统架构表'

        # 自定义索引
        indexes = [
            models.Index(fields=['name'], name='idx_sysios_arch_name'),
        ]


class SysiosConfig(models.Model):
    root_permission_choices = (
        (0, '关'), (1, '开')
    )
    is_active_choices = (
        (0, '关'), (1, '开')
    )
    after_deploy_choices = (
        (0, '关机'), (1, '重启')
    )
    local_user = models.CharField(verbose_name='本地用户', max_length=255)
    local_user_pwd = models.CharField(verbose_name='本地用户密码', max_length=255)
    root_permission = models.IntegerField(verbose_name='本地用户的root权限,0:关,1:开', choices=root_permission_choices, default=0)
    sysios_language = models.ForeignKey(to='SysiosLanguage', on_delete=models.PROTECT, db_constraint=False,
                                 default=1, blank=True, verbose_name='系统语言')
    sysios_keyboard = models.ForeignKey(to='SysiosKeyboard', on_delete=models.PROTECT, db_constraint=False,
                                 default=1, blank=True, verbose_name='系统键盘布局')
    sysios_timezone = models.ForeignKey(to='SysiosTimezone', on_delete=models.PROTECT, db_constraint=False,
                                 default=1, blank=True, verbose_name='系统时区')
    is_active = models.IntegerField(verbose_name='系统激活,0:关,1:开', choices=is_active_choices, default=0)
    active_url = models.CharField(verbose_name='系统激活地址', max_length=255, null=True, blank=True)
    after_deploy = models.IntegerField(verbose_name='部署后活动,0:关机,1:重启', choices=after_deploy_choices, default=0)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    update_at = models.DateTimeField(verbose_name='修改时间', auto_now=True, null=True, blank=True)

    class Meta:
        db_table = 'sysios_config'
        verbose_name = '系统配置表'
        verbose_name_plural = '系统配置表'

        # 自定义索引
        indexes = [
            models.Index(fields=['local_user'], name='idx_sysios_config_local_user'),
        ]


class SysiosLanguage(models.Model):
    name = models.CharField(verbose_name='语言名', max_length=255)
    value = models.CharField(verbose_name='语言', max_length=255, default='zh_CH')
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    update_at = models.DateTimeField(verbose_name='修改时间', auto_now=True, null=True, blank=True)

    class Meta:
        db_table = 'sysios_language'
        verbose_name = '系统语言表'
        verbose_name_plural = '系统语言表'

        # 自定义索引
        indexes = [
            models.Index(fields=['name'], name='idx_sysios_language_name'),
        ]

class SysiosKeyboard(models.Model):
    name = models.CharField(verbose_name='键盘布局名', max_length=255)
    value = models.CharField(verbose_name='键盘布局', max_length=255, default='default')
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    update_at = models.DateTimeField(verbose_name='修改时间', auto_now=True, null=True, blank=True)

    class Meta:
        db_table = 'sysios_keyboard'
        verbose_name = '系统键盘布局表'
        verbose_name_plural = '系统键盘布局表'

        # 自定义索引
        indexes = [
            models.Index(fields=['name'], name='idx_sysios_keyboard_name'),
        ]

class SysiosTimezone(models.Model):
    name = models.CharField(verbose_name='时区名', max_length=255)
    value = models.CharField(verbose_name='时区', max_length=255, default='Asia/Shanghai')
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    update_at = models.DateTimeField(verbose_name='修改时间', auto_now=True, null=True, blank=True)

    class Meta:
        db_table = 'sysios_timezone'
        verbose_name = '系统时区表'
        verbose_name_plural = '系统时区表'

        # 自定义索引
        indexes = [
            models.Index(fields=['name'], name='idx_sysios_timezone_name'),
        ]

class SysiosDrive(models.Model):
    drive_id = models.IntegerField(verbose_name='驱动id,不做外键(冗余字段)', null=True, blank=True)
    name = models.CharField(verbose_name='驱动名', max_length=255)
    sysios_iso = models.ForeignKey(to='SysiosIso', on_delete=models.PROTECT, db_constraint=False,
                                 default=1, blank=True, verbose_name='系统镜像')
    path = models.TextField(verbose_name='驱动路径', null=True, blank=True)

    class Meta:
        db_table = 'sysios_drive'
        verbose_name = '系统需要安装的驱动列表'
        verbose_name_plural = '系统需要安装的驱动列表'

        # 自定义索引
        indexes = [
            models.Index(fields=['name'], name='idx_sysios_drive_name'),
        ]

class SysiosSoftware(models.Model):
    is_uninstall_choices = (
        (0, '否'), (1, '是')
    )
    software_id = models.IntegerField(verbose_name='软件id,不做外键(冗余字段)', null=True, blank=True)
    name = models.CharField(verbose_name='软件名', max_length=255)
    sysios_iso = models.ForeignKey(to='SysiosIso', on_delete=models.PROTECT, db_constraint=False,
                                 default=1, blank=True, verbose_name='系统镜像')
    path = models.TextField(verbose_name='软件路径', null=True, blank=True)
    is_uninstall = models.IntegerField(verbose_name='是否防卸载,0:否,1:是', choices=is_uninstall_choices, default=0)

    class Meta:
        db_table = 'sysios_software'
        verbose_name = '系统需要安装的软件列表'
        verbose_name_plural = '系统需要安装的软件列表'

        # 自定义索引
        indexes = [
            models.Index(fields=['name'], name='idx_sysios_software_name'),
        ]

class SysiosDomain(models.Model):
    url = models.CharField(verbose_name='加域地址', max_length=255)
    admin_user = models.CharField(verbose_name='管理员账号', max_length=255)
    admin_passwd = models.CharField(verbose_name='管理员密码', max_length=255)

    class Meta:
        db_table = 'sysios_domain'
        verbose_name = '系统的加域配置'
        verbose_name_plural = '系统的加域配置'
