# -*- coding: utf-8 -*-
import random

from django.db import models


class SystemConfigs(models.Model):
    code = models.CharField(verbose_name='配置值', max_length=128, unique=True)
    value = models.CharField(verbose_name='配置值', max_length=1024)

    name = models.CharField(verbose_name='配置名称', max_length=128, null=True, blank=True)
    value_type = models.CharField(verbose_name='配置类型', max_length=1024, null=True, blank=True)
    description = models.CharField(verbose_name='描述信息', max_length=255, null=True, blank=True)
    remarks = models.CharField(verbose_name='备注', max_length=255, null=True, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_update = models.DateTimeField(verbose_name='最后修改时间', auto_now=True, null=True, blank=True)

    class Meta:
        db_table = 'system_config'
        verbose_name = '配置表'
        verbose_name_plural = '配置表'
        ordering = ('created_at',)

    def __str__(self):
        return self.code


class DomainRule(models.Model):
    sms_switch = models.SmallIntegerField(verbose_name='短信验证开关 0| 关闭 1|开启', default=0)
    hostname_switch = models.SmallIntegerField(verbose_name='主机名自动命名开关 0| 关闭 1|开启', default=0)
    hostname_first_type = models.SmallIntegerField(
        verbose_name='主机名一段 0| 部门； 1|自定义（小写字母）； 2|用户账号； 3|工号，4|IP地址； 5|MAC地址', null=True,
        blank=True)
    hostname_first_name = models.CharField(verbose_name='主机名一段名称', max_length=255, blank=True, null=True)
    hostname_second_type = models.SmallIntegerField(
        verbose_name='主机名二段 0| 部门； 1|自定义（小写字母）； 2|用户账号； 3|工号，4|IP地址； 5|MAC地址', null=True,
        blank=True)
    hostname_second_name = models.CharField(verbose_name='主机名二段名称', max_length=255, blank=True, null=True)
    edit_hostname_switch = models.SmallIntegerField(verbose_name='是否允许用户编辑主机名 0| 不允许 1|允许', default=0)
    other_org_label = models.CharField(verbose_name='其他部门标签（缩写）', max_length=255, blank=True, null=True)
    is_up_search = models.SmallIntegerField(verbose_name='是否搜索上级部门 0|非； 1|是；', default=1, null=True,
                                            blank=True)

    class Meta:
        db_table = 'sys_domain_rule'


class DeviceLimit(models.Model):
    is_limit_switch = models.SmallIntegerField(verbose_name='客户端限速开关 0|非； 1|是；', default=0, null=True, blank=True)
    limit_rate = models.SmallIntegerField(verbose_name='客户端限速网速；', null=True, blank=True)
    limit_conn_perip = models.SmallIntegerField(verbose_name='客户端最大下载连接；', null=True, blank=True)

    class Meta:
        db_table = 'system_device_limit'


class DeviceOTP(models.Model):
    code = models.CharField(max_length=6, editable=False)
    created_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        db_table = 'system_device_otp'


class DeviceStats(models.Model):
    is_switch = models.SmallIntegerField(verbose_name='终端纳管统计开关 0|禁用,1|启用', default=0, null=True, blank=True)
    stats_max = models.SmallIntegerField(verbose_name='终端纳管总数', null=True, blank=True)

    class Meta:
        db_table = 'system_device_stats'
        verbose_name = '终端纳管统计配置表'
        verbose_name_plural = '终端纳管统计配置表'
