from rest_framework import serializers

from accounts.models import AccountEntity
from devices.models import Devices
from organizations.models import OrgnizationEntity
from system.models import SystemConfigs, DomainRule, DeviceLimit, DeviceOTP, DeviceStats


class SystemConfigsSerializer(serializers.ModelSerializer):
    class Meta:
        model = SystemConfigs
        fields = '__all__'


class TerminalConfigSerializer(serializers.ModelSerializer):
    address = serializers.SerializerMethodField()
    org_info_name = serializers.SerializerMethodField()
    cn_name = serializers.SerializerMethodField()

    class Meta:
        model = AccountEntity
        fields = (
            'id', 'cn_name', 'name', 'address', 'org_info', 'org_info_name', 'is_root', 'system_status',
            'policy_status')

    def get_org_info_name(self, obj):
        org_info_items = OrgnizationEntity.objects.all()
        if org_info_items.filter(id=obj.org_info_id).exists():
            org_info_name = org_info_items.get(id=obj.org_info_id).name
        else:
            org_info_name = None

        return org_info_name

    def get_address(self, obj):
        device_itesms = Devices.objects.exclude(is_domain=0).all()
        if device_itesms.filter(accounts_id=obj.id).exists():
            address = device_itesms.filter(accounts_id=obj.id).first().device_ip
        else:
            address = None
        return address

    def get_cn_name(self, obj):
        device_itesms = Devices.objects.exclude(is_domain=0).all()
        if device_itesms.filter(accounts_id=obj.id).exists():
            cn_name = device_itesms.filter(accounts_id=obj.id).first().device_cnname
        else:
            cn_name = None
        return cn_name


class DomainRuleSerializer(serializers.ModelSerializer):
    class Meta:
        model = DomainRule
        fields = '__all__'


class DeviceLimitSerializer(serializers.ModelSerializer):
    class Meta:
        model = DeviceLimit
        fields = '__all__'

class DeviceOTPSerializer(serializers.ModelSerializer):
    class Meta:
        model = DeviceOTP
        fields = '__all__'


class DeviceStatsSerializer(serializers.ModelSerializer):
    class Meta:
        model = DeviceStats
        fields = '__all__'
