# -*- coding: utf-8 -*-
import datetime
from django.db import models
from users.models import User


class Tasks(models.Model):
    # 任务分类，0 脚本下发、1 系统补丁下发、2 文件下发、3软件下发、4证书下发
    task_class_choices = (
        (0, 'script'), (1, 'patch'), (2, 'file'), (3, 'software'), (4, "CA")
    )
    # 任务属性，0 立即执行、1 定时执行、2周期
    task_quality_choices = (
        (0, 'immediately'), (1, 'timing'), (2, 'cycle')
    )
    # 任务范围，0 按部门、1按终端、2 按分组, 3 组别， 4 ip段
    task_range_choices = (
        (0, 'organizations'), (1, 'devices'), (2, 'groups'), (3, 'user'), (4, 'ip_range')
    )
    # 状态，0 未开始、1暂停中、2执行中、3已取消、4已完成、5 失败
    task_status_choices = (
        (0, 'not start'), (1, 'stop'), (2, 'executing'), (3, 'canceled'), (4, 'completed'), (5, 'failed')
    )
    # 是否删除
    is_default = (
        (0, 'N'), (1, 'Y')
    )

    name = models.CharField(verbose_name='任务名称', max_length=128)
    task_class = models.IntegerField(verbose_name='任务分类', choices=task_class_choices, default=0, blank=True)
    file_ids_range = models.CharField(verbose_name='文件选择范围', max_length=256, null=True, blank=True)
    quality = models.IntegerField(verbose_name='任务属性', choices=task_quality_choices, default=0, blank=True)
    range = models.IntegerField(verbose_name='任务范围', choices=task_range_choices, default=0, blank=True)
    task_range = models.TextField(verbose_name='任务选择范围', null=True, blank=True)
    status = models.IntegerField(verbose_name='状态', choices=task_status_choices, default=0, blank=True)
    org_id = models.TextField(verbose_name='所属组织机构id', null=True, blank=True)
    groups_id = models.TextField(verbose_name='所属分组id', null=True, blank=True)
    # 存储时间戳字符串，兼容多个执行时间
    exec_time = models.CharField(verbose_name='执行时间', max_length=256, null=True, blank=True)
    last_exec_time = models.CharField(verbose_name='最近一次执行时间', max_length=256, null=True, blank=True)
    file_path = models.TextField(verbose_name='文件路径', null=True, blank=True)
    success_time = models.CharField(verbose_name='成功时间', max_length=256, null=True, blank=True)
    rate = models.CharField(verbose_name='完成率', max_length=100, null=True, blank=True)
    remarks = models.CharField(verbose_name='备注', max_length=100, null=True, blank=True)
    create_user = models.ForeignKey(to='users.User', on_delete=models.SET(1), db_constraint=False, null=True,
                                    blank=True, verbose_name='创建人')
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_update = models.DateTimeField(verbose_name='最后修改时间', auto_now=True, null=True, blank=True)
    is_deleted = models.IntegerField(verbose_name='是否是历史记录', choices=is_default, default=0, null=False, blank=True)
    need_reboot = models.IntegerField(verbose_name='是否需要重启 0|不需要， 1|需要', default=0, null=True, blank=True)
    need_prompt = models.IntegerField(verbose_name='是否立即安装 0|不需要， 1|需要', default=0, null=True,
                                      blank=True)
    install_complete_time = models.CharField(verbose_name='预计安装完成时间，默认为分钟', max_length=256, null=True,
                                             blank=True)
    is_software_source = models.IntegerField(verbose_name='是否是软件源任务 0|不是， 1|是', default=0, null=True,
                                             blank=True)

    class Meta:
        db_table = 'tasks_task'
        verbose_name = '任务'
        verbose_name_plural = '任务'
        ordering = ('created_at',)

    def __str__(self):
        return self.name


class TaskDeviceExec(models.Model):
    # 状态，0 未开始、1暂停中、2执行中、3已取消、4已完成、5 失败
    exec_status_choices = (
        (0, 'not start'), (1, 'stop'), (2, 'executing'), (3, 'canceled'), (4, 'completed'), (5, 'failed')
    )
    file_obj = models.CharField(verbose_name='任务选择的文件对象', max_length=255, null=True, blank=True)
    task = models.ForeignKey(to='tasks.Tasks', on_delete=models.PROTECT, db_constraint=False, null=True,
                             blank=True, verbose_name='任务')
    device = models.ForeignKey(to='devices.Devices', on_delete=models.PROTECT, db_constraint=False, null=True,
                               blank=True, verbose_name='终端')
    exec_status = models.IntegerField(verbose_name='执行状态', choices=exec_status_choices, default=0, blank=True)
    exec_time = models.CharField(verbose_name='执行时间', max_length=128, null=True, blank=True)
    jid = models.CharField(verbose_name='jid', blank=True, max_length=255, null=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_update = models.DateTimeField(verbose_name='最后修改时间', auto_now=True, null=True, blank=True)

    class Meta:
        db_table = 'task_device_exec'
        verbose_name = '任务设备执行表'
        verbose_name_plural = '任务设备执行表'
        ordering = ('created_at',)

    def __str__(self):
        return self.task.name
