import datetime
import logging
import os
import tarfile
import time

from django.db.models import Q
from saltapi import tasks as saltapi_task
from devices.serializers import DeviceSerializer
from tasks.models import TaskDeviceExec, Tasks
from filemanager.models import FileManager

logger = logging.getLogger(__name__)


def start_task(task_id, device_id=None):
    """
    启动单任务
    :param task:
    :return:
    """
    task = Tasks.objects.filter(pk=int(task_id), is_deleted=0).first()
    if not task:
        return False
    if task.quality == 0:
        task.exec_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        task.save()
    task_device_execs = TaskDeviceExec.objects.filter(task=task)
    if not task_device_execs:
        # 兼容部门下无设备
        return False
    # 过滤已经成功的
    task_device_execs = task_device_execs.filter(~Q(exec_status=4))

    device_list = [x.device for x in task_device_execs if x]
    if device_id:
        device_list = [x for x in device_list if x.pk == device_id]
    device_list = list(set(device_list))
    device_id_str = ",".join([str(x.id) for x in device_list])
    logging.info("task %s device_id %s" % (task_id, device_id_str))
    device_data = DeviceSerializer(device_list, many=True).data

    if task.task_class == 2:
        # 兼容多文件下发。文件下发允许多个，需要压缩后解压
        local_path = get_tar_file(task)
        #  download_path = get_tar_file(task)
        logging.info("task local_path %s " % local_path)
        # logging.info("task download_path %s " % download_path)

        local_name_list = local_path.split("file_data")
        tmp_file_name = os.path.basename(local_path)

        param = {
            "devices": device_data,
            "device_id_str": device_id_str,
            "task_id": task_id,
            "task_name": "%s_%s" % (str(task_id), str(int(time.time()))),
            "file_name": local_name_list[1],
            "tmp_file_name": tmp_file_name,
            "quality": task.quality
        }
    elif task.task_class == 3:
        # 软件分为本地源软件和软件源软件
        file_list = FileManager.objects.filter(task__pk=task_id)
        software_id_list = []
        local_software_list = []
        for file in file_list:
            if file.software_id:
                # 软件源软件
                software_id_list.append(file.software_id)
            else:
                # 本地源软件
                # path /opt/domain/nfs-domain-mainserver/docker/salt-docker/srv/salt/file_system/file_data/3/25/2023-03-26/linuxqq_2.0.0-b2-1089_amd64.deb
                local_name_list = file.path.split("file_data")
                path = local_name_list[1]
                name = os.path.basename(path)
                local_software_list.append({
                    "path": path,
                    "name": name
                })

            param = {
                "devices": device_data,
                "task_id": task_id,
                "device_id_str": device_id_str,
                "local_software_list": local_software_list,
                "software_id_list": software_id_list,
                "quality": task.quality
            }

    else:
        # 脚本、补丁、证书只支持单文件
        file = FileManager.objects.filter(task__pk=task_id).first()
        local_name_list = file.path.split("file_data")
        local_path = local_name_list[1]
        tmp_file_name = os.path.basename(local_path)
        param = {
            "devices": device_data,
            "task_id": task_id,
            "device_id_str": device_id_str,
            "file_name": local_path,
            "tmp_file_name": tmp_file_name,
            "task_class": task.task_class,
            "quality": task.quality
        }
    logging.info("task.task_class %s" % task.task_class)
    logging.info("task_id %s" % task_id)
    # logging.info("param %s" % param)

    if task.quality == 1:
        # 定时执行
        # 定时执行，时间为时间戳，需要转为时间对象
        exec_time = datetime.datetime.strptime(task.exec_time, "%Y-%m-%d %H:%M:%S").strftime('%Y-%m-%dT%H:%M:%S')
        logging.info("task %s exec_time %s" % (task_id, exec_time))
        param["exec_time"] = exec_time
        if task.task_class == 0:
            # 0 脚本下发
            celery_task = saltapi_task.task_shell_policy.delay(**param)
        elif task.task_class == 1:
            # 1 系统补丁下发
            celery_task = saltapi_task.task_system_patches_policy.delay(**param)
        elif task.task_class == 2:
            # 2 文件下发
            celery_task = saltapi_task.task_file_policy.delay(**param)
        elif task.task_class == 3:
            # 3 软件下发
            celery_task = saltapi_task.task_software_policy.delay(**param)
        else:
            # 4 证书下发
            celery_task = saltapi_task.task_ca_policy.delay(**param)


    elif task.quality == 2:
        del param["devices"]

        # 周期执行
        param["crontab_str"] = task.exec_time
        crontab_str = task.exec_time
        crontab_list = crontab_str.split(' ')
        param["crontab_list"] = crontab_list
        param["exec_time"] = task.exec_time

        if task.task_class == 0:
            # 0 脚本下发
            celery_task = saltapi_task.task_shell_policy.delay(**param)
        elif task.task_class == 1:
            # 1 系统补丁下发
            celery_task = saltapi_task.task_system_patches_policy.delay(**param)
        elif task.task_class == 2:
            # 2 文件下发
            celery_task = saltapi_task.task_file_policy.delay(**param)
        elif task.task_class == 3:
            # 3 软件下发
            celery_task = saltapi_task.task_software_policy.delay(**param)
        else:
            # 4 证书下发
            celery_task = saltapi_task.task_ca_policy.delay(**param)
    else:
        # 立即执行
        if task.task_class == 0:
            # 0 脚本下发 shellname, path, arg
            celery_task = saltapi_task.task_shell_policy.delay(**param)

        elif task.task_class == 1:
            # 1 系统补丁下发
            celery_task = saltapi_task.task_system_patches_policy.delay(**param)

        elif task.task_class == 2:
            # 2 文件下发
            celery_task = saltapi_task.task_file_policy.delay(**param)

        elif task.task_class == 3:
            # 3 软件下发
            celery_task = saltapi_task.task_software_policy.delay(**param)

        else:
            # 4 证书下发
            celery_task = saltapi_task.task_ca_policy.delay(**param)
    task.status = 2
    task.save()
    return True


def get_tar_file(task):
    """
    根据任务将多个文件合并为一个tar文件，返回本地路径和下发路径
    :param task:
    :return:
    """
    task_id = task.id
    # 多文件压缩为一个文件拷贝,然后解压
    file = FileManager.objects.filter(task__pk=task_id)
    file_path_list = [x.path for x in file if file]

    dir_path = os.path.dirname(file_path_list[0])
    tar_name = "%s.tar" % (str(task_id))
    local_file_path = os.path.join(dir_path, tar_name)

    with tarfile.open(local_file_path, "w") as tf:
        for file_path in file_path_list:
            file_name = os.path.basename(file_path)
            tf.add(file_path, arcname=file_name)
    if "/opt/domain" in local_file_path:
        local_file_path = local_file_path.replace("/opt/domain", "/opt")
    return local_file_path


def schedule_task_action(tasks, action):
    for task in tasks:
        # 周期任务和定时任务删除后台执行，立即执行没有
        if task.quality == 0:
            continue
        task_device_execs = TaskDeviceExec.objects.filter(task=task)
        device_hostname_list = [x.device.device_hostname for x in task_device_execs if x]

        param = {
            "task_id": task.id,
            "device_hostname_list": device_hostname_list,
        }
        if action == "disable":
            saltapi_task.schedule_task_stop_policy.delay(**param)
        elif action == "delete":
            saltapi_task.schedule_task_delete_policy.delay(**param)
        else:
            saltapi_task.schedule_task_start_policy.delay(**param)
