# -*- coding: utf-8 -*-

import uuid
import hmac
import datetime
import time
from hashlib import sha1
from django.db import models

from organizations.models import OrgnizationEntity


class Token(models.Model):
    """
    The default authorization token model.
    """
    key = models.CharField(verbose_name='token', max_length=255, primary_key=True, blank=True)
    username = models.CharField(verbose_name='登录名', max_length=255, unique=True, blank=True)

    # last_login_ip = models.GenericIPAddressField(null=True, default=None)
    # created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True)
    # last_accessed = models.DateTimeField(verbose_name='最后访问', blank=True)

    class Meta:
        verbose_name = 'Token'
        verbose_name_plural = 'Token'

    # def save(self, *args, **kwargs):
    #     if not self.key:
    #         self.key = self.generate_key()
    #     return super(Token, self).save(*args, **kwargs)
    #
    # def generate_key(self):
    #     unique = str(uuid.uuid4())
    #     return hmac.new(unique, digestmod=sha1).hexdigest()

    # def __unicode__(self):
    #     return self.key

    objects = models.Manager()

    def __str__(self):
        return self.key


class Permission(models.Model):
    """
    功能权限:目录,菜单,接口
    """
    menu_type_choices = (
        ('目录', '目录'),
        ('菜单', '菜单'),
    )
    name = models.CharField('名称', max_length=30)
    type = models.CharField('类型', max_length=20, choices=menu_type_choices, default='接口')
    parent = models.ForeignKey('self', null=True, blank=True, on_delete=models.SET_NULL, verbose_name='父')
    method = models.CharField('方法/代号', max_length=50, unique=True, null=True, blank=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = '功能权限表'
        verbose_name_plural = verbose_name

class User(models.Model):
    user_type = (
        (0, 'Default'), (1, 'Admin')
    )
    user_active = (
        (0, 'NotActive'), (1, 'Active')
    )
    super_user = (
        (0, 'NO'), (1, 'YES')
    )
    netdom = (
        (0, 'NO'), (1, 'YES')
    )
    org_global_switch_choice = (
        (0, 'N'), (1, 'Y')
    )
    nickname = models.CharField(verbose_name='用户昵称', max_length=255, null=True, blank=True)
    username = models.CharField(verbose_name='登录名', max_length=255, unique=True, blank=True)
    phone = models.CharField(verbose_name='手机号', max_length=255, null=True, blank=True)
    password = models.CharField(verbose_name='密码', max_length=255, blank=True)
    is_staff = models.IntegerField(verbose_name='是否管理', choices=user_type, default=0, blank=True)
    is_active = models.IntegerField(verbose_name='是否锁定', choices=user_active, default=1, blank=True)
    is_superuser = models.IntegerField(verbose_name='是否超级管理员', choices=super_user, default=0, blank=True)
    perms = models.ManyToManyField(Permission, blank=True, verbose_name='功能权限')
    org_perms = models.ManyToManyField(OrgnizationEntity, blank=True, verbose_name='部门权限')
    is_netdom = models.IntegerField(verbose_name='是否加域', choices=netdom, default=0, blank=True)
    role_name = models.CharField(verbose_name='角色名称', max_length=255, null=True, blank=True)
    created_at = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, blank=True)
    last_login_ip = models.GenericIPAddressField(null=True, default=None)
    last_accessed = models.DateTimeField(verbose_name='最后访问', null=True, blank=True)
    email = models.CharField(verbose_name='邮箱', max_length=255, null=True, blank=True)
    remark = models.CharField(verbose_name='备注', max_length=255, null=True, blank=True)
    org_global_switch_user = models.IntegerField(verbose_name='仅展示部门直属用户全局开关,默认0关，关显示所有成员，开只显示部门下成员', choices=org_global_switch_choice, default=0, blank=True)
    org_global_switch_device = models.IntegerField(verbose_name='仅展示部门直属终端全局开关,默认0关，关显示所有成员，开只显示部门下成员', choices=org_global_switch_choice, default=0, blank=True)

    class Meta:
        verbose_name = '用户'
        verbose_name_plural = '用户'

    objects = models.Manager()

    def freeze_user(self):
        self.is_active = 1
        self.save()

    def unfreeze_user(self):
        self.is_active = 0
        self.save()

    def make_password(self, password):
        sha1_obj = sha1()
        sha1_obj.update(str(password).encode('utf-8'))
        self.password = sha1_obj.hexdigest()  # 明文密码经过加密处理
        self.save()

    def check_password(self, password):
        sha1_obj = sha1()
        sha1_obj.update(str(password).encode('utf-8'))
        if self.password != sha1_obj.hexdigest():
            return False
        return True

    def update_password(self, password):
        self.make_password(password)

    def update_nickname(self, nickname):
        self.nickname = nickname
        self.save()

    def is_authenticated(self):
        return True

    def __str__(self):
        return self.username
