# -*- coding: utf-8 -*-
from copy import deepcopy
import logging

from django.conf import settings
from django.core.cache import cache

from organizations.models import OrgnizationEntity
from users.models import Permission

# Get an instance of a logger
logger = logging.getLogger(__name__)


def get_permission_list(user):
    """
    获取权限列表,可用redis存取
    """
    if user is None:
        return []
    if user.is_superuser == 1:
        perms_list = ['superuser']
    else:
        perms = Permission.objects.none()
        user_perms = user.perms.all()
        if user_perms:
            perms = perms | user_perms
        perms_list = perms.values_list('method', flat=True)
        perms_list = list(set(perms_list))
    cache.set(user.username + '__perms', perms_list)
    return perms_list


def get_org_permission_list(user):
    """
    获取部门权限列表,可用redis存取
    """
    if user is None:
        return []
    if user.is_superuser == 1:
        perms_list = []
    else:
        perms = OrgnizationEntity.objects.none()
        user_perms = user.org_perms.all()
        if user_perms:
            perms = perms | user_perms
        perms_list = perms.values_list('id', flat=True)
        perms_list = list(set(perms_list))
    cache.set(user.username + '__org_perms', perms_list)
    return perms_list
