# coding=utf-8
import redis
import os

redis_host = os.environ.get('REDIS_HOST')
redis_port = os.environ.get('REDIS_PORT')
pool = redis.ConnectionPool(host=redis_host, port=redis_port)

class RedisHelper(object):
    _instance = None
    def __init__(self, chan='default_channel'):
        self.__conn = redis.Redis(connection_pool=pool, db=0)
        self.chan_sub = chan
        self.chan_pub = chan

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = object.__new__(cls)
        return cls._instance

    def set_salt_key(self, minion_tag):
        self.__conn.set(minion_tag, 'ok')
        
    def get_salt_key(self, minion_tag):
        self.__conn.get(minion_tag)
        
    #发布者
    def public(self, msg, chan=None):
        if chan:
            self.__conn.publish(chan, msg)   # 选择发送的频道
        else:
            self.__conn.publish(self.chan_pub, msg)
        return True

    #订阅者
    def subscribe(self, chan=None):
        pub = self.__conn.pubsub()
        if chan:
            pub.subscribe(chan)  # 选择接收的频道
        else:
            pub.subscribe(self.chan_sub)
        pub.parse_response()  # 准备接收
        return pub

    def nativestr(self,x):
        return x if isinstance(x, str) else x.decode('utf-8', 'replace')

    def handle_message(self, response):
        message_type = self.nativestr(response[0])
        if message_type == 'pmessage':
            message = {
                'type': message_type,
                'pattern': response[1],
                'channel': response[2],
                'data': response[3]
            }
        elif message_type == 'pong':
            message = {
                'type': message_type,
                'pattern': None,
                'channel': None,
                'data': response[1]
            }
        else:
            message = {
                'type': message_type,
                'pattern': None,
                'channel': response[1],
                'data': response[2]
            }
        return message

    # 队列(左入)
    def lpush(self, key, item):
        '''
        队列:左入
        '''
        self.__conn.lpush(key, item)

    # 队列(左出)
    def lpop(self, key, item):
        '''
        队列:左入
        '''
        self.__conn.lpop(key, item)

    # 队列(右入)
    def rpush(self, key, item):
        '''
        队列:右入
        '''
        self.__conn.rpush(key, item)

    # 队列(右出)
    def rpop(self, key, item):
        '''
        队列:右入
        '''
        self.__conn.rpop(key, item)