"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/2/16 下午1:55
@File: check_passwd.py
@Desc: 校验密码输入是否符合需求规范
"""
import re
from accounts.models import PasswordAccountRule


def valid_length(pwd, min_=8, max_=32, is_passwd=True):
    if min_ <= len(pwd) <= max_:
        return True, None
    else:
        if is_passwd:
            return False, f"密码长度应在{min_}到32位"
        else:
            return False, f"账号长度应在{min_}到32位"


def valid_number(pwd):
    match = re.search("[0-9]+", pwd)
    if match:
        return True, 1
    else:
        return False, "不包含数字"


def valid_letter(pwd):
    res = re.search("[a-zA-Z]+", pwd)
    if res:
        return True, 1
    else:
        return False, '不包含子母'


def check_special(pwd):
    res = re.search(r"[~!@#$%^&*()\-_=+{}|\[\]\\:\";\'<>?,./]+", pwd)
    if res:
        return True, 1
    else:
        return False, "不包含特殊字符"


def check_pwd(pwd):
    count = 0
    is_ok, msg = valid_length(pwd)
    if not is_ok:
        return is_ok, msg
    for temp in [valid_letter, valid_number, check_special]:
        is_ok, msg = temp(pwd)
        if is_ok:
            count = count + msg
    if count < 2:
        return False, '密码必须是含有字母，数字和特殊字符中的2种及以上'
    else:
        return True, '密码校验合格'


def check_account(account):
    account_rule = PasswordAccountRule.objects.all().first()
    if account_rule.accounts_check_status == 0:
        return True, '账号不检查'
    is_ok, msg = valid_length(account, min_=account_rule.account_mix_length, max_=32, is_passwd=False)
    if not is_ok:
        return is_ok, msg
    if not account_rule.is_num:
        match = re.search("[0-9]+", account)
        if match:
            return False, '账号含有数字，不符合设置的账号规则'
    if not account_rule.is_char:
        res = re.search(r"[~!@#$%^&*()\-_=+{}|\[\]\\:\";\'<>?,./]+", account)
        if res:
            return False, '账号含有特殊字符，不符合设置的账号规则'
    if not account_rule.is_lower:
        res = re.search("[a-z]+", account)
        if res:
            return False, '账号含有小写子母，不符合设置的账号规则'
    if not account_rule.is_upper:
        res = re.search("[A-Z]+", account)
        if res:
            return False, '账号含有大写子母，不符合设置的账号规则'

    return True, '账号校验合格'


if __name__ == '__main__':
    code, msg = check_pwd('12wud2juc4y]')
    print(code, msg)
