# -*- coding: utf-8 -*-

import logging
import json
from clientlog.models import ClientLog
from devices.models import Devices
from accounts.models import AccountEntity
from datetime import datetime

logger = logging.getLogger(__name__)

class ClientLogHelper(object):
    help = 'util of Client Log'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def add(self, client_log: ClientLog):
        """
        添加客户端日志
        """
        try:
            log_queryset = ClientLog.objects.filter(operate_uuid=client_log.operate_uuid).first()
            # 如果有operate_uuid,则不会重复插入
            if log_queryset is None:
                # 获取终端和用户的所属部门
                if client_log.device_info_json is None:
                    device = Devices.objects.filter(device_uuid=client_log.device_uuid).first()
                    if device:
                        client_log.device_org_id = device.org_info.id
                        client_log.device_cnname = device.device_cnname
                        client_log.device_info_json = json.dumps({
                            'device_hostname': device.device_hostname,
                            'device_ip': device.device_ip
                        })
                # 获取终端用户信息
                if client_log.account_info_json is None and client_log.account_name is not None:
                    account = AccountEntity.objects.filter(name=client_log.account_name).first()
                    if account:
                        client_log.account_org_id = account.org_info.id
                        client_log.account_cnname = account.cn_name
                        client_log.account_info_json = json.dumps({
                            'email': account.email
                        })
                # 操作时间不要微妙
                now = datetime.now()
                operation_time = now.replace(microsecond=0)
                new_client_log = {
                    'device_uuid': client_log.device_uuid,
                    'device_cnname': client_log.device_cnname,
                    'device_org_id': client_log.device_org_id,
                    'account_name': client_log.account_name,
                    'account_cnname': client_log.account_cnname,
                    'account_org_id': client_log.account_org_id,
                    'module': client_log.module,
                    'action': client_log.action,
                    'operate_uuid': client_log.operate_uuid,
                    'result_status': client_log.result_status,
                    'request': client_log.request,
                    'response': client_log.response,
                    'device_info_json': client_log.device_info_json,
                    'account_info_json': client_log.account_info_json,
                    'operation_time': operation_time
                }
                ClientLog.objects.create(**new_client_log)
        except Exception as exp:
            logger.error(f"添加客户端日志异常:{exp}")


    def update_result_status(self, client_log: ClientLog):
        """
        修改客户端日志的处理结果
        """
        try:
            # 根据请求唯一码和状态结果查询
            log_queryset = ClientLog.objects.filter(operate_uuid=client_log.operate_uuid, result_status__isnull=True).first()
            # 存在就修改
            if log_queryset:
                log_queryset.result_status = client_log.result_status
                log_queryset.response = client_log.response
                log_queryset.save()
        except Exception as exp:
            logger.error(f"修改客户端日志的处理结果异常:{exp}")