"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/11/24 下午3:24
@File: middleware.py.py
@Desc: 
"""
import logging
import json
from adminlog.tasks import admin_log_add
from django.http import HttpResponse, FileResponse

logger = logging.getLogger(__name__)

class ApiLoggingMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response
        self.apiLogger = logging.getLogger(__name__)

    def __call__(self, request):
        try:
            body = json.loads(request.body)
        except Exception:
            body = dict()
        body.update(dict(request.POST))
        requestID = request.META.get('HTTP_REQUESTID', None)
        response = self.get_response(request)
        if request.method != 'GET':
            try:
                self.apiLogger.info("{} {} {} {} {} {} {} {}".format(
                    request.user, requestID, request.method, request.path, body,
                    response.status_code, response.reason_phrase, response.content.decode('utf-8')))
            except Exception as exp:
                self.apiLogger.info("{} {} {} {} {} {} {}".format(
                    request.user, requestID, request.method, request.path, body,
                    response.status_code, response.reason_phrase))
        else:
            self.apiLogger.info("{} {} {} {} {} {}".format(
                request.user, requestID, request.method, request.path,
                response.status_code, response.reason_phrase))
            
        # 管理员日志处理
        self._process_admin_log_(request, response, body)

        return response

    def _process_admin_log_(self, request, response, request_body):
        """
        管理员日志处理:分析request和response并存入数据库
        """
        try:
            request_file = {}
            request_is_file = False
            if request.FILES:
                request_is_file = True
                for file_item, file_object in request.FILES.items():
                    request_file['name'] = file_object.name
                    request_file['size'] = file_object.size
            request_dict = {
                'request_user': str(request.user) if request.user else None,
                'request_method': str(request.method) if request.method else None,
                'request_path': str(request.path) if request.path else None,
                'request_meta_data': {key: value for key, value in request.META.items()},
                'request_body': json.dumps(request_body),
                'request_content_type': str(request.content_type) if request.content_type else None,
                'request_is_file': request_is_file,
                'request_file': request_file
            }

            # 如果是文件或二进制,不处理
            response_content = None
            if isinstance(response, FileResponse):
                response_content = None
            elif isinstance(response, HttpResponse):
                # 普通处理 HttpResponse
                response_content = response.content.decode('utf-8')
                # 文件的处理
                if request.FILES:
                    # 尝试json转换,不能转换的就配置为request_file(如文件下载时是二进制)
                    try:
                        json.loads(response_content)
                    except Exception as e2:
                        logger.error(f"json.loads response.content error: {e2}")
                        response_content = request_file

            response_dict = {
                'response_status_code' : response.status_code if response.status_code else 0,
                'response_content': response_content,
            }

            # celery异步任务:管理员日志处理
            admin_log_add.delay(request_dict=request_dict, response_dict=response_dict)
        except Exception as exp:
            logger.error(f"_process_admin_log_ error:{exp}")