#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Author  : hu_cl
# @Contact : 760730895@qq.com 
# @Date    : 2020/12/10 16:33
# @File    : pagination.py
import logging

from rest_framework.response import Response
from collections import OrderedDict
from rest_framework.pagination import PageNumberPagination
from django.core.paginator import Paginator
from django.utils.functional import cached_property

logger = logging.getLogger(__name__)


class FasterDjangoPaginator(Paginator):
    @cached_property
    def count(self):
        # only select 'id' for counting, much cheaper
        return self.object_list.values('id').count()


class MyPagination(PageNumberPagination):
    django_paginator_class = FasterDjangoPaginator

    page_size = 10  # 每页显示多少条
    page_size_query_param = 'pageSize'  # URL中每页显示条数的参数
    page_query_param = 'current'  # URL中页码的参数
    max_page_size = 100000  # 前端最多能设置的每页数量

    def get_paginated_response(self, data):
        return Response(OrderedDict([
            ('current', self.page.number),
            ('pageCount', self.page.paginator.num_pages),
            ('pageSize', self.page.paginator.per_page),
            ('total', self.page.paginator.count),
            ('data', data),
        ]))


def get_page_data(query_set, request, serializer_obj):
    """
    分页
    :param query_set: orm查询结构体
    :param request: 请求
    :param serializer_obj: 表的对应serializer
    :return:
    """
    page = PageNumberPagination()
    page.page_size = 10  # 默认每页显示记录
    page.page_query_param = 'current'  # 默认查询参数名为 page
    page.page_size_query_param = 'pageSize'
    page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大

    res_list = []
    total = 0
    try:
        ret = page.paginate_queryset(query_set, request)
    except:
        return False, res_list, total
    total = len(query_set)
    res_list = serializer_obj(ret, many=True).data
    return True, res_list, total
