"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2024/1/29 下午1:48
@File: redis_domain_device.py
@Desc: 项目启动将数据缓存到redis
"""
import json
import logging

from django_redis import get_redis_connection

from devices.models import Devices

logger = logging.getLogger(__name__)
redis_conn = get_redis_connection('redis')


class RedisCacheDevice(object):
    help = 'Cache specified database data to Redis'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.device_hash_name = "domain_device"  # 设备redis中hash表的键

    def handle(self, *args, **options):
        # 获取指定数据库数据
        queryset = Devices.objects.all().exclude(is_domain=0).values("device_hostname", "device_ip", "is_domain")
        for item in queryset:
            redis_conn.hset(self.device_hash_name, item.get("device_hostname"), json.dumps(item, ensure_ascii=False))

    def update_redis_cache(self, field):
        """
        更新redis中键值对
        :param field: 键
        :return:
        """
        values = Devices.objects.exclude(is_domain=0).filter(device_hostname=field).values("device_hostname",
                                                                                           "device_ip",
                                                                                           "is_domain")
        for item in values:
            redis_conn.hset(self.device_hash_name, field, json.dumps(item, ensure_ascii=False))

    def delete_redis_cache(self, field):
        """
        删除redis的键值对
        :param field:
        :return:
        """
        redis_conn.hdel(self.device_hash_name, field)

    def device_is_domain(self, field):
        """
        检查设备主机名是否在域中
        :param field: 设备主机名
        :return: True or False
        """
        device_redis_dict = redis_conn.hget(self.device_hash_name, field)
        res_result = False
        if device_redis_dict:
            device_redis_dict = device_redis_dict.decode('utf-8')
            device_redis_dict = json.loads(device_redis_dict)
            device_is_domain = device_redis_dict.get('is_domain', None)
            if isinstance(device_is_domain, int):
                if int(device_is_domain) != 0:
                    res_result = True
            else:
                pass
        else:
            pass

        return res_result
