# -*- coding: utf-8 -*-

import json
import logging
import os
from datetime import datetime
from reports.models import HistoryDeviceOnline, HistoryAccountOnline
from asgiref.sync import async_to_sync, sync_to_async

logger = logging.getLogger(__name__)

class ReportsHelper(object):
    help = 'util of Reports or Stats'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @sync_to_async
    def save_history_device_online_up(self, device_uuid):
        """
        终端设备上线信息入库
        根据device_uuid查询是否已有记录:
        1, 如果down_time是Null,表明用户还没有下线,不再新增
        2, 如果down_time不是Null,表明用户下线过,这是再次上线,需要新增
        """
        try:
            history_device_online = HistoryDeviceOnline.objects.filter(device_uuid=device_uuid,down_time__isnull=True).first()
            # down_time是Null的数据没有时,就新增
            if history_device_online is None:
                HistoryDeviceOnline.objects.create(
                    device_uuid=device_uuid
                )
        except Exception as exp:
            logger.error(f"终端设备上线信息入库异常:{exp}")

    @sync_to_async
    def save_history_device_online_down(self, device_uuid):
        """
        终端设备下线信息入库
        根据device_uuid查询是否已有记录:
        1, 如果down_time是Null,表明终端设备要下线,更新下线时间
        """
        try:
            now = datetime.now()
            history_device_online = HistoryDeviceOnline.objects.filter(device_uuid=device_uuid,down_time__isnull=True).first()
            if history_device_online is not None:
                history_device_online.down_time = now
                history_device_online.save()
        except Exception as exp:
            logger.error(f"终端设备下线信息入库异常:{exp}")


    def save_history_account_online_up(self, account_name, device_uuid):
        """
        终端设备用户上线信息入库
        根据account_name和device_uuid查询是否已有记录:
        1, 如果down_time是Null,表明用户还没有下线,不再新增
        2, 如果down_time不是Null,表明用户下线过,这是再次上线,需要新增
        """
        try:
            history_account_online = HistoryAccountOnline.objects.filter(account_name=account_name,device_uuid=device_uuid,down_time__isnull=True).first()
            # down_time是Null的数据没有时,就新增
            if history_account_online is None:
                HistoryAccountOnline.objects.create(
                    account_name=account_name,
                    device_uuid=device_uuid
                )
        except Exception as exp:
            logger.error(f"终端设备用户上线信息入库异常:{exp}")

    
    @sync_to_async
    def save_history_account_online_down(self, account_name, device_uuid):
        """
        终端设备用户下线信息入库
        根据account_name和device_uuid查询是否已有记录:
        1, 如果down_time是Null,表明用户要下线,更新下线时间
        """
        try:
            now = datetime.now()
            history_account_online = HistoryAccountOnline.objects.filter(account_name=account_name,device_uuid=device_uuid,down_time__isnull=True).first()
            if history_account_online:
                history_account_online.down_time = now
                history_account_online.save()
        except Exception as exp:
            logger.error(f"终端设备用户下线信息入库异常:{exp}")