import datetime
import os
import io
from xmlrpc.client import ServerProxy, Binary
from threading import Thread, Event
from django.conf import settings


class RpcClient(object):
    def __init__(self):
        self.rpc = ServerProxy("http://{}:5252".format(os.environ.get("DOMAIN_IP")))

    def ntp_rpc(self, ntp):
        try:
            rpc_response = self.rpc.ntp_rpc_interface('nfs', ntp)
        except Exception as e:
            return False
        if rpc_response.get('code') == 400:
            return False
        if not rpc_response.get('data'):
            return False
        return rpc_response.get('data')


class SaltRpcClient(object):
    def __init__(self):
        self.rpc = ServerProxy("http://{}:15252".format(os.environ.get("DOMAIN_IP")))

    def remove_ssh_host(self, host):
        res = self.rpc.remove_ssh_host_interface('nfs', host)
        return res

    def move_file(self, source_path, destination_path):
        res = self.rpc.move_file('nfs', source_path, destination_path)
        return res

    def delete_old_log_files(self, destination_path):
        res = self.rpc.delete_old_log_files('nfs', destination_path)
        return res


class RepreproRpcClient(object):
    def __init__(self):
        self.rpc = ServerProxy("http://{}:25252".format(os.environ.get("DOMAIN_IP")))

    def send_file(self, file, filename):
        with open(file, 'rb') as f:
            data = f.read()
        res = self.rpc.send_file('nfs', filename, Binary(data))
        # f = open(file, 'rb')
        # res = self.rpc.send_file('nfs', filename, Binary(f.read()))
        return res


class FreeipaRpcClient(object):
    def __init__(self):
        self.rpc = ServerProxy("http://{}:35252".format(os.environ.get("DOMAIN_IP")))

    def java_sm4_hash(self, data):
        res = self.rpc.java_sm4_hash('nfs', data)
        return res

    def java_sm4_decrypt(self, data):
        res = self.rpc.java_sm4_decrypt('nfs', data)
        return res
