"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/11/22 下午5:52
@File: sch_backend_task.py
@Desc: 后台定时任务
"""
import datetime
import logging

import pandas as pd
from apscheduler.schedulers.background import BackgroundScheduler
from django.core.cache import caches
from django.db import close_old_connections
from django_redis import get_redis_connection

from devices.models import Devices

logger = logging.getLogger(__name__)
scheduler = BackgroundScheduler()
redis_conn = get_redis_connection('redis')


def get_device_info():
    device_items = Devices.objects.all()
    device_items_count = device_items.exclude(is_domain=0).count()
    items_exclude = device_items.exclude(is_domain=0)
    return device_items, device_items_count, items_exclude


def collection_line_task():
    """
    统计设备在线率任务
    :return:
    """
    logger.info(f"统计设备在线任务启动 {datetime.datetime.now()}")
    close_old_connections()
    cache = caches['redis']
    logger.info('######## chan=collectOnline #######')
    conn = get_redis_connection('redis')
    devices_id_list = []
    device_items, device_items_count, items_exclude = get_device_info()
    for item in items_exclude:
        if conn.hget('actives', item.device_hostname) == b'1':
            devices_id_list.append(item.id)
    device_list = [{
        'time': datetime.datetime.now(),
        'total': len(devices_id_list),
        'online_percent': '{:.2f}%'.format(
            100 * (len(devices_id_list) / device_items_count)) if device_items_count != 0 else "0%",
        'ids_list': devices_id_list
    }]
    if cache.get('devices_online') is not None:
        df1 = cache.get('devices_online')
        df2 = pd.DataFrame(device_list)
        df = pd.concat([df1, df2])
        df = df[df['time'] >= (datetime.datetime.now() - datetime.timedelta(days=7))]
    else:
        df = pd.DataFrame(device_list)

    # logger.info(f"######## chan=collectOnline {df.to_dict('records')}#######")
    cache.set('devices_online', df, 7 * 24 * 60 * 60)
    close_old_connections()
    logger.info(f"统计设备在线任务结束 {datetime.datetime.now()}")


def my_listener(event):
    if event.exception:
        job_name = scheduler.get_job(event.job_id).func_ref.split(":")[-1]
        logger.info(f'后台定时任务出错了！！！！！！,{job_name}')
        if job_name == "patch_task":
            scheduler.reschedule_job(job_id=event.job_id, trigger='interval', minutes=5)
            logger.info(f'重新启用后台定时任务！！！！！！,{job_name}')
        elif job_name == "collection_line_task":
            scheduler.reschedule_job(job_id=event.job_id, trigger='cron', hour='0-23', )
            logger.info(f'重新启用后台定时任务！！！！！！,{job_name}')
        elif job_name == "software_calculation_policy":
            scheduler.reschedule_job(job_id=event.job_id, trigger='cron', hour='0-23', )
            logger.info(f'重新启用后台定时任务！！！！！！,{job_name}')
        else:
            pass
    else:
        pass
