"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2024/9/21 下午1:48
@File: software_policy_analyse.py
@Desc: 软件策略信息分析下发
"""
import datetime

import pandas as pd
import logging
import json
import ipaddress
from django.db.models import Q, Max

from django.db import close_old_connections, transaction

from accounts.models import AccountEntity
from devices.client_consumers import msg_to_client
from devices.models import Devices
from groups.models import GroupsDetail
from software.models import SoftwarePolicy, SoftwareInfoModel, SoftwareUserPolicy, SoftwareDevicePolicy
from django_redis import get_redis_connection

logger = logging.getLogger(__name__)
redis_conn = get_redis_connection('redis')


class SoftwarePolicyTask(object):

    def __init__(self):
        self.now_time = datetime.datetime.now()
        self.software_policy_items = SoftwarePolicy.objects.all()
        self.software_policy_scp_items = (self.software_policy_items.filter(status=1)
                                          .filter(Q(effective_date__lt=self.now_time, expiration_date__gt=self.now_time)
                                                  | Q(effective_date__isnull=True)))
        self.device_items = Devices.objects.exclude(is_domain=0).all()
        self.group_chat_items = GroupsDetail.objects.all()
        self.account_items = AccountEntity.objects.all()
        self.software_user_items = SoftwareUserPolicy.objects.all()
        self.software_device_items = SoftwareDevicePolicy.objects.all()
        self.software_info = SoftwareInfoModel.objects.filter(is_delete=0)

    def get_apply_range_list(self, row):
        """获取策略实际应用的设备或者用户的"""
        # logger.info(f"{row}")
        range_type = int(row.get('range_type'))  # 0|按部门, 1|按终端, 2|按 IP 网段, 3|按人员, 4|按终端组, 5|按用户组
        range_value = row.get('range_value')
        policy_type = int(row.get('policy_type'))  # 软件策略的类型,0|终端策略 1|用户策略
        exception_range_type = int(row.get('exception_range_type'))
        exception_range_value = row.get('exception_range_value')
        if exception_range_value is "":
            exception_range_value = None
        res_apply_list = []
        res_exceptions_list = []
        if policy_type == 0:
            # 软件策略的类型,0|终端策略
            if range_type == 0:
                # 部门处理
                organ_list = range_value.split(',')
                if self.device_items.filter(org_info_id__in=organ_list).exists():
                    res_apply_list = self.device_items.filter(org_info_id__in=organ_list).values_list('id', flat=True)
            elif range_type == 1:
                # 终端处理
                res_apply_list = range_value.split(',')
            elif range_type == 2:
                # ip处理
                ip_list = range_value.split(',')
                mid_ip_list = []
                for i in ip_list:
                    start_ip, end_ip = i.split('-')
                    # 解析起始和结束IP地址为IPv4Address对象
                    start_ip = ipaddress.IPv4Address(start_ip)
                    end_ip = ipaddress.IPv4Address(end_ip)
                    range_ip_list = [str(ipaddress.IPv4Address(ip)) for ip in range(int(start_ip), int(end_ip) + 1)]
                    mid_ip_list = range_ip_list + mid_ip_list
                if self.device_items.filter(device_ip__in=mid_ip_list).exists():
                    res_apply_list = self.device_items.filter(device_ip__in=mid_ip_list).values_list('id', flat=True)
            elif range_type == 4:
                # 按终端组处理
                group_chat_list = range_value.split(',')
                if self.group_chat_items.filter(groups_id__in=group_chat_list).exists():
                    group_chat_filter_items = self.group_chat_items.filter(groups_id__in=group_chat_list)
                    items_0_range = group_chat_filter_items.filter(range=0).values_list('org_id', flat=True)
                    items_1_range = group_chat_filter_items.filter(range=1).values_list('device_id', flat=True)
                    # 处理部门数据
                    if self.device_items.filter(org_info_id__in=items_0_range).exists():
                        device_0_list = self.device_items.filter(org_info_id__in=items_0_range).values_list('id',
                                                                                                            flat=True)
                        device_0_list = [str(i) for i in device_0_list]
                    else:
                        device_0_list = []
                    # 处理设备信息
                    device_1_list = [str(i) for i in items_1_range]
                    device_list = device_0_list + device_1_list
                    res_apply_list = list(set(device_list))
            else:
                pass

            # 终端策略计算例外范围
            if res_apply_list:
                if exception_range_type == 0:
                    # 部门处理
                    exception_range_value = exception_range_value.split(
                        ',') if exception_range_value is not None else []
                    if self.device_items.filter(org_info_id__in=exception_range_value).exists():
                        res_exceptions_list = self.device_items.filter(
                            org_info_id__in=exception_range_value).values_list('id',
                                                                               flat=True)
                elif exception_range_type == 1:
                    # 终端处理
                    res_exceptions_list = exception_range_value.split(',') if exception_range_value is not None else []
                elif exception_range_type == 2:
                    # ip处理
                    ip_list = exception_range_value.split(',') if exception_range_value is not None else []
                    mid_ip_list = []
                    for i in ip_list:
                        start_ip, end_ip = i.split('-')
                        # 解析起始和结束IP地址为IPv4Address对象
                        start_ip = ipaddress.IPv4Address(start_ip)
                        end_ip = ipaddress.IPv4Address(end_ip)
                        range_ip_list = [str(ipaddress.IPv4Address(ip)) for ip in range(int(start_ip), int(end_ip) + 1)]
                        mid_ip_list = range_ip_list + mid_ip_list
                    if self.device_items.filter(device_ip__in=mid_ip_list).exists():
                        res_exceptions_list = self.device_items.filter(device_ip__in=mid_ip_list).values_list('id',
                                                                                                              flat=True)
                elif exception_range_type == 4:
                    # 按终端组处理
                    group_chat_list = exception_range_value.split(',') if exception_range_value is not None else []
                    if self.group_chat_items.filter(groups_id__in=group_chat_list).exists():
                        group_chat_filter_items = self.group_chat_items.filter(groups_id__in=group_chat_list)
                        items_0_range = group_chat_filter_items.filter(range=0).values_list('org_id', flat=True)
                        items_1_range = group_chat_filter_items.filter(range=1).values_list('device_id', flat=True)
                        # 处理部门数据
                        if self.device_items.filter(org_info_id__in=items_0_range).exists():
                            device_0_list = self.device_items.filter(org_info_id__in=items_0_range).values_list('id',
                                                                                                                flat=True)
                            device_0_list = [str(i) for i in device_0_list]
                        else:
                            device_0_list = []
                        # 处理设备信息
                        device_1_list = [str(i) for i in items_1_range]
                        device_list = device_0_list + device_1_list
                        res_exceptions_list = list(set(device_list))
                else:
                    pass

            res_apply_list = [int(i) for i in res_apply_list]
            res_exceptions_list = [int(i) for i in res_exceptions_list]
            set1 = set(res_apply_list)
            set2 = set(res_exceptions_list)
            res_apply_list = list(set1 - set2)
            res_apply_list = list(set(res_apply_list))  # 去重处理
            res_apply_list = [str(i) for i in res_apply_list]
            res_apply_list = ','.join(res_apply_list) if len(res_apply_list) != 0 else None

        elif policy_type == 1:
            # 软件策略的类型 1|用户策略
            if range_type == 0:
                # 部门处理
                organ_list = range_value.split(',')
                if self.account_items.filter(org_info_id__in=organ_list).exists():
                    res_apply_list = self.account_items.filter(org_info_id__in=organ_list).values_list('id', flat=True)
            elif range_type == 3:
                # 人员处理
                res_apply_list = range_value.split(',')
            elif range_type == 5:
                # 用户组处理逻辑
                group_chat_list = range_value.split(',')
                if self.group_chat_items.filter(groups_id__in=group_chat_list).exists():
                    group_chat_filter_items = self.group_chat_items.filter(groups_id__in=group_chat_list)
                    items_0_range = group_chat_filter_items.filter(range=0).values_list('org_id', flat=True)
                    items_3_range = group_chat_filter_items.filter(range=1).values_list('account_id', flat=True)
                    # 处理部门数据
                    if self.account_items.filter(org_info_id__in=items_0_range).exists():
                        users_0_list = self.account_items.filter(org_info_id__in=items_0_range).values_list('id',
                                                                                                            flat=True)
                        users_0_list = [str(i) for i in users_0_list]
                    else:
                        users_0_list = []
                    # 处理用户数据
                    users_3_list = [str(i) for i in items_3_range]
                    users_list = users_0_list + users_3_list
                    res_apply_list = list(set(users_list))

            # 用户策略计算例外范围
            if res_apply_list:
                if exception_range_type == 0:  # 例外范围为部门
                    exception_range_value = [int(i) for i in exception_range_value.split(
                        ',')] if exception_range_value is not None else []  # 强转整形
                    res_exceptions_list = self.account_items.filter(org_info_id__in=exception_range_value).values_list(
                        'id', flat=True)
                elif exception_range_type == 3:
                    # 人员处理
                    res_exceptions_list = exception_range_value.split(',') if exception_range_value is not None else []
                elif exception_range_type == 5:
                    # 用户组处理逻辑
                    group_chat_list = exception_range_value.split(',') if exception_range_value is not None else []
                    if self.group_chat_items.filter(groups_id__in=group_chat_list).exists():
                        group_chat_filter_items = self.group_chat_items.filter(groups_id__in=group_chat_list)
                        items_0_range = group_chat_filter_items.filter(range=0).values_list('org_id', flat=True)
                        items_3_range = group_chat_filter_items.filter(range=1).values_list('account_id', flat=True)
                        # 处理部门数据
                        if self.account_items.filter(org_info_id__in=items_0_range).exists():
                            users_0_list = self.account_items.filter(org_info_id__in=items_0_range).values_list('id',
                                                                                                                flat=True)
                            users_0_list = [str(i) for i in users_0_list]
                        else:
                            users_0_list = []
                        # 处理用户数据
                        users_3_list = [str(i) for i in items_3_range]
                        users_list = users_0_list + users_3_list
                        res_exceptions_list = list(set(users_list))

            res_apply_list = [int(i) for i in res_apply_list]
            res_exceptions_list = [int(i) for i in res_exceptions_list]
            set1 = set(res_apply_list)
            set2 = set(res_exceptions_list)
            res_apply_list = list(set1 - set2)
            res_apply_list = list(set(res_apply_list))  # 去重处理
            res_apply_list = [str(i) for i in res_apply_list]
            res_apply_list = ','.join(res_apply_list) if len(res_apply_list) != 0 else None
        else:
            res_apply_list = ','.join(res_apply_list) if len(res_apply_list) != 0 else None

        return res_apply_list

    @staticmethod
    def software_user_policy(items):
        """
        计算当前所有策略涉及到的用户
        """
        user_ids_list = []
        user_policy_queryset = items.filter(policy_type=1).values_list('apply_range_value',
                                                                       flat=True)
        for i in user_policy_queryset:
            user_ids_list = user_ids_list + i.split(',') if i is not None else []
        user_ids_list = [int(i) for i in user_ids_list]
        user_ids_list = list(set(user_ids_list))

        return user_ids_list

    @staticmethod
    def software_device_policy(items):
        """
        计算当前所有策略涉及到的终端
        """
        device_ids_list = []
        device_policy_queryset = items.filter(policy_type=0).values_list('apply_range_value',
                                                                         flat=True)
        for i in device_policy_queryset:
            device_ids_list = device_ids_list + i.split(',') if i is not None else []
        device_ids_list = [int(i) for i in device_ids_list]
        device_ids_list = list(set(device_ids_list))

        return device_ids_list

    def assign_account_values(self, account_id):
        # 定义一个函数，用于根据 account_id 值返回相应的其他列的值
        retry_count = 0
        is_notify_user = 0
        is_user_cancel = 0
        software_ids = None
        software_black_ids = None
        is_send = 0
        software_debpackage = []
        software_black_debpackage = []
        software_ids_names = None
        software_black_ids_names = None
        user_query = (
                Q(apply_range_value__icontains=f',{account_id},') |
                Q(apply_range_value=account_id) |
                Q(apply_range_value__icontains=f'{account_id},') |
                Q(apply_range_value__icontains=f',{account_id}')
        )
        software_user_policy_queryset = self.software_policy_scp_items.filter(user_query).filter(policy_type=1)
        if software_user_policy_queryset.exists():
            if software_user_policy_queryset.filter(is_enforced=1).exists():
                # 强制策略计算逻辑
                retry_count = software_user_policy_queryset.filter(is_enforced=1).aggregate(Max('retry_count')).get(
                    'retry_count__max', None)
                if software_user_policy_queryset.filter(is_enforced=1, is_notify_user=1).exists():
                    is_notify_user = 1
                    if software_user_policy_queryset.filter(is_enforced=1, is_notify_user=1, is_user_cancel=1).exists():
                        is_user_cancel = 1
                    else:
                        is_user_cancel = 0
                else:
                    is_notify_user = 0
                    is_user_cancel = 0
            else:
                # 非强制策略计算
                retry_count = software_user_policy_queryset.aggregate(Max('retry_count')).get('retry_count__max', None)
                if software_user_policy_queryset.filter(is_notify_user=1).exists():
                    is_notify_user = 1
                    if software_user_policy_queryset.filter(is_notify_user=1, is_user_cancel=1).exists():
                        is_user_cancel = 1
                    else:
                        is_user_cancel = 0
                else:
                    is_notify_user = 0
                    is_user_cancel = 0
            # 优先级 强制策略的黑名单 > 强制策略的软件安装列表 > 非强制策略的黑名单 > 非强制策略的软件安装列表
            # 获取用户强制策略中的黑名单的软件列表
            software_black_force_ids_list = []
            for i in software_user_policy_queryset.filter(is_enforced=1).values_list('software_black_ids', flat=True):
                software_black_force_ids_list = software_black_force_ids_list + i.split(',') if i is not None else []
            # 获取用户策略中的黑名单的软件列表
            software_black_ids_list = []
            for i in software_user_policy_queryset.filter(is_enforced=0).values_list('software_black_ids',flat=True):
                software_black_ids_list = software_black_ids_list + i.split(',') if i is not None else []
            # 获取用户强制策略中的软件列表
            software_force_ids_list = []
            for i in software_user_policy_queryset.filter(is_enforced=1).values_list('software_ids',flat=True):
                software_force_ids_list = software_force_ids_list + i.split(',') if i is not None else []
            # 获取用户策略中的软件列表
            software_ids_list = []
            for i in software_user_policy_queryset.filter(is_enforced=0).values_list('software_ids', flat=True):
                software_ids_list = software_ids_list + i.split(',') if i is not None else []
            software_black_force_ids_list = [int(i) for i in software_black_force_ids_list]
            software_black_ids_list = [int(i) for i in software_black_ids_list]
            software_force_ids_list = [int(i) for i in software_force_ids_list]
            software_ids_list = [int(i) for i in software_ids_list]
            # 转为集合方便处理
            set_software_black_force_ids = set(software_black_force_ids_list)
            set_software_black_ids = set(software_black_ids_list)
            set_software_force_ids = set(software_force_ids_list)
            set_software_ids = set(software_ids_list)
            software_black_ids_set = set_software_black_force_ids | (set_software_black_ids - set_software_force_ids)
            software_ids_set = (set_software_force_ids - set_software_black_force_ids) | (
                    set_software_ids - set_software_black_force_ids - set_software_black_ids)
            software_ids_list = [str(i) for i in software_ids_set]
            software_black_ids_list = [str(i) for i in software_black_ids_set]
            software_ids = ','.join(software_ids_list) if software_ids_list else None
            software_black_ids = ','.join(software_black_ids_list) if software_black_ids_list else None
            software_ids_items = self.software_info.filter(id__in=software_ids_list)
            for item in software_ids_items:
                software_debpackage.append({'name': item.main_package.lower(), 'version': item.main_version})

            software_black_ids_items = self.software_info.filter(id__in=software_black_ids_list)
            for item in software_black_ids_items:
                software_black_debpackage.append({'name': item.main_package.lower(), 'version': item.main_version})
            software_ids_names = json.dumps(software_debpackage, ensure_ascii=False)
            software_black_ids_names = json.dumps(software_black_debpackage, ensure_ascii=False)
        else:
            pass

        if self.software_user_items.filter(account_id=account_id, retry_count=retry_count,
                                           is_notify_user=is_notify_user, is_user_cancel=is_user_cancel,
                                           software_ids=software_ids, software_black_ids=software_black_ids).exists():
            serial_number = self.software_user_items.get(account_id=account_id).serial_number
            update_time = self.software_user_items.get(account_id=account_id).update_time
        else:
            serial_number = datetime.datetime.today().strftime("%Y%m%d%H%M%S")
            is_send = 1
            update_time = datetime.datetime.now()

        return (serial_number, retry_count, is_notify_user, is_user_cancel, software_ids, software_black_ids, is_send,
                update_time, software_ids_names, software_black_ids_names)

    def assign_device_values(self, device_id):
        # 定义一个函数，用于根据 device_id 值返回相应的其他列的值
        retry_count = 0
        is_notify_user = 0
        is_user_cancel = 0
        software_ids = None
        software_black_ids = None
        is_send = 0
        software_debpackage = []
        software_black_debpackage = []
        software_ids_names = None
        software_black_ids_names = None
        device_query = (
                Q(apply_range_value__icontains=f',{device_id},') |
                Q(apply_range_value=device_id) |
                Q(apply_range_value__icontains=f'{device_id},') |
                Q(apply_range_value__icontains=f',{device_id}')
        )
        software_device_policy_queryset = self.software_policy_scp_items.filter(device_query).filter(policy_type=0)
        if software_device_policy_queryset.exists():
            if software_device_policy_queryset.filter(is_enforced=1).exists():
                # 强制策略计算逻辑
                retry_count = software_device_policy_queryset.filter(is_enforced=1).aggregate(Max('retry_count')).get(
                    'retry_count__max', None)
                if software_device_policy_queryset.filter(is_enforced=1, is_notify_user=1).exists():
                    is_notify_user = 1
                    if software_device_policy_queryset.filter(is_enforced=1, is_notify_user=1,
                                                              is_user_cancel=1).exists():
                        is_user_cancel = 1
                    else:
                        is_user_cancel = 0
                else:
                    is_notify_user = 0
                    is_user_cancel = 0
            else:
                # 非强制策略计算
                retry_count = software_device_policy_queryset.aggregate(Max('retry_count')).get('retry_count__max',
                                                                                                None)
                if software_device_policy_queryset.filter(is_notify_user=1).exists():
                    is_notify_user = 1
                    if software_device_policy_queryset.filter(is_notify_user=1, is_user_cancel=1).exists():
                        is_user_cancel = 1
                    else:
                        is_user_cancel = 0
                else:
                    is_notify_user = 0
                    is_user_cancel = 0
            # 优先级 强制策略的黑名单 > 强制策略的软件安装列表 > 非强制策略的黑名单 > 非强制策略的软件安装列表
            # 获取终端强制策略中的黑名单的软件列表
            software_black_force_ids_list = []
            for i in software_device_policy_queryset.filter(is_enforced=1).values_list('software_black_ids',flat=True):
                software_black_force_ids_list = software_black_force_ids_list + i.split(',') if i is not None else []
            # 获取终端策略中的黑名单的软件列表
            software_black_ids_list = []
            for i in software_device_policy_queryset.filter(is_enforced=0).values_list('software_black_ids',flat=True):
                software_black_ids_list = software_black_ids_list + i.split(',') if i is not None else []
            # 获取终端强制策略中的软件列表
            software_force_ids_list = []
            for i in software_device_policy_queryset.filter(is_enforced=1).values_list('software_ids',flat=True):
                software_force_ids_list = software_force_ids_list + i.split(',') if i is not None else []
            # 获取终端策略中的软件列表
            software_ids_list = []
            for i in software_device_policy_queryset.filter(is_enforced=0).values_list('software_ids',flat=True):
                software_ids_list = software_ids_list + i.split(',') if i is not None else []
            software_black_force_ids_list = [int(i) for i in software_black_force_ids_list]
            software_black_ids_list = [int(i) for i in software_black_ids_list]
            software_force_ids_list = [int(i) for i in software_force_ids_list]
            software_ids_list = [int(i) for i in software_ids_list]
            # 转为集合方便处理
            set_software_black_force_ids = set(software_black_force_ids_list)
            set_software_black_ids = set(software_black_ids_list)
            set_software_force_ids = set(software_force_ids_list)
            set_software_ids = set(software_ids_list)
            software_black_ids_set = set_software_black_force_ids | (set_software_black_ids - set_software_force_ids)
            software_ids_set = (set_software_force_ids - set_software_black_force_ids) | (
                    set_software_ids - set_software_black_force_ids - set_software_black_ids)
            software_ids_list = [str(i) for i in software_ids_set]
            software_black_ids_list = [str(i) for i in software_black_ids_set]
            software_ids = ','.join(software_ids_list) if software_ids_list else None
            software_black_ids = ','.join(software_black_ids_list) if software_black_ids_list else None
            software_ids_items = self.software_info.filter(id__in=software_ids_list)
            for item in software_ids_items:
                software_debpackage.append({'name': item.main_package.lower(), 'version': item.main_version})

            software_black_ids_items = self.software_info.filter(id__in=software_black_ids_list)
            for item in software_black_ids_items:
                software_black_debpackage.append({'name': item.main_package.lower(), 'version': item.main_version})
            software_ids_names = json.dumps(software_debpackage, ensure_ascii=False)
            software_black_ids_names = json.dumps(software_black_debpackage, ensure_ascii=False)
        else:
            pass

        if self.software_device_items.filter(device_id=device_id, retry_count=retry_count,
                                             is_notify_user=is_notify_user, is_user_cancel=is_user_cancel,
                                             software_ids=software_ids, software_black_ids=software_black_ids).exists():
            serial_number = self.software_device_items.get(device_id=device_id).serial_number
            update_time = self.software_device_items.get(device_id=device_id).update_time
        else:
            serial_number = datetime.datetime.today().strftime("%Y%m%d%H%M%S")
            is_send = 1
            update_time = datetime.datetime.now()

        return (serial_number, retry_count, is_notify_user, is_user_cancel, software_ids, software_black_ids, is_send,
                update_time, software_ids_names, software_black_ids_names)

    def software_user_pd(self, user_ids_list):
        """
        用户软件pd数据创建，处理
        """
        columns = ['serial_number', 'account_id', 'account_name', 'retry_count', 'is_notify_user', 'is_user_cancel',
                   'software_ids', 'software_black_ids', 'update_time']
        user_df = pd.DataFrame(columns=columns)
        user_df['account_id'] = user_ids_list
        user_df['account_name'] = user_df['account_id'].apply(
            lambda x: self.account_items.get(id=x).name if self.account_items.filter(id=x).exists() else None
        )
        user_df[['serial_number', 'retry_count', 'is_notify_user',
                 'is_user_cancel', 'software_ids',
                 'software_black_ids', 'is_send', 'update_time',
                 'software_ids_names', 'software_black_ids_names']] = user_df['account_id'].apply(
            self.assign_account_values).apply(pd.Series)
        # 筛选出 is_send=1 的数据, 用户策略变动的
        user_filtered_df = user_df[user_df['is_send'] == 1]
        # 将 DataFrame 转换为记录字典（每行作为一个字典）
        user_records = user_filtered_df.to_dict(orient='records')
        user_records = user_records if len(user_records) > 0 else []
        return user_records

    def software_device_pd(self, device_ids_list):
        """
        设备软件pd数据创建
        """
        columns = ['serial_number', 'device_id', 'device_name', 'retry_count', 'is_notify_user', 'is_user_cancel',
                   'software_ids', 'software_black_ids', 'update_time']
        device_df = pd.DataFrame(columns=columns)
        device_df['device_id'] = device_ids_list
        device_df['device_name'] = device_df['device_id'].apply(
            lambda x: self.device_items.get(id=x).device_uuid if self.device_items.filter(id=x).exists() else None
        )
        device_df[['serial_number', 'retry_count', 'is_notify_user',
                   'is_user_cancel', 'software_ids',
                   'software_black_ids', 'is_send', 'update_time',
                   'software_ids_names', 'software_black_ids_names']] = device_df['device_id'].apply(
            self.assign_device_values).apply(pd.Series)
        # 筛选出 is_send=1 的数据
        device_filtered_df = device_df[device_df['is_send'] == 1]
        # 将 DataFrame 转换为记录字典（每行作为一个字典）
        device_records = device_filtered_df.to_dict(orient='records')
        device_records = device_records if len(device_records) > 0 else []
        return device_records

    def software_user_db(self, records):
        """更新用户软件信息记录"""
        with transaction.atomic():  # 原子事务处理
            for record in records:
                self.software_user_items.update_or_create(
                    account_id=record['account_id'],
                    defaults={
                        'account_name': record['account_name'],
                        'serial_number': record['serial_number'],
                        'retry_count': record['retry_count'],
                        'is_notify_user': record['is_notify_user'],
                        'is_user_cancel': record['is_user_cancel'],
                        'software_ids': record['software_ids'],
                        'software_black_ids': record['software_black_ids'],
                        'update_time': record['update_time'],
                        'software_ids_names': record['software_ids_names'],
                        'software_black_ids_names': record['software_black_ids_names']
                    }
                )

    def software_device_db(self, records):
        """更新终端软件信息记录"""
        with transaction.atomic():  # 原子事务处理
            for record in records:
                self.software_device_items.update_or_create(
                    device_id=record['device_id'],
                    defaults={
                        'device_name': record['device_name'],
                        'serial_number': record['serial_number'],
                        'retry_count': record['retry_count'],
                        'is_notify_user': record['is_notify_user'],
                        'is_user_cancel': record['is_user_cancel'],
                        'software_ids': record['software_ids'],
                        'software_black_ids': record['software_black_ids'],
                        'update_time': record['update_time'],
                        'software_ids_names': record['software_ids_names'],
                        'software_black_ids_names': record['software_black_ids_names']
                    }
                )

    def send_msg_to_client(self, records, type):
        """
        type 0 设备 1 用户
        dict_string = {
            'sender': 'SERVER',
            'sn': '9-2-1',
            'data': {
                    'taskid': '888',
                    'task_name': '软件策略',
                    'user_software': [{
                                        'user_account': '用户账号',
                                        'user_serial_number': '用户软件策略序列号',
                                        'is_notify_user': '软件安装是否通知用户,0|不通知；1|通知',
                                        'is_user_cancel': '软件安装是否允许用户取消,0|不允许；1|允许',
                                        'software_debpackage': [{'name': '*****', 'version': '***'}, {'name': '*****', 'version': '***'}],    # 软件安装
                                        'software_black_debpackage': [{'name': '*****', 'version': '***'}, {'name': '*****', 'version': '***'}], # 软件黑名单
                                    }],
                    'device_software': {
                                     'device_hostname': '终端主机名',
                                     'device_serial_number': '终端软件策略序列号',
                                     'is_notify_user': '软件安装是否通知用户,0|不通知；1|通知',
                                     'is_user_cancel': '软件安装是否允许用户取消,0|不允许；1|允许',
                                     'software_debpackage': [{'name': '*****', 'version': '***'}, {'name': '*****', 'version': '***'}],
                                     'software_black_debpackage': [{'name': '*****', 'version': '***'}, {'name': '*****', 'version': '***'}],}
            }}
        """
        send_msg = {'sender': 'SERVER', 'sn': '9-2-1', 'data': None}
        if type == 0:
            retry_count = records.get('retry_count', 0)
            device_id = records.get('device_id')
            device_name = records.get('device_name')
            software_ids_names = records.get('software_ids_names')
            software_black_ids_names = records.get('software_black_ids_names')

            user_list = self.device_items.get(id=device_id).bind_account.all()
            user_software = []
            software_bind_user_items = self.software_user_items.filter(account_id__in=user_list)
            if software_bind_user_items.exists():
                for item in software_bind_user_items:
                    software_ids_names = item.software_ids_names
                    software_black_ids_names = item.software_black_ids_names
                    user_software.append(
                        {
                            'user_account': item.account_name,
                            'user_serial_number': item.serial_number,
                            'is_notify_user': item.is_notify_user,
                            'is_user_cancel': item.is_user_cancel,
                            'software_debpackage': json.loads(software_ids_names) if software_ids_names else None,  # 软件安装
                            'software_black_debpackage': json.loads(software_black_ids_names) if software_black_ids_names else None,  # 软件黑名单
                        }
                    )
            send_msg.update(
                {'data': {
                    'taskid': '888',
                    'task_name': '软件策略',
                    'user_software': user_software,
                    'device_software': {
                        'device_hostname': device_name,
                        'device_serial_number': records.get('serial_number'),
                        'is_notify_user': records.get('is_notify_user'),
                        'is_user_cancel': records.get('is_user_cancel'),
                        'software_debpackage': json.loads(software_ids_names),
                        'software_black_debpackage': json.loads(software_black_ids_names)
                    }
                }})
            retry_count = int(retry_count) if int(retry_count) >= 1 else 1
            attempts = 0
            while True:
                try:
                    channel_client_name = redis_conn.hget('client_channel_name', device_name)
                    if channel_client_name:
                        channel_client_name = channel_client_name.decode('utf-8')
                        msg_to_client(1, channel_client_name, send_msg)
                        break
                except Exception as exp:
                    logging.info(f"尝试 {attempts}: 查询终端通讯信息失败，错误信息: {exp}")
                    attempts = attempts + 1
                    if attempts >= retry_count:
                        device_unsent_value = redis_conn.hget('client_websocket_unsent', device_name)
                        if device_unsent_value:
                            device_unsent_value = device_unsent_value.decode('utf-8')
                            device_unsent_value = json.loads(device_unsent_value)
                            device_unsent_value.append(send_msg)
                            redis_conn.hset('client_websocket_unsent', device_name,
                                            json.dumps(device_unsent_value, ensure_ascii=False))
                        else:
                            redis_conn.hset('client_websocket_unsent', device_name,
                                            json.dumps([send_msg], ensure_ascii=False))
                        break
        elif type == 1:
            retry_count = records.get('retry_count', 0)
            account_id = records.get('account_id')
            send_device_items = self.device_items.filter(bind_account=account_id)
            for item in send_device_items:
                user_list = item.bind_account.all()
                user_software = []
                software_bind_user_items = self.software_user_items.filter(account_id__in=user_list)
                if software_bind_user_items.exists():
                    for obj in software_bind_user_items:
                        software_ids_names = obj.software_ids_names
                        software_black_ids_names = obj.software_black_ids_names
                        user_software.append(
                            {
                                'user_account': obj.account_name,
                                'user_serial_number': obj.serial_number,
                                'is_notify_user': obj.is_notify_user,
                                'is_user_cancel': obj.is_user_cancel,
                                'software_debpackage': json.loads(software_ids_names) if software_ids_names else None,  # 软件安装
                                'software_black_debpackage': json.loads(software_black_ids_names) if software_black_ids_names else None,  # 软件黑名单
                            }
                        )
                software_device_items = self.software_device_items.filter(device_id=item.id)
                send_msg.update(
                    {'data': {
                        'taskid': '888',
                        'task_name': '软件策略',
                        'user_software': user_software,
                        'device_software': {
                            'device_hostname': software_device_items.first().device_name if software_device_items.exists() else None,
                            'device_serial_number': software_device_items.first().serial_number if software_device_items.exists() else None,
                            'is_notify_user': software_device_items.first().is_notify_user if software_device_items.exists() else 0,
                            'is_user_cancel': software_device_items.first().is_user_cancel if software_device_items.exists() else 0,
                            'software_debpackage': json.loads(
                                software_device_items.first().software_ids_names) if software_device_items.exists() else None,
                            'software_black_debpackage': json.loads(
                                software_device_items.first().software_black_ids_names) if software_device_items.exists() else None
                        }
                    }})
                retry_count = int(retry_count) if int(retry_count) >= 1 else 1
                attempts = 0
                while True:
                    try:
                        channel_client_name = redis_conn.hget('client_channel_name', item.device_hostname)
                        if channel_client_name:
                            channel_client_name = channel_client_name.decode('utf-8')
                            msg_to_client(1, channel_client_name, send_msg)
                            break
                    except Exception as exp:
                        logging.info(f"尝试 {attempts}: 查询终端通讯信息失败，错误信息: {exp}")
                        attempts = attempts + 1
                        if attempts >= retry_count:
                            device_unsent_value = redis_conn.hget('client_websocket_unsent', item.device_hostname)
                            if device_unsent_value:
                                device_unsent_value = device_unsent_value.decode('utf-8')
                                device_unsent_value = json.loads(device_unsent_value)
                                device_unsent_value.append(send_msg)
                                redis_conn.hset('client_websocket_unsent', item.device_hostname,
                                                json.dumps(device_unsent_value, ensure_ascii=False))
                            else:
                                redis_conn.hset('client_websocket_unsent', item.device_hostname,
                                                json.dumps([send_msg], ensure_ascii=False))
                            break
        else:
            logging.info(f"send_msg_to_client 传入参数错误 type = {type}")

    def software_calculation_policy(self):
        """
        计算当前软件策略，并获取用户和设备获取的策略
        """
        # 查询当前时间需要计算的策略
        software_policy_queryset = self.software_policy_items.filter(Q(effective_date__lt=self.now_time,
                                                                       expiration_date__gt=self.now_time)
                                                                     | Q(effective_date__isnull=True))
        user_ids_list = self.software_user_policy(software_policy_queryset)
        device_ids_list = self.software_device_policy(software_policy_queryset)
        if len(user_ids_list) > 0:
            user_records = self.software_user_pd(user_ids_list)
            if len(user_records) > 0:
                # 更新数据库记录
                self.software_user_db(user_records)
                for user_record in user_records:
                    self.send_msg_to_client(user_record, 1)
        if len(device_ids_list) > 0:
            device_records = self.software_device_pd(device_ids_list)
            if len(device_records) > 0:
                self.software_device_db(device_records)
                for device_record in device_records:
                    self.send_msg_to_client(device_record, 0)

        close_old_connections()  # 清理数据库连接

    def software_login_device(self, device_hostname):
        if self.software_device_items.filter(device_name=device_hostname).exists():
            device_item = self.software_device_items.filter(device_name=device_hostname).values()[0]
            self.send_msg_to_client(device_item, 0)
        else:
            pass

        close_old_connections()  # 清理数据库连接


def software_calculation_policy():
    """
    供给定时任务调用
    Returns:

    """
    start_time = datetime.datetime.now()
    logger.info(f"****{start_time}  software_calculation start******")
    software_policy_task = SoftwarePolicyTask()
    software_policy_items = software_policy_task.software_policy_items
    # 动态刷新软件策略应用范围
    for item in software_policy_items.values():
        apply_range_value = software_policy_task.get_apply_range_list(row=item)
        software_policy_items.filter(id=item['id']).update(apply_range_value=apply_range_value)
    # 更新下发策略信息
    software_policy_task.software_calculation_policy()
    close_old_connections()
    logger.info(f"****{start_time} software_calculation end******")
