# encoding:utf-8
import json
import time
import re
import logging
import os
import ctypes
import uuid
import openpyxl
import sys
import qrcode
import yaml
import socket
import subprocess
import base64
import signal as sig
from gmssl import sm4
from django.conf import settings
from django.db import close_old_connections

from salt.client.ssh.client import SSHClient
from six import BytesIO

from devices.models import Devices
from saltapi.utils import SaltAPI, SaltSHH
from utils.RedisHelper import RedisHelper
from accounts.models import BlackWhiteLists,BlackWhiteListsRule
# from multiprocessing import Process, Queue
# from threading import Thread


logger = logging.getLogger(__name__)
ssh_client_file = os.path.join(settings.SALT_CONF, 'etc/salt/roster')
c_path = os.path.join(settings.SALT_CONF, 'etc/salt/master')


# queue = Queue(1)

def sm4_hash(s:str, soap_sm4_keys:str) -> str:
    sm4_crypt = sm4.CryptSM4()
    sm4_crypt.set_key(soap_sm4_keys.encode(), sm4.SM4_ENCRYPT)
    encrypted_data = sm4_crypt.crypt_ecb(s.encode())
    return encrypted_data.hex()

def sm4_decrypt(b:str, soap_sm4_keys:str) -> str:
    sm4_crypt = sm4.CryptSM4()
    sm4_crypt.set_key(soap_sm4_keys.encode(), sm4.SM4_DECRYPT)
    decrypted_data = sm4_crypt.crypt_ecb(bytes.fromhex(b)).decode('utf-8')
    return decrypted_data

def java_sm4_hash(data):
    #logger.info(data)
    #base64_str = base64.b64encode(data.encode()).decode()
    if os.environ.get('DEPLOY_ARCH') == 'cluster':
        cmd_args = f'docker exec nfs-domain-mainserver_freeipa_1 bash ./java_sm4.sh sm4Hash \'{data}\''
    else:
        cmd_args = f'cd ../nfs_domain_mainserver/conf_ini/ && ./java_sm4.sh sm4Hash \'{data}\''

    res = subprocess.Popen(cmd_args, shell=True,
                           stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE)

    if res.wait() != 0:
        logger.info(res.stderr.read().decode('utf-8').strip())
        return
    s = res.stdout.read().decode('utf-8').strip()
    return s

def java_sm4_decrypt(data):
    #base64_str = base64.b64encode(data.encode()).decode()
    if os.environ.get('DEPLOY_ARCH') == 'cluster':
        cmd_args = f'docker exec nfs-domain-mainserver_freeipa_1 bash ./java_sm4.sh sm4Decrypt \'{data}\''
    else:
        cmd_args = f'cd ../nfs_domain_mainserver/conf_ini/ && ./java_sm4.sh sm4Decrypt \'{data}\''
    res = subprocess.Popen(cmd_args, shell=True,
                           stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE)
    if res.wait() != 0:
        logger.info(res.stderr.read().decode('utf-8').strip())
        return
    s = res.stdout.read().decode('utf-8').strip()
    return s

def get_qrcode(url):
    qr = qrcode.QRCode(version=1, error_correction=qrcode.constants.ERROR_CORRECT_L,
                       box_size=8,
                       border=8, )
    qr.add_data(url)  # 传入网址计算出二维码图片字节数据
    qr.make(fit=True)
    img = qr.make_image()
    # img.show()
    buf = BytesIO()  # 创建一个BytesIO临时保存生成图片数据
    img.save(buf)  # 在BytesIO临时保存拿出数据
    return buf.getvalue()


def read_licenseB(mac, license, salt=""):
    check_res = {}
    # check license len
    if len(license) != 30:
        check_res['register_res'] = 0
        return check_res

    mac_str = ctypes.c_char_p(bytes(mac, "utf-8"))
    reg_license = ctypes.c_char_p(bytes(license, "utf-8"))
    check_reg_new = ctypes.cdll.LoadLibrary(settings.LICENSE_B_CHECK_LIBRARY)
    # check_reg_new = ctypes.cdll.LoadLibrary(LICENSE_B_CHECK_LIBRARY)
    check_reg_new.is_register.restype = ctypes.c_int
    register_res = check_reg_new.is_register(mac_str, reg_license, salt)
    # 校验结果 0：失败， 1：成功
    check_res['register_res'] = ctypes.c_int(register_res).value

    # get time
    check_reg_new.get_expiration.restype = ctypes.c_char_p
    expiration_res = check_reg_new.get_expiration(reg_license)
    # print((ctypes.c_char_p(expiration_res).value).decode("utf-8"))
    check_res['expiration_res'] = ctypes.c_char_p(expiration_res).value

    # get max user
    check_reg_new.get_max_users.restype = ctypes.c_char_p
    max_users_res = check_reg_new.get_max_users(reg_license)
    # print((ctypes.c_char_p(max_users_res).value).decode("utf-8"))
    check_res['max_users_res'] = ctypes.c_char_p(max_users_res).value

    return check_res


def get_mac_address():
    if sys.platform == "win32":
        for line in os.popen("ipconfig /all"):
            if line.lstrip().startswith("Physical Address"):
                mac = line.split(":")[1].strip().replace("-", ":")
                return mac
    node = uuid.getnode()
    mac = uuid.UUID(int=node).hex[-12:]
    mac = ":".join(re.findall(r".{2}", mac))
    return mac


def bytes_to_string(b):
    return str(b, encoding="utf-8")


def pager_currency(request, total_number):
    '''
    current_page: 当前页
    per_page： 每页显示数量
    page_count： 总页数
    '''
    current_page = int(request.GET.get('current', 1))
    per_page = int(request.GET.get('pageSize', 10))
    if total_number <= per_page:
        page_count = 1
    elif total_number % per_page:
        page_count = (total_number // per_page) + 1
    else:
        page_count = total_number // per_page
    start = (current_page - 1) * per_page  # 0
    limit = per_page + start  # 10
    if total_number - limit:
        has_next_page = True
    else:
        has_next_page = False
    page_info = {
        'start': start,
        'limit': limit,
        'has_next_page': has_next_page,
        'current_page': current_page,
        'page_count': page_count,
        'per_page': per_page,
    }
    return page_info


def page_func(request, data_list):
    current_page = request.data.get('current', None)
    if not current_page:
        current_page = request.GET.get('current', None)
    per_page = request.data.get('pageSize', None)
    if not per_page:
        per_page = request.GET.get('pageSize', '50')
    if not current_page:
        return {'has_next_page': False, 'current_page': 0, 'page_count': 0}, data_list
    try:
        current_page = int(current_page)
        per_page = int(per_page)
    except ValueError:
        current_page = 1
        per_page = 50
    if len(data_list) <= per_page:
        page_count = 1
    elif len(data_list) % per_page:
        page_count = (len(data_list) // per_page) + 1
    else:
        page_count = len(data_list) // per_page
    start = (current_page - 1) * per_page  # 0
    limit = per_page + start  # 10
    res_list = data_list[start:limit]
    if len(data_list[limit:]):
        has_next_page = True
    else:
        has_next_page = False
    page_info = {
        'has_next_page': has_next_page,
        'current_page': current_page,
        'page_count': page_count,
        'per_page': per_page,
    }
    return page_info, res_list


def format_expiration(expiration):
    timeArray = time.strptime(expiration, "%Y-%m-%d")
    return time.strftime("%Y%m%d", timeArray)


def format_max_user(max_user):
    return max_user.zfill(4)


def get_user_ip(request):
    if request.META.get('HTTP_X_FORWARDED_FOR'):
        ip = request.META.get("HTTP_X_FORWARDED_FOR")
    else:
        ip = request.META.get("REMOTE_ADDR")
    return ip


def is_valid_mac(mac):
    if re.match(r"^\s*([0-9a-fA-F]{2,2}:){5,5}[0-9a-fA-F]{2,2}\s*$", mac):
        return True
    return False


def is_valid_ip(ip):
    if re.match(r"^\s*\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\s*$", ip):
        return True
    return False


def write_xls(sheet_name, head, data_list):
    """write listed data into excel
    """
    try:
        wb = openpyxl.Workbook()
        ws = wb.get_active_sheet()
    except Exception as e:
        logger.error(e)
        return None

    ws.title = sheet_name

    row_num = 0

    # write table head
    for col_num in range(len(head)):
        c = ws.cell(row=row_num + 1, column=col_num + 1)
        c.value = head[col_num]

    # write table data
    for row in data_list:
        row_num += 1
        for col_num in range(len(row)):
            c = ws.cell(row=row_num + 1, column=col_num + 1)
            c.value = row[col_num]

    return wb


def str_to_list(data_str):
    if not data_str:
        return []
    if "," in data_str:
        data = data_str.split(",")
        return [int(x) for x in data]
    elif "，" in data_str:
        data = data_str.split("，")
        return [int(x) for x in data]
    else:
        return [int(data_str)]


def list_to_str(data_list):
    if len(data_list) == 0:
        return ""
    elif len(data_list) == 1:
        return data_list[0]
    else:
        return ",".join([str(x) for x in data_list])


def get_serializer_error(serializer):
    data = serializer.errors
    error_list = []
    for k, v in data.items():
        error_list.append("%s %s" % (k, v[0]))
    error_str = ",".join(error_list)
    return error_str


def diff_list(old_list, new_list):
    add_list = list(set(new_list) - set(old_list))
    delete_list = list(set(old_list) - set(new_list))
    return add_list, delete_list


def get_device(range_action, org_id, device_id):
    from devices.models import Devices
    device_list = []
    if not org_id and not device_id:
        return device_list

    if range_action == 0:
        # 部门选择
        org_id_list = str_to_list(org_id)
        device_list = Devices.objects.filter(org_info__in=org_id_list).exclude(is_domain=0)
    else:
        # 终端选择
        device_id_list = str_to_list(device_id)
        device_list = Devices.objects.filter(id__in=device_id_list).exclude(is_domain=0)
    return device_list


def auto_load_salt_shh(host, port, username, passwd, hostname):
    """
    自动化载入配置信息到
    /etc/host/router
    :param host:
    :param port:
    :param username:
    :param passwd:
    :param hostname:
    :return:
    """
    try:
        res_code = 0
        res_msg = None
        with open(ssh_client_file, 'r+') as roster_file:
            lines = roster_file.readlines()
            for line in lines:
                # logger.info(f"{line}")
                if hostname in line:
                    res_code = 1
                    res_msg = f'主机名{hostname}已存在,请更换'

        with open(ssh_client_file, 'a+') as roster_file:
            if res_code == 0:
                roster_file.write(
                    f"{hostname}:\n  "
                    f"host: {host}\n  "
                    f"user: {username}\n  "
                    f"passwd: {passwd}\n  "
                    f"port: {port}\n")
    except Exception as exp:
        res_code = 1
        res_msg = exp

    return res_code, res_msg

def set_roster(data):
    # roster_file = os.path.join(settings.SALT_CONF, 'etc/salt/roster_{}'.format(time.time()))
    roster_file = os.path.join(settings.SALT_CONF, 'etc/salt/roster')
    with open(roster_file, 'r') as file:
        content = yaml.load(file, Loader=yaml.FullLoader)
    if not content:
        with open(roster_file, 'w') as file:
            yaml.dump(data, file)
        return roster_file
    keys = data.keys()
    for key in keys:
        info = content.get(key)
        if info:
            content.pop(key)
        content[key] = data[key]
    with open(roster_file, 'w') as file:
        yaml.dump(content, file)
    return roster_file

def del_roster(key):
    roster_file = os.path.join(settings.SALT_CONF, 'etc/salt/roster')
    with open(roster_file, 'r') as file:
        content = yaml.load(file, Loader=yaml.FullLoader)
    if content.get(key):
        content.pop(key)
    with open(roster_file, 'w') as file:
        yaml.dump(content, file)
    return roster_file

def auto_remove_salt_shh(hostname):
    """
    自动删除配置信息到
    /etc/host/router
    :param hostname:
    :return:
    """
    min_line = None  # 节点行
    n = -1  # 索引行
    try:
        with open(ssh_client_file, 'r+') as roster_file:
            lines = roster_file.readlines()
            roster_file.seek(0)
            for line in lines:
                if hostname in line:
                    min_line = lines.index(line)
            logger.info(f"salt-ssh 的 节点 名称 {hostname} 的 行数 {min_line}")
            if min_line is not None:
                roster_file.truncate()
                for line in lines:
                    n = n + 1
                    # logger.info(f"{min_line}, n 的大小{n}, roster {line} ")
                    # if lines.index(line, n) < min_line or lines.index(line, n) > min_line + 4:
                    if (n < min_line) or (n > min_line + 4):
                        roster_file.write(line)
                    else:
                        pass
    except Exception as exp:
        res_code = 1
        res_msg = exp
        logger.info(f"salt-ssh 的 节点 名称 {hostname}, 异常 {exp}")
    else:
        if min_line == 0:
            res_code = 1
            res_msg = '记录不存在'
        else:
            res_code = 0
            res_msg = None

    return res_code, res_msg


def check_auto_result(res, hostname):
    # logger.info(f'检查结果0{type(res)},{res}')
    if not res:
        device = Devices.objects.filter(device_hostname=f'{hostname}.{os.environ.get("DOMAIN_NAME")}').first()
        if device:
            device.is_domain = 0
            device.is_up = 1
            device.org_info_id = 1
            device.belong_user = None
            device.note = None
            device.save()
        # auto_remove_salt_shh(hostname)
        return False
    for v in res.values():
        # logger.info(f'检查结果1{type(res)},{v}')
        if v:
            retcode = v.get('retcode', None) if 'retcode' in v.keys() else None
            # logger.info(f'检查结果2{retcode}')
            if retcode is not None:
                # logger.info(f'检查结果3{retcode}')
                if retcode != 0:
                    # 清除 device记录
                    # 清除 router记录
                    device = Devices.objects.filter(device_hostname=f'{hostname}.{os.environ.get("DOMAIN_NAME")}').first()
                    if device:
                        device.is_domain = 0
                        device.is_up = 1
                        device.org_info_id = 1
                        device.belong_user = None
                        device.note = None
                        device.save()
                    # auto_remove_salt_shh(hostname)
                    return False
                else:
                    return True
            else:
                device = Devices.objects.filter(device_hostname=f'{hostname}.{os.environ.get("DOMAIN_NAME")}').first()
                if device:
                    device.is_domain = 0
                    device.is_up = 1
                    device.org_info_id = 1
                    device.belong_user = None
                    device.note = None
                    device.save()
                # auto_remove_salt_shh(hostname)
                return False
        else:
            # 清除 device记录
            # 清除 router记录
            device = Devices.objects.filter(device_hostname=f'{hostname}.{os.environ.get("DOMAIN_NAME")}').first()
            if device:
                device.is_domain = 0
                device.is_up = 1
                device.org_info_id = 1
                device.belong_user = None
                device.note = None
                device.save()
            # auto_remove_salt_shh(hostname)
            return False


def auto_install_client(hostname, device_ip, user, passwd, is_client, is_domain):
    """
    自动化安装客户端
    :param passwd: ssh密码
    :param hostname: 主机名
    :return: res_code  0 失败  1 成功  res_msg 返回信息
    """
    ip = os.environ.get("DOMAIN_IP")
    domain_server = os.environ.get("DOMAIN_NAME")
    port = 8082
    public = RedisHelper()
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    # salt_ssh = SaltSHH(roster_file)
    signal = {
        'event_type': 'auto_install_client',
        'device_ip': device_ip,
        'device_hostname': f'{hostname}.{domain_server}',
        'success': True,
        'msg': None,
        'is_domain': is_domain
    }

    client_url = f'wget -O install-domain-client_2.0.sh http://{ip}:{port}/mainapi/client/install-domain-client_2.0.sh/'
    install_client_cmd = f'echo {passwd}|sudo -S bash install-domain-client_2.0.sh {ip} server.{domain_server} {is_domain}'

    res_code = 0
    res_msg = None

    if is_client:
        res = salt_api.ssh_client(tgt=hostname, arg=client_url)
        logger.info(f'自动下载安装客户端{res}')
        if not check_auto_result(res, hostname):
            signal['success'] = False
            signal['msg'] = '自动下载客户端失败'
            public.public(json.dumps(signal), chan='freeipa')
            return 1, res_msg

        res = salt_api.ssh_client(tgt=hostname, arg=install_client_cmd)
        logger.info(f'自动安装客户端{res}')
        if not check_auto_result(res, hostname):
            signal['success'] = False
            signal['msg'] = '自动安装客户端失败'
            public.public(json.dumps(signal), chan='freeipa')
            return 1, res_msg

    domain_url = f'wget -O add_domain.sh http://{ip}:{port}/mainapi/client/add_domain.sh/'
    domian_hostname = f'{hostname}.{domain_server}'
    add_domain_cmd = f'echo {passwd}|sudo -S bash add_domain.sh linux {ip} {port} admin {os.environ.get("DOMAIN_SERVER_ADMIN_PASSWD")} {domian_hostname} {is_domain}'
    clean_script_cmd = f"[ -f /home/{user}/install-domain-client_2.0.sh  ] && rm /home/{user}/install-domain-client_2.0.sh; [ -f /home/{user}/add_domain.sh ] && rm /home/{user}/add_domain.sh"
    if is_domain in (1, 2):
        res = salt_api.ssh_client(tgt=hostname, arg=domain_url)
        logger.info(f'自动化下载加域脚本{res}')
        if not check_auto_result(res, hostname):
            signal['success'] = False
            signal['msg'] = '自动下载加域脚本失败'
            public.public(json.dumps(signal), chan='freeipa')
            return 1, res_msg
        res = salt_api.ssh_client(tgt=hostname, arg=add_domain_cmd)
        logger.info(f'自动化加域{res}')
        if not check_auto_result(res, hostname):
            signal['success'] = False
            signal['msg'] = '自动安装加域脚本失败'
            public.public(json.dumps(signal), chan='freeipa')
            return 1, res_msg
    # res = salt_api.ssh_grains(tgt=hostname, arg=['lsb_distrib_release', 'lsb_distrib_id', 'lsb_distrib_description', 'osarch'])
    # logger.info(f'{device_ip}设备硬件信息{res}')
    # device_os = res[hostname].get('return').get('lsb_distrib_id')
    # os_release = res[hostname].get('return').get('lsb_distrib_release')
    # os_arch = res[hostname].get('return').get('osarch').lower()
    # signal['os'] = device_os
    # signal['os_release'] = os_release
    # signal['os_arch'] = os_arch.lower()
    signal['msg'] = f'自动添加终端{hostname}成功'
    public.public(json.dumps(signal), chan='freeipa')
    salt_api.ssh_client(tgt=hostname, arg=clean_script_cmd)
    close_old_connections()
    return res_code, res_msg

def test_ip_port_connection(ip, port,timeout=5):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(timeout)  # 设置超时时间
    try:
        sock.connect((ip, port))
        logger.info(f"地址 {ip}:{port} 可以联通")
        return True
    except (socket.timeout, ConnectionRefusedError):
        logger.info(f"地址 {ip}:{port} 无法联通")
        return False
    finally:
        sock.close()

def handle_timeout(signum, frame):
    raise TimeoutError("函数执行超时")

def timeout_func(timeout):
    def fun(f, timeout=timeout):
        def inner(*args,**kwargs):
            # 设置超时信号处理函数
            sig.signal(sig.SIGALRM, handle_timeout)
            # 设置超时时间
            sig.alarm(timeout)  # 单位为秒
            try:
                f(*args,**kwargs)
            finally:
                # 执行完毕后，取消超时信号
                sig.alarm(0)
        return inner
    return fun

def get_uuid(key=None):
    # 生成一个随机的UUID
    # random_uuid = uuid.uuid4()
    # print(random_uuid)
    # 生成基于字符串的UUID
    if key:
        return uuid.uuid5(uuid.NAMESPACE_DNS, key).__str__()
    # 生成基于时间的UUID
    return uuid.uuid1().__str__()

def is_satisfy_black_lists(name):
    query = BlackWhiteListsRule.objects.filter(id=1).first()
    lists_query = BlackWhiteLists.objects.all().order_by('-id')
    if query:
        if query.black_status == 1:
            if query.white_status == 1:  # 当黑名单开启时，再校验白名单
                white_query = lists_query.filter(list_type='white').values('op_type','content')
                if query.white_list_range == 1:  # 全部
                    compare = len(white_query)
                    num = 0
                    for condition in white_query:
                        op_type = condition['op_type']
                        content = condition['content']
                        if op_type == 'startswith' and name.startswith(content):
                            num += 1
                        elif op_type == 'endswith' and name.endswith(content):
                            num += 1
                        elif op_type == 'equals' and name == content:
                            num += 1
                        elif op_type == 'contains' and content in name:
                            num += 1
                    if compare == num:
                        return False
                else:  # 任意
                    for condition in white_query:
                        op_type = condition['op_type']
                        content = condition['content']
                        if op_type == 'startswith' and name.startswith(content):
                            return False
                        elif op_type == 'endswith' and name.endswith(content):
                            return False
                        elif op_type == 'equals' and name == content:
                            return False
                        elif op_type == 'contains' and content in name:
                            return False
            black_query = lists_query.filter(list_type='black').values('op_type', 'content')
            if query.black_list_range == 1:
                compare = len(black_query)
                num = 0
                for condition in black_query:
                    op_type = condition['op_type']
                    content = condition['content']
                    if op_type == 'startswith' and name.startswith(content):
                        num += 1
                    elif op_type == 'endswith' and name.endswith(content):
                        num += 1
                    elif op_type == 'equals' and name == content:
                        num += 1
                    elif op_type == 'contains' and content in name:
                        num += 1
                if compare == num:
                    return True
            else:
                for condition in black_query:
                    op_type = condition['op_type']
                    content = condition['content']
                    if op_type == 'startswith' and name.startswith(content):
                        return True
                    elif op_type == 'endswith' and name.endswith(content):
                        return True
                    elif op_type == 'equals' and name == content:
                        return True
                    elif op_type == 'contains' and content in name:
                        return True