"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/7/26 下午1:48
@File: zeep_api.py
@Desc: 调用SOAP-based web service 封装
"""
import json
import logging
from zeep import Client, exceptions
import time

logger = logging.getLogger(__name__)


class SOAPClient:
    def __init__(self, wsdl_url):
        self.client = None
        self.wsdl_url = wsdl_url
        self.max_retries = 3

    def connect(self):
        try:
            self.client = Client(self.wsdl_url)
            logger.info("Connected to SOAP service.")
        except exceptions.TransportError as e:
            logger.info(f"Connection error: {e}")
            return 0
        except exceptions.XMLSyntaxError as e:
            logger.info(f"XML syntax error: {e}")
            return 0
        except exceptions.Error as e:
            logger.info(f"soap Error: {e}")
            return 0
        except Exception as e:
            logger.info(f"soap unknown Error: {e}")
            return 0
        else:
            return 1

    def call(self, method_name, args, args1=None):
        if self.client is None:
            code = self.connect()
            if code != 1:
                return None
        for i in range(self.max_retries):
            try:
                if args1:
                    result = self.client.service[method_name](args, args1)
                else:
                    result = self.client.service[method_name](args)
                if result:
                    # return result
                    return json.dumps(result.__values__)  # OrderedDict -> dict
                else:
                    return None
            except exceptions.Fault as fault:
                logger.info(f'SOAP Fault: {fault}')
            except exceptions.TransportError as transport_error:
                logger.info(f'Transport Error: {transport_error}')
            except Exception as e:
                logger.info(f'Unexpected error: {e}')

            time.sleep(2)
        else:
            logger.info(f'Failed to call SOAP service after maximum retries. methode_name: {method_name}')
            return None
